/*----------------------------------------------------------------------+
|									|
|  Copyright (c) 1985-93;  Bentley Systems, Inc., All rights reserved.	|
|									|
| "MicroStation", "MDL", and "MicroCSL" are trademarks of Bentley	|
|  Systems, Inc.							|
|									|
|  Limited permission is hereby granted to reproduce and modify this	|
|  copyrighted material provided that the resulting code is used only 	|
|  in conjunction with Bentley Systems products under the terms of the	|
|  license agreement provided therein, and that this notice is retained	|
|  in its entirety in any such reproduction or modification.		|
|									|
+----------------------------------------------------------------------*/
/*----------------------------------------------------------------------+
|									|
|   dbdefs.h -- database definitions					|
|									|
|   $Workfile:   dbdefs.h  $
|   $Revision:   5.25  $
|      	$Date:   13 May 1994 14:42:08  $
|									|
+----------------------------------------------------------------------*/
#ifndef __dbdefsH__
#define __dbdefsH__

#include <basetype.h>

/*----------------------------------------------------------------------+
|									|
|   Linkage generation modes.           				|
|									|
+----------------------------------------------------------------------*/
#define NOLINK		    -1	        /* null linkage mode */
#define NEWLINK		     0	        /* new linkage generation mode */
#define INFOLINK	     1	        /* informational linkage mode */ 
#define DUPLINK		     2	        /* duplicate linkage mode */
						
/*----------------------------------------------------------------------+
|									|
|   Linkage types recognized by MicroStation servers.   		|
|									|
+----------------------------------------------------------------------*/
#define	INFORMIX_LINKAGE     1
#define	DMRS_LINKAGE	     2
#define	XBASE_LINKAGE	     4
/* DBASE_LINKAGE only exists for backwards compatibility, in future use XBASE_LINKAGE */
#define DBASE_LINKAGE	     4
#define	ORACLE_LINKAGE	     8
#define	RIS_LINKAGE	    16
#define INGRES_LINKAGE	    32

/*----------------------------------------------------------------------+
|									|
|   Signature words used for user data linkages.			|
|									|
+----------------------------------------------------------------------*/
#define INFORMIX_ID	    0x3948	/* Informix user data signature */
#define XBASE_ID	    0x1971	/* Xbase(dBASE) user data signature */
#define DBASE_ID	    0x1971	/* exists for backwards compatibility to Xbase */
#define RIS_ID		    0x71fb	/* RIS user data signature */
#define ORACLE_ID	    0x6091	/* Oracle user data signature */
#define INGRES_ID	    0x3a77	/* INGRES user data signature (ING) */
    
#define PRIMARY_ID	    0x6091	/* ID for Oracle linkage */
#define SECONDARY_ID	    0xf81	/* Secondary ID link (BSI radix 50) */
						
/*----------------------------------------------------------------------+
|									|
|   Masks for user attribute properties word.   			|
|									|
+----------------------------------------------------------------------*/
#define DB_PROP_INFO	    0x8000
#define DB_PROP_REMOTE	    0x4000
#define DB_PROP_MODIFIED    0x2000
#define DB_PROP_USER	    0x1000
#define DB_PROPERTY_MASK    0xf000
						
#define USER_LINKAGE	    0x1000	/* user data linkage */
#define INFO_LINKAGE	    0x8000	/* informational linkage */

/*----------------------------------------------------------------------+
|									|
|   Miscellaneous constants - some of these are related to the		|
|	size of mscatalog columns.  They should be kept in synch.	|
|									|
+----------------------------------------------------------------------*/
#define MAX_TABLE_LENGTH	    32	/* maximum length of table name	    */
#define MAX_SCREEN_FORM		    64
#define MAX_FORM_TABLE		    64
#define MAX_FORM_TYPE		    64
#define MAX_REPORT_TABLE	    64
#define	MAX_SQL_REVIEW		   240
#define MAX_FENCE_FILTER	   240
#define	MAX_DAS_TABLE		    32
#define MAX_COLUMN_LENGTH	    32	/* maximum length of column name    */
#define MAX_FORM_TYPE		    64

/*----------------------------------------------------------------------+
|									|
|   Form Types.								|
|									|
+----------------------------------------------------------------------*/
#define FORM_TYPE_REVIEW    1
#define FORM_TYPE_EDIT	    2

/*----------------------------------------------------------------------+
|									|
|   Miscellaneous masks.                				|
|									|
+----------------------------------------------------------------------*/
#define NULL_MASK	    0x8000 /* null/not null in type descriptor */
#define TYPE_MASK	    0x7fff /* mask type off type descriptor */

/*----------------------------------------------------------------------+
|									|
|   MicroStation message resource defines.      			|
|									|
+----------------------------------------------------------------------*/
#define SERVER_INIT            353  /* database server initialized */
#define DBLOAD	       	       354  /* loading and initializing database */
#define SELECT_SOURCE_LINK     601  /* select element to provide linkage */
    
#define DBSERVER_NOT_ACTIVE   2143  /* server not active */
#define SERVER_INIT_ERROR     2157  /* unable to initialize server */  
#define MSG_SEND_ERROR        2158  /* unable to send database request */
#define MSG_RCV_ERROR         2159  /* unable to receive database response */
#define SERVER_STARTUP_ERROR  2160  /* unable to startup database server */
#define DB_NOT_CONFIGURED     2161  /* not configured with the database. */
#define MAX_ADATA	      2162  /* maximum user data size exceeded */
#define DB_NO_DATYPE	      2200  /* no display attribute type specified */
#define SERVER_DIED	      2254  /* database server died */

/*----------------------------------------------------------------------+
|									|
|   Request defines for mdlDB_setFunction ().  				|
|									|
+----------------------------------------------------------------------*/
#define DB_HANDLE_REQUEST	 1  /* database request hook for MDL */
#define MCSL_HANDLE_REQUEST	 2  /* MicroCSL call back hook for MDL */
    
/*----------------------------------------------------------------------+
|									|
|   Column data types.                        				|
|									|
+----------------------------------------------------------------------*/
#define DB_CHAR		    1
#define DB_NUMBER	    2
#define DB_REAL		    2
#define DB_DATE		    3
#define DB_INTEGER          4
#define DB_RAW		    5
#define DB_BINARY	    6

/*----------------------------------------------------------------------+
|									|
|   Defines for mdlDB_fetchRow ().      				|
|									|
+----------------------------------------------------------------------*/
#define QUERY_NOT_FINISHED  0	/* current query is not finished */
#define QUERY_FINISHED	    1	/* currey query has finished */

/*----------------------------------------------------------------------+
|									|
|   Cursor typedef							|
|									|
+----------------------------------------------------------------------*/
typedef long	CursorID;

/*----------------------------------------------------------------------+
|									|
|   Structure typedefs.							|
|									|
+----------------------------------------------------------------------*/
#if !defined (resource)

#if defined (rs6000)
#pragma options align=twobyte
#endif
typedef struct user_link
    {
#if defined (BITFIELDS_REVERSED)
    unsigned short info:1;	/* boolean: informational linkage */
    unsigned short remote:1;	/* boolean: remote linkage */
    unsigned short modified:1;	/* boolean: user linkage has been modified */
    unsigned short user:1;	/* boolean: user data linkage */
    unsigned short class:4;	/* linkage family class */
    unsigned short words:8;	/* words to follow in linkage */
#else
    unsigned short words:8;	/* words to follow in linkage */
    unsigned short class:4;	/* linkage family class */
    unsigned short user:1;	/* boolean: user data linkage */
    unsigned short modified:1;  /* boolean: user linkage has been modified */
    unsigned short remote:1;	/* boolean: remote linkage */
    unsigned short info:1;	/* boolean: informational linkage */
#endif
    UShort   id;	    /* User ID number */
    } User_Link;
#if defined (rs6000)
#pragma options align=reset
#endif

typedef struct
    {
    UShort	information;	/* informational linkage */
    UShort	remote;		/* remote linkage */
    UShort	modified;	/* user linkage has been modified */
    UShort	user;		/* user data linkage */
    UShort	class;		/* linkage family class */
    } LinkProps;
						
typedef struct
    {
    UShort	linkSize;	/* total size (bytes) of linkage */
    UShort	linkType;	/* type (ORACLE, DMRS, XBASE...) */
    UShort	entity;		/* entity number */
    char	tablename[64];  /* database table */
    ULong	mslink;		/* MSLINK key of linkage */
    LinkProps	props;		/* properties */
    UShort	dasType;	/* displayable attributes */
    } DatabaseLink;
    
typedef struct 
    {
    short     numColumns;	/* number of columns in tuple */
    char    **name;		/* array of column names      */
    char    **value;		/* array of column values     */
    short    *type;		/* array of column types      */
    short    *length;		/* array of column lengths    */
    short    *scale;		/* array of numeric scale     */
    short    *prec;		/* array of numeric precision */
    short    *null;		/* array of nulls permitted   */
    } MS_sqlda;

typedef struct 
    {
    char     name[MAX_COLUMN_LENGTH];	/* column name       */
    short    type;			/* column type       */
    short    length;			/* column length     */
    short    scale;			/* numeric scale     */
    short    prec;			/* numeric precision */
    short    null;			/* nulls permitted   */
    } ColumnDescr;
    
#endif /* !resource	*/    
#endif /* __dbdefsH__	*/
