#include <stdio.h>
#include <io.h>
#include <dos.h>
#include <stdlib.h>
#include <string.h>
#include <limits.h>
#pragma pack(1)

typedef unsigned char BYTE;
typedef struct          /* element symbology word */
{
  unsigned style:3;     /* element line style */
  unsigned weight:5;    /* current line weight */
  unsigned color:8;     /* current color */
} SYMBOLOGY;

typedef struct
{
  unsigned short class:4;     /* element class */
  unsigned short :4;          /* reserved by Intergraph */
  unsigned short l:1;         /* locked */
  unsigned short n:1;         /* new */
  unsigned short m:1;         /* modified */
  unsigned short a:1;         /* attributes present */
  unsigned short r:1;         /* relative to: 0=database 1=screen */
  unsigned short p:1;         /* planar */
  unsigned short s:1;         /* 0=snappable, 1=nonsnappable */
  unsigned short h:1;         /* hole or solid (usually) */
} PROPERTIES;

typedef struct
{
  unsigned short level:6;    /* level element is on */
  unsigned short igresvd:1;  /* reserved by Intergraph */
  unsigned short complex:1;  /* part of complex element (cell) if set */
  unsigned short type:7;      /* type of element */
  unsigned short deleted:1;  /* set if element is deleted */

  unsigned short words; /* number of words to follow in element */
  unsigned long xlow;   /* range of element - low (reversed longs) */
  unsigned long ylow;
  unsigned long zlow;
  unsigned long xhigh;  /* range of element - high (reversed longs) */
  unsigned long yhigh;
  unsigned long zhigh;        
} ELM_HDR;

typedef struct
{
  unsigned long x;
  unsigned long y;
} POINT;
typedef struct
{
  double x;
  double y;
} DPOINT;

typedef struct
{
  unsigned short grphgrp;  /* graphics group number */
  short attindx;           /* words between this and attribute linkage */
  PROPERTIES props;
  SYMBOLOGY symb;
} DISP_HDR;

typedef struct
{
  unsigned        grid_lock:1;    /* grid lock in effect */
  unsigned        unit_lock:1;    /* unit lock in effect */
  unsigned        rsrv3:1;        /* reserved */
  unsigned        stream_mode:1;  /* point/stream mode */
  unsigned        delay_pattrn:1; /* delayed patterning */
  unsigned        dis_asp:1;      /* display aspect ratio */
  unsigned        rsrv2:2;        /* reserved */
  unsigned        inh_msg:1;      /* inhibit CF, ST and PR messages */
  unsigned        inh_err:1;      /* inhibit err msgs (except fatal) */
  unsigned        rsrv1:4;        /* reserved */
  unsigned        wind_alt:1;     /* display window altered */
  unsigned        overview:1;     /* overview on */
} CNTRLWD;
typedef struct
{
  unsigned      fast_curve:1;   /* fast curve display */
  unsigned      fast_text:1;    /* fast text */
  unsigned      fast_font:1;    /* fast font text */
  unsigned      line_wghts:1;   /* line weights */
  unsigned      patterns:1;     /* pattern display */
  unsigned      text_nodes:1;   /* text node display */
  unsigned      ed_fields:1;    /* enter data field underlines */
  unsigned      on_off:1;     /* view on or off */
  unsigned      delay:1;      /* delay on */
  unsigned      grid:1;     /* grid on */
  unsigned      lev_symb:1;     /* level symbology */
  unsigned      points:1;     /* points */
  unsigned      constructs:1;   /* line constructs */
  unsigned      dimens:1;     /* dimensioning */
  unsigned      fast_cell:1;    /* fast cells */
  unsigned      def:1;      /* def/undef ?? */
} VIEWFLAGS;
typedef struct
{
  VIEWFLAGS     flags;      /* view flags */
  short         levels[4];      /* active levels (64 bit array) */
  POINT         origin;     /* origin (3d point made of longs) */
  unsigned long originZ;
  POINT         delta;      /* delta to other corner of view */
  unsigned long deltaZ;
  double        transmatrx[9];  /* view transformation matrix */
  double        conversion;     /* conversion digitizer units to uors */
  unsigned long activeZ;      /* view active z */
} VIEWINFO;

typedef struct
{
  CNTRLWD       control;       // control word
  short         filler_1[7];
  BYTE          vwfull1;       // view # on scrn 1 (full scrn mode)
  BYTE          vwfull2;       //    -- "" --    2
  VIEWINFO      view[9];       // view information for each view
  unsigned long mastunits;     // master units per design (unsigned)
  unsigned long subpermast;    // sub-units per master unit (unsigned)
  unsigned long uorpersub;     // uors per sub unit (unsigned)
  char          mastname[2];   // master units name
  char          subname[2];    // sub units name
  short         filler_2[206];
} TCB;
typedef struct
{
  BYTE      font;         /* text font used */
  BYTE      just;         /* justification type */
  long      lngthmult;    /* length multiplier */
  long      hghtmult;     /* height multiplier */
  long      rotation;     /* rotation angle */
  POINT     origin;       /* origin */
  BYTE      numchars;     /* # of characters */
  BYTE      edflds;       /* # of enter data fields */
} TEXT_HDR;
typedef struct
{
  long      startang;     /* start angle */
  long      sweepang;     /* sweep angle */
  double    primary;      /* primary axis */
  double    secondary;      /* secondary axis */
  long      rotation;     /* rotation angle */
  DPOINT    origin;     /* origin */
} ARC;
typedef struct
{
  short     celltype;     /* cell type */
  short     attindx;      /* attribute linkage */
  short     name[2];     /* radix-50 cell name */
  unsigned short  numwords;     /* number of words in descriptions */
  short     properties;     /* properties */
  short     dispsymb;     /* display symbology */
  short     class;        /* cell class (always 0) */
  short     levels[4];      /* levels used in cell */
  short     descrip[9];     /* cell description */
} CELL_LIB_HDR;
typedef struct
{
  double a1, a2;
  BYTE   b1, b2, b3, lines;
  int    vert;
  int    brks;
  int    fill[6];
} PL_HDR;
typedef struct
{
  unsigned style:3;
  unsigned weight:5;
  unsigned color:8;
  unsigned useStyle:1;
  unsigned useWeight:1;
  unsigned useColor:1;
  unsigned interArc:1;
  unsigned outerArc:1;
  unsigned Line:1;
  unsigned useClass:1;
  unsigned dmy:1;
  unsigned level:6;
  unsigned class:2;
} PL_CAP;
typedef struct
{
  long dist;
  unsigned style:3;
  unsigned weight:5;
  unsigned color:8;
  unsigned useStyle:1;
  unsigned useWeight:1;
  unsigned useColor:1;
  unsigned interArc:1;
  unsigned outerArc:1;
  unsigned Line:1;
  unsigned useClass:1;
  unsigned dmy:1;
  unsigned level:6;
  unsigned class:2;
} PL_LIN;
typedef struct
{
  long x, y;
  int i1, i2, i3;
  BYTE b1, b2;
} PL_PNT;
typedef struct
{
  long l1, l2, l3;
} PL_BRK;


char *types[] = { "???",
                  "LIBRARY_CELL",
                  "CELL",
                  "LINE",
                  "LINE_STRING",
                  "GROUP_DATA",
                  "SHAPE",
                  "NODE",
                  "DIGITIZER",
                  "DESIGN_FILE (HEADER)",
                  "LEVEL_SYMB",
                  "CURVE",
                  "CONNECTED_STRING",
                  "???",
                  "COMPLEX_SHAPE",
                  "ELLIPSE",
                  "ARC",
                  "TEXT",
                  "SURFACE",
                  "CAPPED_SURFACE",
                  "SUBFIGURE",
                  "B_spline POLE",
                  "POINT_STRING",
                  "TRUNCATED_CONE",
                  "B_spline surface HEADER",
                  "B_spline surface BOUNDARY",
                  "B_spline KNOT",
                  "B_spline CURVE_HEADER",
                  "B_spline WEIGHT FACTOR" };

FILE *infile;
int buff[1024];
TCB tcb;

long SwapLong( long l );
void DispViewInfo( BYTE n );
void VaxToIeee( double *d );
int asciiToRadix( char *input, short *output, short length );
int radixToAscii( unsigned short *input, char *output, short length );

void main( int argc, char *argv[] )
{
  ELM_HDR ehdr;
  DISP_HDR dhdr;
  TEXT_HDR thdr;
  ARC arc;
  POINT point;
  CELL_LIB_HDR clhdr;
  char ch, name[7], descr[28];
  long l;
  int i = 0, j, k, n, p, typ, nv;
  double start, sweep, rot;
  PL_HDR PlHdr;
  PL_CAP PlCap;
  PL_LIN PlLin;
  PL_PNT PlPnt;
  PL_BRK PlBrk;

  if( (infile = fopen( argv[1], "rb" )) == NULL )
    exit( 1 );
 
  while( 1 )
  {
    fread( &n, sizeof( int ), 1, infile );
    if( n == -1 )
      break;
    fseek( infile, -2L, SEEK_CUR );
    fread( &ehdr, sizeof( ELM_HDR ), 1, infile );
    typ = ehdr.type;
    i++;
    j = typ > 28 ? 0 : typ;
    if( typ == 36 )
    {
      printf( "\n%d.zaznam - typ: %d (POLYLINE), uroven: %d, delka: %d", i,
                                                          typ,
                                                          ehdr.level,
                                                          ehdr.words + 2 );
      if( ehdr.complex )
        printf( " (C)" );
      if( ehdr.deleted )
        printf( " (D)" );
      printf( "\n" );
    }
    ehdr.xlow = ( SwapLong( ehdr.xlow ) - LONG_MAX ) - 1;
    ehdr.ylow = ( SwapLong( ehdr.ylow ) - LONG_MAX ) - 1;
    ehdr.zlow = ( SwapLong( ehdr.zlow ) - LONG_MAX ) - 1;
    ehdr.xhigh = ( SwapLong( ehdr.xhigh ) - LONG_MAX ) - 1;
    ehdr.yhigh = ( SwapLong( ehdr.yhigh ) - LONG_MAX ) - 1;
    ehdr.zhigh = ( SwapLong( ehdr.zhigh ) - LONG_MAX ) - 1;
    if( typ == 36 )
      printf( " limity [%ld,%ld,%ld:%ld,%ld,%ld]\n", ehdr.xlow,
                                                   ehdr.ylow,
                                                   ehdr.zlow,
                                                   ehdr.xhigh,
                                                   ehdr.yhigh,
                                                   ehdr.zhigh );
    n = 12;
    k = 0;
    if( typ == 36 )
    {
      fread( &dhdr, sizeof( DISP_HDR ), 1, infile );
      n = 16;
      printf( "graf grp: %d, attr.index: %d", dhdr.grphgrp,
                                              dhdr.attindx );
      if( k = ehdr.words - dhdr.attindx - 14 )
        printf( ", slov pro atributy: %d", k );
      printf( "\ntyp: %d, barva: %d, sirka: %d\n", dhdr.symb.style,
                                                   dhdr.symb.color,
                                                   dhdr.symb.weight );
      printf( "class: %d, locked: %d, new: %d, modified: %d, attributes: %d\n", dhdr.props.class,
                                                                                dhdr.props.l,
                                                                                dhdr.props.n,
                                                                                dhdr.props.m,
                                                                                dhdr.props.a );
      printf( "relative: %d, planar: %d, snappable: %d, hole: %d\n", dhdr.props.r,
                                                                     dhdr.props.p,
                                                                     dhdr.props.s,
                                                                     dhdr.props.h );
    }
    if( typ == 36 )
    {
      fread( &PlHdr, sizeof( PL_HDR ), 1, infile );
      n += sizeof( PL_HDR ) / 2;
      VaxToIeee( &(PlHdr.a1) );
      VaxToIeee( &(PlHdr.a2) );
      printf( "a1: %.3lf, a2: %.3lf, b1: %d, b2: %d, b3: %d, lines: %d, vert: %d, break: %d\n", PlHdr.a1,
                                                                                     PlHdr.a2,
                                                                                     PlHdr.b1,
                                                                                     PlHdr.b2,
                                                                                     PlHdr.b3,
                                                                                     PlHdr.lines,
                                                                                     PlHdr.vert,
                                                                                     PlHdr.brks );
      printf( "%d %d %d %d %d %d\n", PlHdr.fill[0],
                                     PlHdr.fill[1],
                                     PlHdr.fill[2],
                                     PlHdr.fill[3],
                                     PlHdr.fill[4],
                                     PlHdr.fill[5] );
      for( j = 0; j < 3; j++ )
      {
        fread( &PlCap, sizeof( PL_CAP ), 1, infile );
        n += sizeof( PL_CAP ) / 2;
        printf( " %d. style: %d, weight: %d, color: %d, class: %d, level: %d\n", j + 1,
                                                                                 PlCap.style,
                                                                                 PlCap.weight,
                                                                                 PlCap.color,
                                                                                 PlCap.class,
                                                                                 PlCap.level );
        printf( "dmy: %d, useClass: %d, Line: %d, outerArc: %d, interArc: %d, useColor: %d, useWeight: %d, useStyle: %d\n", PlCap.dmy,
                                                                                                                            PlCap.useClass,
                                                                                                                            PlCap.Line,
                                                                                                                            PlCap.outerArc,
                                                                                                                            PlCap.interArc,
                                                                                                                            PlCap.useColor,
                                                                                                                            PlCap.useWeight,
                                                                                                                            PlCap.useStyle );

      }
      for( j = 0; j < 6; j++ )
      {
        fread( &l, sizeof( long ), 1, infile );
        n += 2;
        printf( "%d. %ld\n", j + 1, l );
      }
      for( j = 0; j < (int)(PlHdr.lines); j++ )
      {
        fread( &PlLin, sizeof( PL_LIN ), 1, infile );
        n += sizeof( PL_LIN ) / 2;
        printf( "%d. dist: %ld, style: %d, weight: %d, color: %d, class: %d, level: %d\n", j + 1,
                                                                                           PlLin.dist,
                                                                                           PlLin.style,
                                                                                           PlLin.weight,
                                                                                           PlLin.color,
                                                                                           PlLin.class,
                                                                                           PlLin.level );

        printf( "dmy: %d, useClass: %d, Line: %d, outerArc: %d, interArc: %d, useColor: %d, useWeight: %d, useStyle: %d\n", PlLin.dmy,
                                                                                                                            PlLin.useClass,
                                                                                                                            PlLin.Line,
                                                                                                                            PlLin.outerArc,
                                                                                                                            PlLin.interArc,
                                                                                                                            PlLin.useColor,
                                                                                                                            PlLin.useWeight,
                                                                                                                            PlLin.useStyle );

      }
      for( j = 0; j < PlHdr.vert; j++ )
      {
        fread( &PlPnt, sizeof( PL_PNT ), 1, infile );
        n += sizeof( PL_PNT ) / 2;
        printf( " %d. x: %ld, y: %ld, l1: %d, l2: %d, l3: %d, b1: %d, b2: %d\n", j + 1,
                                                           PlPnt.x,
                                                           PlPnt.y,
                                                           PlPnt.i1,
                                                           PlPnt.i2,
                                                           PlPnt.i3,
                                                           PlPnt.b1,
                                                           PlPnt.b2 );
      }
      for( j = 0; j < PlHdr.brks; j++ )
      {
        fread( &PlBrk, sizeof( PL_BRK ), 1, infile );
        n += sizeof( PL_PNT ) / 2;
        printf( " %d. %ld, %ld, %ld\n", j + 1,
                                                           PlBrk.l1,
                                                           PlBrk.l2,
                                                           PlBrk.l3 );
      }
    }
/*
    else if( typ == 1 )
    {
      fread( &clhdr, sizeof( CELL_LIB_HDR ), 1, infile );
      radixToAscii( clhdr.name, name, 6 );
      name[6] = 0;
      radixToAscii( clhdr.descrip, descr, 27 );
      descr[27] = 0;
      printf( " \"%s\", \"%s\"\n", name, descr );
      printf( " celltype: %d, attindx: %d, numwords: %d, properties: %.4X\n", clhdr.celltype,
                                                                              clhdr.attindx,
                                                                              clhdr.numwords,
                                                                              clhdr.properties );
      printf( " dispsymb: %.4X, class: %d, levels[: %.4X %.4X %.4X %.4X\n", clhdr.dispsymb,
                                                                            clhdr.class,
                                                                            clhdr.levels[0],
                                                                            clhdr.levels[1],
                                                                            clhdr.levels[2],
                                                                            clhdr.levels[3] );
      n += sizeof( CELL_LIB_HDR ) / 2;
      _getch();
    }
    else if( typ == 4 )
    {
      fread( &nv, sizeof( int ), 1, infile );
      printf( "pocet bodu: %d\n", nv );
      n++;
      for( j = 0; j < nv; j++ )
      {
        fread( &point, sizeof( POINT ), 1, infile );
        point.x = SwapLong( point.x );
        point.y = SwapLong( point.y );
        printf( "        [ %ld, %ld]\n", point.x, point.y );
        n += 4;
      }
      printf( "\n" );
    }
    else if( typ == 14 )
    {
      fread( &j, sizeof( int ), 1, infile );
      fread( &nv, sizeof( int ), 1, infile );
      printf( "total len: %d, pocet prvku: %d\n", j, nv );
      n += 2;
    }
    else if( typ == 16 )
    {
      fread( &arc, sizeof( ARC ), 1, infile );
      arc.startang = SwapLong( arc.startang );
      start = (double)arc.startang / 360000.0;
      arc.sweepang = SwapLong( arc.sweepang );
      sweep = (double)( arc.sweepang & 0x7FFFFFFF )/ 360000.0;
      arc.rotation = SwapLong( arc.rotation );
      rot = (double)arc.rotation / 360000.0;
      VaxToIeee( &(arc.origin.x) );
      VaxToIeee( &(arc.origin.y) );
      VaxToIeee( &(arc.primary) );
      VaxToIeee( &(arc.secondary) );
      printf( "  Origin: [ %.3lf, %.3lf]\n", arc.origin.x, arc.origin.y );
      printf( "  Axes:  %.3lf, %.3lf\n", arc.primary, arc.secondary );
      printf( "  Start: %.3lf, sweep: %.3lf (%c), rot: %.3lf\n", start, sweep,
                            arc.sweepang & 0x80000000 ? '-' : '+', rot );
      n += sizeof( ARC ) / 2;
    }
    else if( typ == 17 )
    {
      fread( &thdr, sizeof( TEXT_HDR ), 1, infile );
      thdr.lngthmult = SwapLong( thdr.lngthmult );
      thdr.hghtmult = SwapLong( thdr.hghtmult );
      thdr.rotation = SwapLong( thdr.rotation );
      rot = (double)thdr.rotation / 360000.0;
      thdr.origin.x = SwapLong( thdr.origin.x );
      thdr.origin.y = SwapLong( thdr.origin.y );
      printf( "  Origin: [ %ld, %ld]\n", thdr.origin.x, thdr.origin.y );

      printf( "  font: %d, just: %d, numchars: %d, edflds: %d\n", thdr.font,
                                                                  thdr.just,
                                                                  thdr.numchars,
                                                                  thdr.edflds );
      printf( "  lngthmult: %ld, hghtmult: %ld, rotation: %.3lf\n", thdr.lngthmult,
                                                                  thdr.hghtmult,
                                                                  rot );
      printf( "  \"" );
      for( j = 0; j < thdr.numchars; j++ )
      {
        fread( &ch, 1, 1, infile );
        printf( "%c", ch );
      }
      printf( "\"\n" );
      if( thdr.numchars & 1 )
      {
        thdr.numchars++;
        fread( &ch, 1, 1, infile );
      }
      n += ( thdr.numchars + sizeof( TEXT_HDR ) ) / 2;
    }
*/
    if( ( n = ehdr.words - n ) > 0 )
      fread( buff, 2, n, infile );
    if( typ == 5 )
      for( i = 0; i < n; i++ )
        printf( "%d: %d\n", i + 1, buff[i] );
    if( k && !( k & 3 ) )
    {
      for( j = n - k, p = 0; j < n; j++, p++ )
        printf( "%.4X ", buff[j] );
      printf( "\n" );
    }

  }
  fclose( infile );
}
void DispViewInfo( BYTE n )
{
  int i = n - 1;
  unsigned long xo, yo, xd, yd;

  printf( "Pohled %d", n );
  xo = SwapLong( tcb.view[i].origin.x );
  yo = SwapLong( tcb.view[i].origin.y );
  printf( "  pocatek: %ld, %ld\n" , xo, yo );

  xd = SwapLong( tcb.view[i].delta.x );
  yd = SwapLong( tcb.view[i].delta.y );
  printf( "            delta: %ld, %ld\n" , xd, yd );
}
long SwapLong( long l )
{
  _asm
  {
    mov   ax, word ptr l + 2
    mov   dx, word ptr l
  }
}
void VaxToIeee( double *d )
{
  _asm
  {
      push  di
      push  si
      push  ds
      lds   si,[bp+06]
      mov   ax,word ptr [si]
      mov   bx,word ptr [si+02]
      mov   cx,word ptr [si+04]
      mov   dx,word ptr [si+06]
      mov   di,ax
      and   di,0x8000
      and   ax,0x7fff
      shr   ax,1
      rcr   bx,1
      rcr   cx,1
      rcr   dx,1
      shr   ax,1
      rcr   bx,1
      rcr   cx,1
      rcr   dx,1
      shr   ax,1
      rcr   bx,1
      rcr   cx,1
      rcr   dx,1
      or    ax,ax
      je    Nula
      add   ax,0x37e0
Nula:
      or    ax,di
      mov   word ptr [si],dx
      mov   word ptr [si+02],cx
      mov   word ptr [si+04],bx
      mov   word ptr [si+06],ax
      pop   ds
      pop   si
      pop   di
  }
}
int asciiToRadix( char *input, short *output, short length )
{
  int status, c, i, n;

  *output = status = i = n = 0;
  while( 1 )
  {
    c = *input++;
    if( c == ' ' )
      c = 0;
    else if( c >= 'A' && c <= 'Z' )
      c = c - 64;
    else if( c == '$' )
      c = 27;
    else if( c == '.' )
      c = 28;
    else if( c >= '0' && c <= '9' )
      c = c - 18;
    else
    {
      c = 0;
      status = -1;
    }
    *output = 40 * *output + c;
    if( ++i == length )
      break;
    if( ++n == 3 )
      *(++output) = n = 0;
  }

  while( ++n != 3 )
    *output *= 40;

  return status;
}
int radixToAscii( unsigned short *input, char *output, short length )
{
  int status, c, i;
  unsigned short n;

  status = 0;
  for( i = 0; i < length; i++ )
  {
    switch( i % 3 )
    {
      case 0:
        n = *input++;
        c = n / 1600;
        n %= 1600;
        break;
      case 1:
        c = n / 40;
        n %= 40;
        break;
      case 2:
        c = n;
    }
    if( c == 0 )
      c = ' ';
    else if( c >= 1 && c <= 26 )
      c = c + 64;
    else if( c == 27 )
      c = '$';
    else if( c == 28 )
      c = '.';
    else if( c >= 30 && c <= 39 )
      c = c + 18;
    else
    {
      c = ' ';
      status = -1;
    }
    *output++ = (char)c;
  }

  return status;
}
/*
void printTcb( TCB *tcb )
{
  unsigned fofs = _FP_OFF( tcb );
  void *adr;

  printf( "control:      %Fp   28\n", &(tcb->control)     );
  printf( "rndunit:      %Fp", &(tcb->rndunit)     );  adr = &(tcb->rndunit);
  printf( " %4d\n", _FP_OFF( adr ) - fofs + 28 );
  printf( "uorgrid:      %Fp", &(tcb->uorgrid)     );  adr = &(tcb->uorgrid);
  printf( " %4d\n", _FP_OFF( adr ) - fofs + 28 );
  printf( "refgrid:      %Fp", &(tcb->refgrid)     );   adr = &(tcb->refgrid)     ;
  printf( " %4d\n", _FP_OFF( adr ) - fofs + 28 );
  printf( "vwfull1:      %Fp", &(tcb->vwfull1)     );   adr = &(tcb->vwfull1)     ;
  printf( " %4d\n", _FP_OFF( adr ) - fofs + 28 );
  printf( "vwfull2:      %Fp", &(tcb->vwfull2)     );   adr = &(tcb->vwfull2)     ;
  printf( " %4d\n", _FP_OFF( adr ) - fofs + 28 );
  printf( "view:         %Fp", &(tcb->view)        );   adr = &(tcb->view)        ;
  printf( " %4d\n", _FP_OFF( adr ) - fofs + 28 );
  printf( "mastunits:    %Fp", &(tcb->mastunits)   );   adr = &(tcb->mastunits)   ;
  printf( " %4d\n", _FP_OFF( adr ) - fofs + 28 );
  printf( "subpermast:   %Fp", &(tcb->subpermast)  );   adr = &(tcb->subpermast)  ;
  printf( " %4d\n", _FP_OFF( adr ) - fofs + 28 );
  printf( "uorpersub:    %Fp", &(tcb->uorpersub)   );   adr = &(tcb->uorpersub)   ;
  printf( " %4d\n", _FP_OFF( adr ) - fofs + 28 );
  printf( "mastname:     %Fp", &(tcb->mastname)    );   adr = &(tcb->mastname)    ;
  printf( " %4d\n", _FP_OFF( adr ) - fofs + 28 );
  printf( "subname:      %Fp", &(tcb->subname)     );   adr = &(tcb->subname)     ;
  printf( " %4d\n", _FP_OFF( adr ) - fofs + 28 );
  printf( "actangle:     %Fp", &(tcb->actangle)    );   adr = &(tcb->actangle)    ;
  printf( " %4d\n", _FP_OFF( adr ) - fofs + 28 );
  printf( "anglernd:     %Fp", &(tcb->anglernd)    );   adr = &(tcb->anglernd)    ;
  printf( " %4d\n", _FP_OFF( adr ) - fofs + 28 );
  printf( "xactscle:     %Fp", &(tcb->xactscle)    );   adr = &(tcb->xactscle)    ;
  printf( " %4d\n", _FP_OFF( adr ) - fofs + 28 );
  printf( "yactscle:     %Fp", &(tcb->yactscle)    );   adr = &(tcb->yactscle)    ;
  printf( " %4d\n", _FP_OFF( adr ) - fofs + 28 );
  printf( "zactscle:     %Fp", &(tcb->zactscle)    );   adr = &(tcb->zactscle)    ;
  printf( " %4d\n", _FP_OFF( adr ) - fofs + 28 );
  printf( "rndscle:      %Fp", &(tcb->rndscle)     );   adr = &(tcb->rndscle)     ;
  printf( " %4d\n", _FP_OFF( adr ) - fofs + 28 );
  printf( "chheight:     %Fp", &(tcb->chheight)    );   adr = &(tcb->chheight)    ;
  printf( " %4d\n", _FP_OFF( adr ) - fofs + 28 );
  printf( "chwidth:      %Fp", &(tcb->chwidth)     );   adr = &(tcb->chwidth)     ;
  printf( " %4d\n", _FP_OFF( adr ) - fofs + 28 );
  printf( "nodespace:    %Fp", &(tcb->nodespace)   );   adr = &(tcb->nodespace)   ;
  printf( " %4d\n", _FP_OFF( adr ) - fofs + 28 );
  printf( "nodelen:      %Fp", &(tcb->nodelen)     );   adr = &(tcb->nodelen)     ;
  printf( " %4d\n", _FP_OFF( adr ) - fofs + 28 );
  printf( "nodejust:     %Fp", &(tcb->nodejust)    );   adr = &(tcb->nodejust)    ;
  printf( " %4d\n", _FP_OFF( adr ) - fofs + 28 );
  printf( "actfont:      %Fp", &(tcb->actfont)     );   adr = &(tcb->actfont)     ;
  printf( " %4d\n", _FP_OFF( adr ) - fofs + 28 );
  printf( "anglefmt:     %Fp", &(tcb->anglefmt)    );   adr = &(tcb->anglefmt)    ;
  printf( " %4d\n", _FP_OFF( adr ) - fofs + 28 );
  printf( "graphic:      %Fp", &(tcb->graphic)     );   adr = &(tcb->graphic)     ;
  printf( " %4d\n", _FP_OFF( adr ) - fofs + 28 );
  printf( "txtjust:      %Fp", &(tcb->txtjust)     );   adr = &(tcb->txtjust)     ;
  printf( " %4d\n", _FP_OFF( adr ) - fofs + 28 );
  printf( "rfcntrl:      %Fp", &(tcb->rfcntrl)     );   adr = &(tcb->rfcntrl)     ;
  printf( " %4d\n", _FP_OFF( adr ) - fofs + 28 );
  printf( "tentmode:     %Fp", &(tcb->tentmode)    );   adr = &(tcb->tentmode)    ;
  printf( " %4d\n", _FP_OFF( adr ) - fofs + 28 );
  printf( "tentsubmode:  %Fp", &(tcb->tentsubmode) );   adr = &(tcb->tentsubmode) ;
  printf( " %4d\n", _FP_OFF( adr ) - fofs + 28 );
  printf( "azimuth:      %Fp", &(tcb->azimuth)     );   adr = &(tcb->azimuth)     ;
  printf( " %4d\n", _FP_OFF( adr ) - fofs + 28 );
  printf( "ad1:          %Fp", &(tcb->ad1)         );   adr = &(tcb->ad1)         ;
  printf( " %4d\n", _FP_OFF( adr ) - fofs + 28 );
  printf( "props:        %Fp", &(tcb->props)       );   adr = &(tcb->props)       ;
  printf( " %4d\n", _FP_OFF( adr ) - fofs + 28 );
  printf( "symbology:    %Fp", &(tcb->symbology)   );   adr = &(tcb->symbology)   ;
  printf( " %4d\n", _FP_OFF( adr ) - fofs + 28 );
  printf( "fbfdcn:       %Fp", &(tcb->fbfdcn)      );   adr = &(tcb->fbfdcn)      ;
  printf( " %4d\n", _FP_OFF( adr ) - fofs + 28 );
  printf( "low:          %Fp", &(tcb->low)         );   adr = &(tcb->low)         ;
  printf( " %4d\n", _FP_OFF( adr ) - fofs + 28 );
  printf( "high:         %Fp", &(tcb->high)        );   adr = &(tcb->high)        ;
  printf( " %4d\n", _FP_OFF( adr ) - fofs + 28 );
  printf( "globorg:      %Fp", &(tcb->globorg)     );   adr = &(tcb->globorg)     ;
  printf( " %4d\n", _FP_OFF( adr ) - fofs + 28 );
  printf( "celllib:      %Fp", &(tcb->celllib)     );   adr = &(tcb->celllib)     ;
  printf( " %4d\n", _FP_OFF( adr ) - fofs + 28 );
  printf( "cellendsec:   %Fp", &(tcb->cellendsec)  );   adr = &(tcb->cellendsec)  ;
  printf( " %4d\n", _FP_OFF( adr ) - fofs + 28 );
  printf( "cellendbyte:  %Fp", &(tcb->cellendbyte) );   adr = &(tcb->cellendbyte) ;
  printf( " %4d\n", _FP_OFF( adr ) - fofs + 28 );
  printf( "grgrpbas:     %Fp", &(tcb->grgrpbas)    );   adr = &(tcb->grgrpbas)    ;
  printf( " %4d\n", _FP_OFF( adr ) - fofs + 28 );
  printf( "nodebase:     %Fp", &(tcb->nodebase)    );   adr = &(tcb->nodebase)    ;
  printf( " %4d\n", _FP_OFF( adr ) - fofs + 28 );
  printf( "canode:       %Fp", &(tcb->canode)      );   adr = &(tcb->canode)      ;
  printf( " %4d\n", _FP_OFF( adr ) - fofs + 28 );
  printf( "parent:       %Fp", &(tcb->parent)      );   adr = &(tcb->parent)      ;
  printf( " %4d\n", _FP_OFF( adr ) - fofs + 28 );
  printf( "sysclass:     %Fp", &(tcb->sysclass)    );   adr = &(tcb->sysclass)    ;
  printf( " %4d\n", _FP_OFF( adr ) - fofs + 28 );
  printf( "usercmd:      %Fp", &(tcb->usercmd)     );   adr = &(tcb->usercmd)     ;
  printf( " %4d\n", _FP_OFF( adr ) - fofs + 28 );
  printf( "ucdata:       %Fp", &(tcb->ucdata)      );   adr = &(tcb->ucdata)      ;
  printf( " %4d\n", _FP_OFF( adr ) - fofs + 28 );
  printf( "dmrsfile:     %Fp", &(tcb->dmrsfile)    );   adr = &(tcb->dmrsfile)    ;
  printf( " %4d\n", _FP_OFF( adr ) - fofs + 28 );
  printf( "oldref:       %Fp", &(tcb->oldref)      );   adr = &(tcb->oldref)      ;
  printf( " %4d\n", _FP_OFF( adr ) - fofs + 28 );
  printf( "linkgen:      %Fp", &(tcb->linkgen)     );   adr = &(tcb->linkgen)     ;
  printf( " %4d\n", _FP_OFF( adr ) - fofs + 28 );
  printf( "dmrsflg:      %Fp", &(tcb->dmrsflg)     );   adr = &(tcb->dmrsflg)     ;
  printf( " %4d\n", _FP_OFF( adr ) - fofs + 28 );
  printf( "tutview:      %Fp", &(tcb->tutview)     );   adr = &(tcb->tutview)     ;
  printf( " %4d\n", _FP_OFF( adr ) - fofs + 28 );
  printf( "iactfl:       %Fp", &(tcb->iactfl)      );   adr = &(tcb->iactfl)      ;
  printf( " %4d\n", _FP_OFF( adr ) - fofs + 28 );
  printf( "ad2:          %Fp", &(tcb->ad2)         );   adr = &(tcb->ad2)         ;
  printf( " %4d\n", _FP_OFF( adr ) - fofs + 28 );
  printf( "dynamicorg:   %Fp", &(tcb->dynamicorg)  );   adr = &(tcb->dynamicorg)  ;
  printf( " %4d\n", _FP_OFF( adr ) - fofs + 28 );
  printf( "dynamicext:   %Fp", &(tcb->dynamicext)  );   adr = &(tcb->dynamicext)  ;
  printf( " %4d\n", _FP_OFF( adr ) - fofs + 28 );
  printf( "dytab:        %Fp", &(tcb->dytab)       );   adr = &(tcb->dytab)       ;
  printf( " %4d\n", _FP_OFF( adr ) - fofs + 28 );
  printf( "cntrl1:       %Fp", &(tcb->cntrl1)      );   adr = &(tcb->cntrl1)      ;
  printf( " %4d\n", _FP_OFF( adr ) - fofs + 28 );
  printf( "ad3:          %Fp", &(tcb->ad3)         );   adr = &(tcb->ad3)         ;
  printf( " %4d\n", _FP_OFF( adr ) - fofs + 28 );
  printf( "auxtype:      %Fp", &(tcb->auxtype)     );   adr = &(tcb->auxtype)     ;
  printf( " %4d\n", _FP_OFF( adr ) - fofs + 28 );
  printf( "dyndwnext:    %Fp", &(tcb->dyndwnext)   );   adr = &(tcb->dyndwnext)   ;
  printf( " %4d\n", _FP_OFF( adr ) - fofs + 28 );
  printf( "fbfdcn2:      %Fp", &(tcb->fbfdcn2)     );   adr = &(tcb->fbfdcn2)     ;
  printf( " %4d\n", _FP_OFF( adr ) - fofs + 28 );
  printf( "reserv:       %Fp", &(tcb->reserv)      );   adr = &(tcb->reserv)      ;
  printf( " %4d\n", _FP_OFF( adr ) - fofs + 28 );
  printf( "tcbend:       %Fp", &(tcb->tcbend)      );   adr = &(tcb->tcbend)      ;
  printf( " %4d\n", _FP_OFF( adr ) - fofs + 28 );
}
*/
