unit T_selare;

interface

uses
   Controls,
// KERNEL
   Elements,
   Tools,
// SPACE
   e_Space,
// GUI
   M_InterA,
   t_InterA,
// EDITGRAP
   i_Choice,
   t_SpPick
   ;

type


TObjectSelector = class(TInteractiveTool)
private
  FDontViewProps: boolean;
  FDontViewDBF  : boolean;
  FOnSelect: TTool;
  FShowProperties: boolean;
  FPixelTolerance: integer;

  procedure DoOnSelect(SelectedObject: TElement);
protected
  procedure InterpretIt(Element: TElement; aX, aY: integer);
public
  constructor Create; override;
published
  property ShowProperties: boolean read FShowProperties write FShowProperties;
  property DontViewProps: boolean read FDontViewProps write FDontViewProps;
  property DontViewDBF: boolean read FDontViewDBF write FDontViewDBF;
  property onSelect: TTool read FOnSelect write FOnSelect;
  property PixelTolerance: integer read FPixelTolerance write FPixelTolerance;
end;


TPointSelector = class(TObjectSelector)
protected
   function   GetCursor: TCursor; override;
 public
   procedure On_tm_MouseUp(var aMessage: TOrbisMouseEvent; var Handled: boolean); override;
end;


TLineSelector = class(TObjectSelector)
protected
   function   GetCursor: TCursor; override;
 public
   procedure On_tm_MouseUp(var aMessage: TOrbisMouseEvent; var Handled: boolean); override;
end;


TAreaSelector = class(TObjectSelector)
protected
   function   GetCursor: TCursor; override;
 public
   procedure On_tm_MouseUp(var aMessage: TOrbisMouseEvent; var Handled: boolean); override;
end;


TPointCollector = class(TObjectSelector)
private
   FX: double;
   FY: double;
protected
   function   GetCursor: TCursor; override;
public
   procedure On_tm_MouseUp(var aMessage: TOrbisMouseEvent; var Handled: boolean); override;
published
   property X: double read FX write FX;
   property Y: double read FY write FY;
end;

{
TSelectToDDE = class(TTool)
public
   procedure DoIt; override;
end;
}

implementation

uses
   { Test }
   SysUtils, Dialogs, WinTypes, Classes,
   Graphics,

// KERNEL
   e_Class,
// MAIN
   M_Proj,
   u_Cursor;


{
procedure TSelectToDDE.DoIt;
begin
   if (Input <> nil) then
   begin
      F_StartForm.DDEServerItem.Text := 'SELECTED='+NameParameter['FTM'].Name;
      Output := Input;
   end;
end;

}                     { TObjectSelector }

constructor TObjectSelector.Create;
begin
  inherited Create;
  FPixelTolerance:= 5
end;


procedure TObjectSelector.DoOnSelect(SelectedObject: TElement);
begin
  PostMsg(tm_SelectOK, nil, SelectedObject, false);
end;


procedure TObjectSelector.InterpretIt(Element: TElement; aX, aY: integer);
var
   Interpret: TChoicedElementInterpret;
   aPoint: TPoint;
begin
  Interpret:= TChoicedElementInterpret.Create;
  Interpret.Input:= Element;
  Interpret.DontViewProps:= DontViewProps;
  Interpret.DontViewDBF  := DontViewDBF;
  Interpret.Left:= 10;
  Interpret.Top := 10;
  try
    aPoint:= TFormToolManager(FindOwnerClass(TFormToolManager)).Form.ClientToScreen(Point(aX, aY));
    Interpret.Left:= aPoint.X + 10;
    Interpret.Top := aPoint.Y + 10;
  except
    aPoint:= aPoint;
  end;

  Interpret.DoIt;
  Interpret.Free
end;

function  TLineSelector.GetCursor: TCursor;
begin
   result := cr_SelectLine;
end;


procedure TLineSelector.On_tm_MouseUp(var aMessage: TOrbisMouseEvent; var Handled: boolean);
var
   HledacLinie: TPickUpPolyLine;
   Line: TPolyLineElement;
   //X, Y: longInt;

   procedure KolmyKrizek(aX, aY: double; Pixlu: integer);
   var
     Vel: double;
   begin
     Vel:= Device.Projection.GetModelLength(Pixlu);
     Device.MoveTo(aX-Vel, aY);
     Device.LineTo(aX+Vel, aY);
     Device.MoveTo(aX, aY-Vel);
     Device.LineTo(aX, aY+Vel);
   end;


   procedure PricnyKrizek(aX, aY: double; Pixlu: integer);
   var
     Vel: double;
   begin
     Vel:= Device.Projection.GetModelLength(Pixlu);
     Device.MoveTo(aX-Vel, aY-Vel);
     Device.LineTo(aX+Vel, aY+Vel);
     Device.MoveTo(aX+Vel, aY-Vel);
     Device.LineTo(aX-Vel, aY+Vel);
   end;


   procedure VyznacLinii;
   var
     I: longInt;
   begin
     Device.Pen.Color:= clRed;
     Device.Pen.Style:= psSolid;
     try
       PricnyKrizek(Line.Xi[0], Line.Yi[0], 5);
       KolmyKrizek(Line.Xi[0], Line.Yi[0], 5);
       for I:= 1 to Line.Count-1 do
         PricnyKrizek(Line.Xi[I], Line.Yi[I], 3);

       Device.Pen.Color:= clWhite;
       Device.Pen.Style:= psSolid;
       PricnyKrizek(Line.X, Line.Y, 6);
       Device.Pen.Color:= clBlack;
       KolmyKrizek(Line.X, Line.Y, 6);
     except
     end
   end;

begin
    Handled := true;
    if aMessage.Button = mbLeft then
    begin
      try
       HledacLinie:= TPickUpPolyLine.Create;
       HledacLinie.Tolerance:= Device.Projection.GetModelLength(PixelTolerance);
       HledacLinie.X:= Device.Projection.GetModelX(aMessage.x);
       HledacLinie.Y:= Device.Projection.GetModelY(aMessage.y);
       HledacLinie.Input:= Self.Input;
       HledacLinie.DoIt;
       Line:= TPolyLineElement(HledacLinie.Output);
       HledacLinie.Free;
       if Line <> nil then
       begin
         VyznacLinii;
         if ShowProperties then InterpretIt(Line, aMessage.x, aMessage.y);
         Output := Line;

         DoOnSelect(Line);

         if OnSelect <> nil then
         begin
            Onselect.Input := Line;
            OnSelect.NameParameter['FTM']:= TTools(Owners[0]).Owner;
            OnSelect.DoIt;
         end;
       end;
      except
        on E: Exception do
         ShowMessage('Hleda linie: ' + E.Message);
      end;
    end;
end;




function  TPointSelector.GetCursor: TCursor;
begin
   result := cr_SelectPoint;
end;


procedure TPointSelector.On_tm_MouseUp(var aMessage: TOrbisMouseEvent; var Handled: boolean);
var
   HledacBodu: TPickUpPoint;
   Bod: TPointElement;

   procedure KolmyKrizek(aX, aY: double; Pixlu: integer);
   var
     Vel: double;
   begin
     Vel:= Device.Projection.GetModelLength(Pixlu);
     Device.MoveTo(aX-Vel, aY);
     Device.LineTo(aX+Vel, aY);
     Device.MoveTo(aX, aY-Vel);
     Device.LineTo(aX, aY+Vel);
   end;


   procedure PricnyKrizek(aX, aY: double; Pixlu: integer);
   var
     Vel: double;
   begin
     Vel:= Device.Projection.GetModelLength(Pixlu);
     Device.MoveTo(aX-Vel, aY-Vel);
     Device.LineTo(aX+Vel, aY+Vel);
     Device.MoveTo(aX+Vel, aY-Vel);
     Device.LineTo(aX-Vel, aY+Vel);
   end;


   procedure VyznacBod;
   begin
     Device.Pen.Color:= clRed;
     Device.Pen.Style:= psSolid;
     try
       PricnyKrizek(Bod.X, Bod.Y, 5);
       KolmyKrizek(Bod.X, Bod.Y, 5);
     except
     end
   end;

begin
    Handled := true;
    if aMessage.Button = mbLeft then
    begin
      try
       HledacBodu:= TPickUpPoint.Create;
       HledacBodu.Tolerance:= Device.Projection.GetModelLength(PixelTolerance);
       HledacBodu.X:= Device.Projection.GetModelX(aMessage.x);
       HledacBodu.Y:= Device.Projection.GetModelY(aMessage.y);
       HledacBodu.Input:= Self.Input;
       HledacBodu.DoIt;
       Bod:= TPointElement(HledacBodu.Output);
       HledacBodu.Free;
       if Bod <> nil then
       begin
         VyznacBod;
         if ShowProperties then InterpretIt(Bod, aMessage.x, aMessage.y);
         Output := Bod;

         DoOnSelect(Bod);

         if OnSelect <> nil then
         begin
            Onselect.Input := Bod;
            OnSelect.NameParameter['FTM']:= TTools(Owners[0]).Owner;
            OnSelect.DoIt;
         end;
       end;
      except
        on E: Exception do
         ShowMessage('Hleda Bodu: ' + E.Message);
      end;
    end;
end;


{ TPointCollector }

function  TPointCollector.GetCursor: TCursor;
begin
   result := cr_EditUMS;
end;


procedure TPointCollector.On_tm_MouseUp(var aMessage: TOrbisMouseEvent; var Handled: boolean);
begin
    Handled := true;
    if aMessage.Button = mbLeft then
    begin
      try
       X:= Device.Projection.GetModelX(aMessage.x);
       Y:= Device.Projection.GetModelY(aMessage.y);
       begin
         Output := Self;
         if OnSelect <> nil then
         begin
            Onselect.Input := Self;
            OnSelect.NameParameter['FTM']:= TTools(Owners[0]).Owner;
            OnSelect.DoIt;
         end;
       end;
      except
        on E: Exception do
         ShowMessage('Snma Bodu: ' + E.Message);
      end;
    end;
end;


{ TAreaSelector }


function  TAreaSelector.GetCursor: TCursor;
begin
   result := cr_SelectArea;
end;




procedure TAreaSelector.On_tm_MouseUp(var aMessage: TOrbisMouseEvent; var Handled: boolean);
var
   HledacPlochy: TPickUpArea;
   Plocha: TSpaceEntity;
   NPlocha: TAreaElement;

   procedure KolmyKrizek(aX, aY: double; Pixlu: integer);
   var
     Vel: double;
   begin
     Vel:= Device.Projection.GetModelLength(Pixlu);
     Device.MoveTo(aX-Vel, aY);
     Device.LineTo(aX+Vel, aY);
     Device.MoveTo(aX, aY-Vel);
     Device.LineTo(aX, aY+Vel);
   end;


   procedure PricnyKrizek(aX, aY: double; Pixlu: integer);
   var
     Vel: double;
   begin
     Vel:= Device.Projection.GetModelLength(Pixlu);
     Device.MoveTo(aX-Vel, aY-Vel);
     Device.LineTo(aX+Vel, aY+Vel);
     Device.MoveTo(aX+Vel, aY-Vel);
     Device.LineTo(aX-Vel, aY+Vel);
   end;


   procedure VyznacBod;
   begin
     try
       Device.Pen.Color:= clRed;
       PricnyKrizek(Plocha.X, Plocha.Y, 5);
       Device.Pen.Color:= clWhite;
       KolmyKrizek(Plocha.X, Plocha.Y, 5);
     except
     end
   end;

begin
    Handled := true;

    if aMessage.Button = mbLeft then
    begin
      try
       HledacPlochy:= TPickUpArea.Create;
       HledacPlochy.X:= Device.Projection.GetModelX(aMessage.x);
       HledacPlochy.Y:= Device.Projection.GetModelY(aMessage.y);
       HledacPlochy.Input:= Self.Input;
       HledacPlochy.DoIt;
       Plocha:= TSpaceEntity(HledacPlochy.Output);
       HledacPlochy.Free;

       if not (Plocha is TAreaElement) then exit;

       if (Plocha is TPolyLineAreaElement) then
          NPlocha:= TPolyLineAreaElement(Plocha).CreateAreaElement
       else
         NPlocha:= TAreaElement(Plocha);

       Device.Brush.Color:= clWhite;
       Device.Brush.Style:= bsBDiagonal;
       Device.Pen.Color:= clRed;
       Device.Pen.Style:= psClear;
       Device.PolyPolygon(NPlocha);
       Device.Brush.Color:= clBlack;
       Device.Brush.Style:= bsFDiagonal;
       Device.Pen.Color:= clRed;
       Device.Pen.Style:= psSolid;
       Device.PolyPolygon(NPlocha);
       VyznacBod;

       if (Plocha is TPolyLineAreaElement) then
       begin
{         NPlocha.Clear; }
          NPlocha.Free;
       end;

       if ShowProperties then InterpretIt(Plocha, aMessage.x, aMessage.y);
       Output := Plocha;

       DoOnSelect(Plocha);

       if OnSelect <> nil then
       begin
          Onselect.Input := Plocha;
          OnSelect.NameParameter['FTM']:= TTools(Owners[0]).Owner;
          OnSelect.DoIt;
       end;

      except
        on E: Exception do
         ShowMessage('Hleda plochy: ' + E.Message);
      end;
    end;
end;


BEGIN
   RegisterOrbisClass(TObjectSelector);
   //RegisterOrbisClass(TPointSelector);
   //RegisterOrbisClass(TAreaSelector);
   RegisterOrbisClass(TLineSelector);
   RegisterOrbisClass(TPointCollector);
{   RegisterOrbisClass(TSelectToDDE); }
END.
