unit T_loader;
{
***********************************************************************
**       [STREAM]                                                    **
**                                                                   **
**             ORBIS ... Stream index                                **
**                                                                   **
**                       Topol Pro 1996                             **
**                                                                   **
***********************************************************************
}

interface

uses
   Classes,
   e_Class,
   Elements,
   Tools;

type

TMagic = Array[0..3] of char;

TOrbisLoader = class(TTool)
private
   FFileName: string;
   FIndex: longInt;
   FActualPropertyIndex: integer;
   FContainer: TQuickEntity;
   FActualElement: TElement;
   FActualClass: TOrbisClass;
   FInstance: TOrbisInstance;
   FClasses: TOrbisClasses;
   FMagic: TMagic;
    FLastElementIndex: integer;
    FContainerEntity: TEntity;
   //FGauge: TF_gaugeForm;
   procedure SetClasses(aValue: TOrbisClasses);
    procedure SetLastElementIndex(const Value: integer);
    procedure SetContainerEntity(const Value: TEntity);
protected
   property  Index: longInt read FIndex write FIndex default 0;
public
   constructor CreateAll(const aFileName: string);
   procedure   DoIt; override;
   procedure   StartGauge; virtual;
   procedure   EndGauge; virtual;
   procedure   UpdateGauge; virtual;
   function    GetGaugeMax: longInt; virtual; abstract;
   function    NumberToPropertyName(aNumber: longInt): string; override;
   procedure   Prepare; virtual;
   procedure   Finish; virtual;
   procedure   PrepareClasses; virtual;
   procedure   LoadInfo; virtual; abstract;
   procedure   LoadClasses; virtual;
   procedure   LoadClass; virtual; abstract;
   procedure   AddClass(aName: string); virtual;
   procedure   AddClassProperty(const aName: string; aType: integer); virtual;
   procedure   LoadAll; virtual; abstract;
   procedure   NewElement(aClass: longInt); virtual;
   procedure   LoadProperties; virtual; abstract;
   procedure   SetProperty(aIndex: integer; aType: integer; var aValue); virtual;
   procedure   LoadElements; virtual; abstract;
   procedure   AddNextElement(aElement: longInt); virtual;
   property    OrbisClasses: TOrbisCLasses read FClasses write SetClasses;
   property    ActualClass: TOrbisClass read FActualClass write FActualClass;
   property    ActualElement: TElement read FActualElement;
   property    ActualPropertyIndex: integer read FActualPropertyIndex write FActualPropertyIndex;
   property    Instance: TOrbisInstance read FInstance;
   property    LastElementIndex: integer read FLastElementIndex write SetLastElementIndex;
   property    Container: TQuickEntity read FContainer write FContainer;
published
   property    FileName: string read FFileName write FFileName;
   property    _Container: TQuickEntity read FContainer stored false;
   property    ContainerEntity: TEntity read FContainerEntity write SetContainerEntity stored false;
end;



TStreamLoader = class(TOrbisLoader)
private
   FReader: TReader;
   FStream: TStream;
    FLastPosition: integer;
    procedure SetLastPosition(const Value: integer);
protected
   property  Reader: TReader read FReader write FReader;
   property  Stream: TStream read FStream write FStream;
   property  LastPosition: integer read FLastPosition write SetLastPosition;
public
   procedure   DoIt; override;
   function    GetGaugeMax: longInt; override;
   procedure   UpdateGauge; override;
   procedure   UseOldLoader; virtual;
   procedure   LoadInfo; override;
   procedure   LoadClass; override;
   procedure   LoadAll; override;
   procedure   LoadProperties; override;
   procedure   LoadElements; override;
end;

TLoad = class(TStreamLoader);


TLoader = class(TTool)
 private
   FIndex: longInt;
   FReader: TReader;
   FStream: TStream;
   FContainer: TQuickEntity;
 protected
   procedure CreateProperties;
   procedure FreeProperties;
   property  Index: longInt read FIndex write FIndex default 0;
   property  Reader: TReader read FReader write FReader;
   property  Stream: TStream read FStream write FStream;
   property  Container: TQuickEntity read FContainer write FContainer;
 public
   constructor Create; override;
   constructor CreateAll(aStreamName: string);
   procedure   DoIt; override;
end;


implementation

uses

  {!t_Main,}
  {!PubUnits,}

  Controls,
  Dialogs,
  Forms,
  SysUtils,
  TypInfo,
  m_Orbis,
  u_String,
  Worlds;


type

EBadStreamName = class(Exception);
EBadStream = class(Exception);


{ TOrbisLoader }


constructor TOrbisLoader.CreateAll(const aFileName: string);
begin
   inherited Create;
   FileName := aFileName;
end;

procedure   TOrbisLoader.DoIt;
var
   GE: TElement;
begin
   StartGauge;
   Prepare;
   LoadInfo;
   LoadClasses;
   LoadAll;
   GE := Container.elements[Container.Count - 1];
   if GE is TYouShallNotPass then
   begin
      Output := Container.elements[Container.Count - 2];
      GE.Free;
   end
   else
      Output := Container.elements[Container.Count - 1];
   Finish;
   EndGauge;
end;

procedure TOrbisLoader.StartGauge;
var
   i: longInt;
begin
   Screen.Cursor:= crHourGlass;
   //FGauge:= TF_GaugeForm.Create(Application);
   //FGauge.Caption:= 'Natn modelu ze souboru ' + FileName;
   i := GetGaugeMax;
   //FGauge.MaxValue:= i; {!!!!!!!!!!!!!!!!!!!!!!!!}
   //FGauge.Show
   StartProgress('Natn modelu ze souboru ' + FileName,0,i);
   Application.ProcessMessages;
end;



procedure TOrbisLoader.EndGauge;
begin
   Screen.Cursor:= crDefault;
   (*
   if FGauge <> nil then
   begin
      FGauge.Free;
      FGauge:= nil
   end;
   *)
   EndProgress;
end;

procedure   TOrbisLoader.UpdateGauge;
begin
end;



function    TOrbisLoader.NumberToPropertyName(aNumber: longInt): string;
begin
   if aNumber = 0 then
      result := 'FileName'
   else
      result := 'ContainerEntity';
end;



procedure TOrbisLoader.SetClasses(aValue: TOrbisClasses);
begin
   if FClasses <> nil then
   begin
      FreeElement(FCLasses);
      FClasses := nil;
   end;
   FClasses := aValue;
   AddElement(FClasses);
end;



procedure   TOrbisLoader.Prepare;
begin
   Container := TQuickEntity.Create;
   Finstance := TOrbisInstance.Create;
   AddElement(FInstance);
   PrepareClasses;
end;


procedure   TOrbisLoader.Finish;
begin
   if Container = nil then exit;
   if ContainerEntity <> nil then
      ContainerEntity.AddAllFrom := Container;
   Container.Clear;
   Container.Free;
end;

procedure   TOrbisLoader.PrepareClasses;
begin
   OrbisClasses := TOrbisClasses.Create;
end;


procedure   TOrbisLoader.LoadClasses;
begin
end;

procedure   TOrbisLoader.AddClass(aName: string);
var
   OC: TOrbisClass;
begin
   ActualClass := TOrbisClass.Create;
   OrbisClasses.AddElement(ActualClass);
   ActualPropertyIndex := 0;
   OC := TOrbisClasses(World.ObjectClasses).GetOrbisClassByName(aName);
   if OC = nil then
   begin
      { neregistrovan tda !!!!!!!!!!};
      PostTextMsg(tm_AddToProtocol, aName +' - Unknown class !',nil);
      ActualClass.ElementClass := TOwnerEntity;
      ActualClass.Name := aName+' ?';
      exit;
   end;
   ActualClass.ElementClass := OC.ElementClass;
end;


procedure   TOrbisLoader.AddClassProperty(const aName: string; aType: integer);
var
   OCP: TOrbisClassProperty;
   i: integer;
   S: string;
   PStored: boolean;

function CompareTypeKinds(T1,T2: TTypeKind): boolean;
begin
   result := true;
   if T1 = T2 then exit;
   if (T1 in [tkString,tkLString]) then
   begin
      if (T2 in [tkString,tkLString]) then exit;
      if T2 <> tkClass then
      begin
         PostTextMsg(tm_AddToProtocol,ActualClass.Name+' property "'+aName+'" type conv.: '+OCP.GetPropertyStringByName('LostTypeKind')+' to '+OCP.GetPropertyStringByName('TypeKind'),nil);
         OCP.TypeMishmash := true;
         exit;
      end;
   end;
   if (T2 in [tkString,tkLString]) then
   begin
      if T1 <> tkClass then
      begin
         PostTextMsg(tm_AddToProtocol,ActualClass.Name+' property "'+aName+'" type conv.: '+OCP.GetPropertyStringByName('LostTypeKind')+' to '+OCP.GetPropertyStringByName('TypeKind'),nil);
         OCP.TypeMishmash := true;
         exit;
      end;
   end;
   result := false;
end;

begin
   S := UpperCase(aName);
   if ActualClass = nil then exit;
   OCP := TOrbisClassProperty.Create;
   OCP.Name := aName;
   //OCP.TrueIndex := -1;
   ActualClass.AddElement(OCP);

   OCP.TrueIndex := ActualClass.PropertyIndex(aName);
   OCP.LostTypeKind := TTypeKind(aType);
   if OCP.TrueIndex < 0 then
   begin
      OCP.LostProperty := true;
      PostTextMsg(tm_AddToProtocol,ActualClass.Name+' lost property: '+aName,nil);
      exit;
   end;
   OCP.PropInfo  := ActualClass.PropInfo[OCP.TrueIndex];
   PStored := ActualClass.PropertyStored[OCP.TrueIndex];
   if not CompareTypeKinds(ActualClass.PropertyTypeKind[OCP.TrueIndex],TTypeKind(aType)) then
   begin
      OCP.TrueIndex := -1;
      PostTextMsg(tm_AddToProtocol,ActualClass.Name+' property "'+aName+'" type error: '+IntToStr(aType),nil);
   end
   else
   if PStored and (TTypeKind(aType) = tkClass) and not ActualClass.PropertyClass[OCP.TrueIndex].InheritsFrom(TElement) then
   begin
      PostTextMsg(tm_AddToProtocol,ActualClass.Name+' property "'+aName+'" is not Element!',nil);
   end;
   if PStored and ActualClass.PropertyIsReadOrWriteOnly[OCP.TrueIndex] then
   begin
      PostTextMsg(tm_AddToProtocol,ActualClass.Name+' property "'+aName+'" is Read or Write Only!',nil);
   end;
   inc(FActualPropertyIndex);
   (*
   { Kontrola propert .. nesed-li typ nebo co, TrueIndex zstv -1 }
   { Nen to nhodou aktuln properta ? }
   if AnsiCompareText(aName,ActualClass.PropertyName[ActualPropertyIndex])= 0 then
   begin
      { Jo ... kontrola typu }
      if CompareTypeKinds(ActualClass.PropertyTypeKind[ActualPropertyIndex],TTypeKind(aType)) then
      { Typ Souhlas }
      begin
         OCP.TrueIndex := ActualPropertyIndex;
         OCP.PropInfo  := ActualClass.PropInfo[ActualPropertyIndex];
         inc(FActualPropertyIndex);
      end
      else
      begin
         { Chyba typu !!!!!!}
         ShowMessage(ActualClass.Name + ' ' + aName + ' property type error');
      end;
      exit;
   end
   else
   for i := 0 to ActualClass.PropCount - 1 do
   begin
      if AnsiCompareText(aName,ActualClass.PropertyName[i]) = 0 then
      { properta existuje }
      begin
         { kontrola typu }
         if CompareTypeKinds(ActualCLass.PropertyTypeKind[i],TTypeKind(aType)) then
         { Typ Souhlas }
         begin
            OCP.TrueIndex := i;
            OCP.PropInfo  := ActualClass.PropInfo[i];
            ActualPropertyIndex := i + 1;
         end
         else
         begin
            { Chyba typu !!!!!!}
         ShowMessage(ActualClass.Name + ' ' + aName + ' property type error');
         end;
         exit;
      end;
   end;
   ShowMessage(ActualClass.Name + ' ' + aName + ' property not found');
   { properta neexistuje !!!!!}
   *)
end;

procedure   TOrbisLoader.NewElement(aClass: longInt);
var
   OC: TOrbisClass;
begin
   OC := TOrbisClass(OrbisClasses.Elements[aClass]);
   if OC = nil then
   begin
      Instance.Instance := nil;
      exit;
   end;
   ActualClass := OC;
   if ActualClass.ElementClass = nil then exit;
   Instance.Instance := ActualClass.ElementClass.Create;
   FActualElement := Instance.Instance;
   Container.AddElement(ActualElement);
   FActualPropertyIndex := 0;
end;

procedure   TOrbisLoader.SetProperty(aIndex: integer; aType: integer; var aValue);
var
   Idx: integer;
begin
   try
   if (ActualClass = nil) or (ActualClass.ElementClass = nil) then exit;
   Idx := TOrbisClassProperty(ActualClass.Elements[aIndex]).TrueIndex;
   if Idx < 0 then exit;
   if TTypeKind(aType) <> Instance.PropertyTypeKind[Idx] then exit; {  chyba !!!!!!!!!!!!!}
   if Instance.PropertyIsReadOrWriteOnly[Idx] then exit;
   Instance.SetPropertyValue(Idx, aValue);
   except
      //ShowMessage('Property Set Error '+ActualClass.ElementClass.ClassName+' '+Instance.PropertyName[Idx]);
      PostTextMsg(tm_AddToProtocol,'Loader Property Set Error: '+ActualClass.Name+' '+Instance.PropertyName[Idx],nil);
   end;
end;

procedure   TOrbisLoader.AddNextElement(aElement: longInt);
var
   E: TElement;
begin
   try
      E := Container.Elements[aElement];
      TEntity(ActualElement).AddElement(E);
   except
   end;
end;

procedure TOrbisLoader.SetLastElementIndex(const Value: integer);
begin
   FLastElementIndex := Value;
end;

procedure TOrbisLoader.SetContainerEntity(const Value: TEntity);
begin
  FContainerEntity := Value;
end;

{ TStreamLoader }

procedure   TStreamLoader.DoIt;
var
   Magic: array[0..3] of Char;
begin
   if FileName = '' then raise EBadStreamName.Create('Chybn jmno souboru : '+FileName+' !');
{$IFDEF FILEEVENT}
    PostTextMsg(tm_FileBeforeLoad,FileName,nil);
    Application.ProcessMessages;
{$ENDIF}

   if not FileExists(FileName) then raise EBadStream.Create('Soubor '+FileName+' neexistuje!');
   Stream := TMemoryStream.Create;
   try
      TMemoryStream(Stream).LoadFromFile(FileName);
   except
      Stream.Free;
      Stream := TFileStream.Create(FileName, fmOpenRead{Write !!!!!!!} );
   end;
   LastPosition := 0;
   Reader := TReader.Create(Stream, 4096);
   Reader.Read(Magic, SizeOf(Magic));
   if Magic = 'TPF0'then
   begin
      Reader.Free;
      Stream.Free;
      UseOldLoader;
      exit;
   end
   else if Magic <> 'ORB0'then
   begin
      Reader.Free;
      Stream.Free;
      raise EBadStream.Create(FileName+' ... neznm typ souboru');
   end;
   inherited DoIt;
   Reader.Free;
   Stream.Free;
end;

function    TStreamLoader.GetGaugeMax: longInt;
begin
   result := Stream.Size;
end;

procedure   TStreamLoader.UpdateGauge;
begin
   if (Stream.Position > LastPosition) and (Stream.Position mod 200{4096} = 0) then
   begin
      //PostTextMsg(tm_AddToProtocol,IntToStr(Stream.position),nil);
      LastPosition := Stream.Position;
      Progress := LastPosition;
      ProgressText := IntToStr(LastPosition)+' byt z celkovch '+IntToStr(Stream.Size);
      //FGauge.Progress:= FStream.Position;
      Application.ProcessMessages
   end;
end;

procedure   TStreamLoader.UseOldLoader;
var
   L: TLoader;
begin
   L := TLoader.Create;
   L.Name := FileName;
   L.DoIt;
   Output := L.Output;
   L.Free;
end;


procedure   TStreamLoader.LoadInfo;
var
   S: string;
begin
   S := Reader.ReadString;
   S := Reader.ReadString;
   S := Reader.ReadString;
end;

procedure   TStreamLoader.LoadClass;
var
   aClassName: string;
   aClassID: longInt;
   aPropertyName: string;
   aPropertyType: integer;
begin
   aClassName := Reader.ReadString;
   aClassID   := Reader.ReadInteger;
   AddClass(aClassName);
   Reader.ReadListBegin;
   while not Reader.EndOfList do
   begin
      begin
         aPropertyName := Reader.ReadString;
         aPropertyType := Reader.ReadInteger;
         AddClassProperty(aPropertyName, aPropertyType);
      end;
   end;
   Reader.ReadListEnd;
end;

procedure   TStreamLoader.LoadAll;
var
   aClass: longInt;
   aId: longInt;
begin
   Reader.ReadListBegin;
   while not Reader.EndOfList do
   begin
      aClass := Reader.ReadInteger;
      if aClass = -1 then LoadClass
      else
      begin
         try
         NewElement(aClass);
         aId := Reader.ReadInteger;
         if aId = 143888 then
         begin
            aId := aId;
         end;
         LoadProperties;
         LoadElements;
         LastElementIndex := aId;
         except
            aId := aId;
         end;
      end;
      UpdateGauge;
   end;
   Reader.ReadListEnd;
end;


procedure   TStreamLoader.LoadProperties;
var
   aIndex: integer;
   tmpInteger: integer;
   tmpChar: char;
   tmpString: string;
   tmpFloat: extended;
   tmpElement: TElement;
   TypeKind: TTypeKind;
   NTypeKind: TTypeKind;
   _Index: longInt;
   _ActualPropertyIndex: integer;
   _ActualElement: TElement;
   _ActualClass: TOrbisClass;
   OCP: TOrbisClassProperty;
begin
   Reader.ReadListBegin;
   while not Reader.EndOfList do
   begin
      aIndex := Reader.ReadInteger;
      OCP := TOrbisClassProperty(ActualClass.Elements[aIndex]);
      TypeKind  := OCP.LostTypeKind;    // oprava na Lost
      NTypeKind := OCP.TypeKind;
      case TypeKind of
         tkInteger,tkEnumeration,tkSet:
         begin
            tmpInteger := Reader.ReadInteger;
            if OCP.TypeMishmash then
            begin
               tmpString := IntToStr(tmpInteger);
               SetProperty(aIndex, integer(NTypeKind), tmpString);
            end
            else
            begin
               SetProperty(aIndex, integer(TypeKind), tmpInteger);
            end;
         end;
         tkChar:
         begin
            tmpChar := Reader.ReadChar;
            if OCP.TypeMishmash then
            begin
               tmpString := tmpChar;
               SetProperty(aIndex, integer(NTypeKind), tmpString);
            end
            else
            begin
               SetProperty(aIndex, integer(TypeKind), tmpChar);
            end;
         end;
         tkFloat:
         begin
            tmpFloat := Reader.ReadFloat;
            if OCP.TypeMishmash then
            begin
               tmpString := FloatToStr(tmpFloat);
               SetProperty(aIndex, integer(NTypeKind), tmpString);
            end
            else
            begin
               SetProperty(aIndex, integer(TypeKind), tmpFloat);
            end;
         end;
         tkString
         {$IFDEF Win32}
         ,tkLString
         {$ENDIF}
                   :
         begin
            tmpString := Reader.ReadString;
            if OCP.TypeMishmash then
            begin
               case NTypeKind of
                  tkInteger,tkEnumeration,tkSet:
                  begin
                     tmpInteger := StrToIntDef(tmpString,0);
                     SetProperty(aIndex, integer(NTypeKind), tmpInteger);
                  end;
                  tkChar:
                  begin
                     if tmpString <> '' then
                        tmpChar := tmpString[1]
                     else
                        tmpChar := '-';
                     SetProperty(aIndex, integer(NTypeKind), tmpChar);
                  end;
                  tkFloat:
                  begin
                     tmpFloat := StrToExtended(tmpString);
                     SetProperty(aIndex, integer(NTypeKind), tmpFloat);
                  end;
               end;
            end
            else
            begin
               SetProperty(aIndex, integer(TypeKind), tmpString);
            end;
         end;

         tkClass:
         begin
            tmpInteger := Reader.ReadInteger;
            if tmpInteger = -1 then tmpElement := nil
            else
            if tmpInteger = -2 then
            begin
               _Index               := aIndex;
               _ActualPropertyIndex := ActualPropertyIndex;
               _ActualElement       := ActualElement;
               _ActualClass         := ActualClass;

               LoadAll;
               tmpElement := Container.Elements[LastElementIndex];
               tmpElement.ID := -1;
               aIndex              := _Index;
               ActualPropertyIndex := _ActualPropertyIndex;
               FActualElement      := _ActualElement;
               ActualClass         := _ActualClass;
               Instance.Instance   := _ActualElement;
            end
            else
            begin
               // pokus o een chybn uloenho souboru
               if (tmpInteger > Container.LastIndex) or (tmpInteger < 0) then
               begin
                  tmpElement := nil;
                  PostTextMsg(tm_AddToProtocol,'Loader Error: '+ActualClass.Name+' ClassProperty '+ActualClass.Elements[aIndex].Name,nil);
               end
               else
                  tmpElement := Container.Elements[tmpInteger];
            end;
            SetProperty(aIndex, integer(TypeKind), tmpElement);
         end;
      end;
   end;
   Reader.ReadListEnd;
end;

procedure   TStreamLoader.LoadElements;
var
   Realy: integer;
   aIndex: longInt;
begin
   Realy := Reader.ReadInteger;
   if not boolean(Realy) then exit;
   Reader.ReadListBegin;
   while not Reader.EndOfList do
   begin
      aIndex := Reader.ReadInteger;
      AddNextElement(aIndex);
   end;
   Reader.ReadListEnd;
end;


procedure TStreamLoader.SetLastPosition(const Value: integer);
begin
   FLastPosition := Value;
end;

{ TLoader }

constructor TLoader.Create;
begin
   inherited Create;
   Name := 'ORBIS.ORB';
end;



constructor TLoader.CreateAll(aStreamName: string);
begin
   inherited Create;
   Name := aStreamName;
end;


procedure   TLoader.CreateProperties;
begin
   if Name = '' then raise EBadStreamName.Create('Chybn jmno streamu : '+Name+' !');
   Stream := TFileStream.Create(Name, fmOpenRead);
   Reader := TReader.Create(Stream, 64000);
   Container := TQuickEntity.Create;
end;

procedure   TLoader.FreeProperties;
begin
   Container.Clear;
   Container.Free;
   Reader.Free;
   Stream.Free;
end;


procedure   TLoader.DoIt;
var
   aClassName: string;
   aClass: TClass;
   Element: tElement;
begin
   Screen.Cursor := crHourGlass;
   CreateProperties;
   Reader.ReadSignature;
   Reader.ReadListBegin;
   while not Reader.EndOfList do
   begin
     try
      aClassName := Reader.ReadString;
      aClass := GetOrbisClass(aClassName);
      if aClass = nil then
      begin
         MessageDlg('Neregistrovan element: '+ aClassName, mtError, [mbOK], 0);
         FreeProperties;
         Screen.Cursor := crDefault;
         exit;
      end;
     except
       MessageDlg('Chyba pi ten elementu: '+ aClassName, mtError, [mbOK], 0);
     end;

      try
         Element := TElement(aClass.Create);
         Element.Load(Reader, Container);
         Container.AddElement(Element);
         {
         Application.ProcessMessages;
         }
      except
         MessageDlg('Chyba pi ten elementu: '+ aClassName, mtError, [mbOK], 0);
         FreeProperties;
         Screen.Cursor := crDefault;
         exit;
      end;
   end;
   Reader.ReadListEnd;
   Output := Container.elements[Container.Count - 1];
   FreeProperties;
   Screen.Cursor := crDefault;
end;


BEGIN
   RegisterOrbisClass(TLoader);
   RegisterOrbisClass(TLoad);
END.


