unit i_WMS;

interface
uses
   Elements,
   i_Basic;

type

Tgis_WMSLayerInterpret = class(TInterpret)
  private
    FLastY: double;
    FLastX: double;
    FLastScale: double;
    procedure SetLastScale(const Value: double);
    procedure SetLastX(const Value: double);
    procedure SetLastY(const Value: double);
protected
   function  IsItsClass(AObject: TObject): boolean; override;
   function  QGoInside(Element: TElement): boolean; override;
   procedure Process(Element: TElement); override;
   function  DownloadURLFile(const Xml, XmlLocal: string): boolean;
public
   property  LastScale: double read FLastScale write SetLastScale;
   property  LastX: double read FLastX write SetLastX;
   property  LastY: double read FLastY write SetLastY;
end;


implementation
uses
   Controls,
   ExtActns,
   Forms,
   Graphics,
   SysUtils,
   Windows,
   m_Orbis,
   m_Proj,
   e_Class,
   e_Device,
   e_WMS,
   g_Elems,
   Worlds,
   u_String, Tools, E_page;

{ Tgis_WMSLayerInterpret }

function Tgis_WMSLayerInterpret.DownloadURLFile(const Xml,
  XmlLocal: string): boolean;
var
   D: TDownLoadURL;
begin
   Result := True;
   D := TDownLoadURL.Create(nil);
   try
     D.URL := Xml;
     D.Filename := XmlLocal;
     try
       D.ExecuteTarget(nil);
     except
       Result := False;
     end;
   finally
     D.Free;
   end;
end;

function Tgis_WMSLayerInterpret.IsItsClass(AObject: TObject): boolean;
begin
   result := aObject is Tgis_WMSLayer;
end;

procedure Tgis_WMSLayerInterpret.Process(Element: TElement);
var
   eWMS: Tgis_WMSLayer;
   Xml,xml_getmap,smap,konc: string;
   LocalXMl, LocalImg : string;
   i, dd:integer;
   Sminx,Sminy,Smaxx,Smaxy:string; //hodnoty u parametru BBOX tzn. ohraniceni mapy odkud kam ji chceme mit (Ohranicuje rohy: vlevo dole, vpravo nahore; v jendotkach SRS )
   bTrans,bBGcl,bExc,bTime,bElev,bkonc:boolean;  //pokud jsou volitelne parametry zadavany
   adresa,
   sLayers,sStyles,sSrs,sBbox,SWidth,SHeight,sFormat,sTransparent_v,
   sBgcolor_v,sExceptions_v,sTime_v,SElevation_v,srsStr:string;
   R: TDoubleRect;
   aScale,aX,aY: double;
   Picture: TPicture;
   S,SD: string;
   e: TElement;
   SaveCurs: integer;
   MaxWidth: integer;
   MaxHeight: integer;
   TileX,TileY,rTileX,rTileY,Xcount,YCount,txi,tyi,prgi: integer;
   DeviceX,DeviceY,StartX,StartY: integer;
   isPrint: boolean;
   Rect: TRect;
begin
   if not (Element is Tgis_WMSLayer) then exit;
   eWMS := Tgis_WMSLayer(Element);
   if Device = nil then exit;
   isPrint := Device is TPrintDevice;
   MaxWidth := eWMS.MaxWidth;
   MaxHeight := eWMS.MaxHeight;
   if MaxWidth = 0 then
      MaxWidth := 1000;
   if MaxHeight = 0 then
      MaxHeight := 1000;
   // priprava dlazdic
   // ----------------
   XCount := 1;
   YCount := 1;
   if Device.Page.Clipping then
      Rect := Device.Page.ClipRegion
   else
      Rect := Device.Page.Rect;
   StartX := Rect.Left;
   StartY := Rect.Top;
   TileX := Rect.Right - Rect.Left; //Device.Page.Width;
   TileY := Rect.Bottom - Rect.Top; //Device.Page.Height;
   while (TileX > MaxWidth) or (TileY > MaxHeight) do
   //***while TilesCount < 5 do
   begin
      if TileX > TileY then
      begin
         TileX  := (TileX div 2) + (TileX mod 2);  // Tile X je polovicni
         XCount := XCount * 2;                     // dvojnasobny pocet horizontalne
      end
      else
      begin
         YCount := YCount * 2;
         TileY := (TileY div 2) + (TileY mod 2);
      end;
   end;
   SaveCurs := Screen.Cursor;
   Screen.Cursor := crSQLWait;
   PostTextMsg(tm_ShowText,'wms '+eWMS.Name+' ...start ',nil);
   Application.ProcessMessages;
   //Xml := Edit.Text;      // pokud zadano takhle chce osetrit aby stranky nekoncili lomitkem (tvar musi byt viz niz) nevim jak na jinych strankach
   //adresa:='http://wms.cuzk.cz/wms.asp';
   adresa := eWMS.URL;
   Xml:=adresa+'?service=WMS&version=1.1&request=GetCapabilities';
   //minx:=-910000; miny:=-1.23e+006; maxx:=-430000; maxy:=-930000; //vyplnovani hodnot je to tu jen pro zkousku
   // a jedem pres vsechny dlazdice
   if isPrint then
      S := 'Print: '
   else
      S := 'Rendering: ';
   if IsPrint or (YCount > 1) or (XCount > 1) then
      StartProgress('WMS '+S+ eWMS.Name,0,YCount * XCount);
   prgi := 0;
   for tyi := 0 to YCount - 1 do
   begin
      for txi := 0 to XCount - 1 do
      begin
         Inc(prgi);
         Progress := prgi;
         DeviceX := StartX + (TileX * txi);
         DeviceY := StartY + (TileY * tyi);
         rTileX := TileX;
         rTileY := TileY;
         // uprava posledni dlazdice
         if (DeviceX + rTileX) > Rect.Right then
            rTileX := Rect.Right - DeviceX;
         if (DeviceY + rTileY) > Rect.Bottom then
            rTileY := Rect.Bottom - DeviceY;
         R.MinX := Device.Projection.GetModelX(DeviceX);
         R.MinY := Device.Projection.GetModelY(DeviceY + rTileY);
         R.MaxX := Device.Projection.GetModelX(DeviceX + rTileX);
         R.MaxY := Device.Projection.GetModelY(DeviceY);
         Sminx:=FloatToStr(R.MinX);
         Sminy:=FloatToStr(R.MinY);
         Smaxx:=FloatToStr(R.MaxX);
         Smaxy:=FloatToStr(R.MaxY); //vyplnovani hodnot je to tu jen pro zkousku
         ChangeCharInString(SMinX,',','.',true);
         ChangeCharInString(SMinY,',','.',true);
         ChangeCharInString(SMaxX,',','.',true);
         ChangeCharInString(SMaxY,',','.',true);
         sLayers:=eWMS.Layers;
         sStyles:=''; // styles je volitelny nemaji to kazde stranky (minimalne cuzk.cz nemaji i kdyz je psano ze je to povinny parametr)
         sSrs:='EPSG:102067';  // JTSK
         SWidth:=IntToStr(rTileX);
         SHeight:=IntToStr(rTileY);
         sFormat:='image/jpeg';   //konec zadavani hodnot, podle navodu na WMS sluzby nemely byt jine dalsi parametry pripadne bych to dodelala mela by byt otazka nekolika minut
         srsStr := 'srs';
         if (eWMS.Version <> '') and (Length(eWMS.Version) > 2) and (StrToIntDef(eWMS.Version[3],1)>2) then
            srsStr := 'crs';

         smap:='?service=wms&version='+eWMS.Version+'&request=Getmap'+'&layers='+sLayers+'&styles='+sStyles+
               '&'+srsStr+'='+sSrs+'&bbox='+Sminx+','+Sminy+','+Smaxx+
               ','+Smaxy+'&width='+SWidth+'&height='+SHeight+'&format='+sFormat;//Width a height= sirka a vyska mapy obrazu v pixelech;
         if bTrans=true then smap:=smap+'&TRANSPARENT='+sTransparent_v;
         if bBGcl=true  then smap:=smap+'&BGCOLOR='+sBgcolor_v;
         if bExc=true   then smap:=smap+'&EXCEPTION='+sExceptions_v;
         if bTime=true  then smap:=smap+'&TIME'+sTime_v;
         if bElev=true  then smap:=smap+'&ELEVATION'+SElevation_v;
         xml_getmap:=adresa+smap;
         konc:='';
         bkonc:=false;
         for i:=1 to length(sformat) do
           begin
             if sformat[i]=';' then bkonc:=false;
             if bkonc=true then konc:=konc+sformat[i];
             if sformat[i]='/' then bkonc:=true;
           end;
         S := 'Store';
         SD := 'StoreDirectory';
         e :=  World.MainTool.FindElementName('TOptions');
         if e <> nil then
         begin
            if e.HasPropertyByName(SD) then
               S := e.GetPropertyStringByName(SD);
         end;
         LocalXml := IncludeTrailingPathDelimiter(GetAbsolutePath(S)) + eWMS.Name+'.xml';
         LocalImg := IncludeTrailingPathDelimiter(GetAbsolutePath(S)) + eWMS.Name+'.jpg';
         //aScale := Device.Projection.Scale;
         //aX := Device.Projection.Center_X;
         //aY := Device.Projection.Center_Y;
         //if (aScale > 0) and (aScale = LastScale) and (aX = LastX) and (aY = LastY) and FileExists(LocalImg) then
         //begin
         //   PostTextMsg(tm_ShowText,'wms Same projection!!',nil);
         //   Application.ProcessMessages;
         //end
         //else
         begin
         try
            ProgressText := 'Download ... tile '+IntToStr(prgi);
            Application.ProcessMessages;

            if not DownloadURLFile(xml_getmap, LocalImg)  then
            begin
              PostTextMsg(tm_ShowText,'wms Chyba pipojen!!',nil);
              Screen.Cursor := SaveCurs;
              EndProgress;
              Application.ProcessMessages;
              Exit;
            end;
            finally
              //DeleteFile(LocalImg);
              Screen.Cursor := SaveCurs;
            end;
         end;
         if FileExists(LocalImg) then
         begin
            ProgressText := 'Rendering ...';
            Application.ProcessMessages;
            Picture:= TPicture.Create;
            try
            Picture.LoadFromFile(LocalImg);
            Application.ProcessMessages;
            if Device is TPrintDevice then
               TPrintDevice(Device).StretchDrawDevice(DeviceX,DeviceY,rTileX,rTileY,0,0,rTileX,rTileY,Picture.Graphic)
            else
               TCanvasDevice(Device).DrawDevice(DeviceX, DeviceY, Picture.Graphic);

            ProgressText := 'Done ...';
            Application.ProcessMessages;
            except
               PostTextMsg(tm_Run,'LED\Etc\WMS\'+eWMS.Name+'_Error.oil',nil);
               Application.ProcessMessages;
            end;
            Picture.Free;

         end
         else
         begin
            PostTextMsg(tm_ShowText,'wms Soubor chybn uloen!!',nil);
            EndProgress;
            Application.ProcessMessages;
            Screen.Cursor := SaveCurs;
            Exit;
         end;
      end;
   end;
   EndProgress;
   Screen.Cursor := SaveCurs;
   PostTextMsg(tm_ShowText,'wms Done!!',nil);
   Application.ProcessMessages;
end;

function Tgis_WMSLayerInterpret.QGoInside(Element: TElement): boolean;
begin
   result := false;
end;


procedure Tgis_WMSLayerInterpret.SetLastScale(const Value: double);
begin
  FLastScale := Value;
end;

procedure Tgis_WMSLayerInterpret.SetLastX(const Value: double);
begin
  FLastX := Value;
end;

procedure Tgis_WMSLayerInterpret.SetLastY(const Value: double);
begin
  FLastY := Value;
end;

INITIALIZATION
   RegisterOrbisClass(Tgis_WMSLayerInterpret);
END.
