unit r_ThematicLayerEditor;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls,
  e_ThematicMap, ExtCtrls;

type
  TThematicLayerEditor_Frame = class(TFrame)
    l_LayerName: TLabel;
    cb_Active: TCheckBox;
    Bevel1: TBevel;
    ColorPanel: TPanel;
    cb_UseColor: TCheckBox;
    b_Setup: TButton;
    procedure FrameDragOver(Sender, Source: TObject; X, Y: Integer;
      State: TDragState; var Accept: Boolean);
    procedure cb_ActiveClick(Sender: TObject);
    procedure ColorPanelClick(Sender: TObject);
    procedure cb_UseColorClick(Sender: TObject);
    procedure b_SetupClick(Sender: TObject);
  private
    FLayer: TThematicLayer;
    FColorDialog: TColorDialog;
    FLayerConnection: boolean;
    FConnection: TThematicConnection;
    FSetting: boolean;
    FSetupScript: string;
    procedure SetColorDialog(const Value: TColorDialog);
    procedure SetLayerConnection(const Value: boolean);
    procedure SetConnection(const Value: TThematicConnection);
    procedure SetSetupScript(const Value: string);
    { Private declarations }
  public
     procedure SetLayer(const Value: TThematicLayer); virtual;
     property Layer: TThematicLayer read FLayer write SetLayer;
     property Connection: TThematicConnection read FConnection write SetConnection;
     property LayerConnection: boolean read FLayerConnection write SetLayerConnection;
     property ColorDialog: TColorDialog read FColorDialog write SetColorDialog;
     property SetupScript: string read FSetupScript write SetSetupScript;
    { Public declarations }
  end;

implementation
uses
   m_Proj,
   Elements,
   i_Top,
   i_AreaGeometryElement;

{$R *.DFM}

{ TThematicLayerEditor_Frame }

procedure TThematicLayerEditor_Frame.SetLayer(const Value: TThematicLayer);
begin
   FSetting := true;
   if not (TObject(Value) is TThematicLayer) then
   begin
      FLayer := nil;
      cb_Active.Visible := false;
      ColorPanel.Visible := false;
      cb_UseColor.Visible := false;
      FSetting := false;
      exit;
   end;
   FLayer := Value;
   cb_Active.Checked := not Value.SwitchedOff;
   if Value.SetupScript <> '' then
   begin
      ColorPanel.Visible := false;
      cb_UseColor.Visible := false;
      b_Setup.Visible := true;
      SetupScript := Value.SetupScript;
   end
   else
   begin
      ColorPanel.Color := Value.Color;
      cb_UseColor.Checked := Value.UseColor;
   end;
   FSetting := false;
end;

procedure TThematicLayerEditor_Frame.FrameDragOver(Sender, Source: TObject;
  X, Y: Integer; State: TDragState; var Accept: Boolean);
begin
   Accept := true;
end;

procedure TThematicLayerEditor_Frame.cb_ActiveClick(Sender: TObject);
var
   L : TThematicLayer;
   i: integer;
   e: TElement;
begin
   if FSetting then exit;
   if LayerConnection then
   begin
      if Connection = nil then exit;
      for i := 0 to Connection.LastIndex do
      begin
         e := Connection.Elements[i];
         if not (e is TThematicLayer) then continue;
         L := TThematicLayer(e);
         L.SwitchedOff := not cb_Active.Checked;
      end;
   end
   else
   begin
      Layer.SwitchedOff := not cb_Active.Checked;
   end;
end;

procedure TThematicLayerEditor_Frame.SetColorDialog(
  const Value: TColorDialog);
begin
  FColorDialog := Value;
end;

procedure TThematicLayerEditor_Frame.ColorPanelClick(Sender: TObject);
var
   L : TThematicLayer;
   i: integer;
   e: TElement;
begin
   if FSetting then exit;
   if ColorDialog = nil then exit;
   ColorDialog.Color := ColorPanel.Color;
   if not ColorDialog.execute then exit;
   ColorPanel.Color := ColorDialog.Color;
   if LayerConnection then
   begin
      if Connection = nil then exit;
      for i := 0 to Connection.LastIndex do
      begin
         e := Connection.Elements[i];
         if not (e is TThematicLayer) then continue;
         L := TThematicLayer(e);
         L.UseColor := cb_UseColor.Checked;
         L.Color := ColorPanel.Color;
      end;
   end
   else
   begin
      Layer.UseColor := cb_UseColor.Checked;
      Layer.Color := ColorPanel.Color;
   end;
end;

procedure TThematicLayerEditor_Frame.cb_UseColorClick(Sender: TObject);
var
   L : TThematicLayer;
   i: integer;
   e: TElement;
begin
   if FSetting then exit;
   if LayerConnection then
   begin
      if Connection = nil then exit;
      for i := 0 to Connection.LastIndex do
      begin
         e := Connection.Elements[i];
         if not (e is TThematicLayer) then continue;
         L := TThematicLayer(e);
         L.UseColor := cb_UseColor.Checked;
         L.Color := ColorPanel.Color;
      end;
   end
   else
   begin
      Layer.UseColor := cb_UseColor.Checked;
      Layer.Color := ColorPanel.Color;
   end;
end;

procedure TThematicLayerEditor_Frame.SetLayerConnection(
  const Value: boolean);
begin
  FLayerConnection := Value;
end;

procedure TThematicLayerEditor_Frame.SetConnection(const Value: TThematicConnection);
var
   L: TThematicLayer;
begin
   FSetting := true;
   if not (TObject(Value) is TThematicConnection) then
   begin
      FConnection := nil;
      cb_Active.Visible := false;
      ColorPanel.Visible := false;
      cb_UseColor.Visible := false;
      FSetting := false;
      exit;
   end;
   FConnection := Value;
   L := TThematicLayer(Value.FIndElementClass(TThematicLayer));
   if L = nil then
   begin
      cb_Active.Checked := true;
      //ColorPanel.Color := L.Color;
      cb_UseColor.Checked := false;
   end
   else
   begin
      cb_Active.Checked := not L.SwitchedOff;
      ColorPanel.Color := L.Color;
      cb_UseColor.Checked := L.UseColor;
   end;
   FSetting := false;
end;

procedure TThematicLayerEditor_Frame.SetSetupScript(const Value: string);
begin
  FSetupScript := Value;
end;

procedure TThematicLayerEditor_Frame.b_SetupClick(Sender: TObject);
begin
   if SetupScript <> '' then
   begin
      Layer.PostTextMsg(tm_Run,SetupScript,nil);
      Application.ProcessMessages;
   end;
end;

end.
