unit t_DynamicPrint;

interface
uses
   e_Device,
   i_Basic,
   i_Uni,
   Tools;

type
TDynamicPrint = class(TTool)
  private
    FTiled: boolean;
    FBackupDevice: TDevice;
    FFitToPage: boolean;
    function GetInterpret: TInterpret;
    procedure SetInterpret(const Value: TInterpret);
    procedure SetTiled(const Value: boolean);
    procedure SetFitToPage(const Value: boolean);
    procedure SetBeginDoc(const Value: boolean);
    procedure SetEndDoc(const Value: boolean);
    procedure SetNewPage(const Value: boolean);
public
   procedure DoIt; override;
   procedure PrintDesktop(PrintDevice: TPrintDevice);
   property BackupDevice: TDevice read FBackupDevice write FBackupDevice;
published
   property Interpret: TInterpret read GetInterpret write SetInterpret;
   property Tiled: boolean read FTiled write SetTiled;
   property FitToPage: boolean read FFitToPage write SetFitToPage;
   property BeginDoc: boolean write SetBeginDoc;
   property EndDoc: boolean write SetEndDoc;
   property NewPage: boolean write SetNewPage;
end;


implementation
uses
   SysUtils,
   Printers_f,
   g_Elems,
   e_Class,
   e_Desktp,
   m_Orbis,
   Elements;

{ TDynamicPrint }

procedure TDynamicPrint.DoIt;
var
   PrintDevice: TPrintDevice;
   aCenterPoint: TDoubleCoord;
   aScale: double;
begin
   Error := 0;
   if Interpret = nil then
   begin
      Error := 1;
      exit;
   end;
   if Interpret is TUniversalInterpret then
      TUniversalInterpret(Interpret).Tile := Tiled;
   if Interpret.Device = nil then
   begin
     Error := 2;
     exit
   end;
   if not Printer.Printing then
   begin
     Error := 3;
     exit
   end;

   try
     BackupDevice := Interpret.Device;
     PrintDevice:= TPrintDevice.CreateExternalPrint;
     Interpret.Printing:= true;
     aCenterPoint := BackupDevice.Projection.CenterPoint;
     aScale := BackupDevice.Projection.Scale;

     PrintDevice.Projection.ChangeProjection
     (aCenterPoint, aScale, PrintDevice.Page);

     PrintDevice.Projection.VirtualScale:=BackupDevice.Projection.VirtualScale;

     Interpret.Device:= PrintDevice;

     try
       if (Interpret.Input is TEntity) then
       begin
          if (TEntity(Interpret.Input).FindElementClass(TDesktop) <> nil) then
             PrintDeskTop(PrintDevice)
          else
          //if (TEntity(Interpret.Input).FindElementClass(TMultiPageDesktop) <> nil) then
          //   PrintMultiPageDeskTop
          //else
             Interpret.DoIt
       end
       else
          Interpret.DoIt
     except
       on E: exception do
       begin
         //Printer.Abort;
         Error := 4;
         //ShowMessage('Problmy s tiskem: ' + E.Message)
       end;
     end;

     Interpret.Device:= BackupDevice;

     PrintDevice.Free;
     Interpret.Printing:= false;
     Interpret.PostMsgToOwners(tm_EndOfPrint, nil, DontKill);
   except
     on E: exception do
     begin
       Error := 5;
       //ShowMessage('Problmy s ppravou tisku: ' + E.Message)
     end;
   end;

//---------------------------------------------------------
   if Interpret is TUniversalInterpret then
      TUniversalInterpret(Interpret).Tile := false;
end;

function TDynamicPrint.GetInterpret: TInterpret;
begin
   result := nil;
   if not (Input is TInterpret) then exit;
   result := TInterpret(Input);
end;

procedure TDynamicPrint.PrintDesktop(PrintDevice: TPrintDevice);
var
   DeskTop: TDeskTop;
   aScale: double;
   Scale1, Scale2: double;
begin
   DeskTop:= TDeskTop(TEntity(Interpret.Input).FindElementClass(TDesktop));
   aScale := 1;
   if FitToPage then
   begin
      Scale1 := Desktop.Width/PrintDevice.RealWidth;
      Scale2 := Desktop.Height/PrintDevice.RealHeight;
      if Scale1 > Scale2 then aScale := Scale1
                         else aScale := Scale2;
   end;
   PrintDevice.Projection.ChangeProjection(Desktop.Rect.CenterCoord, aScale, PrintDevice.Page);
   Interpret.DoIt;
end;

procedure TDynamicPrint.SetBeginDoc(const Value: boolean);
begin
   Printer.BeginDoc;
end;

procedure TDynamicPrint.SetEndDoc(const Value: boolean);
begin
   Printer.EndDoc;
end;

procedure TDynamicPrint.SetFitToPage(const Value: boolean);
begin
  FFitToPage := Value;
end;

procedure TDynamicPrint.SetInterpret(const Value: TInterpret);
begin
   if not (Value is TInterpret) then exit;
   Input := Value;
end;

procedure TDynamicPrint.SetNewPage(const Value: boolean);
begin
   Printer.NewPage;
end;

procedure TDynamicPrint.SetTiled(const Value: boolean);
begin
  FTiled := Value;
end;


INITIALIZATION
   RegisterOrbisClass(TDynamicPrint);
END.
