unit t_GeometryTools;

interface
uses
   e_Geometry,
   Elements,
   t_AreMak,
   Tools;

const
   // Geometry tools errors:
   gteBadParameters       = 1;
   gteMissingParameter    = 2;
   gteParametersTypeError = 3;
   gteNoAreasContact      = 4;
   gteUnknownError        = 5;
   gteNotImplemented      = 6;

type

TGeometryTool = class(TTool)
private
   FGeometry1: TGeometry;
   FGeometry2: TGeometry;
   procedure SetGeometry1(const Value: TGeometry);
   procedure SetGeometry2(const Value: TGeometry);
protected
   procedure   SetOutput(aOutput: TElement); override;
public
   function NumberToPropertyName(aNumber: longInt): string; override;
   function CHeckParameters: boolean; virtual;
published
   property Geometry1: TGeometry read FGeometry1 write SetGeometry1;
   property Geometry2: TGeometry read FGeometry2 write SetGeometry2;
end;
(*
UNION sjednocen dvou geometry element
oba mus bt stejnho typu, tedy bu AreaElement nebo PolyLineAreaElement
vsledky:
   Area + Area = Area  [0]
   Line + Line = Line  [0]
   statn asi nemaj smysl - zatm
*)
TGeometryUnion = class(TGeometryTool)
public
   function CHeckParameters: boolean; override;
   procedure DoIt; override;
   procedure AreaUnion;
   procedure PolyLineAreaUnion;
end;
(*
DIFFERENCE rozdl dvou geometry element
vsledky:
   Area - Area = Area  [0]
   statn asi nemaj smysl - zatm
*)
TGeometryDifference = class(TGeometryTool)
public
   function CHeckParameters: boolean; override;
   procedure DoIt; override;
end;
(*
INTERSECTION prnik dvou geometry element
vsledky:
   Area ^ Area = Area  [0]
   Area ^ Line = Line  [0]
   statn asi nemaj smysl - zatm
*)

TGeometryIntersection = class(TGeometryTool)
public
   function CHeckParameters: boolean; override;
   procedure DoIt; override;
end;

implementation
uses
   G_Elems,
   e_Class;
{ TGeometryTool }

function TGeometryTool.CHeckParameters: boolean;
begin
   Error := gteBadParameters;
   result := false;
end;

function TGeometryTool.NumberToPropertyName(aNumber: Integer): string;
begin
   result := '';
   case aNumber of
      1: result := 'Geometry1';
      2: result := 'Geometry2';
   end;
end;

procedure TGeometryTool.SetGeometry1(const Value: TGeometry);
begin
  FGeometry1 := Value;
end;

procedure TGeometryTool.SetGeometry2(const Value: TGeometry);
begin
  FGeometry2 := Value;
end;

procedure TGeometryTool.SetOutput(aOutput: TElement);
begin
   ExchangeFieldElement(FOutput, aOutput);
end;

{ TGeometryUnion }

procedure TGeometryUnion.DoIt;
begin
   Output := nil;
   if not CHeckParameters then exit;
   if Geometry1.ClassType = TAreaElement then
      AreaUnion
   else
      PolyLineAreaUnion;
end;

function TGeometryUnion.CHeckParameters: boolean;
var
   Class1: TElementClass;
   Class2: TElementClass;
begin
   result := false;
   Error := gteBadParameters;
   // CHybjc parametr
   if (Geometry1 = nil) or (Geometry2 = nil) then
   begin
      Error := gteMissingParameter;
      exit;
   end;
   // parametr mus bt Area
   if not (Geometry1 is TAreaElement) or not (Geometry2 is TAreaElement) then
   begin
      Error := gteBadParameters;
      exit;
   end;
   Class1 := TElementClass(Geometry1.ClassType);
   Class2 := TElementClass(Geometry2.ClassType);
   //  a to Area nebo PolyLineArea
   if not ((Class1 = TAreaElement) or (Class1 = TPolyLineAreaElement)) then
   begin
      Error := gteBadParameters;
      exit;
   end;
   // a mus bt shodnho typu
   if not (Class1 = Class2) then
   begin
      Error := gteParametersTypeError;
      exit;
   end;
   // jinak O.K.
   Error := 0;
   result := true;
end;

procedure TGeometryUnion.AreaUnion;
begin
   Output := nil;
   Error := gteNotImplemented;
end;

procedure ExtractExclusiveLines(Area1: TPolyLineAreaElement; var Lines, BorderLines: TRectEntity; var Contact: boolean);
var
   i,h: integer;
   Line: TPolyLineElement;
   //E: TElement;
   Hole: TPolyLineAreaElement;

   procedure ProcessLine;
   var
      Ix1,Ix2: integer;
   begin
      Ix1 := Lines.IndexOf(Line);
      Ix2 := BorderLines.IndexOf(Line);
      if Ix1 < 0 then
      begin
         if Ix2 < 0 then
            Lines.AddElement(Line);
      end
      else
      begin
         if Ix2 < 0 then
            BorderLines.AddElement(Line);
         Contact := true;
         Lines.RemoveElement(Line);
      end;
   end;

begin
   Contact := false;
   for i := 0 to Area1.LastIndex do
   begin
      Line := TPolyLineElement(Area1.Elements[i]);
      ProcessLine;
   end;
   if Area1.Holes = nil then exit;
   for h := 0 to Area1.Holes.LastIndex do
   begin
      Hole := TPolyLineAreaElement(Area1.Holes.Elements[h]);
      for i := 0 to Hole.LastIndex do
      begin
         Line := TPolyLineElement(Hole.Elements[i]);
         ProcessLine;
      end;
   end;
end;

procedure TGeometryUnion.PolyLineAreaUnion;
var
   Are1,Are2: TPolyLineAreaElement;
   Lines, BorderLines: TRectEntity;
   i: integer;
   MA: TAreasFromClearLines;
   Contact: boolean;
   Areas: TEntity;
   Area: TpolyLineAreaElement;
   E: TElement;
begin
   Are1 := TPolyLineAreaElement(Geometry1);
   Are2 := TPolyLineAreaElement(Geometry2);
   Lines := TRectEntity.Create;
   BorderLines := TRectEntity.Create;
   ExtractExclusiveLines(Are1, Lines, BorderLines, Contact);
   ExtractExclusiveLines(Are2, Lines, BorderLines, Contact);
   if  Contact then
   begin
      Lines.ResetAndUpdate;
      MA := TAreasFromClearLines.Create;
      MA.Input := Lines;
      MA.DoIt;
      Areas := TEntity(MA.Output);
      if Areas <> nil then
      begin
         for i := 0 to Areas.LastIndex do
         begin
            E := Areas.Elements[i];
            if not (E is TPolyLineAreaElement) then
               continue;
            Area := TPolyLineAreaElement(E);
            if Area.IsHitByXY(Are1.X,Are1.Y,0) then
            begin
               Error := 0;
               Output := Area;
               Break;
            end;
         end;
         Areas.Free;
      end;
      MA.Free;
   end
   else
   begin
      Error := gteNoAreasContact;
   end;
   Lines.Free;
   BorderLines.Free;
end;

{ TGeometryDifference }

procedure TGeometryDifference.DoIt;
begin
   Error := gteNotImplemented;
   Output := nil;
end;

function TGeometryDifference.CHeckParameters: boolean;
begin
   result := false;
end;

{ TGeometryIntersection }

procedure TGeometryIntersection.DoIt;
begin
   Error := gteNotImplemented;
   Output := nil;
end;

function TGeometryIntersection.CHeckParameters: boolean;
begin
   result := false;
end;

INITIALIZATION
   RegisterOrbisClass(TGeometryTool);
   RegisterOrbisClass(TGeometryUnion);
   RegisterOrbisClass(TGeometryDifference);
   RegisterOrbisClass(TGeometryIntersection);
END.
