unit t_Orbis_XML;

interface
uses
  classes,
// KERNEL
   Elements,
   e_Class,
// 0TEMP
   T_Loader,
   T_Saver;

type

TCharSet = set of char;

TOrbisXMLLoader = class(TOrbisLoader)
private
   FReader: TReader;
   FStream: TStream;
   FLineNo: integer;
   FXMLSize: integer;
   FFWChar: char;        // dopedn naten znak
   function EndOfFile: Boolean;
   function IsSpace(aChar: char): Boolean;
   function  GetSingleChar: char;
   function  GetChar: char;
   function  GetXMLName(aCharSet: TCharSet): string; // naten etzec po space
   procedure AddParam(aElement: TElement; aName, aParam: string);
   procedure LoadElemElemProperty(aElement: TElement; aPropName: string);
   procedure LoadElemAttribute(aSlovo: string; aElement: TElement);
   procedure LoadElement(aName: string; aElement: TElement; aProp: string);{ <Name      /> }
   function  LoadComment(aName: string; aElement: TElement; aProp: string): string;  { <!--           --> }
   procedure LoadProcessingInstruction(aName: string; aElement: TElement; aProp: string);{ <?Name      /> }
   procedure LoadDeclaration(aName: string; aElement: TElement); { <!Name      /> }
   function  LoadTag(aElement: TElement): boolean;   { < > }

protected
   property  Reader: TReader read FReader write FReader;
   property  Stream: TStream read FStream write FStream;
public
   procedure   DoIt; override;
   function    GetGaugeMax: longInt; override;
   procedure   UpdateGauge; override;
   procedure   LoadInfo; override;
   procedure   LoadClass; override;
   procedure   LoadAll; override;
   procedure   LoadElements; override;
published
   property XMLSize: integer read FXMLSize;
end;



TOrbisXMLSaver = class(TOrbisSaver)
 private
    FWriter: TWriter;
    FStream: TStream;
    FShortCut: Boolean;

    function Distance: string;
    procedure SaveLine(const aLine: string);
 protected
    function  QProcessBefore(Element: TElement): boolean; override;
    function  QGoInside(Element: TElement): boolean; override;
    function  QProcessAfter(Element: TElement): boolean; override;
    procedure Process(Element: TElement); override;
 public
    procedure   DoIt; override;
    procedure   SaveInfo; override;
    procedure   SaveClass(aClass: TOrbisClass); override;
    procedure   NewElement; override;
    procedure   SaveProperties; override;
    procedure   SaveString(const aValue: string); override;
    property    Writer: TWriter read FWriter write FWriter;
    property    Stream: TStream read FStream write FStream;
    property    ShortCut: Boolean read FShortCut write FShortCut;
end;




implementation

uses
  SysUtils, Forms, Dialogs, TypInfo,
  //f_Gauge,
//KERNEL
  e_Type;




               { TXMLLoader }



function TOrbisXMLLoader.EndOfFile: Boolean;
begin
  Result:= Reader.Position >= XMLSize
end;


function TOrbisXMLLoader.GetSingleChar: char;
var
  aChar: char;
begin
  Reader.read(aChar, 1);
  if aChar = #13 then
     Inc(FLineNo);
  Result:= aChar;
  UpdateGauge
end;



function TOrbisXMLLoader.IsSpace(aChar: char): Boolean;
begin
  Result:= aChar in [#9, #10, #13, #32]
end;



function TOrbisXMLLoader.GetChar: char;
var
  aChar: char;
begin
{ dopedn ten znaku, aby se vynechvaly space znaky }
  Result:= FFWChar;
  aChar:= FFWChar;

  if IsSpace(FFWChar) then
  begin
    repeat
      aChar:= GetSingleChar
    until not IsSpace(aChar) or EndOfFile;
  end
  else
    aChar:= GetSingleChar;

  FFWChar:= aChar;
end;


function  TOrbisXMLLoader.GetXMLName(aCharSet: TCharSet): string; // naten etzec po space
var
  aChar: char;
begin
  Result:= '';
  if EndOfFile then
     Exit;
  aChar:= GetChar;
  while not isSpace(aChar) and not EndOfFile do
  begin
     Result:= Result + aChar;
     if aChar in aCharSet then Exit;
     aChar:= GetChar;
  end;
end;


function TOrbisXMLLoader.LoadTag(aElement: TElement): boolean;   { < > }
var
  Znak: char;
  aStr: string;
  aName: string;
  OnlySpace: Boolean;
begin
  Result:= True;

  aStr:= '';
  OnlySpace:= True;
  repeat
    Znak:= FFWChar;
    FFWChar:= GetSingleChar;

    if (Znak <> '<') then
    begin
      if not IsSpace(Znak) then
         OnlySpace:= False;
      aStr:= aStr + Znak;
    end
  until (Znak = '<') or EndOfFile;

  if EndOfFile then
  begin
    Result:= False;
    Exit;
  end;

  if (aElement is TEntity) and not OnlySpace then
  begin
    TEntity(aElement).AddElement(TStringElement.CreateAll('TEXT', aStr));
  end;

  aName:= GetXMLName(['>']);
  if Copy(aName, 1, 3) = '!--' then
     LoadComment(aName, aElement, '')
  else
  if Copy(aName, 1, 1) = '!' then
     LoadDeclaration(aName, aElement)
  else
  if Copy(aName, 1, 1) = '?' then
     LoadProcessingInstruction(aName, aElement, '')
  else
  if Copy(aName, 1, 1) = '/' then
     Result:= false
  else
     LoadElement(aName, aElement, '')
end;


procedure TOrbisXMLLoader.AddParam(aElement: TElement; aName, aParam: string);
begin
  if aElement.HasPropertyByName(aName) then
     aElement.SetPropertyStringByName(aName, aParam)
  else
  if aElement is TStringListElement then
     TStringListElement(aElement).AddNameString(aName, aParam)
end;



procedure TOrbisXMLLoader.LoadElemElemProperty(aElement: TElement; aPropName: string);
var
  aName: string;
begin
  aName:= GetXMLName(['>']);
  if Copy(aName, 1, 3) = '!--' then
     LoadComment(aName, aElement, '')
  else
  if Copy(aName, 1, 1) = '!' then
     LoadDeclaration(aName, aElement)
  else
  if Copy(aName, 1, 1) = '?' then
     LoadProcessingInstruction(aName, aElement, aPropName)
  else
     LoadElement(aName, aElement, aPropName)
end;



procedure TOrbisXMLLoader.LoadElemAttribute(aSlovo: string; aElement: TElement);
var
  aName, aParam: string;
  PosEq: integer;
  aChar: char;
  FirstDel: char;
  aChyba: Boolean;
begin
  PosEq:= Pos('=', aSlovo);
  if PosEq <= 0 then
  begin
     aName:= aSlovo;
     aParam:= GetXMLName(['=']);
  end
  else
  begin
    aName:= Copy(aSlovo, 1, PosEq-1);
  end;

  repeat // Peskoen vodn vozovky nebo apostrofu nebo <
    FirstDel:= GetChar;
  until (FirstDel in ['''', '"', '<']) or EndOfFile;


  if not (FirstDel in ['''','"', '<']) then
  begin
    ShowMessage('CHYBA pi natn atributu "' + aName + '", nenalezen vodn apostrof nebo uvozovka'
                + #13 + #10 + 'dek .' + IntToStr(FLineNo));
    Exit;
  end;


  if FirstDel = '<' then
  begin
    LoadElemElemProperty(aElement, aName);
    Exit;
  end;

  aParam:= '';
  aChyba:= False;
  repeat
    aChar:= FFWChar;
    FFWChar:= GetSingleChar;

    if aChar=FirstDel then
    begin
      break;
    end;
    aParam:= aParam + aChar;
  until EndOfFile;

  if EndOfFile then
  begin
    ShowMessage('CHYBA pi natn atributu "' + aName + '"'
                + #13 + #10 + 'dek .' + IntToStr(FLineNo));
    Exit;
  end;

  AddParam(aElement, aName, aParam);
end;



procedure TOrbisXMLLoader.LoadElement(aName: string; aElement: TElement; aProp: string);  { <ElementName     /> }
var
  aSlovo: string;
  EmptyElement: Boolean;
  ReadContent: Boolean;
  aSubElement: TElement;
begin
  { oven sprvnosti jmna elementu }

  { naten hlaviky, zatm nevm jestli je to przdn element }
  EmptyElement:= False;
  ReadContent:= False;

  if Pos('/>', aName) > 0 then
  begin
    EmptyElement:= True;
    aName:= Copy(aName, 1, Pos('/>', aName)-2);
  end
  else if Pos('>', aName) > 0 then
  begin
    ReadContent:= True;
    aName:= Copy(aName, 1, Pos('>', aName)-1);
  end;

  { Zaloen aSubElementu a pidn do aElementu }
  if GetOrbisClass(aName) <> nil then
     aSubElement:= CreateElementByName(aName)
  else
  if GetOrbisClass('T' + aName) <> nil then
     aSubElement:= CreateElementByName('T' + aName)
  else
  begin
    aSubElement:= TStringListElement.Create;
    aSubElement.Name:= aName;
  end;

  if aSubElement <> nil then
  begin
    Container.AddElement(aSubElement);

    if aProp = '' then
    begin
      if aElement is TEntity then
         TEntity(aElement).AddElement(aSubElement)
    end
    else
      aElement.SetPropertyLongIntByName(aProp, integer(aSubElement));
  end;


  if not EmptyElement and not ReadContent then
  repeat
    aSlovo:= getXMLName(['>', '=']);
    if Pos('!--', aSlovo) > 0 then
    begin
      LoadComment(aSlovo, aSubElement, aProp);
      aSlovo:= '';
    end
    else
    if Pos('/>', aSlovo) > 0 then
    begin
      EmptyElement:= True;
      aSlovo:= copy(aSlovo, 1, Pos('/>', aSlovo)-1);
    end
    else if Pos('>', aSlovo) > 0 then
    begin
      ReadContent:= True;
      aSlovo:= copy(aSlovo, 1, Pos('>', aSlovo)-1);
    end
    else if Pos('=', aSlovo) > 0 then
    begin
       LoadElemAttribute(aSlovo, aSubElement);
       aSlovo:= ''
    end;
    if aSlovo <> '' then
       LoadElemAttribute(aSlovo, aSubElement);
  until EmptyElement or ReadContent;

  if ReadContent then
  begin
    repeat
    until not LoadTag(aSubElement);
  end;
end;



function TOrbisXMLLoader.LoadComment(aName: string; aElement: TElement; aProp: string): string;  { <!--           --> }
var
  aChar: char;
  aStat: integer; {0: bezne, 1: 1.pom, 2: 2.pom za 1.}
begin
{ proztmn natn, polyk mezery, atd. }
  aStat:= 0;
  Result:= '' + FFWChar;
  aName:= Copy(aName, Pos('!--', aName) + 3, Length(aName));
  FFWChar:= GetSingleChar;
  while not EndOfFile do
  begin
    aChar:= FFWChar;
    FFWChar:= GetSingleChar;

    case aStat of
    0: if aChar = '-' then
          aStat:= 1
       else
          Result:= Result + aChar;
    1: if aChar = '-' then
          aStat:= 2
       else
       begin
          Result:= Result + '-' + aChar;
          aStat:= 0
       end;
    2: if aChar = '>' then
       begin
         Result:= aName + Result;
//         ShowMessage('<!-- ' + result + ' -->');
         Exit
       end
       else
       begin
          Result:= Result + '--' + aChar;
          aStat:= 0
       end;
    end;
  end;
end;



procedure TOrbisXMLLoader.LoadProcessingInstruction
          (aName: string; aElement: TElement; aProp: string); { <?PITarget      ?> }
var
  anID, aRes: integer;
  e: TElement;
begin
  if aName = '?SHORTCUT' then
  begin
    aName:= GetXMLName(['>']);
    Val(aName, anID, aRes);
    if aRes > 0 then
       ShowMessage('SHORTCUT ERROR ' + #13 + #10 + IntToStr(FLineNo))
    else
    begin
    try
      E := Container.Elements[anID];
      if aProp = '' then
         TEntity(aElement).AddElement(E)
      else
         aElement.SetPropertyLongIntByName(aProp, integer(E))
    except
       ShowMessage('SHORTCUT ERROR ' + #13 + #10 + IntToStr(FLineNo))
    end;
    end;
  end;

  while (Pos('?>', aName) <= 0) and not EndOfFile do
  begin
    aName:= GetXMLName(['>']);
  end;
end;


procedure TOrbisXMLLoader.LoadDeclaration(aName: string; aElement: TElement); { <!Name      /> }
begin
  while (Pos('>', aName) <= 0) and not EndOfFile do
  begin
    aName:= GetXMLName(['>']);
  end;
end;


procedure   TOrbisXMLLoader.DoIt;
begin
   Stream := TFileStream.Create(FileName, fmOpenRead{Write !!!!!!!} );
   FXMLSize:= Stream.Size;
   Reader := TReader.Create(Stream, 4096);

   FLineNo:= 1;
   // by OrbisLoader
   StartGauge;
   Prepare;
   LoadInfo;
   LoadClasses;
   LoadAll;
   Output := Container.elements[0];
   Finish;
   EndGauge;
   // by OrbisLoader

   Reader.Free;
   Stream.Free;
end;


function    TOrbisXMLLoader.GetGaugeMax: longInt;
begin
   result := XMLSize;
end;


procedure   TOrbisXMLLoader.UpdateGauge;
begin
   if Stream.Position mod 100 = 0 then
   begin
      Progress:= Stream.Position;
      Application.ProcessMessages
   end;
end;


procedure   TOrbisXMLLoader.LoadInfo;
begin
end;


procedure   TOrbisXMLLoader.LoadClass;
var
   aClassName: string;
   aClassID: longInt;
   aPropertyName: string;
   aPropertyType: integer;
begin
   aClassName := Reader.ReadString;
   aClassID   := Reader.ReadInteger;
   AddClass(aClassName);
   Reader.ReadListBegin;
   while not Reader.EndOfList do
   begin
      begin
         aPropertyName := Reader.ReadString;
         aPropertyType := Reader.ReadInteger;
         AddClassProperty(aPropertyName, aPropertyType);
      end;
   end;
   Reader.ReadListEnd;
end;


procedure   TOrbisXMLLoader.LoadAll;
begin
   FFWChar:= GetSingleChar;
   while not EndOfFile do
   begin
     LoadTag(Input)
   end;
end;



procedure   TOrbisXMLLoader.LoadElements;
var
   Realy: integer;
   aIndex: longInt;
begin
   Realy := Reader.ReadInteger;
   if not boolean(Realy) then exit;
   Reader.ReadListBegin;
   while not Reader.EndOfList do
   begin
      aIndex := Reader.ReadInteger;
      AddNextElement(aIndex);
   end;
   Reader.ReadListEnd;
end;




                {  TXMLSaver  }



function  TOrbisXMLSaver.QProcessBefore(Element: TElement): Boolean;
begin
  Result:= True;
  ShortCut:= False;
  if (Element.ID > BaseID) and (Element.ID <> -1) then
  begin
    SaveLine('<?SHORTCUT ' + IntToStr(Element.ID) +  ' ?>');

{
    SaveLine('<' + Element.ClassName +
            ' ID="' + Element.GetPropertyStringByName('ID') + '" />');
}
    ShortCut:= true;
    Result:= false;
  end;
end;


function  TOrbisXMLSaver.QGoInside(Element: TElement): boolean;
begin
  Result:= false;
  if ShortCut then
     Exit;
  result := Element is TEntity;
end;


function  TOrbisXMLSaver.QProcessAfter(Element: TElement): Boolean;
begin
  Result:= false;

  if ShortCut then
  begin
    ShortCut:= False;
    Exit;
  end;

  if not (Element is TEntity) then
     Exit; // prazdny element

  SaveLine('</' + Element.ClassName + '>');
end;


procedure   TOrbisXMLSaver.DoIt;
begin
  try
   Stream := TFileStream.Create(FileName, fmCreate);

   Writer := TWriter.Create(Stream, 4096);
   {!ShowMessage('Before DoIt');}
   inherited DoIt;
   {!ShowMessage('After DoIt');}
   Writer.Free;
   Stream.Free;
  except
    ShowMessage('CHYBA pi ukldn do "' + FileName + '"'
                + #13 + #10 + IntToStr(ActualElement.ID));

  end;
end;


procedure   TOrbisXMLSaver.SaveInfo;
const
   Magic: array[0..3] of Char = 'ORB0';
begin
end;


procedure   TOrbisXMLSaver.SaveClass(aClass: TOrbisClass);
begin
end;


procedure   TOrbisXMLSaver.NewElement;
var
  aName: string;
  ElLine: string;
begin
  aName:= ActualElement.Name;
  ElLine:= '<' + ActualElement.ClassName +
           ' ID="' + IntToStr(ActualElement.ID) + '"';

  if aName <> ActualElement.ClassName then
     ElLine:= ElLine + ' NAME=''' + ActualElement.Name + '''';

  SaveLine(ElLine)
end;


procedure   TOrbisXMLSaver.SaveProperties;
var
   i: integer;
   Count: integer;
   aName, aValue: string;
   aActEl: TElement;
   aInEl: TElement;
begin
  Count:= Instance.PropCount;
  for i := 2 to Count - 1 do
  begin
    try

    aName:= Instance.PropertyName[i];

    if not Instance.PropertyStored[i] then
       Continue;

    if Instance.PropertyTypeKind[i] = tkClass then
    begin
      aActEl:= ActualElement;
      aInEl:= TElement(ActualElement.GetPropertyLongIntByName(aName));
      try
        if not (TObject(aInEl) is TElement) then
        begin
           continue;
        end
      except
        continue;
      end;

      SaveLine('  <!-- Element property ' + aName + ' -->');
      SaveLine('  ' + aName + '=');

      Level:= Level + 3;
      if QProcessBefore(aInEl) then
         Process(aInEl);
      if QGoInside(aInEl) then
         GoInside(aInEl);
      if QProcessAfter(aInEl) then
         Process(aInEl);

      Level:= Level - 3;

      Instance.Instance:= aActEl;
      ActualElement:= aActEl;
    end
    else
    begin // obyejn properta
      aValue:= ActualElement.GetPropertyStringByName(aName);
      SaveLine('  ' + aName + '="' + aValue + '"');
    end;

    except
      ShowMessage('CHYBA pi ukldn property do "' + FileName + '"'
                  + #13 + #10 + 'ActualElement.ID = ' + IntToStr(ActualElement.ID)
                  + #13 + #10 + 'Properta: ' + aName);
    end;
  end;

  if ActualElement is TEntity then
     SaveLine('>')
  else
     SaveLine('/>');
end;



procedure TOrbisXMLSaver.Process(Element: TElement);
begin
   Element.ID := Index;
   Index:= Index + 1;
   if (Index mod 100) = 0 then
   begin
      Progress := Index;
      Application.ProcessMessages;
   end;
   ActualElement := Element;
   Instance.Instance := Element;
   if (not CheckClass) and (not PrepareClassesBefore)then
      SaveClass(OrbisClasses.ActualClass);
   NewElement;
   SaveProperties;
end;





procedure   TOrbisXMLSaver.SaveString(const aValue: string);
begin
   Writer.write(Pointer(aValue)^, Length(aValue));
end;



function TOrbisXMLSaver.Distance: string;
var
  I: Integer;
begin
  Result:= '';
  for I:= 0 to Level do
      Result:= Result + ' '
end;


procedure TOrbisXMLSaver.SaveLine(const aLine: string);
begin
  SaveString(Distance);
  SaveString(aLine);
  SaveString(#13 + #10);
end;



end.
