unit c_Actions;

interface
uses
   ActnList,
   Classes,
   Tools;

type

TOrbisAction = class(TAction)
  private
    FCLient: boolean;
protected
    procedure SetCLient(const Value: boolean); virtual;
published
   property CLient: boolean read FCLient write SetCLient;
end;
TOrbisActionClass = class of TOrbisAction;

TScriptAction = class(TOrbisAction)
private
   FScript: string;
   procedure SetScript(const Value: string);
public
   function Execute: Boolean; override;
published
   property Script: string read FScript write SetScript;
end;

TToolAction = class(TOrbisAction)
private
   FTool: TTool;
   FDirectExecute: boolean;
    FOrbisMonitor: boolean;
   procedure SetTool(const Value: TTool);
   procedure SetDirectExecute(const Value: boolean);
    procedure SetOrbisMonitor(const Value: boolean);
protected
    procedure SetCLient(const Value: boolean); override;
public
    constructor Create(AOwner: TComponent); override;
   function Execute: Boolean; override;
   procedure CreateTool; virtual;
   property Tool: TTool read FTool write SetTool;
published
   property DirectExecute: boolean read FDirectExecute write SetDirectExecute;
   property OrbisMonitor: boolean read FOrbisMonitor write SetOrbisMonitor;
end;



implementation
uses
   SysUtils,
   T_CmdLn,
   u_String;

{ TScriptAction }

function TScriptAction.Execute: Boolean;
var
   CL: TCommandLine;
   S: string;
begin
   result := inherited Execute;
   if Script = '' then exit;
   CL := TCommandLine.Create;
   CL.FileName := GetAbsolutePath(Script);
   Cl.DoIt;
   CL.Free;
end;

procedure TScriptAction.SetScript(const Value: string);
begin
  FScript := Value;
end;

{ TToolAction }

constructor TToolAction.Create(AOwner: TComponent);
begin
  inherited;
  DisableIfNoHandler := false;
end;

procedure TToolAction.CreateTool;
begin

end;

function TToolAction.Execute: Boolean;
begin
   result := inherited Execute;
   if DirectExecute and (FTool <> nil) then FTool.DoIt;
end;

procedure TToolAction.SetCLient(const Value: boolean);
begin
   inherited;
   if Tool <> nil then
      Tool.Client := Value;
end;

procedure TToolAction.SetDirectExecute(const Value: boolean);
begin
  FDirectExecute := Value;
end;

procedure TToolAction.SetOrbisMonitor(const Value: boolean);
begin
  FOrbisMonitor := Value;
end;

procedure TToolAction.SetTool(const Value: TTool);
begin
   FTool := Value;
   if Name <> ClassName then exit;
   if Value <> nil then Name := 'Action_'+Value.Name;
end;

{ TOrbisAction }

procedure TOrbisAction.SetCLient(const Value: boolean);
begin
  FCLient := Value;
end;

end.
