unit f_GraphicForm;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  f_ExtendedOrbisForm, ActnList, Menus, ComCtrls, MenuBar, ToolWin,
  ExtCtrls,
  // KERNEL
  Tools;

type
TGraphic_Form = class(TExtendedORBIS_Form)
   Image: TImage;
   procedure ImageDragDrop(Sender, Source: TObject; X, Y: Integer);
   procedure ImageDragOver(Sender, Source: TObject; X, Y: Integer; State: TDragState; var Accept: Boolean);
   procedure ImageEndDrag(Sender, Target: TObject; X, Y: Integer);
   procedure ImageMouseDown(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
   procedure ImageMouseMove(Sender: TObject; Shift: TShiftState; X, Y: Integer);
   procedure ImageMouseUp(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
   procedure ImageStartDrag(Sender: TObject; var DragObject: TDragObject);
    procedure FormResize(Sender: TObject);
    procedure FormKeyDown(Sender: TObject; var Key: Word;
      Shift: TShiftState);
private
   IsActive: boolean;
   LockMouseUp: boolean;
public
   procedure SetTool(const Value: TTool); override;
end;


implementation
uses
   // KERNEL
   m_ORBIS,
   // DEVICE
   t_Device;

{$R *.DFM}

procedure TGraphic_Form.ImageDragDrop(Sender, Source: TObject; X,
  Y: Integer);
begin
   inherited;
   if Tool = nil then exit;
   TDeviceTool(Tool).PostMouseEvent(tm_DragDrop, Tool, mbLeft, [], X, Y, IsActive);
   //Tool.PostTextMsg(tm_AddToProtocol,'DragDrop',nil);
end;

procedure TGraphic_Form.SetTool(const Value: TTool);
begin
   inherited;
   if not (Tool is TDeviceTool) then Tool := nil;
end;

procedure TGraphic_Form.ImageDragOver(Sender, Source: TObject; X,
  Y: Integer; State: TDragState; var Accept: Boolean);
begin
   inherited;
   Accept := true;
   if Tool = nil then exit;
   TDeviceTool(Tool).PostMouseEvent(tm_DragOver, Tool, mbLeft, [], X, Y, IsActive);
   //Tool.PostTextMsg(tm_AddToProtocol,'...DragOver '+ IntToStr(X)+' '+IntToStr(Y),nil);
end;

procedure TGraphic_Form.ImageEndDrag(Sender, Target: TObject; X,
  Y: Integer);
begin
   inherited;
   Image.DragCursor := crDrag;
   if Tool = nil then exit;
   TDeviceTool(Tool).PostMouseEvent(tm_DragEnd, Tool, mbLeft, [], X, Y, IsActive);
   //Tool.PostTextMsg(tm_AddToProtocol,'EndDrag',nil);
   LockMouseUp := false;
end;

procedure TGraphic_Form.ImageMouseDown(Sender: TObject;
  Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
begin
   inherited;
   if Tool = nil then exit;
   TDeviceTool(Tool).PostMouseEvent(tm_MouseDown, Tool, Button, Shift, X, Y, IsActive);
   //Tool.PostTextMsg(tm_AddToProtocol,'***********************',nil);
   //Tool.PostTextMsg(tm_AddToProtocol,'MouseDown',nil);
end;

procedure TGraphic_Form.ImageMouseMove(Sender: TObject; Shift: TShiftState;
  X, Y: Integer);
begin
   inherited;
   if Tool = nil then exit;
   TDeviceTool(Tool).PostMouseEvent(tm_MouseMove, Tool, mbLeft, Shift, X, Y, IsActive);
   //Tool.PostTextMsg(tm_AddToProtocol,'MouseMove '+ IntToStr(X)+' '+IntToStr(Y),nil);
   if ssLeft in Shift then
   begin
      LockMouseUp := true;
      Image.BeginDrag(false,2);
   end;

end;

procedure TGraphic_Form.ImageMouseUp(Sender: TObject; Button: TMouseButton;
  Shift: TShiftState; X, Y: Integer);
begin
   inherited;
   if Tool = nil then exit;
   if LockMouseUp then exit;
   TDeviceTool(Tool).PostMouseEvent(tm_MouseUp, Tool, Button, Shift, X, Y, IsActive);
   //Tool.PostTextMsg(tm_AddToProtocol,'MouseUp',nil);
   IsActive := true;
end;

procedure TGraphic_Form.ImageStartDrag(Sender: TObject;
  var DragObject: TDragObject);
begin
   inherited;
   if Tool = nil then exit;
   TDeviceTool(Tool).PostMouseEvent(tm_StartDrag, Tool, mbLeft, [], 0, 0, IsActive);
   //Tool.PostTextMsg(tm_AddToProtocol,'StartDrag',nil);
end;

procedure TGraphic_Form.FormResize(Sender: TObject);
begin
   inherited;
   try
      if not Visible then exit;
      Image.Picture.Bitmap.Height:= Image.Height;
      Image.Picture.Bitmap.Width:= Image.Width;
      if Tool = nil then exit;
      Tool.PostMsg(tm_ReSize, Tool, nil, false);
      {
      Tool.Device.SetupPage(nil);
      Tool.PostMsg(tm_ReView, Tool, nil, DontKill);
      }
   except
   end;
end;

procedure TGraphic_Form.FormKeyDown(Sender: TObject; var Key: Word;
  Shift: TShiftState);
begin
   inherited;
   if Tool = nil then exit;
   TDeviceTool(Tool).PostKeyEvent(tm_KeyDown, Tool, Key, Shift);
   //Tool.PostTextMsg(tm_AddToProtocol,'Key: '+IntToStr(Key)+ ' '+CHar(Key),nil);
end;

end.
