unit r_OrbisGraphicFrame;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, r_OrbisFrame, Tools,ExtCtrls;

type
  TOrbisGraphic_Frame = class(TOrbis_Frame)
    Image: TImage;
    procedure ImageDragDrop(Sender, Source: TObject; X, Y: Integer);
    procedure ImageDragOver(Sender, Source: TObject; X, Y: Integer;
      State: TDragState; var Accept: Boolean);
    procedure ImageEndDrag(Sender, Target: TObject; X, Y: Integer);
    procedure ImageMouseDown(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure ImageMouseMove(Sender: TObject; Shift: TShiftState; X,
      Y: Integer);
    procedure ImageMouseUp(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure ImageStartDrag(Sender: TObject; var DragObject: TDragObject);
    procedure FrameResize(Sender: TObject);
  private
    FSizeMove: boolean;
    FImageWidth: integer;
    FImageHeight: integer;
    procedure SetSizeMove(const Value: boolean);
    procedure SetImageHeight(const Value: integer);
    procedure SetImageWidth(const Value: integer);
  private
    IsActive: boolean;
    LockMouseUp: boolean;
    RealyDraged: boolean;
    DragShift: TShiftState;
    property  SizeMove: boolean read FSizeMove write SetSizeMove;
  public
    procedure SetTool(const Value: TTool); override;
    procedure StartSizeMove;
    procedure EndSizeMove;
    property ImageHeight: integer read FImageHeight write SetImageHeight;
    Property ImageWidth: integer read FImageWidth write SetImageWidth;
  end;


implementation
uses
   m_Orbis,
   t_Device;

{$R *.dfm}

procedure TOrbisGraphic_Frame.ImageDragDrop(Sender, Source: TObject; X,
  Y: Integer);
begin
   inherited;
   if Tool = nil then exit;
   TDeviceTool(Tool).PostMouseEvent(tm_DragDrop, Tool, mbLeft, DragShift, X, Y, IsActive);
   {$IFDEF OCX}
   Application.ProcessMessages;
   {$ENDIF}
end;

procedure TOrbisGraphic_Frame.ImageDragOver(Sender, Source: TObject; X,
  Y: Integer; State: TDragState; var Accept: Boolean);
begin
   inherited;
   Accept := true;
   RealyDraged := true;
   if Tool = nil then exit;
   TDeviceTool(Tool).PostMouseEvent(tm_DragOver, Tool, mbLeft, DragShift, X, Y, IsActive);
   {$IFDEF OCX}
   Application.ProcessMessages;
   {$ENDIF}
end;

procedure TOrbisGraphic_Frame.ImageEndDrag(Sender, Target: TObject; X,
  Y: Integer);
var
   P: TPoint;
begin
   inherited;
   Image.DragCursor := crDrag;
   if Tool = nil then exit;
   TDeviceTool(Tool).PostMouseEvent(tm_DragEnd, Tool, mbLeft, DragShift, X, Y, IsActive);
   if not RealyDraged then
   begin
      P.X := X;
      P.Y := Y;
      P := Image.ScreenToClient(P);
      TDeviceTool(Tool).PostMouseEvent(tm_MouseUp, Tool, mbLeft, DragShift, P.X, P.Y, IsActive);
   end;
   //Tool.PostTextMsg(tm_AddToProtocol,'EndDrag',nil);
   RealyDraged := false;
   LockMouseUp := false;
   {$IFDEF OCX}
   Application.ProcessMessages;
   {$ENDIF}
end;

procedure TOrbisGraphic_Frame.ImageMouseDown(Sender: TObject;
  Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
begin
   inherited;
   if Tool = nil then exit;
   DragShift := Shift;
   TDeviceTool(Tool).PostMouseEvent(tm_MouseDown, Tool, Button, Shift, X, Y, IsActive);
   {$IFDEF OCX}
   Application.ProcessMessages;
   {$ENDIF}
end;

procedure TOrbisGraphic_Frame.ImageMouseMove(Sender: TObject; Shift: TShiftState;
  X, Y: Integer);
begin
   inherited;
   if Tool = nil then exit;
   TDeviceTool(Tool).PostMouseEvent(tm_MouseMove, Tool, mbLeft, Shift, X, Y, IsActive);
   //Tool.PostTextMsg(tm_AddToProtocol,'MouseMove '+ IntToStr(X)+' '+IntToStr(Y),nil);
   if ssLeft in Shift then
   begin
      LockMouseUp := true;
      DragShift := Shift;
      Image.BeginDrag(false,1);
   end;
   {$IFDEF OCX}
   Application.ProcessMessages;
   {$ENDIF}

end;

procedure TOrbisGraphic_Frame.ImageMouseUp(Sender: TObject; Button: TMouseButton;
  Shift: TShiftState; X, Y: Integer);
begin
   inherited;
   if Tool = nil then exit;
   if LockMouseUp then exit;
   TDeviceTool(Tool).PostMouseEvent(tm_MouseUp, Tool, Button, Shift, X, Y, IsActive);
   //Tool.PostTextMsg(tm_AddToProtocol,'MouseUp',nil);
   IsActive := true;
   {$IFDEF OCX}
   Application.ProcessMessages;
   {$ENDIF}
end;

procedure TOrbisGraphic_Frame.ImageStartDrag(Sender: TObject;
  var DragObject: TDragObject);
begin
   inherited;
   if Tool = nil then exit;
   TDeviceTool(Tool).PostMouseEvent(tm_StartDrag, Tool, mbLeft, DragShift, 0, 0, IsActive);
   {$IFDEF OCX}
   Application.ProcessMessages;
   {$ENDIF}
end;

procedure TOrbisGraphic_Frame.FrameResize(Sender: TObject);
begin
   try
      if not Visible then exit;
      if (ImageHeight = Image.Height) and (ImageWidth = Image.Width) then exit;
      Image.Picture.Bitmap.Height:= Image.Height;
      Image.Picture.Bitmap.Width:= Image.Width;
      if SizeMove  then exit;
      if Tool = nil then exit;
      if Tool.Idle then exit;
      Tool.PostMsg(tm_ReSize, Tool, nil, false);
      Application.ProcessMessages;
      ImageHeight := Image.Height;
      ImageWidth := Image.Width;
      {
      Tool.Device.SetupPage(nil);
      Tool.PostMsg(tm_ReView, Tool, nil, DontKill);
      }
   except
   end;
   {$IFDEF OCX}
   Application.ProcessMessages;
   {$ENDIF}
end;

procedure TOrbisGraphic_Frame.SetTool(const Value: TTool);
begin
   if not (Value is TDeviceTool) then exit;
   inherited;
end;



procedure TOrbisGraphic_Frame.EndSizeMove;
begin
    SizeMove := false;
    FrameResize(nil);
end;


procedure TOrbisGraphic_Frame.SetSizeMove(const Value: boolean);
begin
  FSizeMove := Value;
end;

procedure TOrbisGraphic_Frame.StartSizeMove;
begin
   SizeMove := true;
   Image.Picture.Bitmap.Canvas.Brush.Color := $FFFFFF;
end;

procedure TOrbisGraphic_Frame.SetImageHeight(const Value: integer);
begin
  FImageHeight := Value;
end;

procedure TOrbisGraphic_Frame.SetImageWidth(const Value: integer);
begin
  FImageWidth := Value;
end;

end.

