unit t_MeasureMonitor;
interface
uses
   Elements,
   m_InterA,
   r_MeasureMonitor,
   t_OrbisFrame;

type

TMeasureMonitorFrame = class(TOrbisFrame)
  private
    FAreaText: string;
    FLengthFormat: string;
    FLengthText: string;
    FAreaFormat: string;
    FAreaInHA: boolean;
    function GetMeasureFrame: TMeasureMonitor_Frame;
    procedure SetAreaFormat(const Value: string);
    procedure SetAreaText(const Value: string);
    procedure SetLengthFormat(const Value: string);
    procedure SetLengthText(const Value: string);
    procedure SetAreaInHA(const Value: boolean);
public
   procedure CreateFrame; override;
   procedure On_tm_LengthChanged(var aMessage: TOrbisMessage; var Handled: boolean);
   procedure On_tm_AreaChanged(var aMessage: TOrbisMessage; var Handled: boolean);
   procedure HandleMessage(var aMessage: TOrbisMessage; var Handled: boolean); override;
   property  MeasureFrame: TMeasureMonitor_Frame read GetMeasureFrame;
published
   property  LengthText: string read FLengthText write SetLengthText;
   property  LengthFormat: string read FLengthFormat write SetLengthFormat;
   property  AreaText: string read FAreaText write SetAreaText;
   property  AreaFormat: string read FAreaFormat write SetAreaFormat;
   property  AreaInHA: boolean read FAreaInHA write SetAreaInHA;
end;

implementation
uses
   SysUtils,
   m_Orbis,
   e_Class,
   u_String;

{ TMeasureMonitorFrame }

procedure TMeasureMonitorFrame.CreateFrame;
begin
   aFrame := TMeasureMonitor_Frame.Create(nil);
end;

function TMeasureMonitorFrame.GetMeasureFrame: TMeasureMonitor_Frame;
begin
   result := TMeasureMonitor_Frame(aFrame);
end;


procedure TMeasureMonitorFrame.HandleMessage(var aMessage: TOrbisMessage;
  var Handled: boolean);
begin
   inherited HandleMessage(aMessage, Handled);
   if Handled then exit;
   case aMessage.Message of
      tm_LengthCHanged: on_tm_LengthChanged(aMessage, Handled);
      tm_AreaCHanged: on_tm_AreaChanged(aMessage, Handled);
   end;
end;

procedure TMeasureMonitorFrame.On_tm_AreaChanged(var aMessage: TOrbisMessage; var Handled: boolean);
var
   D: double;
   S,S1,S2: string;
begin
   if not (aMessage is TOrbisTextMessage) then exit;
   StringAsTwoParams(TOrbisTextMessage(aMessage).Text,':',S1,S2);
   D := StrToFloatDef(S1,0);
   if AreaInHA then
   begin
      D := D/10000;
      S := ' ha';
   end
   else
      S := ' m2';
   MeasureFrame.l_Text.Caption := AreaText + ' ' + FormatFloat(AreaFormat,D)+S;

end;



procedure TMeasureMonitorFrame.On_tm_LengthChanged(var aMessage: TOrbisMessage; var Handled: boolean);
var
   D: double;
   S1,S2: string;
begin
   if not (aMessage is TOrbisTextMessage) then exit;
   StringAsTwoParams(TOrbisTextMessage(aMessage).Text,':',S1,S2);
   D := StrToFloatDef(S1,0);
   MeasureFrame.l_Text.Caption := LengthText + ' ' + FormatFloat(LengthFormat,D)+' m';

end;


procedure TMeasureMonitorFrame.SetAreaFormat(const Value: string);
begin
  FAreaFormat := Value;
end;

procedure TMeasureMonitorFrame.SetAreaInHA(const Value: boolean);
begin
  FAreaInHA := Value;
end;

procedure TMeasureMonitorFrame.SetAreaText(const Value: string);
begin
  FAreaText := Value;
end;

procedure TMeasureMonitorFrame.SetLengthFormat(const Value: string);
begin
  FLengthFormat := Value;
end;

procedure TMeasureMonitorFrame.SetLengthText(const Value: string);
begin
  FLengthText := Value;
end;

INITIALIZATION
   RegisterOrbisClass(TMeasureMonitorFrame);
END.
