unit t_OrbisControlElements;

interface
uses
   Forms,
   Buttons,
   Classes,
   ExtCtrls,
   Gauges,
   Grids,
   Graphics,
   CheckLst,
   StdCtrls,
   t_ControlTool,
   Elements;

type

TOrbis_Button = class(TControlTool)
  private
    FOnClickScript: string;
    function GetButton: TBitBtn;
    procedure SetOnClickScript(const Value: string);
    function GetModalResult: integer;
    procedure SetModalResult(const Value: integer);
protected
    function GetCaption: string; override;
    function GetFont: TFont; override;
    procedure SetCaption(const Value: string); override;
    procedure OnClick(Sender: TObject);
public
   procedure CreateControl; override;
   property  Button: TBitBtn read GetButton;
published
   property OnClickScript: string read FOnClickScript write SetOnClickScript;
   property ModalResult: integer read GetModalResult write SetModalResult;
end;

TOrbis_Label = class(TControlTool)
private
    function GetLabel: TLabel;
protected
    function GetFont: TFont; override;
    function GetAutoSize: boolean; override;
    procedure SetAutoSize(const Value: boolean); override;
    function GetCaption: string; override;
    procedure SetCaption(const Value: string); override;
public
   procedure CreateControl; override;
   property  Label_: TLabel read GetLabel;
end;

TOrbis_Panel = class(TControlTool)
  private
    FOnClickScript: string;
    function GetPanel: TPanel;
    procedure SetOnClickScript(const Value: string);
protected
   function GetFont: TFont; override;
   function GetColor: integer; override;
   procedure SetColor(const Value: integer); override;
    function GetCaption: string; override;
    procedure SetCaption(const Value: string); override;
   procedure OnClick(Sender: TObject);
public
   procedure CreateControl; override;
   property  Panel: TPanel read GetPanel;
published
   property  OnClickScript: string read FOnClickScript write SetOnClickScript;
end;

TOrbis_ScrollBox = class(TControlTool)
  private
    function GetScrollBox: TScrollBox;
    function GetAutoScroll: boolean;
    function GetHoriz: boolean;
    function GetVert: boolean;
    procedure SetAutoScroll(const Value: boolean);
    procedure Sethoriz(const Value: boolean);
    procedure SetVert(const Value: boolean);
protected
   function GetFont: TFont; override;
   function GetColor: integer; override;
   procedure SetColor(const Value: integer); override;
public
   procedure CreateControl; override;
   property  ScrollBox: TScrollBox read GetScrollBox;
published
   property  AutoScroll: boolean read GetAutoScroll write SetAutoScroll;
   property  Horiz: boolean read GetHoriz write Sethoriz;
   property  Vert: boolean  read GetVert  write SetVert;
end;

TOrbis_Splitter = class(TControlTool)
  private
    function GetSplitter: TSplitter;
public
   procedure CreateControl; override;
   property  Splitter: TSplitter read GetSplitter;
end;

TOrbis_ComboBox = class(TControlTool)
  private
    FOnChangeScript: string;
    function GetComboBox: TComboBox;
    function GetSelectedElement: TElement;
    function GetDropDownCount: integer;
    procedure SetDropDownCount(const Value: integer);
    function GetText: string;
    procedure SetText(const Value: string);
    procedure SetOnChangeScript(const Value: string);
protected
   function    GetFont: TFont; override;
   procedure   SetInput(aInput: TElement); override;
public
   procedure OnChange(Sender: TObject);
   procedure CreateControl; override;
   property  ComboBox: TComboBox read GetComboBox;
published
   property SelectedElement: TElement read GetSelectedElement stored false;
   property DropDownCount: integer read GetDropDownCount write SetDropDownCount;
   property Text: string read GetText write SetText;
   property OnChangeScript: string read FOnChangeScript write SetOnChangeScript;
end;

TOrbis_ListBox = class(TControlTool)
private
   FSelectedElements: TEntity;
    FOnClickScript: string;
    function GetListBox: TListBox;
    function GetSelectedElement: TElement;
    function GetSelectedElements: TEntity;
    function GetExtendedSelect: boolean;
    function GetMultiSelect: boolean;
    procedure SetExtendedSelect(const Value: boolean);
    procedure SetMultiSelect(const Value: boolean);
    procedure SetOnClickScript(const Value: string);
    procedure SetSelectedElement(const Value: TElement);
protected
   procedure   SetInput(aInput: TElement); override;
    procedure OnClick(Sender: TObject);
    function GetFont: TFont; override;
    function GetColor: integer; override;
    procedure SetColor(const Value: integer); override;
public
   procedure CreateControl; override;
   property  ListBox: TListBox read GetListBox;
published
   property SelectedElement: TElement read GetSelectedElement write SetSelectedElement stored false;
   property SelectedElements: TEntity read GetSelectedElements stored false;
   property OnClickScript: string read FOnClickScript write SetOnClickScript;
   property ExtendedSelect: boolean read GetExtendedSelect write SetExtendedSelect;
   property MultiSelect: boolean read GetMultiSelect write SetMultiSelect;
end;

TOrbis_CheckListBox = class(TControlTool)
private
    FSelectedElements: TEntity;
    FCheckedElements: TEntity;
    FOnClickScript: string;
    FCheckByProperty: boolean;
    FCheckPropertyName: string;
    FCHeckByPropertyReverse: boolean;
    FOnClickCheckScript: string;
    function GetSelectedElement: TElement;
    function GetSelectedElements: TEntity;
    function GetMultiSelect: boolean;
    procedure SetMultiSelect(const Value: boolean);
    procedure SetOnClickScript(const Value: string);
    function GetCheckedElements: TEntity;
    function GetCHListBox: TCheckListBox;
    procedure SetCheckedElements(const Value: TEntity);
    procedure SetCheckAll(const Value: boolean);
    function GetCheckedString: string;
    procedure SetCheckedString(const Value: string);
    procedure SetCheckByProperty(const Value: boolean);
    procedure SetCheckPropertyName(const Value: string);
    procedure SetCHeckByPropertyReverse(const Value: boolean);
    procedure SetOnClickCheckScript(const Value: string);
    function GetSelectedIsChecked: boolean;
    procedure SetSelectedIsChecked(const Value: boolean);
    procedure SetSelectedElement(const Value: TElement);
protected
   procedure   SetInput(aInput: TElement); override;
   procedure OnClick(Sender: TObject);
   procedure OnClickCheck(Sender: TObject);
    function GetFont: TFont; override;
    function GetColor: integer; override;
    procedure SetColor(const Value: integer); override;
public
   procedure CreateControl; override;
   property  ChListBox: TCheckListBox read GetCHListBox;
published
   property SelectedElement: TElement read GetSelectedElement write SetSelectedElement stored false;
   property SelectedElements: TEntity read GetSelectedElements stored false;
   property CheckedElements: TEntity read GetCheckedElements write SetCheckedElements stored false;
   property SelectedIsChecked: boolean read GetSelectedIsChecked write SetSelectedIsChecked;
   property CheckedString: string read GetCheckedString write SetCheckedString;
   property OnClickScript: string read FOnClickScript write SetOnClickScript;
   property OnClickCheckScript: string read FOnClickCheckScript write SetOnClickCheckScript;
   property MultiSelect: boolean read GetMultiSelect write SetMultiSelect;
   property CheckAll: boolean write SetCheckAll;
   property CheckByProperty: boolean read FCheckByProperty write SetCheckByProperty;
   property CHeckByPropertyReverse: boolean read FCHeckByPropertyReverse write SetCHeckByPropertyReverse;
   property CheckPropertyName: string read FCheckPropertyName write SetCheckPropertyName;
end;

TOrbis_Edit = class(TControlTool)
  private
    FOnChangeScript: string;
    function GetEdit: TEdit;
    procedure SetText(const Value: string);
    function GetText: string;
    procedure SetOnChangeScript(const Value: string);
protected
    function GetFont: TFont; override;
public
   procedure CreateControl; override;
   property  Edit: TEdit read GetEdit;
   procedure OnChange(Sender: TObject);
published
   property Text: string read GetText write SetText;
   property OnChangeScript: string read FOnChangeScript write SetOnChangeScript;
end;

TOrbis_CheckBox = class(TControlTool)
  private
    FOnClickScript: string;
    function GetChecked: boolean;
    procedure SetChecked(const Value: boolean);
    function GetCheckBox: TCheckBox;
    function GetAllignment: TLeftRight;
    procedure SetAllignment(const Value: TLeftRight);
    procedure SetOnClickScript(const Value: string);
protected
    procedure OnClick(Sender: TObject);
    function GetFont: TFont; override;
    function GetCaption: string; override;
    procedure SetCaption(const Value: string); override;
public
   procedure CreateControl; override;
   property  CheckBox: TCheckBox read GetCheckBox;
published
   property Checked: boolean read GetChecked write SetChecked;
   property Allignment: TLeftRight read GetAllignment write SetAllignment;
   property OnClickScript: string read FOnClickScript write SetOnClickScript;
end;

TOrbis_RadioButton = class(TControlTool)
  private
    function GetChecked: boolean;
    procedure SetChecked(const Value: boolean);
    function GetAllignment: TLeftRight;
    function GetRadioButton: TRadioButton;
    procedure SetAllignment(const Value: TLeftRight);
protected
    function GetFont: TFont; override;
    function GetCaption: string; override;
    procedure SetCaption(const Value: string); override;
public
   procedure CreateControl; override;
   property  RadioButton: TRadioButton read GetRadioButton;
published
   property Checked: boolean read GetChecked write SetChecked;
   property Allignment: TLeftRight read GetAllignment write SetAllignment;
end;

TOrbis_RadioGroup= class(TControlTool)
  private
    FOnClickScript: string;
    function GetRadioGroup: TRadioGroup;
    procedure SetItemIndex(const Value: integer);
    procedure SetNextItem(const Value: string);
    function GetItemIndex: integer;
    function GetColumns: integer;
    procedure SetColumns(const Value: integer);
    procedure SetOnClickScript(const Value: string);
protected
    procedure OnClick(Sender: TObject);
    function GetFont: TFont; override;
    function GetCaption: string; override;
    procedure SetCaption(const Value: string); override;
public
   procedure CreateControl; override;
   property  RadioGroup: TRadioGroup read GetRadioGroup;
published
   property NextItem: string write SetNextItem;
   property ItemIndex: integer read GetItemIndex write SetItemIndex;
   property Columns: integer read GetColumns write SetColumns;
   property OnClickScript: string read FOnClickScript write SetOnClickScript;
end;


TOrbis_StringGrid = class(TControlTool)
  private
    FACol: integer;
    FARow: integer;
    function GetStringGrid: TStringGrid;
    procedure SetACol(const Value: integer);
    procedure SetARow(const Value: integer);
    procedure SetAValue(const Value: string);
    function GetAValue: string;
protected
    function GetFont: TFont; override;
public
   procedure CreateControl; override;
   property  StringGrid: TStringGrid read GetStringGrid;
published
   property ARow: integer read FARow write SetARow;
   property ACol: integer read FACol write SetACol;
   property AValue: string read GetAValue write SetAValue;
end;

TOrbis_Memo = class(TControlTool)
  private
    function GetMemo: TMemo;
    procedure SetClearIt(const Value: boolean);
    procedure SetNextText(const Value: string);
    function GetScrollbars: TScrollStyle;
    procedure SetScrollbars(const Value: TScrollStyle);
    function GetLastText: string;
    procedure SetLastText(const Value: string);
    function GetWordWrap: boolean;
    procedure SetWordWrap(const Value: boolean);
    procedure SetLoadFromFile(const Value: string);
    procedure SetSaveToFile(const Value: string);
protected
    function GetFont: TFont; override;
public
   procedure CreateControl; override;
   property  Memo: TMemo read GetMemo;
published
   property ClearIt: boolean write SetClearIt stored false;
   property NextText: string write SetNextText stored false;
   property LastText: string read GetLastText write SetLastText stored false;
   property Scrollbars:TScrollStyle read GetScrollbars write SetScrollbars;
   property WordWrap: boolean read GetWordWrap write SetWordWrap;
   property LoadFromFile: string write SetLoadFromFile stored false;
   property SaveToFile: string write SetSaveToFile stored false;
end;


TOrbis_Gauge = class(TControlTool)
  private
    function GetGauge: TGauge;
    function GetBackColor: TColor;
    function GetForeColor: TColor;
    function GetMaxValue: integer;
    function GetMinValue: integer;
    function GetProgress: integer;
    procedure SetBackColor(const Value: TColor);
    procedure SetForeColor(const Value: TColor);
    procedure SetMaxValue(const Value: integer);
    procedure SetMinValue(const Value: integer);
    procedure SetProgress(const Value: integer);
protected
    function GetFont: TFont; override;
public
   procedure CreateControl; override;
   property  Gauge: TGauge read GetGauge;
published
   property BackColor: TColor read GetBackColor write SetBackColor;  // clWhite
   property ForeColor: TColor read GetForeColor write SetForeColor; // clSKyBlue
   property MaxValue: integer read GetMaxValue write SetMaxValue;
   property MinValue: integer read GetMinValue write SetMinValue;
   property Progress: integer read GetProgress write SetProgress;
end;

implementation
uses
   e_Class,
   m_Proj;

{ TOrbis_Button }

procedure TOrbis_Button.CreateControl;
begin
   Control := TBitBtn.Create(Application);
   Button.OnClick := OnClick;
end;


function TOrbis_Button.GetButton: TBitBtn;
begin
   result := nil;
   if Control <> nil then
      result := TBitBtn(Control);
end;

function TOrbis_Button.GetCaption: string;
begin
   result := '';
   if Button <> nil then
      result := Button.Caption;
end;


function TOrbis_Button.GetFont: TFont;
begin
   result := nil;
   if Button <> nil then
      result := Button.Font;
end;

function TOrbis_Button.GetModalResult: integer;
begin
   if Button <> nil then
      result := Button.ModalResult;
end;

procedure TOrbis_Button.OnClick(Sender: TObject);
begin
   if OnClickScript <> '' then
 { TODO -oPPK -cOpravit : sputn scriptu je pomoc tm_Run, kter je v m_Proje, to je na hlavu }
      PostTextMsg(tm_Run,OnClickScript,nil);
end;

procedure TOrbis_Button.SetCaption(const Value: string);
begin
   if Button <> nil then
      Button.Caption := Value;
end;

procedure TOrbis_Button.SetModalResult(const Value: integer);
begin
   if Button <> nil then
      Button.ModalResult := Value;
end;

procedure TOrbis_Button.SetOnClickScript(const Value: string);
begin
  FOnClickScript := Value;
end;

{ TOrbis_Label }

procedure TOrbis_Label.CreateControl;
begin
   Control := TLabel.Create(Application);
end;

function TOrbis_Label.GetAutoSize: boolean;
begin
   result := false;
   if Label_ <> nil then
      result := Label_.AutoSize;
end;

function TOrbis_Label.GetCaption: string;
begin
   result := '';
   if Label_ <> nil then
      result := Label_.Caption;
end;

function TOrbis_Label.GetFont: TFont;
begin
   result := nil;
   if Label_ <> nil then
      result := Label_.Font;
end;

function TOrbis_Label.GetLabel: TLabel;
begin
   result := nil;
   if Control <> nil then
      result := TLabel(Control);
end;

procedure TOrbis_Label.SetAutoSize(const Value: boolean);
begin
   if Label_ <> nil then
      Label_.AutoSize := Value;
end;

procedure TOrbis_Label.SetCaption(const Value: string);
begin
   if Label_ <> nil then
      Label_.Caption := Value;
end;

{ TOrbis_Panel }

procedure TOrbis_Panel.CreateControl;
begin
   Control := TPanel.Create(Application);
   Panel.OnClick := OnClick;
end;

function TOrbis_Panel.GetCaption: string;
begin
   result := '';
   if Panel <> nil then
      result := Panel.Caption;
end;

function TOrbis_Panel.GetColor: integer;
begin
   result := 0;
   if Panel <> nil then
      result := Panel.Color;
end;

function TOrbis_Panel.GetFont: TFont;
begin
   result := nil;
   if Panel <> nil then
      result := Panel.Font;
end;

function TOrbis_Panel.GetPanel: TPanel;
begin
   result := nil;
   if Control <> nil then
      result := TPanel(Control);
end;

procedure TOrbis_Panel.OnClick(Sender: TObject);
begin
   if OnClickScript <> '' then
      PostTextMsg(tm_Run,OnClickScript,nil);
end;

procedure TOrbis_Panel.SetCaption(const Value: string);
begin
   if Panel <> nil then
      Panel.Caption := Value;
end;

procedure TOrbis_Panel.SetColor(const Value: integer);
begin
   if Panel <> nil then
      Panel.Color := Value;
end;

procedure TOrbis_Panel.SetOnClickScript(const Value: string);
begin
   FOnClickScript := Value;
end;


{ TOrbis_Splitter }

procedure TOrbis_Splitter.CreateControl;
begin
   Control := TSplitter.Create(Application);

end;

function TOrbis_Splitter.GetSplitter: TSplitter;
begin
   result := nil;
   if Control <> nil then
      result := TSplitter(Control);
end;

{ TOrbis_ComboBox }

procedure TOrbis_ComboBox.CreateControl;
begin
   Control := TComboBox.Create(Application);
   ComboBox.OnChange := OnChange;
end;

function TOrbis_ComboBox.GetComboBox: TComboBox;
begin
   result := nil;
   if Control <> nil then
      result := TComboBox(Control);
end;

function TOrbis_ComboBox.GetDropDownCount: integer;
begin
   result := 0;
   if ComboBox = nil then exit;
   result := ComboBox.DropDownCount;
end;

function TOrbis_ComboBox.GetFont: TFont;
begin
   result := nil;
   if ComboBox <> nil then
      result := ComboBox.Font;
end;

function TOrbis_ComboBox.GetSelectedElement: TElement;
begin
   result := TElement(ComboBox.Items.Objects[ComboBox.ItemIndex]);
end;

function TOrbis_ComboBox.GetText: string;
begin
   result := '';
   if ComboBox = nil then exit;
   result := ComboBox.Text;
end;

procedure TOrbis_ComboBox.OnChange(Sender: TObject);
begin
   if OnChangeScript <> '' then
      PostTextMsg(tm_Run,OnChangeScript,nil);
end;

procedure TOrbis_ComboBox.SetDropDownCount(const Value: integer);
begin
   if ComboBox = nil then exit;
   ComboBox.DropDownCount := Value;
end;

procedure TOrbis_ComboBox.SetInput(aInput: TElement);
var
   En: TEntity;
   E: TElement;
   i: integer;
begin
   inherited;
   ComboBox.Items.Clear;
   if aInput = nil then exit;
   if not (aInput is TEntity) then exit;
   En := TEntity(aInput);
   if En.NoElements then exit;
   for i := 0 to En.LastIndex do
   begin
      E := En.Elements[i];
      ComboBox.AddItem(E.Name,E);
   end;
   ComboBox.ItemIndex := 0;
end;

procedure TOrbis_ComboBox.SetOnChangeScript(const Value: string);
begin
  FOnChangeScript := Value;
end;

procedure TOrbis_ComboBox.SetText(const Value: string);
begin
   if ComboBox = nil then exit;
   ComboBox.Text := Value;
end;

{ TOrbis_ListBox }

procedure TOrbis_ListBox.CreateControl;
begin
   Control := TListBox.Create(Application);
   ListBox.OnClick := OnClick;
end;

function TOrbis_ListBox.GetColor: integer;
begin
   result := Listbox.Color;
end;

function TOrbis_ListBox.GetExtendedSelect: boolean;
begin
   result := ListBox.ExtendedSelect;
end;

function TOrbis_ListBox.GetFont: TFont;
begin
   result := nil;
   if ListBox <> nil then
      result := ListBox.Font;
end;

function TOrbis_ListBox.GetListBox: TListBox;
begin
   result := nil;
   if Control <> nil then
      result := TListBox(Control);
end;



function TOrbis_ListBox.GetMultiSelect: boolean;
begin
   result := ListBox.MultiSelect;
end;

function TOrbis_ListBox.GetSelectedElement: TElement;
begin
   result := TElement(ListBox.Items.Objects[ListBox.ItemIndex]);
end;

function TOrbis_ListBox.GetSelectedElements: TEntity;
var
   i: integer;
begin
   if FSelectedElements = nil then
   begin
      FSelectedElements := TOwnerEntity.Create;
      FSelectedElements.Name := 'ListBox_Select';
      AddElement(FSelectedElements);
   end;
   result := FSelectedElements;
   result.FreeAll;
   for i := 0 to Listbox.Items.Count - 1 do
   begin
      if ListBox.Selected[i] then
         result.AddElement(TElement(ListBox.Items.Objects[i]));
   end;
end;

procedure TOrbis_ListBox.OnClick(Sender: TObject);
begin
   if OnClickScript <> '' then
      PostTextMsg(tm_Run,OnClickScript,nil);
end;

procedure TOrbis_ListBox.SetColor(const Value: integer);
begin
   Listbox.Color := Value;
end;

procedure TOrbis_ListBox.SetExtendedSelect(const Value: boolean);
begin
   ListBox.ExtendedSelect := Value;
end;

procedure TOrbis_ListBox.SetInput(aInput: TElement);
var
   En: TEntity;
   E: TElement;
   i: integer;
begin
   inherited;
   ListBox.Items.Clear;
   if aInput = nil then exit;
   if not (aInput is TEntity) then exit;
   En := TEntity(aInput);
   if En.NoElements then exit;
   for i := 0 to En.LastIndex do
   begin
      E := En.Elements[i];
      ListBox.AddItem(E.Name,E);
   end;
   ListBox.ItemIndex := 0;
end;

procedure TOrbis_ListBox.SetMultiSelect(const Value: boolean);
begin
   ListBox.MultiSelect := Value;
end;

procedure TOrbis_ListBox.SetOnClickScript(const Value: string);
begin
  FOnClickScript := Value;
end;

procedure TOrbis_ListBox.SetSelectedElement(const Value: TElement);
var
   i: integer;
begin
   if Value = nil then
   begin
      Listbox.ItemIndex := -1;
   end;
   for i := 0 to ListBox.Items.Count - 1 do
   begin
      if ListBox.Items.Objects[i]=Value then
      begin
         Listbox.ItemIndex := i;
         exit;
      end;
   end;
   ListBox.ItemIndex := -1;
end;

{ TOrbis_Edit }

procedure TOrbis_Edit.CreateControl;
begin
   Control := TEdit.Create(Application);
   Edit.OnChange := OnChange;
end;

function TOrbis_Edit.GetEdit: TEdit;
begin
   result := nil;
   if Control <> nil then
      result := TEdit(Control);
end;

function TOrbis_Edit.GetFont: TFont;
begin
   result := nil;
   if Edit <> nil then
      result := Edit.Font;
end;

function TOrbis_Edit.GetText: string;
begin
   result := Edit.Text;
end;

procedure TOrbis_Edit.OnChange(Sender: TObject);
begin
   if OnChangeScript <> '' then
      PostTextMsg(tm_Run,OnChangeScript,nil);
end;

procedure TOrbis_Edit.SetOnChangeScript(const Value: string);
begin
  FOnChangeScript := Value;
end;

procedure TOrbis_Edit.SetText(const Value: string);
begin
  Edit.Text := Value;
end;

{ TOrbis_CheckListBox }

procedure TOrbis_CheckListBox.CreateControl;
begin
   Control := TCheckListBox.Create(Application);
   ChListBox.OnClick := OnClick;
   ChListBox.OnClickCheck := OnClickCheck;
end;


function TOrbis_CheckListBox.GetCheckedElements: TEntity;
var
   i: integer;
begin
   if FCheckedElements = nil then
   begin
      FCheckedElements := TOwnerEntity.Create;
      FCheckedElements.Name := 'ListBox_Check';
      AddElement(FCheckedElements);
   end;
   result := FCheckedElements;
   result.FreeAll;
   for i := 0 to CHListbox.Items.Count - 1 do
   begin
      if CHListBox.Checked[i] then
         result.AddElement(TElement(ChListBox.Items.Objects[i]));
   end;
end;

function TOrbis_CheckListBox.GetCheckedString: string;
var
   En: TEntity;
   i: integer;
begin
   result := '';
   En := CheckedElements;
   if En.NoElements then exit;
   for i := 0 to En.LastIndex do
   begin
      if i > 0 then
         result := result +',';
      result := result + En.Elements[i].Name;
   end;
end;

function TOrbis_CheckListBox.GetCHListBox: TCheckListBox;
begin
   result := nil;
   if Control <> nil then
      result := TCheckListBox(Control);
end;

function TOrbis_CheckListBox.GetColor: integer;
begin
   result := ChListbox.Color;
end;

function TOrbis_CheckListBox.GetFont: TFont;
begin
   result := nil;
   if ChListbox <> nil then
      result := ChListbox.Font;
end;

function TOrbis_CheckListBox.GetMultiSelect: boolean;
begin
   result := ChListBox.MultiSelect;
end;

function TOrbis_CheckListBox.GetSelectedElement: TElement;
begin
   result := TElement(ChListBox.Items.Objects[ChListBox.ItemIndex]);
end;

function TOrbis_CheckListBox.GetSelectedElements: TEntity;
var
   i: integer;
begin
   if FSelectedElements = nil then
   begin
      FSelectedElements := TOwnerEntity.Create;
      FSelectedElements.Name := 'ListBox_Select';
      AddElement(FSelectedElements);
   end;
   result := FSelectedElements;
   result.FreeAll;
   for i := 0 to CHListbox.Items.Count - 1 do
   begin
      if CHListBox.Selected[i] then
         result.AddElement(TElement(ChListBox.Items.Objects[i]));
   end;
end;

function TOrbis_CheckListBox.GetSelectedIsChecked: boolean;
var
   i: integer;
begin
   i := ChListBox.ItemIndex;
   result := CHListBox.Checked[i];
end;

procedure TOrbis_CheckListBox.OnClick(Sender: TObject);
begin
   if OnClickScript <> '' then
      PostTextMsg(tm_Run,OnClickScript,nil);
end;


procedure TOrbis_CheckListBox.OnClickCheck(Sender: TObject);
begin
   if OnClickCheckScript <> '' then
      PostTextMsg(tm_Run,OnClickCheckScript,nil);
end;

procedure TOrbis_CheckListBox.SetCheckAll(const Value: boolean);
var
   i: integer;
begin
   for i := 0 to CHListbox.Items.Count - 1 do
   begin
      CHListBox.Checked[i] := Value;
   end;
end;

procedure TOrbis_CheckListBox.SetCheckByProperty(const Value: boolean);
begin
  FCheckByProperty := Value;
end;

procedure TOrbis_CheckListBox.SetCHeckByPropertyReverse(
  const Value: boolean);
begin
  FCHeckByPropertyReverse := Value;
end;

procedure TOrbis_CheckListBox.SetCheckedElements(const Value: TEntity);
var
   i: integer;
   En: TENtity;
   E: TElement;
begin
   EN := TEntity(Input);
   if EN = nil then exit;
   if EN.NoElements then exit;
   if Value = nil then exit;
   if Value.NoElements then exit;
   for i := 0 to CHListBox.Count-1 do
   begin
      e := TElement(CHListBox.Items.Objects[i]);
      ChListBox.Checked[i] := Value.FindElementName(e.Name) <> nil;
   end;
end;

procedure TOrbis_CheckListBox.SetCheckedString(const Value: string);
begin

end;

procedure TOrbis_CheckListBox.SetCheckPropertyName(const Value: string);
begin
  FCheckPropertyName := Value;
end;

procedure TOrbis_CheckListBox.SetColor(const Value: integer);
begin
   ChListBox.Color := Value;
end;

procedure TOrbis_CheckListBox.SetInput(aInput: TElement);
var
   En: TEntity;
   E: TElement;
   i: integer;
   B: boolean;
begin
   inherited;
   ChListBox.Items.Clear;
   if aInput = nil then exit;
   if not (aInput is TEntity) then exit;
   En := TEntity(aInput);
   if En.NoElements then exit;
   for i := 0 to En.LastIndex do
   begin
      E := En.Elements[i];
      ChListBox.AddItem(E.Name,E);
      if CheckByProperty then
      begin
         B := E.GetPropertyBooleanByName(CheckPropertyName);
         if CHeckByPropertyReverse then
            B := not B;
         ChListBox.Checked[i] := B;
      end;
   end;
   ChListBox.ItemIndex := -1;
end;

procedure TOrbis_CheckListBox.SetMultiSelect(const Value: boolean);
begin
   ChListBox.MultiSelect := Value;
end;

procedure TOrbis_CheckListBox.SetOnClickCheckScript(const Value: string);
begin
  FOnClickCheckScript := Value;
end;

procedure TOrbis_CheckListBox.SetOnClickScript(const Value: string);
begin
  FOnClickScript := Value;
end;

procedure TOrbis_CheckListBox.SetSelectedElement(const Value: TElement);
var
   i: integer;
begin
   if Value = nil then
   begin
      ChListbox.ItemIndex := -1;
   end;
   for i := 0 to ChListBox.Items.Count - 1 do
   begin
      if ChListBox.Items.Objects[i]=Value then
      begin
         ChListbox.ItemIndex := i;
         exit;
      end;
   end;
   ChListBox.ItemIndex := -1;
end;

procedure TOrbis_CheckListBox.SetSelectedIsChecked(const Value: boolean);
var
   i: integer;
begin
   i := CHListBox.ItemIndex;
   ChListBox.Checked[i] := Value;
end;

{ TOrbis_StringGrid }

procedure TOrbis_StringGrid.CreateControl;
begin
   Control := TStringGrid.Create(Application);
   StringGrid.ColCount := 2;
   StringGrid.RowCount := 2;
   StringGrid.FixedCols := 0;
   StringGrid.FixedRows := 0;
end;

function TOrbis_StringGrid.GetFont: TFont;
begin
   result := nil;
   if StringGrid <> nil then
      result := StringGrid.Font;
end;

function TOrbis_StringGrid.GetStringGrid: TStringGrid;
begin
   result := nil;
   if Control <> nil then
      result := TStringGrid(Control);
end;

function TOrbis_StringGrid.GetAValue: string;
begin
   result := StringGrid.Cells[aCol,aRow];
end;

procedure TOrbis_StringGrid.SetACol(const Value: integer);
begin
  FACol := Value;
end;

procedure TOrbis_StringGrid.SetARow(const Value: integer);
begin
  FARow := Value;
end;

procedure TOrbis_StringGrid.SetAValue(const Value: string);
begin
  StringGrid.Cells[aCol,aRow] := Value;
end;

{ TOrbis_Memo }

procedure TOrbis_Memo.CreateControl;
begin
   Control := TMemo.Create(Application);
   //Memo.Font.Name := 'Courier New';
   //Memo.Font.Size := 10;
end;

function TOrbis_Memo.GetFont: TFont;
begin
   result := nil;
   if Memo <> nil then
      result := Memo.Font;
end;

function TOrbis_Memo.GetLastText: string;
begin
   result := '';
   if Memo <> nil then
      if Memo.Lines.Count > 0 then
         result := Memo.Lines.Strings[Memo.Lines.Count-1];
end;

function TOrbis_Memo.GetMemo: TMemo;
begin
   result := nil;
   if Control <> nil then
      result := TMemo(Control);
end;

function TOrbis_Memo.GetScrollbars: TScrollStyle;
begin
   result := Memo.ScrollBars;
end;

function TOrbis_Memo.GetWordWrap: boolean;
begin
   result := false;
   if Memo <> nil then
      result := Memo.WordWrap;
end;

procedure TOrbis_Memo.SetClearIt(const Value: boolean);
begin
   Memo.Lines.Clear;
end;

procedure TOrbis_Memo.SetLastText(const Value: string);
begin
   if Memo = nil then exit;
   if Memo.Lines.Count = 0 then
      NextText := Value
   else
      Memo.Lines.Strings[Memo.Lines.Count-1] := Value;
end;

procedure TOrbis_Memo.SetLoadFromFile(const Value: string);
begin
   if Memo = nil then exit;
   Memo.Lines.LoadFromFile(Value);
end;

procedure TOrbis_Memo.SetNextText(const Value: string);
begin
   Memo.Lines.Add(Value);
end;

procedure TOrbis_Memo.SetSaveToFile(const Value: string);
begin
   if Memo = nil then exit;
   Memo.Lines.SaveToFile(Value);
end;

procedure TOrbis_Memo.SetScrollbars(const Value: TScrollStyle);
begin
   Memo.ScrollBars := Value;
end;

procedure TOrbis_Memo.SetWordWrap(const Value: boolean);
begin
   if Memo <> nil then
      Memo.WordWrap := Value;
end;

{ TOrbis_CheckBox }

procedure TOrbis_CheckBox.CreateControl;
begin
   Control := TCheckBox.Create(Application);
   CheckBox.OnClick := OnClick;
end;

function TOrbis_CheckBox.GetAllignment: TLeftRight;
begin
   result := taRightJustify;
   if CheckBox <> nil then
      result := CheckBox.Alignment;
end;

function TOrbis_CheckBox.GetCaption: string;
begin
   result := '';
   if CheckBox <> nil then
      result := CheckBox.Caption;
end;


function TOrbis_CheckBox.GetCheckBox: TCheckBox;
begin
   result := TCheckBox(Control);
end;

function TOrbis_CheckBox.GetChecked: boolean;
begin
   result := false;
   if CheckBox <> nil then
      result := CheckBox.Checked;
end;

function TOrbis_CheckBox.GetFont: TFont;
begin
   result := nil;
   if CheckBox <> nil then
      result := CheckBox.Font;
end;

procedure TOrbis_CheckBox.OnClick(Sender: TObject);
begin
   if OnClickScript <> '' then
      PostTextMsg(tm_Run,OnClickScript,nil);
end;

procedure TOrbis_CheckBox.SetAllignment(const Value: TLeftRight);
begin
   if CheckBox <> nil then
      CheckBox.Alignment := Value;
end;

procedure TOrbis_CheckBox.SetCaption(const Value: string);
begin
   if CheckBox <> nil then
      CheckBox.Caption := Value;
end;

procedure TOrbis_CheckBox.SetChecked(const Value: boolean);
begin
   if CheckBox <> nil then
      CheckBox.Checked := Value;
end;

procedure TOrbis_CheckBox.SetOnClickScript(const Value: string);
begin
  FOnClickScript := Value;
end;

{ TOrbis_RadioButton }

procedure TOrbis_RadioButton.CreateControl;
begin
   Control := TRadioButton.Create(Application);
end;

function TOrbis_RadioButton.GetAllignment: TLeftRight;
begin
   result := taRightJustify;
   if RadioButton <> nil then
      result := RadioButton.Alignment;
end;

function TOrbis_RadioButton.GetCaption: string;
begin
   result := '';
   if RadioButton <> nil then
      result := RadioButton.Caption;
end;

function TOrbis_RadioButton.GetChecked: boolean;
begin
   result := false;
   if RadioButton <> nil then
      result := RadioButton.Checked;
end;

function TOrbis_RadioButton.GetFont: TFont;
begin
   result := nil;
   if RadioButton <> nil then
      result := RadioButton.Font;
end;

function TOrbis_RadioButton.GetRadioButton: TRadioButton;
begin
   result := TRadioButton(Control);
end;

procedure TOrbis_RadioButton.SetAllignment(const Value: TLeftRight);
begin
   if RadioButton <> nil then
      RadioButton.Alignment := Value;
end;

procedure TOrbis_RadioButton.SetCaption(const Value: string);
begin
   if RadioButton <> nil then
      RadioButton.Caption := Value;
end;

procedure TOrbis_RadioButton.SetChecked(const Value: boolean);
begin
   if RadioButton <> nil then
      RadioButton.Checked := Value;
end;

{ TOrbis_RadioGroup }

procedure TOrbis_RadioGroup.CreateControl;
begin
   Control := TRadioGroup.Create(Application);
   RadioGroup.OnClick := OnClick;
end;

function TOrbis_RadioGroup.GetCaption: string;
begin
   result := '';
   if RadioGroup <> nil then
      result := RadioGroup.Caption;
end;

function TOrbis_RadioGroup.GetColumns: integer;
begin
   result := 0;
   if RadioGroup <> nil then
      result := RadioGroup.Columns;
end;

function TOrbis_RadioGroup.GetFont: TFont;
begin
   result := nil;
   if RadioGroup <> nil then
      result := RadioGroup.Font;
end;

function TOrbis_RadioGroup.GetItemIndex: integer;
begin
   result := -1;
   if RadioGroup <> nil then
      result := RadioGroup.ItemIndex;
end;

function TOrbis_RadioGroup.GetRadioGroup: TRadioGroup;
begin
   result := TRadioGroup(Control);
end;

procedure TOrbis_RadioGroup.OnClick(Sender: TObject);
begin
   if OnClickScript <> '' then
      PostTextMsg(tm_Run,OnClickScript,nil);
end;

procedure TOrbis_RadioGroup.SetCaption(const Value: string);
begin
   if RadioGroup <> nil then
      RadioGroup.Caption := Value;
end;

procedure TOrbis_RadioGroup.SetColumns(const Value: integer);
begin
   if RadioGroup <> nil then
      RadioGroup.Columns := Value;
end;

procedure TOrbis_RadioGroup.SetItemIndex(const Value: integer);
begin
   if RadioGroup <> nil then
      RadioGroup.ItemIndex := Value;
end;

procedure TOrbis_RadioGroup.SetNextItem(const Value: string);
begin
   if RadioGroup = nil then exit;
   RadioGroup.Items.Add(Value);
end;

procedure TOrbis_RadioGroup.SetOnClickScript(const Value: string);
begin
  FOnClickScript := Value;
end;

{ TOrbis_Gauge }

procedure TOrbis_Gauge.CreateControl;
begin
   Control := TGauge.Create(Application);
end;

function TOrbis_Gauge.GetBackColor: TColor;
begin
   result := 0;
   if Gauge <> nil then
      result := Gauge.BackColor;
end;

function TOrbis_Gauge.GetFont: TFont;
begin
   result := nil;
   if Gauge <> nil then
      result := Gauge.Font;
end;

function TOrbis_Gauge.GetForeColor: TColor;
begin
   result := 0;
   if Gauge <> nil then
      result := Gauge.ForeColor;
end;

function TOrbis_Gauge.GetGauge: TGauge;
begin
   result := TGauge(Control);
end;

function TOrbis_Gauge.GetMaxValue: integer;
begin
   result := 0;
   if Gauge <> nil then
      result := Gauge.MaxValue;
end;

function TOrbis_Gauge.GetMinValue: integer;
begin
   result := 0;
   if Gauge <> nil then
      result := Gauge.MinValue;
end;

function TOrbis_Gauge.GetProgress: integer;
begin
   result := 0;
   if Gauge <> nil then
      result := Gauge.Progress;
end;

procedure TOrbis_Gauge.SetBackColor(const Value: TColor);
begin
   if Gauge <> nil then
      Gauge.BackColor := Value;
end;

procedure TOrbis_Gauge.SetForeColor(const Value: TColor);
begin
   if Gauge <> nil then
      Gauge.ForeColor := Value;
end;

procedure TOrbis_Gauge.SetMaxValue(const Value: integer);
begin
   if Gauge <> nil then
      Gauge.MaxValue := Value;
end;

procedure TOrbis_Gauge.SetMinValue(const Value: integer);
begin
   if Gauge <> nil then
      Gauge.MinValue := Value;
end;

procedure TOrbis_Gauge.SetProgress(const Value: integer);
begin
   if Gauge <> nil then
      Gauge.Progress := Value;
end;

{ TOrbis_ScrollBox }

procedure TOrbis_ScrollBox.CreateControl;
begin
   Control := TScrollBox.Create(Application);
   ScrollBox.AutoScroll := true;
end;

function TOrbis_ScrollBox.GetAutoScroll: boolean;
begin
   result := false;
   if ScrollBox <> nil then
      result := ScrollBox.AutoScroll;
end;

function TOrbis_ScrollBox.GetColor: integer;
begin
   result := 0;
   if ScrollBox <> nil then
      result := ScrollBox.Color;
end;

function TOrbis_ScrollBox.GetFont: TFont;
begin
   result := nil;
   if ScrollBox <> nil then
      result := ScrollBox.Font;
end;

function TOrbis_ScrollBox.GetHoriz: boolean;
begin
   result := false;
   if ScrollBox <> nil then
      result := ScrollBox.HorzScrollBar.Visible;
end;

function TOrbis_ScrollBox.GetScrollBox: TScrollBox;
begin
   result := nil;
   if Control <> nil then
      result := TScrollBox(Control);
end;


function TOrbis_ScrollBox.GetVert: boolean;
begin
   result := false;
   if ScrollBox <> nil then
      result := ScrollBox.VertScrollBar.Visible;
end;

procedure TOrbis_ScrollBox.SetAutoScroll(const Value: boolean);
begin
   if ScrollBox <> nil then
      ScrollBox.AutoScroll := Value;

end;

procedure TOrbis_ScrollBox.SetColor(const Value: integer);
begin
   if ScrollBox <> nil then
      ScrollBox.Color := Value;
end;

procedure TOrbis_ScrollBox.Sethoriz(const Value: boolean);
begin
   if ScrollBox <> nil then
      ScrollBox.HorzScrollBar.Visible := Value;
end;

procedure TOrbis_ScrollBox.SetVert(const Value: boolean);
begin
   if ScrollBox <> nil then
      ScrollBox.VertScrollBar.Visible := Value;
end;

INITIALIZATION
   RegisterOrbisClass(TOrbis_Button);
   RegisterOrbisClass(TOrbis_Label);
   RegisterOrbisClass(TOrbis_Panel);
   RegisterOrbisClass(TOrbis_ScrollBox);
   RegisterOrbisClass(TOrbis_Splitter);
   RegisterOrbisClass(TOrbis_ComboBox);
   RegisterOrbisClass(TOrbis_ListBox);
   RegisterOrbisClass(TOrbis_CheckListBox);
   RegisterOrbisClass(TOrbis_CheckBox);
   RegisterOrbisClass(TOrbis_RadioButton);
   RegisterOrbisClass(TOrbis_RadioGroup);
   RegisterOrbisClass(TOrbis_Edit);
   RegisterOrbisClass(TOrbis_StringGrid);
   RegisterOrbisClass(TOrbis_Memo);
   RegisterOrbisClass(TOrbis_Gauge);
END.
