unit t_ConditionFilter;

interface
uses
   f_ConditionFilter,
   Elements,
   Tools;

type

TConditionFilterDialog =  class(TTool)
  private
    FLayerDescription: string;
    FFilterProperties: string;
    FFileName: string;
    FModel: TOwnerEntity;
    FForm: TConditionFilterForm;
    FData: TElement;
    FCondition: string;
    FDataDirectory: string;
    FDataIndex: integer;
    procedure SetFileName(const Value: string);
    procedure SetFilterProperties(const Value: string);
    procedure SetLayerDescription(const Value: string);
    procedure SetModel(const Value: TOwnerEntity);
    procedure SetForm(const Value: TConditionFilterForm);
    procedure SetData(const Value: TElement);
    procedure SetCondition(const Value: string);
    procedure SetDataDirectory(const Value: string);
    procedure SetDataIndex(const Value: integer);
public
   procedure DoIt; override;
   procedure PrepareModel;
   procedure AcceptModel;
   procedure SaveModel;
   procedure CreateModelByProperties;
   procedure FillProperties;
   function  GetCondition: string;
   function  GetDefaultFileName: string;
   procedure CreateData;
   procedure InsertDataToOutput;
   property Form: TConditionFilterForm read FForm write SetForm;
   property Model: TOwnerEntity read FModel write SetModel;
published
   property Data: TElement read FData write SetData;
   property DataIndex: integer read FDataIndex write SetDataIndex;
   property DataDirectory: string read FDataDirectory write SetDataDirectory;
   property LayerDescription: string read FLayerDescription write SetLayerDescription;
   property Condition: string read FCondition write SetCondition;
   property FilterProperties: string read FFilterProperties write SetFilterProperties;
   property FileName: string read FFileName write SetFileName;
end;

implementation
uses
   Controls,
   Classes,
   SysUtils,
   Forms,
   e_Class,
   e_Type,
   t_Loader,
   t_Saver,
   t_Layscr,
   t_LMCre,
   u_String;

const
   CFDFile = 'ConditionFilter.orb';
   CFSizeElement = '@_SIZE';
   CFWidth = '_Width';
{ TConditionFilterDialog }

procedure TConditionFilterDialog.AcceptModel;
var
   SLE: TStringListElement;
   SLESize: TStringListElement;
   S: TStrings;
   i,j: integer;
   WrongRow: boolean;
begin
   CreateModelByProperties;
   if Model.NoElements then exit;
   for i := 1 to Form.SG.RowCount - 1 do
   begin
      WrongRow := false;
      for j := 0 to Form.SG.ColCount -1 do
      begin
         if Form.SG.Cells[j,i] = '' then WrongRow := true;
      end;
      if not WrongRow then
      begin
         for j := 0 to Form.SG.ColCount -1 do
         begin
            TStringListElement(Model.Elements[j]).AddString(Form.SG.Cells[j,i]);
         end;
      end;
   end;
   SLESize := TStringListElement(Model.FindElementName(CFSizeElement));
   if SLESize <> nil then
   begin
      SLESize.SetIntegerByName('LEFT', Form.Left);
      SLESize.SetIntegerByName('TOP', Form.Top);
      SLESize.SetIntegerByName('WIDTH', Form.Width);
      SLESize.SetIntegerByName('HEIGHT', Form.Height);
      for j := 0 to Form.SG.ColCount -1 do
      begin
         SLESize.SetIntegerByName(Form.SG.Cells[j,0]+CFWidth, Form.SG.ColWidths[j]);
      end;
   end;
   Form.SG.RowCount := 1;
   FillProperties;
   Condition := GetCondition;
end;

procedure TConditionFilterDialog.CreateData;
var
   MSC:  TModelScriptCreator;
   MBSC: TModelByScriptCreator;
begin
   MSC := TModelScriptCreator.Create;
   MBSC := TModelByScriptCreator.Create;
   MSC.DataDirectory := DataDirectory;
   MSC.NextLayer := LayerDescription;
   MSC.NextLayerProperty := Condition;
   MSC.DoIt;
   MBSC.Input := MSC.Output;
   MBSC.Compatibility30 := true;
   MBSC.DoIt;
   Data := MBSC.Output;
   //Output := Data;
   MSC.Free;
   MBSC.Free;
end;

procedure TConditionFilterDialog.CreateModelByProperties;
var
  S,S1: string;
  SLE: TStringListElement;
begin
   S := FilterProperties;
   Model := TOwnerEntity.Create;
   Model.Name := S;
   if S <> '' then
   begin
      while StringAsTwoParams(S,',',S,S1)do
      begin
         SLE := TStringListElement.Create;
         SLE.Name := S;
         Model.AddElement(SLE);
         S := S1;
      end;
      SLE := TStringListElement.Create;
      SLE.Name := S;
      Model.AddElement(SLE);
   end;
   SLE := TStringListElement.Create;
   SLE.Name := CFSizeElement;
   Model.AddElement(SLE);
end;




procedure TConditionFilterDialog.DoIt;
begin
   PrepareModel;
   Form := TConditionFilterForm.Create(Application);
   Form.Tool := self;
   FillProperties;
   if Form.ShowModal = mrOK then
   begin
      AcceptModel;
      SaveModel;
      CreateData;
      InsertDataToOutput;
   end;
   Form.Free;
   Form := nil;
end;


procedure TConditionFilterDialog.FillProperties;
var
   i,j,k: integer;
   SLE: TStringListElement;
   SLESize: TStringListElement;
begin
   if Form = nil then exit;
   if Model = nil then exit;
   if Model.NoElements then exit;
   SLE := TStringListElement(Model.Elements[0]);
   SLESize := TStringListElement(Model.FindElementName(CFSizeElement));
   if SLESize <> nil then
   begin
      i := SLESize.GetIntegerByName('LEFT');
      if i > 0 then Form.Left := i;
      i := SLESize.GetIntegerByName('TOP');
      if i > 0 then Form.Top := i;
      i := SLESize.GetIntegerByName('WIDTH');
      if i > 0 then Form.Width := i;
      i := SLESize.GetIntegerByName('HEIGHT');
      if i > 0 then Form.Height := i;
   end;
   Form.SG.ColCount := Model.Count - 1;
   if not SLE.NoElements then Form.SG.RowCount := SLE.Count+1;
   for i := 0 to Model.LastIndex do
   begin
      SLE := TStringListElement(Model.Elements[i]);
      if SLE = SLESize then Continue;
      Form.SG.Cells[i,0] := SLE.Name;
      if SLE.NoElements then Continue;
      for j := 0 to SLE.LastIndex do
      begin
         Form.SG.Cells[i,j+1] := SLE.Strings[j];
      end;
      if SLESize <> nil then
      begin
         k := SLESize.GetIntegerByName(SLE.Name+CFWidth);
         if k > 0 then Form.SG.ColWidths[i] := k;
      end;
   end;
end;

function TConditionFilterDialog.GetCondition: string;
var
   i,j: integer;
   S: string;
   V,Code: integer;
begin
   result := '@CONDITION=';
   for i := 1 to Form.SG.RowCount - 1 do
   begin
      if i > 1 then result := result + 'OR('
               else result := result + '(';
      for j := 0 to Form.SG.ColCount - 1 do
      begin
         if j > 0 then result := result + 'AND('
                  else result := result + '(';
         result := result + Form.SG.Cells[j,0] + '=';
         S := Form.SG.Cells[j,i];
         val(S,V,Code);
         if Code = 0 then result := result + Form.SG.Cells[j,i]
                     else result := result + ''''+Form.SG.Cells[j,i]+'''';
         result := result + ')';
      end;
      result := result + ')';
   end;
end;

function TConditionFilterDialog.GetDefaultFileName: string;
begin
   result := ExtractFilePath(ParamStr(0));
   CheckBackSlash(result);
   result := result + CFDFile;
end;

procedure TConditionFilterDialog.InsertDataToOutput;
var
   E: TEntity;
begin
   if Input = nil then exit;
   if not(Input is TEntity) then exit;
   E := TEntity(Input);
   if Data = nil then exit;
   if E.NoElements or (DataIndex > E.LastIndex) then E.AddElement(Data)
                                                else E.InsertElement(DataIndex, Data);
end;

procedure TConditionFilterDialog.PrepareModel;
var
   S: string;
   Load: TLoad;
   M: TOwnerEntity;
   E: TElement;
begin
   M := nil;
   S := GetDefaultFileName;
   if FileExists(S) then
   begin
      Load := TLoad.Create;
      Load.FileName := S;
      try
         Load.DoIt;
         M := TOwnerEntity(Load.Output);
      except
      end;
      Load.Free;
   end;
   if M <> nil then
   begin
      if CompareText(M.Name,FilterProperties) = 0 then Model := M
      else
      begin
         M.Free;
         M := nil;
         CreateModelByProperties;
      end;
   end
   else
   begin
      CreateModelByProperties;
   end;
end;

procedure TConditionFilterDialog.SaveModel;
var
   S: string;
   Save: TSave;
begin
   Save := TSave.Create;
   Save.FileName := GetDefaultFileName;
   Save.Input := Model;
   Save.DoIt;
   Save.Free;
end;

procedure TConditionFilterDialog.SetCondition(const Value: string);
begin
  FCondition := Value;
end;

procedure TConditionFilterDialog.SetData(const Value: TElement);
begin
   if FData <> nil then
   begin
      FreeElement(FData);
      FData := nil;
   end;
   FData := Value;
   AddElement(FData);
end;

procedure TConditionFilterDialog.SetDataDirectory(const Value: string);
begin
  FDataDirectory := Value;
end;

procedure TConditionFilterDialog.SetDataIndex(const Value: integer);
begin
  FDataIndex := Value;
end;

procedure TConditionFilterDialog.SetFileName(const Value: string);
begin
  FFileName := Value;
end;

procedure TConditionFilterDialog.SetFilterProperties(const Value: string);
begin
  FFilterProperties := Value;
end;

procedure TConditionFilterDialog.SetForm(
  const Value: TConditionFilterForm);
begin
  FForm := Value;
end;

procedure TConditionFilterDialog.SetLayerDescription(const Value: string);
begin
  FLayerDescription := Value;
end;

procedure TConditionFilterDialog.SetModel(const Value: TOwnerEntity);
begin
   if FModel <> nil then
   begin
      FreeElement(FModel);
      FModel := nil;
   end;
  FModel := Value;
  AddElement(FModel);
end;

INITIALIZATION
   RegisterOrbisClass(TConditionFilterDialog);
END.
