unit E_proje;

{
[Unit_Description]
Description=Projekce
Admin=Petr
TechTopic=Elements
Topic=Zobrazen dat
Version=1.0

[Edit]
 8.8.1997 19:59=Registrace
17.9.1997 21:30=Zaveden ClipModelRect
}

interface

uses
  Classes,
  WinTypes,
  // KERNEL
  Elements,
  // GLIB
  G_Elems,
  // DEVICE
  e_Page;

type



TProjection = class(TOwnerElement)
private
   FAngle: double;
   FRotateCenter_X: double;
   FRotateCenter_Y: double;
   procedure SetAngle(const Value: double);
   procedure SetRotateCenter_X(const Value: double);
   procedure SetRotateCenter_Y(const Value: double);
    function GetModelMaxX: double;
    function GetModelMaxY: double;
    function GetModelMinX: double;
    function GetModelMinY: double;
protected
   FCenterPoint: TDoubleCoord;
   FScale: double;
   FVirtualScale: double;
   MultX, MultY, UniMult, AddX, AddY: double;
   function  GetModelView: TDoubleRect; virtual; abstract;
   function  GetClipModelView: TDoubleRect; virtual; abstract;
   function  GetVirtualScale: double;
   procedure SetCenterPoint(const aCenterPoint: TDoubleCoord); virtual; abstract;
   procedure SetCenterX(const aValue: double); virtual; abstract;
   procedure SetCenterY(const aValue: double); virtual; abstract;
   procedure SetScale(const aScale: double); virtual; abstract;
   procedure SetModelView(const aRect: TDoubleRect); virtual; abstract;
   procedure SetClipModelView(const aRect: TDoubleRect); virtual; abstract;
   function  GetIsOK: boolean;
public
   procedure Load(aReader: TReader; aIndex: TEntity); override;
   procedure Save(aWriter: TWriter); override;
   procedure Assign(Source: TPersistent); override;
   property  CenterPoint: TDoubleCoord read FCenterPoint write SetCenterPoint;
   property  CLipModelView: TDoubleRect read GetClipModelView write SetCLipModelView;
published
   property  ModelView: TDoubleRect read GetModelView write SetModelView;
   property  Scale: double read FScale write SetScale;
   property  Angle: double read FAngle write SetAngle;
   property  RotateCenter_X: double read FRotateCenter_X write SetRotateCenter_X;
   property  RotateCenter_Y: double read FRotateCenter_Y write SetRotateCenter_Y;
   property  VirtualScale: double read GetVirtualScale write FVirtualScale;
   property  Center_X: double read FCenterPoint.X write SetCenterX;
   property  Center_Y: double read FCenterPoint.Y write SetCenterY;
   property  IsOK: boolean read GetIsOK;
   property  ModelMinX: double read GetModelMinX stored false;
   property  ModelMinY: double read GetModelMinY stored false;
   property  ModelMaxX: double read GetModelMaxX stored false;
   property  ModelMaxY: double read GetModelMaxY stored false;
end;



TPageProjection = class(TProjection)
protected
   FPage: T_Page;
   function  GetModelView: TDoubleRect; override;
   function  GetCLipModelView: TDoubleRect; override;
   procedure SetCenterPoint(const aCenterPoint: TDoubleCoord); override;
   procedure SetCenterX(const aValue: double); override;
   procedure SetCenterY(const aValue: double); override;
   procedure SetScale(const aScale: double); override;
   procedure SetPage(const aPage: T_Page);
   procedure SetModelView(const aRect: TDoubleRect); override;
   procedure SetClipModelView(const aRect: TDoubleRect); override;
public
   procedure Load(aReader: TReader; aIndex: TEntity); override;
   procedure Save(aWriter: TWriter); override;
   procedure ChangeProjection (const aCenterPoint: TDoubleCoord; const aScale: double;
                               const aPage: T_Page);
   procedure ChangeProjectionSimple (const aPage: T_Page);
   procedure ChangeProjectionByRect (const aModelRect: TDoubleRect; const aPage: T_Page);
   procedure ChangeProjectionByRectangle (const aModelRect: TRectangle; const aPage: T_Page);
   function  GetPageX(const ModelX: double): longInt;
   function  GetPageY(const ModelY: double): longInt;
   function  GetModelX(PageX: integer): double;
   function  GetModelY(PageY: integer): double;
   function  GetModelArea(aWidth, aHeight: integer): double;

   function  GetPageLength(const ModelLength: double): longInt;
   function  GetPageLengthHoriz1(const ModelLength: double): longInt;
   function  GetPageLengthVert1(const ModelLength: double): longInt;

   function  GetPagePoint(const ModelX, ModelY: double): {WinTypes.}TPoint;
   function  GetPageRect(const MinX, MinY, MaxX, MaxY: double): {WinTypes.}TRect;

   function  GetModelLength(const PageLength: longInt): double;
   function  GetModelLengthHoriz(const PageLength: longInt): double;
   function  GetModelLengthVert(const PageLength: longInt): double;

   function  GetModelPoint(PageX, PageY: integer): TDoubleCoord;
   function  GetModelRect(Left, Top, Right, Bottom: integer): TDoubleRect;
   function  GetModelPointElement(PageX, PageY: integer): TCoordinate;
   property  Page: T_Page read FPage write SetPage;
end;


implementation

uses
   {!t_Main,}
   // KERNEL
   e_Class;


{ TProjection }

procedure  TProjection.Load(aReader: TReader; aIndex: TEntity);
begin
   inherited Load(aReader, aIndex);
   aReader.Read(FCenterPoint, SizeOf(double)*8);
end;


procedure  TProjection.Save(aWriter: TWriter);
begin
   inherited Save(aWriter);
   aWriter.Write(FCenterPoint, SizeOf(double)*8);
end;


procedure TProjection.Assign(Source: TPersistent);
var
   aProje: TProjection;
begin
   if not (Source is TProjection) then exit;
   aProje := TProjection(Source);
   CenterPoint := aProje.CenterPoint;
   Scale       := aProje.Scale;
   VirtualScale:= aProje.VirtualScale;
   UniMult     := aProje.UniMult;
   MultX       := aProje.MultX;
   MultY       := aProje.MultY;
   AddX        := aProje.AddX;
   AddY        := aProje.AddY;
end;


function  TProjection.GetVirtualScale: double;
begin
  if FVirtualScale = 0 then
     FVirtualScale:= 5000;
   result := FVirtualScale
end;


function  TProjection.GetIsOK: boolean;
begin
  result:= ((Center_X <> 0) or (Center_Y <> 0)) and
           (Scale > 0) and (Scale < 1E30)
end;




procedure TProjection.SetAngle(const Value: double);
begin
   FAngle := Value;
   if (RotateCenter_x = 0) and (RotateCenter_y = 0) then
   begin
      RotateCenter_x := Center_x;
      RotateCenter_y := Center_y;
   end;
end;

procedure TProjection.SetRotateCenter_X(const Value: double);
begin
   FRotateCenter_X := Value;
end;

procedure TProjection.SetRotateCenter_Y(const Value: double);
begin
   FRotateCenter_Y := Value;
end;

function TProjection.GetModelMaxX: double;
begin
   result := ModelView.MaxX;
end;

function TProjection.GetModelMaxY: double;
begin
   result := ModelView.MaxY;
end;

function TProjection.GetModelMinX: double;
begin
   result := ModelView.MinX;
end;

function TProjection.GetModelMinY: double;
begin
   result := ModelView.MinY;
end;

{ TPageProjection }

procedure  TPageProjection.Load(aReader: TReader; aIndex: TEntity);
var
   i: longInt;
begin
   inherited Load(aReader, aIndex);
   i := aReader.ReadInteger;
   if i > -1 then Page := T_Page(aIndex.Elements[i])
             else Page := nil;
end;


procedure  TPageProjection.Save(aWriter: TWriter);
begin
   inherited Save(aWriter);
   if Page <> nil then aWriter.Writeinteger(Page.ID)
                  else aWriter.Writeinteger(-1);
end;


function  TPageProjection.GetModelView: TDoubleRect;
begin
   result.Minx := GetModelX(Page.Rect.Left);
   result.Miny := GetModely(Page.Rect.Bottom);
   result.Maxx := GetModelX(Page.Rect.Right);
   result.Maxy := GetModely(Page.Rect.Top);
end;

function  TPageProjection.GetClipModelView: TDoubleRect;
begin
   if Page.Clipping then
   begin
      result.Minx := GetModelX(Page.CLipRegion.Left);
      result.Miny := GetModely(Page.CLipRegion.Bottom);
      result.Maxx := GetModelX(Page.CLipRegion.Right);
      result.Maxy := GetModely(Page.CLipRegion.Top);
   end
   else
      result := GetModelView;
end;

procedure TPageProjection.SetCenterPoint(const aCenterPoint: TDoubleCoord);
begin
   ChangeProjection(aCenterPoint, Scale, Page);
end;

procedure TPageProjection.SetCenterX(const aValue: double);
begin
  FCenterPoint.X:= aValue;
  ChangeProjection(FCenterPoint, Scale, Page);
end;

procedure TPageProjection.SetCenterY(const aValue: double);
begin
  FCenterPoint.Y:= aValue;
  ChangeProjection(FCenterPoint, Scale, Page);
end;


procedure TPageProjection.SetScale(const aScale: double);
begin
   ChangeProjection(CenterPoint, aScale, Page);
end;

procedure TPageProjection.SetPage(const aPage: T_Page);
begin
   ChangeProjection(CenterPoint, Scale, aPage);
end;


procedure TPageProjection.ChangeProjection (const aCenterPoint: TDoubleCoord;
                                              const aScale: double;
                                              const aPage: T_Page);
var
   CenterX, CenterY: longInt;
begin
   FCenterPoint := aCenterPoint;
   FScale       := aScale;

   FPage        := aPage;
   with Page do
   begin
      if Clipping and CenterToClipRegion then
      begin
         CenterX := longInt(ClipRegion.Left)+
                    ((longInt(ClipRegion.Right) - longInt(ClipRegion.Left)) div 2);
         CenterY := longInt(ClipRegion.Top)+
                    ((longInt(ClipRegion.Bottom) - longInt(ClipRegion.Top)) div 2);
      end
      else
      begin
         CenterX := (Rect.Right - Rect.Left) div 2;
         CenterY := (Rect.Bottom - Rect.Top) div 2;
      end;
      MultX := Scale/Resolution.x;
      MultY := (Scale/Resolution.y);
      UniMult:= MultX;

      if MultX < MultY then
         UniMult:= MultY;

      AddX := CenterPoint.x - (CenterX * MultX);
      AddY := CenterPoint.y + (CenterY * MultY);
   end;

end;

procedure TPageProjection.ChangeProjectionByRect (const aModelRect: TDoubleRect;
                                                    const aPage: T_Page);
begin
   FPage := aPage;
   ModelView := aModelRect;
end;


procedure TPageProjection.ChangeProjectionByRectangle (const aModelRect: TRectangle;
                                                    const aPage: T_Page);
var
   R: TDoubleRect;
begin
   with aModelRect do
   begin
      R.Min.X:= Min.X;
      R.Min.Y:= Min.Y;
      R.Max.X:= Max.X;
      R.Max.Y:= Max.Y;
   end;
   ChangeProjectionByRect(R, aPage);
end;


procedure TPageProjection.ChangeProjectionSimple(const aPage: T_Page);
var
   CenterX, CenterY: longInt;
   aScale: double;
begin
   aScale := 1;
   //FCenterPoint := aCenterPoint;
   FScale       := aScale;
   FPage        := aPage;
   with Page do
   begin
      if Clipping and CenterToClipRegion then
      begin
         CenterX := longInt(ClipRegion.Left)+
                    ((longInt(ClipRegion.Right) - longInt(ClipRegion.Left)) div 2);
         CenterY := longInt(ClipRegion.Top)+
                    ((longInt(ClipRegion.Bottom) - longInt(ClipRegion.Top)) div 2);
      end
      else
      begin
         CenterX := (Rect.Right - Rect.Left) div 2;
         CenterY := (Rect.Bottom - Rect.Top) div 2;
      end;
      MultX := Scale/Resolution.x;
      MultY := Scale/Resolution.y;
      UniMult:= MultX;

      if MultX < MultY then
         UniMult:= MultY;
      FCenterPoint.X := CenterX * MultX;
      FCenterPoint.Y := CenterY * MultY;
      //AddX := 0;
      //AddY := Rect.Bottom * MultY;
      AddX := CenterPoint.x - (CenterX * MultX);
      AddY := CenterPoint.y + (CenterY * MultY);
   end;

end;

procedure TPageProjection.SetModelView(const aRect: TDoubleRect);
var
   aCenterPoint: TDoubleCoord;
   aScale: double;
   ScaleX: double;
   ScaleY: double;
   dX,dY: double;
begin
   dX := aRect.MaxX - aRect.MinX;
   dY := aRect.MaxY - aRect.MinY;
   try
     ScaleX := dX/(Page.Rect.Right/Page.Resolution.x);
     ScaleY := dY/(Page.Rect.Bottom/Page.Resolution.y);
     if ScaleX > ScaleY then aScale := ScaleX
                        else aScale := ScaleY;
     aCenterPoint.x :=  aRect.MinX + dX/2;
     aCenterPoint.y :=  aRect.MinY + dY/2;
     ChangeProjection(aCenterPoint, aScale, Page)
   except
   end
end;

procedure TPageProjection.SetClipModelView(const aRect: TDoubleRect);
begin
   Page.ClipRegion := GetPageRect(aRect.Minx,aRect.MinY,aRect.MaxX,aRect.MaxY);
end;



function  TPageProjection.GetModelX(PageX: integer): double;
begin
   Result:= MultX*PageX + AddX ;
end;


function  TPageProjection.GetModelY(PageY: integer): double;
begin
   Result:=  AddY-(MultY*PageY);
end;


function  TPageProjection.GetModelArea(aWidth, aHeight: integer): double;
begin
  Result:= aWidth * MultX * aHeight * MultY;
end;


function ShortRound(aValue: double): integer;
begin
  if aValue > High(integer) then
     aValue:= High(Integer);
  if aValue < Low(Integer) then
     aValue:= Low(Integer);
  result:= round(aValue)
end;

function TPageProjection.GetPageX(const ModelX: double): longInt;
begin
  result:= 0;
  if MultX = 0 then exit;
  try
   Result:= ShortRound((ModelX - AddX)/MultX);
  except
  end
end;


function TPageProjection.GetPageY(const ModelY: double): longInt;
begin
  result:= 0;
  if MultY = 0 then exit;
  try
   Result:= ShortRound((AddY-ModelY)/MultY);
  except
  end
end;


function TPageProjection.GetPageLength(const ModelLength: double): longInt;
begin
  result:= 0;
  if MultY = 0 then exit;
  try
    Result:= round(ModelLength / MultY);
{    Result:= round(ModelLength / UniMult); }
  except
  end
end;




function TPageProjection.GetPageLengthHoriz1(const ModelLength: double): longInt;
begin
  result:= 0;
  if MultX = 0 then exit;
  try
    Result:= round(ModelLength / MultX);
  except
  end
end;



function TPageProjection.GetPageLengthVert1(const ModelLength: double): longInt;
begin
  result:= 0;
  if MultY = 0 then exit;
  try
    Result:= round(ModelLength / MultY);
  except
  end
end;


function  TPageProjection.GetModelLength(const PageLength: longInt): double;
begin
  Result:= PageLength * UniMult;
end;

function  TPageProjection.GetModelLengthHoriz(const PageLength: longInt): double;
begin
  Result:= PageLength * MultX;
end;

function  TPageProjection.GetModelLengthVert(const PageLength: longInt): double;
begin
  Result:= PageLength * MultY;
end;


function  TPageProjection.GetPagePoint(const ModelX, ModelY: double): WinTypes.TPoint;
begin
  result.X:= 0;
  result.Y:= 0;
  if MultX = 0 then exit;
  if MultY = 0 then exit;
  try
    Result.X:= round((ModelX - AddX)/MultX);
    Result.Y:= round((AddY - ModelY)/MultY);
  except
  end
end;


function  TPageProjection.GetPageRect(const MinX, MinY, MaxX, MaxY: double): WinTypes.TRect;
begin
   Result.TopLeft     := GetPagePoint(MinX, MaxY);
   Result.BottomRight := GetPagePoint(MaxX, MinY);
end;

function  TPageProjection.GetModelPoint(PageX, PageY: integer): TDoubleCoord;
begin
   Result.x := MultX*PageX + AddX;
   Result.y := AddY - (MultY*PageY);
end;


function  TPageProjection.GetModelPointElement(PageX, PageY: integer): TCoordinate;
begin
  Result:= T2DCoordinate.Create;
  Result.X:= MultX*PageX + AddX;
  Result.Y:= AddY - (MultY*PageY);
end;


function  TPageProjection.GetModelRect(Left, Top, Right, Bottom: integer): TDoubleRect;
begin
   result.Min := GetModelPoint(Left, Bottom);
   result.Max := GetModelPoint(Right, Top);
end;

BEGIN
   RegisterOrbisClass(TProjection);
   RegisterOrbisClass(TPageProjection);
END.
