unit t_JoinLines;

interface
uses
   Elements,
   e_QTree,
   e_GeometryElement,
   Tools;

type

TJoinSimpleLines = class(TTool)
public
   procedure DoIt; override;
end;

TJoinTopolLines = class(TTool)
  private
    FLines: TGeometryElementCollection;
    FRefLines: TEntity;
    FIgnorePresnost: boolean;
    FJoinRefLines: boolean;
    FSameDirection: boolean;
    procedure SetRefLines(const Value: TEntity);
    function GetLines: TGeometryElementCollection;
    procedure SetIgnorePresnost(const Value: boolean);
    procedure SetJoinRefLines(const Value: boolean);
    procedure SetSameDirection(const Value: boolean);
    procedure SetNextSwitch(const Value: string);
public
   procedure DoIt; override;
   property  Lines: TGeometryElementCollection read GetLines;
   function  NumberToPropertyName(aNumber: longInt): string;override;
published
   property  RefLines: TEntity read FRefLines write SetRefLines;
   property  JoinRefLines: boolean read FJoinRefLines write SetJoinRefLines;
   property  IgnorePresnost: boolean read FIgnorePresnost write SetIgnorePresnost;
   property  SameDirection: boolean read FSameDirection write SetSameDirection;
   property  NextSwitch: string write SetNextSwitch stored false;
end;

// Gravitacni spojovani nebo synchronizace dvou vrstev linii
// Jde o to ze postupne zpracovava jevy ve vrstvach jako jsou:
//     - uzly .... podle poctu vetvi a uhlu ruzna gravitace
//				 - singularity ... mista lomu a prudkych zmen smeru linii ... vyssi G nez okoli
//     - smer linii ... definuje gravitacni prohluben, linie by nemely byt pritahovany nastorc
//     - a vubec ... za tohle asi nobelovu cenu nedostanu
//       kazda cast linie se synchronizuje postupne podle poradi dulezitosti


THBridge = class(TOwnerEntity)
  private
    FSameDirection: boolean;
    FA1: TElement;
    FA2: TElement;
    FA1b: boolean;
    FA2b: boolean;
    FDone: boolean;
    FA1Index: integer;
    FA2Index: integer;
    FA1Index1: integer;
    FA2Index1: integer;
    procedure SetA1(const Value: TElement);
    procedure SetA2(const Value: TElement);
    procedure SetSameDirection(const Value: boolean);
    procedure SetA1b(const Value: boolean);
    procedure SetA2b(const Value: boolean);
    procedure SetDone(const Value: boolean);
    procedure SetA1Index(const Value: integer);
    procedure SetA2Index(const Value: integer);
    procedure SetA1Index1(const Value: integer);
    procedure SetA2Index1(const Value: integer);
public
   function IsKomplementaryElement(Value: TElement): boolean;
published
   property A1: TElement read FA1 write SetA1;
   property A2: TElement read FA2 write SetA2;
   property A1Index: integer read FA1Index write SetA1Index;
   property A2Index: integer read FA2Index write SetA2Index;
   property A1Index1: integer read FA1Index1 write SetA1Index1;
   property A2Index1: integer read FA2Index1 write SetA2Index1;
   property A1b: boolean read FA1b write SetA1b;
   property A2b: boolean read FA2b write SetA2b;
   property SameDirection: boolean read FSameDirection write SetSameDirection;
   property Done: boolean read FDone write SetDone;
end;

TSynchronizeLinesByGravity = class(TTool)
  private
    FTest: boolean;
    FGravity: double;
    FLines: TEntity;
    FFixedLines: TEntity;
    FSrcNodes: TEntity;
    FDestNodes: TEntity;
    FQTreeDest: TRectEntityQTree;
    FQTreeSrc: TRectEntityQTree;
    FKomplementaries: TEntity;
    FGIndexSetting: boolean;
    FSolvedN: TEntity;
    FSolvedL: TEntity;
    procedure SetFixedLines(const Value: TEntity);
    procedure SetGravity(const Value: double);
    procedure SetLines(const Value: TEntity);
    procedure SetTest(const Value: boolean);
    procedure SetDestNodes(const Value: TEntity);
    procedure SetSrcNodes(const Value: TEntity);
    procedure SetQTreeDest(const Value: TRectEntityQTree);
    procedure SetQTreeSrc(const Value: TRectEntityQTree);
    procedure SetKomplementaries(const Value: TEntity);
    procedure SetGIndexSetting(const Value: boolean);
    procedure SetSolvedN(const Value: TEntity);
    procedure SetSolvedL(const Value: TEntity);
public
   procedure DoIt; override;
   procedure SimilarNodesMoving(aTest: boolean);
   procedure DivideLinesByNodes(aNodes,aLines: TEntity);
   procedure DivideLineAtIndex(aL: TElement; aIndex: integer);
   procedure SolveKomplementaryLines(aHB: THBridge);
   procedure SolveKomplementaryLinesAndParts(aHB: THBridge);
   procedure SolveKomplementaryParts(aHB: THBridge);
   function  NumberToPropertyName(aNumber: longInt): string;override;
   procedure CheckBridgesAndShiftIndex(aL: TElement; aIndex,aShift: integer);
   procedure FindLineAnglesAtIndex(aL: TElement; aIndex: integer; var A1, A2: double);
   function  FindBridgeforKomplementaryLine(aL: TElement): THBridge;
   property  DestNodes: TEntity read FDestNodes write SetDestNodes;
   property  SrcNodes: TEntity read FSrcNodes write SetSrcNodes;
   property  QTreeDest: TRectEntityQTree read FQTreeDest write SetQTreeDest;
   property  QTreeSrc: TRectEntityQTree read FQTreeSrc write SetQTreeSrc;
   property  Komplementaries: TEntity read FKomplementaries write SetKomplementaries;
   property  SolvedN: TEntity read FSolvedN write SetSolvedN;
   property  SolvedL: TEntity read FSolvedL write SetSolvedL;
published
   property FixedLines: TEntity read FFixedLines write SetFixedLines;
   property Lines: TEntity read FLines write SetLines;
   property Test: boolean read FTest write SetTest;
   property Gravity: double read FGravity write SetGravity;
   property GIndexSetting: boolean read FGIndexSetting write SetGIndexSetting;
end;

implementation
uses
   Dialogs,
   Forms,
   SysUtils,
   MathLib,
   m_Orbis,
   e_Node,
   e_Class,

   e_Geometry,
   e_TopolElements,
   g_Elems,
   t_Snap,
   T_QTreeTools,
   T_FindBreakLineConnections;

{ TJoinSimpleLines }

procedure TJoinSimpleLines.DoIt;
var
  FB: TFindBreakLineConnections;
  E: TEntity; // entita uzl
  El: TElement;
  N,NN: TNode;
  i,j: integer;
  //TL1,TL2,TL: TTopolLineElement;
  L1,L2,L: TPolyLineElement;
  aX,aY,X1,Y1: double;
  B1,B2: boolean;
  NB1,NB2,NB: TNodeBranch;
  LinesEntity: TEntity;
begin
   if Input = nil then exit;
   if not (Input is TEntity) then exit;
   LinesEntity := TEntity(Input);
   if LinesEntity.NoElements then exit;
   StartProgress('Spojen lini...',0,0);
   Application.ProcessMessages;
   FB := TFindBreakLineConnections.Create;
   FB.ConnectionsNumber := 2; //budu hledat uzly dvou lini
   FB.Input := LinesEntity;
   FB.DoIt;
   E := TEntity(FB.output);
   FB.Free;
   MaxProgress := E.LastIndex;
   for i := 0 to E.LastIndex do
   begin
      Progress := i;
      Application.ProcessMessages;
      N := TNode(E.Elements[i]);
      NN := N;
      L1 := nil;
      L2 := nil;
      NB1 := nil;
      NB2 := nil;
      for j := 0 to N.LastIndex do
      begin
         El := N.Elements[j];
         if not (El is TNodeBranch) then continue;
         NB := TNodeBranch(El);
         if L1 = nil then
         begin
            NB1 := NB;
            L1 := NB1.Line;
            //TL1 := TTopolLineElement(L1.FindClassWithOwnerInOwners(TTopolLineElement,LinesEntity));
         end
         else
         begin
            NB2 := NB;
            L2 := NB2.Line;
            //TL2 := TTopolLineElement(L2.FindClassWithOwnerInOwners(TTopolLineElement,LinesEntity));
            Break;
         end;
      end;
      if (L1 = nil) or (L2 = nil) then continue;
      if L1 = L2 then continue;
      {
      if (TL1 = nil) or (TL2 = nil) then continue;
      if (TL1.Druh <> TL2.Druh) or (TL1.Presnost <> TL2.Presnost) then continue;
      if (TL1.Symbol > 0) and (TL2.Symbol > 0) then continue;
      if (TL1.Symbol <= 0) and (TL2.Symbol > 0) then
      begin
         TL := TL2;
         L := L2;
         NB := NB2;
         TL2 := TL1;
         L2 := L1;
         NB2 := NB1;
         TL1 := TL;
         L1 := L;
         NB1 := NB;
      end;
      aX := TL1.AbsRefX;
      aY := TL1.AbsRefY;
      }
      for j := 1 to L2.LastIndex do
      begin
         L2.GetOrientedCoord(j,NB2.LineBegin,X1,Y1);
         if NB1.LineBegin then //bude se pidvat na zatek
            L1.InsertXY(0,X1,Y1)
         else                  //nebo na konec
            L1.AddXY(X1,Y1);
      end;
      L1.RecalculateRect;
      L1.CalculateLabelProps;
      //L1.AbsRefX := aX;
      //L1.AbsRefY := aY;
      NN.FreeAll;
      GiveNodeBranchForLine(L2, not(NB2.LineBegin),N,NB);
      if (N <> nil) and (NB <> nil) then
      begin
         N.RemoveElement(L2);
         N.AddElement(L1);
         NB.Line := L1;
         NB.LineBegin := NB1.LineBegin;
         //NB.Free;
      end;
      //if NB2 <> nil then NB2.Free;
      //TL2.Free;
      L2.Free;
   end;
   E.Free;
   //Progress := 0;
end;

{ TJoinTopolLines }

procedure TJoinTopolLines.DoIt;
var
  FB: TFindBreakLineConnections;
  E,E1: TEntity; // entita uzl
  El: TElement;
  N,NN: TNode;
  i,j: integer;
  TL1,TL2,TL: TTopolLineElement;
  L1,L2,L: TPolyLineElement;
  aX,aY,X1,Y1: double;
  B1,B2: boolean;
  NB1,NB2,NB: TNodeBranch;
  LinesEntity: TEntity;
begin
   if Input = nil then exit;
   if not (Input is TEntity) then exit;
   LinesEntity := TEntity(Input);
   if LinesEntity.NoElements then exit;
   Lines.AddAllFrom := LinesEntity;
   if (RefLines <> nil) and not RefLines.NoElements then
      Lines.AddAllFrom := RefLines;
   Lines.Update;
   StartProgress('Spojen T lini...',0,0);
   Application.ProcessMessages;
   FB := TFindBreakLineConnections.Create;
   FB.ConnectionsNumber := 2; //budu hledat uzly dvou lini
   FB.Input := Lines;
   FB.DoIt;
   E := TEntity(FB.output);
   FB.Free;
   if E = nil then
   begin
      ShowMessage('Spojen lini se z njakho dvodu nepodailo, asi bude pret');
      exit;
   end;
   MaxProgress := E.LastIndex;
   for i := 0 to E.LastIndex do
   begin
      Progress := i;
      Application.ProcessMessages;
      N := TNode(E.Elements[i]);
      NN := N;
      L1 := nil;
      L2 := nil;
      TL1 := nil;
      TL2 := nil;
      NB1 := nil;
      NB2 := nil;
      for j := 0 to N.LastIndex do
      begin
         El := N.Elements[j];
         if not (El is TNodeBranch) then continue;
         NB := TNodeBranch(El);
         E1 := nil;
         if L1 = nil then
         begin
            NB1 := NB;
            L1 := NB1.Line;
            TL1 := TTopolLineElement(L1.FindClassWithOwnerInOwners(TTopolLineElement,LinesEntity));
            if RefLines <> nil then
               E1 := TTopolLineElement(L1.FindClassWithOwnerInOwners(TTopolLineElement,RefLines));
            if (E1 <> nil) and not JoinRefLines  then
            begin
               L1 := nil;
               break;
            end;
         end
         else
         begin
            NB2 := NB;
            L2 := NB2.Line;
            TL2 := TTopolLineElement(L2.FindClassWithOwnerInOwners(TTopolLineElement,LinesEntity));
            if RefLines <> nil then
               E1 := TTopolLineElement(L1.FindClassWithOwnerInOwners(TTopolLineElement,RefLines));
            if (E1 <> nil) and not JoinRefLines  then
               L2 := nil;
            Break;
         end;
      end;
      if (L1 = nil) or (L2 = nil) then continue;
      if L1 = L2 then continue;
      if (TL1 = nil) or (TL2 = nil) then continue;
      if (TL1.Druh <> TL2.Druh)then continue;
      if (TL1.Presnost <> TL2.Presnost) and not IgnorePresnost then continue;
      if (NB1.LineBegin = NB2.LineBegin) and SameDirection then continue;
      if (TL1.Symbol > 0) and (TL2.Symbol > 0) then continue;
      if (TL1.Symbol <= 0) and (TL2.Symbol > 0) then
      begin
         TL := TL2;
         L := L2;
         NB := NB2;
         TL2 := TL1;
         L2 := L1;
         NB2 := NB1;
         TL1 := TL;
         L1 := L;
         NB1 := NB;
      end;
      aX := TL1.AbsRefX;
      aY := TL1.AbsRefY;
      for j := 1 to L2.LastIndex do
      begin
         L2.GetOrientedCoord(j,NB2.LineBegin,X1,Y1);
         if NB1.LineBegin then //bude se pidvat na zatek
            L1.InsertXY(0,X1,Y1)
         else                  //nebo na konec
            L1.AddXY(X1,Y1);
      end;
      L1.RecalculateRect;
      //L1.CalculateLabelProps;
      L1.AbsRefX := aX;
      L1.AbsRefY := aY;
      NN.FreeAll;
      GiveNodeBranchForLine(L2, not(NB2.LineBegin),N,NB);
      if (N <> nil) and (NB <> nil) then
      begin
         N.RemoveElement(L2);
         N.AddElement(L1);
         NB.Line := L1;
         NB.LineBegin := NB1.LineBegin;
         //NB.Free;
      end;
      //if NB2 <> nil then NB2.Free;
      L2.Free;
      TL2.Free;
   end;
   E.Free;
   //Progress := 0;
end;


function TJoinTopolLines.GetLines: TGeometryElementCollection;
begin
   if FLines = nil then
   begin
      FLines := TGeometryElementCollection.Create;
      AddElement(FLines);
   end;
   result := FLines;
end;

function TJoinTopolLines.NumberToPropertyName(aNumber: Integer): string;
begin
   Result := 'NextSwitch';
end;

procedure TJoinTopolLines.SetIgnorePresnost(const Value: boolean);
begin
  FIgnorePresnost := Value;
end;

procedure TJoinTopolLines.SetJoinRefLines(const Value: boolean);
begin
  FJoinRefLines := Value;
end;

procedure TJoinTopolLines.SetNextSwitch(const Value: string);
var
   S: string;
   Ch: char;
begin
   if Value = '' then exit;
   S := UpperCase(Value);
   CH := S[1];
   case CH of
      'P': IgnorePresnost := true;
      'D': SameDirection := true;
   end;
end;

procedure TJoinTopolLines.SetRefLines(const Value: TEntity);
begin
  FRefLines := Value;
end;

procedure TJoinTopolLines.SetSameDirection(const Value: boolean);
begin
  FSameDirection := Value;
end;

{ TSynchronizeLinesByGravity }

procedure TSynchronizeLinesByGravity.CheckBridgesAndShiftIndex(aL: TElement; aIndex,aShift: integer);
var
   i: integer;
   hb: THBridge;
   e: TEntity;
begin
   for i := 0 to aL.OwnersCount - 1 do
   begin
      e := aL.Owners[i];
      if not (e is THBridge) then continue;
      hb := THBridge(e);
      if hb.A1 = aL then
      begin
          if hb.A1Index >= aIndex then hb.A1Index := hb.A1Index + aShift;
          if hb.A1Index1 >= aIndex then hb.A1Index1 := hb.A1Index1 + aShift;
      end
      else
      if hb.A2 = aL then
      begin
          if hb.A2Index >= aIndex then hb.A2Index := hb.A2Index + aShift;
          if hb.A2Index1 >= aIndex then hb.A2Index1 := hb.A2Index1 + aShift;
      end;
   end;
end;

procedure TSynchronizeLinesByGravity.DivideLineAtIndex(aL: TElement; aIndex: integer);
var
   L,L1: TPolyLineElement;
   i: integer;
begin
   L := TPolyLineElement(aL);
   L1 := TPolyLineElement.Create;
   for i := 0 to aIndex do
       L1.AddXY(L.Xi[i],L.Yi[i]);
   for i := aIndex - 1 DownTo 0 do
       L.FreeElement(L.Elements[i]);
   L.RecalculateRect;
   L.CalculateLabelProps;
   L.Angle := 0;
   L1.RecalculateRect;
   L1.CalculateLabelProps;
   L.PostMsgToOwners(tm_DivideGeometry,L1,false);
   Application.ProcessMessages;
end;

procedure TSynchronizeLinesByGravity.DivideLinesByNodes(aNodes, aLines: TEntity);
var
   GEL: TGeometryElementCollection;
   LevelQT: TRectEntityQTree;
   TQ: TAddEntityToQTree;
   Snap: TPointLineSnapTool;
   QTW: TQTreeWalker;
   E: TElement;
   sN,N,dN,xN,oN: TNode;  // dN nejbli Node, xN node s max Branch,oN Node se stejnm p. Branch
      d,dd,xd,od,ssd: double; // vzd. dd nejblizsi, xd nejbl.xN, od nejbl. oN
   SS,SSS: TSingularity;
   UseSing,LB: boolean;
   NB,NB1,NB2: TNodeBranch;
   HB: THBridge;
   L,L1,L2 : TPolyLineElement;
   i,j,ni,nii, sBC,BC, Snapi: integer;    // Branch count source and actual Node...
   aX1,aX2,aY1,aY2,dA,dAA,A0,A1,A2: double;
   C: T2DCoordinate;
begin
   // Zkusime, jestli by uzly ktere se neresily nepricucly nejake src linie...
   GEL := TGeometryElementCollection(aNodes);
   ProgressText := 'Nodes Snapping...';
   MaxProgress := GEL.LastIndex;
   Progress := 0;
   Application.ProcessMessages;
   LevelQT :=TRectEntityQTree.Create;
   AddElement(LevelQT);
   LevelQT.Name := 'Lines QT';
   TQ := TAddEntityToQTree.Create;
   TQ.Input := aLines;
   TQ.Output := LevelQT;
   TQ.DoIt;
   TQ.Free;
   Snap := TPointLineSnapTool.Create;
   Snap.FrameQT := LevelQT;
   Snap.SnapToLine := true;
   Snap.SnapToLinePoint := true;
   Snap.SnapToLineBody := true;
   Snap.SnapDistance := Gravity * 0.75;
   Snap.SnapInAngle := false;
   Snap.NotEnds := false;
   //Snap.AngleTolerance := AngleTolerance;
   //SnapCount := 0;
   for i := 0 to GEL.LastIndex do
   begin
      if (i mod 50) = 0 then
      begin
         Progress := i;
         Application.ProcessMessages;
      end;
      if not (GEL.Elements[i] is TNode) then continue;//asi singularita
      N := TNode(GEL.Elements[i]);
      if N.IsMyOwner(SolvedN) then
         continue;           // timhle uzlem jsem se asi uz zabyval ...  takze hop
      Snap.X := N.X;
      Snap.Y := N.Y;
      Snap.DoIt;
      if Snap.Snapped then
      begin
         if Snap.WasEnd then continue; // konec uz mel bejt resenej....
         L := TPolyLineElement(Snap.Output);
         Snapi := Snap.SnapIndex;
         // pokud bylo telo linie, vlozime tam bod
         if Snap.SnapStatus = 2 then
         begin
            L.InsertXY(Snapi + 1, Snap.SnapX, Snap.SnapY);
            Inc(Snapi);  // novej bod, musime zvysit index
         end;
         // rozdeleni linie v lomovem bodu ... nebo bode?
         DivideLineAtIndex(L, Snapi);
      end;
   end;
   Snap.Free;
   LevelQT.Free;
end;

procedure TSynchronizeLinesByGravity.DoIt;
var
   FB: TFindBreakLineConnections;
   FS: TFindSingularities;
   GEL: TGeometryElementCollection;
   LevelQT: TRectEntityQTree;
   TQ: TAddEntityToQTree;
   Snap: TPointLineSnapTool;
   QTW: TQTreeWalker;
   E: TElement;
   sN,N,dN,xN,oN: TNode;  // dN nejbli Node, xN node s max Branch,oN Node se stejnm p. Branch
      d,dd,xd,od,ssd: double; // vzd. dd nejblizsi, xd nejbl.xN, od nejbl. oN
   SS,SSS: TSingularity;
   UseSing,LB: boolean;
   NB,NB1,NB2: TNodeBranch;
   HB: THBridge;
   L,L1,L2 : TPolyLineElement;
   i,j,ni,nii, sBC,BC, Snapi: integer;    // Branch count source and actual Node...
   aX1,aX2,aY1,aY2,dA,dAA,A0,A1,A2: double;
   C: T2DCoordinate;
   En: TEntity;
begin
   Error := 0;
   // nalezeni inputu
   if (Lines = nil) or (Lines.NoElements) then
   begin
      if (Input <> nil) and (Input is TEntity) then
         Lines := TEntity(Input);
   end;
   if (Lines = nil) or (Lines.NoElements) then
   begin
      Error := 1;
      exit;
   end;
   if (FixedLines = nil) or (FixedLines.NoElements) then
   begin
      Error := 2;
      exit;
   end;
   // kontrola spravnosti a existence lines...
   // vytvoreni uzlu zdroje a cile
   StartProgress('Gravitace T lini...',0,0);
   if GIndexSetting then
   for i := 0 to Lines.LastIndex do
   begin
      TGeometryElement(Lines.Elements[i]).GIndex := 1;
   end;
   if GIndexSetting then
   for i := 0 to FixedLines.LastIndex do
   begin
      TGeometryElement(FixedLines.Elements[i]).GIndex := 1;
   end;


   ProgressText := 'Singularities 1';
   En := TOwnerEntity.Create;
   Application.ProcessMessages;
   FS := TFindSingularities.Create;
   FS.Input := FIxedLines;
   FS.Output := En;
   FS.DoIt;
   FS.Free;
   for i := En.LastIndex downto 0 do
   begin
      SS := TSingularity(En.Elements[i]);
      DivideLineAtIndex(SS.Line,SS.CoordIndex);
   end;
   En.Free;

   ProgressText := 'Singularities 2';
   En := TOwnerEntity.Create;
   Application.ProcessMessages;
   FS := TFindSingularities.Create;
   FS.Input := Lines;
   FS.Output := En;
   FS.DoIt;
   FS.Free;
   for i := En.LastIndex downto 0 do
   begin
      SS := TSingularity(En.Elements[i]);
      DivideLineAtIndex(SS.Line,SS.CoordIndex);
   end;
   En.Free;

   ProgressText := 'Destination nodes';
   GEL := TGeometryElementCollection.Create;
   GEL.Name := 'DstNode';
   DestNodes := GEL;
   Application.ProcessMessages;
   FB := TFindBreakLineConnections.Create;
   //FB.ConnectionsNumber := 2; //budu hledat vsechny uzly
   FB.AllNodes := true;
   FB.Input := FixedLines;
   FB.Output := GEL;
   FB.DoIt;
   E := TEntity(FB.output);
   FB.Free;
   GEL.Update;

   ProgressText := 'Source nodes';
   Application.ProcessMessages;

   GEL := TGeometryElementCollection.Create;
   GEL.Name := 'SrcNode';
   SrcNodes := GEL;
   Application.ProcessMessages;
   FB := TFindBreakLineConnections.Create;
   //FB.ConnectionsNumber := 2; //budu hledat vsechny uzly
   FB.AllNodes := true;
   FB.Input := Lines;
   FB.Output := GEL;
   FB.DoIt;
   FB.Free;
   GEL.Update;

   ProgressText := 'Destination nodes QTree';
   Application.ProcessMessages;

   // nahazeni uzlu cile do QTree   // prochazeni uzlu zdroje a pasovani na cil podle gravitacnich pravidel...
   TQ := TAddEntityToQTree.Create;
   TQ.Input := DestNodes;
   TQ.DoIt;
   QTreeDest :=TRectEntityQTree(TQ.Output);
   QTreeDest.Name := 'DestNode QT';
   TQ.Free;

   GEL := TGeometryElementCollection(SrcNodes);

   SolvedN := TRectEntity.Create;
   SolvedN.Name := 'Solved N';
   Komplementaries := TRectEntity.Create;
   Komplementaries.Name := 'Komplement';

   // zkusime umistit uzly .....
   SimilarNodesMoving(True);  // ma byt test

   // a ted rozmlatime zdrojovou kresbu podle cilovych uzlu
   DivideLinesByNodes(DestNodes, Lines);

   // a totez opacne
   DivideLinesByNodes(SrcNodes, FixedLines);
   
   // a znova uzly a tak
   SolvedN.FreeAll;
   DestNodes := nil;
   ProgressText := 'Destination nodes again';
   GEL := TGeometryElementCollection.Create;
   GEL.Name := 'DstNode';
   DestNodes := GEL;
   Application.ProcessMessages;
   FB := TFindBreakLineConnections.Create;
   //FB.ConnectionsNumber := 2; //budu hledat vsechny uzly
   FB.AllNodes := true;
   FB.Input := FixedLines;
   FB.Output := GEL;
   FB.DoIt;
   E := TEntity(FB.output);
   FB.Free;

   GEL.Update;

   ProgressText := 'Source nodes';
   SrcNodes := nil;
   Application.ProcessMessages;

   GEL := TGeometryElementCollection.Create;
   GEL.Name := 'SrcNode';
   SrcNodes := GEL;
   Application.ProcessMessages;
   FB := TFindBreakLineConnections.Create;
   //FB.ConnectionsNumber := 2; //budu hledat vsechny uzly
   FB.AllNodes := true;
   FB.Input := Lines;
   FB.Output := GEL;
   FB.DoIt;
   FB.Free;
   GEL.Update;

   ProgressText := 'Destination nodes QTree';
   Application.ProcessMessages;

   // nahazeni uzlu cile do QTree   // prochazeni uzlu zdroje a pasovani na cil podle gravitacnich pravidel...
   QTreeDest := nil;
   TQ := TAddEntityToQTree.Create;
   TQ.Input := DestNodes;
   TQ.DoIt;
   QTreeDest :=TRectEntityQTree(TQ.Output);
   QTreeDest.Name := 'DestNode QT';
   TQ.Free;

   SolvedL := TRectEntity.Create;
   SolvedL.Name := 'Solved L';

   GEL := TGeometryElementCollection(SrcNodes);
   // zkusime umistit uzly ..... tentokrat opravdu ....
   SimilarNodesMoving(False);
   //exit;
   // No a tady si asi strihnem doreseni zbylych komplementarit ....
   //exit;
   for i := 0 to Komplementaries.LastIndex do
   begin
      HB := THBridge(Komplementaries.Elements[i]);
      if not HB.Done then
         SolveKomplementaryParts(HB);
   end;
end;

function TSynchronizeLinesByGravity.FindBridgeforKomplementaryLine(aL: TElement): THBridge;
var
   e: TEntity;
   i: integer;
   HB: THBridge;
begin
   result := nil;
   for i := 0 to aL.OwnersCount - 1 do
   begin
      e := aL.Owners[i];
      if not (e is THBridge) then continue;
      HB := THBridge(e);
      if HB.IsKomplementaryElement(aL) then
      begin
         result := HB;
         Break;
      end;
   end;
end;

procedure TSynchronizeLinesByGravity.FindLineAnglesAtIndex(aL: TElement; aIndex: integer; var A1, A2: double);
var
   aLine: TPolyLineElement;
   X1, Y1, X2, Y2: double;
   i, ix: integer;
   ll,l,AA,A: double;
begin
   A1 := 0;
   A2 := 0;
   if aL = nil then exit;
   if not (aL is TPolyLineElement) then exit;
   aLine := TPolylineElement(aL);
   ll := 0;
   // uhel po smeru
   ix := aLine.Count div 2;
   ix := aIndex + ix;
   if ix > aLine.LastIndex then ix := LastIndex - 1;
   for i := aIndex to ix do
   begin
      aLine.GetOrientedCoord(i, true, X1, Y1);
      aLine.GetOrientedCoord(i + 1, true, X2, Y2);
      A := ATan2M(X2-X1, Y2-Y1);
      l := Distance(X1,Y1,X2,Y2);
      LL := LL + l;
      AA := AA + (A * l);
   end;
   A1 := AA / LL;
   // uhel proti srsti
   ix := aLine.Count div 2;
   ix := aIndex - ix;
   if ix < 1 then ix := 1;
   for i := aIndex downto ix do
   begin
      aLine.GetOrientedCoord(i, false, X1, Y1);
      aLine.GetOrientedCoord(i - 1, false, X2, Y2);
      A := ATan2M(X2-X1, Y2-Y1);
      l := Distance(X1,Y1,X2,Y2);
      LL := LL + l;
      AA := AA + (A * l);
   end;
   A2 := AA / LL;
end;

function TSynchronizeLinesByGravity.NumberToPropertyName(aNumber: Integer): string;
begin
   case aNumber of
      0: result := 'FixedLines';
      1: result := 'Gravity';
      2: result := 'Test';
      // a co zbytek?
   end
end;

procedure TSynchronizeLinesByGravity.SetDestNodes(const Value: TEntity);
begin
   ExchangeFieldElement(TElement(FDestNodes), Value);
end;

procedure TSynchronizeLinesByGravity.SetFixedLines(const Value: TEntity);
begin
   FFixedLines := Value;
end;

procedure TSynchronizeLinesByGravity.SetGIndexSetting(
  const Value: boolean);
begin
  FGIndexSetting := Value;
end;

procedure TSynchronizeLinesByGravity.SetGravity(const Value: double);
begin
  FGravity := Value;
end;

procedure TSynchronizeLinesByGravity.SetKomplementaries(const Value: TEntity);
begin
   ExchangeFieldElement(TElement(FKomplementaries), Value);
end;

procedure TSynchronizeLinesByGravity.SetLines(const Value: TEntity);
begin
  FLines := Value;
end;

procedure TSynchronizeLinesByGravity.SetQTreeDest(const Value: TRectEntityQTree);
begin
   ExchangeFieldElement(TElement(FQTreeDest), Value);
end;

procedure TSynchronizeLinesByGravity.SetQTreeSrc(const Value: TRectEntityQTree);
begin
   ExchangeFieldElement(TElement(FQTreeSrc), Value);
end;

procedure TSynchronizeLinesByGravity.SetSolvedL(const Value: TEntity);
begin
    ExchangeFieldElement(TElement(FSolvedL), Value);
end;

procedure TSynchronizeLinesByGravity.SetSolvedN(const Value: TEntity);
begin
    ExchangeFieldElement(TElement(FSolvedN), Value);
end;

procedure TSynchronizeLinesByGravity.SetSrcNodes(const Value: TEntity);
begin
   ExchangeFieldElement(TElement(FSrcNodes), Value);
end;

procedure TSynchronizeLinesByGravity.SetTest(const Value: boolean);
begin
  FTest := Value;
end;

procedure TSynchronizeLinesByGravity.SimilarNodesMoving(aTest: boolean);
var
   GEL: TGeometryElementCollection;
   LevelQT: TRectEntityQTree;
   TQ: TAddEntityToQTree;
   Snap: TPointLineSnapTool;
   QTW: TQTreeWalker;
   E: TElement;
   sN,N,dN,xN,oN: TNode;  // dN nejbli Node, xN node s max Branch,oN Node se stejnm p. Branch
      d,dd,xd,od,ssd: double; // vzd. dd nejblizsi, xd nejbl.xN, od nejbl. oN
   SS,SSS: TSingularity;
   UseSing,LB: boolean;
   NB,NB1,NB2: TNodeBranch;
   HB: THBridge;
   L,L1,L2 : TPolyLineElement;
   i,j,ni,nii, sBC,BC, Snapi: integer;    // Branch count source and actual Node...
   aX1,aX2,aY1,aY2,dA,dAA,A0,A1,A2: double;
   C: T2DCoordinate;
begin

   GEL := TGeometryElementCollection(SrcNodes);

   ProgressText := 'Nodes moving ... test';
   MaxProgress := GEL.LastIndex;
   Progress := 0;
   Application.ProcessMessages;
   for i := 0 to GEL.LastIndex do
   begin
       if (i mod 50) = 0 then
       begin
          Progress := i;
          Application.ProcessMessages;
       end;
       sN := TNode(GEL.Elements[i]);
       aX1 := sN.X - Gravity;
       aY1 := sN.Y - Gravity;
       aX2 := sN.X + Gravity;
       aY2 := sN.Y + Gravity;
       sBC := sN.BranchCount;
       // kontrola jestli nahodou neni simple uzel s linii pod gravity, tuhle eventualitu vynechame
       //-----------------------------
       if sBC = 1 then
       begin
          NB1 := TNodeBranch(sN.FindElementClass(TNodeBranch));
          if NB1 <> nil then
             if NB1.Line <> nil then
                if NB1.Line.Length < Gravity then
                   continue; // ano, jednoduchy uzel s kratkou linii proste budu ignorovat
       end;
       QTW := TQTreeWalker.Create;
       e := QTW.GetFirstElementForRectangle(QTreeDest,aX1,aY1,aX2,aY2,0);
       dd := 2*Gravity;
       xd := dd;
       od := dd;
      ssd := dd;
       xN := nil;
       oN := nil;
       dN := nil;
       SSS := nil;
       UseSing := false;
       while e <> nil do
       begin
          // nalezeni uzlu  s co nejvetsi podobnosti ...
          // podobnost resi pocet vetvi a uhly
          if e is TNode then
          begin
             N  := TNode(e);
             BC := N.BranchCount;
             d := Distance(sN.X,sN.Y,N.X,N.Y);
             if d <= Gravity then
             begin
                if d < dd then
                begin
                   dd := d;
                   dN := N;
                end;
                if (BC = sBC) and (d < od) then
                begin
                   od := d;
                   oN := N;
                end;
                if (xN = nil) or (BC > xN.BranchCount) or ((BC = xN.BranchCount) and (d < xd))  then
                begin
                   xN := N;
                   xd := d;
                end;
             end;
          end
          else
          if e is TSingularity then
          begin
             SS := TSingularity(e);
             d := Distance(sN.X,sN.Y,SS.X,SS.Y);
             if d <= Gravity then
             begin
                if d < ssd then
                begin
                   ssd := d;
                   SSS := SS;
                end;
             end;
          end;
          e := QTW.GetNextElement;
       end;
       QTW.Free;
       N := nil;
       //je optimal ?
       if (oN <> nil) and ((xN = nil) or (xd > od))then
       begin
          N := oN;
          d := od;
       end
       else
       // a co takhle max vetvi?
       if (xN <> nil) and ((SSS = nil) or (xN.BranchCount > 1)) then
       begin
          N := xN;
          d := xd;
       end
       else
       //treba singularita?
       if SSS <> nil then
       begin
          UseSing := true;
          N := sN;
          d := ssd;
       end;
       //else
       // no tak a co nejblizsi kdyz bude opravdu ale opravdu blizko tak asi jo ne?
       if (dN <> nil) and ((N = nil) or (dd < d / 2)) then
       begin
          N := dN;
          UseSing := false;
       end;
       // no a pokud teda je ceho se chytat
       if (N <> nil)  or UseSing then
       begin
          if not aTest then
          begin
             //d := distance(sN.X,sN.Y,N.X,N.Y);
             if UseSing then
             begin
                // u singularity na komplementaritu seru
                // coz ovsem zpusobi pretahovani linii z uzlu ... asi
                sN.X := SSS.X;
                sN.Y := SSS.Y;
             end
             else
             begin
                // definovat komplementarni dvojice
                for j := 0 to sN.LastIndex do
                begin
                   e := sN.Elements[j];
                   if not (e is TNodeBranch) then continue;
                   NB1 := TNodeBranch(e);
                   A1 := NB1.{M}Angle;
                   L := NB1.Line;
                   HB := THBridge(L.FindOwnerClass(THBridge));
                   // zjistujeme, jestli nahodou uz mustek neexistuje ....
                   // mozna by to bylo lepsi zjistit pred dalsim divadlem
                   // jo a jinak hledame vetev s nejmensi odchylkou ... kdyby neco
                   // ovsem je treba resit uhel na vetsi vzd., jeden bod nestaci ...

                   dA := 2*Pi;
                   NB := nil;
                   for ni := 0 to N.LastIndex do
                   begin
                      e := N.Elements[ni];
                      if not (e is TNodeBranch) then continue;
                      NB2 := TNodeBranch(e);
                      A2 := NB2.{M}Angle;
                      L2 := NB2.Line;
                      if HB <> nil then
                      begin
                         if HB.IsKomplementaryElement(L2) then
                         begin
                            // tak bacha, tahle cara se bude komplet kopirovat z predlohy ... asi...
                            if not HB.Done then
                               SolveKomplementaryLines(HB);
                            break;  // no a co tady .. rychle ven
                         end;
                      end;
                      dAA := abs(A1 - A2);
                      if dAA > pi then
                         dAA := 2*pi - dAA;
                      if (dAA < pi/6) and (dAA < dA) then //22.5 st. jako max odchylka;
                      begin
                         dA := dAA;
                         NB := NB2;
                      end;
                   end;
                   if NB  <> nil then  // asi sme nasli  komplementarni linii
                   begin
                      // predstava je, ze poprve se proste mustek vytvori, priste, v pripade stejne
                      // linie uz bude jasne, ze linie jsou totozne
                      // Mustky ktere  zbudou potrebuji prostorove doresit ....
                      // jedna z linii je pravdepodobne kratsi a tak podobne

                      if GIndexSetting then
                      begin
                         if HB <> nil then
                         begin
                            NB.Line.GIndex := NB.Line.GIndex + 30;
                            NB1.Line.GIndex := NB1.Line.GIndex + 30;
                         end
                         else
                         begin
                            NB.Line.GIndex := NB.Line.GIndex + 10;
                            NB1.Line.GIndex := NB1.Line.GIndex + 10;
                         end;
                      end;
                      HB := THBridge.Create;
                      HB.A1 := NB.Line;
                      HB.A2 := NB1.Line;
                      HB.A1b := NB.LineBegin;
                      HB.A2b := NB1.LineBegin;
                      HB.SameDirection := NB.LineBegin = NB1.LineBegin;
                      Komplementaries.AddElement(HB);
                   end;
                end;

                sN.X := N.X;
                sN.Y := N.Y;
                if not N.IsMyOwner(SolvedN) then
                   SolvedN.AddElement(N);
             end;
             sN.MinX := sN.X - 1;
             sN.MinY := sN.Y - 1;
             sN.MaxX := sN.X + 1;
             sN.MaxY := sN.Y + 1;
             sN.AbsRefX := sN.X;
             sN.AbsRefY := sN.Y;
             for j := 0 to sN.LastIndex do
             begin
                e := sN.Elements[j];
                if not (e is TNodeBranch) then continue;
                NB := TNodeBranch(e);
                L := NB.Line;
                C := L.GetOrientedCoordElement(0,NB.LineBegin);
                C.X := sN.X;
                C.Y := sN.Y;
             end;
          end
          else
          begin
             // jenom testujem, nic se nehejbe ...
             if not N.IsMyOwner(SolvedN) then
             begin
                SolvedN.AddElement(N);
                SolvedN.AddElement(sN);
             end;
          end;
       end;
   end;
end;

procedure TSynchronizeLinesByGravity.SolveKomplementaryLines(aHB: THBridge);
var
   L1,L2: TPolyLineElement;
   i: integer;
   aX,aY: double;
   C: T2DCoordinate;
begin
   // synchronizace linii .... asi A2 podle A1 ... no nebo naopak, kdo to ma sakra vedet...
   if aHB = nil then exit;
   L1 := TPolyLineElement(aHB.A1);
   L2 := TPolyLineElement(aHB.A2);
   if GIndexSetting then
   begin
      L1.GIndex := L1.GIndex + 200;
      L2.GIndex := L2.GIndex + 200;
   end;
   if (L1 = nil) or (L2 = nil) then exit;
   L2.FreeAll();
   for i := 0 to L1.LastIndex do
   begin
      L1.GetOrientedCoord(i,aHB.SameDirection,aX,aY);
      L2.AddXY(aX,aY);
   end;
   L2.RecalculateRect;
   L2.RecalculateLabelProps := true;
   // no a kdyz uz to vlastne je, je za
   //Komplementaries.ElementToFree := aHB;
   aHB.Done := true;
   SolvedL.AddElement(L1);
   SolvedL.AddElement(L2);
end;

procedure TSynchronizeLinesByGravity.SolveKomplementaryLinesAndParts(aHB: THBridge);
begin
end;

procedure TSynchronizeLinesByGravity.SolveKomplementaryParts(aHB: THBridge);
var
   ii: integer;
   idx: integer;
   XX,YY: double;
   L1,L2: TPolyLineElement;
   N: TNode;
   NB: TNodeBranch;
   isNode: boolean;
   ex: boolean;

   procedure FindLinePoint(Lin: TPolyLineElement; aX,aY: double; Poc: boolean; var ix: integer; var exact: boolean);
   var
     I: longInt;
     Uvnitr: boolean;
     XP, YP, Vzd: double;
   begin
     //Result:= false;
     exact := true;
     ix := -1;
     if poc then
     begin
        for i := 0 to Lin.LastIndex do
           if InTolerance(aX,aY,Lin.Xi[i],Lin.Yi[i],Gravity) then
              ix := i;
     end
     else
     begin
        for i := Lin.LastIndex downto 0 do
           if InTolerance(aX,aY,Lin.Xi[i],Lin.Yi[i],Gravity) then
              ix := i;
     end;
     if ix < 0 then
     for I:= 0 to Lin.Count-2 do
     begin
       DejVztahyBod_Usecka(aX, aY, Lin.Xi[I], Lin.Yi[I], Lin.Xi[I+1], Lin.Yi[I+1],
                           Uvnitr, Xp, Yp, Vzd);
       if Uvnitr and (Vzd < Gravity) then
       begin
         ix := i;
         exact := false;
         //Result:= true;
         Break
       end
     end
   end;

begin
   //exit;
   L1 := TPolyLineElement(aHB.A1);
   L2 := TPolyLineElement(aHB.A2);
   if L2.IsMyOwner(SolvedL) then exit; // asi uz je narovnana ....
   aHB.Done := true;
   // husty ... a co ted jako s tim?
   // asi zjistime, ktera linie je delsi ...
   // jo a kdybych zapomnel, pasujem A2 na A1 ....
   if L2.Length < L1.Length then
   // pasovana je kratsi ... to bude asi minimalne, ale co uz
   // najdem bod na druhe linii a tak nejak ji priprcnem
   begin
      L2.GetOrientedCoord(0,not aHB.A2b,XX,YY);
      if GIndexSetting then
         L2.GIndex := 10;
      FindLinePoint(L1,XX,YY,aHB.A1b,idx,Ex);
      if idx < 0 then exit;
      L2.FreeAll;  // oho, trochu smele ...ne?
      if aHB.A1b then
      begin
         for ii := 0 to idx do
         begin
            L1.GetOrientedCoord(ii,true,XX,YY);
            L2.AddXY(XX,YY);
         end;
      end
      else
      begin
         for ii := L1.LastIndex downto idx do
         begin
            L1.GetOrientedCoord(ii,true,XX,YY);
            L2.AddXY(XX,YY);
         end;
      end;
      L2.RecalculateRect;
      L2.RecalculateLabelProps := true;
   end
   else
   // tak ne, je kratsi ten mustr....
   // hledame bod na L2 ktery napasujem na konec L1
   begin
      L1.GetOrientedCoord(0,not aHB.A1b,XX,YY);
      FindLinePoint(L2,XX,YY,aHB.A2b,idx,Ex);
      if idx < 0 then exit;
      if GIndexSetting then
         L2.GIndex := 9;
      //kontrola jestli na opacnem koncu neni uzel ....
      GiveNodeBranchForLine(L2,not aHB.A2b,N,NB);
      isNode := (N <> nil) and (N.BranchCount > 1);
      // ruseni sour
      if aHB.A2b then
      begin
         if IsNode then
            if not Ex or (idx = L2.LastIndex) then
               Dec(idx);
         for ii := idx downto 0 do
         L2.FreeElementIndex(ii);
         for ii := L1.LastIndex downto 0 do
         begin
            L1.GetOrientedCoord(ii,aHB.A1b,XX,YY);
            L2.InsertXY(0,XX,YY);
         end;
      end
      else
      begin
         if IsNode then
            if not Ex or (idx = 0) then
               Inc(idx);
         for ii := L2.LastIndex downto idx do
            L2.FreeElementIndex(ii);
         for ii := 0 to L1.LastIndex do
         begin
            L1.GetOrientedCoord(ii,not aHB.A1b,XX,YY);
            L2.AddXY(XX,YY);
         end;
      end;
      L2.RecalculateRect;
      L2.RecalculateLabelProps := true;
   end;

end;

{ THBridge }

function THBridge.IsKomplementaryElement(Value: TElement): boolean;
begin
   result := false;
   if Value = nil then exit;
   result := (Value = A1) or (Value = A2);
end;

procedure THBridge.SetA1(const Value: TElement);
begin
   ExchangeFieldElement(FA1, Value);
end;

procedure THBridge.SetA1b(const Value: boolean);
begin
  FA1b := Value;
end;

procedure THBridge.SetA1Index(const Value: integer);
begin
  FA1Index := Value;
end;

procedure THBridge.SetA1Index1(const Value: integer);
begin
  FA1Index1 := Value;
end;

procedure THBridge.SetA2(const Value: TElement);
begin
   ExchangeFieldElement(FA2, Value);
end;

procedure THBridge.SetA2b(const Value: boolean);
begin
  FA2b := Value;
end;

procedure THBridge.SetA2Index(const Value: integer);
begin
  FA2Index := Value;
end;

procedure THBridge.SetA2Index1(const Value: integer);
begin
  FA2Index1 := Value;
end;

procedure THBridge.SetDone(const Value: boolean);
begin
  FDone := Value;
end;

procedure THBridge.SetSameDirection(const Value: boolean);
begin
  FSameDirection := Value;
end;

INITIALIZATION
   RegisterOrbisClass(TJoinSimpleLines);
   RegisterOrbisClass(TJoinTopolLines);
   RegisterOrbisClass(TSynchronizeLinesByGravity);
END.

