unit t_LineDialog;

interface
uses
   f_LineDialog,
   t_OrbisDialog,
   t_Snap;

type

TSimpleLineDialog = class(TOrbisDialog)
private
    FSnapTool: TPointLineSnapTool;
    FFreeToDelete: boolean;
    FFreeToChangeGeometry: boolean;
    FAllowSwapDirection: boolean;
    procedure SetSnapTool(const Value: TPointLineSnapTool);
public
    procedure SetFreeToDelete(const Value: boolean); virtual;
    procedure SetFreeToChangeGeometry(const Value: boolean); virtual;
    procedure SetAllowSwapDirection(const Value: boolean); virtual;
published
   property  SnapTool: TPointLineSnapTool read FSnapTool write SetSnapTool;
   property  FreeToDelete: boolean read FFreeToDelete write SetFreeToDelete;
   property  FreeToChangeGeometry: boolean read FFreeToChangeGeometry write SetFreeToChangeGeometry;
   property  AllowSwapDirection: boolean read FAllowSwapDirection write SetAllowSwapDirection;
end;

TOnUserLineDialogEvent = procedure (Karto: boolean; Length: double; CoordCount: integer; var Snap,SnapToLine,SnapToLinePoint,SnapToLineBody: boolean; var SnapDistance: double; DruhyList: string; var Druh,Barva,Znacka,Presnost,aresult: integer) of object;


TLineDialog = class(TSimpleLineDialog)
private
    FOnUserLineDialog: TOnUserLineDialogEvent;
    function GetForm: TLineDialog_Form;
    procedure SetOnUserLineDialog(const Value: TOnUserLineDialogEvent);
public
   procedure SetFreeToDelete(const Value: boolean); override;
   procedure SetFreeToChangeGeometry(const Value: boolean); override;
   procedure SetAllowSwapDirection(const Value: boolean); override;
   procedure DoIt; override;
   procedure CreateForm; override;
   property  Form: TLineDialog_Form read GetForm;
   property  OnUserLineDialog: TOnUserLineDialogEvent read FOnUserLineDialog write SetOnUserLineDialog;
end;


implementation
uses
   Forms,
   SysUtils,
   e_Class,
   e_Geometry,
   e_GeometryElement;
   //e_Space;

{ TLineDialog }

procedure TLineDialog.CreateForm;
begin
   aForm := TLineDialog_Form.Create(Application);
   aForm.Tool := Self;
   aForm.IsPossibleToClose := false;
end;

procedure TLineDialog.DoIt;
var
   _Druh,_Barva,_Znacka,_Presnost,_CrdCount,_Result: integer;
   _Snap,_SnapToLine,_SnapToLinePoint,_SnapToLineBody: boolean;
   _SnapDistance,_Length: double;
   isUserD: boolean;

   Lin: TPolyLineElement;
   //S: string;
begin
   if input = nil then exit;
   if (Input is TGeometryElement) then
   begin
      if not (TGeometryElement(Input).Geometry is TPolyLineElement) then
         exit
      else
         Lin :=TPolyLineElement(TGeometryElement(Input).Geometry);
   end
   else
   begin
      if not(Input is TPolyLineElement) then exit;
      Lin := TPolyLineElement(Input);
   end;
   Form.l_length.Caption := FormatFloat('0.000;;Neni co mit',Lin.Length);
   Form.l_Count.Caption := IntToStr(Lin.Count);
   {
   if AllowSwapDirection then
   begin
      Form.b_Close.Caption := 'Otoit';
   end;
   }
   isUserD := Assigned(OnUserLineDialog);
   _Snap := false;
   _SnapToLine := false;
   _SnapToLinePoint := false;
   _SnapToLineBody := false;
   _SnapDistance := 0;
   if SnapTool <> nil then
   begin
      if isUserD  then
      begin
         _Snap := true;
         _SnapToLine := SnapTool.SnapToLine;
         _SnapToLinePoint := SnapTool.SnapToLinePoint;
         _SnapToLineBody := SnapTool.SnapToLineBody;
         _SnapDistance := SnapTool.SnapDistance;
      end
      else
   begin
      Form.cb_SnapToLine.Checked := SnapTool.SnapToLine;
      Form.cb_SnapToLinePoint.CHecked := SnapTool.SnapToLinePoint;
      Form.cb_SnapToLineBody.Checked := SnapTool.SnapToLineBody;
      Form.ed_SnapDistance.Text := SnapTool.GetPropertyStringByName('SnapDistance');
      Form.CHeckSnap;
   end;
   end;
   if isUserD then
   begin
      _Druh := 0;
      _Barva := 0;
      _Znacka := 0;
      _Presnost := 0;
      _Length := Lin.Length;
      _CrdCount := Lin.Count;
      OnUserLineDialog(true,_Length,_CrdCount,_Snap,_SnapToLine,_SnapToLinePoint,_SnapToLineBody,_SnapDistance,
                      '',_Druh,_Barva,_Znacka,_Presnost,_Result);
      Result := _Result;
   end
   else
   begin
   inherited;
   end;
   if SnapTool <> nil then
   begin
      if isUserD then
      begin
         if SnapTool <> nil then
         begin
            SnapTool.SnapToLine := _SnapToLine;
            SnapTool.SnapToLinePoint := _SnapToLinePoint;
            SnapTool.SnapToLineBody := _SnapToLineBody;
         end;
      end
      else
   begin
      SnapTool.SnapToLine := Form.cb_SnapToLine.Checked;
      SnapTool.SnapToLinePoint := Form.cb_SnapToLinePoint.CHecked;
      SnapTool.SnapToLineBody := Form.cb_SnapToLineBody.Checked;
      SnapTool.SetPropertyStringByName('SnapDistance',Form.ed_SnapDistance.Text);
   end;
end;
end;

function TLineDialog.GetForm: TLineDialog_Form;
begin
   result := TLineDialog_Form(aForm);
end;

procedure TLineDialog.SetFreeToDelete(const Value: boolean);
begin
   FFreeToDelete := Value;
   Form.b_DeleteLine.Visible := Value;
end;

procedure TLineDialog.SetOnUserLineDialog(const Value: TOnUserLineDialogEvent);
begin
  FOnUserLineDialog := Value;
end;

procedure TLineDialog.SetFreeToChangeGeometry(const Value: boolean);
begin
   FFreeToChangeGeometry := Value;
   Form.b_DeletePoint.Visible := Value;
   Form.b_Close.Visible := Value;
   Form.b_Swap.Visible := Value;
   Form.b_Continue.Visible := Value;
   Form.b_CreateCopy.Visible := not Value and FreeToDelete;
end;


procedure TLineDialog.SetAllowSwapDirection(const Value: boolean);
begin
  FAllowSwapDirection := Value;
end;

{ TSimpleLineDialog }

procedure TSimpleLineDialog.SetAllowSwapDirection(const Value: boolean);
begin
  FAllowSwapDirection := Value;
end;

procedure TSimpleLineDialog.SetFreeToDelete(const Value: boolean);
begin
   FFreeToDelete := Value;
end;

procedure TSimpleLineDialog.SetFreeToChangeGeometry(const Value: boolean);
begin
   FFreeToChangeGeometry := Value;
end;

procedure TSimpleLineDialog.SetSnapTool(const Value: TPointLineSnapTool);
begin
   FSnapTool := Value;
end;

INITIALIZATION
   RegisterOrbisClass(TSimpleLineDialog);
   RegisterOrbisClass(TLineDialog);
END.
