unit t_Snap;

interface
uses
   Elements,
   e_QTree,
   Tools;

type

TSnapTool = class (TTool)
  private
    FLastDistance: double;
    FSnapStatus: integer;
    procedure SetLastDistance(const Value: double);
    procedure SetSnapStatus(const Value: integer);
private
   FSnapped: boolean;
   FX: double;
   FSnapX: double;
   FSnapY: double;
   FSnapDistance: double;
   FY: double;
   procedure SetSnapDistance(const Value: double);
   procedure SetSnapped(const Value: boolean);
   procedure SetSnapX(const Value: double);
   procedure SetSnapY(const Value: double);
   procedure SetX(const Value: double);
   procedure SetY(const Value: double);
   property  LastDistance: double read FLastDistance write SetLastDistance;
published
   property SnapDistance: double read FSnapDistance write SetSnapDistance;
   property Snapped: boolean read FSnapped write SetSnapped;
   property X: double read FX write SetX;
   property Y: double read FY write SetY;
   property SnapX: double read FSnapX write SetSnapX;
   property SnapY: double read FSnapY write SetSnapY;
   property SnapStatus: integer read FSnapStatus write SetSnapStatus;
end;



TPointLineSnapTool = class(TSnapTool)
  private
    FLastAngle: double;
    FSnapToLinePoint: boolean;
    FSnapToLine: boolean;
    FSnapToLineBody: boolean;
    FSnapToPoint: boolean;
    FSnapLines: TQuickEntity;
    FQTWalker: TQtreeWalker;
    FFrameQT: TRectEntityQTree;
    FTabuElement: TElement;
    FSnapQT: TRectEntityQTree;
    FSnapIndex: integer;
    FSnapInAngle: boolean;
    FAngleTolerance: double;
    FXa: double;
    FAngle: double;
    FYa: double;
    FNotEnds: boolean;
    FWasEnd: boolean;
    procedure SetLastAngle(const Value: double);
    procedure SetSnapToLineBody(const Value: boolean);
    procedure SetSnapToLinePoint(const Value: boolean);
    procedure SetSnapToPoint(const Value: boolean);
    function GetSnapLines: TQuickEntity;
    function GetQTWalker: TQtreeWalker;
    procedure SetSnapToLine(const Value: boolean);
    procedure SetFrameQT(const Value: TRectEntityQTree);
    procedure SetTabuElement(const Value: TElement);
    procedure SetSnapQT(const Value: TRectEntityQTree);
    procedure SetSnapIndex(const Value: integer);
    procedure SetAngle(const Value: double);
    procedure SetAngleTolerance(const Value: double);
    procedure SetSnapInAngle(const Value: boolean);
    procedure SetXa(const Value: double);
    procedure SetYa(const Value: double);
    property  LastAngle: double read FLastAngle write SetLastAngle;
    procedure SetNotEnds(const Value: boolean);
    procedure SetWasEnd(const Value: boolean);
public
   destructor Destroy; override;
   procedure DoIt; override;
   procedure PrepareSnapInAngle;
   procedure FindLinesInFrame;
   procedure TrySnapToLinePoint;
   procedure TrySnapToLineBody;
   property  QTWalker: TQtreeWalker read GetQTWalker;
published
   property  FrameQT: TRectEntityQTree read FFrameQT write SetFrameQT;
   property  SnapQT: TRectEntityQTree read FSnapQT write SetSnapQT;
   property  TabuElement: TElement read FTabuElement write SetTabuElement;
   property  SnapLines: TQuickEntity read GetSnapLines;
   property SnapInAngle: boolean read FSnapInAngle write SetSnapInAngle;
   property Angle: double read FAngle write SetAngle;
   property AngleTolerance: double read FAngleTolerance write SetAngleTolerance;
   property Xa: double read FXa write SetXa; // koncovy bod pro uhel
   property Ya: double read FYa write SetYa;
   property SnapToPoint: boolean read FSnapToPoint write SetSnapToPoint;
   property SnapToLine: boolean read FSnapToLine write SetSnapToLine;
   property SnapToLinePoint: boolean read FSnapToLinePoint write SetSnapToLinePoint;
   property SnapToLineBody: boolean read FSnapToLineBody write SetSnapToLineBody;
   property SnapIndex: integer read FSnapIndex write SetSnapIndex;
   property NotEnds: boolean read FNotEnds write SetNotEnds;
   property WasEnd: boolean read FWasEnd write SetWasEnd;
end;

implementation
uses
   e_Class,
   e_Geometry,
   e_GeometryElement,
   g_Elems,
   MathLib;

{ TSnapTool }

procedure TSnapTool.SetLastDistance(const Value: double);
begin
   FLastDistance := Value;
end;

procedure TSnapTool.SetSnapDistance(const Value: double);
begin
   FSnapDistance := Value;
end;

procedure TSnapTool.SetSnapped(const Value: boolean);
begin
   FSnapped := Value;
end;

procedure TSnapTool.SetSnapX(const Value: double);
begin
   FSnapX := Value;
end;

procedure TSnapTool.SetSnapY(const Value: double);
begin
   FSnapY := Value;
end;

procedure TSnapTool.SetSnapStatus(const Value: integer);
begin
  FSnapStatus := Value;
end;

procedure TSnapTool.SetX(const Value: double);
begin
   FX := Value;
end;

procedure TSnapTool.SetY(const Value: double);
begin
   FY := Value;
end;

{ TPointLineSnapTool }

destructor TPointLineSnapTool.Destroy;
begin
   if FSnapLines <> nil then
   begin
      FSnapLines.Clear;
      FSnapLines.Free;
   end;
   if FQTWalker <> nil then
      FQTWalker.Free;
   inherited;
end;

procedure TPointLineSnapTool.DoIt;
begin
   Snapped := false;
   SnapStatus := 0;
   WasEnd := false;
   SnapIndex := -1;
   Output := nil;
   if FrameQT = nil then exit;
   LastDistance := SnapDistance * 2;
   LastAngle := AngleTolerance * 2;
   if not SnapToLine then exit;
   if SnapInAngle then
      PrepareSnapInAngle;
   FindLinesInFrame;
   if SnapToLinePoint then
      TrySnapToLinePoint;
   if Snapped then
   begin
      SnapStatus := 1;
      exit;
   end;
   if SnapToLineBody then
      TrySnapToLineBody;
   if Snapped then
      SnapStatus := 2;
end;

procedure TPointLineSnapTool.FindLinesInFrame;
var
   El: TElement;
   G: TElement;
begin
   SnapLines.Clear;
   if FrameQT = nil then exit;
   El := QTWalker.GetFirstElementForRectangle(FrameQT,X-SnapDistance,Y-SnapDistance,X+SnapDistance,Y+SnapDistance,1);
   while El <> nil do
   begin
      if El is TPolyLineElement then
         SnapLines.AddElement(El)
      else
      if El is TGeometryElement then
      begin
         G := TGeometryElement(El).Geometry;
         if G is TPolyLineElement then
            SnapLines.AddElement(G)
      end;
      El := QTWalker.GetNextElement;
   end;
   //jet dohledn ve snap QTree
   if SnapQT = nil then exit;
   El := QTWalker.GetFirstElementForRectangle(SnapQT,X-SnapDistance,Y-SnapDistance,X+SnapDistance,Y+SnapDistance,1);
   while El <> nil do
   begin
      if El is TPolyLineElement then
         SnapLines.AddElement(El)
      else
      if El is TGeometryElement then
      begin
         G := TGeometryElement(El).Geometry;
         if G is TPolyLineElement then
            SnapLines.AddElement(G)
      end;
      El := QTWalker.GetNextElement;
   end;
end;


function TPointLineSnapTool.GetQTWalker: TQtreeWalker;
begin
   if FQTWalker = nil then
      FQTWalker := TQTreeWalker.Create;
   result := FQTWalker;
end;

function TPointLineSnapTool.GetSnapLines: TQuickEntity;
begin
   if FSnapLines = nil then
      FSnapLines := TQuickEntity.Create;
   result := FSnapLines;
end;

procedure TPointLineSnapTool.PrepareSnapInAngle;
var
   A,B: TDoubleCoord;
begin
   A.X := X;
   A.Y := Y;
   VratPosunutyBod(A,Angle,SnapDistance * 1.5,B);
   Xa := B.X;
   Ya := B.Y;
end;

procedure TPointLineSnapTool.SetAngle(const Value: double);
begin
  FAngle := Value;
end;

procedure TPointLineSnapTool.SetAngleTolerance(const Value: double);
begin
  FAngleTolerance := Value;
end;

procedure TPointLineSnapTool.SetFrameQT(const Value: TRectEntityQTree);
begin
   FFrameQT := Value;
end;

procedure TPointLineSnapTool.SetLastAngle(const Value: double);
begin
  FLastAngle := Value;
end;

procedure TPointLineSnapTool.SetNotEnds(const Value: boolean);
begin
  FNotEnds := Value;
end;

procedure TPointLineSnapTool.SetSnapInAngle(const Value: boolean);
begin
  FSnapInAngle := Value;
end;

procedure TPointLineSnapTool.SetSnapIndex(const Value: integer);
begin
  FSnapIndex := Value;
end;

procedure TPointLineSnapTool.SetSnapQT(const Value: TRectEntityQTree);
begin
  FSnapQT := Value;
end;

procedure TPointLineSnapTool.SetSnapToLine(const Value: boolean);
begin
  FSnapToLine := Value;
end;

procedure TPointLineSnapTool.SetSnapToLineBody(const Value: boolean);
begin
  FSnapToLineBody := Value;
end;

procedure TPointLineSnapTool.SetSnapToLinePoint(const Value: boolean);
begin
  FSnapToLinePoint := Value;
end;

procedure TPointLineSnapTool.SetSnapToPoint(const Value: boolean);
begin
  FSnapToPoint := Value;
end;

procedure TPointLineSnapTool.SetTabuElement(const Value: TElement);
begin
  FTabuElement := Value;
end;

procedure TPointLineSnapTool.SetWasEnd(const Value: boolean);
begin
  FWasEnd := Value;
end;

procedure TPointLineSnapTool.SetXa(const Value: double);
begin
  FXa := Value;
end;

procedure TPointLineSnapTool.SetYa(const Value: double);
begin
  FYa := Value;
end;

procedure TPointLineSnapTool.TrySnapToLineBody;
var
   i,j: integer;
   L: TPolyLineElement;
   X1,Y1,X2,Y2,pX,pY: double;
   aDistance: double;
   IsIn: boolean;
   Rovn,U1,U2: boolean;
begin
   Snapped := false;
   if SnapLines.NoElements then exit;
   for i := 0 to SnapLines.LastIndex do
   begin
      L := TPolyLineElement(SnapLines.Elements[i]);
      if L = TabuElement then continue;
      if L.CoordsCount < 2 then continue;
      for j := 0 to L.CoordsCount - 2 do
      begin
         X1 := L.Xi[j];
         Y1 := L.Yi[j];
         X2 := L.Xi[j+1];
         Y2 := L.Yi[j+1];
         if SnapInAngle then
         begin
            if not RectanglesOverlay(X1, Y1, X2, Y2, X, Y, Xa, Ya, 0) then
               continue;
            PrusecikUsecek(X,Y,Xa,Ya,X1,Y1,X2,Y2,pX,pY,Rovn,U1,U2);
            if U1 and U2 then
            begin
               aDistance := Distance(X,Y,pX,pY);
               if (aDistance <= SnapDistance) and (aDistance < LastDistance) then
               begin
                  Snapped := True;
                  LastDistance := aDistance;
                  Output := L;
                  SnapX := pX;
                  SnapY := pY;
                  SnapIndex := j;
               end;
            end;
         end
         else
         begin
            if not RectanglesOverlay(X1, Y1, X2, Y2, X-SnapDistance, Y-SnapDistance,
                                                     X+SnapDistance, Y+SnapDistance,0) then
               continue;
            DejVztahyBod_Usecka( X, Y, X1, Y1, X2, Y2, IsIn, pX, pY, aDistance);
            if IsIn and (aDistance <= SnapDistance) and (aDistance < LastDistance) then
            begin
               Snapped := True;
               LastDistance := aDistance;
               Output := L;
               SnapX := pX;
               SnapY := pY;
               SnapIndex := j;
            end;
         end;
      end;
   end;
end;

procedure TPointLineSnapTool.TrySnapToLinePoint;
var
   i,j,i1,i2: integer;
   L: TPolyLineElement;
   X1,Y1: double;
   aDistance: double;
   anAngle: double;
   A,B,C: TDoubleCoord;
begin
   Snapped := false;
   if SnapLines.NoElements then exit;
   for i := 0 to SnapLines.LastIndex do
   begin
      L := TPolyLineElement(SnapLines.Elements[i]);
      if L = TabuElement then continue;
      if notEnds then
      begin
         if L.CoordsCount < 3 then continue;
         i1 := 1;
         i2 := L.CoordsCount - 2;
      end
      else
      begin
         i1 := 0;
         i2 := L.CoordsCount - 1;
      end;
      for j := i1 to i2 do
      begin
         X1 := L.Xi[j];
         Y1 := L.Yi[j];
         if (abs(X1-X) <= SnapDistance) and (abs(Y1-Y) <= SnapDistance) then
         begin
            aDistance := Distance(X,Y,X1,Y1);
            // podle uhlu ...
            if SnapInAngle then
            begin
               A.X := Xa;
               A.Y := Ya;
               B.X := X;
               B.Y := Y;
               C.X := X1;
               C.Y := Y1;
               anAngle := Uhel2Usecek(A,B,C);
               if (aDistance <= SnapDistance) and (anAngle <= AngleTolerance) then
               begin
                  if anAngle < LastAngle then
                  begin
                     Snapped := True;
                     LastAngle := anANgle;
                     Output := L;
                     SnapX := X1;
                     SnapY := Y1;
                     SnapIndex := j;
                     WasEnd := not NotEnds and ((j = i1) or (j = i2));
                  end;
               end;
            end
            else
            // podle vzdalenosti
            if (aDistance <= SnapDistance) and (aDistance < LastDistance) then
            begin
               Snapped := True;
               LastDistance := aDistance;
               Output := L;
               SnapX := X1;
               SnapY := Y1;
               SnapIndex := j;
               WasEnd := not NotEnds and ((j = i1) or (j = i2));
            end;
         end;
      end;
   end;
end;

INITIALIZATION
   RegisterOrbisClass(TPointLineSnapTool);
END.
