unit t_SolveErrors;

interface
uses
   Elements,
   e_Chk,
   e_Geometry,
   e_GeometryElement,
   e_Node,
   e_TopolDruhy,
   Tools;
type

TSolveErrors = class(TTool)
end;

TSolveShortLine = class(TSolveErrors)
  private
    FLines: TEntity;
    FBaseLines: TEntity;
    FBaseLin: TPolyLineCollection;
    FTestLin: TPolyLineCollection;
    FLin: TPolyLineCollection;
    FGeomLines: TPolyLineCollection;
    FNodes: TGeometryElementCollection;
    FTest: boolean;
    FNode1: TNode;
    FNode2: TNode;
    FBaseAdmin: boolean;
    FBranch1: TNodeBranch;
    FBranch2: TNodeBranch;
    FLineTestTool: TBooleanTool;
    FTestAll: boolean;
    FSwap: boolean;
    procedure SetBaseLines(const Value: TEntity);
    procedure SetLines(const Value: TEntity);
    function GetLine: TPolyLineElement;
    function GetBaseLin: TPolyLineCollection;
    function GetLin: TPolyLineCollection;
    function GetGeomLines: TPolyLineCollection;
    function GetNodes: TGeometryElementCollection;
    procedure SetTest(const Value: boolean);
    procedure SetNode1(const Value: TNode);
    procedure SetNode2(const Value: TNode);
    procedure SetBaseAdmin(const Value: boolean);
    procedure SetBranch1(const Value: TNodeBranch);
    procedure SetBranch2(const Value: TNodeBranch);
    procedure SetLineTestTool(const Value: TBooleanTool);
    function GetTestLin: TPolyLineCollection;
    procedure SetTestAll(const Value: boolean);
    procedure SetSwap(const Value: boolean);
public
   procedure DoIt; override;
   procedure ClearOutputs;
   procedure FindLines;
   procedure MakeNodes;
   procedure Finish;
   property  Line: TPolyLineElement read GetLine;
   property  Node1: TNode read FNode1 write SetNode1;
   property  Node2: TNode read FNode2 write SetNode2;
   property  Branch1: TNodeBranch read FBranch1 write SetBranch1;
   property  Branch2: TNodeBranch read FBranch2 write SetBranch2;
   property  BaseLin: TPolyLineCollection read GetBaseLin;
   property  TestLin: TPolyLineCollection read GetTestLin;
   property  Lin: TPolyLineCollection read GetLin;
   property  GeomLines: TPolyLineCollection read GetGeomLines;
   property  Nodes: TGeometryElementCollection read GetNodes;
published
   property BaseLines: TEntity read FBaseLines write SetBaseLines;
   property Lines: TEntity read FLines write SetLines;
   property Test: boolean read FTest write SetTest;
   property BaseAdmin: boolean read FBaseAdmin write SetBaseAdmin;
   property LineTestTool: TBooleanTool read FLineTestTool write SetLineTestTool;
   property TestAll: boolean read FTestAll write SetTestAll;
   property Swap: boolean read FSwap write SetSwap;
end;

TSolveCriticalLinePointDistances =class(TSolveErrors)
  private
    FTolerance: double;
    FRefLines: TEntity;
    FKillErrorElements: boolean;
    FShowMessages: boolean;
    procedure SetTolerance(const Value: double);
    procedure SetRefLines(const Value: TEntity);
    procedure SetKillErrorElements(const Value: boolean);
    procedure SetShowMessages(const Value: boolean);
public
   procedure DoIt; override;
published
   property  Tolerance: double read FTolerance write SetTolerance;
   property  RefLines: TEntity read FRefLines write SetRefLines;
   property  KillErrorElements: boolean read FKillErrorElements write SetKillErrorElements;
   property  ShowMessages: boolean read FShowMessages write SetShowMessages;
end;

TSolveZeroLine =class(TSolveErrors)
  private
    FShowMessages: boolean;
    FKillErrorElements: boolean;
    procedure SetKillErrorElements(const Value: boolean);
    procedure SetShowMessages(const Value: boolean);
public
   procedure DoIt; override;
published
   property  KillErrorElements: boolean read FKillErrorElements write SetKillErrorElements;
   property  ShowMessages: boolean read FShowMessages write SetShowMessages;
end;

TSolveDruhBarvaZnacka = class(TSolveErrors)
  private
    FByDruh: boolean;
    FLayer: TEntity;
    FDruhy: TTopolDruhCollection;
    FKillErrorElements: boolean;
    procedure SetByDruh(const Value: boolean);
    procedure SetDruhy(const Value: TTopolDruhCollection);
    procedure SetLayer(const Value: TEntity);
    procedure SetKillErrorElements(const Value: boolean);
public
   procedure DoIt; override;
   function  SolveOneError(var ErrE: TErrorGeometryElement): boolean;
published
   property  Druhy: TTopolDruhCollection read FDruhy write SetDruhy;
   property  Layer: TEntity read FLayer write SetLayer;
   property  ByDruh: boolean read FByDruh write SetByDruh;
   property  KillErrorElements: boolean read FKillErrorElements write SetKillErrorElements;
end;

implementation
uses
   Forms,
   Dialogs,
   SysUtils,
   e_Class,
   e_QTree,
   e_TopolElements,
   g_Elems,
   mathlib,
   t_AreMak,
   t_Topol,
   t_ToTool;

{ TSolveShortLine }

procedure TSolveShortLine.ClearOutputs;
begin
   BaseLin.FreeAll;
   BaseLin.Rect.Reset;
   Lin.FreeAll;
   Lin.Rect.Reset;
   GeomLines.FreeAll;
   GeomLines.Rect.Reset;
   Nodes.FreeAll;
   Nodes.Rect.Reset;
end;

procedure TSolveShortLine.DoIt;
begin
   if input = nil then exit;
   if not (Input is TPolyLineElement) then exit;
   ClearOutputs;
   FindLines;
   MakeNodes;
   if Test then exit;
   Finish;
end;

procedure TSolveShortLine.FindLines;
var
   i: integer;
   e: TElement;
   G: TGeometry;
   Tst: boolean;
begin
   if (BaseLines <> nil) and not BaseLines.NoElements then
   for i := 0 to BaseLines.LastIndex do
   begin
      e := BaseLines.Elements[i];
      if not (e is TGeometryElement) then continue;
      G := TGeometryElement(e).Geometry;
      if G = nil then continue;
      if not G.IsOverRegion(Line.Minx-1,Line.MinY-1,Line.MaxX+1,Line.MaxY+1) then continue;
      if not (G is TPolyLineElement) then continue;
      Tst := true;
      if LineTestTool <> nil then
      begin
         LineTestTool.Input := G;
         LineTestTool.DoIt;
         Tst := LineTestTool.Result;
      end;
      if not Tst then
         TestLin.AddAndUpdateRect(G)
      else
         BaseLin.AddAndUpdateRect(G);
      GeomLines.AddAndUpdateRect(e);
   end;
   if (Lines <> nil) and not Lines.NoElements then
   for i := 0 to Lines.LastIndex do
   begin
      e := Lines.Elements[i];
      if not (e is TGeometryElement) then continue;
      G := TGeometryElement(e).Geometry;
      if G = nil then continue;
      if G.IsMyOwner(BaseLin) or G.IsMyOwner(TestLin) then
      begin
         GeomLines.AddAndUpdateRect(e);
         continue;
      end;
      if not G.IsOverRegion(Line.Minx-1,Line.MinY-1,Line.MaxX+1,Line.MaxY+1) then continue;
      if not (G is TPolyLineElement) then continue;
      Tst := true;
      if TestAll and (LineTestTool <> nil) then
      begin
         LineTestTool.Input := G;
         LineTestTool.DoIt;
         Tst := LineTestTool.Result;
      end;
      if not Tst then
         TestLin.AddAndUpdateRect(G)
      else
         Lin.AddAndUpdateRect(G);
      GeomLines.AddAndUpdateRect(e);
   end;

end;

procedure TSolveShortLine.Finish;
var
   FixNode,MoveNode: TNode;
   Branch,FixBranch,MoveBranch: TNodeBranch;
   i: integer;
   C: TCoordinate;
   G: TGeometryElement;
   CL: TClearLines;
   e: TElement;
   L: TPolyLineElement;
   procedure ClearDuplicities(En: TEntity);
   var
      ON: TOwnerEntity;
      ii: integer;
      ee: TElement;
   begin
      ON := TOwnerEntity.Create;
      for ii := En.LastIndex downto 0 do
      begin
         ee := En.Elements[ii];
         if not ee.IsMyOwner(ON) then
            ON.AddElement(ee);
         En.RemoveElement(ee);
      end;
      En.AddAllFrom := ON;
      ON.Free;
   end;
begin
   if (Node1.Name = '2') or (Node2.Name = '2') then
   begin
      if Node1.Name = '2' then
      begin
         MoveNode := Node1;
         MoveBranch := Branch1;
         FixNode := Node2;
         FixBranch := Branch2;
      end
      else
      begin
         MoveNode := Node2;
         MoveBranch := Branch2;
         FixNode := Node1;
         FixBranch := Branch1;
      end;
   end
   else
   begin
      ShowMessage('Nelze eit automaticky!');
      exit;
   end;
   Branch := MoveBranch;
   for i := 0 to MoveNode.BranchCount - 1 do
   begin
      Branch := MoveNode.GetNextBranchRight(Branch);
      C := Branch.Line.GetOrientedCoordElement(0,Branch.LineBegin);
      C.X := FixNode.X;
      C.Y := FixNode.Y;
      Branch.Line.RecalculateRect;
      Branch.Line.CalculateLabelProps;
      if Branch.Line.IsMyOwner(BaseLin) then
      begin
          Node1.Name := '0';
      end;
   end;
   // ruen nulovch lini
   ClearDuplicities(Lin);
   for i := Lin.LastIndex downto 0 do
   begin
      e := Lin.Elements[i];
      if not (e is TPolyLineElement) then continue;
      L := TPolyLineElement(e);
      if L.Length = 0 then
         L.Free;
   end;
   ClearDuplicities(BaseLin);
   for i := BaseLin.LastIndex downto 0 do
   begin
      e := BaseLin.Elements[i];
      if not (e is TPolyLineElement) then continue;
      L := TPolyLineElement(e);
      if L.Length = 0 then
         L.Free;
   end;
   //Line.Free;
   CL := TClearLines.Create;
   CL.Input := GeomLines;
   CL.DoIt;
   CL.Free;
   //Line := nil;
   FixNode.FreeElement(FixBranch);
   MoveNode.Free;
   for i := GeomLines.LastIndex downto 0 do
   begin
      G := TGeometryElement(GeomLines.Elements[i]);
      if G.NoGeometry then
         G.Free;
   end;
end;

function TSolveShortLine.GetBaseLin: TPolyLineCollection;
begin
   if FBaseLin = nil then
   begin
      FBaseLin := TPolyLineCollection.Create;
      FBaseLin.Name := 'BaseLin';
      AddElement(FBaseLin);
   end;
   result := FBaseLin;
end;

function TSolveShortLine.GetGeomLines: TPolyLineCollection;
begin
   if FGeomLines = nil then
   begin
      FGeomLines := TPolyLineCollection.Create;
      FGeomLines.Name := 'GeomLines';
      AddElement(FGeomLines);
   end;
   result := FGeomLines;
end;

function TSolveShortLine.GetLin: TPolyLineCollection;
begin
   if FLin = nil then
   begin
      FLin := TPolyLineCollection.Create;
      FLin.Name := 'Lin';
      AddElement(FLin);
   end;
   result := FLin;
end;

function TSolveShortLine.GetLine: TPolyLineElement;
begin
   result := TPolyLineElement(Input);
end;

function TSolveShortLine.GetNodes: TGeometryElementCollection;
begin
   if FNodes = nil then
   begin
      FNodes := TGeometryElementCollection.Create;
      FNodes.Name := 'Nodes';
      AddElement(FNodes);
   end;
   result := FNodes;
end;

function TSolveShortLine.GetTestLin: TPolyLineCollection;
begin
   if FTestLin = nil then
   begin
      FTestLin := TPolyLineCollection.Create;
      FTestLin.Name := 'TestLin';
      AddElement(FTestLin);
   end;
   result := FTestLin;
end;

procedure TSolveShortLine.MakeNodes;
var
  Tool: TReadLinesIntoQTree;
  DoNodes: TPutNodesIntoQTree;
//  TvorPlochy: TCreateAreasForQTree;
  NodeCon: TNodesConector;
  RusLin: TDeleteDirectRelationsToLinesInQTree;
  QTree: TSpaceLineBegEndCell;
  S: string;
  RE: TRectEntity;
  i: integer;
  G: TGeometryElement;
  E: TElement;
  Node,SwapNode: TNode;
  Branch,B1,B2: TNodeBranch;
  Angle,Angle1,Angle2: double;
  A,B,C: TDoubleCoord;
  BL1,BL2: integer;
begin
  RE := TRectEntity.Create;
  RE.AddAndUpdateRect(TestLin);
  RE.AddAndUpdateRect(BaseLin);
  RE.AddAndUpdateRect(Lin);

  S := 'Natn lini do QTREE';
  //ProgressText := S;
  Tool:= TReadLinesIntoQTree.Create;
  Tool.Input:= RE;
  Tool.Name:= S;
  Tool.DoIt;
  //Progress := 1;
  QTree:= TSpaceLineBegEndCell(Tool.Output);
  Tool.Free;

  S := 'Vytven uzl v QTREE';
  //ProgressText := S;
  DoNodes:= TPutNodesIntoQTree.Create;
  DoNodes.Name:= S;
  DoNodes.Input:= QTree;
  DoNodes.DoIt;
  DoNodes.Free;
  //Progress := 2;

  S := 'Zapojovn uzl v QTree';
  //ProgressText := S;
  NodeCon:= TNodesConector.Create;
  NodeCon.Name:= S;
  NodeCon.Input:= QTree;
  NodeCon.DeletePseudoNodes:= false; //ReadIniBoolean('EXPORT', 'RusitPseudouzly', false);
  NodeCon.JoinDifferentLines:= false; //ReadIniBoolean('EXPORT', 'RusitPseudouzlyIProRuzneDruhy', true);
  NodeCon.MaxCoordsCount:= 200; //ReadIniInteger('EXPORT', 'MaxPocetSouradnic', 100);
  NodeCon.DoIt;
  NodeCon.Free;
  //Progress := 3;

  S := 'Ruen QTree vazeb na linie';
  ProgressText := S;
  RusLin:= TDeleteDirectRelationsToLinesInQTree.Create;
  RusLin.Name:= S;
  RusLin.Input:= QTree;
  RusLin.DoIt;
  RusLin.Free;
  //Progress := 5;

  GiveNodeBranchForLine(Line,True,Node,Branch);
  Node1 := Node;
  Branch1 := Branch;
  Nodes.AddAndUpdateRect(Node1);
  GiveNodeBranchForLine(Line,False,Node,Branch);
  Node2 := Node;
  Branch2 := Branch;
  Nodes.AddAndUpdateRect(Node2);
  //Node2.Name := '2';
  QTree.Free;
  RE.Free;
  BL1 := 0;
  BL2 := 0;
  Node1.Name := '1';
  Branch := Branch1;
  //zjitn, kter z uzl le v baselines - kd 0
  //pokud obsahuje pouze lines, tak kd 1
  //Uzel obsahujici TestLin ma kod 3 ... nebude se hybat ani omylem
  //if not BaseAdmin then
  for i := 0 to Node1.BranchCount - 1 do
  begin
     Branch := Node1.GetNextBranchRight(Branch);
     if Branch.Line.IsMyOwner(TestLin) then
     begin
        Node1.Name := '3';
        break;
     end;
     if Branch.Line.IsMyOwner(BaseLin) then
     begin
         Node1.Name := '0';
         inc(BL1);
     end;
  end;
  Node2.Name := '1';
  Branch := Branch2;
  //if not BaseAdmin then
  for i := 0 to Node2.BranchCount - 1 do
  begin
     Branch := Node2.GetNextBranchRight(Branch);
     if Branch.Line.IsMyOwner(TestLin) then
     begin
        Node2.Name := '3';
        break;
     end;
     if Branch.Line.IsMyOwner(BaseLin) then
     begin
         Node2.Name := '0';
         inc(BL2);
     end;
  end;
  Node := nil;
  // uren pohyblivosti uzlu - kd 2
  // - Oba uzly obsahuji Testlin ... nic se nebude hybat
  if (Node1.Name = '3') and (Node2.Name = '3') then
  begin
     Swap := false;
     exit;
  end;

  // 0) Admin mod uzly s 0 se posunuji rozhoduje BL1 a BL2
  if BaseAdmin  then
  begin
     if Node1.Name = '0' then      // uzel s nulou se bude automaticky posunovat
        Node1.Name := '1';
     if Node2.Name = '0' then
        Node2.Name := '1';
  end;
  if (Node1.Name = '0') and (Node2.Name = '0') then
  begin
  end
  else
  // je ance, e alespo jeden z uzl je pohybliv
  begin
     if Node1.Name = '1' then
        Node := Node1;
     if Node2.Name = '1' then
     begin
        if Node <> nil then
        begin
           if (BL1 > 0) or (BL2 > 0) then  // bude se rozhodovat podle Baselines
           begin
              if BL2 < BL1 then   // vice Baselines ... to je jasny
                 Node := Node2
              else
              if (BL1 = 2) and (BL2 = 2) then  // a tady se bude resit uhel
              begin
                 // Node1
                 B1 := Branch1;
                 While not B1.Line.IsMyOwner(BaseLin) do
                    B1 := Node.GetNextBranchRight(B1);
                 Branch := Node.GetNextBranchRight(B1);
                 While not Branch.Line.IsMyOwner(BaseLin) do
                    Branch := Node.GetNextBranchRight(Branch);
                 B.X := Node.X;
                 B.Y := Node.Y;
                 B1.Line.GetOrientedCoord(1, B1.LineBegin, A.X, A.Y);
                 Branch.Line.GetOrientedCoord(1, Branch.LineBegin, C.X, C.Y);
                 Angle := Uhel2Usecek(A,B,C);
                 Angle1 := 2*PI - Angle;
                 if Angle < Angle1 then
                    ANgle1 := Angle;
                 // Node2
                 B2 := Branch2;
                 While not B2.Line.IsMyOwner(BaseLin) do
                    B2 := Node2.GetNextBranchRight(B2);
                 Branch := Node2.GetNextBranchRight(B2);
                 While not Branch.Line.IsMyOwner(BaseLin) do
                    Branch := Node2.GetNextBranchRight(Branch);
                 B.X := Node2.X;
                 B.Y := Node2.Y;
                 B2.Line.GetOrientedCoord(1, B2.LineBegin, A.X, A.Y);
                 Branch.Line.GetOrientedCoord(1, Branch.LineBegin, C.X, C.Y);
                 Angle := Uhel2Usecek(A,B,C);
                 Angle2 := 2*PI - Angle;
                 if Angle < Angle2 then
                    Angle2 := Angle;
                 if Angle2 > Angle1 then
                    Node := Node2;
              end;
           end
           else
           if Node2.BranchCount < Node.BranchCount then
              Node := Node2
           else
           // speco ppad - oba uzly maj jen dv linie, rozhoduje hel (men,samo)
           if (Node2.BranchCount = 2) and (Node.BranchCount = 2) then
           begin
              // Node1
              Branch := Node.GetNextBranchRight(Branch1);
              B.X := Node.X;
              B.Y := Node.Y;
              Branch1.Line.GetOrientedCoord(1, Branch1.LineBegin, A.X, A.Y);
              Branch.Line.GetOrientedCoord(1, Branch.LineBegin, C.X, C.Y);
              Angle := Uhel2Usecek(A,B,C);
              Angle1 := 2*PI - Angle;
              if Angle < Angle1 then
                 ANgle1 := Angle;
              // Node2
              Branch := Node2.GetNextBranchRight(Branch2);
              B.X := Node2.X;
              B.Y := Node2.Y;
              Branch2.Line.GetOrientedCoord(1, Branch2.LineBegin, A.X, A.Y);
              Branch.Line.GetOrientedCoord(1, Branch.LineBegin, C.X, C.Y);
              Angle := Uhel2Usecek(A,B,C);
              Angle2 := 2*PI - Angle;
              if Angle < Angle2 then
                 Angle2 := Angle;
              if Angle2 > Angle1 then
                 Node := Node2;
           end;
        end
        else
           Node := Node2;
     end;
  end;
  if Node <> nil then
  begin
     Node.Name := '2';
     if Swap then           // pokus o prehozeni uzlu
     begin
        if Node = Node1 then
           SwapNode := Node2
        else
           SwapNode := Node1;
        if SwapNode.Name = '1' then
        begin
           SwapNode.Name := '2';
           Node.Name := '1';
        end
        else
        begin
           ShowMessage('Nelze zmnit pohybliv uzel!');
           Swap := false;
           Test := true;
        end;
     end;
  end
  else
     Swap := false;
end;

procedure TSolveShortLine.SetBaseAdmin(const Value: boolean);
begin
  FBaseAdmin := Value;
end;

procedure TSolveShortLine.SetBaseLines(const Value: TEntity);
begin
  FBaseLines := Value;
end;

procedure TSolveShortLine.SetBranch1(const Value: TNodeBranch);
begin
  FBranch1 := Value;
end;

procedure TSolveShortLine.SetBranch2(const Value: TNodeBranch);
begin
  FBranch2 := Value;
end;

procedure TSolveShortLine.SetLines(const Value: TEntity);
begin
  FLines := Value;
end;


procedure TSolveShortLine.SetLineTestTool(const Value: TBooleanTool);
begin
   ExchangeFieldElement(TElement(FLineTestTool), Value);
end;

procedure TSolveShortLine.SetNode1(const Value: TNode);
begin
  FNode1 := Value;
end;

procedure TSolveShortLine.SetNode2(const Value: TNode);
begin
  FNode2 := Value;
end;

procedure TSolveShortLine.SetSwap(const Value: boolean);
begin
  FSwap := Value;
end;

procedure TSolveShortLine.SetTest(const Value: boolean);
begin
  FTest := Value;
end;

procedure TSolveShortLine.SetTestAll(const Value: boolean);
begin
  FTestAll := Value;
end;

{ TSolveCriticalLinePointDistances }

procedure TSolveCriticalLinePointDistances.DoIt;
var
   L: TPolyLineElement;
   i: integer;
   e: TElement;
   en: TEntity;
   Er: TErrorGeometryElement;
begin
   if Input = nil then exit;
   if Tolerance <= 0 then exit;
   if not (Input is TPolyLineElement) then exit;
   L := TPolyLineElement(Input);
   if L.CoordsCount < 3 then // tady bych asi neml co vyhodit
   begin
      if ShowMessages then
         ShowMessage('Nelze opravit ... Krtk linie !');
      exit;
   end;
   if (RefLines <> nil) and (L.FindClassWithOwnerInOwners(TTopolLineElement,RefLines) <> nil) then
   begin
      if ShowMessages then
         ShowMessage('Nelze opravit ... je soust ref.vrstvy: '+RefLines.Name);
      exit;
   end;
   for i := L.LastIndex-1 downto 1 do
   begin
      if InTolerance(L.Xi[i],L.Yi[i],L.Xi[i+1],L.Yi[i+1],Tolerance) then
         L.FreeElement(L.Elements[i]);
   end;
   if L.Count > 2 then  // jet prvn seka
      if InTolerance(L.Xi[0],L.Yi[0],L.Xi[1],L.Yi[1],Tolerance) then
         L.FreeElement(L.Elements[1]);
   if KillErrorElements then
   for i := L.OwnersCount - 1 downto 0 do
   begin
      en := L.Owners[i];
      if en is TErrorGeometryElement then
         en.Free;
   end;
end;

procedure TSolveCriticalLinePointDistances.SetKillErrorElements(
  const Value: boolean);
begin
  FKillErrorElements := Value;
end;

procedure TSolveCriticalLinePointDistances.SetRefLines(
  const Value: TEntity);
begin
  FRefLines := Value;
end;

procedure TSolveCriticalLinePointDistances.SetShowMessages(
  const Value: boolean);
begin
  FShowMessages := Value;
end;

procedure TSolveCriticalLinePointDistances.SetTolerance(
  const Value: double);
begin
  FTolerance := Value;
end;

{ TSolveZeroLine }

procedure TSolveZeroLine.DoIt;
var
   EE: TErrorGeometryElement;
   L: TPolyLineElement;
begin
   EE := nil;
   L := nil;
   if Input = nil then exit;
   if (Input is TErrorGeometryElement) then
   begin
      EE := TErrorGeometryElement(Input);
      if EE.NoGeometry then exit;
      if not (EE.Geometry is TPolyLineElement) then exit;
      L := TPolyLineElement(EE.Geometry);
   end
   else
   begin
      if not (Input is TPolyLineElement) then exit;
      L := TPolyLineElement(Input);
   end;
   if (L.Count < 2) or (L.Length = 0) then
   begin
      L.Free;
      if  KillErrorElements and (EE <> nil) then
         EE.Free;
   end
   else
   begin
      if ShowMessages then
         ShowMessage('Ruit je mon jen linie s nulovou dlkou a podobnmi chylkami!');
   end;
end;



procedure TSolveZeroLine.SetKillErrorElements(const Value: boolean);
begin
  FKillErrorElements := Value;
end;

procedure TSolveZeroLine.SetShowMessages(const Value: boolean);
begin
  FShowMessages := Value;
end;

{ TSolveDruhBarvaZnacka }

procedure TSolveDruhBarvaZnacka.DoIt;
var
   ie: TEntity;
   i: integer;
   ee: TErrorGeometryElement;
   e: TElement;
   ni,oi: integer;

begin
   if Input = nil then exit;
   if not (Input is TEntity) then exit;
   ie := TEntity(Input);
   if ie.NoElements then exit;
   if Druhy = nil then exit;
   if Layer = nil then exit;
   StartProgress('Oprava Druhu podle barvy a Znaky...',0,ie.LastIndex);
   ni := 0;
   oi := 0;
   for i := ie.LastIndex downto 0 do
   begin
      e := ie.Elements[i];
      if not (e is TErrorGeometryElement) then continue;
      ee := TErrorGeometryElement(e);
      if not SolveOneError(ee) then
      begin
         Inc(ni);
      end
      else
      begin
         Inc(oi);
      end;
      Progress := i;
      ProgressText := IntToStr(oi)+' objekt opraveno, '+IntToStr(ni)+' nelze opravit automaticky';
      Application.ProcessMessages;
   end;
end;

procedure TSolveDruhBarvaZnacka.SetByDruh(const Value: boolean);
begin
  FByDruh := Value;
end;

procedure TSolveDruhBarvaZnacka.SetDruhy(
  const Value: TTopolDruhCollection);
begin
  FDruhy := Value;
end;

procedure TSolveDruhBarvaZnacka.SetKillErrorElements(const Value: boolean);
begin
  FKillErrorElements := Value;
end;

procedure TSolveDruhBarvaZnacka.SetLayer(const Value: TEntity);
begin
  FLayer := Value;
end;

function TSolveDruhBarvaZnacka.SolveOneError(var ErrE: TErrorGeometryElement): boolean;
var
   GE: TGeometryElement;
   G: TGeometry;
   TD,TD1: TTopolDruh;
   e: TElement;
   aBarva, aZnacka, aDruh,i: integer;
begin
   result := false;
   G := ErrE.Geometry;
   if G = nil then exit;
   GE := TGeometryElement(G.FindClassWithOwnerInOwners(TGeometryElement,Layer));
   if GE = nil then exit;
   aDruh := GE.GetPropertyLongIntByName('Druh');
   aBarva := GE.GetPropertyLongIntByName('Barva');
   aZnacka := GE.GetPropertyLongIntByName('Znacka');
   // hladanie druhu
   TD := nil;
   TD1 := nil;
   for i := 0 to Druhy.LastIndex do
   begin
      e := Druhy.Elements[i];
      if not (e is TTopolDruh) then exit;
      TD := TTopolDruh(e);
      if (TD.Barva = aBarva) and (TD.Znacka = aZnacka) then
      begin
         if TD1 <> nil then exit; // vic druhu pro kombinaci .. najde opravit samo
         TD1 := TD;
      end;
   end;
   if TD1 <> nil then
   begin
      GE.SetPropertyLongIntByName('Druh',TD1.KOD);
      ErrE.Free;
   end;
end;

INITIALIZATION
   RegisterOrbisClass(TSolveShortLine);
   RegisterOrbisClass(TSolveCriticalLinePointDistances);
   RegisterOrbisClass(TSolveZeroLine);
   RegisterOrbisClass(TSolveDruhBarvaZnacka);
END.

