unit E_layscr;

interface
uses
   // KERNEL
   Elements,
   E_Type,
   // GLIB
   g_Elems,
   // DEVICE
   e_Proje;

type


TLayerScript = class(TRectEntity)
private
  FDescription: string;
  FProperties: TPropertyList;
  procedure SetGoIntoDirs(aValue: boolean);
  function  GetGoIntoDirs: boolean;
  procedure DeleteAllPaths;
public
  procedure FillWildNames(aDataDir: string);
  procedure AddProperty(aStringElement: TStringElement);
  function AddPropertyByName(aName: string): TStringElement;
  function GetProperties: TPropertyList;
  function GetIntegerProp(aPropName: string): longInt;
  procedure SetIntegerProp(aPropName: string; aValue: longInt);
  function GetDoubleProp(aPropName: string): double;
  procedure SetDoubleProp(aPropName: string; aValue: double);
  procedure FreeAll; override;
published
  property Description: string read FDescription write FDescription;
  property GoIntoDirs: boolean read GetGoIntoDirs write SetGoIntoDirs;
end;


TModelScript = class(TRectEntity)
private
  FLayerScript: TLayerScript;
public
  procedure AddLayerScript(aType, aFunction: string);
  procedure AddLayerProp(aName, aValue: string);
  procedure AddLayerOrParameter(aParameterName, aHodnota: string);
  procedure FillWildNames(aDataDir: string);
end;




function OKLayerType(aType: string): boolean;

function LayerTypeForExt(anExt: string): string;

function TopolLayer(anExt: string): boolean;


implementation
uses
  Classes,
  Graphics,
  SysUtils,
  // KERNEL
  e_Class,
  U_String,
  U_File;




function OKLayerType(aType: string): boolean;
begin
  result:=  (aType = 'LIN') or (aType = 'ARE') or
            (aType = 'PNT') or (aType = 'TXT') or
            (aType = 'LINSYM') or
            (aType = 'ELEMENT') or
            (aType = 'RAS') or
            (aType = 'TAB')
end;



function LayerTypeForExt(anExt: string): string;
begin
  if anExt = '.ARE' then result:= 'ARE'
  else
  if anExt = '.LIN' then result:= 'LIN'
  else
  if anExt = '.PNT' then result:= 'PNT'
  else
  if anExt = '.TBX' then result:= 'TXT'
  else
  if anExt = '.TTX' then result:= 'TXT'
  else
  if anExt = '.CRD' then result:= 'LIN'
  else
  if anExt = '.RAS' then result:= 'RAS'
  else
  if anExt = '.RAK' then result:= 'RAS'
  else
  if anExt = '.TIF' then result:= 'RAS'
  else
  if anExt = '.CIT' then result:= 'RAS'
  else
  if anExt = '.BMP' then result:= 'RAS'
  else
  if anExt = '.SHP' then result:= 'ARE'
  else
  if anExt = '.DBF' then result:= 'TAB'
  else
  if anExt = '.DB' then result:= 'TAB'
  else
    result:= 'FILE'
end;


function TopolLayer(anExt: string): boolean;
begin
  result:= (anExt = '.ARE') or (anExt = '.LIN') or (anExt = '.PNT') or
           (anExt = '.TBX') or (anExt = '.TTX') or (anExt = '.CRD')
end;


       { TLayerScript }

procedure TLayerScript.SetGoIntoDirs(aValue: boolean);
begin
  if aValue and (Description[1] <> '+') then
     Description:= '+' + Description
  else
  if not aValue and (Description[1] = '+') then
     Description:= copy(Description, 2, 255)
end;


function  TLayerScript.GetGoIntoDirs: boolean;
begin
  result:= Description[1] = '+'
end;


function TLayerScript.GetIntegerProp(aPropName: string): longInt;
begin
  result := 0;
  if aPropName = '' then exit;
  if FProperties = nil then exit;
  result:= FProperties.GetIntegerByName(aPropName)
end;


function TLayerScript.GetDoubleProp(aPropName: string): double;
begin
  result := 0;
  if aPropName = '' then exit;
  if FProperties = nil then exit;
  result:= FProperties.GetDoubleByName(aPropName)
end;

procedure TLayerScript.SetIntegerProp(aPropName: string; aValue: longInt);
begin
  if (FProperties = nil) then
     AddPropertyByName(aPropName);
  FProperties.SetIntegerByName(aPropName, aValue)
end;

procedure TLayerScript.SetDoubleProp(aPropName: string; aValue: double);
begin
  if (FProperties = nil) then
     AddPropertyByName(aPropName);
  FProperties.SetDoubleByName(aPropName, aValue)
end;

procedure TLayerScript.DeleteAllPaths;
var
  I: longInt;
  Path: TElement;
begin
  for I:= count-1 downTo 0 do
  begin
    Path:= Elements[I];
    if Path is TPathElement then
       FreeElement(Path)
  end;
end;


function AbsolutePath(aPath: string): boolean;
begin
  result := false;
  if aPath = '' then exit;
  Result:= (Pos(':', aPath) > 0) or (aPath[1] = '\')
end;


procedure TLayerScript.FillWildNames(aDataDir: string);
var
  J: longInt;
  aPath, OldPath, OldName: string;
  SubPathes: TStringList;
  Attr: word;
  S: string;
begin
  if Name = 'ELEMENT' then exit;

  DeleteAllPaths;

  if Description = '' then exit;
  S := UpperCase(ExtractFileExt(Description));

  // prava ... adrese se vyhledvaj pouze pro BLK, jinak soubory (S)
  if (S = '.BLK') and ((Name = 'LIN') or (Name = 'PNT') or (Name = 'ARE') or (Name = 'TXT') or (Name = 'LINSYM')) then
     Attr:= faDirectory
  else
//     Attr:= faArchive;
     Attr:= faAnyFile-faDirectory;
  { bude se rozvinovat ? }
  if GoIntoDirs then
  begin
    aPath:= copy(Description, 2, 255);
    OldPath:= ExtractFilePath(aPath);
    if OldPath = '' then OldPath:= aDataDir;
    if not AbsolutePath(OldPath) then
       OldPath:= AddBackSlash(aDataDir) + OldPath;
    OldName:= ExtractFileName(aPath);

    SubPathes:= GetFilesInSubDirs(OldPath, OldName, Attr);
    if SubPathes <> nil then
    begin
      for J:= 0 to SubPathes.Count-1 do
      begin
        InsertElement(0, TPathElement.CreateAll('PATH', SubPathes.Strings[J]))
      end
    end
  end
  else
  begin
    OldPath:= ExtractFilePath(Description);
    OldName:= ExtractFileName(Description);

    if AbsolutePath(OldPath) then {absolutni adresa}
    begin
      InsertElement(0, TPathElement.CreateAll('PATH', Description))
    end
    else
    if Pos('*', OldName) > 0 then
    begin
      if OldPath = '' then OldPath:= aDataDir;
      if not AbsolutePath(OldPath) then
         OldPath:= AddBackSlash(aDataDir) + OldPath;

      SubPathes:= GetFilesInDir(OldPath, OldName, Attr);
      if SubPathes <> nil then
      begin
        for J:= 0 to SubPathes.Count-1 do
        begin
          InsertElement(0, TPathElement.CreateAll('PATH', SubPathes.Strings[J]))
        end
      end
    end
    else
    { varianta bez prav }
      if OldPath = '' then
      begin
        aPath:= AddBackSlash(aDataDir) + OldName;
        InsertElement(0, TPathElement.CreateAll('PATH', aPath))
      end
      else
      begin
        aPath:= AddBackSlash(aDataDir) + Description;
        InsertElement(0, TPathElement.CreateAll('PATH', aPath))
      end
  end
end;




procedure TLayerScript.AddProperty(aStringElement: TStringElement);
begin
  if FProperties = nil then
  begin
     FProperties:= TPropertyList.Create;
     AddElement(FProperties)
  end;

  if FProperties = nil then exit;
  FProperties.AddElement(aStringElement)
end;


function TLayerScript.AddPropertyByName(aName: string): TStringElement;
var
  SE: TStringElement;
begin
  SE:= TStringElement.CreateAll(aName, '');
  AddProperty(SE);
  result:= SE
end;



function TLayerScript.GetProperties: TPropertyList;
begin
  result:= FProperties;
end;



procedure TLayerScript.FreeAll;
begin
  inherited FreeAll;
  FProperties:= nil;
  FDescription:= ''
end;


       { TModelScript }


procedure TModelScript.AddLayerScript(aType, aFunction: string);
begin
  FLayerScript:= TLayerScript.Create;
  FLayerScript.Name:= aType;
  FLayerScript.Description:= aFunction;
  AddElement(FLayerScript);
end;


procedure TModelScript.AddLayerProp(aName, aValue: string);
begin
  if FLayerScript = nil then exit;

  if FLayerScript <> nil then
  begin
    FLayerScript.AddProperty(TStringElement.CreateAll(aName, aValue))
  end
end;



procedure TModelScript.AddLayerOrParameter(aParameterName, aHodnota: string);
begin
  if (aParameterName = 'LIN') or (aParameterName = 'ARE') or
     (aParameterName = 'PNT') or (aParameterName = 'TXT') or
     (aParameterName = 'LINSYM') or
     (aParameterName = 'ELEMENT') or
     (aParameterName = 'RAS') then
       AddLayerScript(aParameterName, aHodnota)
  else
  if aParameterName[1] in ['@','!','<','>','+','-'] then
       AddLayerProp(aParameterName, aHodnota)
end;





procedure TModelScript.FillWildNames(aDataDir: string);
var
  I: longInt;
begin
  for I:= 0 to Count-1 do
  { cyklus pes vechny vrstvy }
  begin
    if Elements[I] is TLayerScript then
       TLayerScript(Elements[I]).FillWildNames(aDataDir)
  end;
end;


BEGIN
   RegisterOrbisClass(TLayerScript);
   RegisterOrbisClass(TModelScript);
END.


