unit G_elems;
{
*********************************************************************
*                                                                   *
*  ORBIS ... Tdy grafickch, t.j. souadnicov zvislch objekt  *
*                                                                   *
*                       Topol Pro 1996                             *
*                                                                   *
*********************************************************************
[Unit_Description]
Description=Zkladn tdy souadnicov zvislch element
Admin=Olda
TechTopic=Graphics Elements
Topic=Grafika
Version=1.0

[Edit]
 28.4.1998 9:00=Registrace
}
INTERFACE
uses
   Classes,
   Graphics,
// KERNEL
   Elements;

const
  gcIniMax = 1E30;
  gcMinDouble = 1E-20;

{ RectEntity Allign constants }
{-----------------------------}
  ra_TOP   = 'TOP';
  ra_DOWN  = 'DOWN';
  ra_LEFT  = 'LEFT';
  ra_Right = 'RIGHT';
  ra_Center= 'CENTER';
  ra_OTHER = 'OTHER';

type



TTextAlignment = (taLeft, ta);

TDoubleCoord = record
   X,Y: double;
end;

TDoubleRect = record
   case Integer of
      0: (MinX, MinY, MaxX, MaxY: Double);
      1: (Min,Max: TDoubleCoord);
   end;
{end;}


TCoordinate = class(TElement)
 protected
   FCS: byte; { Coordinate system }
   function  GetX: double; virtual;
   function  GetY: double; virtual;
   function  GetZ: double; virtual;
   procedure SetX(aX: double); virtual;
   procedure SetY(aY: double); virtual;
   procedure SetZ(aZ: double); virtual;
 public
   procedure  Load(aReader: TReader; aIndex: TEntity); override;
   procedure  Save(aWriter: TWriter); override;
   function   CreateDuplicate: TCoordinate; virtual;
   function   CreateEnvelope: TElement; override;
   function   GetRegion: TDoubleRect; virtual;
   property   Z: double read GetZ write SetZ;
 published
   property CS: byte read FCS write FCS;
   property X: double read GetX write SetX;
   property Y: double read GetY write SetY;
end;

T2DCoordinate = class(TCoordinate)
 private
   FX: double;
   FY: double;
 protected
   function  GetX: double; override;
   function  GetY: double; override;
   procedure SetX(aX: double); override;
   procedure SetY(aY: double); override;
 public
   procedure  Load(aReader: TReader; aIndex: TEntity); override;
   procedure  Save(aWriter: TWriter); override;
end;


T3DCoordinate = class(T2DCoordinate)
 private
   FZ: double;
 protected
   function  GetZ: double; override;
   procedure SetZ(aZ: double); override;
 public
   procedure  Load(aReader: TReader; aIndex: TEntity); override;
   procedure  Save(aWriter: TWriter); override;
 published
   property Z;
end;


TCoordinateList = class(TSuperList);



TRectangle = class(TElement)
private
   FMin: TCoordinate;
   FMax: TCoordinate;
   function  GetMin: TCoordinate;
   function  GetMax: TCoordinate;
   function  GetHeight: double;
   function  GetWidth : double;
   function  GetMinX: double;
   function  GetMinY: double;
   function  GetMaxX: double;
   function  GetMaxY: double;
   procedure SetMin(Crd: TCoordinate);
   procedure SetMax(Crd: TCoordinate);
   procedure SetHeight(Val: double);
   procedure SetWidth(Val: double);
   procedure  SetMinX(aValue: double);
   procedure  SetMinY(aValue: double);
   procedure  SetMaxX(aValue: double);
   procedure  SetMaxY(aValue: double);

   function   GetCenter: TDoubleCoord;
    function GetDoubleRect: TDoubleRect;
protected
   function   GetInstanceSize: longInt; override;
public
   constructor Create; override;
   constructor CreateDoubleRect(DoubleRect: TDoubleRect);
   destructor  Destroy; override;
   procedure   Reset;
   procedure   Load(aReader: TReader; aIndex: TEntity); override;
   procedure   Save(aWriter: TWriter); override;
   function    CreateEnvelope: TElement; override;
   procedure   Assign(Source: TPersistent); override;
   procedure   Sort;
   procedure   UpdateByXY(X, Y: double);
   procedure   UpdateByCoordinate(Crd: TCoordinate);
   procedure   UpdateByRectangle(aRect: TRectangle);
   procedure   UpdateByDoubleRect(aDRect: TDoubleRect);
   function    IsValid: boolean;
   function    Dimension: double;
   procedure   Grow(const DeltaX, DeltaY: double);
   procedure   GrowbyKoef(const aKoef: double);
   procedure   Move(const DeltaX, DeltaY: double);
   function    CreateDuplicate: TRectangle; virtual;
   function    AsDoubleRect: TDoubleRect;
   function    IsOverlap(const aRect: TRectangle): boolean;

   property Min: TCoordinate read GetMin write SetMin;
   property Max: TCoordinate read GetMax write SetMax;
   property CenterCoord: TDoubleCoord read GetCenter;
published
   property MinX: double   read GetMinX   write SetMinX;
   property MinY: double   read GetMinY   write SetMinY;
   property MaxX: double   read GetMaxX   write SetMaxX;
   property MaxY: double   read GetMaxY   write SetMaxY;
   property Height: double read GetHeight write SetHeight;
   property Width: double  read GetWidth  write SetWidth;
   property DoubleRect: TDoubleRect read GetDoubleRect;
end;



{ !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
TCoordinateEntity = class(TOwnerEntity)
protected
  FCoordinate: TCoordinate

end;
}

TRefPosition = 0..9;


TRectEntity  = class(TOwnerEntity)
 private
   procedure DoDeltaX(aValue: double);
   procedure DoDeltaY(aValue: double);
   procedure PUpdateNoReset(aBool: boolean);
   function  GetIsValid: boolean;
 protected
{              0 nedefinovno
   Ref posice: 0 default, u RectEntity odpovd 1
          3   6   9
          2   5   8
          1   4   7
}
   FRect  : TRectangle;
   FIndex : longInt;
   FRefPosition: TCoordinate;
   function  GetInstanceSize: longInt; override;
   function GetMinX: double;
   function GetMinY: double;
   function GetMaxX: double;
   function GetMaxY: double;
   function GetCenterX: double;
   function GetCenterY: double;
   function GetHeight: double;
   function GetWidth : double;
   function GetRefX: double;
   function GetRefY: double;
   function GetRelRefX: double;
   function GetRelRefY: double;
   function GetDeltaRefX: double;
   function GetDeltaRefY: double;

   function GetHorizAllign: string;
   function GetVertAllign: string;

   procedure SetMinX(aValue: double); virtual;
   procedure SetMinY(aValue: double); virtual;
   procedure SetMaxX(aValue: double);
   procedure SetMaxY(aValue: double);
   procedure SetHeight(Val: double);
   procedure SetWidth(Val: double);
   procedure SetRefX(aValue: double);
   procedure SetRefY(aValue: double);
   procedure SetRefPosition(aValue: TRefPosition); virtual;
   function  GetRefPosition: TRefPosition;

   procedure SetRelRefX(aValue: double);
   procedure SetRelRefY(aValue: double);
   procedure SetDeltaRefX(aValue: double);
   procedure SetDeltaRefY(aValue: double);
   procedure SetAbsRefX(aValue: double);
   procedure SetAbsRefY(aValue: double);

   procedure SetHorizAllign(const aValue: string);
   procedure SetVertAllign(const aValue: string);


 public
   destructor  Destroy; override;
   function   GetRect: TRectangle; virtual;
   procedure  SetRect(aRect: TRectangle); virtual;
   function   GetRefCoordinate: TCoordinate; virtual;
   procedure  SetRefCoordinate(aValue: TCoordinate); virtual;

   procedure   Assign(Source: TPersistent); override;
   procedure   Load(aReader: TReader; aIndex: TEntity); override;
   procedure   SetByDoubleRect(aRect: TDoubleRect);
   procedure   Save(aWriter: TWriter); override;
   function    CreateEnvelope: TElement; override;
   procedure   UpdateRect(aElement: TElement);
{   procedure   AddElement(aElement: TElement); override; }
   procedure   AddAndUpdateRect(aElement: TElement);
   function    IsInsideRegion(Minx, MinY, MaxX, MaxY: double): Boolean;
   function    RegionIsInside(Minx, MinY, MaxX, MaxY: double): Boolean;
   function    IsOverRegion(Minx, MinY, MaxX, MaxY: double): Boolean;
   function    IsOverRegionEx(Minx, MinY, MaxX, MaxY: double): Boolean; virtual;
   function    PointIsInside(X, Y: double): Boolean;
   function    IsHit(CoordEl: TCoordinate; Tolerance: double): boolean; virtual;
   function    IsHitByXY(X, Y: double; Tolerance: double): boolean; virtual;
   function    InterSectByRect(const aRect: TDoubleRect): TDoubleRect;
   procedure   Update; virtual;
   procedure   ResetAndUpdate; virtual;

   procedure   Move(const DeltaX, DeltaY: double);
   function    GetMaxLength: double;
   procedure   SetRelRefCoords(aX, aY: double);

   property Rect: TRectangle       read GetRect       write SetRect;
   property RefCoords: TCoordinate read GetRefCoordinate  write SetRefCoordinate;

 published
   property Index: longInt            read Findex         write FIndex;
   property MinX: double              read GetMinX        write SetMinX;
   property MinY: double              read GetMinY        write SetMinY;
   property MaxX: double              read GetMaxX        write SetMaxX       stored false;
   property MaxY: double              read GetMaxY        write SetMaxY       stored false;
   property IsValid: Boolean          read GetIsValid                         stored False;
   property Width : double            read GetWidth       write SetWidth;
   property Height: double            read GetHeight      write SetHeight;
   property CenterX: double           read GetCenterX                         stored false;
   property CenterY: double           read GetCenterY                         stored false;
   property X: double                 read GetRefX        write SetRefX       stored false;
   property Y: double                 read GetRefY        write SetRefY       stored false;

   property RelRefX: double           read GetRelRefX     write SetRelRefX    stored false;
   property RelRefY: double           read GetRelRefY     write SetRelRefY    stored false;

   property DeltaRefX: double         read GetDeltaRefX   write SetDeltaRefX  stored false;
   property DeltaRefY: double         read GetDeltaRefY   write SetDeltaRefY  stored false;

   property AbsRefX: double           read GetRefX     write SetAbsRefX    stored true;
   property AbsRefY: double           read GetRefY     write SetAbsRefY    stored true;

   property HorizAllign: string       read GetHorizAllign write SetHorizAllign stored false;
   property VertAllign: string        read GetVertAllign  write SetVertAllign  stored false;

   property RefPosition: TRefPosition read GetRefPosition write SetRefPosition stored false;
   property DeltaX: double                                write DoDeltaX       stored false;
   property DeltaY: double                                write DoDeltaY       stored false;
   property UpdateNoReset: boolean                        write PUpdateNoReset stored false;
   property UpdateByElement: TElement                     write UpdateRect     stored false;
end;


TRectEntityClass = class of TRectEntity;


TSelectedElement = class(TRectEntity)
public
   Sel: TRectEntity;
   function GetSelName: string;
published
   property SelectName: string read GetSelName stored false;
end;


TDrawRectangle = class(TRectEntity)
private
  FPenWidth: double;
  FPenColor: TColor;
  FPenStyle: TPenStyle;
  FBrushColor: TColor;
  FBrushStyle: TBrushStyle;

  procedure SetPen(aPen: TPen);
  procedure SetBrush(aBrush: TBrush);

public
  destructor Destroy; override;
  function  CreateEnvelope: TElement; override;
  property  Pen: TPen write SetPen;
  property  Brush: TBrush write SetBrush;
published
  property PenColor: TColor read FPenColor write FPenColor;
  property PenStyle: TPenStyle read FPenStyle write FPenStyle;
  property PenWidth: double read FPenWidth write FPenWidth;
  property BrushColor: TColor read FBrushColor write FBrushColor;
  property BrushStyle: TBrushStyle read FBrushStyle write FBrushStyle;
end;




TText = class(TRectEntity)
 private
   FText: String;
   FOrientation: double;
   FColor   : TColor;
   FFontName: String;
   FFontStyle: TFontStyles;
   FCharSet: TFontCharSet;
   function  GetText: string;
   procedure SetText(const aText: string);
   function  GetFontName: string;
   procedure SetFontName(aValue: string);
   procedure SetFont(aFont: TFont);
public
   procedure   Load(aReader: TReader; aIndex: TEntity); override;
   procedure   Save(aWriter: TWriter); override;
   function    CreateEnvelope: TElement; override;
   procedure   Update; override;
   property Font: TFont  write SetFont;
published
   property Text       : string read GetText write SetText;
   property Orientation: double read FOrientation write FOrientation;
   property Color      : TColor read FColor write FColor;
   property FontName   : string read GetFontName write SetFontName;
   property FontStyle  : TFontStyles read FFontStyle write FFontStyle;
   property Charset    : TFontCharSet read FCharSet write FCharSet;
end;




TTextBlok = class(TText)
end;

{O.S. 2.9.98
TLine = class(TRectEntity)
end;


TArea = class(TRectEntity)
end;




TLineString  = class(TLine)
end;
}


IMPLEMENTATION

uses
   SysUtils,
// KERNEL
   e_Class,
   u_String;

{ TCoordinate }


procedure    TCoordinate.Load(aReader: TReader; aIndex: TEntity);
begin
   inherited Load(aReader, aIndex);
   CS := aReader.ReadInteger;
end;


procedure    TCoordinate.Save(aWriter: TWriter);
begin
   inherited Save(aWriter);
   aWriter.WriteInteger(CS);
end;



function  TCoordinate.GetX: double;
begin
   result := -1;
end;

function  TCoordinate.GetY: double;
begin
   result := -1;
end;

function  TCoordinate.GetZ: double;
begin
   result := -1;
end;

procedure TCoordinate.SetX(aX: double);
begin
end;

procedure TCoordinate.SetY(aY: double);
begin
end;

procedure TCoordinate.SetZ(aZ: double);
begin
end;

function TCoordinate.CreateDuplicate: TCoordinate;
begin
  Result:= TCoordinate(TElementClass(Self.ClassType).Create);
  Result.X:= Self.X;
  Result.Y:= Self.Y;
  Result.Z:= Self.Z;
  Result.CS:= Self.CS
end;


function TCoordinate.CreateEnvelope: TElement;
begin
   result := inherited CreateEnvelope;
   TCoordinate(result).x := X;
   TCoordinate(result).y := Y;
   TCoordinate(result).z := Z;
end;


function TCoordinate.GetRegion: TDoubleRect;
begin
   result.Min.x := X;
   result.Max.x := X;
   result.Min.y := Y;
   result.Max.y := Y;
end;




{ T2DCoordinate ---------------------------------------------------------------}


procedure    T2DCoordinate.Load(aReader: TReader; aIndex: TEntity);
begin
   inherited Load(aReader, aIndex);
   aReader.Read(FX, SizeOf(Double)+SizeOf(Double));
end;


procedure    T2DCoordinate.Save(aWriter: TWriter);
begin
   inherited Save(aWriter);
   aWriter.Write(FX, SizeOf(Double)+SizeOf(Double));

end;


function T2DCoordinate.GetX: double;
begin
   Result := FX;
end;

function T2DCoordinate.GetY: double;
begin
   Result := FY;
end;


procedure T2DCoordinate.SetX(aX: double);
begin
   FX := aX;
end;

procedure T2DCoordinate.SetY(aY: double);
begin
   FY := aY;
end;


{ T3DCoordinate ---------------------------------------------------------------}

procedure    T3DCoordinate.Load(aReader: TReader; aIndex: TEntity);
begin
   inherited Load(aReader, aIndex);
   FZ := aReader.ReadFloat;
end;


procedure    T3DCoordinate.Save(aWriter: TWriter);
begin
   inherited Save(aWriter);
   aWriter.WriteFloat(FZ);
end;


function T3DCoordinate.GetZ: double;
begin
   Result := FZ;
end;

procedure T3DCoordinate.SetZ(aZ: double);
begin
   FZ := aZ;
end;



{ TRectangle ------------------------------------------------------------------}


function    TRectangle.GetCenter: TDoubleCoord;
begin
   result.x := MinX + ((MaxX - MinX) / 2);
   result.y := MinY + ((MaxY - MinY) / 2);
end;


procedure   TRectangle.Load(aReader: TReader; aIndex: TEntity);
begin
   inherited Load(aReader, aIndex);
   MinX := aReader.ReadFloat;
   MinY := aReader.ReadFloat;
   MaxX := aReader.ReadFloat;
   MaxY := aReader.ReadFloat;
end;


procedure   TRectangle.Save(aWriter: TWriter);
begin
   inherited Save(aWriter);
   aWriter.WriteFloat(MinX);
   aWriter.WriteFloat(MinY);
   aWriter.WriteFloat(MaxX);
   aWriter.WriteFloat(MaxY);
end;


constructor TRectangle.Create;
begin
  inherited Create;
  FMin:= T2DCoordinate.Create;
  FMin.X:= gcIniMax;
  FMin.Y:= gcIniMax;
  FMax:= T2DCoordinate.Create;
  FMax.X:= -gcIniMax;
  FMax.Y:= -gcIniMax;
end;


function TRectangle.CreateEnvelope: TElement;
begin
   result := CreateDuplicate;
end;


constructor TRectangle.CreateDoubleRect(DoubleRect: TDoubleRect);
begin
  inherited Create;
  MinX:= DoubleRect.MinX;
  MinY:= DoubleRect.MinY;
  MaxX:= DoubleRect.MaxX;
  MaxY:= DoubleRect.MaxY;
end;


destructor  TRectangle.Destroy;
begin
  FMin.Free;
  FMax.Free;
  inherited Destroy
end;

procedure   TRectangle.Reset;
begin
  Min.X:= 1E30;
  Min.Y:= 1E30;
  Max.X:= -1E30;
  Max.Y:= -1E30;
end;


procedure TRectangle.Assign(Source: TPersistent);
var
  aRect: TRectangle;
begin
   if Source is TRectangle then
   begin
     aRect:= TRectangle(Source);
     Min.X := aRect.Min.X;
     Min.Y := aRect.Min.Y;
     Max.X := aRect.Max.X;
     Max.Y := aRect.Max.Y;
   end;
end;


function   TRectangle.GetInstanceSize: longInt;
begin
  result:= inherited GetInstanceSize;
  if FMin <> nil then
     result:= result + FMin.Size;
  if FMax <> nil then
     result:= result + FMax.Size
end;



procedure TRectangle.Sort;
var
  Pom: double;

begin
  if (FMin <> nil) and (FMax <> nil) then
  begin
    if FMin.X > FMax.X then
    begin
      Pom:= FMin.X; FMin.X:= FMax.X; FMax.X:= Pom
    end;

    if FMin.Y > FMax.Y then
    begin
      Pom:= FMin.Y; FMin.Y:= FMax.Y; FMax.Y:= Pom
    end
  end
end;


function  TRectangle.GetMin: TCoordinate;
begin
   if not assigned(FMin) then FMin := T2DCoordinate.Create;
   result := FMin;
end;

function  TRectangle.GetMax: TCoordinate;
begin
   if not assigned(FMax) then FMax := T2DCoordinate.Create;
   result := FMax;
end;





procedure TRectangle.SetMin(Crd: TCoordinate);
begin
  if assigned(FMin) then FMin.Free;
  FMin := nil;
  FMin := Crd;
  Sort
end;


procedure TRectangle.SetMax(Crd: TCoordinate);
begin
  if assigned(FMax) then FMax.Free;
  FMax := nil;
  FMax := Crd;
  Sort
end;




function TRectangle.CreateDuplicate: TRectangle;
begin
  Result:= TRectangle(TElementClass(Self.ClassType).Create);
  Result.MinX:= Self.MinX;
  Result.MinY:= Self.MinY;
  Result.MaxX:= Self.MaxX;
  Result.MaxY:= Self.MaxY;
{  Result.Max:= Self.Max.CreateDuplicate; }
end;


function TRectangle.AsDoubleRect: TDoubleRect;
begin
   result.Min.x := Min.x - 10;
   result.Min.y := Min.y - 10;
   result.Max.x := Max.x + 10;
   result.Max.y := Max.y + 10;
end;


procedure TRectangle.UpdateByXY(X, Y: double);
begin
  if X < FMin.X then FMin.X:= X;
  if X > FMax.X then FMax.X:= X;

  if Y < FMin.Y then FMin.Y:= Y;
  if Y > FMax.Y then FMax.Y:= Y;
end;


procedure   TRectangle.UpdateByDoubleRect(aDRect: TDoubleRect);
begin
  UpdateByXY(aDRect.MinX, aDRect.MinY);
  UpdateByXY(aDRect.MaxX, aDRect.MaxY);
end;



procedure TRectangle.UpdateByCoordinate(Crd: TCoordinate);
begin
  UpdateByXY(Crd.X, Crd.Y);
end;

procedure TRectangle.UpdateByRectangle(aRect: TRectangle);
begin
   if not aRect.isValid then exit;
   UpdateByCoordinate(aRect.Min);
   UpdateByCoordinate(aRect.Max);
end;

procedure TRectangle.SetHeight(Val: double);
begin
   Max.y := Min.Y + Val;
end;

procedure TRectangle.SetWidth(Val: double);
begin
   Max.X := Min.X + Val;
end;

function  TRectangle.GetHeight: double;
begin
   Result:= Max.Y - Min.Y
end;

function  TRectangle.GetWidth : double;
begin
   Result:= Max.X - Min.X
end;



function TRectangle.IsValid: boolean;
begin
   Result:= (Max.X >= Min.X) and (Max.Y >= Min.Y) and
                         (abs(Min.X) < gcIniMax) and
                         (abs(Min.Y) < gcIniMax) and
                         (abs(Max.X) < gcIniMax) and
                         (abs(Max.Y) < gcIniMax)
end;


function TRectangle.Dimension: double;
begin
   if Height > Width then Result:= Height
                     else Result:= Width;
end;


procedure TRectangle.Grow(const DeltaX, DeltaY: double);
var
  aWidth, aHeight: double;
begin
  aWidth:= Width + DeltaX;
  aHeight:= Height + DeltaY;
  if aWidth < 0 then aWidth := 0;
  if aHeight < 0 then aHeight := 0;
  Width  := aWidth;
  Height := aHeight;
  Move(DeltaX/-2, DeltaY/-2);
end;


procedure TRectangle.GrowByKoef(const aKoef: double);
var
  NW, NH: double;
begin
  try
    NW:= Width * aKoef;
    NH:= Height * aKoef;
    Grow((NW-Width)/2, (NH-Height)/2)
  except
  end
end;


procedure TRectangle.Move(const DeltaX, DeltaY: double);
begin
   Min.x := Min.x + DeltaX;
   Min.Y := Min.y + DeltaY;
   Max.x := Max.x + DeltaX;
   Max.Y := Max.y + DeltaY;
end;





function   TRectangle.GetMinX: double;
begin
   result := Min.x;
end;

function   TRectangle.GetMinY: double;
begin
   result := Min.y;
end;

function   TRectangle.GetMaxX: double;
begin
   result := Max.x;
end;

function   TRectangle.GetMaxY: double;
begin
   result := Max.y;
end;

procedure  TRectangle.SetMinX(aValue: double);
begin
   Min.X := aValue;
end;

procedure  TRectangle.SetMinY(aValue: double);
begin
   Min.Y := aValue;
end;

procedure  TRectangle.SetMaxX(aValue: double);
begin
   Max.X := aValue;
end;

procedure  TRectangle.SetMaxY(aValue: double);
begin
   Max.Y := aValue;
end;




function TRectangle.IsOverlap(const aRect: TRectangle): boolean;
begin
   result := false;
   if (MinX > aRect.Max.X) or (MaxX < aRect.Min.x) or
      (MinY > aRect.Max.Y) or (MaxY < aRect.Min.y) then exit;
   result := true;
end;




{ TDrawRectangle -----------------------------------------------------------------}


destructor TDrawRectangle.Destroy;
begin
  inherited Destroy;
end;



function TDrawRectangle.CreateEnvelope: TElement;
begin
  result := inherited CreateEnvelope;
  TDrawRectangle(result).PenWidth:= PenWidth;
  TDrawRectangle(result).PenColor:= PenColor;
  TDrawRectangle(result).PenStyle:= PenStyle;
  TDrawRectangle(result).BrushColor:= BrushColor;
  TDrawRectangle(result).BrushStyle:= BrushStyle;
end;


procedure TDrawRectangle.SetPen(aPen: TPen);
begin
  PenColor:= aPen.Color;
  PenStyle:= aPen.Style;
end;


procedure TDrawRectangle.SetBrush(aBrush: TBrush);
begin
  BrushColor:= aBrush.Color;
  BrushStyle:= aBrush.Style;
end;


{ TLineString -----------------------------------------------------------------}


{ TRectEntity }

procedure  TRectEntity.Load(aReader: TReader; aIndex: TEntity);
begin
  inherited Load(aReader, aIndex);
  Rect.MinX := aReader.ReadFloat;
  Rect.MinY := aReader.ReadFloat;
  Rect.MaxX := aReader.ReadFloat;
  Rect.MaxY := aReader.ReadFloat;
end;


procedure  TRectEntity.Save(aWriter: TWriter);
begin
   inherited Save(aWriter);
   aWriter.WriteFloat(Rect.MinX);
   aWriter.WriteFloat(Rect.MinY);
   aWriter.WriteFloat(Rect.MaxX);
   aWriter.WriteFloat(Rect.MaxY);
end;




procedure   TRectEntity.Assign(Source: TPersistent);
var
  RectEnt: TRectEntity;
begin
  if Source is TRectEntity then
  begin
    RectEnt:= TRectEntity(Source);
    Index:= RectEnt.Index;
    Rect:= RectEnt.Rect.CreateDuplicate;
  end;
end;



destructor  TRectEntity.Destroy;
var
   R: TElement;
   RP: TElement;
begin
   R := FRect;
   RP:= FRefPosition;
   {
   if FRect <> nil then   // kontrola kvli nslednmu sdlen pozinch objekt
      FRect.Free;
   if FRefPosition <> nil then
      FRefPosition.Free;
   }
   inherited Destroy;
   if R <> nil then   // kontrola kvli nslednmu sdlen pozinch objekt
      R.Free;
   if RP <> nil then
      RP.Free;
end;



function   TRectEntity.GetInstanceSize: longInt;
begin
  result:= inherited GetInstanceSize;
  if FRect <> nil then
     result:= result + FRect.Size;
  if FRefPosition <> nil then
     result:= result + FrefPosition.Size;
end;


procedure   TRectEntity.SetRelRefCoords(aX, aY: double);
var
   dX, dY: double;
begin
   if (RefCoords.x = aX) and (RefCoords.y = aY) then exit;
   dx := DeltaRefX;
   dy := DeltaRefY;
   RefCoords.x := aX;
   RefCoords.Y := aY;
   Move(dx - DeltaRefX, dY - DeltaRefY);
end;



function  TRectEntity.GetHorizAllign: string;
begin
   result:= ra_OTHER;
   if RefCoords.x = 0   then result := ra_LEFT
   else
   if RefCoords.x = 0.5 then result := ra_CENTER
   else
   if RefCoords.x = 1   then result := ra_RIGHT;
end;

function  TRectEntity.GetVertAllign: string;
begin
   result:= ra_OTHER;
   if RefCoords.y = 0   then result := ra_DOWN
   else
   if RefCoords.y = 0.5 then result := ra_CENTER
   else
   if RefCoords.y = 1   then result := ra_TOP;
end;


procedure TRectEntity.SetHorizAllign(const aValue: string);
begin
   if UpperCase(aValue) = ra_LEFT then RelRefX := 0
   else
   if UpperCase(aValue) = ra_CENTER then RelRefX := 0.5
   else
   if UpperCase(aValue) = ra_RIGHT then RelRefX := 1;
end;


procedure TRectEntity.SetVertAllign(const aValue: string);
begin
   if UpperCase(aValue) = ra_DOWN   then RelRefY := 0
   else
   if UpperCase(aValue) = ra_CENTER then RelRefY := 0.5
   else
   if UpperCase(aValue) = ra_TOP    then RelRefY := 1;
end;


function   TRectEntity.GetRefCoordinate: TCoordinate;
begin
   if FRefPosition = nil then FRefPosition:= T2DCoordinate.Create;
   result:= FRefPosition;
end;


procedure   TRectEntity.SetRefCoordinate(aValue: TCoordinate);
begin
   if FRefPosition <> nil then FrefPosition.Free;
   FRefPosition := aValue;
   if FRefPosition <> nil then SetRelRefCoords(FRefPosition.x, FRefPosition.y);
end;

function  TRectEntity.GetRect: TRectangle;
begin
   if not assigned(FRect) then FRect := TRectangle.Create;
   result := FRect;
end;

procedure TRectEntity.SetRect(aRect: TRectangle);
begin
   if Assigned(FRect) then FRect.Free;
   FRect := nil;
   FRect := aRect;
end;

function TRectEntity.GetMinX: double;
begin
  Result:= Rect.MinX
end;

function TRectEntity.GetMinY: double;
begin
  Result:= Rect.MinY
end;

function TRectEntity.GetMaxX: double;
begin
  Result:= Rect.MaxX
end;

function TRectEntity.GetMaxY: double;
begin
  Result:= Rect.MaxY
end;

function TRectEntity.GetCenterX: double;
begin
   result:= Rect.CenterCoord.x;
end;

function TRectEntity.GetCenterY: double;
begin
   result:= Rect.CenterCoord.y;
end;




function   TRectEntity.GetRefX: double;
begin
   result := RefCoords.x * Width + MinX;
end;

function   TRectEntity.GetRefY: double;
begin
   result := RefCoords.y * Height + MinY;
end;


function TRectEntity.GetRelRefX: double;
begin
   result := RefCoords.x;
end;

function TRectEntity.GetRelRefY: double;
begin
   result := RefCoords.y;
end;

function TRectEntity.GetDeltaRefX: double;
begin
   result := RefCoords.x * Width;
end;

function TRectEntity.GetDeltaRefY: double;
begin
   result := RefCoords.y * Height;
end;



procedure TRectEntity.SetRelRefX(aValue: double);
var
   dX: double;
begin
   if RefCoords.x = aValue then exit;
   dx := DeltaRefX;
   RefCoords.x := aValue;
   Move(dx - DeltaRefX, 0);
end;

procedure TRectEntity.SetRelRefY(aValue: double);
var
   dY: double;
begin
   if RefCoords.Y = aValue then exit;
   dy := DeltaRefY;
   RefCoords.Y := aValue;
   Move(0, dY - DeltaRefY);
end;

procedure TRectEntity.SetDeltaRefX(aValue: double);
begin
   if (aValue = 0) or (Width < gcMinDouble) then RelRefX := 0
   else RelRefX := aValue / Width;
end;

procedure TRectEntity.SetDeltaRefY(aValue: double);
begin
   if (aValue = 0)  or (Width < gcMinDouble) then RelRefY := 0
   else RelRefY := aValue / Height;
end;

procedure TRectEntity.SetAbsRefX(aValue: double);
var
   dx: double;
begin
   dx := aValue - MinX;
   if Width < gcMinDouble then
      RefCoords.x:= 0
   else
      RefCoords.x := dx / Width;
end;

procedure TRectEntity.SetAbsRefY(aValue: double);
var
   dy: double;
begin
   dY := aValue - MinY;
   if Height < gcMinDouble then
      RefCoords.y := 0
   else
      RefCoords.y := dy / Height;
end;




procedure   TRectEntity.SetByDoubleRect(aRect: TDoubleRect);
begin
  MinX:= aRect.MinX;
  MinY:= aRect.MinY;
  MaxX:= aRect.MaxX;
  MaxY:= aRect.MaxY;
end;


procedure  TRectEntity.SetMinX(aValue: double);
begin
   Rect.MinX := aValue;
end;

procedure  TRectEntity.SetMinY(aValue: double);
begin
   Rect.MinY := aValue;
end;

function TRectEntity.GetIsValid: Boolean;
begin
  Result:= Rect.IsValid;
end;

procedure  TRectEntity.SetRefX(aValue: double);
begin
   if not Rect.IsValid  then
   begin
      Minx := aValue;
      Maxx := aValue;
   end
   else Move(aValue - x, 0);
end;


procedure  TRectEntity.SetRefY(aValue: double);
begin
   if not Rect.IsValid  then
   begin
      Miny := aValue;
      Maxy := aValue;
   end
   else Move(0, aValue - y);
end;

{
          3   6   9
          2   5   8
          1   4   7
}

procedure TRectEntity.SetRefPosition(aValue: TRefPosition);
begin
   case aValue of
      1: SetRelRefCoords(0, 0);
      2: SetRelRefCoords(0, 0.5);
      3: SetRelRefCoords(0, 1);
      4: SetRelRefCoords(0.5, 0);
      5: SetRelRefCoords(0.5, 0.5);
      6: SetRelRefCoords(0.5, 1);
      7: SetRelRefCoords(1, 0);
      8: SetRelRefCoords(1, 0.5);
      9: SetRelRefCoords(0, 1);
   end;
end;


function  TRectEntity.GetRefPosition: TRefPosition;
begin
   result:= 0;
   if RefCoords.x = 0 then
   begin
      if RefCoords.y = 0 then result := 1
      else
      if RefCoords.y = 0.5 then result := 2
      else
      if RefCoords.y = 1 then result := 3;
   end
   else
   if RefCoords.x = 0.5 then
   begin
      if RefCoords.y = 0 then result := 4
      else
      if RefCoords.y = 0.5 then result := 5
      else
      if RefCoords.y = 1 then result := 6;
   end
   else
   if RefCoords.x = 1 then
   begin
      if RefCoords.y = 0 then result := 7
      else
      if RefCoords.y = 0.5 then result := 8
      else
      if RefCoords.y = 1 then result := 9;
   end;
end;


procedure  TRectEntity.SetMaxX(aValue: double);
begin
   Rect.MaxX := aValue;
end;

procedure  TRectEntity.SetMaxY(aValue: double);
begin
   Rect.MaxY := aValue;
end;

procedure TRectEntity.SetHeight(Val: double);
var
   aY: double;
begin
   aY := y;
   Rect.Height:= Val;
   SetAbsRefY (ay);
end;

procedure TRectEntity.SetWidth(Val: double);
var
   aX: double;
begin
   aX := X;
   Rect.Width:= Val;
   SetAbsRefX(aX);
end;

procedure TRectEntity.DoDeltaX(aValue: double);
begin
  Move(aValue, 0)
end;

procedure TRectEntity.DoDeltaY(aValue: double);
begin
  Move(0, aValue)
end;



function  TRectEntity.GetHeight: double;
begin
   Result:= Rect.Height;
   if result = 0 then
      result := 0.00001;
end;

function  TRectEntity.GetWidth : double;
begin
   Result:= Rect.Width;
   if result = 0 then
      result := 0.00001;
end;


procedure TRectEntity.PUpdateNoReset(aBool: boolean);
begin
  if aBool then
     Update
  else
     Rect.Reset
end;


function TRectEntity.CreateEnvelope: TElement;
begin
   result := inherited CreateEnvelope;
   TRectEntity(result).Rect.Assign(Rect);
end;


procedure TRectEntity.UpdateRect(aElement: TElement);
begin
   if not assigned(aElement) then exit;
   if aElement is TCoordinate then Rect.UpdateByCoordinate(TCoordinate(aElement)) else
   if aElement is TRectEntity then
   begin
      Rect.UpdateByRectangle(TRectEntity(aElement).Rect);
   end;
end;

{
procedure TRectEntity.AddElement(aElement: TElement);
begin
  UpdateRect(aElement);
  Inherited AddElement(aElement);
end;
}


procedure TRectEntity.AddAndUpdateRect(aElement: TElement);
begin
  UpdateRect(aElement);
  AddElement(aElement);
end;


function   TRectEntity.IsInsideRegion(Minx, MinY, MaxX, MaxY: double): Boolean;
begin
   result:= (MinX <= Rect.Min.X) and (MaxX >= Rect.Max.x) and
            (MinY <= Rect.Min.Y) and (MaxY >= Rect.Max.y)
end;

function    TRectEntity.RegionIsInside(Minx, MinY, MaxX, MaxY: double): Boolean;
begin
   result:= (MinX >= Rect.Min.X) and (MaxX <= Rect.Max.x) and
            (MinY >= Rect.Min.Y) and (MaxY <= Rect.Max.y)
end;



function   TRectEntity.IsOverRegion(Minx, MinY, MaxX, MaxY: double): Boolean;
begin
   result := false;
   if (MinX > Rect.Max.X) or (MaxX < Rect.Min.x) or
      (MinY > Rect.Max.Y) or (MaxY < Rect.Min.y) then exit;
   result := true;
end;

function    TRectEntity.PointIsInside(X, Y: double): Boolean;
begin
   result:= (X >= Rect.MinX) and (X <= Rect.MaxX) and
            (Y >= Rect.MinY) and (Y <= Rect.MaxY);
end;


function    TRectEntity.InterSectByRect(const aRect: TDoubleRect): TDoubleRect;
var
   A, B: double;
begin
   A := Rect.Min.X ;
   B := aRect.Min.X;
   if A > B then result.Min.x := A
            else result.Min.x := B;
   A := Rect.Max.X ;
   B := aRect.Max.X;
   if B > A then result.Max.x := A
            else result.Max.x := B;
   if result.Min.X >= result.Max.X then with result do
   begin
      MinX := 0;
      MinY := 0;
      MaxX := 0;
      MaxY := 0;
      exit;
   end;
   A := Rect.Min.Y ;
   B := aRect.Min.Y;
   if A > B then result.Min.y := A
            else result.Min.y := B;
   A := Rect.Max.Y ;
   B := aRect.Max.Y;
   if B > A then result.Max.y := A
            else result.Max.y := B;
   if result.Min.Y >= result.Max.Y then with result do
   begin
      MinX := 0;
      MinY := 0;
      MaxX := 0;
      MaxY := 0;
      exit;
   end;
end;


procedure TRectEntity.Update;
var
   i: longInt;
begin
   {
   Rect.Reset;
   }
   for i := 0 to Count - 1 do
   begin
      UpdateRect(Elements[i]);
   end;
end;


procedure TRectEntity.ResetAndUpdate;
begin
   Rect.Reset;
   Update;
end;


procedure TRectEntity.Move(const DeltaX, DeltaY: double);
begin
   Rect.Move(DeltaX, DeltaY);
end;


function  TRectEntity.GetMaxLength: double;
begin
   with Rect do
   begin
      result:= Max.X-Min.X;
      if Max.Y-Min.Y > result then
         result:= Max.Y-Min.Y
   end
end;



function TRectEntity.IsHit(CoordEl: TCoordinate; Tolerance: double): boolean;
begin
   result := false;
   if CoordEl = nil then exit;
   result := PointIsInside(CoordEl.X,CoordEl.Y);
end;


function TRectEntity.IsHitByXY(X, Y, Tolerance: double): boolean;
begin
   result := PointIsInside(X,Y);
end;

function TRectEntity.IsOverRegionEx(Minx, MinY, MaxX, MaxY: double): Boolean;
begin
   result := IsOverRegion(Minx, MinY, MaxX, MaxY);
end;

{ TText }


procedure  TText.Load(aReader: TReader; aIndex: TEntity);
begin
   inherited Load(aReader, aIndex);
   Text        := aReader.ReadString;
   {FFont: TFont; ... Uloit font !!!!!!!!!!!!!!!!!!!!!!!!!!!}
   Orientation := aReader.ReadFloat;
   Height      := aReader.ReadFloat;
end;


procedure  TText.Save(aWriter: TWriter);
begin
   inherited Save(aWriter);
   aWriter.WriteString(Text);
   {FFont: TFont; ... Uloit font !!!!!!!!!!!!!!!!!!!!!!!!!!!}
   aWriter.WriteFloat(Orientation);
   aWriter.WriteFloat(Height);
end;

function TText.CreateEnvelope: TElement;
begin
   result := inherited CreateEnvelope;
   TText(result).Text := Text;
   TText(result).FontName:= FontName;
   TText(result).FontStyle:= FontStyle;
   TText(result).CharSet:= CharSet;
   TText(result).Color:= Color;
   TText(result).Orientation := Orientation;
   TText(result).Height := Height;
end;


procedure  TText.Update;
begin
   Height := MaxY - MinY;
end;


procedure TText.SetFont(aFont: TFont);
begin
  FontName:= aFont.Name;
  Color   := aFont.Color;
  FontStyle:= aFont.Style;
  CharSet := aFont.Charset;
end;



function  TText.GetText: string;
begin
   result := FText;
end;

procedure TText.SetText(const aText: string);
begin
   FText := aText;
end;


function  TText.GetFontName: string;
begin
  result := FFontName;
end;

procedure TText.SetFontName(aValue: string);
begin
  FFontName := aValue;
end;



{ TSelectedElement }

function TSelectedElement.GetSelName: string;
begin
   try
      result := Sel.Name;
   except
   end;
end;



function TRectangle.GetDoubleRect: TDoubleRect;
begin
  result.MinX:= MinX;
  result.MaxX:= MaxX;
  result.MinY:= MinY;
  result.MaxY:= MaxY;
end;

initialization
   RegisterOrbisClass(T2DCoordinate);
   RegisterOrbisClass(T3DCoordinate);
   RegisterOrbisClass(TRectangle);
   RegisterOrbisClass(TDrawRectangle);
   RegisterOrbisClass(TText);
   RegisterOrbisClass(TRectEntity);
   RegisterOrbisClass(TSelectedElement);
end.


