unit E_imodel;

interface

uses
// KERNEL
   e_ElementModel,
   Elements;

type


TInterpretModel = class(TOwnerEntity)
private
   FInterpretClassName: string;
   FElementClassName: string;
    FCoverMePlease: boolean;
   function GetPropertyModels: TOwnerEntity;
   function GetSubModels: TOwnerEntity;
   function GetIsSubInterpret: boolean;
   function GetParentInterpret: TInterpretModel;
   function GetParentInterpretName: string;
   function GetNoSubModels: boolean;
   function GetNoPropertyModels: boolean;
    procedure SetCoverMePlease(const Value: boolean);
public
   property NoSubModels: boolean read GetNoSubModels;
   property NoPropertyModels: boolean read GetNoPropertyModels;
   property ParentInterpret: TInterpretModel read GetParentInterpret;
   property ParentInterpretName: string read GetParentInterpretName;
published
   property InterpretClassName: string read FInterpretClassName write FInterpretClassName;
   property ElementClassName: string read FElementClassName write FElementClassName;
   property PropertyModels: TOwnerEntity read GetPropertyModels stored false;
   property SubModels: TOwnerEntity read GetSubModels stored false;
   property IsSubInterpret: boolean read GetIsSubInterpret stored false;
   property CoverMePlease: boolean read FCoverMePlease write SetCoverMePlease;
end;

TInterpretModelEntity = class(TOwnerEntity)
private
   FActualModel: TInterpretModel;
    FVersion: integer;
   procedure SetModelToAdd(aValue: TInterpretModel);
   procedure SetModelToDelete(aValue: TInterpretModel);
   procedure SetFindIMClassName(const aValue: string);
   procedure AddModel(aModel: TInterpretModel; full: boolean);
   procedure AddSubModel(aModel: TInterpretModel);
    procedure SetVersion(const Value: integer);
public
   procedure AddProperties(FromModel, ToModel: TInterpretModel);
published
   property ActualModel: TInterpretModel read FActualModel write FActualModel stored false;
   property FindModelInterpretClassName: string write SetFindIMClassName stored false;
   property ModelToAdd: TInterpretModel write SetModelToAdd stored false;
   property ModelToDelete: TInterpretModel write SetModelToDelete stored false;
   property Version: integer read FVersion write SetVersion;
end;


implementation

uses
   {!t_Main,}
   SysUtils,
// KERNEL   
   e_Class;

{ TInterpretModel }

function TInterpretModel.GetPropertyModels: TOwnerEntity;
var
   S: string[20];
begin
   S := 'PropertyModels';
   result := TOwnerEntity(FindElementName(S));
   if result = nil then
   begin
      result := TOwnerEntity.Create;
      AddElement(result);
      result.Name := S;
   end;
end;

function TInterpretModel.GetSubModels: TOwnerEntity;
var
   S: string[20];
begin
   S := 'SubModels';
   result := TOwnerEntity(FindElementName(S));
   if result = nil then
   begin
      result := TOwnerEntity.Create;
      AddElement(result);
      result.Name := S;
   end;
end;

function TInterpretModel.GetIsSubInterpret: boolean;
begin
   result := FindOwnerName('SubModels') <> nil;
end;

function TInterpretModel.GetParentInterpret: TInterpretModel;
var
   SM: TEntity;
begin
   result := nil;
   SM := FindOwnerName('SubModels');
   if SM = nil then exit;
   result := TInterpretModel(SM.FindOwnerClass(TInterpretModel));
end;

function TInterpretModel.GetParentInterpretName: string;
var
   IM: TInterpretModel;
begin
   result := '';
   IM := ParentInterpret;
   if IM = nil then exit;
   result := IM.Name;
end;

function TInterpretModel.GetNoSubModels: boolean;
begin
   result := FindElementName('SubModels') = nil;
end;

function TInterpretModel.GetNoPropertyModels: boolean;
begin
   result := FindElementName('PropertyModels') = nil;
end;


procedure TInterpretModel.SetCoverMePlease(const Value: boolean);
begin
  FCoverMePlease := Value;
end;

{ TInterpretModelEntity }

procedure TInterpretModelEntity.AddModel(aModel: TInterpretModel; full: boolean);
var
   MI, SI: TInterpretModel;
   //PM,NPM: TPropertyModel;
   i: longInt;
begin
   MI := TInterpretModel(FindElementName(aModel.Name));
   if MI = nil then
   begin
      MI := TInterpretModel(CreateElementCopy(aModel));
      {
      MI := TInterpretModel.Create;
      MI.Name := aModel.Name;
      MI.InterpretClassName := aModel.InterpretClassName;
      MI.ElementClassName := aModel.ElementClassName;
      }
      AddProperties(aModel, MI);
      AddElement(MI);
   end;
   if not full then exit;
   if not aModel.NoSubModels then
   for i := 0 to aModel.SubModels.Count - 1 do
   begin
      SI := TInterpretModel(aMOdel.SubModels.Elements[i]);
      if not (TElement(SI) is TInterpretModel) then Continue;
      if not MI.NoSubModels then
         if MI.SubModels.FindElementName(SI.Name) <> nil then Continue;
      SetModelToAdd(SI);
   end;
end;

procedure TInterpretModelEntity.AddSubModel(aModel: TInterpretModel);
var
   MI, SI: TInterpretModel;
   //i: longInt;
begin
   if not (TElement(aModel) is TInterpretModel) then exit;
   if not aModel.IsSubInterpret then exit;
   MI := TInterpretModel(FindElementName(aModel.ParentInterpretName));
   if MI = nil then AddModel(aModel.ParentInterpret, false);
   MI := TInterpretModel(FindElementName(aModel.ParentInterpretName));
   if MI = nil then exit;
   if not MI.NoSubModels then
      if MI.Submodels.FindElementName(aModel.Name) <> nil then exit;
   SI := TInterpretModel(CreateElementCopy(aModel));
   {
   SI := TInterpretModel.Create;
   SI.Name := aModel.Name;
   SI.InterpretClassName := aModel.InterpretClassName;
   SI.ElementClassName := aModel.ElementClassName;
   }
   AddProperties(aModel, SI);
   MI.SubModels.AddElement(SI);
end;

procedure TInterpretModelEntity.AddProperties(FromModel, ToModel: TInterpretModel);
var
   PM,NPM: TPropertyModel;
   i: longInt;
   E: TElement;
begin
    if not FromModel.NoPropertyModels then
    for i := 0 to FromModel.PropertyModels.Count -1 do
    begin
       PM := TPropertyModel(FromModel.PropertyModels.Elements[i]);
       if not (TElement(PM) is TPropertyModel) then continue;
       E := ToModel.PropertyModels.FindElementName(PM.Name);
       NPM := TPropertyModel(E);
       if NPM = nil then
       begin
          NPM := TPropertyModel.Create;
          NPM.Name := PM.Name;
          NPM.Alias := PM.Alias;
          NPM.Value := PM.Value;
          ToModel.PropertyModels.AddElement(NPM);
       end;
    end;
end;

procedure TInterpretModelEntity.SetModelToAdd(aValue: TInterpretModel);
//var
   //MI, SI: TInterpretModel;
   //PM,NPM: TPropertyModel;
   //i: longInt;
begin
   if aValue= nil then exit;
   if not aValue.IsSubInterpret then AddModel(aValue, true)
                                else AddSubModel(aValue);
   FActualModel := aValue;
end;

procedure TInterpretModelEntity.SetModelToDelete(aValue: TInterpretModel);
begin
   if not aValue.IsSubInterpret then
   begin
      FreeElement(aValue);
      if Count > 0 then FActualModel := TInterpretModel(Elements[0])
                   else FActualModel := nil;
   end
   else
   begin
      FActualModel := aValue.ParentInterpret;
      FActualModel.Submodels.FreeElement(aValue);
   end;
end;

procedure TInterpretModelEntity.SetFindIMClassName(const aValue: string);
var
   i, j: longInt;
   S,SF: string;
   IM, SM: TInterpretModel;
begin
   FActualModel := nil;
   if NoElements then exit;
   S := aValue;
   for i := 0 to Count - 1 do
   begin
      IM := TInterpretModel(Elements[i]);
      SF := IM.InterpretClassName;
      if CompareText(S, SF) = 0 then
      begin
         FActualModel := IM;
         exit;
      end;
      if not IM.NoSubModels then
      for j := 0 to IM.SubModels.Count - 1 do
      begin
         SM := TInterpretModel(IM.SubModels.Elements[j]);
         SF := SM.InterpretClassName;
         if CompareText(S, SF) = 0 then
         begin
            FActualModel := SM;
            exit;
         end;
      end;
   end;
end;



procedure TInterpretModelEntity.SetVersion(const Value: integer);
begin
   FVersion := Value;
end;

INITIALIZATION
   RegisterOrbisClass(TPropertyModel);
   RegisterOrbisClass(TInterpretModel);
   RegisterOrbisClass(TInterpretModelEntity);
END.
