unit I_gener;

interface
uses
   Graphics,
   // KERNEL
   Elements,
   Tools,
   e_Pale,
   t_CmdLn,
   t_ITable,
   t_Recurs,
   // INTERPRET
   I_Basic
   ;

type

TGeneralInterpret = class(TRectInterpret)
private
   FCommandLine: TCommandLine;
   FFileName: string;
   FInsideClass: TElementClass;
   FProcessClass: TElementClass;
   FSymbolsFile: string;

   function  GetInsideClassName:  string;
   function  GetProcessClassName: string;
   function  GetCommandLine: TCommandLine;
   procedure SetFileName(aValue: string);
   procedure SetInsideClassName (aValue: string);
   procedure SetProcessClassName(aValue: string);
protected
   function  QGoInside(Element: TElement): boolean; override;
   function  QProcessBefore(Element: TElement): boolean; override;
   function  IsItsClass(AObject: TObject): boolean; override;
public
   property InsideClass: TElementClass read FInsideClass     write FInsideClass;
   property ProcessClass: TElementClass read FProcessClass   write FProcessClass;
   property CommandLine: TCommandLine read GetCommandLine;
published
   property FileName:         string read FFileName           write SetFileName;
   property SymbolsFile:      string read FSymbolsFile        write FSymbolsFile;
   property InsideClassName:  string read GetInsideClassName  write SetInsideClassName;
   property ProcessClassName: string read GetProcessClassName write SetProcessClassName;
end;


TGeneralColorInterpret = class(TGeneralInterpret)
private
   //FValColor: longInt;
   FColor: TColor;
   FUseColor: boolean;
   FColorByTopol: boolean;
   FPropColor: string;
   FToolColor: TResultTool;
   FPalette: TColorPalette;
   FColorByPalette: boolean;
   procedure SetPalette(aValue: TColorPalette);
published
   property   Palette: TColorPalette read FPalette write SetPalette;
   property   PropColor: string read FPropColor write FPropColor;
   property   ToolColor: TResultTool read FToolColor write FToolColor;
//   property   ValColor: longInt  read FValColor   write FValColor;
   property   Color: TColor read FColor write FColor;
   property   UseColor: boolean read FUseColor write FUseColor;
   property   ColorByTopol: boolean read FColorByTopol write FColorByTopol;
   property   ColorByPalette: boolean read FColorByPalette write FColorByPalette;
end;

TGeneralXYInterpret = class(TGeneralColorInterpret)
  private
     FValX: double;
     FValY: double;
     FPropX: string;
     FPropY: string;
     FToolX: TResultTool;
     FToolY: TResultTool;
     FValHeight: double;
     FPropHeight: string;
     FToolHeight: TResultTool;
  published
     property ValX: double       read FValX       write FValX;
     property ValY: double       read FValY       write FValY;
     property PropX: string      read FPropX      write FPropX;
     property PropY: string      read FPropY      write FPropY;
     property ToolX: TResultTool read FToolX  write FToolX;
     property ToolY: TResultTool read FToolY  write FToolY;
     property ValHeight: double  read FValHeight  write FValHeight;
     property PropHeight: string read FPropHeight write FPropHeight;
     property ToolHeight: TResultTool  read FToolHeight  write FToolHeight;
end;


TGeneralBrushInterpret = class(TGeneralXYInterpret)
private
   FValBrush: longInt;
   FPropBrush: string;
   FToolBrush: TResultTool;
   FUseBrush: Boolean;
published
   property   UseBrush: Boolean read FUseBrush write FUseBrush;
   property   Brush: longInt read FValBrush write FValBrush;
   property   PropBrush: string read FPropBrush write FPropBrush;
   property   ToolBrush: TResultTool read FToolBrush write FToolBrush;
end;



implementation

uses
   e_Class;

{ TGeneralInterpret }

function  TGeneralInterpret.GetCommandLine: TCommandLine;
begin
   if FCommandLine = nil then
   begin
      FCommandLine := TCommandLine.Create;
      AddElement(FCommandLine);
   end;
   result := FCommandLine;
end;


procedure TGeneralInterpret.SetFileName(aValue: string);
var
   CL: TCommandLine;
begin
   CL := TCommandLine.Create;
   CL.ActiveElement := Self;
   CL.FileName := aValue;
   CL.DoIt;
   CL.Free;
end;


function  TGeneralInterpret.GetInsideClassName:  string;
begin
   if InsideClass <> nil then result := InsideClass.ClassName
                         else result := '';
end;

function  TGeneralInterpret.GetProcessClassName: string;
begin
   if ProcessClass <> nil then result := ProcessClass.ClassName
                          else result := '';
end;


procedure TGeneralInterpret.SetInsideClassName (aValue: string);
begin
   InsideClass := GetOrbisClass(aValue);
end;

procedure TGeneralInterpret.SetProcessClassName(aValue: string);
begin
   ProcessClass := GetOrbisClass(aValue);
end;


function  TGeneralInterpret.QGoInside(Element: TElement): boolean;
begin
   result := (InsideClass <> nil) and (Element is InsideClass);
end;

function  TGeneralInterpret.QProcessBefore(Element: TElement): boolean;
begin
   result := (ProcessClass <> nil) and (Element is ProcessClass);
end;


function  TGeneralInterpret.IsItsClass(AObject: TObject): boolean;
begin
   result := (ProcessClass <> nil) and (aObject is ProcessClass);
end;

{ TGeneralColorInterpret }
procedure TGeneralColorInterpret.SetPalette(aValue: TColorPalette);
begin
   if FPalette <> nil then
   begin
      FreeElement(FPalette);
      FPalette := nil;
   end;
   FPalette := aValue;
   AddElement(FPalette);
end;




BEGIN
   RegisterOrbisClass(TGeneralInterpret);
END.
