unit CtiPar;


{&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&} interface {&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&}


{&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&}


type

  TCteniParametru = class
    F: text;
    CisloRadku: longInt;
    Radek: string;
    Hodnota: string;
    SouborOK: boolean;
    ChybaParametru: boolean;

    constructor Create(JmSouboru: string; var OKej: boolean);
    destructor Destroy; override;

    procedure DejRadek(var JeToParametr, JeToInfo, PosledniRadek: boolean;
                       var NazevParametru: string;
                       var HodnotaParametru: string);

    function DejBooleanParametr: boolean;
    function DejRealParametr: real48;
    function DejExtendedParametr: real48;
    function DejLongIntParametr: longInt;
    function BylaChybaParametru: boolean;

  end;




  TCtiDefInfa = class
  public
    F: file of char;
    JmSouboru: string;
    Pozice: longInt;
    CisloRadku: longInt;
    Radek, Funkce: string;
    SouborOK: boolean;
    ChybaParametru: boolean;

    constructor Init(IJmSouboru: string; var OKej: boolean);
    destructor Destroy; override;

    procedure NactiRadek;

    procedure NastavPozici(NovaPozice: longInt);

    procedure DejRadek
              (var JeToInfo, PosledniRadek: boolean;
               var Obsah: string);

  end;



{&&&&&&&&&&&&&&&&&&&&&&&&&&&&&} implementation {&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&}

uses
   SysUtils,
   CtiRadek,
   u_String;
{
   KeyProc, StrProc, CtiRadek, FilProc;
}

{&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&}

const

      moStandard  = $02; { Read and write access }
      moOpenRead  = $00; { Read access only }
      moOpenWrite = $01; { Write access only }
      moOpen      = $02; { Read and write access }
      moOpenRWDD  = $12; { Read and write, deny all }
      moOpenRWND  = $22; { Read and write, deny write but enabled read }
      moOpenRWNN  = $42; { Read and write, enabled all }
      moOpenROND  = $20; { Read only, deny write but enabled read }
      moOpenRONN  = $40; { Read only, enabled all }

      moClosed    = $FF;

      Enter       = #$0D;
      LineFeed    = #$0A;


constructor TCteniParametru.Create(JmSouboru: string; var OKej: boolean);
begin
  inherited Create;
  OKej:= false;
  SouborOK:= false;
  ChybaParametru:= true;
  Radek:= '';
  Hodnota:= '';
  CisloRadku:= 0;

  if JmSouboru <> '' then
  begin
    assign(F, JmSouboru);
    fileMode:= moOpenRONN;
    {$I-} reset(F) {$I+};
    OKej:= ioResult = 0;
    fileMode:= moStandard;

    Radek:= '';
    Hodnota:= '';

    if OKej then
    begin
      CisloRadku:= 1;
      if not eof(F) then readln(F, Radek)
      else OKej:= false
    end;

    SouborOK:= OKej;
    ChybaParametru:= false;
  end
end;






destructor TCteniParametru.Destroy;
begin
  if SouborOK then close(F);
  inherited Destroy
end;





procedure TCteniParametru.DejRadek
          (var JeToParametr, JeToInfo, PosledniRadek: boolean;
           var NazevParametru: string;
           var HodnotaParametru: string);
var
  CtiR: TCtiRadekClass;
  PrvniZnak: char;

begin
  Inc(CisloRadku);
  PosledniRadek:= eof(F);
  JeToInfo:= false;
  JeToParametr:= false;
  NazevParametru:= '';
  HodnotaParametru:= '';

  Radek:= BezVnejsichMezer(Radek);

  if (Radek <> '') then
  begin
    CtiR:= TCtiRadekClass.Create(Radek);
    PrvniZnak:= CtiR.DejChar;

    if PrvniZnak <> ';' then
    begin
      if PrvniZnak = '[' then
      begin
        JeToInfo:= true;
        HodnotaParametru:= BezVnejsichMezer(UpperCase(CtiR.DejStringPoMnozinu([']'])));

        if pos('[',HodnotaParametru) > 0 then
        begin
          CtiR.StepVp;
          HodnotaParametru:= HodnotaParametru + ']' +
                             BezVnejsichMezer
                              (UpperCase(CtiR.DejStringPoMnozinu([']'])));
        end;

        Hodnota:= HodnotaParametru
      end
      else
      begin
        CtiR.StepVl;
        NazevParametru:= UpperCase(BezVnejsichMezer(CtiR.DejStringPoMnozinu(['='])));

        if not CtiR.DosloKChybe then
        begin
          CtiR.StepVp;
          HodnotaParametru:= BezVnejsichMezer(CtiR.DejString(250));
          JeToParametr:= true;
          Hodnota:= HodnotaParametru
        end
      end
    end;

    CtiR.Free
  end;

  if SouborOK and not eof(F) then readln(F, Radek)
end;





function TCteniParametru.BylaChybaParametru: boolean;
begin
  BylaChybaParametru:= ChybaParametru
end;



function TCteniParametru.DejBooleanParametr: boolean;
var
  Znak: char;

begin
  Znak:= upCase(Hodnota[1]);
  DejBooleanParametr:= Znak in ['1', 'T', 'Y', 'J', 'A']
end;




function TCteniParametru.DejRealParametr: real48;
var
  CtiR: TCtiRadekClass;

begin
  CtiR:= TCtiRadekClass.Create(Hodnota);
  DejRealParametr:= CtiR.DejReal;
  ChybaParametru:= CtiR.DosloKChybe;
  CtiR.Free
end;




function TCteniParametru.DejExtendedParametr: real48;
var
  CtiR: TCtiRadekClass;

begin
  CtiR:= TCtiRadekClass.Create(Hodnota);
  DejExtendedParametr:= CtiR.DejExtended;
  ChybaParametru:= CtiR.DosloKChybe;
  CtiR.Free
end;




function TCteniParametru.DejLongIntParametr: longInt;
var
  CtiR: TCtiRadekClass;

begin
  CtiR:= TCtiRadekClass.Create(Hodnota);
  DejLongIntParametr:= CtiR.DejLongInt;
  ChybaParametru:= CtiR.DosloKChybe;
  CtiR.Free
end;







constructor TCtiDefInfa.Init(IJmSouboru: string; var OKej: boolean);
begin
  inherited Create;
  JmSouboru:= IJmSouboru;

  assign(F, JmSouboru);
  fileMode:= moOpenRONN;

  {$I-} reset(F) {$I+};
  OKej:= ioResult = 0;
  fileMode:= moStandard;

  Radek:= '';
  Funkce:= '';
  Pozice:= 0;
  CisloRadku:= 0;


  if OKej then
  begin
    CisloRadku:= 1;
    if not eof(F) then NactiRadek
    else OKej:= false
  end;

  SouborOK:= OKej;
  ChybaParametru:= false;
end;




procedure TCtiDefInfa.NactiRadek;
var
  Znak, PZnak: char;

begin
  Radek:= '';
  repeat
    seek(F, Pozice);
    inc(Pozice);
    read(F, Znak);
    if not (Znak in [Enter, LINEFEED]) then
       Radek:= Radek + Znak
    else
    if Znak=Enter then
    begin
      seek(F, Pozice);
      read(F, PZnak);
      if PZnak = LINEFEED then inc(Pozice)
    end
  until (Znak = Enter) or eof(F)
end;



procedure TCtiDefInfa.NastavPozici(NovaPozice: longInt);
begin
  Pozice:= NovaPozice;
  seek(F, Pozice);
end;




destructor TCtiDefInfa.Destroy;
begin
  if SouborOK then close(F);
  inherited Destroy
end;





procedure TCtiDefInfa.DejRadek
          (var JeToInfo, PosledniRadek: boolean;
           var Obsah: string);
var
  CtiR: TCtiRadekClass;
  PrvniZnak: char;

begin
  PosledniRadek:= eof(F);
  JeToInfo:= false;
  Obsah:= '';
  Funkce:= '';

  Radek:= BezVnejsichMezer(Radek);

  if (Radek <> '') then
  begin
    CtiR:= TCtiRadekClass.Create(Radek);
    PrvniZnak:= CtiR.DejChar;

    if PrvniZnak <> ';' then
    begin
      if PrvniZnak = '[' then
      begin
        JeToInfo:= true;
        CtiR.PreskocMezery;
        Funkce:= UpperCase(CtiR.DejStringPoMnozinu([' ',']']));
        Obsah:= BezVnejsichMezer(CtiR.DejStringPoMnozinu([']']));
      end
      else
      begin
        CtiR.StepVl;
        Obsah:= BezVnejsichMezer(CtiR.DejString(255));
      end
    end;

    CtiR.Free
  end;

  if SouborOK and not eof(F) then NactiRadek
end;





END.
