unit e_ElementModel;

interface
uses
   //KERNEL
   Elements,
   e_Type;

type

TModelLink = class(TOwnerEntity)
protected
    function GetDestination: TElement; virtual;
    function GetSource: TElement; virtual;
public
   procedure  Link;
   procedure  LinkTo(Value: TElement);
published
   property   Source: TElement read GetSource stored false;
   property   Destination: TElement read GetDestination stored false;
end;

TSourceModelLink = class (TModelLink)
protected
    function GetDestination: TElement; override;
    function GetSource: TElement; override;
end;

TDestinationModelLink = class (TModelLink)
protected
    function GetDestination: TElement; override;
    function GetSource: TElement; override;
end;

TPropertyModel = class(TOwnerEntity)
  private
    FLinkOnly: boolean;
    function getStrLinks: string;
    procedure SetNewDestinationLink(const Value: TElement);
    procedure SetNewSourceLink(const Value: TElement);
private
   FAlias: string;
   FValue: string;
   procedure SetAlias(const Value: string);
   property StrLinks: string read getStrLinks;
    function GetLinks: TOwnerEntity;
    function GetNoLinks: boolean;
public
   function GetInstance: TElement; virtual;
published
   property Alias: string read FAlias write SetAlias stored true;
   property Value: string read FValue write FValue stored true;
   property LinkOnly: boolean read FLinkOnly write FLinkOnly stored true;
   property Links: TOwnerEntity read GetLinks stored false;
   property NoLinks: boolean read GetNoLinks stored false;
   property NewSourceLink: TElement write SetNewSourceLink stored false;
   property NewDestinationLink: TElement write SetNewDestinationLink stored false;
end;

TModelProperties = class(TOwnerEntity)
public
    function     GetPropertyStringByName(AName: string): string; override;
    procedure    SetPropertyStringByName(AName: string; aValue: string); override;
end;

TElementModel = class(TPropertyModel)
private
   FImageIndex: integer;
   FElementClassName: string;
   FComment: string;
   FAttachToPropertyName: string;
   function GetstrImages: string;
   procedure SetImageIndex(const Value: integer);
   function GetImages: TStringListElement;
   function GetNoImages: boolean;
   procedure SetNewImage(const Value: string);
   function GetModelElements: TOwnerEntity;
   function GetNoModelElements: boolean;
   function GetModelProperties: TOwnerEntity;
   function GetNoModelProperties: boolean;
   function GetstrElements: string;
   function GetstrProperties: string;
   procedure SetElementClassName(const Value: string);
   procedure SetNewElementName(const Value: string);
   procedure SetNewPropertyName(const Value: string);
   property  strElements: string read GetstrElements;
   property  strProperties: string read GetstrProperties;
   property  strImages: string read GetstrImages;
    procedure SetComment(const Value: string);
    procedure SetAttachToPropertyName(const Value: string);
    function GetLastPropertyModel: TPropertyModel;
public
   function GetInstance: TElement; override;
published
   property ElementClassName: string read FElementClassName write SetElementClassName;
   property AttachToPropertyName: string read FAttachToPropertyName write SetAttachToPropertyName;
   property Comment: string read FComment write SetComment;
   property ModelProperties: TOwnerEntity read GetModelProperties stored false;
   property Properties: TOwnerEntity read GetModelProperties stored false;
   property ModelElements: TOwnerEntity read GetModelElements stored false;
   property Images: TStringListElement read GetImages stored false;
   property ImageIndex: integer read FImageIndex write SetImageIndex;
   property NoModelProperties: boolean read GetNoModelProperties stored false;
   property NoModelElements: boolean read GetNoModelElements stored false;
   property NoImages: boolean read GetNoImages stored false;
   property NewPropertyName: string write SetNewPropertyName stored false;
   property LastPropertyModel: TPropertyModel read GetLastPropertyModel stored false;
   property NewElementName: string write SetNewElementName stored false;
   property NewImage: string write SetNewImage stored false;
end;


implementation
uses
   SysUtils,
   // KERNEL
   e_Class;
{ TElementModel }

function TElementModel.GetModelElements: TOwnerEntity;
begin
   result := TOwnerEntity(FindOrCreateElementName(TOwnerEntity,strElements));
end;

function TElementModel.GetNoModelElements: boolean;
var
   E: TOwnerEntity;
begin
   result := true;
   E := TOwnerEntity(FindElementName(strElements));
   if (E = nil) or (E.NoElements) then exit;
   result := false;
end;

function TElementModel.GetModelProperties: TOwnerEntity;
begin
   result := TOwnerEntity(FindOrCreateElementName(TModelProperties,strProperties));
end;

function TElementModel.GetNoModelProperties: boolean;
var
   E: TOwnerEntity;
begin
   result := true;
   E := TOwnerEntity(FindElementName(strProperties));
   if (E = nil) or (E.NoElements) then exit;
   result := false;
end;

procedure TElementModel.SetElementClassName(const Value: string);
begin
   FElementClassName := Value;
end;

procedure TElementModel.SetNewElementName(const Value: string);
var
   EM: TElementModel;
begin
   EM := TElementModel.Create;
   EM.Name := Value;
   ModelElements.AddElement(EM);
end;

procedure TElementModel.SetNewPropertyName(const Value: string);
var
   PM: TPropertyModel;
begin
   PM := TPropertyModel.Create;
   PM.Name := Value;
   ModelProperties.AddElement(PM);
end;

function TElementModel.GetstrElements: string;
begin
   result := 'ELEMENTS';
end;

function TElementModel.GetstrProperties: string;
begin
   result := 'PROPERTIES';
end;

function TElementModel.GetstrImages: string;
begin
   result := 'IMAGES';
end;

procedure TElementModel.SetImageIndex(const Value: integer);
begin
  FImageIndex := Value;
end;

function TElementModel.GetImages: TStringListElement;
begin
   result := TStringListElement(FindOrCreateElementName(TStringListElement,strImages));
end;

function TElementModel.GetNoImages: boolean;
var
   S: TStringListElement;
begin
   result := true;
   S := TStringListElement(FindElementName(strImages));
   if (S = nil) or (S.NoElements) then exit;
   result := false;
end;

procedure TElementModel.SetNewImage(const Value: string);
begin
   Images.AddString(Value);
end;

procedure TElementModel.SetComment(const Value: string);
begin
   FComment := Value;
end;


procedure TElementModel.SetAttachToPropertyName(const Value: string);
begin
   FAttachToPropertyName := Value;
end;

function TElementModel.GetLastPropertyModel: TPropertyModel;
begin
   result := nil;
   if NoModelProperties then exit;
   Result:= TPropertyModel(ModelProperties.Elements[ModelProperties.Count-1])
end;

function TElementModel.GetInstance: TElement;
var
   i: integer;
   E: TEntity;
begin
   result := nil;
   if NoOwners then exit;
   for i := 0 to OwnersCount - 1 do
   begin
      E := Owners[i];
      if (AnsiCompareText(E.Name, Name) = 0) and
         (CompareText(E.ClassName,ElementClassName) = 0) then
      begin
         result := E;
         exit;
      end;
   end;
end;

{ TModelLink }

function TModelLink.GetDestination: TElement;
begin
   result := nil;
end;

function TModelLink.GetSource: TElement;
begin
   result := nil;
end;

procedure TModelLink.Link;
var
   aSource: TElement;
   aDestination: TElement;
   SourceElement: TElement;
   DestinationElement: TElement;
   SourceIsProperty: boolean;
   DestinationIsProperty: boolean;
   //EM: TElementModel;
   PM: TPropertyModel;
   SourceValue: string;
   E: TEntity;
begin
   SourceIsProperty := false;
   DestinationIsProperty := false;
   aSource := Source;
   aDestination := Destination;
   // Kontrola zprvnosti parametr
   if (aSource = nil) or (aDestination = nil) then exit;
   if aSource.ClassType = TPropertyModel then
      SourceIsProperty := true
   else
      if not (aSource is TElementModel) then exit;
      // error !!!!!!!!!!!
   if aDestination.ClassType = TPropertyModel then
      DestinationIsProperty := true
   else
      if not (aDestination is TElementModel) then exit;
      // error !!!!!!!!!!!
   // Hledn SourceElementu
   PM := TPropertyModel(aSource);
   SourceElement := PM.GetInstance;
   if SourceElement = nil then exit;
   // Hledn DestinationElementu
   PM := TPropertyModel(aDestination);
   DestinationElement := PM.GetInstance;
   if DestinationElement = nil then exit;
   // Zdrojov hodnota
   PM := TPropertyModel(aSource);
   if SourceIsProperty then
      SourceValue := SourceElement.GetPropertyStringByName(PM.Name)
   else
      SourceValue := IntToStr(longInt(SourceElement));
   // Naplnn cle
   if DestinationIsProperty then
   begin
      PM := TPropertyModel(aDestination);
      DestinationElement.SetPropertyStringByName(PM.Name,SourceValue);
   end
   else
   begin
      if not (DestinationElement is TEntity) then exit;
      E := TEntity(DestinationElement);
      if E.IndexOf(SourceElement) < 0 then
         E.AddElement(SourceElement);
   end;
end;

procedure TModelLink.LinkTo(Value: TElement);
begin
   if not NoElements then
      FreeAll;
   AddElement(Value);
end;

{ TPropertyModel }

function TPropertyModel.GetInstance: TElement;
var
   EM: TElementModel;
begin
   result := nil;
   EM := TElementModel(FindClassInOwners(TElementModel));
   if EM = nil then exit;
   result := EM.GetInstance;
end;

function TPropertyModel.GetLinks: TOwnerEntity;
begin
   result := TOwnerEntity(FindOrCreateElementName(TOwnerEntity,strLinks));
end;

function TPropertyModel.GetNoLinks: boolean;
var
   E: TOwnerEntity;
begin
   result := true;
   E := TOwnerEntity(FindElementName(strLinks));
   if (E = nil) or (E.NoElements) then exit;
   result := false;
end;

function TPropertyModel.getStrLinks: string;
begin
   result := 'LINKS'
end;

procedure TPropertyModel.SetAlias(const Value: string);
begin
  FAlias := Value;
end;

procedure TPropertyModel.SetNewDestinationLink(const Value: TElement);
var
   LNK: TModelLink;
begin
   if Value = nil then exit;
   LNK := TDestinationModelLink.Create;
   //LNK.Source := Self;
   LNK.LinkTo(Value);
   Links.AddElement(LNK);
end;

procedure TPropertyModel.SetNewSourceLink(const Value: TElement);
var
   LNK: TModelLink;
begin
   if Value = nil then exit;
   LNK := TSourceModelLink.Create;
   LNK.LinkTo(Value);
   //LNK.Destination := Self;
   Links.AddElement(LNK);
end;

{ TModelProperties }

function TModelProperties.GetPropertyStringByName(AName: string): string;
var
   PM: TPropertyModel;
begin
   PM := TPropertyModel(FindElementName(aName));
   if PM = nil then
   begin
      PM := TPropertyModel.Create;
      PM.Name := aName;
      AddElement(PM);
   end;
   result := IntToStr(longInt(PM));
end;

procedure TModelProperties.SetPropertyStringByName(AName, aValue: string);
var
   PM: TPropertyModel;
begin
   PM := TPropertyModel(FindElementName(aName));
   if PM = nil then
   begin
      PM := TPropertyModel.Create;
      PM.Name := aName;
      AddElement(PM);
   end;
   PM.Value := aValue;
end;

{ TSourceModelLink }

function TSourceModelLink.GetDestination: TElement;
begin
   result := FindClassInOwners(TPropertyModel);
end;

function TSourceModelLink.GetSource: TElement;
begin
   result := nil;
   if not NoElements then
      result := Elements[0];
end;

{ TDestinationModelLink }

function TDestinationModelLink.GetDestination: TElement;
begin
   result := nil;
   if not NoElements then
      result := Elements[0];
end;

function TDestinationModelLink.GetSource: TElement;
begin
   result := FindClassInOwners(TPropertyModel);
end;

INITIALIZATION
   RegisterOrbisClass(TModelLink);
   RegisterOrbisClass(TSourceModelLink);
   RegisterOrbisClass(TDestinationModelLink);
   RegisterOrbisClass(TPropertyModel);
   RegisterOrbisClass(TModelProperties);
   RegisterOrbisClass(TElementModel);
END.


