unit T_CreateForestModel;

interface
uses
   Elements,
   e_LHCModel,
   e_KatastrModel,
   T_CreateModelFromlayer,
   t_ForestModelSelector,
   t_ParcelModelSelector;

type

TCreateForestModel = class(TCreateModelFromLayer)
  private
    FSkupinaGroup: integer;
    FPorostGroup: integer;
    FOddeleniPropertyName: string;
    FDilecPropertyName: string;
    FSkupinaPropertyName: string;
    FPorostPropertyName: string;
    FLHCPropertyName: string;
    FKatastrPropertyName: string;
    FSkupinaParcelPropertyName: string;
    FParcelModel: Tgis_ParcelModel;
    FLSPropertyName: string;
    FFMSelector: TForestModelselector;
    FPMSelector: TParcelModelselector;
    FUseAttributes: boolean;
    FCreateByLinesAndPoints: boolean;
    FSKPByComment: boolean;
    function GetForestModel: Tgis_ForestModel;
    procedure SetDilecPropertyName(const Value: string);
    procedure SetForestModel(const Value: Tgis_ForestModel);
    procedure SetLHCPropertyName(const Value: string);
    procedure SetOddeleniPropertyName(const Value: string);
    procedure SetPorostGroup(const Value: integer);
    procedure SetPorostPropertyName(const Value: string);
    procedure SetSkupinaGroup(const Value: integer);
    procedure SetSkupinaPropertyName(const Value: string);
    procedure SetKatastrPropertyName(const Value: string);
    procedure SetParcelModel(const Value: Tgis_ParcelModel);
    procedure SetSkupinaParcelPropertyName(const Value: string);
    procedure SetLSPropertyName(const Value: string);
    function GetFMSelector: TForestModelSelector;
    function GetPMSelector: TParcelModelSelector;
    procedure SetUseAttributes(const Value: boolean);
    procedure SetCreateByLinesAndPoints(const Value: boolean);
    procedure SetSKPByComment(const Value: boolean);
public
   procedure CreateModel; override;
   procedure CreateModelFromLinesAndPoints;
   property  FMSelector: TForestModelSelector read GetFMSelector;
   property  PMSelector: TParcelModelSelector read GetPMSelector;
published
   property ForestModel: Tgis_ForestModel read GetForestModel write SetForestModel;
   property ParcelModel: Tgis_ParcelModel read FParcelModel write SetParcelModel;
   property PorostGroup: integer read FPorostGroup write SetPorostGroup;
   property SkupinaGroup: integer read FSkupinaGroup write SetSkupinaGroup;
   property LHCPropertyName: string read FLHCPropertyName write SetLHCPropertyName;
   property LSPropertyName: string read FLSPropertyName write SetLSPropertyName;
   property OddeleniPropertyName: string read FOddeleniPropertyName write SetOddeleniPropertyName;
   property DilecPropertyName: string read FDilecPropertyName write SetDilecPropertyName;
   property PorostPropertyName: string read FPorostPropertyName write SetPorostPropertyName;
   property SkupinaPropertyName: string read FSkupinaPropertyName write SetSkupinaPropertyName;
   property KatastrPropertyName: string read FKatastrPropertyName write SetKatastrPropertyName;
   property SkupinaParcelPropertyName: string read FSkupinaParcelPropertyName write SetSkupinaParcelPropertyName;
   property UseAttributes: boolean read FUseAttributes write SetUseAttributes;
   property CreateByLinesAndPoints: boolean read FCreateByLinesAndPoints write SetCreateByLinesAndPoints;
   property SKPByComment: boolean read FSKPByComment write SetSKPByComment;
end;

TCreateSimpleHighLevelForestModel = class(TCreateForestModel)
  private
    FDilecGroup: integer;
    FLHCGroup: integer;
    FOddeleniGroup: integer;
    procedure SetDilecGroup(const Value: integer);
    procedure SetLHCGroup(const Value: integer);
    procedure SetOddeleniGroup(const Value: integer); // tedy LHC,ODD,Dil - to co neni v normalnim
public
   procedure CreateModel; override;
published
   property LHCGroup: integer read FLHCGroup write SetLHCGroup;
   property OddeleniGroup: integer read FOddeleniGroup write SetOddeleniGroup;
   property DilecGroup: integer read FDilecGroup write SetDilecGroup;
end;

TCreateSRFModel = class(TCreateModelFromLayer)
  private
    FAreasGroup: integer;
    procedure SetAreasGroup(const Value: integer);
public
   procedure CreateModel; override;
published
   property AreasGroup: integer read FAreasGroup write SetAreasGroup;
end;

TCreateTEZModel = class(TCreateForestModel)
  private
    FAreasGroup: integer;
    procedure SetAreasGroup(const Value: integer);
public
   procedure CreateModel; override;
published
   property AreasGroup: integer read FAreasGroup write SetAreasGroup;
end;

TCreateTYPModel = class(TCreateModelFromLayer)
  private
    FAreasGroup: integer;
    procedure SetAreasGroup(const Value: integer);
public
   procedure CreateModel; override;
published
   property AreasGroup: integer read FAreasGroup write SetAreasGroup;
end;


TCreateREV = class (TCreateForestModel)
  private
    FAreasGroup: integer;
    FRevPropertyName: string;
    procedure SetAreasGroup(const Value: integer);
    procedure SetRevPropertyName(const Value: string);
public
   procedure CreateModel; override;
published
   property AreasGroup: integer read FAreasGroup write SetAreasGroup;
   property RevPropertyName: string read FRevPropertyName write SetRevPropertyName;
end;

implementation
uses
   Dialogs,
   Forms,
   SysUtils,
   e_Class,
   e_Geometry,
   e_GeometryElement,
   e_LHCElements,
   e_Props,
   e_QTree,
   e_Type,
   g_Elems,
   m_Orbis,
   t_AreMak,
   t_QTreeTools;

{ TCreateForestModel }

procedure TCreateForestModel.CreateModel;
var
   AC: TAreaCollection;
   PC: TPolyLineCollection;
   G: TGeometryElement;
   FinalG: TGeometryElement;
   PGE: TPropertyGeometryElement;
   E: TElement;
   i,j: integer;
   IsSkupina: boolean;
   LHC: integer;
   LS: integer;
   Oddeleni: integer;
   Dilec: string;
   Porost: string;
   Skupina: string;
   Katastr: integer;
   SkupinaParcel: integer;
   S: TStringElement;
   NewObject: boolean;
   CreateSKPGeometry: boolean;
   SFixAre,SFixArei,SKatparKod: string;
begin
   Error := 1;
   if CreateByLinesAndPoints then
   begin
      CreateModelFromLinesAndPoints;
      exit;
   end;
   if Data.NoElements then exit;
   // Linie
   E := Data.FindElementName('LIN');
   if not (E is TEntity) then exit; // dn linie
   if TEntity(E).NoElements  then exit;
   E := TEntity(E).Elements[0];
   if not (E is TPolyLineCollection) then exit;
   PC := TPolyLineCollection(E);
   if LinesLayer = nil then
   begin
      ShowMessage(Comment+': Nen nastavena entita pro linie');
   end
   else
   begin
      if NoLineTransferModel then
      begin
         LinesLayer.AddAllFrom := E;
         if LinesLayer is TRectEntity then
            TRectEntity(LinesLayer).ResetAndUpdate;
      end
      else
      begin
         if PC.Count > 50 then
         begin
            MaxProgress := PC.LastIndex;
            Application.ProcessMessages;
         end;
         for i := 0 to PC.LastIndex do
         begin
            if i mod 50 = 0 then
            begin
               Progress := i;
               Application.ProcessMessages;
            end;
            E := PC.Elements[i];
            for j := 0 to LineTransferModel.LastIndex do
            begin
               S := LineTransferModel.StringElements[j];
               E.SetPropertyStringByName(S.Value,E.GetPropertyStringByName(S.Name));
               Application.ProcessMessages;
               if LinesLayer is TRectEntity then
                  TRectEntity(LinesLayer).AddAndUpdateRect(E)
               else
                  LinesLayer.AddElement(E);
            end;
         end;
      end;
   end;
   Progress := 0;
   Application.ProcessMessages;
   // Plochy
   E := Data.FindElementName('ARE');
   if not (E is TEntity) then exit;
   if TEntity(E).NoElements  then exit;
   E := TEntity(E).Elements[0];
   if not (E is TAreaCollection) then exit;
   AC := TAreaCollection(E);
   CreateSKPGeometry := (ParcelModel <> nil) and ParcelModel.NoElements;
   if AC.Count > 50 then
   begin
      MaxProgress := AC.LastIndex;
      Application.ProcessMessages;
   end;
   for i := 0 to AC.LastIndex do
   begin
      E := AC.Elements[i];
      if not (E is TGeometryElement) then continue;
      G := TGeometryElement(E);
      if i mod 50 = 0 then
      begin
         Progress := i;
         Application.ProcessMessages;
      end;
      IsSkupina := G.Group = SkupinaGroup;
      // jet kontrola, jestli je to skupina parcel ... na neplochy kalu
      if not IsSkupina and (G.Group <> PorostGroup) then continue;
      LHC := G.GetPropertyLongIntByName(LHCPropertyName);
      LS := G.GetPropertyLongIntByName(LSPropertyName);
      if LS < 1 then LS := 1;
      Oddeleni := G.GetPropertyLongIntByName(OddeleniPropertyName);
      Dilec := G.GetPropertyStringByName(DilecPropertyName);
      { TODO 5 -oPPK -cDodlat : Import LHC - Kontroly poloek a zaazovn u chybnch hodnot }
      Porost := G.GetPropertyStringByName(PorostPropertyName);
      if Porost = '' then Porost := 'a';
      if (Dilec = '') or not(Dilec[1] in ['a'..'z','A'..'Z']) then
         Dilec := UpperCase(Porost);
      if ForestModel.GetLHCCislo(LHC) = nil then
         ForestModel.CreateLHCCislo(LHC);
      if ForestModel.GetLSCislo(LS) = nil then
         ForestModel.CreateLSCislo(LS);
      if ForestModel.GetOddeleniCislo(Oddeleni) = nil then
         ForestModel.CreateOddeleniCislo(Oddeleni);
      if ForestModel.GetDilecName(Dilec) = nil then
         ForestModel.CreateDilecName(Dilec);
      NewObject := false;
      if ForestModel.GetPorostName(Porost) = nil then
      begin
         ForestModel.CreatePorostName(Porost);
         NewObject := true;
      end;
      FinalG := ForestModel.ActualPorost;
      if IsSkupina then
      begin
         NewObject := false;
         Skupina := G.GetPropertyStringByName(SkupinaPropertyName);
         if ForestModel.GetSkupinaName(Skupina) = nil then
         begin
            ForestModel.CreateSkupinaName(Skupina);
            NewObject := true;
         end;
         FinalG := ForestModel.ActualSkupina;
      end;
      FinalG.AddGeometry(G.Geometry);
      if IsSkupina then
      begin
         ForestModel.ActualPorost.UpdateRect(FinalG);
         //ForestModel.ActualPorost.Items.UpdateRect(FinalG);
      end;
      ForestModel.ActualDilec.UpdateRect(FinalG);
      // ForestModel.ActualDilec.Items.UpdateRect(FinalG);
      ForestModel.ActualOddeleni.UpdateRect(FinalG);
      //ForestModel.ActualOddeleni.Items.UpdateRect(FinalG);
      ForestModel.ActualLS.UpdateRect(FinalG);
      ForestModel.ActualLHC.UpdateRect(FinalG);
      //ForestModel.ActualLHC.Items.UpdateRect(FinalG);
      ForestModel.UpdateRect(FinalG);
      if NoTransferModel then continue;
      SFixAre := '0';
      SFixArei := '0';
      SKatparKod := '0';
      for j := 0 to TransferModel.LastIndex do
      begin
         S := TransferModel.StringElements[j];
         if S.Name='DYNAMIC' then
         begin
            if FinalG is TPropertyGeometryElement then
            begin
               PGE := TPropertyGeometryElement(FinalG);
               if PGE.NoProperties then
                  PGE.Properties := TDynamicProperties.Create;
               PGE.SetPropertyStringByName(S.Value,G.GetPropertyStringByName(S.Value));
            end;
         end
         else
         if S.Value = 'FIXEDAREA' then
         begin
            SFixAre := G.GetPropertyStringByName(S.Name);
            G.Geometry.SetPropertyStringByName('FixedArea',SFixAre);
         end
         else
         if S.Value = 'FIXEDAREAINDEX' then
         begin
            SFixArei := G.GetPropertyStringByName(S.Name);
            G.Geometry.SetPropertyStringByName('FixedAreaIndex',SFixArei);
         end
         else
         if S.Value = 'KATPAR_KOD' then
         begin
            SKatparkod := G.GetPropertyStringByName(S.Name);
         end
         else
            FinalG.SetPropertyStringByName(S.Value,G.GetPropertyStringByName(S.Name));
      end;
      if (AreasLayer <> nil) and not FinalG.IsMyOwner(AreasLayer) then
      begin
         if (AreasLayer is TRectEntity) then
            TRectEntity(AreasLayer).AddAndUpdateRect(FinalG)
         else
            AreasLayer.AddElement(FinalG);
      end;
      if (ParcelModel <> nil) and (KatastrPropertyName <> '') and
         (SkupinaParcelPropertyName <> '')then
      begin
         Katastr := G.GetPropertyLongIntByName(KatastrPropertyName);
         SkupinaParcel := G.GetPropertyLongIntByName(SkupinaParcelPropertyName);
         if (Katastr > 0) and (SkupinaParcel > 0) then
         begin
            ParcelModel.CreateKatastrCislo(Katastr);
            ParcelModel.CreateSkupinaParcelCislo(SkupinaParcel);
            if CreateSKPGeometry then
            begin
               ParcelModel.ActualSkupinaParcel.SetPropertyStringByName('AddVymera',SFixAre);
               ParcelModel.ActualSkupinaParcel.SetPropertyStringByName('KVAL_P',SFixArei);
               ParcelModel.ActualSkupinaParcel.SetPropertyStringByName('KATPAR_KOD',SKatparKod);
               ParcelModel.ActualSkupinaParcel.AddGeometry(G.Geometry);
            end;
            FinalG.SetPropertyLongIntByName('SkupinaParcel',longInt(ParcelModel.ActualSkupinaParcel));
         end;
      end;
   end;
   ForestModel.UpdateAll := true;
   Error := 0;
end;

function TCreateForestModel.GetForestModel: Tgis_ForestModel;
begin
   result := Tgis_ForestModel(Output);
end;


procedure TCreateForestModel.SetDilecPropertyName(const Value: string);
begin
  FDilecPropertyName := Value;
end;

procedure TCreateForestModel.SetForestModel(const Value: Tgis_ForestModel);
begin
   Output := Value;
end;

procedure TCreateForestModel.SetKatastrPropertyName(const Value: string);
begin
  FKatastrPropertyName := Value;
end;

procedure TCreateForestModel.SetLHCPropertyName(const Value: string);
begin
  FLHCPropertyName := Value;
end;

procedure TCreateForestModel.SetOddeleniPropertyName(const Value: string);
begin
  FOddeleniPropertyName := Value;
end;

procedure TCreateForestModel.SetParcelModel(const Value: Tgis_ParcelModel);
begin
   FParcelModel := Value;
end;

procedure TCreateForestModel.SetPorostGroup(const Value: integer);
begin
  FPorostGroup := Value;
end;

procedure TCreateForestModel.SetPorostPropertyName(const Value: string);
begin
  FPorostPropertyName := Value;
end;

procedure TCreateForestModel.SetSkupinaGroup(const Value: integer);
begin
  FSkupinaGroup := Value;
end;

procedure TCreateForestModel.SetSkupinaParcelPropertyName(
  const Value: string);
begin
  FSkupinaParcelPropertyName := Value;
end;

procedure TCreateForestModel.SetSkupinaPropertyName(const Value: string);
begin
  FSkupinaPropertyName := Value;
end;


procedure TCreateForestModel.SetLSPropertyName(const Value: string);
begin
  FLSPropertyName := Value;
end;

function TCreateForestModel.GetFMSelector: TForestModelSelector;
begin
   if FFMSelector = nil then
   begin
      ExchangeFieldElement(TElement(FFMSelector),TForestModelSelector.Create);
   end;
   result := FFMSelector;
end;

function TCreateForestModel.GetPMSelector: TParcelModelSelector;
begin
   if FPMSelector = nil then
   begin
      ExchangeFieldElement(TElement(FPMSelector),TParcelModelSelector.Create);
   end;
   result := FPMSelector;
end;

procedure TCreateForestModel.SetUseAttributes(const Value: boolean);
begin
  FUseAttributes := Value;
end;

procedure TCreateForestModel.SetCreateByLinesAndPoints(
  const Value: boolean);
begin
  FCreateByLinesAndPoints := Value;
end;

procedure TCreateForestModel.CreateModelFromLinesAndPoints;
var
   PC: TPointCollection;
   Ares: TAreaString;
   G: TGeometryElement;
   GG: TGeometry;
   FinalG: TGeometryElement;
   PGE: TPropertyGeometryElement;
   E: TElement;
   i,j: integer;
   IsSkupina: boolean;
   LHC: integer;
   LS: integer;
   Oddeleni: integer;
   Dilec: string;
   Porost: string;
   Skupina: string;
   Katastr: integer;
   SkupinaParcel: integer;
   S: TStringElement;
   NewObject: boolean;
   CreateSKPGeometry: boolean;
   SFixAre,SFixArei,SKatparKod: string;
   AFL: TCreateAreasFromLinesTool;
   AQT: TRectEntityQTree;
   TQ: TAddEntityToQTree;
   QTFinder: TFindRectEntityInQTreeByXY;
   SolvedA: TAreaCollection;
   ER0: TPointCollection;
   ER1: TPointCollection;
   OE: TOwnerEntity;
begin
   Error := 1;
   if Data.NoElements then exit;
   // Linie
   if (LinesLayer = nil) or (LinesLayer.NoElements) then
   begin
      ShowMessage(Comment+': Vrstva lini chyb nebo je przdn!');
   end;
   Progress := 0;
   Application.ProcessMessages;
   // Body
   E := Data.FindElementName('PNT');
   if not (E is TEntity) then exit;
   if TEntity(E).NoElements  then exit;
   E := TEntity(E).Elements[0];
   if not (E is TPointCollection) then exit;
   PC := TPointCollection(E);
   //CreateSKPGeometry := (ParcelModel <> nil) and ParcelModel.NoElements;
   // pomocne struktury
   CreateSKPGeometry := false;
   {
   OE := TOwnerEntity.Create;
   OE.Name := 'Errors';
   AddElement(OE);
   ER1 := TAreaCollection.Create;
   ERA.Name := 'Obsazen plochy'
   OE.AddElement(ERA);
   }
   SolvedA := TAreaCollection.Create;
   SolvedA.Name := 'SolvedA';
   AddElement(SolvedA);
   // Vytvoreni ploch
   AFL := TCreateAreasFromLinesTool.Create;
   AFL.Input := LinesLayer;
   AFL.DoIt;
   Ares := TAreaString(AFL.Output);
   AddElement(Ares);
   AFL.Free;
   AQT := TRectEntityQTree.Create;
   AddElement(AQT);
   TQ := TAddEntityToQTree.Create;
   TQ.Input := Ares;
   TQ.Output := AQT;
   TQ.DoIt;
   TQ.Free;
   // A jeste zalozeni vyhledavatka
   QTFinder := TFindRectEntityInQTreeByXY.Create;
   QTFinder.Input := AQT;
   QTFinder.Tolerance := 0;
   if PC.Count > 50 then
   begin
      MaxProgress := PC.LastIndex;
      Application.ProcessMessages;
   end;
   for i := 0 to PC.LastIndex do
   begin
      E := PC.Elements[i];
      if not (E is TGeometryElement) then continue;
      G := TGeometryElement(E);
      if i mod 50 = 0 then
      begin
         Progress := i;
         Application.ProcessMessages;
      end;
      QTFinder.X := G.X;
      QTFinder.Y := G.Y;
      QTFinder.DoIt;
      if QTFinder.Output = nil then continue;// pridat do seznamu
      if QTFinder.Output.IsMyOwner(SolvedA) then continue;
      SolvedA.AddElement(QTFinder.Output);
      IsSkupina := G.Group = SkupinaGroup;
      // jet kontrola, jestli je to skupina parcel ... na neplochy kalu
      if not IsSkupina and (G.Group <> PorostGroup) then continue;
      LHC := G.GetPropertyLongIntByName(LHCPropertyName);
      LS := G.GetPropertyLongIntByName(LSPropertyName);
      if LS < 1 then LS := 1;
      Oddeleni := G.GetPropertyLongIntByName(OddeleniPropertyName);
      Dilec := G.GetPropertyStringByName(DilecPropertyName);
      { TODO 5 -oPPK -cDodlat : Import LHC - Kontroly poloek a zaazovn u chybnch hodnot }
      Porost := G.GetPropertyStringByName(PorostPropertyName);
      if Porost = '' then Porost := 'a';
      if (Dilec = '') or not(Dilec[1] in ['a'..'z','A'..'Z']) then
         Dilec := UpperCase(Porost);
      Porost := LowerCase(Porost);
      if ForestModel.GetLHCCislo(LHC) = nil then
         ForestModel.CreateLHCCislo(LHC);
      if ForestModel.GetLSCislo(LS) = nil then
         ForestModel.CreateLSCislo(LS);
      if ForestModel.GetOddeleniCislo(Oddeleni) = nil then
         ForestModel.CreateOddeleniCislo(Oddeleni);
      if ForestModel.GetDilecName(Dilec) = nil then
         ForestModel.CreateDilecName(Dilec);
      NewObject := false;
      if ForestModel.GetPorostName(Porost) = nil then
      begin
         ForestModel.CreatePorostName(Porost);
         NewObject := true;
      end;
      FinalG := ForestModel.ActualPorost;
      if IsSkupina then
      begin
         NewObject := false;
         Skupina := G.GetPropertyStringByName(SkupinaPropertyName);
         if ForestModel.GetSkupinaName(Skupina) = nil then
         begin
            ForestModel.CreateSkupinaName(Skupina);
            NewObject := true;
         end;
         FinalG := ForestModel.ActualSkupina;
      end;
      GG := TGeometry(QTFinder.Output);
      FinalG.AddGeometry(GG);
      if IsSkupina then
      begin
         ForestModel.ActualPorost.UpdateRect(FinalG);
         //ForestModel.ActualPorost.Items.UpdateRect(FinalG);
      end;
      ForestModel.ActualDilec.UpdateRect(FinalG);
      // ForestModel.ActualDilec.Items.UpdateRect(FinalG);
      ForestModel.ActualOddeleni.UpdateRect(FinalG);
      //ForestModel.ActualOddeleni.Items.UpdateRect(FinalG);
      ForestModel.ActualLS.UpdateRect(FinalG);
      ForestModel.ActualLHC.UpdateRect(FinalG);
      //ForestModel.ActualLHC.Items.UpdateRect(FinalG);
      ForestModel.UpdateRect(FinalG);
      SFixAre := '0';
      SFixArei := '0';
      SKatparKod := '0';
      if not NoTransferModel then
      for j := 0 to TransferModel.LastIndex do
      begin
         S := TransferModel.StringElements[j];
         if S.Name='DYNAMIC' then
         begin
            if FinalG is TPropertyGeometryElement then
            begin
               PGE := TPropertyGeometryElement(FinalG);
               if PGE.NoProperties then
                  PGE.Properties := TDynamicProperties.Create;
               PGE.SetPropertyStringByName(S.Value,G.GetPropertyStringByName(S.Value));
            end;
         end
         else
         if S.Value = 'FIXEDAREA' then
         begin
            SFixAre := G.GetPropertyStringByName(S.Name);
            GG.SetPropertyStringByName('FixedArea',SFixAre);
         end
         else
         if S.Value = 'FIXEDAREAINDEX' then
         begin
            SFixArei := G.GetPropertyStringByName(S.Name);
            GG.SetPropertyStringByName('FixedAreaIndex',SFixArei);
         end
         else
         if S.Value = 'KATPAR_KOD' then
         begin
            SKatparkod := G.GetPropertyStringByName(S.Name);
         end
         else
            FinalG.SetPropertyStringByName(S.Value,G.GetPropertyStringByName(S.Name));
      end;
      if (AreasLayer <> nil) and not FinalG.IsMyOwner(AreasLayer) then
      begin
         if (AreasLayer is TRectEntity) then
            TRectEntity(AreasLayer).AddAndUpdateRect(FinalG)
         else
            AreasLayer.AddElement(FinalG);
      end;
      if (ParcelModel <> nil) and (KatastrPropertyName <> '') and
         (SkupinaParcelPropertyName <> '')then
      begin
         Katastr := G.GetPropertyLongIntByName(KatastrPropertyName);
         SkupinaParcel := G.GetPropertyLongIntByName(SkupinaParcelPropertyName);
         if (Katastr > 0) and (SkupinaParcel > 0) then
         begin
            ParcelModel.CreateKatastrCislo(Katastr);
            ParcelModel.CreateSkupinaParcelCislo(SkupinaParcel);
            if CreateSKPGeometry then
            begin
               ParcelModel.ActualSkupinaParcel.SetPropertyStringByName('AddVymera',SFixAre);
               ParcelModel.ActualSkupinaParcel.SetPropertyStringByName('KVAL_P',SFixArei);
               ParcelModel.ActualSkupinaParcel.SetPropertyStringByName('KATPAR_KOD',SKatparKod);
               ParcelModel.ActualSkupinaParcel.AddGeometry(GG);
            end;
            FinalG.SetPropertyLongIntByName('SkupinaParcel',longInt(ParcelModel.ActualSkupinaParcel));
         end;
      end;
   end;
   ForestModel.UpdateAll := true;
   Error := 0;
end;

procedure TCreateForestModel.SetSKPByComment(const Value: boolean);
begin
   FSKPByComment := Value;
end;

{ TCreateSRFModel }

procedure TCreateSRFModel.CreateModel;
var
   E: TElement;
   AC: TAreaCollection;
   PC: TPointCollection;
   G: TGeometryElement;
   SRFAre: TSRFPlocha;
   SRFPoint: TSRFBod;
   i,j: integer;
   S: TStringElement;
begin
   Error := 1;
   // linie
   if Data.NoElements then
   begin
      //ShowMessage(Comment+': Nebyla nalezena dn data kter by stla za import');
      PostTextMsg(tm_AddToProtocol,Comment+': Nebyla nalezena dn data kter by stla za import',nil);
      exit;
   end;
   // Plochy
   E := Data.FindElementName('ARE');
   if (E <> nil) and (E is TEntity) and not TEntity(E).NoElements  then
   begin
      E := TEntity(E).Elements[0];
      if (E is TAreaCollection) then
      begin
         if AreasLayer = nil then
         begin
            ShowMessage(Comment+': Nen nastavena entita pro Plochy');
         end
         else
         begin
            AC := TAreaCollection(E);
            for i := 0 to AC.LastIndex do
            begin
               G := TGeometryElement(AC.Elements[i]);
               if G.Group <> AreasGroup then Continue; // neplocha
               SRFAre := TSRFPlocha.Create;
               SRFAre.Geometry := G.Geometry;
               TRectEntity(AreasLayer).AddAndUpdateRect(SRFAre);
               if NoTransferModel then continue;
               for j := 0 to TransferModel.LastIndex do
               begin
                  S := TransferModel.StringElements[j];
                  SRFAre.SetPropertyStringByName(S.Value,G.GetPropertyStringByName(S.Name));
               end;
            end;
         end;
      end;
   end;
   // Body
   E := Data.FindElementName('PNT');
   if (E <> nil) and (E is TEntity) and not TEntity(E).NoElements  then
   begin
      E := TEntity(E).Elements[0];
      if (E is TPointCollection) then
      begin
         if PointsLayer = nil then
         begin
            ShowMessage(Comment+': Nen nastavena entita pro body');
         end
         else
         begin
            PC := TPointCollection(E);
            for i := 0 to PC.LastIndex do
            begin
               G := TGeometryElement(PC.Elements[i]);
               SRFPoint := TSRFBod.Create;
               SRFPoint.Geometry := G.Geometry;
               TRectEntity(PointsLayer).AddAndUpdateRect(SRFPoint);
               if NoTransferModel then continue;
               for j := 0 to TransferModel.LastIndex do
               begin
                  S := TransferModel.StringElements[j];
                  SRFPoint.SetPropertyStringByName(S.Value,G.GetPropertyStringByName(S.Name));
               end;
            end;
         end;
      end;
   end;
end;

procedure TCreateSRFModel.SetAreasGroup(const Value: integer);
begin
  FAreasGroup := Value;
end;



{ TCreateTEZModel }

procedure TCreateTEZModel.CreateModel;
var
   AC: TAreaCollection;
   PC: TPointCollection;
   G: TGeometryElement;
   E: TElement;
   i,j: integer;
   LHC: integer;
   LS: integer;
   Oddeleni: integer;
   Dilec: string;
   Porost: string;
   Skupina: string;
   S: TStringElement;
   TEZBod: TTezBod;
   TEZPlocha: TTezPlocha;
   PSK: Tgis_PorostniSkupina;
begin
   Error := 1;
   if Data.NoElements then exit;
   // Linie neee
   // Body
   E := Data.FindElementName('PNT');
   if (E <> nil) and (E is TEntity) and not TEntity(E).NoElements  then
   begin
      E := TEntity(E).Elements[0];
      if (E is TPointCollection) then
      begin
         PC := TPointCollection(E);
         if PointsLayer = nil then
         begin
            ShowMessage(Comment+': Nen nastavena entita pro body');
         end
         else
         begin
            for i := 0 to PC.LastIndex do
            begin
               G := TGeometryElement(PC.Elements[i]);
               TEZBod := TTEZBod.Create;
               TEZBod.Geometry := G.Geometry;
               if PointsLayer is TRectEntity then
                  TRectEntity(PointsLayer).AddAndUpdateRect(TEZBod)
               else
                  PointsLayer.AddElement(TEZBod);
               if NoTransferModel then continue;
               for j := 0 to TransferModel.LastIndex do
               begin
                  S := TransferModel.StringElements[j];
                  TEZBod.SetPropertyStringByName(S.Value,G.GetPropertyStringByName(S.Name));
               end;
            end;
         end;
      end;
   end;
   // Plochy
   E := Data.FindElementName('ARE');
   if not (E is TEntity) then exit;
   if TEntity(E).NoElements  then exit;
   E := TEntity(E).Elements[0];
   if not (E is TAreaCollection) then exit;
   AC := TAreaCollection(E);
   if AC.Count > 50 then
   begin
      MaxProgress := AC.LastIndex;
      Application.ProcessMessages;
   end;

   for i := 0 to AC.LastIndex do
   begin
      E := AC.Elements[i];
      if not (E is TGeometryElement) then continue;
      G := TGeometryElement(E);
      if i mod 50 = 0 then
      begin
         Progress := i;
         Application.ProcessMessages;
      end;
      // jet kontrola, jestli je to tebn plocha - neplochy nebrat
      if G.Group <> AreasGroup then continue;
      // Vroba plochy
      TEZPlocha := TTEZPlocha.Create;
      TEZPlocha.Geometry := G.Geometry;
      // Hledme skupinu podle prostoru
      FMSelector.X := TezPlocha.X;
      FMSelector.Y := TezPlocha.Y;
      FMSelector.FrameClass := Tgis_PorostniSkupina;
      FMSelector.DoIt;
      if FMSelector.ActualFrame <> nil then // skupina dle prostoru
      begin
         PSK := Tgis_Porostniskupina(FMSelector.ActualFrame);
         TEZPlocha._Skupina := PSK;
      end
      else
      begin  // Skupina dle Atributu
         LHC := G.GetPropertyLongIntByName(LHCPropertyName);
         LS := G.GetPropertyLongIntByName(LSPropertyName);
         if LS < 1 then LS := 1;
         Oddeleni := G.GetPropertyLongIntByName(OddeleniPropertyName);
         Dilec := G.GetPropertyStringByName(DilecPropertyName);
         { TODO 5 -oPPK -cDodlat : Import LHC - Kontroly poloek a zaazovn u chybnch hodnot }
         Porost := G.GetPropertyStringByName(PorostPropertyName);
         Skupina := G.GetPropertyStringByName(SkupinaPropertyName);
         if (Dilec = '') or not(Dilec[1] in ['a'..'z','A'..'Z']) then
            Dilec := Porost;
         UseAttributes := (LHC > 0) and (Oddeleni > 0) and (Porost <> '') and
                        (Porost[1] in ['a'..'z','A'..'Z']) and (Skupina <> '');
         if UseAttributes then
         begin
            PSK := ForestModel.CreateSkupina(LHC,LS,Oddeleni,Dilec,Porost,Skupina);
            TEZPlocha._Skupina := PSK;
            ForestModel.ActualSkupina.UpdateRect(TEZPlocha);
            ForestModel.ActualPorost.UpdateRect(TEZPlocha);
            ForestModel.ActualDilec.UpdateRect(TEZPlocha);
            ForestModel.ActualOddeleni.UpdateRect(TEZPlocha);
            ForestModel.ActualLS.UpdateRect(TEZPlocha);
            ForestModel.ActualLHC.UpdateRect(TEZPlocha);
            ForestModel.UpdateRect(TEZPlocha);
         end;
      end;
      if NoTransferModel then continue;
      for j := 0 to TransferModel.LastIndex do
      begin
         S := TransferModel.StringElements[j];
         TEZPlocha.SetPropertyStringByName(S.Value,G.GetPropertyStringByName(S.Name));
      end;
      if (AreasLayer <> nil) then
      begin
         if (AreasLayer is TRectEntity) then
            TRectEntity(AreasLayer).AddAndUpdateRect(TEZPlocha)
         else
            AreasLayer.AddElement(TEZPlocha);
      end;
   end;
   Error := 0;
end;

procedure TCreateTEZModel.SetAreasGroup(const Value: integer);
begin
   FAreasGroup := Value;
end;

{ TCreateREV }

procedure TCreateREV.CreateModel;
var
   E: TElement;
   AC: TAreaCollection;
   G: TGeometryElement;
   REV: Tgis_LS;
   REVCis: string;
   i,j: integer;
   S: TStringElement;
   Newman: boolean;
   PC: TPolyLineCollection;
begin
   //inherited;
   // Plochy
   E := Data.FindElementName('LIN');
   if not (E is TEntity) then exit; // dn linie
   if TEntity(E).NoElements  then exit;
   E := TEntity(E).Elements[0];
   if not (E is TPolyLineCollection) then exit;
   PC := TPolyLineCollection(E);
   if LinesLayer = nil then
   begin
      ShowMessage(Comment+': Nen nastavena entita pro linie');
   end
   else
   begin
      if NoLineTransferModel then
      begin
         LinesLayer.AddAllFrom := E;
         if LinesLayer is TRectEntity then
            TRectEntity(LinesLayer).ResetAndUpdate;
      end
      else
      begin
         if PC.Count > 50 then
         begin
            MaxProgress := PC.LastIndex;
            Application.ProcessMessages;
         end;
         for i := 0 to PC.LastIndex do
         begin
            if i mod 50 = 0 then
            begin
               Progress := i;
               Application.ProcessMessages;
            end;
            E := PC.Elements[i];
            for j := 0 to LineTransferModel.LastIndex do
            begin
               S := LineTransferModel.StringElements[j];
               E.SetPropertyStringByName(S.Value,E.GetPropertyStringByName(S.Name));
               Application.ProcessMessages;
               if LinesLayer is TRectEntity then
                  TRectEntity(LinesLayer).AddAndUpdateRect(E)
               else
                  LinesLayer.AddElement(E);
            end;
         end;
      end;
   end;
   Progress := 0;
   Application.ProcessMessages;
   E := Data.FindElementName('ARE');
   if (E <> nil) and (E is TEntity) and not TEntity(E).NoElements  then
   begin
      E := TEntity(E).Elements[0];
      if (E is TAreaCollection) then
      begin
         if AreasLayer = nil then
         begin
            ShowMessage(Comment+': Nen nastavena entita pro Plochy');
         end
         else
         begin
            AC := TAreaCollection(E);
            for i := 0 to AC.LastIndex do
            begin
               G := TGeometryElement(AC.Elements[i]);
               if G.Group <> AreasGroup then Continue; // neplocha nebo co
               RevCis := G.GetPropertyStringByName(RevPropertyName);
               REV := Tgis_LS(AreasLayer.FindElementName(RevCis));
               NewMan := false;
               if REV = nil then
               begin
                  REV := Tgis_LS.Create;
                  REV.Name := RevCis;
                  NewMan := True;
               end;
               REV.AddGeometry(G.Geometry);
               if NewMan then
                  TRectEntity(AreasLayer).AddAndUpdateRect(REV)
               else
                  TRectEntity(AreasLayer).UpdateRect(REV);
               if NoTransferModel then continue;
               for j := 0 to TransferModel.LastIndex do
               begin
                  S := TransferModel.StringElements[j];
                  REV.SetPropertyStringByName(S.Value,G.GetPropertyStringByName(S.Name));
               end;
            end;
         end;
      end;
   end;
end;

procedure TCreateREV.SetAreasGroup(const Value: integer);
begin
  FAreasGroup := Value;
end;

procedure TCreateREV.SetRevPropertyName(const Value: string);
begin
   FRevPropertyName := Value;
end;

{ TCreateTYPModel }

procedure TCreateTYPModel.CreateModel;
{ TODO -oPPK -cfilosofovat : Typologii nasledne predelat nebo dodelat jako obecne plochy }
var
   AC: TAreaCollection;
   PC: TPolyLineCollection;
   G: TGeometryElement;
   FinalG: TGeometryElement;
   PGE: TPropertyGeometryElement;
   E: TElement;
   i,j: integer;
   S: TStringElement;
begin
   Error := 1;
   if Data.NoElements then exit;
   // Linie
   E := Data.FindElementName('LIN');
   if not (E is TEntity) then exit; // dn linie
   if TEntity(E).NoElements  then exit;
   E := TEntity(E).Elements[0];
   if not (E is TPolyLineCollection) then exit;
   PC := TPolyLineCollection(E);
   if LinesLayer = nil then
   begin
      ShowMessage(Comment+': Nen nastavena entita pro linie');
   end
   else
   begin
      if NoLineTransferModel then
      begin
         LinesLayer.AddAllFrom := E;
         if LinesLayer is TRectEntity then
            TRectEntity(LinesLayer).ResetAndUpdate;
      end
      else
      begin
         if PC.Count > 50 then
         begin
            MaxProgress := PC.LastIndex;
            Application.ProcessMessages;
         end;
         for i := 0 to PC.LastIndex do
         begin
            if i mod 50 = 0 then
            begin
               Progress := i;
               Application.ProcessMessages;
            end;
            E := PC.Elements[i];
            for j := 0 to LineTransferModel.LastIndex do
            begin
               S := LineTransferModel.StringElements[j];
               E.SetPropertyStringByName(S.Value,E.GetPropertyStringByName(S.Name));
               Application.ProcessMessages;
               if LinesLayer is TRectEntity then
                  TRectEntity(LinesLayer).AddAndUpdateRect(E)
               else
                  LinesLayer.AddElement(E);
            end;
         end;
      end;
   end;
   Progress := 0;
   Application.ProcessMessages;
   // Plochy
   E := Data.FindElementName('ARE');
   if not (E is TEntity) then exit;
   if TEntity(E).NoElements  then exit;
   E := TEntity(E).Elements[0];
   if not (E is TAreaCollection) then exit;
   AC := TAreaCollection(E);
   if AC.Count > 50 then
   begin
      MaxProgress := AC.LastIndex;
      Application.ProcessMessages;
   end;
   for i := 0 to AC.LastIndex do
   begin
      E := AC.Elements[i];
      if not (E is TGeometryElement) then continue;
      G := TGeometryElement(E);
      if i mod 50 = 0 then
      begin
         Progress := i;
         Application.ProcessMessages;
      end;
      // jet kontrola, jestli je to skupina parcel ... na neplochy kalu
      if (G.Group <> AreasGroup) then continue;
      FinalG := Tgis_LT.Create;
      FinalG.AddGeometry(G.Geometry);
      if NoTransferModel then continue;
      for j := 0 to TransferModel.LastIndex do
      begin
         S := TransferModel.StringElements[j];
         if S.Name='DYNAMIC' then
         begin
            if FinalG is TPropertyGeometryElement then
            begin
               PGE := TPropertyGeometryElement(FinalG);
               if PGE.NoProperties then
                  PGE.Properties := TDynamicProperties.Create;
               PGE.SetPropertyStringByName(S.Value,G.GetPropertyStringByName(S.Value));
            end;
         end
         else
            FinalG.SetPropertyStringByName(S.Value,G.GetPropertyStringByName(S.Name));
      end;
      if (AreasLayer <> nil) and not FinalG.IsMyOwner(AreasLayer) then
      begin
         if (AreasLayer is TRectEntity) then
            TRectEntity(AreasLayer).AddAndUpdateRect(FinalG)
         else
            AreasLayer.AddElement(FinalG);
      end;
   end;
   Error := 0;
end;

procedure TCreateTYPModel.SetAreasGroup(const Value: integer);
begin
   FAreasGroup := Value;
end;

{ TCreateSimpleHighLevelForestModel }

procedure TCreateSimpleHighLevelForestModel.CreateModel;
var
   AC: TAreaCollection;
   G: TGeometryElement;
   E: TElement;
   i,j: integer;
   IsLHC,IsOddeleni,IsDilec: boolean;
   LHC: integer;
   LS: integer;
   Oddeleni: integer;
   Dilec: string;
   S: TStringElement;
   NewObject: boolean;
begin
   Error := 1;
   if Data.NoElements then exit;
   Progress := 0;
   Application.ProcessMessages;
   // Plochy
   E := Data.FindElementName('ARE');
   if not (E is TEntity) then exit;
   if TEntity(E).NoElements  then exit;
   E := TEntity(E).Elements[0];
   if not (E is TAreaCollection) then exit;
   AC := TAreaCollection(E);
   if AC.Count > 50 then
   begin
      MaxProgress := AC.LastIndex;
      Application.ProcessMessages;
   end;
   for i := 0 to AC.LastIndex do
   begin
      E := AC.Elements[i];
      if not (E is TGeometryElement) then continue;
      G := TGeometryElement(E);
      if i mod 50 = 0 then
      begin
         Progress := i;
         Application.ProcessMessages;
      end;
      j := G.Group;
      IsLHC :=  j = LHCGroup;
      IsOddeleni := j = OddeleniGroup;
      IsDilec := j = DilecGroup;

      // jet kontrola, jestli je to aspon neco ... na neplochy kalu
      if not (IsLHC or IsOddeleni or IsDilec) then continue;
      LHC := G.GetPropertyLongIntByName(LHCPropertyName);
      ForestModel.CreateLHCCislo(LHC);
      if IsLHC then
      begin
         ForestModel.ActualLHC.AddGeometry(G.Geometry);
         continue;
      end;
      LS := G.GetPropertyLongIntByName(LSPropertyName);
      if LS < 1 then LS := 1;
      ForestModel.CreateLSCislo(LS);
      Oddeleni := G.GetPropertyLongIntByName(OddeleniPropertyName);
      ForestModel.CreateOddeleniCislo(Oddeleni);
      if IsOddeleni then
      begin
         ForestModel.ActualOddeleni.AddGeometry(G.Geometry);
         continue;
      end;
      Dilec := G.GetPropertyStringByName(DilecPropertyName);
      ForestModel.CreateDilecName(Dilec);
      ForestModel.ActualDilec.AddGeometry(G.Geometry);
   end;
   ForestModel.UpdateAll := true;
   Error := 0;
end;

procedure TCreateSimpleHighLevelForestModel.SetDilecGroup(const Value: integer);
begin
  FDilecGroup := Value;
end;

procedure TCreateSimpleHighLevelForestModel.SetLHCGroup(const Value: integer);
begin
  FLHCGroup := Value;
end;

procedure TCreateSimpleHighLevelForestModel.SetOddeleniGroup(const Value: integer);
begin
  FOddeleniGroup := Value;
end;

INITIALIZATION
   RegisterOrbisClass(TCreateForestModel);
   RegisterOrbisClass(TCreateSimpleHighLevelForestModel);
   RegisterOrbisClass(TCreateSRFModel);
   RegisterOrbisClass(TCreateTEZModel);
   RegisterOrbisClass(TCreateTYPModel);
   RegisterOrbisClass(TCreateREV);
END.
