unit T_ExpLHC;

interface
uses
// KERNEL
   Elements,
   Tools,
   e_Class,
// SPACE
   e_Geometry,
   e_GeometryElement;
   //e_Space;

type


TExportLHCBasic = class(TTool)
private
   FBlockName: string;
   FControlDIR: string;
    FComment: string;
    FFreeRegisterIfPossible: boolean;
    FDBFromSource: boolean;
    FAllInOriginal: boolean;
    procedure SetComment(const Value: string);
    procedure SetFreeRegisterIfPossible(const Value: boolean);
    procedure DeleteRegister;
    procedure SetControlDIR(const Value: string);
    procedure SetDBFromSource(const Value: boolean);
    procedure SetAllInOriginal(const Value: boolean);
published
   property BlockName: string read FBlockName write FBlockName;
   property ControlDIR: string read FControlDIR write SetControlDIR;
   property Comment: string read FComment write SetComment;
   property DBFromSource: boolean read FDBFromSource write SetDBFromSource;
   property AllInOriginal: boolean read FAllInOriginal write SetAllInOriginal;
   property FreeRegisterIfPossible: boolean read FFreeRegisterIfPossible write SetFreeRegisterIfPossible;
end;

TExportLHCLines = class(TExportLHCBasic)
private
   FLinesEntity: TEntity;
    FCreateTopology: boolean;
    FJoinLines: boolean;
    FCreateLinesCopy: boolean;
    FGeneralizeLines: boolean;
    FGenerTolerance: double;
    FChangeLinesCopy: boolean;
    FLineCopyGroup: integer;
    FLineCopySymbol: integer;
    FLineCopyColor: integer;
    FJoin_IgnorePresnost: boolean;
    FJoin_SameDirection: boolean;
   procedure SetLinesEntity(const Value: TEntity);
    procedure SetCreateTopology(const Value: boolean);
    procedure SetCreateLinesCopy(const Value: boolean);
    procedure SetGeneralizeLines(const Value: boolean);
    procedure SetGenerTolerance(const Value: double);
    procedure SetJoinLines(const Value: boolean);
    procedure SetChangeLinesCopy(const Value: boolean);
    procedure SetLineCopyColor(const Value: integer);
    procedure SetLineCopyGroup(const Value: integer);
    procedure SetLineCopySymbol(const Value: integer);
    procedure SetJoin_IgnorePresnost(const Value: boolean);
    procedure SetJoin_SameDirection(const Value: boolean);
public
   procedure DoIt; override;
   procedure PrepareLines; virtual;
   procedure DoCreateLinesCopy;
   procedure DoJoinLines;
   procedure DoGeneralizeLines;
published
   property LinesEntity: TEntity read FLinesEntity write SetLinesEntity;
   property CreateTopology: boolean read FCreateTopology write SetCreateTopology;
   // CreateTopology m smysl jen u exportovn samotnch lini
   property CreateLinesCopy: boolean read FCreateLinesCopy write SetCreateLinesCopy;
   property JoinLines: boolean read FJoinLines write SetJoinLines;
   property Join_IgnorePresnost: boolean read FJoin_IgnorePresnost write SetJoin_IgnorePresnost;
   property Join_SameDirection: boolean read FJoin_SameDirection write SetJoin_SameDirection;
   property GeneralizeLines: boolean read FGeneralizeLines write SetGeneralizeLines;
   property GenerTolerance: double read FGenerTolerance write SetGenerTolerance;
   property ChangeLinesCopy: boolean read FChangeLinesCopy write SetChangeLinesCopy;
   property LineCopyGroup: integer read FLineCopyGroup write SetLineCopyGroup;
   property LineCopySymbol: integer read FLineCopySymbol write SetLineCopySymbol;
   property LineCopyColor: integer read FLineCopyColor write SetLineCopyColor;
end;

TExportLHCLinesAndAreas = class(TExportLHCLines)
private
   FFindLinesInAreas: boolean;
   FHolesGroup: integer;
   FHolesSymbol: integer;
   FHolesColor: integer;
   FAreasEntity: TEntity;
    FExportAreas: TAreaCollection;
    FNewAreasString: TAreaString;
    FAreaElementClass: TElementClass;
    FAreaGroup: integer;
    FAreaSymbolPropertyName: string;
    FAreaColorPropertyName: string;
    FLineSymbol: integer;
    FLineGroup: integer;
    FLineColor: integer;
    FExportLines: TPolyLineCollection;
    FAreaInHa: boolean;
    FChangeAreaColorAndSymbol: boolean;
    FAreaSymbol: integer;
    FAreaColor: integer;
    FMaxLineLevel: integer;
    FRoundArea: integer;
   procedure SetAreasEntity(const Value: TEntity);
   procedure SetFindLinesInAreas(const Value: boolean);
   procedure SetHolesColor(const Value: integer);
   procedure SetHolesGroup(const Value: integer);
   procedure SetHolesSymbol(const Value: integer);
    procedure SetExportAreas(const Value: TAreaCollection);
    procedure SetNewAreasString(const Value: TAreaString);
    procedure SetAreaElementClass(const Value: TElementClass);
    procedure SetAreaElementClassName(const Value: string);
    function  GetAreaElementClassName: string;
    procedure SetAreaColorPropertyName(const Value: string);
    procedure SetAreaGroup(const Value: integer);
    procedure SetAreaSymbolPropertyName(const Value: string);
    procedure SetLineColor(const Value: integer);
    procedure SetLineGroup(const Value: integer);
    procedure SetLineSymbol(const Value: integer);
    procedure SetExportLines(const Value: TPolyLineCollection);
    procedure SetAreaInHa(const Value: boolean);
    procedure SetAreaColor(const Value: integer);
    procedure SetAreaSymbol(const Value: integer);
    procedure SetChangeAreaColorAndSymbol(const Value: boolean);
    procedure SetMaxLineLevel(const Value: integer);
    procedure SetRoundArea(const Value: integer);
public
   procedure DoIt; override;
   procedure CreateNewAreas;
   procedure CreateExportAreas;
   procedure CreateExportLines;
   procedure DoExport;
   property  NewAreasString: TAreaString read FNewAreasString write SetNewAreasString;
   property  ExportAreas: TAreaCollection read FExportAreas write SetExportAreas;
   property  ExportLines: TPolyLineCollection read FExportLines write SetExportLines;
   property  AreaElementClass: TElementClass read FAreaElementClass write SetAreaElementClass;
published
   property AreasEntity: TEntity read FAreasEntity write SetAreasEntity;
   property FindLinesInAreas: boolean read FFindLinesInAreas write SetFindLinesInAreas;
   property MaxLineLevel: integer read FMaxLineLevel write SetMaxLineLevel;
   property AreaInHa: boolean read FAreaInHa write SetAreaInHa;
   property RoundArea: integer read FRoundArea write SetRoundArea;
   property AreaGroup: integer read FAreaGroup write SetAreaGroup;
   property ChangeAreaColorAndSymbol: boolean read FChangeAreaColorAndSymbol write SetChangeAreaColorAndSymbol;
   property AreaColor: integer read FAreaColor write SetAreaColor;
   property AreaSymbol: integer read FAreaSymbol write SetAreaSymbol;
   property AreaColorPropertyName: string read FAreaColorPropertyName write SetAreaColorPropertyName;
   property AreaSymbolPropertyName: string read FAreaSymbolPropertyName write SetAreaSymbolPropertyName;
   property HolesGroup: integer read FHolesGroup write SetHolesGroup;
   property HolesColor: integer read FHolesColor write SetHolesColor;
   property HolesSymbol: integer read FHolesSymbol write SetHolesSymbol;
   property AreaElementClassName: string read GetAreaElementClassName write SetAreaElementClassName;
   property LineGroup: integer read FLineGroup write SetLineGroup;
   property LineSymbol: integer read FLineSymbol write SetLineSymbol;
   property LineColor: integer read FLineColor write SetLineColor;
end;

TExportLHCPoints = class(TExportLHCBasic)
  private
    FPointsEntity: TEntity;
    procedure SetPointsEntity(const Value: TEntity);
public
   procedure DoIt; override;
published
   property PointsEntity: TEntity read FPointsEntity write SetPointsEntity;
end;

TExportLHCTexts = class(TExportLHCBasic)
  private
    FCodePageStr: string;
    procedure SetCodePageStr(const Value: string);
public
   procedure DoIt; override;
published
   property CodePageStr: string read FCodePageStr write SetCodePageStr;
end;


implementation
uses
   Classes,
   Dialogs,
   Forms,
// KERNEL
   worlds,
// GLIB
   G_Elems,
   e_QTree,
   MathLib,
// TOPOL
   //T_areare,
   e_TopolElements,
   e_Node,
   u_String,
   u_Symbol,
   T_AreMak,
   T_ExArTL,
   T_ExLnTl,
   T_ExPnTl,
   T_TopExp,
   t_FindBreakLineConnections,
   T_LineExtractor,
   T_QTreeTools,
   T_Topol,
   T_ToTool;

   //E_TplPrp,
//
   //e_LHCElements;



{ TExportLHCLines }

procedure TExportLHCLines.DoCreateLinesCopy;
var
   LC: TPolyLineCollection;
   i,j: integer;
   E: TElement;
   TL,TL1: TTopolLineElement;
   L,L1: TPolyLineElement;
begin
   if LinesEntity = nil then exit;
   LC := TPolyLineCollection.Create;
   AddElement(LC);
   LC.Name := 'Lines copy';
   ProgressText := 'Koprovn lini';
   MaxProgress := LinesEntity.LastIndex;
   for i := 0 to LinesEntity.LastIndex do
   begin
      Progress := i;
      Application.ProcessMessages;
      E := LinesEntity.Elements[i];
      if not (E is TTopolLineElement) then continue;
      TL := TTopolLineElement(E);
      if TL.NoGeometry then continue;
      L := TPolyLineElement(TL.Geometry);
      TL1 := TTopolLineElement(CreateElementCopy(TL));
      TL.AfterCopy(TL1);
      L1 := TPolyLineElement(CreateElementCopy(L));
      for j := 0 to L.LastIndex do
      begin
         L1.AddXY(L.Xi[j],L.Yi[j]);
      end;
      TL1.Geometry := L1;
      LC.AddAndUpdateRect(TL1);
      if ChangeLinesCopy then
      begin
         TL1.Druh := LineCopyGroup;
         TL1.Barva := LineCopyColor;
         TL1.Znacka := LineCopySymbol;
      end;
   end;
   LinesEntity := LC;
   Progress := 0;
   MaxProgress := 0;
end;

procedure TExportLHCLines.DoGeneralizeLines;
var
   i,j: integer;
   TL: TTopolLineElement;
   L: TPolyLineElement;
   E: TElement;
   first,last: integer;

function Test(i1,i2: integer): boolean;
var
   i: integer;
   X1,Y1,X2,Y2,PX,PY,dP: double;
   PataIn: boolean;
begin
   result := true;
   if (i2 < 0)or((i1 - i2) < 2) then
   begin
      result := false;
      exit;
   end;
   X1 := L.Xi[i1];
   Y1 := L.Yi[i1];
   X2 := L.Xi[i2];
   Y2 := L.Yi[i2];
   for i := i1-1 downto i2+1 do
   begin
       DejVztahyBod_Usecka(L.Xi[i], L.Yi[i],X1,Y1,X2,Y2,PataIn,PX,PY,dP);
       if dP > GenerTolerance then
       begin
          result := false;
          exit;
       end;
   end;
end;

begin
   if LinesEntity = nil then exit;
   if GenerTolerance <= 0 then GenerTolerance := 0.001;
   progressText := 'Generalizace...';
   for j := 0 to LinesEntity.LastIndex do
   begin
      Progress := j;
      Application.ProcessMessages;
      E := LinesEntity.Elements[j];
      if not (E is TTopolLineElement) then continue;
      TL := TTopolLineElement(E);
      if TL.NoGeometry then continue;
      L := TPolyLineElement(TL.Geometry);
      if L.Count < 3 then continue;
      first := L.LastIndex;
      Last :=  first - 1;
      while last >= 0 do
      begin
         if Test(first,last-1) then
         begin
            dec(last);
         end
         else
         begin
            if (first - last)>= 2 then
            begin
               for i := first-1 downto last + 1 do
               begin
                  L.FreeElement(L.Elements[i]);
               end;
            end;
            first := last;
            dec(last);
         end;
      end;
   end;
end;

procedure TExportLHCLines.DoIt;
var
   Exp: TExportLinesToTopol;
begin
   if (BlockName = '') or (BlockName = '0') then exit;
   if LinesEntity = nil then exit;
   if LinesEntity.NoElements then exit;
{ TODO -oPPK -cDodlat : Vpis do protokolu }
   StartProgress(Comment,0,0);
   if not AllInOriginal then
   begin
      if CreateLinesCopy then
         DoCreateLinesCopy;
      if JoinLines then
         DoJoinLines;
      if GeneralizeLines then
         DoGeneralizeLines;
   end;
   ProgressText := 'Tvorba Topologie';
   Application.ProcessMessages;
   PrepareLines;
   ProgressText := '... a vlastn export';
   Application.ProcessMessages;
   Exp := TExportLinesToTopol.Create;
   try
     Exp.Input:= LinesEntity;
     Exp.Name:= 'Export lini do bloku ' + BlockName;
     Exp.BlockName:= BlockName;
     Exp.ControlDir:= ControlDIR; //LCRGisDir;
 //  ArExp.DruhLinii:= 1100;
     Exp.RegisterObject := true;
     Exp.ExportDBF:= true;
     Exp.CreateFromSource := DBFromSource;
     Exp.NoTopology := not CreateTopology;
{ TODO -oPPK -cOpravit : DB  u lini .. bhv jak to je }
     Exp.DoIt;
   finally
     Exp.Free;
   end;
   DeleteRegister;
end;

procedure TExportLHCLines.DoJoinLines;
var
  FB: TFindBreakLineConnections;
  E: TEntity; // entita uzl
  El: TElement;
  N,NN: TNode;
  i,j: integer;
  TL1,TL2,TL: TTopolLineElement;
  L1,L2,L: TPolyLineElement;
  aX,aY,X1,Y1: double;
  B1,B2: boolean;
  NB1,NB2,NB: TNodeBranch;
begin
  ProgressText := 'Spojen lini...';
  Application.ProcessMessages;
  FB := TFindBreakLineConnections.Create;
  FB.ConnectionsNumber := 2; //budu hledat uzly dvou lini
  FB.Input := LinesEntity;
  FB.DoIt;
  E := TEntity(FB.output);
  FB.Free;
  if E = nil then
  begin
     ShowMessage('Spojen lini se z njakho dvodu nepodailo, asi bude pret');
     exit;
  end;
  MaxProgress := E.LastIndex;
  for i := 0 to E.LastIndex do
  begin
     Progress := i;
     Application.ProcessMessages;
     N := TNode(E.Elements[i]);
     NN := N;
     L1 := nil;
     L2 := nil;
     TL1 := nil;
     TL2 := nil;
     NB1 := nil;
     NB2 := nil;
     for j := 0 to N.LastIndex do
     begin
        El := N.Elements[j];
        if not (El is TNodeBranch) then continue;
        NB := TNodeBranch(El);
        if L1 = nil then
        begin
           NB1 := NB;
           L1 := NB1.Line;
           TL1 := TTopolLineElement(L1.FindClassWithOwnerInOwners(TTopolLineElement,LinesEntity));
        end
        else
        begin
           NB2 := NB;
           L2 := NB2.Line;
           TL2 := TTopolLineElement(L2.FindClassWithOwnerInOwners(TTopolLineElement,LinesEntity));
           Break;
        end;
     end;
     if (L1 = nil) or (L2 = nil) then continue;
     if L1 = L2 then continue;
     if (TL1 = nil) or (TL2 = nil) then continue;
     if (TL1.Druh <> TL2.Druh) then continue;
     if not Join_IgnorePresnost then
        if (TL1.Presnost <> TL2.Presnost) then continue;
     if (NB1.LineBegin = NB2.LineBegin) and Join_SameDirection then continue;
     if (TL1.Symbol > 0) and (TL2.Symbol > 0) then continue;
     if (TL1.Symbol <= 0) and (TL2.Symbol > 0) then
     begin
        TL := TL2;
        L := L2;
        NB := NB2;
        TL2 := TL1;
        L2 := L1;
        NB2 := NB1;
        TL1 := TL;
        L1 := L;
        NB1 := NB;
     end;
     aX := TL1.AbsRefX;
     aY := TL1.AbsRefY;
     for j := 1 to L2.LastIndex do
     begin
        L2.GetOrientedCoord(j,NB2.LineBegin,X1,Y1);
        if NB1.LineBegin then //bude se pidvat na zatek
           L1.InsertXY(0,X1,Y1)
        else                  //nebo na konec
           L1.AddXY(X1,Y1);
     end;
     L1.RecalculateRect;
     L1.AbsRefX := aX;
     L1.AbsRefY := aY;
     NN.FreeAll;
     GiveNodeBranchForLine(L2, not(NB2.LineBegin),N,NB);
     if (N <> nil) and (NB <> nil) then
     begin
        N.RemoveElement(L2);
        N.AddElement(L1);
        NB.Line := L1;
        NB.LineBegin := NB1.LineBegin;
        //NB.Free;
     end;
     //if NB2 <> nil then NB2.Free;
     TL2.Free;
  end;
  E.Free;
  Progress := 0;
end;

procedure TExportLHCLines.PrepareLines;
var
   S: string;
   Tool: TTool;
   DoNodes: TPutNodesIntoQTree;
   NodeCon: TNodesConector;
   //RusTrc: TRemoveSingleLinesInQTree;
   RusLin: TDeleteDirectRelationsToLinesInQTree;
   QTree: TSpaceLineBegEndCell;
begin
   if LinesEntity = nil then Exit;
   if not CreateTopology then exit;
   S := 'Natn lini do QTREE';
   //ProgressText := S;
   Tool:= TReadLinesIntoQTree.Create;
   Tool.Input:= LinesEntity;
   Tool.Name:= S;
   Tool.DoIt;
   QTree:= TSpaceLineBegEndCell(Tool.Output);
   AddElement(QTree); // QTree mus zstat ... je v nm cel topologie
   Tool.Free;

   S := 'Vytven uzl v QTREE';
   //ProgressText := S;
   DoNodes:= TPutNodesIntoQTree.Create;
   DoNodes.Name:= S;
   DoNodes.Input:= QTree;
   DoNodes.DoIt;
   DoNodes.Free;

   S := 'Zapojovn uzl v QTree';
   //ProgressText := S;
   NodeCon:= TNodesConector.Create;
   NodeCon.Name:= S;
   NodeCon.Input:= QTree;
   NodeCon.DeletePseudoNodes:= false; //ReadIniBoolean('EXPORT', 'RusitPseudouzly', false);
   NodeCon.JoinDifferentLines:= false; //ReadIniBoolean('EXPORT', 'RusitPseudouzlyIProRuzneDruhy', true);
   NodeCon.MaxCoordsCount:= 200; //ReadIniInteger('EXPORT', 'MaxPocetSouradnic', 100);
   NodeCon.DoIt;
   NodeCon.Free;

   (*
   RusTrc:= TRemoveSingleLinesInQTree.Create;
   RusTrc.Input:= QTree;
   RusTrc.DoIt;
   RusTrc.Free;
   *)
   S := 'Ruen QTree vazeb na linie';
   //ProgressText := S;
   RusLin:= TDeleteDirectRelationsToLinesInQTree.Create;
   RusLin.Name:= S;
   RusLin.Input:= QTree;
   RusLin.DoIt;
   RusLin.Free;
end;

procedure TExportLHCLines.SetChangeLinesCopy(const Value: boolean);
begin
  FChangeLinesCopy := Value;
end;

procedure TExportLHCLines.SetCreateLinesCopy(const Value: boolean);
begin
  FCreateLinesCopy := Value;
end;

procedure TExportLHCLines.SetCreateTopology(const Value: boolean);
begin
   FCreateTopology := Value;
end;

procedure TExportLHCLines.SetGeneralizeLines(const Value: boolean);
begin
  FGeneralizeLines := Value;
end;

procedure TExportLHCLines.SetGenerTolerance(const Value: double);
begin
  FGenerTolerance := Value;
end;

procedure TExportLHCLines.SetJoinLines(const Value: boolean);
begin
  FJoinLines := Value;
end;

procedure TExportLHCLines.SetJoin_IgnorePresnost(const Value: boolean);
begin
  FJoin_IgnorePresnost := Value;
end;

procedure TExportLHCLines.SetJoin_SameDirection(const Value: boolean);
begin
  FJoin_SameDirection := Value;
end;

procedure TExportLHCLines.SetLineCopyColor(const Value: integer);
begin
  FLineCopyColor := Value;
end;

procedure TExportLHCLines.SetLineCopyGroup(const Value: integer);
begin
  FLineCopyGroup := Value;
end;

procedure TExportLHCLines.SetLineCopySymbol(const Value: integer);
begin
  FLineCopySymbol := Value;
end;

procedure TExportLHCLines.SetLinesEntity(const Value: TEntity);
begin
  FLinesEntity := Value;
end;

{ TExportLHCLinesAndAreas }

procedure TExportLHCLinesAndAreas.CreateExportAreas;
var
   Are,Are1: TPolyLineAreaElement;
   GAre: TGeometryElement;
   TExpAre: TTopolExportAreaElement;
   i: integer;
   OQTree: TRectEntityQTree;
   AddToQTree: TAddEntityToQTree;
   FindE: TFindRectEntityInQTreeByXY;
begin
   if NewAreasString = nil then exit;
   ExportAreas := TAreaCollection.Create;
   AddToQTree := TAddEntityToQTree.Create;
   AddToQTree.Input := AreasEntity;
   AddToQTree.DoIt;
   OQTree := TRectEntityQTree(AddToQTree.Output);
   AddElement(OQTree);
   AddToQTree.Free;
   FindE := TFindRectEntityInQTreeByXY.Create;
   FindE.Input := OQTree;
   if NewAreasString.LastIndex > 50 then
   begin
      MaxProgress := NewAreasString.LastIndex;
      Application.ProcessMessages;
   end;
   for i := 0 to NewAreasString.LastIndex do
   begin
      if i mod 50 = 0 then
      begin
         Progress := i;
         Application.ProcessMessages;
      end;
      // Vytvoen ExportArea + pipojen star GPlochy
      Are := TPolyLineAreaElement(NewAreasString.Elements[i]);
      //!!!!!!!!!
      //if not Are.IsHitByXY(Are.X,Are.Y,0) then
      //   ShowMessage('popisovej bod v p.....!');
      TExpAre := TTopolExportAreaElement.Create;
      FindE.X := Are.X;
      FindE.Y := Are.Y;
      FindE.DoIt;
      GAre := TGeometryElement(FindE.Output);
      if GAre = nil then // plocha nenalezena - dra, vyrbme neplochu
      begin
         TExpAre.Geometry := Are;
         TExpAre.Druh := HolesGroup;
         TExpAre.Barva := HolesColor;
         TExpAre.Znacka := HolesSymbol;
      end
      else // tak se nala, pipojme j a penesem vlastnosti
      begin
         //TExpAre.Geometry := GAre.GetSimpleGeometryByXY(Are.X,Are.Y,0);
         { TODO -oPPK -cfilosofovat : zmny kvli koprovn lini }
         Are1 := TPolyLineAreaElement(GAre.GetSimpleGeometryByXY(Are.X,Are.Y,0));
         Are.FixedArea := Are1.FixedArea;
         Are.FixedAreaIndex := Are1.FixedAreaIndex;
         // popisove body ... bylo to na hovno
         //Are.AbsRefX := Are1.X;
         //Are.AbsRefY := Are1.Y;
         // popisove body
         TExpAre.Geometry := Are;
         // zmny kvli koprovn lini, spojovn a tak
         // if TExpGeometry.NoGeometry then ;
         // Kontrola, zda Geometry existuje
         TExpAre.ExportProperties := GAre;
         if AreaGroup = 0 then
            TExpAre.Druh := GAre.GetPropertyLongIntByName('Group')
         else
            TExpAre.Druh := AreaGroup;
         if CHangeAreaColorAndSymbol then
         begin
            TExpAre.Barva := AreaColor;
            TExpAre.Znacka := AreaSymbol;
         end
         else
         begin
            TExpAre.Barva := GAre.GetPropertyLongIntByName(AreaColorPropertyName);
            TExpAre.Znacka := GAre.GetPropertyLongIntByName(AreaSymbolPropertyName);
         end;
      end;
      ExportAreas.AddAndUpdateRect(TExpAre);
   end;
   MaxProgress := 0;
   Application.ProcessMessages;
end;

procedure TExportLHCLinesAndAreas.CreateExportLines;
var
   E: TEntity;
   LE: TLineExtractor;
   i: integer;
   L: TPolyLineElement;
   TL: TTopolLineElement;
   El: TElement;
begin
   ExportLines := TPolyLineCollection.Create;
   LE := TLineExtractor.Create;
   LE.Input := AreasEntity;
   LE.NoDuplicates := True;
   LE.MaxLevel := MaxLineLevel;
   LE.DoIt;
   E := TEntity(LE.Output);
   for i := 0 to E.LastIndex do
   begin
      El := E.Elements[i];
      if not(El is TPolyLineElement) then continue;
      L := TPolyLineElement(El);
      TL := TTopolLineElement.Create;
      TL.Geometry := L;
      TL.Druh := LineGroup;
      TL.Barva := LineColor;
      TL.Znacka := LineSymbol;
      ExportLines.AddAndUpdateRect(TL);
   end;
   LinesEntity := ExportLines;
   LE.Free;
end;

procedure TExportLHCLinesAndAreas.CreateNewAreas;
var
   MA: TAreasFromClearLines;
begin
   NewAreasString := nil;
   MA := TAreasFromClearLines.Create;
   MA.Input := LinesEntity;
   MA.DoIt;
   NewAreasString := TAreaString(MA.Output); // nov vytvoen Plochy
   AddElement(MA.QTree);// uchovn topologie lini
   MA.Free;
end;

procedure TExportLHCLinesAndAreas.DoExport;
var
  ArExp: TExportAreasToTopol;
begin
  ArExp:= TExportAreasToTopol.Create;
  try
    if AllInOriginal then
    begin
       ArExp.Input:= AreasEntity;
       ArExp.Lines := LinesEntity;
    end
    else
    begin
       ArExp.Input:= ExportAreas;
       ArExp.Lines := LinesEntity;
    end;
    ArExp.Name:= 'Export ploch a tak do bloku ' + BlockName;
    ArExp.BlockName:= BlockName;
    ArExp.ControlDir:= ControlDIR; //LCRGisDir;
//    ArExp.DruhLinii:= 1100;
    ArExp.RegisterObject := true;
    ArExp.ExportDBF:= true;
    ArExp.DBFLinii := true;
    ArExp.CreateFromSource := DBFromSource;
    ArExp.PlochyVHa := AreaInHa;
    ArExp.RoundArea := RoundArea;
    ArExp.DoIt;
  finally
    ArExp.Free;
  end;

  //ShowMessage('Konec exportu vrstvy ' + LayerName + ' do bloku ' + BlockName);
end;

procedure TExportLHCLinesAndAreas.DoIt;
begin
   // nsledujc kontroly vybavit vpisem do protokolu
   if (BlockName = '') or (BlockName = '0') then exit;
 { TODO -oPPK -cDodlat : Vpis do protokolu }
   if AreasEntity = nil then exit;
   if AreasEntity.NoElements then exit;
   if (LinesEntity = nil) and not FindLinesInAreas then exit;
   StartProgress('Export do bloku: '+BlockName,0,0);
   if FindLinesInAreas then
   begin
      ProgressText := 'Extrakce lini';
      Application.ProcessMessages;
      CreateExportLines;
   end;
   if not AllInOriginal then
   begin
      if CreateLinesCopy then
         DoCreateLinesCopy;
      if JoinLines then
         DoJoinLines;
      if GeneralizeLines then
         DoGeneralizeLines;
      ProgressText := 'Tvorba neploch';
      Application.ProcessMessages;
      CreateNewAreas;
      ProgressText := 'Pprava ploch na export';
      Application.ProcessMessages;
      CreateExportAreas;
   end
   else
   begin
      CreateTopology := true;
      PrepareLines;
   end;
   ProgressText := 'Vlastn export...';
   Application.ProcessMessages;
   DoExport;
   ProgressText := 'Hotovo!';
   Application.ProcessMessages;
   DeleteRegister;
end;

function TExportLHCLinesAndAreas.GetAreaElementClassName: string;
begin
   result := '';
   if AreaElementClass = nil then exit;
   result := AreaElementClass.ClassName;
end;

procedure TExportLHCLinesAndAreas.SetAreaColor(const Value: integer);
begin
  FAreaColor := Value;
end;

procedure TExportLHCLinesAndAreas.SetAreaColorPropertyName(
  const Value: string);
begin
  FAreaColorPropertyName := Value;
end;

procedure TExportLHCLinesAndAreas.SetAreaElementClass(const Value: TElementClass);
begin
   FAreaElementClass := Value;
end;

procedure TExportLHCLinesAndAreas.SetAreaElementClassName(const Value: string);
begin
   AreaElementClass := GetOrbisClass(Value);
end;

procedure TExportLHCLinesAndAreas.SetAreaGroup(const Value: integer);
begin
  FAreaGroup := Value;
end;

procedure TExportLHCLinesAndAreas.SetAreaInHa(const Value: boolean);
begin
  FAreaInHa := Value;
end;

procedure TExportLHCLinesAndAreas.SetAreasEntity(const Value: TEntity);
begin
   ExchangeFieldElement(TElement(FAreasEntity),Value);
end;


procedure TExportLHCLinesAndAreas.SetAreaSymbol(const Value: integer);
begin
  FAreaSymbol := Value;
end;

procedure TExportLHCLinesAndAreas.SetAreaSymbolPropertyName(
  const Value: string);
begin
  FAreaSymbolPropertyName := Value;
end;

procedure TExportLHCLinesAndAreas.SetChangeAreaColorAndSymbol(
  const Value: boolean);
begin
  FChangeAreaColorAndSymbol := Value;
end;

procedure TExportLHCLinesAndAreas.SetExportAreas(const Value: TAreaCollection);
begin
   ExchangeFieldElement(TElement(FExportAreas),Value);
end;

procedure TExportLHCLinesAndAreas.SetExportLines(const Value: TPolyLineCollection);
begin
   ExchangeFieldElement(TElement(FExportLines),Value);
end;

procedure TExportLHCLinesAndAreas.SetFindLinesInAreas(
  const Value: boolean);
begin
  FFindLinesInAreas := Value;
end;

procedure TExportLHCLinesAndAreas.SetHolesColor(const Value: integer);
begin
  FHolesColor := Value;
end;

procedure TExportLHCLinesAndAreas.SetHolesGroup(const Value: integer);
begin
  FHolesGroup := Value;
end;

procedure TExportLHCLinesAndAreas.SetHolesSymbol(const Value: integer);
begin
  FHolesSymbol := Value;
end;


procedure TExportLHCLinesAndAreas.SetLineColor(const Value: integer);
begin
  FLineColor := Value;
end;

procedure TExportLHCLinesAndAreas.SetLineGroup(const Value: integer);
begin
  FLineGroup := Value;
end;

procedure TExportLHCLinesAndAreas.SetLineSymbol(const Value: integer);
begin
  FLineSymbol := Value;
end;

procedure TExportLHCLinesAndAreas.SetMaxLineLevel(const Value: integer);
begin
  FMaxLineLevel := Value;
end;

procedure TExportLHCLinesAndAreas.SetNewAreasString(const Value: TAreaString);
begin
   ExchangeFieldElement(TElement(FNewAreasString), Value);
end;

procedure TExportLHCLinesAndAreas.SetRoundArea(const Value: integer);
begin
  FRoundArea := Value;
end;

{ TExportLHCBasic }

procedure TExportLHCBasic.DeleteRegister;
var
   E: TElement;
begin
   if not FreeRegisterIfPossible then exit;
   E := World.ObjectModel.FindElementName('TRegister');
   if E <> nil then
      E.Free;
end;

procedure TExportLHCBasic.SetAllInOriginal(const Value: boolean);
begin
  FAllInOriginal := Value;
end;

procedure TExportLHCBasic.SetComment(const Value: string);
begin
  FComment := Value;
end;

procedure TExportLHCBasic.SetControlDIR(const Value: string);
begin
  FControlDIR := GetAbsolutePath(Value);
end;

procedure TExportLHCBasic.SetDBFromSource(const Value: boolean);
begin
  FDBFromSource := Value;
end;

procedure TExportLHCBasic.SetFreeRegisterIfPossible(const Value: boolean);
begin
  FFreeRegisterIfPossible := Value;
end;

{ TExportLHCPoints }

procedure TExportLHCPoints.DoIt;
var
  PntExp: TExportPointsToTopol;
begin
   if (BlockName = '') or (BlockName = '0') then exit;
   if PointsEntity = nil then exit;
   if PointsEntity.NoElements then exit;
   StartProgress(Comment,0,0);
   PntExp:= TExportPointsToTopol.Create;
   try
     PntExp.Input:= PointsEntity;
     PntExp.Name:= 'Export bod do bloku ' + BlockName;
     PntExp.BlockName:= BlockName;
     PntExp.ControlDir:= ControlDIR; //LCRGisDir;
 //    ArExp.DruhLinii:= 1100;
     PntExp.ExportDBF:= true;
     PntExp.CreateFromSource := DBFromSource;
     PntExp.DoIt;
   finally
     PntExp.Free;
   end;
   DeleteRegister;
end;

procedure TExportLHCPoints.SetPointsEntity(const Value: TEntity);
begin
  FPointsEntity := Value;
end;

{ TExportLHCTexts }

procedure TExportLHCTexts.DoIt;
var
   TE: TEntity;
   e: TElement;
   TX: TTopolTextElement;
   TW: TTopolTextWriter;
   i: integer;
   SL: TStringList;
   S: string;
begin
   if (BlockName = '') or (BlockName = '0') then exit;
   if Input = nil then exit;
   if not (Input is TEntity) then exit;
   TE := TEntity(Input);
   if TE.NoElements then exit;
   if BlockName = '' then exit;
   TW := TTopolTextWriter.CreateBlockWCS(BlockName,nil);
   TW.WriteText(nil);
   StartProgress(Comment,0,TE.LastIndex);
   ProgressText := BlockName;
   Application.ProcessMessages;
   for i := 0 to TE.LastIndex do
   begin
      if i mod 50 = 0 then
      begin
         Progress := i;
         Application.ProcessMessages;
      end;
      e := TE.Elements[i];
      if not (e is TTopolTextElement) then exit;
      TX := TTopolTextElement(e);
      TX.Cislo := i + 1;
      TW.WriteText(TX);
   end;
   TW.Free;
   SL := TStringList.Create;
   { TODO 1 -oPPK -cDodlat : Kdov strnka ... dodlat (pevody textu do cp ap.) }
   SL.Add('SOURADNYSYSTEM=JTSK');
   S := CodePageStr;
   if S = '' then
   case DefaultCodePage of
      cpLatin2: S := '852';
      cpWindows: S := '1250';
   else
      S := 'HOGOFOGO';
   end;
   SL.Add('CODEPAGE='+S);
   S:= BlockName;
   CheckBackSlash(S);
   SL.SaveToFile(S+'INFO.TXT');
   SL.Free;
end;

procedure TExportLHCTexts.SetCodePageStr(const Value: string);
begin
  FCodePageStr := Value;
end;


INITIALIZATION
  RegisterOrbisClass(TExportLHCLines);
  RegisterOrbisClass(TExportLHCLinesAndAreas);
  RegisterOrbisClass(TExportLHCPoints);
  RegisterOrbisClass(TExportLHCTexts);
END.
