unit t_CheckForestParcelConnection;

interface
uses
   Tools,
   e_KatastrModel,
   Elements;

type

TCheckForestParcelDetailConnection = class(TTool)
  private
    FDetLayer: TEntity;
    FParcelModel: Tgis_ParcelModel;
    FSilentMode: boolean;
    procedure SetDetLayer(const Value: TEntity);
    procedure SetParcelModel(const Value: Tgis_ParcelModel);
    procedure SetSilentMode(const Value: boolean);
public
   procedure DoIt; override;
published
   property DetLayer: TEntity read FDetLayer write SetDetLayer;
   property ParcelModel: Tgis_ParcelModel read FParcelModel write SetParcelModel;
   property SilentMode: boolean read FSilentMode write SetSilentMode;
end;

TCHeckForestParcelZAKConnection = class(TCheckForestParcelDetailConnection)
  private
    FZAKLayer: TEntity;
    procedure SetZAKLayer(const Value: TEntity);
public
   procedure DoIt; override;
published
   property ZAKLayer: TEntity read FZAKLayer write SetZAKLayer;
end;

implementation
uses
   Dialogs,
   Forms,
   SysUtils,
   e_Class,
   e_Geometry,
   e_LHCModel,
   t_ParcelModelSelector;

{ TCheckForestParcelDetailConnection }

procedure TCheckForestParcelDetailConnection.DoIt;
var
   PS: TParcelModelSelector;
   i,j: integer;
   e: TElement;
   Sk: Tgis_PorostniSkupina;
   ErrL,ErrM: integer;
   ERE: TOwnerEntity;
   ErLost: TOwnerEntity;
   ErMulti:TOwnerEntity;
   aX,aY: Double;
   BaseSKP,SKP: Tgis_SkupinaParcel;
   MA: TMultiAreaElement;

   function TestSKP: Tgis_SKupinaParcel;
   begin
      result := nil;
      PS.X := aX;
      PS.Y := aY;
      PS.DoIt;
      result := PS.ActualSkupinaParcel;
   end;

   procedure AddToLost;
   begin
      ErLost.AddElement(Sk);
      Inc(ErrL);
      ProgressText := IntToStr(ErrL)+' SKP chyb a '+IntToStr(ErrM)+' Skupin ve vce SKP';
      Application.ProcessMessages;
   end;

   procedure AddToMulti;
   begin
      ErMulti.AddElement(Sk);
      Inc(ErrM);
      ProgressText := IntToStr(ErrL)+' SKP chyb a '+IntToStr(ErrM)+' Skupin ve vce SKP';
      Application.ProcessMessages;
   end;
begin
   Error := 0;
   if DetLayer = nil then
   begin
      ShowMessage('Nen nastavena vrstva ploch DETailu');
      exit;
   end;
   PS := TParcelModelSelector.Create;
   if ParcelModel = nil then
   begin
      PS.FindInput;
      ParcelModel := PS.ParcelModel;
   end
   else
      PS.ParcelModel := ParcelModel;
   if ParcelModel = nil then
   begin
      ShowMessage('Nelze najt Model parcel');
      PS.Free;
      exit;
   end;
   ParcelModel.UpdateAll := true;
   StartProgress('Kontrola propojen Lesa a parcel',0,DetLayer.LastIndex);
   ErrL := 0;
   ErrM := 0;
   ERE := TOwnerEntity.Create;
   ERE.Name := 'Chyby propojen DETailu na parcely';
   AddElement(ERE);
   Output := ERE;
   ErLost := TOwnerEntity.Create;
   ERE.AddElement(ErLost);
   ErLost.Name := 'Pod st skupiny chyb parcela';
   ErMulti := TOwnerEntity.Create;
   ERE.AddElement(ErMulti);
   ErMulti.Name := 'Skupina ve vce SKP';
   for i := 0 to DetLayer.LastIndex do
   begin
      progress := i;
      Application.ProcessMessages;
      e := DetLayer.Elements[i];
      if not(e is Tgis_PorostniSkupina) then continue;
      Sk := Tgis_PorostniSkupina(e);
      BaseSKP := nil;
      SKP := nil;
      if Sk.NoGeometry then
      begin
         aX := 0;
         aY := 0;
      end
      else
      begin
         if Sk.Geometry is TMultiAreaElement then
         begin
            MA := TMultiAreaElement(Sk.Geometry);
            BaseSKP := nil;
            for j := 0 to MA.LastIndex do
            begin
               aX := MA.AreaElement[j].X;
               aY := MA.AreaElement[j].Y;
               SKP := TestSKP;
               if SKP = nil then
               begin
                  AddToLost;
                  continue;
               end;
               if BaseSKP = nil then
               begin
                  BaseSKP := SKP;
                  SK.SkupinaParcel := SKP;
               end
               else
               begin
                  if BaseSKP <> SKP then
                  begin
                     AddToMulti;
                     Break;
                  end;
               end;
            end;
         end
         else
         begin
            aX := Sk.Geometry.X;
            aY := Sk.Geometry.Y;
            SKP := TestSKP;
            SK.SkupinaParcel := SKP;
            if SKP = nil then
               AddToLost;
         end;
      end;
   end;
   PS.Free;
   Error := ErrL + ErrM;
   if not SilentMode then
      ShowMessage('Hotovo, '+IntToStr(DetLayer.Count)+ ' ploch aktualizovno, '+IntToStr(ErrL)+' skupin parcel nenalezeno, '+IntToStr(ErrM)+'por.sk ve vce SKP');
end;

procedure TCheckForestParcelDetailConnection.SetDetLayer(const Value: TEntity);
begin
  FDetLayer := Value;
end;

procedure TCheckForestParcelDetailConnection.SetParcelModel(const Value: Tgis_ParcelModel);
begin
  FParcelModel := Value;
end;

procedure TCheckForestParcelDetailConnection.SetSilentMode(
  const Value: boolean);
begin
  FSilentMode := Value;
end;

{ TCHeckForestParcelZAKConnection }

procedure TCHeckForestParcelZAKConnection.DoIt;
var
   PS: TParcelModelSelector;
   i,j: integer;
   e: TElement;
   P: Tgis_Porost;
   ErrL,ErrM: integer;
   ERE: TOwnerEntity;
   ErLost: TOwnerEntity;
   ErMulti:TOwnerEntity;
   aX,aY: Double;
   BaseSKP,SKP: Tgis_SkupinaParcel;
   MA: TMultiAreaElement;

   function TestSKP: Tgis_SKupinaParcel;
   begin
      result := nil;
      PS.X := aX;
      PS.Y := aY;
      PS.DoIt;
      result := PS.ActualSkupinaParcel;
   end;

   procedure AddToLost;
   begin
      ErLost.AddElement(P);
      Inc(ErrL);
      ProgressText := IntToStr(ErrL)+' SKP chyb a '+IntToStr(ErrM)+' Porost ve vce SKP';
      Application.ProcessMessages;
   end;

   procedure AddToMulti;
   begin
      ErMulti.AddElement(P);
      Inc(ErrM);
      ProgressText := IntToStr(ErrL)+' SKP chyb a '+IntToStr(ErrM)+' Porost ve vce SKP';
      Application.ProcessMessages;
   end;
begin
   if ZAKLayer = nil then
   begin
      ShowMessage('Nen nastavena vrstva ploch ZAKu');
      exit;
   end;
   PS := TParcelModelSelector.Create;
   if ParcelModel = nil then
   begin
      PS.FindInput;
      ParcelModel := PS.ParcelModel;
   end
   else
      PS.ParcelModel := ParcelModel;
   if ParcelModel = nil then
   begin
      ShowMessage('Nelze najt Model parcel');
      PS.Free;
      exit;
   end;
   ParcelModel.UpdateAll := true;
   StartProgress('Kontrola propojen Lesa a parcel',0,ZAKLayer.LastIndex);
   ErrL := 0;
   ErrM := 0;
   ERE := TOwnerEntity.Create;
   ERE.Name := 'Chyby propojen ZAKu na parcely';
   AddElement(ERE);
   Output := ERE;
   ErLost := TOwnerEntity.Create;
   ERE.AddElement(ErLost);
   ErLost.Name := 'Pod st porostu chyb parcela';
   ErMulti := TOwnerEntity.Create;
   ERE.AddElement(ErMulti);
   ErMulti.Name := 'Porost ve vce SKP';
   for i := 0 to ZAKLayer.LastIndex do
   begin
      progress := i;
      Application.ProcessMessages;
      e := ZAKLayer.Elements[i];
      if not(e is Tgis_Porost) then continue;
      P := Tgis_Porost(e);
      BaseSKP := nil;
      SKP := nil;
      if P.NoGeometry then
      begin
         aX := 0;
         aY := 0;
      end
      else
      begin
         if P.Geometry is TMultiAreaElement then
         begin
            MA := TMultiAreaElement(P.Geometry);
            BaseSKP := nil;
            for j := 0 to MA.LastIndex do
            begin
               aX := MA.AreaElement[j].X;
               aY := MA.AreaElement[j].Y;
               SKP := TestSKP;
               if SKP = nil then
               begin
                  AddToLost;
                  continue;
               end;
               if BaseSKP = nil then
               begin
                  BaseSKP := SKP;
                  P.SkupinaParcel := SKP;
               end
               else
               begin
                  if BaseSKP <> SKP then
                  begin
                     AddToMulti;
                     Break;
                  end;
               end;
            end;
         end
         else
         begin
            aX := P.Geometry.X;
            aY := P.Geometry.Y;
            SKP := TestSKP;
            P.SkupinaParcel := SKP;
            if SKP = nil then
               AddToLost;
         end;
      end;
   end;
   PS.Free;
   ShowMessage('Hotovo, '+IntToStr(ZAKLayer.Count)+ ' ploch aktualizovno, '+IntToStr(ErrL)+' skupin parcel nenalezeno, '+IntToStr(ErrM)+'por. ve vce SKP');
end;

procedure TCHeckForestParcelZAKConnection.SetZAKLayer(const Value: TEntity);
begin
  FZAKLayer := Value;
end;

INITIALIZATION
   RegisterOrbisClass(TCheckForestParcelDetailConnection);
   RegisterOrbisClass(TCheckForestParcelZAKConnection);
END.
