unit t_CopyLinesFromTo;

interface
uses
   Elements,
   Tools;

type

TCopyLinesFromTo = class(TTool)
  private
    FComment: string;
    FDontSaveNewLines: boolean;
    FNewLines: TEntity;
    procedure SetComment(const Value: string);
    procedure SetDontSaveNewLines(const Value: boolean);
    procedure SetNewLines(const Value: TEntity);
public
   procedure DoIt; override;
   function NumberToPropertyName(aNumber: longInt): string;override;
published
   property Comment: string read FComment write SetComment;
   property NewLines: TEntity read FNewLines write SetNewLines;
   property DontSaveNewLines: boolean read FDontSaveNewLines write SetDontSaveNewLines;
end;

THardCopyLinesFromTo = class(TCopyLinesFromTo)
public
   procedure DoIt; override;
   function NumberToPropertyName(aNumber: longInt): string;override;
end;

implementation
uses
   Dialogs,
   Forms,
   e_Class,
   e_Geometry,
   e_GeometryElement,
   G_Elems;

{ TCopyLinesFromTo }

procedure TCopyLinesFromTo.DoIt;
var
   EFrom: TEntity;
   ETo: TEntity;
   Ow: TEntity;
   i,j: integer;
   GE,NewGE: TGeometryElement;
   G: TGeometry;
   E: TElement;
   Found: boolean;
begin
   if Input = nil then
   begin
      ShowMessage(Comment + ' Chyb zdrojov vrstva!');
      exit;
   end;
   if Output = nil then
   begin
      ShowMessage(Comment + ' Chyb clov vrstva!');
      exit;
   end;
   if not (Input is TEntity) then
   begin
      ShowMessage(Comment + ' zdroj('+Input.Name+') nen entita!');
      exit;
   end;
   if not (Output is TEntity) then
   begin
      ShowMessage(Comment + ' cl('+Input.Name+') nen entita!');
      exit;
   end;
   NewLines := TPolyLineCollection.Create;
   AddElement(NewLines);
   EFrom := TEntity(Input);
   ETo := TEntity(Output);
   StartProgress(Comment,0,EFrom.LastIndex);
   for i := 0 to EFrom.LastIndex do
   begin
      if i mod 50 = 0 then
      begin
         Progress := i;
         Application.ProcessMessages;
      end;
      E := EFrom.Elements[i];
      if not (E is TGeometryElement) then continue;
      GE := TGeometryElement(E);
      G := GE.Geometry;
      Found := false;
      for j := 0 to G.OwnersCount -1 do
      begin
         Ow := G.Owners[j];
         if Ow = GE then continue;
         if Ow.IsMyOwner(ETo) then
         begin
            Found := true;
            Break;
         end;
      end;
      if not Found then
      begin
         NewGE := TGeometryElement(CreateElementCopy(GE));
         NewGE.Geometry := G;
         GE.AfterCopy(NewGE);
         if (ETo is TRectEntity) then
            TRectEntity(ETo).AddAndUpdateRect(NewGE)
         else
            ETo.AddElement(NewGE);
         if not DontSaveNewLines then
            TPolyLineCollection(NewLines).AddAndUpdateRect(NewGE);
      end;
   end;
end;

function TCopyLinesFromTo.NumberToPropertyName(aNumber: Integer): string;
begin
   case aNumber of
       0: result := 'Output';
       else
          result := 'Comment';
   end;
end;

procedure TCopyLinesFromTo.SetComment(const Value: string);
begin
  FComment := Value;
end;

procedure TCopyLinesFromTo.SetDontSaveNewLines(const Value: boolean);
begin
  FDontSaveNewLines := Value;
end;

procedure TCopyLinesFromTo.SetNewLines(const Value: TEntity);
begin
  FNewLines := Value;
end;

{ THardCopyLinesFromTo }

procedure THardCopyLinesFromTo.DoIt;
var
   EFrom: TEntity;
   ETo: TEntity;
   Ow: TEntity;
   i,j: integer;
   GE,NewGE: TGeometryElement;
   G: TGeometry;
   E: TElement;
   L,NewL: TPolyLineElement;
begin
   if Input = nil then
   begin
      ShowMessage(Comment + ' Chyb zdrojov vrstva!');
      exit;
   end;
   if Output = nil then
   begin
      ShowMessage(Comment + ' Chyb clov vrstva!');
      exit;
   end;
   if not (Input is TEntity) then
   begin
      ShowMessage(Comment + ' zdroj('+Input.Name+') nen entita!');
      exit;
   end;
   if not (Output is TEntity) then
   begin
      ShowMessage(Comment + ' cl('+Input.Name+') nen entita!');
      exit;
   end;
   EFrom := TEntity(Input);
   ETo := TEntity(Output);
   StartProgress(Comment,0,EFrom.LastIndex);
   for i := 0 to EFrom.LastIndex do
   begin
      if i mod 50 = 0 then
      begin
         Progress := i;
         Application.ProcessMessages;
      end;
      E := EFrom.Elements[i];
      if not (E is TGeometryElement) then continue;
      GE := TGeometryElement(E);
      G := GE.Geometry;
      if not (G is TPolyLineElement) then continue;
      L := TPolyLineElement(G);
      NewGE := TGeometryElement(CreateElementCopy(GE));
      NewL := TPolyLineElement(CreateElementCopy(L));
      for j := 0 to L.CoordsCount - 1 do
      begin
         NewL.AddXY(L.Xi[j],L.Yi[j]);
      end;
      NewL.RecalculateRect;
      NewL.CalculateLabelProps;
      NewGE.Geometry := NewL;
      GE.AfterCopy(NewGE);
      if (ETo is TRectEntity) then
         TRectEntity(ETo).AddAndUpdateRect(NewGE)
      else
         ETo.AddElement(NewGE);
   end;
end;

function THardCopyLinesFromTo.NumberToPropertyName(aNumber: Integer): string;
begin
   result := 'Output';
end;

INITIALIZATION
   RegisterOrbisClass(TCopyLinesFromTo);
   RegisterOrbisClass(THardCopyLinesFromTo);
END.
