unit t_JoinLHCLines;

interface
uses
   Elements,
   e_Geometry,
   Tools;

type

TJoinLHCLines = class(TTool)
  private
    FMasterLines: TEntity;
    FSlaveLines: TEntity;
    FJoinMasterLines: boolean;
    FComment: string;
    FMasterSnapDistance: double;
    FSlaveSnapDistance: double;
    FSnapInAngle: boolean;
    FAngleTolerance: double;
    FSingularities: boolean;
    FSings: TEntity;
    FLResult: TPolyLineElement;
    FLSing: TPolyLineElement;
    FLSlave: TPolyLineElement;
    FNoBorderPoints: boolean;
    FBorderLength: double;
    procedure SetMasterLines(const Value: TEntity);
    procedure SetSlaveLines(const Value: TEntity);
    procedure SetJoinMasterLines(const Value: boolean);
    procedure SetComment(const Value: string);
    procedure SetMasterSnapDistance(const Value: double);
    procedure SetSlaveSnapDistance(const Value: double);
    procedure SetAngleTolerance(const Value: double);
    procedure SetSnapInAngle(const Value: boolean);
    function  FindAngle(aX,aY,aX1,aY1: double): double;
    procedure SetSingularities(const Value: boolean);
    procedure SetSings(const Value: TEntity);
    procedure SetLResult(const Value: TPolyLineElement);
    procedure SetLSing(const Value: TPolyLineElement);
    procedure SetLSlave(const Value: TPolyLineElement);
    procedure SetBorderLength(const Value: double);
    procedure SetNoBorderPoints(const Value: boolean);
public
   procedure DoIt; override;
   property  Sings: TEntity read FSings write SetSings;
published
   property MasterLines: TEntity read FMasterLines write SetMasterLines;
   property SlaveLines: TEntity read FSlaveLines write SetSlaveLines;
   property JoinMasterLines: boolean read FJoinMasterLines write SetJoinMasterLines;
   property MasterSnapDistance: double read FMasterSnapDistance write SetMasterSnapDistance;
   property Singularities: boolean read FSingularities write SetSingularities;
   property NoBorderPoints: boolean read FNoBorderPoints write SetNoBorderPoints;
   property BorderLength: double read FBorderLength write SetBorderLength;
   property SlaveSnapDistance: double read FSlaveSnapDistance write SetSlaveSnapDistance;
   property SnapInAngle: boolean read FSnapInAngle write SetSnapInAngle;
   property AngleTolerance: double read FAngleTolerance write SetAngleTolerance;
   property Comment: string read FComment write SetComment;
   property  LResult: TPolyLineElement read FLResult write SetLResult;
   property  LSing: TPolyLineElement read FLSing write SetLSing;
   property  LSlave: TPolyLineElement read FLSlave write SetLSlave;
end;


implementation
uses
   Dialogs,
   Forms,
   SysUtils,
   MathLib,
   e_Class,
   //e_Geometry,
   e_GeometryElement,
   e_Node,
   e_QTree,
   g_Elems,
   t_FindBreakLineConnections,
   t_QTreeTools,
   t_Snap;


{ TJoinLHCLines }

procedure TJoinLHCLines.DoIt;
var
   Snap: TPointLineSnapTool;
   TQ: TAddEntityToQTree;
   LevelQT: TRectEntityQTree;
   i,SnapCount: integer;
   e: TElement;
   L: TPolyLineElement;
   FS: TFindSingularities;
   Sing: TSingularity;
   Modif: boolean;

   function IsNotShortAndUglyLine(L1,L2: TPolyLineElement; L1index: integer): boolean;
   begin
      result := true;
      if L1.Length < (2 * MasterSnapDistance) then
      begin
         if ((L1.Xi[L1index] = L2.Xi[0]) and (L1.Yi[L1index] = L2.Yi[0])) or
            ((L1.Xi[L1index] = L2.Xi[L2.LastIndex]) and (L1.Yi[L1index] = L2.Yi[L2.LastIndex])) then
               result := false; // protoze z ty linie proste trci, no
      end;
   end;

begin
   if MasterLines = nil then
   begin
      ShowMessage('JoinLHCLines - MasterLines not found !!!!');
      exit;
   end;
   StartProgress(Comment,0,0);

   LResult := TPolyLineElement.Create;
   LResult.Name := 'ResMaster';
   LSing := TPolyLineElement.Create;
   LSing.Name := 'ResSing';
   LSlave := TPolyLineElement.Create;
   LSlave.Name := 'ResSlave';

   ProgressText := 'Pprava vrstev...';
   Application.ProcessMessages;
   LevelQT :=TRectEntityQTree.Create;
   LevelQT.Name := 'Master QT';
   TQ := TAddEntityToQTree.Create;
   TQ.Input := MasterLines;
   TQ.Output := LevelQT;
   TQ.DoIt;
   TQ.Free;

   if Singularities then
   begin
      ProgressText := 'Hledn singularit...';
      Application.ProcessMessages;
      FS := TFindSingularities.Create;
      FS.Input := MasterLines;
      FS.NoBorderPoints := NoBorderPoints;
      FS.BorderLength := BorderLength;
      FS.DoIt;
      Sings := TEntity(FS.Output);
      FS.Free;
   end;

   ProgressText := 'Pipojovn Master Lines';
   MaxProgress := MasterLines.LastIndex;
   Progress := 0;
   Application.ProcessMessages;
   Snap := TPointLineSnapTool.Create;
   Snap.FrameQT := LevelQT;
   Snap.SnapToLine := true;
   Snap.SnapToLinePoint := true;
   Snap.SnapToLineBody := true;
   Snap.SnapDistance := MasterSnapDistance;
   Snap.SnapInAngle := SnapInAngle;
   Snap.AngleTolerance := AngleTolerance;
   SnapCount := 0;
   for i := 0 to MasterLines.LastIndex do
   begin
      if i mod 50 = 0 then
      begin
         Progress := i;
         Application.ProcessMessages;
      end;
      e := MasterLines.Elements[i];
      if not (e is TGeometryElement) then continue;
      L := TPolyLineElement(TGeometryElement(e).Geometry);
      if L = nil then continue;
      Modif := false; // resetujeme informaci o zmene
      // zacatek linie
      Snap.X := L.Xi[0];
      Snap.Y := L.Yi[0];
      if SnapInAngle then
         Snap.Angle := FindAngle(L.Xi[1],L.Yi[1],L.Xi[0],L.Yi[0]);
      Snap.TabuElement := L;
      Snap.DoIt;
      if Snap.Snapped {and (Snap.Output <> L)} then
      begin
         if IsNotShortAndUglyLine(L,TPolyLineElement(Snap.Output),L.LastIndex) then
         if (L.Xi[0] <> Snap.SnapX) or (L.Yi[0] <> Snap.SnapY) then
         begin
            Inc(SnapCount);
            LResult.AddXY(L.Xi[0],L.Yi[0]);
            //L.InsertXY(0,Snap.SnapX,Snap.SnapY);
            L.Xi[0] := Snap.SnapX;
            L.Yi[0] := Snap.SnapY;
            Modif := true;
         end;
      end;
      // konec linie
      Snap.X := L.Xi[L.LastIndex];
      Snap.Y := L.Yi[L.LastIndex];
      if SnapInAngle then
         Snap.Angle := FindAngle(L.Xi[L.LastIndex - 1],L.Yi[L.LastIndex - 1],L.Xi[L.LastIndex],L.Yi[L.LastIndex]);
      Snap.DoIt;
      if Snap.Snapped {and (Snap.Output <> L)} then
      begin
         if IsNotShortAndUglyLine(L,TPolyLineElement(Snap.Output),0) then
         if (L.Xi[L.LastIndex] <> Snap.SnapX) or (L.Yi[L.LastIndex] <> Snap.SnapY) then
         begin
            Inc(SnapCount);
            LResult.AddXY(L.Xi[L.LastIndex],L.Yi[L.LastIndex]);
            //L.AddXY(Snap.SnapX,Snap.SnapY);
            L.Xi[L.LastIndex] := Snap.SnapX;
            L.Yi[L.LastIndex] := Snap.SnapY;
            Modif := true;
         end;
      end;
      if Modif then
      begin
         L.Update;
         L.CalculateLabelProps;
      end;
   end;

   if Singularities then
   begin
      ProgressText := 'Pipojovn Singularit';
      MaxProgress := Sings.LastIndex;
      Progress := 0;
      Snap.SnapInAngle := false;
      for i := 0 to Sings.LastIndex do
      begin
         Progress := i;
         Application.ProcessMessages;
         e := Sings.Elements[i];
         if not (e is TSingularity) then continue;
         Sing := TSingularity(e);
         L := Sing.Line;
         Snap.TabuElement := L;
         Snap.X := Sing.X;
         Snap.Y := Sing.Y;
         Snap.DoIt;
         if Snap.Snapped {and (Snap.Output <> L)} then
         begin
            if (L.Xi[Sing.CoordIndex] <> Snap.SnapX) or (L.Yi[Sing.CoordIndex] <> Snap.SnapY) then
            begin
               Inc(SnapCount);
               LSing.AddXY(Sing.X,Sing.Y);
               //L.AddXY(Snap.SnapX,Snap.SnapY);
               L.Xi[Sing.CoordIndex] := Snap.SnapX;
               L.Yi[Sing.CoordIndex] := Snap.SnapY;
               L.Update;
               L.CalculateLabelProps;
            end;
         end;
      end;
   end;
   //ShowMessage('Master '+IntToStr(SnapCount));
   if SlaveLines = nil then
   begin
      Snap.Free;
      LevelQT.Free;
      exit;
   end;

   Snap.TabuElement := nil;
   Snap.SnapInAngle := SnapInAngle;
   Snap.SnapDistance := SlaveSnapDistance;
   ProgressText := 'Pipojovn Slave Lines';
   MaxProgress := SlaveLines.LastIndex;
   Progress := 0;
   for i := 0 to SlaveLines.LastIndex do
   begin
      if i mod 50 = 0 then
      begin
         Progress := i;
         Application.ProcessMessages;
      end;
      e := SlaveLines.Elements[i];
      if not (e is TGeometryElement) then continue;
      L := TPolyLineElement(TGeometryElement(e).Geometry);
      if L = nil then continue;
      Modif := false;
      // zacatek linie
      Snap.X := L.Xi[0];
      Snap.Y := L.Yi[0];
      if SnapInAngle then
         Snap.Angle := FindAngle(L.Xi[1],L.Yi[1],L.Xi[0],L.Yi[0]);
      Snap.DoIt;
      if Snap.Snapped then
      begin
         if (L.Xi[0] <> Snap.SnapX) or (L.Yi[0] <> Snap.SnapY) then
         begin
            // L.InsertXY(0,Snap.SnapX,Snap.SnapY);
            LSlave.AddXY(L.Xi[0],L.Yi[0]);
            L.Xi[0] := Snap.SnapX;
            L.Yi[0] := Snap.SnapY;
            Modif := true;
         end;
      end
      else
      if SnapInAngle then
      begin
         Snap.X := L.Xi[0];
         Snap.Y := L.Yi[0];
         Snap.SnapInAngle := false;
         Snap.DoIt;
         if Snap.Snapped then
         begin
            if (L.Xi[0] <> Snap.SnapX) or (L.Yi[0] <> Snap.SnapY) then
            begin
               // L.InsertXY(0,Snap.SnapX,Snap.SnapY);
               LSlave.AddXY(L.Xi[0],L.Yi[0]);
               L.Xi[0] := Snap.SnapX;
               L.Yi[0] := Snap.SnapY;
               Modif := true;
            end;
         end;
         Snap.SnapInAngle := true;
      end;
      // konec linie
      Snap.X := L.Xi[L.LastIndex];
      Snap.Y := L.Yi[L.LastIndex];
      if SnapInAngle then
         Snap.Angle := FindAngle(L.Xi[L.LastIndex - 1],L.Yi[L.LastIndex - 1],L.Xi[L.LastIndex],L.Yi[L.LastIndex]);
      Snap.DoIt;
      if Snap.Snapped then
      begin
         if (L.Xi[L.LastIndex] <> Snap.SnapX) or (L.Yi[L.LastIndex] <> Snap.SnapY) then
         begin
            //L.AddXY(Snap.SnapX,Snap.SnapY);
           LSlave.AddXY(L.Xi[L.LastIndex],L.Yi[L.LastIndex]);
           L.Xi[L.LastIndex] := Snap.SnapX;
           L.Yi[L.LastIndex] := Snap.SnapY;
           Modif := true;
         end;
      end
      else
      if SnapInAngle then
      begin
         Snap.SnapInAngle := false;
         Snap.X := L.Xi[L.LastIndex];
         Snap.Y := L.Yi[L.LastIndex];
         Snap.DoIt;
         if Snap.Snapped then
         begin
            if (L.Xi[L.LastIndex] <> Snap.SnapX) or (L.Yi[L.LastIndex] <> Snap.SnapY) then
            begin
               //L.AddXY(Snap.SnapX,Snap.SnapY);
              LSlave.AddXY(L.Xi[L.LastIndex],L.Yi[L.LastIndex]);
              L.Xi[L.LastIndex] := Snap.SnapX;
              L.Yi[L.LastIndex] := Snap.SnapY;
              Modif := true;
            end;
         end;
         Snap.SnapInAngle := true;
      end;
      if Modif then
      begin
         L.Update;
         L.CalculateLabelProps;
      end;
   end;

   Snap.Free;
   LevelQT.Free;
end;

procedure TJoinLHCLines.SetComment(const Value: string);
begin
  FComment := Value;
end;

procedure TJoinLHCLines.SetJoinMasterLines(const Value: boolean);
begin
  FJoinMasterLines := Value;
end;

procedure TJoinLHCLines.SetMasterLines(const Value: TEntity);
begin
  FMasterLines := Value;
end;

procedure TJoinLHCLines.SetSlaveLines(const Value: TEntity);
begin
  FSlaveLines := Value;
end;

procedure TJoinLHCLines.SetMasterSnapDistance(const Value: double);
begin
  FMasterSnapDistance := Value;
end;

procedure TJoinLHCLines.SetSlaveSnapDistance(const Value: double);
begin
  FSlaveSnapDistance := Value;
end;

procedure TJoinLHCLines.SetAngleTolerance(const Value: double);
begin
  FAngleTolerance := Value;
end;

procedure TJoinLHCLines.SetSnapInAngle(const Value: boolean);
begin
  FSnapInAngle := Value;
end;

function TJoinLHCLines.FindAngle(aX,aY,aX1,aY1: double): double;
var
   A, B: TDoubleCoord;
   L,Ang: double;
begin
   A.X := aX;
   A.Y := aY;
   B.X := aX1;
   B.Y := aY1;
   XYToPolar(A,B,L,Ang);
   result := Ang;
end;

procedure TJoinLHCLines.SetSingularities(const Value: boolean);
begin
  FSingularities := Value;
end;

procedure TJoinLHCLines.SetSings(const Value: TEntity);
begin
   ExchangeFieldElement(TElement(FSings), Value);
end;

procedure TJoinLHCLines.SetLResult(const Value: TPolyLineElement);
begin
   ExchangeFieldElement(TELement(FLResult), Value);
end;

procedure TJoinLHCLines.SetLSing(const Value: TPolyLineElement);
begin
   ExchangeFieldElement(TELement(FLSing), Value);
end;

procedure TJoinLHCLines.SetLSlave(const Value: TPolyLineElement);
begin
   ExchangeFieldElement(TELement(FLSlave), Value);
end;

procedure TJoinLHCLines.SetBorderLength(const Value: double);
begin
  FBorderLength := Value;
end;

procedure TJoinLHCLines.SetNoBorderPoints(const Value: boolean);
begin
  FNoBorderPoints := Value;
end;

INITIALIZATION
   RegisterOrbisClass(TJoinLHCLines);
END.
