unit t_MultiParalelLineDialog;

interface
uses
   e_Type,
   f_MultiParalelLineDialog,
   t_LineDialog;

type

TMultiParalelLineDialog = class(TLineDialog)
private
    FDruhy: TIntegerListElement;
    FDruhy1: TIntegerListElement;
    FDistance: double;
    FSatelitLineSymbol: integer;
    FCentralLineColor: integer;
    FCentralLineGroup: integer;
    FCentralLineSymbol: integer;
    FSatelitLineColor: integer;
    FSatelitLineGroup: integer;
    FAtrPuv: integer;
    FArcStep: double;
    FUseArcStep: boolean;
    function GetDruhy: TIntegerListElement;
    function GetDruhy1: TIntegerListElement;
    function GetDruhy1List: string;
    function GetDruhyList: string;
    function GetMPLForm: TMultiParalelLineDialog_Form;
    procedure SetDruhy1List(const Value: string);
    procedure SetDruhyList(const Value: string);
    procedure SetCentralLineColor(const Value: integer);
    procedure SetCentralLineGroup(const Value: integer);
    procedure SetCentralLineSymbol(const Value: integer);
    procedure SetDistance(const Value: double);
    procedure SetSatelitLineColor(const Value: integer);
    procedure SetSatelitLineGroup(const Value: integer);
    procedure SetSatelitLineSymbol(const Value: integer);
    procedure SetAtrPuv(const Value: integer);
    procedure SetArcStep(const Value: double);
    procedure SetUseArcStep(const Value: boolean);
public
   procedure DoIt; override;
   procedure CreateForm; override;
   property  MPLForm: TMultiParalelLineDialog_Form read GetMPLForm;
   property  Druhy: TIntegerListElement read GetDruhy;
   property  Druhy1: TIntegerListElement read GetDruhy1;
published
   property DruhyList: string read GetDruhyList write SetDruhyList;
   property Druhy1List: string read GetDruhy1List write SetDruhy1List;
   property Distance: double read FDistance write SetDistance;
   property CentralLineGroup: integer read FCentralLineGroup write SetCentralLineGroup;
   property CentralLineColor: integer read FCentralLineColor write SetCentralLineColor;
   property CentralLineSymbol: integer read FCentralLineSymbol write SetCentralLineSymbol;
   property SatelitLineGroup: integer read FSatelitLineGroup write SetSatelitLineGroup;
   property SatelitLineColor: integer read FSatelitLineColor write SetSatelitLineColor;
   property SatelitLineSymbol: integer read FSatelitLineSymbol write SetSatelitLineSymbol;
   property AtrPuv: integer read FAtrPuv write SetAtrPuv;
   property ArcStep: double read FArcStep write SetArcStep;
   property UseArcStep: boolean read FUseArcStep write SetUseArcStep;
end;

implementation
uses
   Forms,
   SysUtils,
   e_Class,
   e_TopolDruhy,
   Worlds;

{ TMultiParalelLineDialog }

procedure TMultiParalelLineDialog.CreateForm;
begin
   aForm := TMultiParalelLineDialog_Form.Create(Application);
   aForm.Tool := Self;
   aForm.IsPossibleToClose := false;
end;

procedure TMultiParalelLineDialog.DoIt;
var
   D: TTopolDruhy;
   TD,TD1: TTopolDruh;
   i: integer;
begin
   Output := Input;
   D := TTopolDruhy(World.ObjectModel.FindElementClass(TTopolDruhy));
   if D <> nil then
   begin
      D.FindLDruhKod := CentralLineGroup;
      TD := D.ActualDruh;
      if TD <> nil then
      begin
         i := MPLForm.ComboDruh.Items.IndexOfObject(TD);
         if i < 0 then
         begin
            D.FindLDruhKod := SatelitLineGroup;
            TD1 := D.ActualDruh;
            i := MPLForm.ComboDruh.Items.AddObject(TD.Name+'['+IntToStr(TD.KOD)+']',TD);
            MPLForm.ComboDruh1.Items.AddObject(TD1.Name+'['+IntToStr(TD1.KOD)+']',TD1);
         end;
         MPLForm.ComboDruh.ItemIndex := i;
      end;
   end;
   MPLForm.ed_AtrPuv.Value := AtrPuv;
   MPLForm.ed_Distance.Text := FloatToStr(Distance);
   MPLForm.ed_ArcStep.Text := FloatToStr(ArcStep);
   MPLForm.cb_ArcStep.Checked := UseArcStep;
   if MPLForm.ComboDruh.ItemIndex < 0 then
   begin
      MPLForm.ComboDruh.ItemIndex := 0;
   end;
   MPLForm.ComboDruhChange(nil);
   MPLForm.b_Continue.Visible := true;
   inherited;
   if result in [1,3,4,5,6] then
   begin
      if D <> nil then
      begin
         TD := TTopolDruh(MPLForm.ComboDruh.Items.Objects[MPLForm.ComboDruh.ItemIndex]);
         TD1 := TTopolDruh(MPLForm.ComboDruh1.Items.Objects[MPLForm.ComboDruh1.ItemIndex]);
         CentralLineGroup  := TD.KOD;
         CentralLineColor  := TD.Barva;
         CentralLineSymbol := TD.Znacka;
         SatelitLineGroup  := TD1.KOD;
         SatelitLineColor  := TD1.Barva;
         SatelitLineSymbol := TD1.Znacka;
         AtrPuv := MPLForm.ed_AtrPuv.Value;
         SetPropertyStringByName('Distance',MPLForm.ed_Distance.Text);
         SetPropertyStringByName('ArcStep',MPLForm.ed_ArcStep.Text);
         UseArcStep := MPLForm.cb_ArcStep.Checked;
      end;
   end;
end;

function TMultiParalelLineDialog.GetDruhy: TIntegerListElement;
begin
   if FDruhy = nil then
   begin
      FDruhy := TIntegerListElement.Create;
      AddElement(FDruhy);
   end;
   result := FDruhy;
end;

function TMultiParalelLineDialog.GetDruhy1: TIntegerListElement;
begin
   if FDruhy1 = nil then
   begin
      FDruhy1 := TIntegerListElement.Create;
      AddElement(FDruhy1);
   end;
   result := FDruhy1;
end;

function TMultiParalelLineDialog.GetDruhy1List: string;
var
   i: integer;
   S: string;
begin
   result := '';
   for i := 0 to Druhy1.LastIndex do
   begin
      S := IntToStr(Druhy1.Integers[i]);
      if i = 0 then
         result := S
      else
         result := result + ',' + S;
   end;
end;

function TMultiParalelLineDialog.GetDruhyList: string;
var
   i: integer;
   S: string;
begin
   result := '';
   for i := 0 to Druhy.LastIndex do
   begin
      S := IntToStr(Druhy.Integers[i]);
      if i = 0 then
         result := S
      else
         result := result + ',' + S;
   end;
end;

function TMultiParalelLineDialog.GetMPLForm: TMultiParalelLineDialog_Form;
begin
   result := TMultiParalelLineDialog_Form(aForm);
end;

procedure TMultiParalelLineDialog.SetAtrPuv(const Value: integer);
begin
  FAtrPuv := Value;
end;

procedure TMultiParalelLineDialog.SetCentralLineColor(
  const Value: integer);
begin
  FCentralLineColor := Value;
end;

procedure TMultiParalelLineDialog.SetCentralLineGroup(
  const Value: integer);
begin
  FCentralLineGroup := Value;
end;

procedure TMultiParalelLineDialog.SetCentralLineSymbol(
  const Value: integer);
begin
  FCentralLineSymbol := Value;
end;

procedure TMultiParalelLineDialog.SetDistance(const Value: double);
begin
  FDistance := Value;
end;

procedure TMultiParalelLineDialog.SetDruhy1List(const Value: string);
var
   D: TTopolDruhy;
   TD: TTopolDruh;
   i: integer;
begin
   Druhy1.ValuesString := Value;
   D := TTopolDruhy(World.ObjectModel.FindElementClass(TTopolDruhy));
   if D <> nil then
   begin
      MPLForm.ComboDruh1.Items.Clear;
      for i := 0 to Druhy1.LastIndex do
      begin
         D.FindLDruhKod := Druhy1.Integers[i];
         TD := D.ActualDruh;
         MPLForm.ComboDruh1.Items.AddObject(TD.Name+'['+IntToStr(TD.KOD)+']',TD);
      end;
   end;
end;

procedure TMultiParalelLineDialog.SetDruhyList(const Value: string);
var
   D: TTopolDruhy;
   TD: TTopolDruh;
   i: integer;
begin
   Druhy.ValuesString := Value;
   D := TTopolDruhy(World.ObjectModel.FindElementClass(TTopolDruhy));
   if D <> nil then
   begin
      MPLForm.ComboDruh.Items.Clear;
      for i := 0 to Druhy.LastIndex do
      begin
         D.FindLDruhKod := Druhy.Integers[i];
         TD := D.ActualDruh;
         MPLForm.ComboDruh.Items.AddObject(TD.Name+'['+IntToStr(TD.KOD)+']',TD);
      end;
   end;
end;

procedure TMultiParalelLineDialog.SetArcStep(
  const Value: double);
begin
   FArcStep := Value;
end;

procedure TMultiParalelLineDialog.SetSatelitLineColor(
  const Value: integer);
begin
  FSatelitLineColor := Value;
end;

procedure TMultiParalelLineDialog.SetSatelitLineGroup(
  const Value: integer);
begin
  FSatelitLineGroup := Value;
end;

procedure TMultiParalelLineDialog.SetSatelitLineSymbol(
  const Value: integer);
begin
  FSatelitLineSymbol := Value;
end;

procedure TMultiParalelLineDialog.SetUseArcStep(const Value: boolean);
begin
  FUseArcStep := Value;
end;

INITIALIZATION
   RegisterOrbisClass(TMultiParalelLineDialog);
END.
