unit t_MultiParalelLineEditor;

interface
uses
   Elements,
   E_QTree,
   g_Elems,
   m_Intera,
   t_GraphicEditors;

type

TMultiParalelLineEditor = class(TPolyLineEditorNoFrame)
private
    FDistance: double;
    FLevel1Name: string;
    FLevel1: TEntity;
    FLevelQT1: TRectEntityQTree;
    FCentralLineSymbol: integer;
    FSatelitLineGroup: integer;
    FCentralLineGroup: integer;
    FCentralLineColor: integer;
    FSatelitLineColor: integer;
    FSatelitLineSymbol: integer;
    FRSide: boolean;
    FAtrPuv: integer;
    FSimpleParalel: boolean;
    FUseArcStep: boolean;
    FArcStep: double;
    procedure SetDistance(const Value: double);
    procedure SetLevel1(const Value: TEntity);
    procedure SetLevel1Name(const Value: string);
    procedure SetLevelQT1(const Value: TRectEntityQTree);
    procedure SetCentralLineColor(const Value: integer);
    procedure SetCentralLineGroup(const Value: integer);
    procedure SetCentralLineSymbol(const Value: integer);
    procedure SetSatelitLineColor(const Value: integer);
    procedure SetSatelitLineGroup(const Value: integer);
    procedure SetSatelitLineSymbol(const Value: integer);
    procedure SetRSide(const Value: boolean);
    procedure SetAtrPuv(const Value: integer);
    procedure SetSimpleParalel(const Value: boolean);
    procedure SetArcStep(const Value: double);
    procedure SetUseArcStep(const Value: boolean);
public
   procedure XORDraw; override;
   procedure   On_tm_MouseMove(var aMessage: TOrbisMouseEvent; var Handled: boolean); override;
   procedure   On_tm_MouseDown(var aMessage: TOrbisMouseEvent; var Handled: boolean); override;
   procedure   On_tm_MouseUp(var aMessage: TOrbisMouseEvent; var Handled: boolean); override;
   procedure On_Activate; override;
   procedure RegisterCursors; override;
   procedure StoreObject; override;
   procedure ExecuteDialog; override;
   procedure ClearTopology(aLevel: TEntity; Rel,R:TRectEntity); override;
   procedure PrepareQTree; override;
   property LevelQT1: TRectEntityQTree read FLevelQT1 write SetLevelQT1;
   property Level1: TEntity read FLevel1 write SetLevel1;
   property RSide: boolean read FRSide write SetRSide;
published
   property Level1Name: string read FLevel1Name write SetLevel1Name;
   property Distance: double read FDistance write SetDistance;
   property CentralLineGroup: integer read FCentralLineGroup write SetCentralLineGroup;
   property CentralLineColor: integer read FCentralLineColor write SetCentralLineColor;
   property CentralLineSymbol: integer read FCentralLineSymbol write SetCentralLineSymbol;
   property SatelitLineGroup: integer read FSatelitLineGroup write SetSatelitLineGroup;
   property SatelitLineColor: integer read FSatelitLineColor write SetSatelitLineColor;
   property SatelitLineSymbol: integer read FSatelitLineSymbol write SetSatelitLineSymbol;
   property AtrPuv: integer read FAtrPuv write SetAtrPuv;
   property SimpleParalel: boolean read FSimpleParalel write SetSimpleParalel;
   property UseArcStep: boolean read FUseArcStep write SetUseArcStep;
   property ArcStep: double read FArcStep write SetArcStep;
end;

implementation
uses
   Controls,
   MathLib,
   Forms,
   Graphics,
   SysUtils,
   e_Class,
   e_Geometry,
   e_GeometryElement,
   m_Orbis,
   t_AreMak,
   t_Paral,
   t_QTreeTools,
   e_TopolElements;




{ TMultiParalelLineEditor }

procedure TMultiParalelLineEditor.ClearTopology(aLevel: TEntity; Rel,R: TRectEntity);
var
   W: TQTreeWalker;
   CL: TClearLines;
   El: TElement;
   i: integer;
   G: TGeometryElement;
   QT: TRectEntityQTree;
   En: TEntity;
   RE: TRectEntity;
begin
   if R.Name = '1' then QT := LevelQT1
                   else QT := LevelQT;
   if QT = nil then exit;
   W := TQTreeWalker.Create;
   El := W.GetFirstElementForRectangle(QT,R.Minx,R.MinY,R.MaxX,R.MaxY,1);
   while El <> nil do
   begin
      if not El.IsMyOwner(R) then
         R.AddAndUpdateRect(El);
      El := W.GetNextElement;
   end;
   // Uprava kvuli dlouhym liniim pres pul sveta
   {
   if R.Count > 100 then
   begin
      En := TQuickEntity.Create;
      for i := 0 to R.LastIndex do
      begin
         El := R.Elements[i];
         if not (El is TRectEntity) then continue;
         RE := TRectEntity(El);
         if Rel.IsOverRegionEx(RE.MinX-1,RE.MinY-1,RE.MaxX+1,RE.MaxY+1) then
            En.AddElement(RE);
      end;
      R.FreeAll;
      R.AddAllFrom := En;
      R.ResetAndUpdate;
      En.Clear;
      En.Free;
   end;
   }
   // konec upravy
   //RE.Name := 'Test';
   //AddElement(RE);
   CL := TClearLines.Create;
   CL.Input := R;
   CL.DoIt;
   for i := R.LastIndex downto 0 do
   begin
      El := R.Elements[i];
      if El is TGeometryElement then
      begin
         G := TGeometryElement(El);
         if G.NoGeometry then
            G.Free;
      end;
   end;
   CL.Free;
   W.Free;
end;

procedure TMultiParalelLineEditor.ExecuteDialog;
begin
   if Dialog <> nil then
   begin
      Dialog.Input := PolyElement;
      Dialog.Caption := 'Para';
      Dialog.Left := Mouse.CursorPos.x;
      Dialog.Top := Mouse.CursorPos.y;
      if (Dialog.Left + Dialog.Width) > Screen.DesktopWidth then
         Dialog.Left := Screen.DesktopWidth - Dialog.Width;
      if (Dialog.Top + Dialog.Height) > Screen.Height then
         Dialog.Top := Screen.DesktopHeight - Dialog.Height;
      Dialog.SetPropertyLongIntByName('SnapTool',LongInt(SnapTool));
      if NewObject then
      begin
         Dialog.SetPropertyBooleanByName('FreeToDelete',true);
         Dialog.SetPropertyBooleanByName('FreeToChangeGeometry',true);
      end
      else
      begin
         Dialog.SetPropertyBooleanByName('FreeToDelete',false);
         Dialog.SetPropertyBooleanByName('FreeToChangeGeometry',false);
      end;
      Dialog.SetPropertyLongIntByName('CentralLineGroup',CentralLineGroup);
      Dialog.SetPropertyLongIntByName('CentralLineColor',CentralLineColor);
      Dialog.SetPropertyLongIntByName('CentralLineSymbol',CentralLineSymbol);
      Dialog.SetPropertyLongIntByName('SatelitLineGroup',SatelitLineGroup);
      Dialog.SetPropertyLongIntByName('SatelitLineColor',SatelitLineColor);
      Dialog.SetPropertyLongIntByName('SatelitLineSymbol',SatelitLineSymbol);
      Dialog.SetPropertyStringByName('Distance',FloatToStr(Distance));
      Dialog.SetPropertyBooleanByName('UseArcStep',UseArcStep);
      if ArcStep <= 0 then
      begin
         ArcStep := 5;
         UseArcStep := false;
      end;
      if ArcStep > 99 then
         ArcStep := 99;
      Dialog.SetPropertyStringByName('ArcStep',FloatToStr(ArcStep));
      Dialog.SetPropertyLongIntByName('AtrPuv',AtrPuv);
      Dialog.DoIt;
      if Dialog.Result in [1,3,4,5,6] then
      begin
         CentralLineGroup := Dialog.GetPropertyLongIntByName('CentralLineGroup');
         CentralLineColor := Dialog.GetPropertyLongIntByName('CentralLineColor');
         CentralLineSymbol := Dialog.GetPropertyLongIntByName('CentralLineSymbol');
         SatelitLineGroup := Dialog.GetPropertyLongIntByName('SatelitLineGroup');
         SatelitLineColor := Dialog.GetPropertyLongIntByName('SatelitLineColor');
         SatelitLineSymbol := Dialog.GetPropertyLongIntByName('SatelitLineSymbol');
         XORDraw;
         Distance := StrToFloat(Dialog.GetPropertyStringByName('Distance'));
         ArcStep  := StrToFloat(Dialog.GetPropertyStringByName('ArcStep'));
         UseArcStep := Dialog.GetPropertyBooleanByName('UseArcStep');
         if ArcStep <= 0 then
         begin
            ArcStep := 5;
            UseArcStep := false;
         end;
         if ArcStep > 99 then
            ArcStep := 99;
         XORDraw;
         AtrPuv := Dialog.GetPropertyLongIntByName('AtrPuv');
      end;

      Case Dialog.Result of
         1: StoreObject;
         3: DeleteObject;
         4: DeletePoint;
         5: CloseObject;
         6: ;
         {
         7: begin
               PolyElement := TPolyElement(CreateObjectCopy);
               if GElement <> nil then
                  GElement.Geometry := PolyElement;
               TestForDelete;
               TestForChangeGeometry;
            end;
          }
         else
            if not NewObject then
            begin
               XORDraw;
               PolyElement := nil;
               GElement := nil;
               PolyElementBackUp := nil;
               Modified := false;
               On_Activate;
               Application.ProcessMessages;
               Application.ProcessMessages;
            end
            else
               ConfirmAndStore;
      end;
   end;
end;

procedure TMultiParalelLineEditor.On_Activate;
begin
  inherited;

end;

procedure TMultiParalelLineEditor.On_tm_MouseDown(var aMessage: TOrbisMouseEvent; var Handled: boolean);
begin
   if not(EditorStatus = gesSetup) then
      inherited
   else
   if (aMessage.Button = mbRight) then
      ExecuteDialog;
end;

procedure TMultiParalelLineEditor.On_tm_MouseMove(var aMessage: TOrbisMouseEvent; var Handled: boolean);
var
   i: integer;
   d: double;
   PataUvnitr: boolean;
   PataX, PataY, Vzdalenost,v: double;
   A,B,C: TDoubleCoord;
   L: TPolyElement;
   aRSide: boolean;
begin
   if not(EditorStatus = gesSetup) then
     inherited
   else
   begin
      if PolyElement = nil then exit;
      L := PolyElement;
      ActX:= Device.Projection.GetModelX(aMessage.X);
      ActY:= Device.Projection.GetModelY(aMessage.Y);
      v := 1E30;
      aRSide := RSide;
      for i := 0 to L.LastIndex - 1 do
      begin
         A.X := L.Xi[i];
         A.Y := L.Yi[i];
         B.X := L.Xi[i+1];
         B.Y := L.Yi[i+1];
         C.X := ActX;
         C.Y := ActY;
         DejVztahyBod_Usecka( ActX, ActY, A.X, A.Y, B.X, B.Y,
                              PataUvnitr, PataX, PataY, Vzdalenost);

         if PataUvnitr and (Vzdalenost < v) then
         begin
            v := Vzdalenost;
            aRSide := BodVpravoOdOrientPrimky(A,B,C);
         end;
      end;
      if aRSide <> RSide then
      begin
         XORDraw;
         RSide := aRSide;
         XORDraw;
      end;
   end;
end;

procedure TMultiParalelLineEditor.On_tm_MouseUp(var aMessage: TOrbisMouseEvent; var Handled: boolean);
begin
   if ZeroActivity then exit;
   case EditorStatus of
      gesSelect :
      begin
         if QTFinder.Output = nil then exit;
         HighActivity := True;
         Input := QTFinder.Output;
         //RemoveElementOwners(Input);
         if Input is TGeometryElement then
            GElement := TGeometryElement(Input)
         else
            PolyElement := TPolyElement(Input);
         PolyElementBackUp := TPolyElement(CreateObjectCopy);
         //inherited On_Activate;
         StartEditing;
         Application.ProcessMessages;
         EditorStatus := gesSetup;
         XORDraw;
         //Modified := true;
         {
         TestForDelete;
         TestForChangeGeometry;
         if not FreeToDelete or not FreeToChangeGeometry then
            aMessage.Button := mbRight;
         if aMessage.Button = mbRight then
            On_tm_MouseDown(aMessage,Handled);
         }
      end;
      gesSetup:
      begin
         if not (aMessage.Button = mbRight) then
            StoreObject;
      end;
      //else
        //inherited;
   end;
end;

procedure TMultiParalelLineEditor.PrepareQTree;
var
   TQ: TAddEntityToQTree;
begin
   if Level = nil then exit;
   StartProgress('Pprava vrstev...',0,0);
   LevelQT :=TRectEntityQTree.Create;
   LevelQT.Name := 'LEVEL QT';
   TQ := TAddEntityToQTree.Create;
   TQ.Input := Level;
   TQ.Output := LevelQT;
   TQ.DoIt;
   if SnapTool <> nil then
      SnapTool.FrameQT := LevelQT;
   TQ.Free;

   if Level1 = nil then exit;
   LevelQT1 :=TRectEntityQTree.Create;
   LevelQT1.Name := 'LEVEL1 QT';
   TQ := TAddEntityToQTree.Create;
   TQ.Input := Level1;
   TQ.Output := LevelQT1;
   TQ.DoIt;
   TQ.Free;
   EndProgress;
end;

procedure TMultiParalelLineEditor.RegisterCursors;
begin
  inherited;

end;

procedure TMultiParalelLineEditor.SetArcStep(const Value: double);
begin
  FArcStep := Value;
end;

procedure TMultiParalelLineEditor.SetAtrPuv(const Value: integer);
begin
  FAtrPuv := Value;
end;

procedure TMultiParalelLineEditor.SetCentralLineColor(const Value: integer);
begin
  FCentralLineColor := Value;
end;

procedure TMultiParalelLineEditor.SetCentralLineGroup(const Value: integer);
begin
  FCentralLineGroup := Value;
end;

procedure TMultiParalelLineEditor.SetCentralLineSymbol(const Value: integer);
begin
  FCentralLineSymbol := Value;
end;

procedure TMultiParalelLineEditor.SetDistance(const Value: double);
begin
  FDistance := Value;
end;

procedure TMultiParalelLineEditor.SetLevel1(const Value: TEntity);
begin
  FLevel1 := Value;
end;

procedure TMultiParalelLineEditor.SetLevel1Name(const Value: string);
begin
   FLevel1Name := Value;
   Level1 := FindLevel(Value);
end;

procedure TMultiParalelLineEditor.SetLevelQT1(const Value: TRectEntityQTree);
begin
   ExchangeFieldElement(TElement(FLevelQT1), Value);
end;

procedure TMultiParalelLineEditor.SetRSide(const Value: boolean);
begin
  FRSide := Value;
end;

procedure TMultiParalelLineEditor.SetSatelitLineColor(const Value: integer);
begin
  FSatelitLineColor := Value;
end;

procedure TMultiParalelLineEditor.SetSatelitLineGroup(const Value: integer);
begin
  FSatelitLineGroup := Value;
end;

procedure TMultiParalelLineEditor.SetSatelitLineSymbol(const Value: integer);
begin
  FSatelitLineSymbol := Value;
end;

procedure TMultiParalelLineEditor.SetSimpleParalel(const Value: boolean);
begin
  FSimpleParalel := Value;
end;

procedure TMultiParalelLineEditor.SetUseArcStep(const Value: boolean);
begin
  FUseArcStep := Value;
end;

procedure TMultiParalelLineEditor.StoreObject;
var
   R: TRectEntity;
   GC: TGeometryElementCollection;
   TL,TL1,TL2: TTopolLineElement;
   L,L1,L2: TPolyLineElement;
   Par: TParalel;
   aLevel: TEntity;
begin
   if (PolyElement <> nil) and (PolyElement.Count > 1) then
   begin
      //if not (input is TRectEntity) then
      //   Exit;
      if NewObject then
      begin
         PolyElement.RecalculateRect;
         PolyElement.CalculateLabelProps;
         TL := TTopolLineElement.Create;
         TL.Druh := CentralLineGroup;
         TL.Barva := CentralLineColor;
         TL.Znacka := CentralLineSymbol;
         TL.Presnost := AtrPuv;
         TL.Geometry := PolyElement;

         Par := TParalel.Create;
         Par.Input := PolyElement;
         Par.Rozestup := Distance;
         Par.Vpravo := true;
         if UseArcStep then
            Par.ProcRozs := ArcStep
         else
            Par.ProcRozs := 0;
         Par.DoIt;

         TL1 := TTopolLineElement.Create;
         TL1.Druh := SatelitLineGroup;
         TL1.Barva := SatelitLineColor;
         TL1.Znacka := SatelitLineSymbol;
         TL1.Presnost := AtrPuv;
         L1 := TPolyLineElement.Create;
         R := TRectEntity(Par.Output);
         L1.AddAllFrom := R;
         L1.RecalculateRect;
         L1.CalculateLabelProps;
         TL1.Geometry := L1;
         R.Clear;
         R.Free;
         Par.Free;

         Par := TParalel.Create;
         Par.Input := PolyElement;
         Par.Rozestup := Distance;
         Par.Vpravo := false;
         if UseArcStep then
            Par.ProcRozs := ArcStep
         else
            Par.ProcRozs := 0;
         Par.DoIt;

         TL2 := TTopolLineElement.Create;
         TL2.Druh := SatelitLineGroup;
         TL2.Barva := SatelitLineColor;
         TL2.Znacka := SatelitLineSymbol;
         TL2.Presnost := AtrPuv;
         L2 := TPolyLineElement.Create;
         R := TRectEntity(Par.Output);
         L2.AddAllFrom := R;
         L2.RecalculateRect;
         L2.CalculateLabelProps;
         TL2.Geometry := L2;
         R.Clear;
         R.Free;
         Par.Free;

         R := TL;
         //if (PolyElement <> nil) and (PolyElement.Count > 1) then
         begin
            if Level <> nil then
            begin
               if Level is TRectEntity then
                  TRectEntity(Level).AddAndUpdateRect(TL)
               else
                  Level.AddElement(TL);
            end;
            if LevelQT <> nil then
               LevelQT.NextRectEntity := TL;
            if Modified then PostMsg(tm_ModelChanged,nil,nil,false);
         end;
         begin
            if Level1 <> nil then
            begin
               if Level1 is TRectEntity then
               begin
                  TRectEntity(Level1).AddAndUpdateRect(TL1);
                  TRectEntity(Level1).AddAndUpdateRect(TL2);
               end
               else
               begin
                  Level.AddElement(TL1);
                  Level.AddElement(TL2);
               end;
            end;
            if LevelQT1 <> nil then
            begin
               LevelQT1.NextRectEntity := TL1;
               LevelQT1.NextRectEntity := TL2;
            end;
            if Modified then PostMsg(tm_ModelChanged,nil,nil,false);
         end;
         //TRectEntity(Input).AddAndUpdateRect(PolyElement);
         //PolyElement:= nil;
         XORDraw;
         //Application.ProcessMessages;
         if Modified then
         begin
            GC := TGeometryElementCollection.Create;
            GC.UpdateRect(TL1);
            GC.UpdateRect(TL2);
            GC.Name := '1';
            PolyElement := nil;
            GElement := nil;
            if Topology then
               ClearTopology(aLevel,TL1,GC);
            PostMsgToOwners(tm_ReviewByElement, GC, True);
            //PostMsgToOwners(tm_Review, R, false);
         end
         else
            PostMsgToOwners(tm_Review, R, false);
      //if HighActivity then HighActivity := false;
      end
      else
      begin
         Par := TParalel.Create;
         Par.Input := PolyElement;
         Par.Rozestup := Distance;
         Par.Vpravo := RSide;
         if UseArcStep then
            Par.ProcRozs := ArcStep
         else
            Par.ProcRozs := 0;
         Par.DoIt;

         TL1 := TTopolLineElement.Create;
         TL1.Druh := CentralLineGroup;
         TL1.Barva := CentralLineColor;
         TL1.Znacka := CentralLineSymbol;
         TL1.Presnost := AtrPuv;
         L1 := TPolyLineElement.Create;
         R := TRectEntity(Par.Output);
         L1.AddAllFrom := R;
         L1.RecalculateRect;
         L1.CalculateLabelProps;
         TL1.Geometry := L1;
         R.Clear;
         R.Free;
         Par.Free;

         TL2 := nil;
         if not SimpleParalel then
         begin
            Par := TParalel.Create;
            Par.Input := PolyElement;
            Par.Rozestup := Distance*2;
            Par.Vpravo := RSide;
            if UseArcStep then
               Par.ProcRozs := ArcStep
            else
               Par.ProcRozs := 0;
            Par.DoIt;

            TL2 := TTopolLineElement.Create;
            TL2.Druh := SatelitLineGroup;
            TL2.Barva := SatelitLineColor;
            TL2.Znacka := SatelitLineSymbol;
            TL2.Presnost := AtrPuv;
            L2 := TPolyLineElement.Create;
            R := TRectEntity(Par.Output);
            L2.AddAllFrom := R;
            L2.RecalculateRect;
            L2.CalculateLabelProps;
            TL2.Geometry := L2;
            R.Clear;
            R.Free;
            Par.Free;
         end;
         R := TL1;
         Modified := true;
         if (TL2 <> nil) and (Level <> nil) then
         begin
            if Level is TRectEntity then
               TRectEntity(Level).AddAndUpdateRect(TL2)
            else
               Level.AddElement(TL2);
         end;
         if LevelQT <> nil then
            LevelQT.NextRectEntity := TL2;
         if Modified then PostMsg(tm_ModelChanged,nil,nil,false);
         if Level1 <> nil then
         begin
            if Level1 is TRectEntity then
            begin
               TRectEntity(Level1).AddAndUpdateRect(TL1);
            end
            else
            begin
               Level.AddElement(TL1);
            end;
         end;
         if LevelQT1 <> nil then
         begin
            LevelQT1.NextRectEntity := TL1;
         end;
         if Modified then PostMsg(tm_ModelChanged,nil,nil,false);
         //TRectEntity(Input).AddAndUpdateRect(PolyElement);
         //PolyElement:= nil;
         XORDraw;
         //Application.ProcessMessages;
         if Modified then
         begin
            GC := TGeometryElementCollection.Create;
            GC.UpdateRect(PolyElement);
            if TL2 <> nil then
               GC.UpdateRect(TL2)
            else
               GC.UpdateRect(TL1);
            GC.Name := '2';
            PolyElement := nil;
            GElement := nil;
            if Topology then
               ClearTopology(aLevel,TL1,GC);
            PostMsgToOwners(tm_ReviewByElement, GC, True);
            //PostMsgToOwners(tm_Review, R, false);
         end
         else
            PostMsgToOwners(tm_Review, R, false);
      end;
   end;
   PolyElement := nil;
   GElement := nil;
   PolyElementBackUp := nil;
   Modified := false;
   On_Activate;
   Application.ProcessMessages;
   Application.ProcessMessages;
   //CreateNewObject;
end;

procedure TMultiParalelLineEditor.XORDraw;
var
  aR: double;
  I: integer;
  K: integer;
  //TEST
  Par: TParalel;
  RE: TRectEntity;
  //END TEST
begin
   if PolyElement = nil then
      Exit;
   inherited;

  if PolyElement.CoordsCount > 1 then
  begin
     if NewObject then
     begin
        Par := TParalel.Create;
        Par.Input := PolyElement;
        Par.Rozestup := Distance;
        Par.Vpravo := false;
         if UseArcStep then
            Par.ProcRozs := ArcStep
         else
            Par.ProcRozs := 0;
        Par.DoIt;
        RE := TRectEntity(Par.Output);
        Device.MoveTo(TCoordinate(RE.Elements[0]).X,TCoordinate(RE.Elements[0]).Y);
        for i := 1 to RE.LastIndex do
            Device.LineTo(TCoordinate(RE.Elements[i]).X,TCoordinate(RE.Elements[i]).Y);
        RE.Free;
        Par.Free;

        Par := TParalel.Create;
        Par.Input := PolyElement;
        Par.Rozestup := Distance;
        Par.Vpravo := true;
         if UseArcStep then
            Par.ProcRozs := ArcStep
         else
            Par.ProcRozs := 0;
        Par.DoIt;
        RE := TRectEntity(Par.Output);
        Device.MoveTo(TCoordinate(RE.Elements[0]).X,TCoordinate(RE.Elements[0]).Y);
        for i := 1 to RE.LastIndex do
            Device.LineTo(TCoordinate(RE.Elements[i]).X,TCoordinate(RE.Elements[i]).Y);
        RE.Free;
        Par.Free;
        //END TEST
     end
     else
     begin
        Par := TParalel.Create;
        Par.Input := PolyElement;
        Par.Rozestup := Distance;
        Par.Vpravo := RSide;
         if UseArcStep then
            Par.ProcRozs := ArcStep
         else
            Par.ProcRozs := 0;
        Par.DoIt;
        RE := TRectEntity(Par.Output);
        Device.MoveTo(TCoordinate(RE.Elements[0]).X,TCoordinate(RE.Elements[0]).Y);
        for i := 1 to RE.LastIndex do
            Device.LineTo(TCoordinate(RE.Elements[i]).X,TCoordinate(RE.Elements[i]).Y);
        RE.Free;
        Par.Free;
        if not SimpleParalel then
        begin
           Par := TParalel.Create;
           Par.Input := PolyElement;
           Par.Rozestup := Distance*2;
           Par.Vpravo := RSide;
           if UseArcStep then
              Par.ProcRozs := ArcStep
           else
              Par.ProcRozs := 0;
           Par.DoIt;
           RE := TRectEntity(Par.Output);
           Device.MoveTo(TCoordinate(RE.Elements[0]).X,TCoordinate(RE.Elements[0]).Y);
           for i := 1 to RE.LastIndex do
               Device.LineTo(TCoordinate(RE.Elements[i]).X,TCoordinate(RE.Elements[i]).Y);
           RE.Free;
           Par.Free;
        end;
        //END TEST
     end;
     PostMsgToOwners(tm_Repaint,nil,DontKill);
  end;
end;

INITIALIZATION
   RegisterOrbisClass(TMultiParalelLineEditor);
END.
