unit t_ParcelModelSelector;

interface
uses
   e_KatastrModel,
   t_FrameSelect;

type

TParcelModelSelector = class(TFrameSelector)
  private
    FParcelModel: Tgis_ParcelModel;
    FSelectByParcely: boolean;
    FActualParcela: Tgis_Parcela;
    FActualParCast: Tgis_ParcelaCast;
    FActualKatastr: Tgis_Katastr;
    procedure SetParcelModel(const Value: Tgis_ParcelModel);
    procedure SetActualSkupinaParcel(const Value: Tgis_SkupinaParcel);
    function GetActualSkupinaParcel: Tgis_SkupinaParcel;
    procedure SetSelectByParcely(const Value: boolean);
    procedure SetActualParcela(const Value: Tgis_Parcela);
    procedure SetActualParCast(const Value: Tgis_ParcelaCast);
    procedure SetActualKatastr(const Value: Tgis_Katastr);
public
   procedure DoIt; override;
   procedure FindInput; override;
   function  TestForXY(aSKP: Tgis_SkupinaParcel): boolean;
published
   property ParcelModel: Tgis_ParcelModel read FParcelModel write SetParcelModel;
   property SelectByParcely: boolean read FSelectByParcely write SetSelectByParcely;
   property ActualKatastr: Tgis_Katastr read FActualKatastr write SetActualKatastr;
   property ActualSkupinaParcel: Tgis_SkupinaParcel read GetActualSkupinaParcel write SetActualSkupinaParcel stored false;
   property ActualParcela: Tgis_Parcela read FActualParcela write SetActualParcela stored false;
   property ActualParCast: Tgis_ParcelaCast read FActualParCast write SetActualParCast stored false;
end;

implementation
uses
   Elements,
   e_Class,
   Worlds;

{ TParcelModelSelector }

procedure TParcelModelSelector.DoIt;
var
   i,j,jj: integer;
   KAT: Tgis_Katastr;
   SKP: Tgis_SkupinaParcel;
   PAR: Tgis_Parcela;
   PARC: Tgis_ParcelaCast;
begin
   if ParcelModel = nil then exit;
   ActualParcela := nil;
   ActualParCast := nil;
   if Busy then exit;
   Busy := true;
   if ActualSkupinaParcel <> nil then
   begin
      if TestForXY(ActualSkupinaParcel) then
      begin
         Busy := false;
         exit; // je to ona
      end;
   end;
   ActualKatastr := nil;
   ActualSkupinaParcel := nil;
   for i := 0 to ParcelModel.LastIndex do
   begin
      KAT := Tgis_Katastr(ParcelModel.Elements[i]);
      if not KAT.PointIsInside(X,Y) then continue;
      if not KAT.NoGeometry then
      begin
         if KAT.IsHitByXY(X,Y,0) then
         begin
            ActualKatastr := KAT;
         end;
      end;
      if KAT.NoSkupinaParcel then
      begin
         if KAT.NoParcela then
            continue;
         for j := 0 to KAT.Parcely.LastIndex do
         begin
            PAR := Tgis_Parcela(KAT.Parcely.Elements[j]);
            for jj := 0 to PAR.LastIndex do
            begin
               PARC := Tgis_ParcelaCast(PAR.Elements[jj]);
               if PARC.IsHitByXY(X,Y,0) then
               begin
                  ActualKatastr := KAT;
                  ActualSkupinaParcel := nil;
                  ActualParCast := PARC;
                  Busy := false;
                  exit;
               end;
            end;
         end;
         Continue;
      end;
      for j := 0 to KAT.SkupinyParcel.LastIndex do
      begin
         SKP := Tgis_SkupinaParcel(KAT.SkupinyParcel.Elements[j]);
         if TestForXY(SKP) then
         begin
            ActualKatastr := KAT;
            ActualSkupinaParcel := SKP;
            Busy := false;
            exit;
         end;
      end;
   end;
   Busy := false;
end;

procedure TParcelModelSelector.FindInput;
var
   E: TEntity;
begin
   ParcelModel := nil;
   E := TEntity(World.ObjectModel.FindElementName('Data'));
   if E = nil then exit;
   if not (TElement(E) is TEntity) then exit;
   ParcelModel := Tgis_ParcelModel(E.FindElementClass(Tgis_ParcelModel));
end;

function TParcelModelSelector.GetActualSkupinaParcel: Tgis_SkupinaParcel;
begin
   result := Tgis_SkupinaParcel(Output);
end;

procedure TParcelModelSelector.SetActualKatastr(const Value: Tgis_Katastr);
begin
  FActualKatastr := Value;
end;

procedure TParcelModelSelector.SetActualParCast(const Value: Tgis_ParcelaCast);
begin
  FActualParCast := Value;
end;

procedure TParcelModelSelector.SetActualParcela(const Value: Tgis_Parcela);
begin
  FActualParcela := Value;
end;

procedure TParcelModelSelector.SetActualSkupinaParcel(const Value: Tgis_SkupinaParcel);
begin
    Output := Value;
end;

procedure TParcelModelSelector.SetParcelModel(const Value: Tgis_ParcelModel);
begin
   FParcelModel := Value;
end;

procedure TParcelModelSelector.SetSelectByParcely(const Value: boolean);
begin
  FSelectByParcely := Value;
end;

function TParcelModelSelector.TestForXY(aSKP: Tgis_SkupinaParcel): boolean;
var
   P: Tgis_Parcela;
   PC: Tgis_ParcelaCast;
   i,j: integer;
   e: TElement;
begin
   result := false;
   if not SelectByParcely then
      if not aSKP.PointIsInside(X,Y) then exit;
   if SelectByparcely or aSKP.NoGeometry then
   begin
      if not aSKP.NoParcela then
      if aSKP.Parcely.PointIsInside(X,Y) then
      for i := 0 to aSKP.Parcely.LastIndex do
      begin
         P := Tgis_Parcela(aSKP.Parcely.Elements[i]);
         for j := 0 to P.LastIndex do
         begin
            e := P.Elements[j];
            if not (e is Tgis_ParcelaCast) then continue;
            PC := Tgis_ParcelaCast(e);
            if PC.IsHitByXY(X,Y,0) then
            begin
               result := true;
               ActualParCast := PC;
               ActualParcela := P;
               exit;
            end;
         end;
      end;
      if not aSKP.NoParcast then
      if aSKP.ParCasti.PointIsInside(X,Y) then
      for i := 0 to aSKP.ParCasti.LastIndex do
      begin
         PC := Tgis_ParcelaCast(aSKP.ParCasti.Elements[i]);
         if PC.IsHitByXY(X,Y,0) then
         begin
            result := true;
            ActualParCast := PC;
            ActualParcela := PC.Parcela;
            exit;
         end;
      end;
   end
   else
   begin
      if aSKP.IsHitByXY(X,Y,0) then
      begin
         result := true;
         exit;
      end;
   end;
end;

INITIALIZATION
   RegisterOrbisClass(TParcelModelSelector);
END.
