unit t_SoupisParcel;

interface
uses
   Classes,
{$IF RTLVersion >= 18}    // BDS 2006
   Excel2000,
{$ELSE}
   Excel97,
{$IFEND}
   Elements,
   e_Geometry,
   e_KatastrModel,
   e_LHCModel,
   e_QTree,
   t_ForestModelSelector,
   t_ParcelModelSelector,
   Tools;

type

TSoupisParcel = class(TTool)
private
    FLHCName: string;
    FParcelModel: Tgis_ParcelModel;
    ExcelApplication: TExcelApplication;
    ExcelWorksheet: TExcelWorksheet;
    ExcelWorkbook: TExcelWorkbook;
    FLHO: boolean;
    FLastRow: integer;
    FVlastnikCisloPropertyName: string;
    FVlastnikPropertyName: string;
    FDETLines: TEntity;
    FLevelQT: TRectEntityQTree;
    FFMSelector: TForestModelSelector;
    FPMSelector: TParcelModelSelector;
    FPARLines: TEntity;
    FLines: TPolyLineString;
    FODDs: TEntity;
    FValidSKP: TEntity;
    FByPAR: boolean;
    FForestModel: Tgis_ForestModel;
    FDynamicProps: string;
    FPARDynamicProps: string;
    FByPSK: boolean;
    FMaxOdd: integer;
    FMinODD: integer;
    FTolerance: double;
    FBySKP: boolean;
    FHTML: boolean;
    FText: System.Text;
    FFileName: string;
    FFilterSKP: boolean;
    FPRLBySKP: boolean;
    FAutoValidSKP: boolean;
    FDETAreas: TEntity;
    FFilterElementForPARZARne: TElement;
    FTest: boolean;
    FMaxPRLRowChars: integer;
    FPUPFLne: boolean;
    FKultury: boolean;
    FKultury10: TEntity;
    FKultury11: TEntity;
    FKulturyPUPFL: TEntity;
    FDruh_BzlPropertyName: string;
    FPoznamkaPropertyName: string;
    Format: OleVariant;
    FSKPComment: boolean;
    FObce: boolean;
    FPRLPorost: boolean;
    procedure SetLHCName(const Value: string);
    procedure SetParcelModel(const Value: Tgis_ParcelModel);
    procedure SetLHO(const Value: boolean);
    procedure SetLastRow(const Value: integer);
    procedure SetVlastnikCisloPropertyName(const Value: string);
    procedure SetVlastnikPropertyName(const Value: string);
    procedure SetDETLines(const Value: TEntity);
    procedure SetLevelQT(const Value: TRectEntityQTree);
    function GetFMSelector: TForestModelSelector;
    procedure SetPARLines(const Value: TEntity);
    function GetLines: TPolyLineString;
    procedure SetByPAR(const Value: boolean);
    procedure SetForestModel(const Value: Tgis_ForestModel);
    function GetODDs: TEntity;
    function GetPMSelector: TParcelModelSelector;
    procedure SetDynamicProps(const Value: string);
    procedure SetPARDynamicProps(const Value: string);
    procedure SetByPSK(const Value: boolean);
    procedure SetMaxOdd(const Value: integer);
    procedure SetMinODD(const Value: integer);
    procedure SetTolerance(const Value: double);
    procedure SetBySKP(const Value: boolean);
    procedure SetHTML(const Value: boolean);
    procedure SetFileName(const Value: string);
    procedure TableRowStart;
    procedure TableRowEnd;
    procedure TableCell(Value: string);
    procedure TableCellSpan(Span: integer; Value: string);
    procedure SetFilterSKP(const Value: boolean);
    function GetValidSKP: TEntity;
    procedure SetValidSKP(const Value: TEntity);
    procedure SetPRLBySKP(const Value: boolean);
    procedure SetAutoValidSKP(const Value: boolean);
    procedure SetDETAreas(const Value: TEntity);
    procedure SetFilterElementForPARZARne(const Value: TElement);
    procedure SetTest(const Value: boolean);
    procedure SetMaxPRLRowChars(const Value: integer);
    procedure SetPUPFLne(const Value: boolean);
    procedure SetKultury(const Value: boolean);
    function GetKultury10: TEntity;
    function GetKultury11: TEntity;
    function GetKulturyPUPFL: TEntity;
    procedure SetDruh_BzlPropertyName(const Value: string);
    procedure SetPoznamkaPropertyName(const Value: string);
    procedure SetObce(const Value: boolean);
    procedure SetSKPComment(const Value: boolean);
    procedure SetPRLPorost(const Value: boolean);
public
   procedure DoIt; override;
   function FIndValidSKP: boolean;
   procedure PrepareQTree;
   procedure PrepareODDs;
   //ByPAR true
   procedure ProcessKultury;
   procedure ProcessKultury_z_10;
   procedure ProcessKultury_do_10;
   procedure ProcessKultury_OP;
   procedure ProcessK_z_10(PSKNumber: integer);
   procedure ProcessK_do_10;
   function  GetPRLStringKultury(PRL: TEntity): TStringList;
   procedure processKatastr(K: TGis_Katastr);
   procedure ProcessSkupinaParcel(SKP: Tgis_SkupinaParcel; var SumV,SumVz,SumP,Sum: double);
   procedure processParcela(P: TGis_Parcela);
   function  processParcelaBySKP(P: TGis_Parcela; SKP: Tgis_SkupinaParcel; EPRL,ListPRL: TEntity; var SumVym: double): boolean;
   procedure processParcela_HTML(P: TGis_Parcela);
   procedure ProcessKatastrHeader(K: TGis_Katastr);
   procedure ProcessKatastrHeaderBySKP(K: TGis_Katastr);
   procedure ProcessKatastrHeader_HTML(K: TGis_Katastr);
   procedure ProcessHeaderForKultury;
   function  FindForestUnits(P: Tgis_Parcela;var PSK: Tgis_PorostniSkupina; SKP: Tgis_SkupinaParcel; var CastP: boolean; var DifP: double): TEntity;
   function  FindForestUnitsForKultury(P: Tgis_Parcela; MinSK,MaxSk: integer; BySK: boolean): TEntity;
   //ByPAR false
   procedure ProcessOddsHeader;
   procedure ProcessOddeleni(O: Tgis_Oddeleni);
   procedure ProcessDIlec(D: Tgis_Dilec);
   procedure ProcessPorost(P: Tgis_Porost);
   function  FindParcely(P: Tgis_Porost): TEntity;

   property LastRow: integer read FLastRow write SetLastRow;
   property LevelQT: TRectEntityQTree read FLevelQT write SetLevelQT;
   property FMSelector: TForestModelSelector read GetFMSelector;
   property PMSelector: TParcelModelSelector read GetPMSelector;
   property Lines: TPolyLineString read GetLines;
   property ODDs: TEntity read GetODDs;
   property Kultury10: TEntity read GetKultury10;
   property Kultury11: TEntity read GetKultury11;
   property KulturyPUPFL: TEntity read GetKulturyPUPFL;
published
   property ByPAR: boolean read FByPAR write SetByPAR;
   property BySKP: boolean read FBySKP write SetBySKP;
   property ByPSK: boolean read FByPSK write SetByPSK;
   property ForestModel: Tgis_ForestModel read FForestModel write SetForestModel;
   property ParcelModel: Tgis_ParcelModel read FParcelModel write SetParcelModel;
   property DETLines: TEntity read FDETLines write SetDETLines;
   property DETAreas: TEntity read FDETAreas write SetDETAreas;
   property PARLines: TEntity read FPARLines write SetPARLines;
   property ValidSKP: TEntity read FValidSKP write SetValidSKP;
   property VlastnikPropertyName: string read FVlastnikPropertyName write SetVlastnikPropertyName;
   property VlastnikCisloPropertyName: string read FVlastnikCisloPropertyName write SetVlastnikCisloPropertyName;
   property LHCName: string read FLHCName write SetLHCName;
   property LHO: boolean read FLHO write SetLHO;
   property DynamicProps: string read FDynamicProps write SetDynamicProps;
   property PARDynamicProps: string read FPARDynamicProps write SetPARDynamicProps;
   property MinODD: integer read FMinODD write SetMinODD;
   property MaxOdd: integer read FMaxOdd write SetMaxOdd;
   property Tolerance: double read FTolerance write SetTolerance;
   property HTML: boolean read FHTML write SetHTML;
   property FileName: string read FFileName write SetFileName;
   property FilterSKP: boolean read FFilterSKP write SetFilterSKP;
   property PRLBySKP: boolean read FPRLBySKP write SetPRLBySKP;
   property AutoValidSKP: boolean read FAutoValidSKP write SetAutoValidSKP;
   property FilterElementForPARZARne: TElement read FFilterElementForPARZARne write SetFilterElementForPARZARne;
   property Test: boolean read FTest write SetTest;
   property MaxPRLRowChars: integer read FMaxPRLRowChars write SetMaxPRLRowChars;
   property PUPFLne: boolean read FPUPFLne write SetPUPFLne;
   property Kultury: boolean read FKultury write SetKultury;
   property Obce: boolean read FObce write SetObce;
   property PRLPorost: boolean read FPRLPorost write SetPRLPorost;
   property SKPComment: boolean read FSKPComment write SetSKPComment;
   property PoznamkaPropertyName: string read FPoznamkaPropertyName write SetPoznamkaPropertyName;
   property Druh_BzlPropertyName: string read FDruh_BzlPropertyName write SetDruh_BzlPropertyName;
end;

implementation
uses
   Dialogs,
   u_Math,
   SysUtils,
   Forms,
   Variants,
   Worlds,
   cpConvrt,
   e_Class,
   e_GeometryElement,
   e_Props,
   e_Type,
   g_Elems,
   t_AreMak,
   t_LHCUtils,
   t_LineExtractor,
   t_QTreeTools,
   t_Sort;

{ TSoupisParcel }

procedure TSoupisParcel.DoIt;
var
   OV: OleVariant;
   EWB: _WorkBook;
   EWS: _WorkSheet;
   i: integer;
   K : Tgis_Katastr;
   O: Tgis_Oddeleni;
begin
   if ByPar then
   begin
      if (ParcelModel = nil) or ParcelModel.NoElements then
      begin
         ShowMessage('Model parcel neni definovn nebo neobsahuje katastry!');
         exit;
      end;
   end
   else
   begin
      if (ForestModel = nil) or ForestModel.NoElements then
      begin
         ShowMessage('Model LHC neni definovn nebo je ponkud przdn!');
         exit;
      end;
   end;
   if (DetLines = nil) or DetLines.NoElements then
   begin
      ShowMessage('Vrstva Lini DETailu nen definovna nebo je przdn!');
      exit;
   end;
   if (PARLines = nil) or PARLines.NoElements then
   begin
      ShowMessage('Vrstva Lini PARcel nen definovna nebo je przdn!');
      exit;
   end;
   if ByPAR then   //
   begin
      if AutoValidSKP then
      if (DETAreas = nil) or (DETAreas.NoElements) then
      begin
         ShowMessage('Vrstva ARE_DET pro filtrovn SKP nen definovna nebo je przdn!');
         exit;
      end;
      if AutoValidSKP then
      begin
         if not FIndValidSKP then
         begin
            ShowMessage('Nelze nalzt dn validn SKP ... zkuste propojen PAR a DET!');
            exit;
         end;
      end;
      StartProgress('Soupis Parcel do Excelu podle Parcel',0,ParcelModel.LastIndex);
   end
   else
   begin
      PrepareODDs;
      StartProgress('Soupis Parcel do Excelu podle Oddeleni',0,ODDs.LastIndex);
   end;
   Application.ProcessMessages;
   // Priprava linii a tak
   PrepareQTree;
   //exit;
   if HTML then
   begin
      FileName := 'c:\Dokumenty\XML\Soupis.htm';
      system.assign(FText, FileName);
      system.rewrite(FText);
      system.writeln(FText, '<html>');
      system.writeln(FText, ' ');
      system.writeln(FText, '<head>');
      system.writeln(FText, '<meta http-equiv="Content-Language" content="en-us">');
      system.writeln(FText, '<meta name="GENERATOR" content="ORBIS LED 4.0">');
      system.writeln(FText, '<meta http-equiv="Content-Type" content="text/html; charset=windows-1250">');
      system.writeln(FText, '<title>Soupis parcel</title>');
      system.writeln(FText, '</head>');
      system.writeln(FText, '<body>');
      system.writeln(FText, ' ');
      system.writeln(FText, '<table border="1" cellpadding="0" cellspacing="0" style="border-collapse: collapse" bordercolor="#111111" >');
      system.writeln(FText, '  <tbody>');
      {
      TableRowStart;
      TableCell('<b>Katastr:</b>');
      TableCell('<b>Nzev:</b>');
      TableCell('<b>Pocet parcel:</b>');
      TableRowEnd;
      }

   end
   else
   begin
      ProgressText := 'Inicializace Excelu ...';
      Application.ProcessMessages;
      ExcelApplication := TExcelApplication.Create(Application);
      ExcelWorksheet   := TExcelWorksheet.Create(Application);
      ExcelWorkbook    := TExcelWorkbook.Create(Application);
      try
         ExcelApplication.Connect;
         ExcelApplication.Visible[0] := true;
      except
         ShowMessage('Nepodailo se navzat spojen s EXCELem !');
         ExcelWorksheet.Free;
         ExcelWorkbook.Free;
         ExcelApplication.Free;
         exit;
      end;
      try
         OV := True;
         //EWB := ExcelApplication.Workbooks.Add(OV,0);
         try
         ExcelApplication.Workbooks.Add(XlWBATemplate(xlWBATWorksheet),0);
         except
            //on E: Exception do ShowMessage(E.Message);
         end;
         EWB := ExcelApplication.Workbooks[1];
         ExcelWorkBook.ConnectTo(EWB);
         ExcelWorkSheet.ConnectTo(ExcelApplication.ActiveSheet as _WorkSheet);
         if Kultury then
            ExcelWorkSheet.Name := 'Kultury'
         else
            ExcelWorkSheet.Name := 'Soupis parcel';

         //ExcelWorkSheet.ConnectTo(ExcelWorkSheet.Next as _WorkSheet);
         {
         OV := 0;
         OV1 := 0;
         OV2 := 1;
         OV3 := ExcelWorkSheet.Type_[0]; //xlWorksheet
         ExcelWorkSheet.ConnectTo(ExcelWorkbook.WorkSheets.Add(OV,OV1,OV2,OV3,0) as _WorkSheet);
         ExcelWorkSheet.Name := 'tralala';
         }
         {
         // novej workbook funguje dobre...
         EWB := ExcelApplication.Workbooks.Add(OV,0);
         ExcelWorkBook.ConnectTo(EWB);
         ExcelWorkSheet.ConnectTo(ExcelApplication.ActiveSheet as _WorkSheet);
         ExcelWorkSheet.Name := 'tralala'
         }
      except
         ShowMessage('Nepodailo se navzat spojen s EXCELem !(1)');
         ExcelWorksheet.Free;
         ExcelWorkbook.Free;
         ExcelApplication.Free;
         exit;
      end;
   end;
   if ByPar then
      ParcelModel.Sort := true;
   //MaxProgress := ParcelModel.LastIndex;
   Application.ProcessMessages;
   LastRow := 1;
   try
      if Kultury then
      begin
         ProcessKultury;
      end
      else
      if ByPar then
      begin
         for i := 0 to ParcelModel.LastIndex do
         begin
            Progress := i;
            K := Tgis_Katastr(ParcelModel.Elements[i]);
            ProcessKatastr(K);
         end;
      end
      else
      begin
         ProcessODDSHeader;
         for i := 0 to ODDS.LastIndex do
         begin
            Progress := i;
            O := Tgis_Oddeleni(ODDS.Elements[i]);
            if (O.Cislo >= MinOdd) and ((MaxOdd = 0) or (O.Cislo <= MaxOdd)) then
               ProcessOddeleni(O);
         end;
      end;
   finally
      if HTML then
      begin
         system.writeln(FText, '  </tbody>');
         system.writeln(FText, '</table>');
         system.writeln(FText, '</body>');
         system.writeln(FText, '</html>');
         System.close(FText);
      end
      else
      begin
         ExcelWorksheet.Free;
         ExcelWorkbook.Free;
         ExcelApplication.Free;
      end;
   end;
end;

function TSoupisParcel.FindForestUnits(P: Tgis_Parcela;var PSK: Tgis_PorostniSkupina; SKP: Tgis_SkupinaParcel; var CastP: boolean; var DifP: double): TEntity;
var
   W: TQTreeWalker;
   Lin: TRectEntity;
   El: TElement;
   AM: TAreasFromClearLines;
   Areas: TAreaString;
   i,j: integer;
   Area: TAreaElement;
   _PSK: Tgis_PorostniSkupina;
   Por: Tgis_Porost;
   //Dil: Tgis_Dilec;
   PC: Tgis_ParcelaCast;
   P_isHit: boolean;
   SumPC,SumAre: double;
begin
   CastP := false;
   PSK := nil;
   result := TOwnerEntity.Create;
   Lin := TRectEntity.Create;
   Lin.Name := 'L';
   if LevelQT = nil then exit;
   W := TQTreeWalker.Create;
   El := W.GetFirstElementForRectangle(LevelQT,P.Minx,P.MinY,P.MaxX,P.MaxY,1);
   while El <> nil do
   begin
      if not El.IsMyOwner(Lin) then
         Lin.AddAndUpdateRect(El);
      El := W.GetNextElement;
   end;
   W.Free;
   AM := TAreasFromClearLines.Create;
   AM.Input := Lin;
   AM.DoIt;
   Areas := TAreaString(AM.Output);
   SumPC := 0;
   SumAre := 0;
   for j := 0 to P.LastIndex do
   begin
      El := P.Elements[j];
      if not (El is Tgis_ParcelaCast) then continue;
      PC := Tgis_ParcelaCast(El);
      if (SKP <> nil) and (PC.SkupinaParcel <> SKP) then continue;
      SumPC := SumPC + PC.Plocha;
   end;
   for i := 0 to Areas.LastIndex do
   begin
      El := Areas.Elements[i];
      if not(El is TAreaElement) then continue;
      Area := TAreaElement(El);
{ TODO -oPPK -cOpravit : Soupis - Tolerance a porovnavani ploch zrevidovat }
      //if (Tolerance > 0) AND (Area.Area/10000 < Tolerance) then
      //   continue;
      // ! nemuze se vyhodit mala plocha, protoze pak nesouhlasi celkovy soucet
      //Hitnuti parcely
      P_isHit := false;
      for j := 0 to P.LastIndex do
      begin
         El := P.Elements[j];
         if not (El is Tgis_ParcelaCast) then continue;
         PC := Tgis_ParcelaCast(El);
         if (SKP <> nil) and (PC.SkupinaParcel <> SKP) then continue;
         if not PC.IsHitByXY(Area.X,Area.Y,0) then continue;
         P_isHit := true;
      end;
      //if not P.IsHitByXY(Area.X,Area.Y,0) then continue;
      if not P_isHit then continue;
      FMSelector.X := Area.X;
      FMSelector.Y := Area.Y;
      FMSelector.DoIt;
      _PSK := Tgis_PorostniSkupina(FMSelector.ActualFrame);
      if _PSK = nil then continue;
      PSK := _PSK;
      if BySKP then
      begin
         if not PSK.IsMyOwner(result) then
            result.AddElement(PSK);
         SumAre := SumAre + Area.Area;
         Continue;
      end;
      Por := _PSK._Porost;
      if Por = nil then continue;
      SumAre := SumAre + Area.Area;
      if Por.IsMyOwner(result) then continue;
      result.AddElement(Por);
   end;
   DifP := 0;
   CastP := ABS(SumPC - SumAre)/10000 > Tolerance;
   if CastP then
      DifP := SumPC - SumAre;
   if Areas.Count = 0 then
   begin
      World.ObjectModel.NextElement := Lin;
   end
   else
      Lin.Free;
   AM.Free;
   Areas.Free;
end;

function TSoupisParcel.FindForestUnitsForKultury(P: Tgis_Parcela; MinSK,MaxSK: integer; BySK: boolean): TEntity;
var
   W: TQTreeWalker;
   Lin: TRectEntity;
   El: TElement;
   AM: TAreasFromClearLines;
   Areas: TAreaString;
   i,j: integer;
   Area: TAreaElement;
   MA: TMultiAreaElement;
   _PSK: Tgis_PorostniSkupina;
   SPozn: string;
   SK: TEntity;
   SE: TStringElement;
   PRL: TEntity;
   PC: Tgis_ParcelaCast;
   P_isHit: boolean;
   PSKNumber: integer;
   PSKNumberS: string;
   procedure SetPSKNumber;
   var
      ii: integer;
      S,SV: string;
   begin
      // prevod skupina na cislo
      S := _PSK.Skupina;
      ii := 1;
      if S <> '' then
      begin
         while not (ii > Length(S)) and (S[ii]=' ') do Inc(ii);
         while not (ii > Length(S)) and (S[ii] in ['0'..'9']) do
         begin
            SV := SV + S[ii];
            Inc(ii);
         end;
         PSKNumber := StrToIntDef(SV,0);
         PSKNumberS := SV;
      end;
   end;
begin
   result := TOwnerEntity.Create;
   Lin := TRectEntity.Create;
   Lin.Name := 'L';
   if LevelQT = nil then exit;
   W := TQTreeWalker.Create;
   El := W.GetFirstElementForRectangle(LevelQT,P.Minx,P.MinY,P.MaxX,P.MaxY,1);
   while El <> nil do
   begin
      if not El.IsMyOwner(Lin) then
         Lin.AddAndUpdateRect(El);
      El := W.GetNextElement;
   end;
   W.Free;
   AM := TAreasFromClearLines.Create;
   AM.Input := Lin;
   AM.DoIt;
   Areas := TAreaString(AM.Output);
   for i := 0 to Areas.LastIndex do
   begin
      El := Areas.Elements[i];
      if not(El is TAreaElement) then continue;
      Area := TAreaElement(El);
      //Hitnuti parcely
      P_isHit := false;
      for j := 0 to P.LastIndex do
      begin
         El := P.Elements[j];
         if not (El is Tgis_ParcelaCast) then continue;
         PC := Tgis_ParcelaCast(El);
         if not PC.IsHitByXY(Area.X,Area.Y,0) then continue;
         P_isHit := true;
      end;
      //if not P.IsHitByXY(Area.X,Area.Y,0) then continue;
      if not P_isHit then continue;
      FMSelector.X := Area.X;
      FMSelector.Y := Area.Y;
      FMSelector.DoIt;
      _PSK := Tgis_PorostniSkupina(FMSelector.ActualFrame);
      if _PSK = nil then continue;
      SetPSKNumber;
      if (PSKNumber < MinSK) or (PSKNumber > MaxSK) then continue; // nelezi v intervalu
      if BySK then
      begin
         El := result.FindOrCreateElementName(TOwnerEntity,PSKNumberS);
         SK := TEntity(El);
         El := SK.FindOrCreateElementName(TStringElement,PoznamkaPropertyName);
         SE := TStringElement(El);
         SPozn := _PSK.GetPropertyStringByName(PoznamkaPropertyName);
         if SPozn <> '' then
            SE.Value := SPozn;
         El := SK.FindOrCreateElementName(TStringElement,Druh_BzlPropertyName);
         SE := TStringElement(El);
         SPozn := _PSK.GetPropertyStringByName(Druh_BzlPropertyName);
         if SPozn <> '' then
            SE.Value := SPozn;
         El := SK.FindOrCreateElementName(TMultiAreaElement,'MARE');
         MA := TMultiAreaElement(El);
         MA.AddAndUpdateRect(Area);
         El := SK.FindOrCreateElementName(TOwnerEntity,'PRL');
         PRL := TEntity(El);
         El := PRL.FindOrCreateElementName(TOwnerEntity,_PSK.GetPropertyStringByName('Oddeleni'));
         PRL := TEntity(El);
         if LHO then
         begin
             El := PRL.FindOrCreateElementName(TOwnerEntity,_PSK.GetPropertyStringByName('Dilec'));
             PRL := TEntity(El);
             PRL.FindOrCreateElementName(TOwnerEntity,_PSK.GetPropertyStringByName('Porost'));
         end
         else
         begin
            if not PRLPorost then
               PRL.FindOrCreateElementName(TOwnerEntity,_PSK.GetPropertyStringByName('Dilec'))
            else
               PRL.FindOrCreateElementName(TOwnerEntity,_PSK.GetPropertyStringByName('Porost'));
         end;
      end
      else
      begin
         El := result.FindOrCreateElementName(TMultiAreaElement,'MARE');
         MA := TMultiAreaElement(El);
         MA.AddAndUpdateRect(Area);
         El := result.FindOrCreateElementName(TOwnerEntity,'PRL');
         PRL := TEntity(El);
         El := PRL.FindOrCreateElementName(TOwnerEntity,_PSK.GetPropertyStringByName('Oddeleni'));
         PRL := TEntity(El);
         if LHO then
         begin
             El := PRL.FindOrCreateElementName(TOwnerEntity,_PSK.GetPropertyStringByName('Dilec'));
             PRL := TEntity(El);
             PRL.FindOrCreateElementName(TOwnerEntity,_PSK.GetPropertyStringByName('Porost'));
         end
         else
         begin
            if not PRLPorost then
               PRL.FindOrCreateElementName(TOwnerEntity,_PSK.GetPropertyStringByName('Dilec'))
            else
               PRL.FindOrCreateElementName(TOwnerEntity,_PSK.GetPropertyStringByName('Porost'));
         end;
      end;
   end;
   Lin.Free;
   AM.Free;
   Areas.Free;
end;

function TSoupisParcel.FindParcely(P: Tgis_Porost): TEntity;
var
   W: TQTreeWalker;
   Lin: TRectEntity;
   El: TElement;
   AM: TAreasFromClearLines;
   Areas: TAreaString;
   i: integer;
   Area: TAreaElement;
   PSK: Tgis_PorostniSkupina;
   Por: Tgis_Porost;
   //Dil: Tgis_Dilec;
   FU: Tgis_ForestUnit;
   Par: Tgis_Parcela;
   ParC: Tgis_ParcelaCast;
begin
   result := TOwnerEntity.Create;
   Lin := TRectEntity.Create;
   Lin.Name := 'L';
   if LevelQT = nil then exit;
   W := TQTreeWalker.Create;
   El := W.GetFirstElementForRectangle(LevelQT,P.Minx,P.MinY,P.MaxX,P.MaxY,1);
   while El <> nil do
   begin
      Lin.AddAndUpdateRect(El);
      El := W.GetNextElement;
   end;
   W.Free;
   AM := TAreasFromClearLines.Create;
   AM.Input := Lin;
   AM.DoIt;
   Areas := TAreaString(AM.Output);
   if Areas <> nil then
   for i := 0 to Areas.LastIndex do
   begin
      El := Areas.Elements[i];
      if not(El is TAreaElement) then continue;
      Area := TAreaElement(El);
      //if not P.IsHitByXY(Area.X,Area.Y,0) then continue;
      FMSelector.X := Area.X;
      FMSelector.Y := Area.Y;
      FMSelector.DoIt;
      PSK := Tgis_PorostniSkupina(FMSelector.ActualFrame);
      if PSK = nil then continue;
      Por := PSK._Porost;
      if Por = nil then continue;
      if Por <> P then continue;
      PMSelector.X := Area.X;
      PMSelector.Y := Area.Y;
      PMSelector.DoIt;
      Par := PMSelector.ActualParcela;
      ParC := PMSelector.ActualParCast;
      if Par = nil then continue;
      if ByPSK then
      begin
         FU := Tgis_FOrestUnit(Result.FindOrCreateElementName(Tgis_ForestUnit,PSK.Name));
         //FU.Properties := PSK;
         FU := Tgis_FOrestUnit(FU.FindOrCreateElementName(Tgis_ForestUnit,PAR.Name));
         FU.Properties := PAR;
         FU.AddGeometry(Area);
         FU.AddElement(PSK.Properties);
         FU.AddElement(ParC);
      end
      else
      begin
         FU := Tgis_FOrestUnit(Result.FindOrCreateElementName(Tgis_ForestUnit,PAR.Name));
         FU.Properties := PAR;
         FU.AddGeometry(Area);
         FU.AddElement(PSK.Properties);
         FU.AddElement(ParC);
      end;
   end;
   Lin.Free;
   AM.Free;
   if Areas <> nil then
      Areas.Free;
end;

function TSoupisParcel.FIndValidSKP: boolean;
var
   i: integer;
   En: TEntity;
   e: TElement;
   PSK: Tgis_PorostniSkupina;
   SKP: Tgis_SkupinaParcel;
begin
   result := true;
   En := GetValidSKP;
   for i := 0 to DETAreas.LastIndex do
   begin
      e := DETAreas.Elements[i];
      if not (e is Tgis_PorostniSkupina) then continue;
      PSK := Tgis_PorostniSkupina(e);
      SKP := PSK.SkupinaParcel;
      if not SKP.IsMyOwner(En) then
         En.AddElement(SKP);
   end;
   if En.NoElements then
      result := false;
end;

function TSoupisParcel.GetFMSelector: TForestModelSelector;
begin
   if FFMSelector = nil then
   begin
      FFMSelector := TForestModelSelector.Create;
      FFMSelector.FindInput;
      FFMSelector.FrameClass := Tgis_PorostniSkupina;
      FFMSelector.ReferenceFrameClass := Tgis_PorostniSkupina;
      AddElement(FFMSelector);
   end;
   result := FFMSelector;
end;

function TSoupisParcel.GetKultury10: TEntity;
begin
   if FKultury10 = nil then
   begin
      FKultury10 := TOwnerEntity.Create;
      FKultury10.Name := 'K10';
      AddElement(FKultury10);
   end;
   result := FKultury10;
end;

function TSoupisParcel.GetKultury11: TEntity;
begin
   if FKultury11 = nil then
   begin
      FKultury11 := TOwnerEntity.Create;
      FKultury11.Name := 'K11';
      AddElement(FKultury11);
   end;
   result := FKultury11;
end;

function TSoupisParcel.GetKulturyPUPFL: TEntity;
begin
   if FKulturyPUPFL = nil then
   begin
      FKulturyPUPFL := TOwnerEntity.Create;
      FKulturyPUPFL.Name := 'KPUPFL';
      AddElement(FKulturyPUPFL);
   end;
   result := FKulturyPUPFL;
end;

function TSoupisParcel.GetLines: TPolyLineString;
begin
   if FLines = nil then
   begin
      FLines := TPolyLineString.Create;
      AddElement(FLines);
   end;
   result := FLines;
end;

function TSoupisParcel.GetODDs: TEntity;
begin
   if FODDs = nil then
   begin
      FOdds := TOwnerEntity.Create;
      AddElement(FOdds);
   end;
   result := FODDs;
end;

function TSoupisParcel.GetPMSelector: TParcelModelSelector;
begin
   if FPMSelector = nil then
   begin
      FPMSelector := TParcelModelSelector.Create;
      FPMSelector.FindInput;
      FPMSelector.SelectByParcely := true;
      //FPMSelector.FrameClass := Tgis_PorostniSkupina;
      //FFMSelector.ReferenceFrameClass := Tgis_PorostniSkupina;
      AddElement(FPMSelector);
   end;
   result := FPMSelector;
end;

function TSoupisParcel.GetPRLStringKultury(PRL: TEntity): TStringList;
var
   prli,prlj,prlk,i: integer;
   e: TElement;
   EOdd: TEntity;
   EDil: TEntity;
   SortI: TSortElementsByInteger;
   SortS: TSortElementsByString;
   S: string;
begin
   Result := TStringList.Create;
   SortI := TSortElementsByInteger.Create;
   SortS := TSortElementsByString.Create;
   //result := '';
   SortI.Input := PRL;
   SortI.PropertyName := 'Name';
   SortI.DoIt;
   S := '';
   for prli := 0 to PRL.LastIndex do
   begin
      e := PRL.Elements[prli];
      EOdd := TEntity(e);
      SortS.Input := e;
      SortS.PropertyName := 'Name';
      SortS.DoIt;
      S := S +EOdd.Name+' ';
      for prlj := 0 to EOdd.LastIndex do
      begin
         e := EOdd.Elements[prlj];
         S := S + e.Name;
         if LHO then
         begin
            EDil := TEntity(e);
            SortS.Input := e;
            SortS.PropertyName := 'Name';
            SortS.DoIt;
            for prlk := 0 to EDil.LastIndex do
            begin
               e := EDil.Elements[prlk];
               S := S + e.Name;
            end;
         end;
      end;
      if prli = 0 then // prvni oddeleni
      begin
         result.Add(S);
      end
      else
      begin
         i := result.Count-1;
         if Length(result.Strings[i])+1 +Length(S)>MaxPRLRowChars then
         begin // moc dlouhy radek
            result.Add(S);
         end
         else
         begin
            S := result.Strings[i]+','+S;
            result.Strings[i] := S;
         end;
      end;
      S := '';
   end;
   if result.Count > 0 then
   for i := 0 to result.count -1 do
   begin
      result.Strings[i]:= '="'+result.Strings[i]+'"';
   end;
   SortI.Free;
   SortS.Free;
end;

function TSoupisParcel.GetValidSKP: TEntity;
begin
   if FValidSKP = nil then
   begin
      FValidSKP := TOwnerEntity.Create;
      FValidSKP.Name := 'Valid SKP';
      AddElement(FValidSKP);
   end;
   result := FValidSKP;
end;

procedure TSoupisParcel.PrepareODDs;
var
   i: integer;
   //FU: Tgis_ForestUnit;
   LHC: Tgis_LHC;
   LS: Tgis_LS;
   Sort: TSortElementsByInteger;
begin
   ForestModel.Sort := true;
   ODDS.FreeAll;
   if ForestModel.NoElements then exit;
   LHC := Tgis_LHC(ForestModel.Elements[0]);
   if LHC.NoItems then exit;
   for i := 0 to LHC.Items.LastIndex do
   begin
      LS := Tgis_LS(LHC.Items.ForestUnit[i]);
      if LS.NoItems then continue;
      Odds.AddAllFrom := LS.Items;
   end;
   Sort := TSortElementsByInteger.Create;
   Sort.PropertyName := 'Cislo';
   Sort.Input := Odds;
   Sort.DoIt;
   Sort.Free;
end;

procedure TSoupisParcel.PrepareQTree;
var
   TQ: TAddEntityToQTree;
   LEX: TLineExtractor;
   PARL: TPolyLineString;
   DETL: TPolyLineString;
   CL: TClearLines;
begin
   if DETLines = nil then exit;
   if PARLines = nil then exit;
   ProgressText := 'Koprovn lini';
   Application.ProcessMessages;
   PARL := TPolyLineString.Create;
   AddElement(PARL);
   DETL := TPolyLineString.Create;
   AddElement(DETL);
   LEX := TLineExtractor.Create;
   LEX.CopyLines := true;
   LEX.Input := PARLines;
   LEX.Output := PARL;
   LEX.DoIt;
   LEX.Input := DETLines;
   LEX.Output := DETL;
   LEX.DoIt;
   ProgressText := '... a spojen lini';
   Application.ProcessMessages;
   CL := TClearLines.Create;
   CL.FixedLines := PARL;
   CL.Input := DETL;
   CL.SelfTest := true;
   //CL.SnapDistance := 0.5;//!!!!!!!!!!!!!!
   CL.DoIt;
   CL.Free;
   LEX.CopyLines := false;
   LEX.NoDuplicates := True;
   LEX.Output := Lines;
   LEX.Input := PARL;
   LEX.DoIt;
   LEX.Input := DETL;
   LEX.DoIt;
   PARL.Free;
   DETL.Free;
   LEX.Free;
   LevelQT :=TRectEntityQTree.Create;
   LevelQT.Name := 'LEVEL QT';
   TQ := TAddEntityToQTree.Create;
   TQ.Input := Lines;
   TQ.Output := LevelQT;
   TQ.DoIt;
   TQ.Free;
end;

procedure TSoupisParcel.ProcessDIlec(D: Tgis_Dilec);
var
   i: integer;
   P: Tgis_Porost;
begin
   if D.NoItems then exit;
   for i := 0 to D.Items.LastIndex do
   begin
      P := Tgis_Porost(D.Items.ForestUnit[i]);
      ProcessPorost(P);
   end;
end;

procedure TSoupisParcel.ProcessHeaderForKultury;
var
   A: OleVariant;
   SRow: string;
   StartCH, EndCH: OleVariant;
begin
   SRow := IntToStr(LastRow);
   A := 'A'+SRow;
   ExcelWorkSheet.Range[A,A].Value := 'KD KATASTRU';
   A := 'B'+SRow;
   ExcelWorkSheet.Range[A,A].Value := 'KATASTRLN ZEM';
   A := 'C'+SRow;
   ExcelWorkSheet.Range[A,A].Value := 'PARCELA';
   //ExcelWorkSheet.Range[A,A].Orientation := 90;
   A := 'D'+SRow;
   ExcelWorkSheet.Range[A,A].Value := 'PODLOMEN';
   A := 'E'+SRow;
   ExcelWorkSheet.Range[A,A].Value := 'ST';
   //ExcelWorkSheet.Range[A,A].ColumnWidth := 16;
   A := 'F'+SRow;
   ExcelWorkSheet.Range[A,A].Value := 'VMRA / PLOCHA';
   StartCH := 1;
   EndCH := 6;
   ExcelWorkSheet.Range[A,A].Characters[StartCH,EndCH].Font.Bold := true;
   //ExcelWorkSheet.Range[A,A].ColumnWidth := 10;
   A := 'G'+SRow;
   ExcelWorkSheet.Range[A,A].Value := 'KULTURA DLE KN';
   A := 'H'+SRow;
   //ExcelWorkSheet.Range[A,A].ColumnWidth := 1;
   //ExcelWorkSheet.Range[A,A].Value := 'Opert';
   ExcelWorkSheet.Range[A,A].Value := 'NVRH KULTURY';
   //ExcelWorkSheet.Range[A,A].ColumnWidth := 15;
   A := 'I'+SRow;
   ExcelWorkSheet.Range[A,A].Value := 'JED.ROZ.LESA';
   ExcelWorkSheet.Range[A,A].ColumnWidth := 16;
end;

procedure TSoupisParcel.processKatastr(K: TGis_Katastr);
var
   E: TOwnerEntity;
   El: TElement;
   GE: TGeometryElement;
   SK: string;
   SKP,SKP0: Tgis_SkupinaParcel;
   i,j: integer;
   TT: TSortElementsBy2Integers;
   P: Tgis_Parcela;
   PC: Tgis_ParcelaCast;
   FI,LI: integer;
   SRow: string;
   A: OleVariant;
   SKP_SumV,SKP_SumVz,SKP_SumP,SKP_Sum,SumV: double;
   KPK: integer;
   EmptyKat: boolean;
begin
   if K.NoSkupinaParcel then exit;
   SK := 'Katastr: '+ K.Name+'  ';
   Application.ProcessMessages;
   ProgressText := SK;
   if BySKP then
   begin
      EmptyKat := FilterSKP;
      if FilterSKP then
      for i := 0 to K.SkupinyParcel.LastIndex do
      begin
         SKP := Tgis_SkupinaParcel(K.SkupinyParcel.Elements[i]);
         if SKP.IsMyOwner(ValidSKP)then
         begin
            EmptyKat := false;
            break;
         end;
      end;
      if EmptyKat then exit; // prazdny katastr
      ProcessKatastrHeaderBySKP(K);
      SKP_SumV := 0;
      SKP_SumVz := 0;
      SKP_SumP := 0;
      SKP_Sum := 0;
      if not K.NoSkupinaParcel then
      begin
         SKP0 := nil;
         if SKPComment then
         begin
            E := TOwnerEntity.Create;
            For i := 0 to K.SkupinyParcel.LastIndex do
            begin
               SKP := Tgis_SkupinaParcel(K.SkupinyParcel.Elements[i]);
               P := Tgis_Parcela.Create;
               P.NextElement := SKP;
               P.Name := SKP.Comment;
               E.NextElement := P;
            end;
            TT := TSortElementsBy2Integers.Create;
            TT.PropertyName := 'Parcis';
            TT.PropertyName1 := 'Parpod';
            TT.Input := E;
            TT.DoIt;
            TT.Free;
            K.SkupinyParcel.FreeAll;
            for i := 0 to E.LastIndex do
            begin
               P := Tgis_Parcela(E.Elements[i]);
               K.SkupinyParcel.NextElement := P.FindElementClass(Tgis_SkupinaParcel);
            end;
            E.Free;
         end;
         for i := 0 to K.SkupinyParcel.LastIndex do
         begin
            SKP := Tgis_SkupinaParcel(K.SkupinyParcel.Elements[i]);
            if SKP.Cislo = 0 then
               SKP0 := SKP;
            // test na revir
            if SKPComment then
               ProgressText := SK + 'skp:'+SKP.Comment
            else
               ProgressText := SK + 'skp:'+SKP.Name;
            Application.ProcessMessages;
            if FilterSKP and not SKP.IsMyOwner(ValidSKP)then continue;
            ProcessSkupinaParcel(SKP,SKP_SumV,SKP_SumVz,SKP_SumP,SKP_Sum);
         end;
         SRow := IntToStr(LastRow);
         A := 'A'+SRow;
         ExcelWorkSheet.Range[A,A].Value := 'SOUET:';
         A := 'E'+SRow;
         ExcelWorkSheet.Range[A,A].Value := SKP_SumV;
         Format := FormatFloat('0.0000',0);
         ExcelWorkSheet.Range[A,A].NumberFormatLocal := Format;
         if Obce then
            ExcelWorkSheet.Range[A,A].Font.Bold := true;
         A := 'F'+SRow;
         ExcelWorkSheet.Range[A,A].Value := SKP_SumVz;
         Format := FormatFloat('0.00',0);
         ExcelWorkSheet.Range[A,A].NumberFormatLocal := Format;
         if Obce then
            ExcelWorkSheet.Range[A,A].Font.Bold := true;
         if not Obce then
         begin
            A := 'G'+SRow;
            ExcelWorkSheet.Range[A,A].Value := SKP_SumP;
            ExcelWorkSheet.Range[A,A].NumberFormatLocal := Format;
            LastRow := LastRow + 2;
            SRow := IntToStr(LastRow);
            A := 'A'+SRow;
            ExcelWorkSheet.Range[A,A].Value := 'SOUET UITCH PLOCH A VMR:';
            A := 'F'+SRow;
            ExcelWorkSheet.Range[A,A].Value := SKP_Sum;
            ExcelWorkSheet.Range[A,A].Font.Bold := true;
            ExcelWorkSheet.Range[A,A].NumberFormatLocal := Format;
         end;
         LastRow := LastRow + 2;
         // parzar ne
         if SKP0 = nil then exit;
         GE := nil;
         El := FilterElementForPARZARne;
         if (El is TGeometryElement) then
            GE := TGeometryElement(El);
         E := TOwnerEntity.Create;
         if not SKP0.NoParcela then
         for i := 0 to SKP0.Parcely.LastIndex do
         begin
            P := Tgis_Parcela(SKP0.Parcely.Elements[i]);
            if GE <> nil then
            begin
               for j := 0 to P.LastIndex do
               begin
                  El := P.Elements[j];
                  if not (El is Tgis_ParcelaCast) then continue;
                  PC := Tgis_ParcelaCast(El);
                  if GE.IsHitByXY(PC.X,PC.Y,0) then
                     if not P.IsMyOwner(E) then
                        E.AddElement(P);
               end;
            end
            else
               E.AddElement(P);
         end;
         if not SKP0.NoParcast then
         for i := 0 to SKP0.ParCasti.LastIndex do
         begin
            PC := Tgis_ParcelaCast(SKP0.Parcasti.Elements[i]);
            P := PC.Parcela;
            if GE <> nil then
            begin
               if GE.IsHitByXY(PC.X,PC.Y,0) then
                  if not P.IsMyOwner(E) then
                     E.AddElement(P);
            end
            else
               if not P.IsMyOwner(E) then
                  E.AddElement(P);
         end;
         if E.NoElements then
         begin
            E.Free;
            exit;
         end;
         SRow := IntToStr(LastRow);
         A := 'A'+SRow;
         ExcelWorkSheet.Range[A,A].Value := 'Parcely s vmrou pod 0,0050 ha:';
         LastRow := LastRow + 1;
         SRow := IntToStr(LastRow);
         for i := 0 to E.LastIndex do
         begin
            SumV := 0;
            P := Tgis_Parcela(E.Elements[i]);
            KPK := 11;
            for j := 0 to P.LastIndex do
            begin
               El := P.Elements[j];
               if not (El is Tgis_ParcelaCast) then continue;
               PC := Tgis_ParcelaCast(El);
               if GE <> nil then
               begin
                  if GE.IsHitByXY(PC.X,PC.Y,0) then
                  begin
                     SumV := SumV + PC.Vymera;
                     KPK := PC.KATPAR_KOD;
                  end;
               end
               else
               begin
                  SumV := SumV + PC.Vymera;
                  KPK := PC.KATPAR_KOD;
               end;
            end;
            A := 'C'+SRow;
            ExcelWorkSheet.Range[A,A].Value := P.PARCISAbs;
            A := 'D'+SRow;
            if P.PARPOD > 0 then
               ExcelWorkSheet.Range[A,A].Value := P.PARPOD;
            A := 'E'+SRow;
            ExcelWorkSheet.Range[A,A].Value := SumV;
            Format := FormatFloat('0.0000',0);
            ExcelWorkSheet.Range[A,A].NumberFormatLocal := Format;
            if not Obce then
            begin
               A := 'I'+SRow;
               ExcelWorkSheet.Range[A,A].Value := KPK;
            end;
            LastRow := LastRow + 1;
            SRow := IntToStr(LastRow);
         end;
         E.Free;
         LastRow := LastRow + 2;
      end;
      exit;
   end;
   E := TOwnerEntity.Create;
   E.AddAllFrom := K.Parcely;
   Application.ProcessMessages;
   {
   for j := 0 to K.SkupinyParcel.LastIndex do
   begin
      SKP := Tgis_SkupinaParcel(K.SkupinyParcel.Elements[j]);
      if SKP.NoParcela then continue;
      E.AddAllFrom := SKP.Parcely;
   end;
   }
   ProgressText := SK +'sorting...';
   //Progress := 0;
   //MaxProgress := E.LastIndex;
   //Application.ProcessMessages;
   TT := TSortElementsBy2Integers.Create;
   TT.PropertyName := 'Parcis';
   TT.PropertyName1 := 'Parpod';
   TT.Input := E;
   TT.DoIt;
   TT.Free;
   //Hlavicka
   if HTML then
   begin
      TableRowStart;
      TableCell(' ');
      TableRowEnd;
      ProcessKatastrHeader_HTML(K);
      //exit;
   end
   else
      ProcessKatastrHeader(K);
   FI := LastRow;
   for i := 0 to E.LastIndex do
   begin
      El := E.Elements[i];
      if not (El is Tgis_Parcela) then continue;
      P := TGis_Parcela(El);
      //Progress := 0;
      ProgressText := SK + 'Parcela:'+P.Name;
      Application.ProcessMessages;
      if HTML then
         ProcessParcela_HTML(P)
      else
         ProcessParcela(P);
   end;
   if HTML then exit;
   LI := LastRow -1;
   E.Free;
   SRow := IntToStr(LastRow);
   A := 'B'+SRow;
   ExcelWorkSheet.Range[A,A].Value := 'SUMA: ';
   A := 'C'+SRow;
   ExcelWorkSheet.Range[A,A].Value := '=SUM(C'+IntToStr(FI)+':C'+IntToStr(LI)+')';
   ExcelWorkSheet.Range[A,A].Font.Bold := true;
   A := 'D'+SRow;
   ExcelWorkSheet.Range[A,A].Value := '=SUM(D'+IntToStr(FI)+':D'+IntToStr(LI)+')';
   ExcelWorkSheet.Range[A,A].Font.Bold := true;
   LastRow := LastRow + 1;
   LastRow := LastRow + 1;
end;

procedure TSoupisParcel.ProcessKatastrHeader(K: TGis_Katastr);
var
   A,B: OleVariant;
   SRow: string;
   S: string;
   BB: byte;
   List: TStringList;
   ParList: TStringList;
   j: integer;
begin
   List := TStringList.Create;
   List.CommaText := DynamicProps;
   ParList := TStringList.Create;
   ParList.CommaText := ParDynamicProps;
   SRow := IntToStr(LastRow);
   A := 'A'+SRow;
   ExcelWorkSheet.Range[A,A].Value := 'Katastr: '+K.Name+' '+K.Nazev;
   ExcelWorkSheet.Range[A,A].Font.Size := 10;
   ExcelWorkSheet.Range[A,A].Font.Bold := True;
   //ExcelWorkSheet.Range[A,A].ColumnWidth := 10;
   LastRow := LastRow+1;
   SRow := IntToStr(LastRow);
   A := 'A'+SRow;
   ExcelWorkSheet.Range[A,A].Value := 'Parcela .';
   A := 'B'+SRow;
   ExcelWorkSheet.Range[A,A].Value := 'Podlomen';
   A := 'C'+SRow;
   ExcelWorkSheet.Range[A,A].Value := 'Vmra';
   A := 'D'+SRow;
   ExcelWorkSheet.Range[A,A].Value := 'Plocha';
   if LHO then
   begin
      A := 'E'+SRow;
      ExcelWorkSheet.Range[A,A].Value := 'Jmno vlastnka';
      ExcelWorkSheet.Range[A,A].ColumnWidth := 16;
      A := 'F'+SRow;
      ExcelWorkSheet.Range[A,A].Value := 'slo vlast.';
      ExcelWorkSheet.Range[A,A].ColumnWidth := 10;
      A := 'G'+SRow;
      ExcelWorkSheet.Range[A,A].Value := 'List vlast.';
      A := 'H'+SRow;
      ExcelWorkSheet.Range[A,A].Value := 'Opert';
   end
   else
   begin
      A := 'E'+SRow;
      ExcelWorkSheet.Range[A,A].Value := 'Kvalita';
      //ExcelWorkSheet.Range[A,A].ColumnWidth := 16;
      A := 'F'+SRow;
      ExcelWorkSheet.Range[A,A].Value := 'Kategorie parcel';
      ExcelWorkSheet.Range[A,A].ColumnWidth := 16;
      A := 'G'+SRow;
      ExcelWorkSheet.Range[A,A].Value := 'Skupina parcel';
      A := 'H'+SRow;
      ExcelWorkSheet.Range[A,A].ColumnWidth := 1;
      //ExcelWorkSheet.Range[A,A].Value := 'Opert';
   end;
   A := 'I'+SRow;
   ExcelWorkSheet.Range[A,A].Value := 'Oznaen';
   ExcelWorkSheet.Range[A,A].ColumnWidth := 15;
   A := 'J'+SRow;
   ExcelWorkSheet.Range[A,A].Value := 'Poznmka';
   A := 'A'+SRow;
   B := 'J'+SRow;
   ExcelWorkSheet.Range[A,B].Font.Bold := True;
   ExcelWorkSheet.Range[A,B].HorizontalAlignment := xlHAlignCenter;
   LastRow := LastRow+2;
   BB := Ord('K');
   if List.Count > 0 then
   for j := 0 to List.Count - 1 do
   begin
      A := Char(BB)+SRow;
      S := List.Strings[j];
      ExcelWorkSheet.Range[A,A].Value := S;
      BB := BB + 1;
   end;
   if ParList.Count > 0 then
   for j := 0 to ParList.Count - 1 do
   begin
      A := Char(BB)+SRow;
      S := ParList.Strings[j];
      ExcelWorkSheet.Range[A,A].Value := S;
      BB := BB + 1;
   end;
   List.Free;
   ParList.Free;
end;

procedure TSoupisParcel.ProcessKatastrHeaderBySKP(K: TGis_Katastr);
var
   A,B: OleVariant;
   SRow: string;
   S: string;
   BB: byte;
   List: TStringList;
   ParList: TStringList;
   j: integer;
begin
   List := TStringList.Create;
   List.CommaText := DynamicProps;
   ParList := TStringList.Create;
   ParList.CommaText := ParDynamicProps;
   SRow := IntToStr(LastRow);
   A := 'A'+SRow;
   ExcelWorkSheet.Range[A,A].Value := 'Katastr: '+K.Name+' '+K.Nazev;
   ExcelWorkSheet.Range[A,A].Font.Size := 10;
   ExcelWorkSheet.Range[A,A].Font.Bold := True;
   //ExcelWorkSheet.Range[A,A].ColumnWidth := 10;
   LastRow := LastRow+1;
   SRow := IntToStr(LastRow);
   A := 'A'+SRow;
   ExcelWorkSheet.Range[A,A].Value := 'SK.PARC.';
   A := 'B'+SRow;
   if Obce then
      ExcelWorkSheet.Range[A,A].Value := 'KN/PK'
   else
      ExcelWorkSheet.Range[A,A].Value := 'ST';
   A := 'C'+SRow;
   ExcelWorkSheet.Range[A,A].Value := 'PARCELA';
   //ExcelWorkSheet.Range[A,A].Orientation := 90;
   A := 'D'+SRow;
   ExcelWorkSheet.Range[A,A].Value := 'PODL.';
   A := 'E'+SRow;
   ExcelWorkSheet.Range[A,A].Value := 'VMRA';
   //ExcelWorkSheet.Range[A,A].ColumnWidth := 16;
   A := 'F'+SRow;
   ExcelWorkSheet.Range[A,A].Value := 'ZAOK.VM.';
   ExcelWorkSheet.Range[A,A].ColumnWidth := 10;
   A := 'G'+SRow;
   if not Obce then
   begin
      ExcelWorkSheet.Range[A,A].Value := 'PLOCHA';
      A := 'H'+SRow;
      ExcelWorkSheet.Range[A,A].ColumnWidth := 1;
      //ExcelWorkSheet.Range[A,A].Value := 'Opert';
      A := 'I'+SRow;
      ExcelWorkSheet.Range[A,A].Value := 'KD LHP';
      //ExcelWorkSheet.Range[A,A].ColumnWidth := 15;
      A := 'J'+SRow;
   end;
   ExcelWorkSheet.Range[A,A].Value := 'JED.ROZ.LESA';
   ExcelWorkSheet.Range[A,A].ColumnWidth := 16;
   A := 'A'+SRow;
   B := 'J'+SRow;
   //ExcelWorkSheet.Range[A,B].Font.Bold := True;
   ExcelWorkSheet.Range[A,B].HorizontalAlignment := xlHAlignCenter;
   LastRow := LastRow+2;
   BB := Ord('K');
   if List.Count > 0 then
   for j := 0 to List.Count - 1 do
   begin
      A := Char(BB)+SRow;
      S := List.Strings[j];
      ExcelWorkSheet.Range[A,A].Value := S;
      BB := BB + 1;
   end;
   if ParList.Count > 0 then
   for j := 0 to ParList.Count - 1 do
   begin
      A := Char(BB)+SRow;
      S := ParList.Strings[j];
      ExcelWorkSheet.Range[A,A].Value := S;
      BB := BB + 1;
   end;
   List.Free;
   ParList.Free;
end;

procedure TSoupisParcel.ProcessKatastrHeader_HTML(K: TGis_Katastr);
var
   //A,B: OleVariant;
   //SRow: string;
   S: string;
   //BB: byte;
   List: TStringList;
   ParList: TStringList;
   j: integer;
begin
   List := TStringList.Create;
   List.CommaText := DynamicProps;
   ParList := TStringList.Create;
   ParList.CommaText := ParDynamicProps;
   TableRowStart;
   TableCellSpan(10,'<font size="4"><b> Katastr: '+K.Name+' '+K.Nazev+'</b></font>');
   TableRowEnd;
   TableRowStart;
   TableCell('<b>Parcela .</b>');
   TableCell('<b>Podlomen</b>');
   TableCell('<b>Vmra</b>');
   TableCell('<b>Plocha</b>');
   if LHO then
   begin
      TableCell('<b>Jmno vlastnka</b>');
      TableCell('<b>slo vlast.</b>');
      TableCell('<b>List vlast.</b>');
      TableCell('<b>Opert</b>');
   end
   else
   begin
      TableCell('<b>Kvalita</b>');
      TableCell('<b>Kategorie parcel</b>');
      TableCell('<b>Skupina parcel</b>');
      TableCell('<b>Opert</b>');
   end;
   TableCell('<b>Oznaen</b>');
   TableCell('<b>Poznmka</b>');
   if List.Count > 0 then
   for j := 0 to List.Count - 1 do
   begin
      S := List.Strings[j];
      TableCell('<b>'+S+'</b>');
   end;
   if ParList.Count > 0 then
   for j := 0 to ParList.Count - 1 do
   begin
      S := ParList.Strings[j];
      TableCell('<b>'+S+'</b>');
   end;
   List.Free;
   ParList.Free;
   TableRowEnd;
end;

procedure TSoupisParcel.ProcessKultury;
var
   i,j,ii: integer;
   K: Tgis_Katastr;
   P: Tgis_Parcela;
   PC: Tgis_ParcelaCast;
   SKP: Tgis_SkupinaParcel;
   e: TElement;
begin
   // find kultury 10
   for i := 0 to ParcelModel.LastIndex do
   begin
      e := ParcelModel.Elements[i];
      if not ( e is Tgis_Katastr) then continue;
      K := Tgis_Katastr(e);
      if not K.NoParcela then
      for j := 0 to K.Parcely.LastIndex do
      begin
         e := K.Parcely.Elements[j];
         if not (e is Tgis_Parcela)then continue;
         P := Tgis_Parcela(e);
         for ii := 0 to P.LastIndex do
         begin
            e := P.Elements[ii];
            if not (e is Tgis_ParcelaCast) then continue;
            PC := Tgis_ParcelaCast(e);
            SKP := PC.SkupinaParcel;
            if SKP = nil then continue;
            if FilterSKP then
            begin
               if not SKP.IsMyOwner(ValidSKP) then continue;
            end
            else
               if not PC.ParZar or (SKP.Cislo = 0) or not SKP.ForestConnected then continue;
            if PC.Kultura = 10 then
               Kultury10.AddElement(P)
            else
               Kultury11.AddElement(P);
            if PC.PUPFL then
               KulturyPUPFL.AddElement(P);
            break;
         end;
      end;
   end;
   if PoznamkaPropertyName = '' then
      PoznamkaPropertyName := 'Poznamka';
   if Druh_BzlPropertyName = '' then
      Druh_BzlPropertyName := 'DRUH_BZL';
   ProcessKultury_z_10;
   ProcessKultury_do_10;
   Kultury10.FreeAll;
   Kultury10.AddAllFrom := KulturyPUPFL;
   ProcessKultury_OP;
end;

procedure TSoupisParcel.ProcessKultury_do_10;
var
   A,B: OleVariant;
   SRow: string;
   //S: string;
   //BB: byte;
   //EWB: _WorkBook;
   EWS: _WorkSheet;
   OV: OleVariant;
begin
   {
   OV := true;
   EWB := ExcelApplication.Workbooks.Add(OV,0);
   ExcelWorkBook.ConnectTo(EWB);
   ExcelWorkSheet.ConnectTo(ExcelApplication.ActiveSheet as _WorkSheet);
   }
   //EWS := ExcelWorkBook.Sheets.Add(EmptyParam,EmptyParam,1,xlWorksheet,0) as _WorkSheet;
   try
      ExcelWorkBook.Sheets.Add(EmptyParam,EmptyParam,1,xlWorksheet,0);
   except
   end;
   EWS := ExcelWorkBook.Sheets[1] as _WorkSheet;
   ExcelWorkSheet.ConnectTo(EWS);
   //ExcelWorkSheet.Name := 'List18';

   ExcelWorkSheet.Name := 'Pevod do kultury 10';
   LastRow := 1;
   SRow := IntToStr(LastRow);
   A := 'B'+SRow;
   ExcelWorkSheet.Range[A,A].Value := 'Pevod do kultury 10';
   ExcelWorkSheet.Range[A,A].Font.Size := 10;
   ExcelWorkSheet.Range[A,A].Font.Bold := True;
   //ExcelWorkSheet.Range[A,A].ColumnWidth := 10;
   LastRow := LastRow+1;
   SRow := IntToStr(LastRow);
   ProcessHeaderForKultury;
   A := 'A'+SRow;
   A := 'A'+SRow;
   B := 'K'+SRow;
   ExcelWorkSheet.Range[A,B].Orientation := '90';
   //ExcelWorkSheet.Range[A,B].Font.Bold := True;
   ExcelWorkSheet.Range[A,B].HorizontalAlignment := xlHAlignCenter;
   LastRow := LastRow+1;
   ProcessK_do_10;
end;

procedure TSoupisParcel.ProcessKultury_OP;
var
   A,B: OleVariant;
   SRow: string;
   OV: OleVariant;
   //EWB: _WorkBook;
   EWS: _WorkSheet;
begin
   {
   OV := true;
   EWB := ExcelApplication.Workbooks.Add(OV,0);
   ExcelWorkBook.ConnectTo(EWB);
   ExcelWorkSheet.ConnectTo(ExcelApplication.ActiveSheet as _WorkSheet);
   }
   //EWS := ExcelWorkBook.Sheets.Add(EmptyParam,EmptyParam,1,xlWorksheet,0) as _WorkSheet;
   try
      ExcelWorkBook.Sheets.Add(EmptyParam,EmptyParam,1,xlWorksheet,0);
   except
   end;
   EWS := ExcelWorkBook.Sheets[1] as _WorkSheet;
   ExcelWorkSheet.ConnectTo(EWS);
   LastRow := 1;
   ExcelWorkSheet.Name := 'Ostatn pozemky';
   SRow := IntToStr(LastRow);
   A := 'B'+SRow;
   ExcelWorkSheet.Range[A,A].Value := 'Nesoulad PUPFL - ostatn pozemky';
   ExcelWorkSheet.Range[A,A].Font.Size := 10;
   ExcelWorkSheet.Range[A,A].Font.Bold := True;
   //ExcelWorkSheet.Range[A,A].ColumnWidth := 10;
   LastRow := LastRow+1;
   SRow := IntToStr(LastRow);
   ProcessHeaderForKultury;
   A := 'J'+SRow;
   ExcelWorkSheet.Range[A,A].Value := 'OSTATN POZEMEK';
   A := 'K'+SRow;
   ExcelWorkSheet.Range[A,A].Value := 'POZNMKA';
   A := 'A'+SRow;
   B := 'K'+SRow;
   ExcelWorkSheet.Range[A,B].Orientation := '90';
   //ExcelWorkSheet.Range[A,B].Font.Bold := True;
   ExcelWorkSheet.Range[A,B].HorizontalAlignment := xlHAlignCenter;
   //ExcelWorkSheet.Range[A,B].Columns.AutoFit;
   LastRow := LastRow+1;
   ProcessK_z_10(900);
end;

procedure TSoupisParcel.ProcessKultury_z_10;
var
   A,B: OleVariant;
   SRow: string;
begin
   ExcelWorkSheet.Name := 'Pevod z kultury 10';
   SRow := IntToStr(LastRow);
   A := 'B'+SRow;
   ExcelWorkSheet.Range[A,A].Value := 'Pevod z kultury 10';
   ExcelWorkSheet.Range[A,A].Font.Size := 10;
   ExcelWorkSheet.Range[A,A].Font.Bold := True;
   //ExcelWorkSheet.Range[A,A].ColumnWidth := 10;
   LastRow := LastRow+1;
   SRow := IntToStr(LastRow);
   ProcessHeaderForKultury;// hlava
   A := 'J'+SRow;
   ExcelWorkSheet.Range[A,A].Value := 'JIN POZEMEK';
   A := 'K'+SRow;
   ExcelWorkSheet.Range[A,A].Value := 'POZNMKA';
   A := 'A'+SRow;
   B := 'K'+SRow;
   ExcelWorkSheet.Range[A,B].Orientation := '90';
   //ExcelWorkSheet.Range[A,B].Font.Bold := True;
   ExcelWorkSheet.Range[A,B].HorizontalAlignment := xlHAlignCenter;
   //ExcelWorkSheet.Range[A,B].Columns.AutoFit;
   LastRow := LastRow+1;
   ProcessK_z_10(500);     //telo
end;

procedure TSoupisParcel.ProcessK_do_10;
var
   A,B: OleVariant;
   SRow: string;
   S: string;
   i,j,ii: integer;
   SL: TStringList;
   P: Tgis_Parcela;
   En: TEntity;
   Prl: TEntity;
   MA: TMultiAreaElement;
   Cast: boolean;
   resultVym: double;
   UseVym: boolean;
   SE: TStringElement;
   NKultury: integer;
begin
   SRow := IntToStr(LastRow);
   Progress := 0;
   MaxProgress := Kultury11.LastIndex;
   for i := 0 to Kultury11.LastIndex do
   begin
      P := Tgis_Parcela(Kultury11.Elements[i]);
      ProgressText := 'Les do kultury 10 - parcela '+P.Name;
      Progress := i;
      Application.ProcessMessages;
      EN := FindForestUnitsForKultury(P,0,499,false);
      if EN.NoElements then
      begin
         EN.Free;
         Continue;
      end;
      MA := TMultiAreaElement(EN.FindElementName('MARE'));
      Cast := Abs(P.Plocha - MA.Area)/10000 > Tolerance;
      UseVym := not Cast and (P.Vymera > 0);
      if UseVym then
         ResultVym := P.Vymera
      else
      begin
         ResultVym := MA.Area/10000;
         ResultVym := Simple_RoundTo(ResultVym,-4);
      end;
      PRL := TEntity(EN.FindElementName('PRL'));
      A := 'A'+SRow;
      ExcelWorkSheet.Range[A,A].Value := P.KU_KOD;
      A := 'B'+SRow;
      ExcelWorkSheet.Range[A,A].Value := P.KU_Nazev;
      A := 'C'+SRow;
      ExcelWorkSheet.Range[A,A].Value := P.PARCIS;
      //ExcelWorkSheet.Range[A,A].Orientation := 90;
      A := 'D'+SRow;
      if P.PARPOD > 0 then
         ExcelWorkSheet.Range[A,A].Value := P.PARPOD;
      A := 'E'+SRow;
      if Cast then
         ExcelWorkSheet.Range[A,A].Value := '.';
      ExcelWorkSheet.Range[A,A].HorizontalAlignment := xlHAlignCenter;
      //ExcelWorkSheet.Range[A,A].ColumnWidth := 16;
      A := 'F'+SRow;
      //ExcelWorkSheet.Range[A,A].Value := FormatFloat('0.0000', ResultVym);
      ExcelWorkSheet.Range[A,A].Value := ResultVym;
      Format := FormatFloat('0.0000',0);
      ExcelWorkSheet.Range[A,A].NumberFormatLocal := Format;
      if UseVym then
         ExcelWorkSheet.Range[A,A].Font.Bold := true;
      //ExcelWorkSheet.Range[A,A].ColumnWidth := 10;
      A := 'G'+SRow;
      ExcelWorkSheet.Range[A,A].Value := P.Kultura;
      ExcelWorkSheet.Range[A,A].HorizontalAlignment := xlHAlignCenter;
      A := 'H'+SRow;
      ExcelWorkSheet.Range[A,A].Value := 10;//'NVRH KULTURY';
      ExcelWorkSheet.Range[A,A].HorizontalAlignment := xlHAlignCenter;
      SL := GetPRLStringKultury(PRL);
      if SL.Count > 0 then
      for ii := 0 to SL.Count-1 do
      begin
         A := 'I'+SRow;
         ExcelWorkSheet.Range[A,A].Value := SL.Strings[ii];//'JED.ROZ.LESA';
         ExcelWorkSheet.Range[A,A].HorizontalAlignment := xlHAlignCenter;
         LastRow := LastRow+1;
         SRow := IntToStr(LastRow);
      end;
      SL.Free;
      EN.Free;
   end;
   A := 'A2';
   B := 'I'+SRow;
   ExcelWorkSheet.Range[A,B].Columns.AutoFit;
end;

procedure TSoupisParcel.ProcessK_z_10(PSKNumber: integer);
var
   A,B: OleVariant;
   SRow: string;
   S: string;
   i,j,ii: integer;
   SL: TStringList;
   P: Tgis_Parcela;
   En: TEntity;
   SortI: TSortElementsByInteger;
   SK: TEntity;
   Prl: TEntity;
   MA: TMultiAreaElement;
   Cast: boolean;
   resultVym: double;
   UseVym: boolean;
   SE: TStringElement;
   NKultury: integer;
   PNS: string;
   MaxPSK: integer;
begin
   SRow := IntToStr(LastRow);
   SortI := TSortElementsByInteger.Create;
   Progress := 0;
   MaxProgress := Kultury10.LastIndex;
   PNS := IntToStr(PSKNumber);
   if PSKNumber = 500 then
      MaxPSK := 899
   else
      MaxPSK := 1000;
   for i := 0 to Kultury10.LastIndex do
   begin
      P := Tgis_Parcela(Kultury10.Elements[i]);
      ProgressText := PNS+' z kultury 10 - parcela '+P.Name;
      Progress := i;
      Application.ProcessMessages;
      EN := FindForestUnitsForKultury(P,PSKNumber,MaxPSK,true);
      if EN.NoElements then
      begin
         EN.Free;
         Continue;
      end;
      Sorti.PropertyName := 'Name';
      Sorti.Input := EN;
      Sorti.DoIt;
      for j := 0 to EN.LastIndex do
      begin
         SK := TEntity(En.Elements[j]);
         MA := TMultiAreaElement(SK.FindElementName('MARE'));
         Cast := Abs(P.Plocha - MA.Area)/10000 > Tolerance;
         UseVym := not Cast and (P.Vymera > 0);
         if UseVym then
            ResultVym := P.Vymera
         else
         begin
            ResultVym := MA.Area/10000;
            ResultVym := Simple_RoundTo(ResultVym,-4);
         end;
         PRL := TEntity(SK.FindElementName('PRL'));
         A := 'A'+SRow;
         ExcelWorkSheet.Range[A,A].Value := P.KU_KOD;
         A := 'B'+SRow;
         ExcelWorkSheet.Range[A,A].Value := P.KU_Nazev;
         A := 'C'+SRow;
         ExcelWorkSheet.Range[A,A].Value := P.PARCIS;
         //ExcelWorkSheet.Range[A,A].Orientation := 90;
         A := 'D'+SRow;
         if P.PARPOD > 0 then
            ExcelWorkSheet.Range[A,A].Value := P.PARPOD;
         A := 'E'+SRow;
         if Cast then
            ExcelWorkSheet.Range[A,A].Value := '.';
         ExcelWorkSheet.Range[A,A].HorizontalAlignment := xlHAlignCenter;
         //ExcelWorkSheet.Range[A,A].ColumnWidth := 16;
         A := 'F'+SRow;
         ExcelWorkSheet.Range[A,A].Value := ResultVym;
         Format := FormatFloat('0.0000',0);
         ExcelWorkSheet.Range[A,A].NumberFormatLocal := Format;
         //ExcelWorkSheet.Range[A,A].Value := FormatFloat('00.0000', ResultVym);
         if UseVym then
            ExcelWorkSheet.Range[A,A].Font.Bold := true;
         //ExcelWorkSheet.Range[A,A].ColumnWidth := 10;
         A := 'G'+SRow;
         ExcelWorkSheet.Range[A,A].Value := P.Kultura;
         ExcelWorkSheet.Range[A,A].HorizontalAlignment := xlHAlignCenter;
         A := 'H'+SRow;
         SE := TStringElement(SK.FindElementName(Druh_BzlPropertyName));
         NKultury := 14;
         if SE <> nil then
         begin
            S := SE.Value;
            if SameText(S,'VP') then
               NKultury := 11
            else
            if SameText(S,'ST') then
               NKultury := 13;
         end;
         ExcelWorkSheet.Range[A,A].Value := NKultury;//'NVRH KULTURY';
         ExcelWorkSheet.Range[A,A].HorizontalAlignment := xlHAlignCenter;
         A := 'J'+SRow;
         ExcelWorkSheet.Range[A,A].Value := SK.Name;
         A := 'K'+SRow;
         SE := TStringElement(SK.FindElementName(PoznamkaPropertyName));
         if SE <> nil then
            ExcelWorkSheet.Range[A,A].Value := SE.Value;
         SL := GetPRLStringKultury(PRL);
         if SL.Count > 0 then
         for ii := 0 to SL.Count-1 do
         begin
            A := 'I'+SRow;
            ExcelWorkSheet.Range[A,A].Value := SL.Strings[ii];//'JED.ROZ.LESA';
            ExcelWorkSheet.Range[A,A].HorizontalAlignment := xlHAlignCenter;
            LastRow := LastRow+1;
            SRow := IntToStr(LastRow);
         end;
         SL.Free;
      end;
      EN.Free;
   end;
   SortI.Free;
   A := 'A2';
   B := 'K'+SRow;
   ExcelWorkSheet.Range[A,B].Columns.AutoFit;
end;

procedure TSoupisParcel.ProcessOddeleni(O: Tgis_Oddeleni);
var
   S: string;
   i: integer;
   //TT: TSortElementsBy2Integers;
   D: Tgis_DIlec;
   FI,LI: integer;
   SRow: string;
   A: OleVariant;
begin
   if O.NoItems then exit;
   S := 'Oddlen: '+ O.Name+'  ';
   ProgressText := S;
   //Hlavicka
   //ProcessKatastrHeader(K);
   if O.NoItems then exit;
   for i := 0 to O.Items.LastIndex do
   begin
      D := Tgis_Dilec(O.Items.ForestUnit[i]);
      ProcessDIlec(D);
   end;
   {
   FI := LastRow;
   SRow := IntToStr(LastRow);
   A := 'B'+SRow;
   ExcelWorkSheet.Range[A,A].Value := O.Name;

   LastRow := LastRow + 1;
   }
end;

procedure TSoupisParcel.ProcessOddsHeader;
var
   j: integer;
   A,B: OleVariant;
   SRow: string;
   S: string;
   BB: byte;
   List: TStringList;
   ParList: TStringList;
begin
   List := TStringList.Create;
   List.CommaText := DynamicProps;
   ParList := TStringList.Create;
   ParList.CommaText := ParDynamicProps;
   SRow := IntToStr(LastRow);
   A := 'A'+SRow;
   ExcelWorkSheet.Range[A,A].Value := 'ODDLEN';
   A := 'B'+SRow;
   ExcelWorkSheet.Range[A,A].Value := 'DLEC';
   A := 'C'+SRow;
   ExcelWorkSheet.Range[A,A].Value := 'POROST';
   if ByPSK then
   begin
      A := 'D'+SRow;
      ExcelWorkSheet.Range[A,A].Value := 'SKUPINA';
   end;
   A := 'E'+SRow;
   ExcelWorkSheet.Range[A,A].Value := 'KATASTR';
   A := 'F'+SRow;
   ExcelWorkSheet.Range[A,A].Value := 'NAZEV KU';
   A := 'G'+SRow;
   ExcelWorkSheet.Range[A,A].Value := 'PARCIS';
   A := 'H'+SRow;
   ExcelWorkSheet.Range[A,A].Value := 'PARPOD';
   A := 'I'+SRow;
   ExcelWorkSheet.Range[A,A].Value := 'PARVYM';
   A := 'J'+SRow;
   ExcelWorkSheet.Range[A,A].Value := 'PLOCHA';
   A := 'K'+SRow;
   ExcelWorkSheet.Range[A,A].Value := 'Cela';
   LastRow := LastRow + 1;
   BB := Ord('L');
   if List.Count > 0 then
   for j := 0 to List.Count - 1 do
   begin
      A := Char(BB)+SRow;
      S := List.Strings[j];
      ExcelWorkSheet.Range[A,A].Value := S;
      BB := BB + 1;
   end;
   if ParList.Count > 0 then
   for j := 0 to ParList.Count - 1 do
   begin
      A := Char(BB)+SRow;
      S := ParList.Strings[j];
      ExcelWorkSheet.Range[A,A].Value := S;
      BB := BB + 1;
   end;
   List.Free;
   ParList.Free;
end;

procedure TSoupisParcel.processParcela(P: TGis_Parcela);
var
   A,B: OleVariant;
   SRow: string;
   En: TEntity;
   i,j: integer;
   S: string;
   E: TElement;
   PSK: Tgis_PorostniSkupina;
   Dyn: TDynamicProperties;
   BB: byte;
   List: TStringList;
   ParList: TStringList;
   Sort: TSortElementsByInteger;
   PC: Tgis_ParcelaCast;
   SKP: Tgis_SkupinaParcel;
   CastP: boolean;
   DifP: double;
begin
   PC := Tgis_ParcelaCast(P.FindElementClass(Tgis_ParcelaCast));
   if PC = nil then exit;
   SKP := P.SkupinaParcel;
   List := TStringList.Create;
   List.CommaText := DynamicProps;
   ParList := TStringList.Create;
   ParList.CommaText := ParDynamicProps;
   SRow := IntToStr(LastRow);
   A := 'A'+SRow;
   ExcelWorkSheet.Range[A,A].Value := P.PARCIS;
   A := 'B'+SRow;
   ExcelWorkSheet.Range[A,A].Value := P.PARPOD;
   A := 'C'+SRow;
   ExcelWorkSheet.Range[A,A].Value := P.Vymera;
   A := 'D'+SRow;
   ExcelWorkSheet.Range[A,A].Value := P.Plocha_ha;
   if LHO then
   begin
      A := 'E'+SRow;
      if VlastnikPropertyName <> '' then
         ExcelWorkSheet.Range[A,A].Value := PC.GetPropertyStringByName(VlastnikPropertyName);
      A := 'F'+SRow;
      if VlastnikCisloPropertyName <> '' then
         ExcelWorkSheet.Range[A,A].Value := PC.GetPropertyLongIntByName(VlastnikCisloPropertyName);
      A := 'G'+SRow;
      ExcelWorkSheet.Range[A,A].Value := P.LV;
      A := 'H'+SRow;
      ExcelWorkSheet.Range[A,A].Value := ' ';
   end
   else
   begin
      A := 'E'+SRow;
      if SKP <> nil then
         ExcelWorkSheet.Range[A,A].Value := SKP.KVAL_P;
      A := 'F'+SRow;
      ExcelWorkSheet.Range[A,A].Value := PC.KATPAR_KOD;
      A := 'G'+SRow;
      ExcelWorkSheet.Range[A,A].Value := PC.SKUPAR_CIS;
      A := 'H'+SRow;
      ExcelWorkSheet.Range[A,A].Value := ' ';
   end;
   A := 'I'+SRow;
   EN := FindForestUnits(P,PSK,nil,CastP,DifP);
   S := '';
   Sort := TSortElementsByInteger.Create;
   Sort.PropertyName := 'Oddeleni';
   //Sort.PropertyName1 := 'Parpod';
   Sort.Input := En;
   Sort.DoIt;
   Sort.Free;
   for i := 0 to EN.LastIndex do
   begin
      if i > 0 then
         S := S+',';
      e := EN.Elements[i];
      S := S + e.GetPropertyStringByName('Oddeleni')+' '+e.GetPropertyStringByName('Dilec');
      if LHO then
         S := S+' '+ e.GetPropertyStringByName('Porost');
   end;
   S := '="'+S+'"';
   EN.Free;
   ExcelWorkSheet.Range[A,A].Value := S;
   {
   A := 'J'+SRow;
   ExcelWorkSheet.Range[A,A].Value := 'Poznmka';
   }
   BB := Ord('K');
   if (PSK <> nil) and (List.Count > 0)then
   begin
      //Dyn := TDynamicProperties(FU.FindElementClass(TDynamicProperties));
      //if Dyn <> nil then
      for j := 0 to List.Count - 1 do
      begin
         A := Char(BB)+SRow;
         S := List.Strings[j];
         //S := Dyn.GetPropertyStringByName(S);
         if (UpperCase(S) = 'KVAL_P') and (SKP <> nil) then
            S := SKP.GetPropertyStringByName(S)
         else
            S := PSK.GetPropertyStringByName(S);
         if cp_Is852(S) then
            S := cp852ToWin(S);
         ExcelWorkSheet.Range[A,A].Value := S;
         BB := BB + 1;
      end;
   end;
   if ParList.Count > 0 then
   for j := 0 to ParList.Count - 1 do
   begin
      A := Char(BB)+SRow;
      S := ParList.Strings[j];
      if (UpperCase(S) = 'KVAL_P') and (SKP <> nil) then
         S := SKP.GetPropertyStringByName(S)
      else
         S := PC.GetPropertyStringByName(S);
      ExcelWorkSheet.Range[A,A].Value := S;
      BB := BB + 1;
   end;
   A := 'A'+SRow;
   B := 'H'+SRow;
   ExcelWorkSheet.Range[A,B].HorizontalAlignment := xlHAlignCenter;
   LastRow := LastRow+1;
end;

function TSoupisParcel.processParcelaBySKP(P: TGis_Parcela; SKP: Tgis_SkupinaParcel; EPRL,ListPRL: TEntity; var SumVym: double): boolean;
var
   A,B: OleVariant;
   SRow: string;
   En,EnBZL,EOdd,EDil: TEntity;
   i,j,k: integer;
   S: string;
   E: TElement;
   PSK: Tgis_PorostniSkupina;
   Dyn: TDynamicProperties;
   BB: byte;
   List: TStringList;
   ParList: TStringList;
   Sort: TSortElementsByInteger;
   SortS: TSortElementsByString;
   PC: Tgis_ParcelaCast;
   _PVymera,DifP: double;
   Cast: boolean;
   SelN: TSelectPorostniSkupinaNumber;
begin
   result := true;
   _PVymera := 0;
   Cast := false;
   for i := 0 to P.LastIndex do
   begin
      e := P.Elements[i];
      if not (e is Tgis_ParcelaCast) then continue;
      PC := Tgis_parcelaCast(e);
      if PC.SkupinaParcel <> SKP then
      begin
         Cast := true;
         continue;
      end;
      _PVymera := _PVymera + PC.Vymera;
   end;
   PC := Tgis_ParcelaCast(P.FindElementClass(Tgis_ParcelaCast));
   if PC = nil then exit;

   EN := FindForestUnits(P,PSK,SKP,Cast,DifP);
   if (EN = nil) or EN.NoElements then
   begin
      result := false;
      exit;
   end;
   if Cast then  // prepocet vymery na pomernou cast
   begin
      _PVymera := Simple_RoundTo((P.MultiVymera / P.Plocha) *(P.Plocha - DifP),-4);
   end;
   SumVym := SumVym + _PVymera;
   {
   List := TStringList.Create;
   List.CommaText := DynamicProps;
   ParList := TStringList.Create;
   ParList.CommaText := ParDynamicProps;
   }
   SRow := IntToStr(LastRow);
   if not Obce then
   begin
      A := 'A'+SRow;
      ExcelWorkSheet.Range[A,A].Value := SKP.Cislo;
   end;
   A := 'B'+SRow;
   if Obce then
   begin
      if P.PK then
         S := 'PK'
      else
         S := 'KN'
   end
   else
   begin
      S := '';
      if P.Multiparcela then
         S :='.';
      if Cast then
         S := '.';
   end;
   ExcelWorkSheet.Range[A,A].Value := S;
   A := 'C'+SRow;
   ExcelWorkSheet.Range[A,A].Value := P.PARCISAbs;
   A := 'D'+SRow;
   if P.PARPOD > 0 then
      ExcelWorkSheet.Range[A,A].Value := P.PARPOD;
   A := 'E'+SRow;
   ExcelWorkSheet.Range[A,A].Value := _PVymera;
   Format := FormatFloat('0.0000',0);
   ExcelWorkSheet.Range[A,A].NumberFormatLocal := Format;
   //A := 'F'+SRow;
   //ExcelWorkSheet.Range[A,A].Value := PC.KATPAR_KOD;
   //A := 'G'+SRow;
   //ExcelWorkSheet.Range[A,A].Value := PC.SKUPAR_CIS;
   //A := 'H'+SRow;
   //ExcelWorkSheet.Range[A,A].Value := ' ';
   A := 'I'+SRow;
   if not Obce then
   begin
      ExcelWorkSheet.Range[A,A].Value := SKP.KATPAR_KOD;
      //!!!!!!!!!!!!
      //exit;


      if Cast then
      begin
         A := 'B'+SRow;
         ExcelWorkSheet.Range[A,A].Value := 'p.';
         if Test then
         begin
            A := 'G'+SRow;
            ExcelWorkSheet.Range[A,A].Value := Simple_RoundTo(DifP,-2);
            Format := FormatFloat('0.0000',0);
            ExcelWorkSheet.Range[A,A].NumberFormatLocal := Format;
         end;
      end;
   end;
   if not PRLBySKP then
      EPRL.FreeAll;
   for i := 0 to EN.LastIndex do
   begin
      e := EN.Elements[i];
      if not e.IsMyOwner(ListPRL) then
         ListPRL.AddElement(e);
      EOdd := TEntity(EPRL.FindOrCreateElementName(TOwnerEntity,e.GetPropertyStringByName('Oddeleni')));
      if not PRLPorost then
         EOdd.FindOrCreateElementName(TOwnerEntity,e.GetPropertyStringByName('Dilec'))
      else
         EOdd.FindOrCreateElementName(TOwnerEntity,e.GetPropertyStringByName('Porost'))
   end;
   SelN := TSelectPorostniSkupinaNumber.Create;
   SelN.Min := 100;
   SelN.Max := 1000;
   SelN.Input := EN;
   SelN.DoIt;
   EnBZL := TEntity(SelN.Output);
   for i := 0 to EnBZL.LastIndex do
   begin
      e := EnBZL.Elements[i];
      EOdd := TEntity(EPRL.FindOrCreateElementName(TOwnerEntity,e.GetPropertyStringByName('Oddeleni')));
      if not PRLPorost then
         EDil := TEntity(EOdd.FindOrCreateElementName(TOwnerEntity,e.GetPropertyStringByName('Dilec')))
      else
         EDil := TEntity(EOdd.FindOrCreateElementName(TOwnerEntity,e.GetPropertyStringByName('Porost')));
      EDil.FindOrCreateElementName(TOwnerEntity,e.Name);
   end;
   EnBZL.Free;
   SelN.Free;
   S := '';
   Sort := TSortElementsByInteger.Create;
   Sort.PropertyName := 'Name';
   //Sort.PropertyName1 := 'Parpod';
   Sort.Input := EPRL;
   Sort.DoIt;
   Sort.Free;
   for i := 0 to EPRL.LastIndex do
   begin
      e := EPRL.Elements[i];
      SortS := TSortElementsByString.Create;
      SortS.PropertyName := 'Name';
      //Sort.PropertyName1 := 'Parpod';
      SortS.Input := e;
      SortS.DoIt;
      SortS.Free;
      EOdd := TEntity(e);
      if not EOdd.NoElements then
      for j := 0 to EOdd.LastIndex do
      begin
         e := EOdd.Elements[j];
         Sort := TSortElementsByInteger.Create;
         Sort.PropertyName := 'Name';
         //Sort.PropertyName1 := 'Parpod';
         Sort.Input := e;
         Sort.DoIt;
         Sort.Free;
      end;
   end;
   if not PRLBySKP then
   begin
      for i := 0 to EPRL.LastIndex do
      begin
         if i > 0 then
         begin
            if i mod 5 = 0 then
            begin
               S := '="'+S+'"';
               A := 'J'+SRow;
               ExcelWorkSheet.Range[A,A].Value := S;
               LastRow := LastRow+1;
               SRow := IntToStr(LastRow);
               S := '';
            end
            else
               S := S+',';
         end;
         e := EPRL.Elements[i];
         EOdd := TEntity(e);
         S := S +EOdd.Name+' ';
         for j := 0 to EOdd.LastIndex do
         begin
            e := EOdd.Elements[j];
            S := S + e.Name;
            EDil := TEntity(e);
            if not EDil.NoElements then
            begin
               S := S +'(';
               for k := 0 to EDil.LastIndex do
               begin
                  if k > 0 then
                     S := S + ' ';
                  S := S + EDil.Elements[k].Name;
               end;
               S := S +')';
            end;
         end;
      end;
      if S <> '' then
      begin
         S := '="'+S+'"';
         A := 'J'+SRow;
         ExcelWorkSheet.Range[A,A].Value := S;
         LastRow := LastRow+1;
      end;
   end
   else
      LastRow := LastRow+1;
   EN.Free;
   exit;
   (*
   {
   A := 'J'+SRow;
   ExcelWorkSheet.Range[A,A].Value := 'Poznmka';
   }
   BB := Ord('K');
   if (PSK <> nil) and (List.Count > 0)then
   begin
      //Dyn := TDynamicProperties(FU.FindElementClass(TDynamicProperties));
      //if Dyn <> nil then
      for j := 0 to List.Count - 1 do
      begin
         A := Char(BB)+SRow;
         S := List.Strings[j];
         //S := Dyn.GetPropertyStringByName(S);
         if (UpperCase(S) = 'KVAL_P') and (SKP <> nil) then
            S := SKP.GetPropertyStringByName(S)
         else
            S := PSK.GetPropertyStringByName(S);
         if cp_Is852(S) then
            S := cp852ToWin(S);
         ExcelWorkSheet.Range[A,A].Value := S;
         BB := BB + 1;
      end;
   end;
   if ParList.Count > 0 then
   for j := 0 to ParList.Count - 1 do
   begin
      A := Char(BB)+SRow;
      S := ParList.Strings[j];
      if (UpperCase(S) = 'KVAL_P') and (SKP <> nil) then
         S := SKP.GetPropertyStringByName(S)
      else
         S := PC.GetPropertyStringByName(S);
      ExcelWorkSheet.Range[A,A].Value := S;
      BB := BB + 1;
   end;
   A := 'A'+SRow;
   B := 'H'+SRow;
   ExcelWorkSheet.Range[A,B].HorizontalAlignment := xlHAlignCenter;
   LastRow := LastRow+1;
   *)
end;

procedure TSoupisParcel.processParcela_HTML(P: TGis_Parcela);
var
   En: TEntity;
   i,j: integer;
   S: string;
   E: TElement;
   PSK: Tgis_PorostniSkupina;
   Dyn: TDynamicProperties;
   BB: byte;
   List: TStringList;
   ParList: TStringList;
   Sort: TSortElementsByInteger;
   PC: Tgis_ParcelaCast;
   SKP: Tgis_SkupinaParcel;
begin
   PC := Tgis_ParcelaCast(P.FindElementClass(Tgis_ParcelaCast));
   if PC = nil then exit;
   SKP := P.SkupinaParcel;
   List := TStringList.Create;
   List.CommaText := DynamicProps;
   ParList := TStringList.Create;
   ParList.CommaText := ParDynamicProps;
   TableRowStart;
   TableCell(IntToStr(P.PARCIS));
   TableCell(IntToStr(P.PARPOD));
   TableCell(FloatToStr(P.Vymera));
   TableCell(FloatToStr(Simple_RoundTo(P.Plocha_ha,-4)));
   exit;
   (*
   if LHO then
   begin
      A := 'E'+SRow;
      if VlastnikPropertyName <> '' then
         ExcelWorkSheet.Range[A,A].Value := PC.GetPropertyStringByName(VlastnikPropertyName);
      A := 'F'+SRow;
      if VlastnikCisloPropertyName <> '' then
         ExcelWorkSheet.Range[A,A].Value := PC.GetPropertyLongIntByName(VlastnikCisloPropertyName);
      A := 'G'+SRow;
      ExcelWorkSheet.Range[A,A].Value := P.LV;
      A := 'H'+SRow;
      ExcelWorkSheet.Range[A,A].Value := ' ';
   end
   else
   begin
      A := 'E'+SRow;
      if SKP <> nil then
         ExcelWorkSheet.Range[A,A].Value := SKP.KVAL_P;
      A := 'F'+SRow;
      ExcelWorkSheet.Range[A,A].Value := PC.KATPAR_KOD;
      A := 'G'+SRow;
      ExcelWorkSheet.Range[A,A].Value := PC.SKUPAR_CIS;
      A := 'H'+SRow;
      ExcelWorkSheet.Range[A,A].Value := ' ';
   end;
   A := 'I'+SRow;
   EN := FindForestUnits(P,PSK);
   S := '';
   Sort := TSortElementsByInteger.Create;
   Sort.PropertyName := 'Oddeleni';
   //Sort.PropertyName1 := 'Parpod';
   Sort.Input := En;
   Sort.DoIt;
   Sort.Free;
   for i := 0 to EN.LastIndex do
   begin
      if i > 0 then
         S := S+',';
      e := EN.Elements[i];
      S := S + e.GetPropertyStringByName('Oddeleni')+' '+e.GetPropertyStringByName('Dilec');
      if LHO then
         S := S+' '+ e.GetPropertyStringByName('Porost');
   end;
   S := '="'+S+'"';
   EN.Free;
   ExcelWorkSheet.Range[A,A].Value := S;
   {
   A := 'J'+SRow;
   ExcelWorkSheet.Range[A,A].Value := 'Poznmka';
   }
   BB := Ord('K');
   if (PSK <> nil) and (List.Count > 0)then
   begin
      //Dyn := TDynamicProperties(FU.FindElementClass(TDynamicProperties));
      //if Dyn <> nil then
      for j := 0 to List.Count - 1 do
      begin
         A := Char(BB)+SRow;
         S := List.Strings[j];
         //S := Dyn.GetPropertyStringByName(S);
         if (UpperCase(S) = 'KVAL_P') and (SKP <> nil) then
            S := SKP.GetPropertyStringByName(S)
         else
            S := PSK.GetPropertyStringByName(S);
         if cp_Is852(S) then
            S := cp852ToWin(S);
         ExcelWorkSheet.Range[A,A].Value := S;
         BB := BB + 1;
      end;
   end;
   if ParList.Count > 0 then
   for j := 0 to ParList.Count - 1 do
   begin
      A := Char(BB)+SRow;
      S := ParList.Strings[j];
      if (UpperCase(S) = 'KVAL_P') and (SKP <> nil) then
         S := SKP.GetPropertyStringByName(S)
      else
         S := PC.GetPropertyStringByName(S);
      ExcelWorkSheet.Range[A,A].Value := S;
      BB := BB + 1;
   end;
   A := 'A'+SRow;
   B := 'H'+SRow;
   ExcelWorkSheet.Range[A,B].HorizontalAlignment := xlHAlignCenter;
   LastRow := LastRow+1;
   *)
   TableRowEnd;
end;

procedure TSoupisParcel.ProcessPorost(P: Tgis_Porost);
var
   Enn: TEntity;
   i,j: integer;
   FU,FUU: Tgis_ForestUnit;
   Par: Tgis_Parcela;
   ParC: Tgis_ParcelaCast;
   K: Tgis_Katastr;
   PSK: Tgis_PorostniSkupina;
   A,B: OleVariant;
   SRow: string;
   S: string;
   E: TElement;
   Are: TAreaElement;
   Dyn: TDynamicProperties;
   BB: byte;
   List: TStringList;
   ParList: TStringList;
   Sort: TSortElementsBy2Integers;

procedure SolveParcely(En: TEntity);
var
   i,j: integer;
   Ri,Ci: OleVariant;
   aValue: OleVariant;
begin
   Sort := TSortElementsBy2Integers.Create;
   Sort.PropertyName := 'Parcis';
   Sort.PropertyName1 := 'Parpod';
   Sort.Input := En;
   Sort.DoIt;
   Sort.Free;
   for i := 0 to En.LastIndex do
   begin
      FU := Tgis_ForestUnit(En.Elements[i]);
      Are := TAreaElement(FU.Geometry);
      if (Tolerance > 0) AND (Are.Area/10000 < Tolerance) then
         continue;
      Par := Tgis_Parcela(FU.Properties);
      ParC := Tgis_ParcelaCast(FU.FindElementClass(Tgis_ParcelaCast));
      K := Par.Katastr;
      SRow := IntToStr(LastRow);
      A := 'A'+SRow;
      ExcelWorkSheet.Range[A,A].Value := P.Oddeleni;
      A := 'B'+SRow;
      ExcelWorkSheet.Range[A,A].Value := P.Dilec;
      A := 'C'+SRow;
      ExcelWorkSheet.Range[A,A].Value := P.Name;
      if ByPSK then
      begin
         A := 'D'+SRow;
         ExcelWorkSheet.Range[A,A].Value := ''''+FUU.Name;
         //ExcelWorkSheet.Range[A,A].Value := ''''+FUU.Name;
      end;
      A := 'E'+SRow;
      ExcelWorkSheet.Range[A,A].Value := Par.KU_KOD;
      A := 'F'+SRow;
      if K <> nil then
      begin
         S := K.Nazev;
         if cp_Is852(S) then
            S := cp852ToWin(S);
         ExcelWorkSheet.Range[A,A].Value := S;
      end;
      A := 'G'+SRow;
      ExcelWorkSheet.Range[A,A].Value := Par.PARCISAbs;
      A := 'H'+SRow;
      ExcelWorkSheet.Range[A,A].Value := Par.PARPOD;
      A := 'I'+SRow;
      ExcelWorkSheet.Range[A,A].Value := Par.Vymera;
      A := 'J'+SRow;
      ExcelWorkSheet.Range[A,A].Value := Are.Area/10000;
      A := 'K'+SRow;
      if ABS(Par.Plocha_ha - Are.Area/10000) <= Tolerance then
         ExcelWorkSheet.Range[A,A].Value := 1
      else
         ExcelWorkSheet.Range[A,A].Value := 0;
      LastRow := LastRow + 1;
      BB := Ord('L');
      if List.Count > 0 then
      begin
         Dyn := TDynamicProperties(FU.FindElementClass(TDynamicProperties));
         if Dyn <> nil then
         for j := 0 to List.Count - 1 do
         begin
            A := Char(BB)+SRow;
            S := List.Strings[j];
            S := Dyn.GetPropertyStringByName(S);
            if cp_Is852(S) then
               S := cp852ToWin(S);
            ExcelWorkSheet.Range[A,A].Value := S;
            BB := BB + 1;
         end;
      end;
      if (ParC <> nil) and (ParList.Count > 0) then
      for j := 0 to ParList.Count - 1 do
      begin
         A := Char(BB)+SRow;
         S := ParList.Strings[j];
         S := PARC.GetPropertyStringByName(S);
         ExcelWorkSheet.Range[A,A].Value := S;
         BB := BB + 1;
      end;
      Application.ProcessMessages;
   end;
end;

begin
 { DONE -oPPK -cZkontrolovat : Zkontrolovat funknost soupisu pro Mejzra }
   List := TStringList.Create;
   List.CommaText := DynamicProps;
   ParList := TStringList.Create;
   ParList.CommaText := ParDynamicProps;
   Enn := FindParcely(P);
   if ByPSK then
   begin
      if not Enn.NoElements then
      for i := 0 to Enn.LastIndex do
      begin
         FUU := Tgis_ForestUnit(Enn.Elements[i]);
         //PSK := Tgis_PorostniSkupina(FUU.Properties);
         SolveParcely(FUU);
      end;
   end
   else
      SolveParcely(Enn);
   Enn.Free;
   List.Free;
   ParList.Free;
end;

procedure TSoupisParcel.ProcessSkupinaParcel(SKP: Tgis_SkupinaParcel; var SumV,SumVz,SumP,Sum: double);
var
   A,B: OleVariant;
   APrl: string;
   SRow: string;
   i,j,k: integer;
   P: Tgis_Parcela;
   PC: Tgis_ParcelaCast;
   e: TElement;
   EN,EC: TOwnerEntity;
   EOdd,EDil: TEntity;
   TT: TSortElementsBy2Integers;
   EPRL,ListPRL: TEntity;
   SumVym,SumPRL: double;
   S: string;
   PSK: Tgis_PorostniSkupina;
   Done: boolean;
   DCount: integer;
   SL: TStringList;
begin
   if SKP.Cislo = 0 then exit;
   if SKP.NoParcela and SKP.NoParcast then exit;
   EN := TOwnerEntity.Create;
   EC := TOwnerEntity.Create;
   EPRL := TOwnerEntity.Create;
   ListPRL := TOwnerEntity.Create;
   // parcely
   EN.AddAllFrom := SKP.Parcely;
   // casti parcel
   if not SKP.NoParcast then
   for i := 0 to SKP.ParCasti.LastIndex do
   begin
      PC := Tgis_ParcelaCast(SKP.ParCasti.Elements[i]);
      P := PC.Parcela;
      if not P.IsMyOwner(EN) then
        EN.AddElement(P);
   end;
   //PUPFLNe
   if PUPFLne then
   for i := EN.LastIndex downto 0 do
   begin
      P := Tgis_Parcela(EN.Elements[i]);
      if not P.PUPFL then
         En.RemoveElement(P);
   end;
   if En.NoElements then // nic tam nezbylo
   begin
      EN.Free;
      EC.Free;
      EPRL.Free;
      ListPRL.Free;
      exit;
   end;
   // parcis nebo comment pro obce
   if Obce then
   begin
      SRow := IntToStr(LastRow);
      A := 'A'+SRow;
      if SKPComment then
      begin
         ExcelWorkSheet.Range[A,A].Value := SKP.Comment;
         Format := '@';
         ExcelWorkSheet.Range[A,A].NumberFormat := Format;
      end
      else
         ExcelWorkSheet.Range[A,A].Value := SKP.Name;
   end;
   // setrideni
   TT := TSortElementsBy2Integers.Create;
   TT.PropertyName := 'Parcis';
   TT.PropertyName1 := 'Parpod';
   TT.Input := EN;
   TT.DoIt;
   TT.Free;
   Done := false;
   DCount := EN.Count;
   for i := 0 to EN.LastIndex do
   begin
      P := Tgis_Parcela(EN.Elements[i]);
      if not processParcelaBySKP(P,SKP,EPRL,ListPRL,SumVym) then
         Dec(DCount)
      else
         Done := true;
   end;
   if not DONE then
   begin
      SRow := IntToStr(LastRow);
      A := 'A'+SRow;
      ExcelWorkSheet.Range[A,A].Value := SKP.Name+' ERROR: bu mal plocha nebo lin kresba!';
      ExcelWorkSheet.Range[A,A].Font.Color := '255';
      ExcelWorkSheet.Range[A,A].Font.Bold := true;
      lastRow := LastRow + 1;
   end
   else
   if DCount < 2 then
      lastRow := LastRow - 1;
   // soucet ploch
   SumPRL := 0;
   for i := 0 to ListPRL.LastIndex do
   begin
      e := ListPRL.Elements[i];
      PSK := Tgis_PorostniSkupina(e);
      SumPRL := SumPRL + PSK.Plocha;
   end;
   SumPRL := Simple_RoundTo(SUmPRL,-2);
   SumP := SumP + SumPRL;
   SRow := IntToStr(LastRow);
   A := 'E'+SRow;
   SumV := SumV + SumVym;
   ExcelWorkSheet.Range[A,A].Value := SumVym;//SKP.Vymera;
   Format := FormatFloat('0.0000',0);
   ExcelWorkSheet.Range[A,A].NumberFormatLocal := Format;
   A := 'F'+SRow;
   SumVym := Simple_RoundTo(SumVym,-2);
   SumVz := SumVz + SumVym;
   ExcelWorkSheet.Range[A,A].Value := SumVym;//Simple_RoundTo(SKP.Vymera,-2);
   Format := FormatFloat('0.00',0);
   ExcelWorkSheet.Range[A,A].NumberFormatLocal := Format;
   //ExcelWorkSheet.Range[A,A].Value := FloatTOStr(Simple_RoundTo(SKP.Plocha_ha,-2))+'/'+FloatTOStr(Simple_RoundTo(SumPRL,-2));
   if not Obce then
   begin
      A := 'G'+SRow;
      ExcelWorkSheet.Range[A,A].Value := SumPRL;
      ExcelWorkSheet.Range[A,A].NumberFormatLocal := Format;
      if SKP.KVAL_P=1 then
      begin
         A := 'F'+SRow;
         Sum := Sum + SumVym;
      end
      else
      begin
         A := 'G'+SRow;
         Sum := Sum + SumPRL;
      end;
      ExcelWorkSheet.Range[A,A].Font.Bold := true;
      APrl := 'J';
   end
   else
      APrl := 'G';
   if PRLBySKP then
   begin
      if MaxPRLRowChars = 0 then
         MaxPRLRowChars := 40;
      SL := TStringList.Create;
      S := '';
      for i := 0 to EPRL.LastIndex do
      begin
         e := EPRL.Elements[i];
         EOdd := TEntity(e);
         S := S +EOdd.Name+' ';
         for j := 0 to EOdd.LastIndex do
         begin
            e := EOdd.Elements[j];
            S := S + e.Name;
            EDil := TEntity(e);
            if not EDil.NoElements then
            begin
               S := S +'(';
               for k := 0 to EDil.LastIndex do
               begin
                  if k > 0 then
                     S := S + ' ';
                  S := S + EDil.Elements[k].Name;
               end;
               S := S +')';
            end;
            if (i = 0) and (j = 0) then //  uplny zacatek, text se prida
            begin
               SL.Add(S);
            end
            else         // pridavame dalsi prl, budeme kontrolovat delku
            begin
               if Length(SL.Strings[SL.Count-1])+Length(S)+2 > MaxPRLRowChars then // moc
               begin
                  SL.Add(S); // moc dlouhe, dalsi radek
               end
               else
               begin
                  if j = 0 then // prvni dilec musime oddelit nove oddeleni
                     S := ', '+S;
                  S := SL.Strings[SL.Count-1]+S;
                  SL.Strings[SL.Count-1] := S
               end;
            end;
            S := '';
         end;
      end;
      for i := 0 to SL.Count - 1 do
      begin
         S := '="'+SL.Strings[i]+'"';
         A := APrl+SRow;
         ExcelWorkSheet.Range[A,A].Value := S;
         LastRow := LastRow+1;
         SRow := IntToStr(LastRow);
      end;
      SL.Free;
      LastRow := LastRow+1;
{
      for i := 0 to EPRL.LastIndex do
      begin
         if i > 0 then
         begin
            if (Length(S) > 40) or ((40 - Length(S))< 6) then
            begin
               S := '="'+S+'"';
               A := 'J'+SRow;
               ExcelWorkSheet.Range[A,A].Value := S;
               LastRow := LastRow+1;
               SRow := IntToStr(LastRow);
               S := '';
            end
            else
               S := S+', ';
         end;
         e := EPRL.Elements[i];
         EOdd := TEntity(e);
         S := S +EOdd.Name+' ';
         for j := 0 to EOdd.LastIndex do
         begin
            e := EOdd.Elements[j];
            S := S + e.Name;
            EDil := TEntity(e);
            if not EDil.NoElements then
            begin
               S := S +'(';
               for k := 0 to EDil.LastIndex do
               begin
                  if k > 0 then
                     S := S + ' ';
                  S := S + EDil.Elements[k].Name;
               end;
               S := S +')';
            end;
         end;
      end;
      if S <> '' then
      begin
         S := '="'+S+'"';
         A := 'J'+SRow;
         ExcelWorkSheet.Range[A,A].Value := S;
         LastRow := LastRow+1;
      end;
      LastRow := LastRow+1;
}
   end
   else
      LastRow := LastRow+2;
   EN.Free;
   EC.Free;
   EPRL.Free;
   ListPRL.Free;
end;

procedure TSoupisParcel.SetAutoValidSKP(const Value: boolean);
begin
  FAutoValidSKP := Value;
end;

procedure TSoupisParcel.SetByPAR(const Value: boolean);
begin
  FByPAR := Value;
end;

procedure TSoupisParcel.SetByPSK(const Value: boolean);
begin
  FByPSK := Value;
end;

procedure TSoupisParcel.SetBySKP(const Value: boolean);
begin
  FBySKP := Value;
end;

procedure TSoupisParcel.SetDETAreas(const Value: TEntity);
begin
  FDETAreas := Value;
end;

procedure TSoupisParcel.SetDETLines(const Value: TEntity);
begin
  FDETLines := Value;
end;

procedure TSoupisParcel.SetDruh_BzlPropertyName(const Value: string);
begin
  FDruh_BzlPropertyName := Value;
end;

procedure TSoupisParcel.SetDynamicProps(const Value: string);
begin
  FDynamicProps := Value;
end;

procedure TSoupisParcel.SetFileName(const Value: string);
begin
  FFileName := Value;
end;

procedure TSoupisParcel.SetFilterElementForPARZARne(const Value: TElement);
begin
  FFilterElementForPARZARne := Value;
end;

procedure TSoupisParcel.SetFilterSKP(const Value: boolean);
begin
  FFilterSKP := Value;
end;

procedure TSoupisParcel.SetForestModel(const Value: Tgis_ForestModel);
begin
  FForestModel := Value;
end;

procedure TSoupisParcel.SetHTML(const Value: boolean);
begin
  FHTML := Value;
end;

procedure TSoupisParcel.SetKultury(const Value: boolean);
begin
  FKultury := Value;
end;

procedure TSoupisParcel.SetLastRow(const Value: integer);
begin
  FLastRow := Value;
end;

procedure TSoupisParcel.SetLevelQT(const Value: TRectEntityQTree);
begin
   FLevelQT := Value;
   AddElement(FLevelQT);
end;

procedure TSoupisParcel.SetLHCName(const Value: string);
begin
  FLHCName := Value;
end;

procedure TSoupisParcel.SetLHO(const Value: boolean);
begin
  FLHO := Value;
end;

procedure TSoupisParcel.SetMaxOdd(const Value: integer);
begin
  FMaxOdd := Value;
end;

procedure TSoupisParcel.SetMaxPRLRowChars(const Value: integer);
begin
  FMaxPRLRowChars := Value;
end;

procedure TSoupisParcel.SetMinODD(const Value: integer);
begin
  FMinODD := Value;
end;

procedure TSoupisParcel.SetObce(const Value: boolean);
begin
  FObce := Value;
end;

procedure TSoupisParcel.SetParcelModel(const Value: Tgis_ParcelModel);
begin
  FParcelModel := Value;
end;

procedure TSoupisParcel.SetPARDynamicProps(const Value: string);
begin
  FPARDynamicProps := Value;
end;

procedure TSoupisParcel.SetPARLines(const Value: TEntity);
begin
  FPARLines := Value;
end;

procedure TSoupisParcel.SetPoznamkaPropertyName(const Value: string);
begin
  FPoznamkaPropertyName := Value;
end;

procedure TSoupisParcel.SetPRLBySKP(const Value: boolean);
begin
  FPRLBySKP := Value;
end;

procedure TSoupisParcel.SetPRLPorost(const Value: boolean);
begin
  FPRLPorost := Value;
end;

procedure TSoupisParcel.SetPUPFLne(const Value: boolean);
begin
  FPUPFLne := Value;
end;

procedure TSoupisParcel.SetSKPComment(const Value: boolean);
begin
  FSKPComment := Value;
end;

procedure TSoupisParcel.SetTest(const Value: boolean);
begin
  FTest := Value;
end;

procedure TSoupisParcel.SetTolerance(const Value: double);
begin
  FTolerance := Value;
end;

procedure TSoupisParcel.SetValidSKP(const Value: TEntity);
begin
  FValidSKP := Value;
  AddElement(Value);
end;

procedure TSoupisParcel.SetVlastnikCisloPropertyName(const Value: string);
begin
  FVlastnikCisloPropertyName := Value;
end;

procedure TSoupisParcel.SetVlastnikPropertyName(const Value: string);
begin
  FVlastnikPropertyName := Value;
end;

procedure TSoupisParcel.TableCell(Value: string);
begin
   System.Writeln(FText,'      <td>'+Value+'</td>');
end;

procedure TSoupisParcel.TableCellSpan(Span: integer; Value: string);
begin
   System.Writeln(FText,'      <td colspan="'+IntToStr(Span)+'">'+Value+'</td>');
end;

procedure TSoupisParcel.TableRowEnd;
begin
   System.Writeln(FText,'    </tr>');
end;

procedure TSoupisParcel.TableRowStart;
begin
   System.Writeln(FText,'    <tr>');
end;

INITIALIZATION
   RegisterOrbisClass(TSoupisParcel);
END.

