unit t_TopolPointDialog;

interface
uses
   e_Type,
   e_TopolElements,
   f_TopolPointDialog,
   t_SimplePointDialog;

type

TTopolPointDialog = class(TSimplePointDialog)
private
    FDruhy: TIntegerListElement;
    FDruhyAngle: TIntegerListElement;
    FActiveDruhIndex: integer;
    FDynamicProps: string;
    function GetTPForm: TTopolPointDialog_Form;
    function GetDruhy: TIntegerListElement;
    function GetDruhyList: string;
    procedure SetDruhyList(const Value: string);
    function GetActiveDruhIndex: integer;
    procedure SetActiveDruhIndex(const Value: integer);
    procedure SetNewDruh(const Value: integer);
    function GetDruhyAngle: TIntegerListElement;
    function GetDruhyListAngle: string;
    procedure SetDruhyListAngle(const Value: string);
    procedure SetDynamicProps(const Value: string);
public
   procedure DoIt; override;
   procedure CreateForm; override;
   property  TPForm: TTopolPointDialog_Form read GetTPForm;
   property  Druhy: TIntegerListElement read GetDruhy;
   property  DruhyAngle: TIntegerListElement read GetDruhyAngle;
published
   property DruhyList: string read GetDruhyList write SetDruhyList;
   property DruhyListAngle: string read GetDruhyListAngle write SetDruhyListAngle;
   property ActiveDruhIndex: integer read GetActiveDruhIndex write SetActiveDruhIndex;
   property NewDruh: integer write SetNewDruh;
   property DynamicProps: string read FDynamicProps write SetDynamicProps;
end;

implementation
uses
   Classes,
   Forms,
   SysUtils,
   e_Class,
   e_Color,
   e_Geometry,
   e_Props,
   e_TopolDruhy,
   m_Orbis,
   Worlds;

{ TTopolPointDialog }

procedure TTopolPointDialog.CreateForm;
begin
   aForm := TTopolPointDialog_Form.Create(Application);
   aForm.Tool := Self;
   aForm.IsPossibleToClose := false;
end;

procedure TTopolPointDialog.DoIt;
var
   D: TTopolDruhy;
   TD: TTopolDruh;
   i: integer;
   P: TPointElement;
   TP: TTopolPointElement;
   S: string;
   SL: TStringList;
begin
   if input = nil then exit;
   D := TTopolDruhy(World.ObjectModel.FindElementClass(TTopolDruhy));
   SL := TStringList.Create;
   SL.CommaText := DynamicProps;
   TPForm.DynamicProps := SL;
   if not(Input is TPointElement) then
   begin
      if not(Input is TTopolPointElement) then exit;
      TP := TTopolPointElement(Input);
      P := TPointElement(TP.Geometry);
      //Value_ := GP.GetPropertyLongIntByName(ValueName);
      if D <> nil then
      begin
         D.FindBDruhKod := TP.Druh;
         TD := D.ActualDruh;
         if TD <> nil then
         begin
            i := TPForm.ComboDruh.Items.IndexOfObject(TD);
            if i < 0 then
               i := TPForm.ComboDruh.Items.AddObject(TD.Name+'['+IntToStr(TD.KOD)+']',TD);
            TPForm.ComboDruh.ItemIndex := i;
         end;
      end;
      TPForm.ColorPanel.Color := DejBarvicku(TP.Barva);
      TPForm.ColorPanel.Caption := TP.GetPropertyStringByName('Barva');
      //TPForm.ed_AtrPuv.Value := TL.Presnost;
      TPForm.ed_Znacka.Value := TP.Znacka;
      for i := 0 to SL.Count - 1 do
      begin
         TPForm.SetDynamicProp(SL.Strings[i],TP.GetPropertyStringByName(SL.Strings[i]));
      end;
   end
   else
   begin
      P := TPointElement(Input);
      //if ElementClass = nil then exit;
      TP := TTopolPointElement.Create;
      TP.Geometry := P;
   end;
   Output := TP;
   if TPForm.ComboDruh.ItemIndex < 0 then
   begin
      TPForm.ComboDruh.ItemIndex := 0;
      TPForm.ComboDruhChange(nil);
   end;
   if not DontShowForm then
   begin
      //TPForm.ed_Value.Value := Value_;
      DialogDoIt;
      //if Result = 1 then
         //Value_ := Form.ed_Value.Value;
   end;
   if DontShowForm or (Result = 1) then
   begin
      if D <> nil then
      begin
         TP.SetPropertyLongIntByName('DRUH',TTopolDruh(TPForm.ComboDruh.Items.Objects[TPForm.ComboDruh.ItemIndex]).KOD);
      end;
      TP.SetPropertyLongIntByName('Znacka',TPForm.ed_Znacka.Value);
      TP.SetPropertyLongIntByName('Barva',StrToIntDef(TPForm.ColorPanel.Caption,0));
      //TP.SetPropertyLongIntByName(ValueName,Value_);
      AngleChange := DruhyAngle.ValueExists(TP.Druh);
      if SL.Count > 0 then
      begin
         if TP.NoProperties then
            TP.Properties := TDynamicProperties.Create;
         for i := 0 to SL.Count - 1 do
         begin
            TP.Properties.SetPropertyStringByName(SL.Strings[i],TPForm.GetDynamicProp(SL.Strings[i]));
         end;
      end;
   end;
end;

function TTopolPointDialog.GetActiveDruhIndex: integer;
begin
   result := -1;
   if TPForm = nil then exit;
   result := TPForm.ComboDruh.ItemIndex;
end;


function TTopolPointDialog.GetDruhy: TIntegerListElement;
begin
   if FDruhy = nil then
   begin
      FDruhy := TIntegerListElement.Create;
      AddElement(FDruhy);
   end;
   result := FDruhy;
end;

function TTopolPointDialog.GetDruhyAngle: TIntegerListElement;
begin
   if FDruhyAngle = nil then
   begin
      FDruhyAngle := TIntegerListElement.Create;
      AddElement(FDruhyAngle);
   end;
   result := FDruhyAngle;
end;

function TTopolPointDialog.GetDruhyList: string;
var
   i: integer;
   S: string;
begin
   result := '';
   for i := 0 to Druhy.LastIndex do
   begin
      S := IntToStr(Druhy.Integers[i]);
      if i = 0 then
         result := S
      else
         result := result + ',' + S;
   end;
end;

function TTopolPointDialog.GetDruhyListAngle: string;
var
   i: integer;
   S: string;
begin
   result := '';
   for i := 0 to DruhyAngle.LastIndex do
   begin
      S := IntToStr(DruhyAngle.Integers[i]);
      if i = 0 then
         result := S
      else
         result := result + ',' + S;
   end;
end;

function TTopolPointDialog.GetTPForm: TTopolPointDialog_Form;
begin
   result := TTopolPointDialog_Form(aForm);
end;

procedure TTopolPointDialog.SetActiveDruhIndex(const Value: integer);
begin
   FActiveDruhIndex := Value;
   if TPForm = nil then exit;
   TPForm.ComboDruh.ItemIndex := Value;
   TPForm.ComboDruhChange(nil);
end;


procedure TTopolPointDialog.SetDruhyList(const Value: string);
var
   D: TTopolDruhy;
   TD: TTopolDruh;
   i: integer;
   all: boolean;
begin
   all := (UpperCase(Value) = 'ALL');
   if not all then
      Druhy.ValuesString := Value;
   D := TTopolDruhy(World.ObjectModel.FindElementClass(TTopolDruhy));
   if D <> nil then
   begin
      TPForm.ComboDruh.Items.Clear;
      for i := 0 to D.B.LastIndex do
      begin
         TD := TTopolDruh(D.B.Elements[i]);
         if all or Druhy.ValueExists(TD.KOD) then
         begin
            TPForm.ComboDruh.Items.AddObject(TD.Name+'['+IntToStr(TD.KOD)+']',TD);
            if all then
               Druhy.AddInteger(TD.KOD);
         end;
      end;
      TPForm.ComboDruh.ItemIndex := FActiveDruhIndex;
      TPForm.ComboDruhChange(nil);
   end;
end;

procedure TTopolPointDialog.SetDruhyListAngle(const Value: string);
begin
   DruhyAngle.ValuesString := Value;
end;

procedure TTopolPointDialog.SetDynamicProps(const Value: string);
begin
  FDynamicProps := Value;
end;

procedure TTopolPointDialog.SetNewDruh(const Value: integer);
begin
   if Output = nil then exit;
   PostMsgToOwners(tm_Resize,nil,false);
   Application.ProcessMessages;
   TTopolPointElement(Output).Druh := Value;
   TTopolPointElement(Output).Barva := StrToInt(TPForm.ColorPanel.Caption);
   TTopolPointElement(Output).Znacka := TPForm.ed_Znacka.Value;
   PostMsgToOwners(tm_Resize,nil,false);
   Application.ProcessMessages;
end;


INITIALIZATION
   RegisterOrbisClass(TTopolPointDialog);
END.
