unit E_guide;
{
*********************************************************************
*                                                                   *
*  ORBIS ... Kancel ... prvodce                                  *
*                                                                   *
*                       Topol Pro 1996                             *
*********************************************************************
}


interface

uses
   Classes,
// KERNEL
   Elements,
// GLIB
   g_Elems,
// MAPPER
   e_MSpace,
   e_ZM;


type


TGuideParameters = class
end;

TGuidePage_2 = class(TSMO_2)
end;


TGuidePage_5 = class(TSMO_5)
private
   function GetCIT : boolean;
   function GetRAS : boolean;
   function GetRAK : boolean;
   function GetTRAS: boolean;
   function GetTRAK: boolean;
   function GetFileName: string;
protected
   function  Create_SMO_2: TSMO_2; override;
public
   property  FileName: string read GetFileName;
   procedure FreeEmptySpaces; override;
published
   property CIT: boolean read GetCIT stored false;
   property RAS: boolean read GetRAS stored false;
   property RAK: boolean read GetRAK stored false;
   property TRAS: boolean read GetTRAS stored false;
   property TRAK: boolean read GetTRAK stored false;
end;


TGuidePage_10 = class(TSMO_10)
protected
   function  Create_SMO_5: TSMO_5; override;
public
   procedure FreeEmptySpaces; override;
end;


TGuidePage_25 = class(TSMO_25)
public
   procedure FreeEmptySpaces; override;
end;


TGuidePage_50 = class(TSMO_50)
protected
   function  Create_SMO_10: TSMO_10; override;
   function  Create_SMO_25: TSMO_25; override;
public
   procedure FreeEmptySpaces; override;
end;

TUserMapSpaces = class(TRectEntity);
TVectors=class(TOwnerEntity);

TGuide = class(TSMO)
private
   FVectors: TVectors;
   FuserMapSpaces: TUserMapSpaces;
   FZM: TMapsZM;
   FZMSpace: TZMSpace;
   function  GetVectors: TVectors;
   function  GetUserMapSpaces: TUserMapSpaces;
   procedure SetVectors(aValue: TVectors);
   procedure SetUserMapSpaces(aValue: TUserMapSpaces);
   function  StoreVectors: boolean;
   function  StoreUserMapSpaces: boolean;
   procedure CheckVectorsAndSpaces;
   function  GetUserMapSpace(x, y: double): TUserMapSpace;
    function GetZM: TMapsZM;
    function GetZMSpace: TZMSpace;
protected
   function  Create_SMO_50: TSMO_50; override;
   procedure AddPagesByCIT;
   procedure AddPagesByRAS;
   procedure AddPagesByRAK;
   procedure AddPagesByTRAS;
   procedure AddPagesByTRAK;
   procedure AddPagesByRasters;
   procedure AddPagesByPath(const aPath: string);
   procedure AddPageByRasterName(aName: string);
   function  GetName: string; override;
public
   procedure Load(aReader: TReader; aIndex: TEntity); override;
   procedure Save(aWriter: TWriter); override;
   procedure FreeEmptySpaces(all: boolean);
   procedure CreateByStandardScript;
   procedure CreateByRasters;
   procedure AddUserMapSpace(aValue: TUserMapSpace);

   property  UserMapSpace[x,y: double]: TUserMapSpace read GetUserMapSpace;
published
   property  Vectors: TVectors read GetVectors write SetVectors stored StoreVectors;
   property  UserMapSpaces: TUserMapSpaces read GetUserMapSpaces write SetUserMapSpaces stored StoreUserMapSpaces;
   property  ZM: TMapsZM read GetZM stored false;
   property  ZMSpace: TZMSpace read GetZMSpace stored false;
end;



implementation

uses
   Dialogs,
   SysUtils,
// KERNEL
   e_Class,
   u_String,
   Worlds,
   //e_PWorld,
// MAPPER
   e_SpTree;


{ TGuidePage_5 }

function  TGuidePage_5.GetFileName: string;
begin
   result := Abbrev + IntToStr(Col) + IntToStr(Row);
end;


function  TGuidePage_5.GetCIT : boolean;
begin
  result:= false;
  {
  if TOldMainTool(World.MainTool).Project <> nil then
     result := FileExists(TOldMainTool(World.MainTool).Project.CITPath + FileName+'.CIT');
  }
end;


function  TGuidePage_5.GetRAS : boolean;
begin
  result:= false;
{
  if TOldMainTool(World.MainTool).Project <> nil then
     result := FileExists(TOldMainTool(World.MainTool).Project.RAKPath + FileName+'.RAS');
 }
end;


function  TGuidePage_5.GetRAK : boolean;
begin
  result:= false;
{
  if TOldMainTool(World.MainTool).Project <> nil then
     result := FileExists(TOldMainTool(Uorld.MainTool).Project.RAKPath + FileName+'.RAK');
}
end;


function  TGuidePage_5.GetTRAS: boolean;
begin
  result:= false;
{
  if TOldMainTool(World.MainTool).Project <> nil then
     result := FileExists(TOldMainTool(World.MainTool).Project.TRAKPath + FileName+'T.RAS');
}
end;


function  TGuidePage_5.GetTRAK: boolean;
var
   s: string;
begin
  result:= false;
 {
  if TOldMainTool(World.MainTool).Project <> nil then
  begin
    s := TOldMainTool(World.MainTool).Project.TRAKPath + FileName+'T.RAK';
    result := FileExists(s)
  end
}
end;



function TGuidePage_5.Create_SMO_2: TSMO_2;
begin
   result := TGuidePage_2.Create;
end;

procedure TGuidePage_5.FreeEmptySpaces;
var
   i, j: longInt;
   Col: TSpaces;
   G_2: TGuidePage_2;
begin
   for i := ColumnCount -1 downto 0 do
   begin
      Col := Column[i];
      if not Col.NoSpaces then
      begin
         for j := Col.Count -1 downto 0 do
         begin
            G_2 := TGuidePage_2(Col.Space[j]);
            if not G_2.Selected  then Col.FreeElement(G_2);
         end;
      end;
      if Col.NoSpaces then Matrix.FreeElement(Col);
   end;
end;

{ TGuidePage_10 }



procedure TGuidePage_10.FreeEmptySpaces;
var
   i, j: longInt;
   Col: TSpaces;
   G_5: TGuidePage_5;
begin
   for i := ColumnCount -1 downto 0 do
   begin
      Col := Column[i];
      if not Col.NoSpaces then
      begin
         for j := Col.Count -1 downto 0 do
         begin
            G_5 := TGuidePage_5(Col.Space[j]);
            if not G_5.Selected  then Col.FreeElement(G_5);
         end;
      end;
      if Col.NoSpaces then Matrix.FreeElement(Col);
   end;
end;



function  TGuidePage_10.Create_SMO_5: TSMO_5;
begin
   result := TGuidePage_5.Create;
end;



{ TGuidePage_50 }


procedure TGuidePage_25.FreeEmptySpaces;
var
   i, j: longInt;
   Col: TSpaces;
   G_10: TGuidePage_10;
begin
   for i := 0 to ColumnCount -1 do
   begin
      Col := Column[i];
      if not Col.NoSpaces then
      begin
         for j := 0 to Col.Count -1 do
         begin
            G_10 := TGuidePage_10(Col.Space[j]);
            G_10.FreeEmptySpaces;
         end;
      end;
   end;
   inherited FreeEmptySpaces;
end;




{ TGuidePage_50 }


procedure TGuidePage_50.FreeEmptySpaces;
var
   i, j: longInt;
   Col: TSpaces;
   G_25: TGuidePage_25;
begin
   for i := 0 to ColumnCount -1 do
   begin
      Col := Column[i];
      if not Col.NoSpaces then
      begin
         for j := 0 to Col.Count -1 do
         begin
            G_25 := TGuidePage_25(Col.Space[j]);
            G_25.FreeEmptySpaces;
         end;
      end;
   end;
   inherited FreeEmptySpaces;
end;


function  TGuidePage_50.Create_SMO_10: TSMO_10;
begin
   result := TGuidePage_10.Create;
end;


function  TGuidePage_50.Create_SMO_25: TSMO_25;
begin
   result := TGuidePage_25.Create;
end;


{ TGuide }


procedure  TGuide.Load(aReader: TReader; aIndex: TEntity);
var
   i: longInt;
   V: TVectors;
begin
   inherited Load(aReader, aIndex);
   FVectors := TVectors(LoadElement(aReader, aIndex));

   if (FVectors <> nil) and not (TElement(FVectors) is TVectors) then
   begin
      V := TVectors.Create;
      V.Name := FVectors.Name;
      for i := 0 to FVectors.Count - 1 do
      begin
         V.AddElement(FVectors.Elements[i]);
      end;
      FreeElement(FVectors);
      FVectors := V;
      AddElement(V);
   end;
   FUserMapSpaces := TUserMapSpaces(LoadElement(aReader, aIndex));
end;


procedure  TGuide.Save(aWriter: TWriter);
begin
   inherited Save(aWriter);
   SaveElement(Vectors, aWriter);
   SaveElement(UserMapSpaces, aWriter);
end;





function TGuide.GetVectors: TVectors;
begin
   CheckVectorsAndSpaces;
   result := FVectors;
end;


function TGuide.GetUserMapSpaces: TUserMapSpaces;
begin
   CheckVectorsAndSpaces;
   result := FUserMapSpaces;
end;


procedure TGuide.SetVectors(aValue: TVectors);
begin
   if FVectors <> nil then
   begin
      FVectors.Free;
      FVectors := nil;
   end;
   FVectors := aValue;
end;


procedure TGuide.SetUserMapSpaces(aValue: TUserMapSpaces);
begin
   if FUserMapSpaces <> nil then
   begin
      FUserMapSpaces.Free;
      FUserMapSpaces := nil;
   end;
   FUserMapSpaces := aValue;
end;

function  TGuide.StoreVectors: boolean;
begin
   result := FVectors <> nil;
end;

function  TGuide.StoreUserMapSpaces: boolean;
begin
   result := FUserMapSpaces <> nil;
end;



procedure TGuide.CheckVectorsAndSpaces;
begin
   if FUserMapSpaces = nil then
   begin
      FUserMapSpaces := TUserMapSpaces.Create;
      FUserMapSpaces.Name := 'Uivatelsk mapov prostory';
      AddElement(FUserMapSpaces);
   end;
   if ZMSpace = nil then;
   if ZM = nil then;
   if FVectors = nil then
   begin
      FVectors := TVectors.Create;
      FVectors.Name := 'Vektorov kresba prvodce';
      AddElement(FVectors);
   end;
   if IndexOf(ZM) > IndexOf(FVectors) then
   begin
      RemoveElement(ZM);
      InsertElement(IndexOf(FVectors),ZM);
   end;
   if IndexOf(ZMSpace) > IndexOf(ZM) then
   begin
      RemoveElement(ZMSpace);
      InsertElement(IndexOf(ZM),ZMSpace);
   end;
end;


function  TGuide.GetUserMapSpace(x, y: double): TUserMapSpace;
var
   i: longInt;
   E: TMapSpace;
begin
   result := nil;
   for i := UserMapSpaces.Count -1 downto 0 do
   begin
      E := TMapSpace(UserMapSpaces.Elements[i]);
      if (E is TUserMapSpace) and E.PointIsInside(X,Y) then
      begin
         result := TUserMapSpace(E);
         exit;
      end;
   end;
end;


function  TGuide.Create_SMO_50: TSMO_50;
begin
   result := TGuidePage_50.Create;
end;


procedure TGuide.AddPagesByPath(const aPath: string);
var
   Res: integer;
   SR: TSearchRec;
begin
   Res := FindFirst(aPath, faAnyFile, SR);
   AddPageByRasterName(SR.Name);
   while not wordBool(Res) do
   begin
      Res := FindNext(SR);
      AddPageByRasterName(SR.Name);
   end;
end;


procedure TGuide.AddPagesByCIT;
begin
//   AddPagesByPath(TRTProjectWorld(World).Project.CITPath + '*.CIT');
end;


procedure TGuide.AddPagesByRAS;
begin
//   AddPagesByPath(TRTProjectWorld(World).Project.RAKPath + '*.RAS');
end;


procedure TGuide.AddPagesByRAK;
begin
//   AddPagesByPath(TRTProjectWorld(World).Project.RAKPath + '*.RAK');
end;


procedure TGuide.AddPagesByTRAS;
begin
//   AddPagesByPath(TRTProjectWorld(World).Project.TRAKPath + '*T.RAS');
end;


procedure TGuide.AddPagesByTRAK;
begin
//   AddPagesByPath(TRTProjectWorld(World).Project.RAKPath + '*T.RAK');
end;


procedure TGuide.AddPagesByRasters;
begin
   AddPagesByCIT;
   AddPagesByRAS;
   AddPagesByRAK;
   AddPagesByTRAS;
   AddPagesByTRAK;
end;



procedure TGuide.AddPageByRasterName(aName: string);
var
   s: string;
   aAbbrev: string[4];
   aCol, aRow: integer;
begin
   try
      aAbbrev := copy(aName, 1, 4);
      s := copy(aName, 5, 1);
      aCol := StrToInt(s);
      s := copy(aName, 6, 1);
      aRow := StrToInt(s);
      Add_SMO_5_ByAbbrev(aCol, aRow, aAbbrev);
   except
      ShowMessage('Chybn nzev rastru: ' + aName);
   end;
end;



procedure TGuide.FreeEmptySpaces(all: boolean);
var
   i, j: longInt;
   Col: TSpaces;
   G_50: TGuidePage_50;
begin
   if all then
   for i := 0 to ColumnCount -1 do
   begin
      Col := Column[i];
      if not Col.NoSpaces  then
      begin
         for j := 0 to Col.Count -1 do
         begin
            G_50 := TGuidePage_50(Col.Space[j]);
            G_50.FreeEmptySpaces;
         end;
      end;
   end;
   inherited FreeEmptySpaces;
end;



procedure TGuide.CreateByStandardScript;
var
   S: string;
begin
   S := ExtractFilePath(ParamStr(0));
   Create_SMO_50_ByScript(CheckBackSlash(S)+'MAPLSMO.txt');
end;



procedure TGuide.CreateByRasters;
begin
   CreateByStandardScript;
   AddPagesByRasters;
   {
   FreeEmptySpaces;
   }
end;

procedure TGuide.AddUserMapSpace(aValue: TUserMapSpace);
begin
   UserMapSpaces.AddAndUpdateRect(aValue);
end;



function TGuide.GetName: string;
begin
   result := 'Prvodce ';
end;



function TGuide.GetZM: TMapsZM;
begin
   result := nil;
   if FZM = nil then
   begin
      FZM := TMapsZM(FindOrCreateElementName(TMapsZM,'ZM'));
   end;
   result := FZM;
end;


function TGuide.GetZMSpace: TZMSpace;
begin
   result := nil;
   if FZMSpace = nil then
   begin
      FZMSpace := TZMSpace(FindElementClass(TZMSpace));
      if FZMSpace = nil then
      begin
         FZMSpace := TZMSpace.Create;
         AddElement(FZMSpace);
      end;
   end;
   result := FZMSpace;
end;

BEGIN
   RegisterOrbisClass(TGuidePage_2);
   RegisterOrbisClass(TGuidePage_5);
   RegisterOrbisClass(TGuidePage_10);
   RegisterOrbisClass(TGuidePage_25);
   RegisterOrbisClass(TGuidePage_50);
   RegisterOrbisClass(TGuide);
   RegisterOrbisClass(TUserMapSpaces);
   RegisterOrbisClass(TVectors);
END.
