unit E_MSpace;
{
*********************************************************************
*                                                                   *
*  ORBIS ... Mapov prostory                                        *
*                                                                   *
*                       Topol Pro 1996                             *
*********************************************************************
}

interface
uses
   Classes,
// KERNEL
   Elements,
// GLIB
   e_WCS,
   g_Elems,
// MAPPER
   e_ZM,
   e_SpTree;


const
   SMO_50_H = 25000;
   SMO_50_V = 20000;
   SMO_25_H = 12500;
   SMO_25_V = 10000;
   SMO_10_H =  5000;
   SMO_10_V =  4000;
   SMO_5_H  =  2500;
   SMO_5_V  =  2000;
   SMO_2_H  =  1250;
   SMO_2_V  =  1000;



type

TSMO = class;

TMapSpace = class(TSpaceTree)
private
   FNumber: longInt;
   FAbbrev:  String[4];
   FSelected: boolean;
   function  GetAbbrev: string;
   procedure SetAbbrev(aValue: string);
   function  GetPageWidth: double; virtual;
   function  GetPageHeight: double; virtual;
protected
   function  GetScale: double; virtual;
   procedure SetScale(aValue: double); virtual;
   function  GetTopMapSpace: TMapSpace; virtual;
   function  GetBottomMapSpace: TMapSpace; virtual;
   function  GetLeftMapSpace: TMapSpace; virtual;
   function  GetRightMapSpace: TMapSpace; virtual;
   function  GetTopMSName: string; virtual;
   function  GetBottomMSName: string; virtual;
   function  GetLeftMSName: string; virtual;
   function  GetRightMSName: string; virtual;
   function  GetTopMSFullName: string; virtual;
   function  GetBottomMSFullName: string; virtual;
   function  GetLeftMSFullName: string; virtual;
   function  GetRightMSFullName: string; virtual;
   function  GetSMO: TSMO; virtual;
   function  GetSelfCopyWithDetail: TPersistent; virtual;
   function  GetFullName: string; virtual;
   function  GetColRowText: string; virtual;
public
   procedure MakeDetail; virtual;
   procedure Load(aReader: TReader; aIndex: TEntity); override;
   procedure Save(aWriter: TWriter); override;
   property  Abbrev: string read GetAbbrev write SetAbbrev;
   property  Number: longInt read FNumber write Fnumber;
   property  SMO: TSMO read GetSMO;
   property  TopMapSpace: TMapSpace read GetTopMapSpace;
   property  BottomMapSpace: TMapSpace read GetBottomMapSpace;
   property  LeftMapSpace: TMapSpace read GetLeftMapSpace;
   property  RightMapSpace: TMapSpace read GetRightMapSpace;
published
   property  Jmeno: string read GetName write SetName stored false;
   property  Zkratka: string read GetAbbrev write SetAbbrev;
   property  FullName: string read GetFullName stored false;
   property  ColRowText: string read GetColRowText stored false;
   property  Cislo: longInt read FNumber write Fnumber;
   property  Selected: boolean read FSelected write FSelected default false;
   property  Scale: double read GetScale write SetScale;
   property  TopMSName: string read GetTopMSName stored false;
   property  BottomMSName: string read GetBottomMSName stored false;
   property  LeftMSName: string read GetLeftMSName stored false;
   property  RightMSName: string read GetRightMSName stored false;
   property  TopMSFullName: string read GetTopMSFullName stored false;
   property  BottomMSFullName: string read GetBottomMSFullName stored false;
   property  LeftMSFullName: string read GetLeftMSFullName stored false;
   property  RightMSFullName: string read GetRightMSFullName stored false;
   property  SelfCopyWithDetail: TPersistent read GetSelfCopyWithDetail stored false;
   property  PageWidth: double read GetPageWidth stored false;
   property  PageHeight: double read GetPageHeight stored false;
end;

TUserMapSpace = class(TMapSpace)
private
   FScale: double;
protected
   function  GetScale: double; override;
   procedure  SetScale(aValue: double); override;
end;

TZMSpace = class(TMapSpace)
private
   FZM: TMapsZM;
   procedure SetAngle(const Value: double);
   function GetZM: TMapsZM;
   procedure SetZM(const Value: TMapsZM);
   function GetAngle: double;
   function GetRAngle: double;
protected
   function GetScale: double; override;
   procedure SetScale(aValue: double); override;
public
   property ZM: TMapsZM read GetZM write SetZM;
published
   property Scale: double read GetScale write SetScale stored false;
   property Angle: double read GetAngle write SetAngle stored false;
   property RAngle: double read GetRAngle stored false;
end;

TSMO_10 = class;
TSMO_50 = class;

TSMO_2 = class(TMapSpace)
private
   FCol: integer;
   FRow: integer;
protected
   function  GetScale:  double; override;
   function  GetTopMapSpace: TMapSpace; override;
   function  GetBottomMapSpace: TMapSpace; override;
   function  GetLeftMapSpace: TMapSpace; override;
   function  GetRightMapSpace: TMapSpace; override;
   function  GetTopMSName: string; override;
   function  GetBottomMSName: string; override;
   function  GetLeftMSName: string; override;
   function  GetRightMSName: string; override;
   function  GetTopMSFullName: string; override;
   function  GetBottomMSFullName: string; override;
   function  GetLeftMSFullName: string; override;
   function  GetRightMSFullName: string; override;
   function  GetColRowText: string; override;
   function  GetFullName: string; override;
public
   procedure Load(aReader: TReader; aIndex: TEntity); override;
   procedure Save(aWriter: TWriter); override;
   function  Create_SMO_10: TSMO_10;
published
   property Col: integer read FCol write FCol;
   property Row: integer read FRow write FRow;
end;


TSMO_5 = class(TSMO_2)
  private
    //function Create_SMO_10: TSMO_10;
protected
   function  GetScale:  double; override;
   function  GetTopMapSpace: TMapSpace; override;
   function  GetBottomMapSpace: TMapSpace; override;
   function  GetLeftMapSpace: TMapSpace; override;
   function  GetRightMapSpace: TMapSpace; override;
   function  GetTopMSName: string; override;
   function  GetBottomMSName: string; override;
   function  GetLeftMSName: string; override;
   function  GetRightMSName: string; override;
   function  GetTopMSFullName: string; override;
   function  GetBottomMSFullName: string; override;
   function  GetLeftMSFullName: string; override;
   function  GetRightMSFullName: string; override;
   function  GetColRowText: string; override;
public
   procedure Load(aReader: TReader; aIndex: TEntity); override;
   procedure Save(aWriter: TWriter); override;
   function  Create_SMO_2: TSMO_2; virtual;
   function  GetSelfCopyWithDetail: TPersistent; override;
   procedure MakeDetail; override;
   function  Get_SMO_2_ByCR(aCol, aRow: integer): TSMO_2;
   function  Add_SMO_2_ByCR(aCol, aRow: integer): TSMO_2;
end;


TSMO_10 = class(TSMO_5)
protected
   function  GetScale:  double; override;
   function  Create_SMO_5: TSMO_5; virtual;
   function  GetTopMapSpace: TMapSpace; override;
   function  GetBottomMapSpace: TMapSpace; override;
   function  GetLeftMapSpace: TMapSpace; override;
   function  GetRightMapSpace: TMapSpace; override;
   function  GetTopMSName: string; override;
   function  GetBottomMSName: string; override;
   function  GetLeftMSName: string; override;
   function  GetRightMSName: string; override;
   function  GetTopMSFullName: string; override;
   function  GetBottomMSFullName: string; override;
   function  GetLeftMSFullName: string; override;
   function  GetRightMSFullName: string; override;
public
   function  GetSelfCopyWithDetail: TPersistent; override;
   procedure MakeDetail; override;
   function  Get_SMO_5_ByCR(aCol, aRow: integer): TSMO_5;
   function  Add_SMO_5_ByCR(aCol, aRow: integer): TSMO_5;
end;


TSMO_25 = class(TSMO_5)
protected
   function  GetScale: double; override;
   function  GetTopMapSpace: TMapSpace; override;
   function  GetBottomMapSpace: TMapSpace; override;
   function  GetLeftMapSpace: TMapSpace; override;
   function  GetRightMapSpace: TMapSpace; override;
   function  GetTopMSName: string; override;
   function  GetBottomMSName: string; override;
   function  GetLeftMSName: string; override;
   function  GetRightMSName: string; override;
   function  GetTopMSFullName: string; override;
   function  GetBottomMSFullName: string; override;
   function  GetLeftMSFullName: string; override;
   function  GetRightMSFullName: string; override;
   function  GetColRowText: string; override;
public
   function  GetSelfCopyWithDetail: TPersistent; override;
   procedure Make5Detail; virtual;
end;





TSMO_50 = class(TSMO_10)
protected
   function  GetScale: double; override;
   function  Create_SMO_10: TSMO_10; virtual;
   function  Create_SMO_25: TSMO_25; virtual;
   function  GetTopMapSpace: TMapSpace; override;
   function  GetBottomMapSpace: TMapSpace; override;
   function  GetLeftMapSpace: TMapSpace; override;
   function  GetRightMapSpace: TMapSpace; override;
   function  GetTopMSName: string; override;
   function  GetBottomMSName: string; override;
   function  GetLeftMSName: string; override;
   function  GetRightMSName: string; override;
   function  GetTopMSFullName: string; override;
   function  GetBottomMSFullName: string; override;
   function  GetLeftMSFullName: string; override;
   function  GetRightMSFullName: string; override;
   function  GetColRowText: string; override;
   function  Get25From5(aCol, aRow: integer): integer;
public
   procedure MakeDetail; override;
   procedure Make10Detail; virtual;
   procedure Make25Detail; virtual;
   function  Get_SMO_10_ByCR(aCol, aRow: integer): TSMO_10;
   function  Get_SMO_5_ByCR(aCol, aRow: integer): TSMO_5;
   function  Add_SMO_10_ByCR(aCol, aRow: integer): TSMO_10;
   function  Add_SMO_5_ByCR(aCol, aRow: integer): TSMO_5;
   procedure Add_SMO_10(aSMO_10: TSMO_10);
   function  Get_SMO_25_ByCR(aCol, aRow: integer): TSMO_25;
   function  Add_SMO_25_ByNo(aNo: integer): TSMO_25;
   function  Add_SMO_25_ByCR(aCol, aRow: integer): TSMO_25;
   procedure Add_SMO_25(aSMO_25: TSMO_25);
   function  GetSelfCopyWithDetail10: TPersistent; virtual;
   function  GetSelfCopyWithDetail25: TPersistent; virtual;
published
   property  SelfCopyWithDetail10: TPersistent read GetSelfCopyWithDetail10 stored false;
   property  SelfCopyWithDetail25: TPersistent read GetSelfCopyWithDetail25 stored false;
end;



TSMO = class(TSMO_50)
private
   FWCS: TWCS;
   function GeTUserMapSpace(X, Y: double): TUserMapSpace;
   function GetSMO_50(X, Y: double): TSMO_50;
   function GetSMO_25(X, Y: double): TSMO_25;
   function GetSMO_10(X, Y: double): TSMO_10;
   function GetSMO_5 (X, Y: double): TSMO_5;
   function GetSMO_50Name(aX, aY: double): string;
   function GetSMO_25Name(aX, aY: double): string;
   function GetSMO_10Name(aX, aY: double): string;
   function GetSMO_5Name (aX, aY: double): string;
   function GetSMO_50FullName(aX, aY: double): string;
   function GetSMO_25FullName(aX, aY: double): string;
   function GetSMO_10FullName(aX, aY: double): string;
   function GetSMO_5FullName (aX, aY: double): string;
    function GetSMO_2(aX, aY: double): TSMO_2;
    function GetSMO_2FullName(aX, aY: double): string;
    function GetSMO_2Name(aX, aY: double): string;
    procedure SetSMO50ByScript(const Value: string);
    procedure SetAutoCreate50(const Value: boolean);
    procedure SetCreate10DetailByEntity(const Value: TEntity);
    procedure SetCreate5DetailByEntity(const Value: TEntity);
protected
   function  Create_SMO_50: TSMO_50; virtual;
public
   constructor Create; override;
   procedure Load(aReader: TReader; aIndex: TEntity); override;
   procedure Save(aWriter: TWriter); override;
   function  CreateEnvelope: TElement; override;
   procedure Create_SMO_50_ByScript(aScript: string);
   function  Get_SMO_50_ByName(aName: string): TSMO_50;
   function  Get_SMO_50_ByAbbrev(aAbbrev: string): TSMO_50;
   function  Get_SMO_25_ByName(aName: string): TSMO_25;
   function  Get_SMO_25_ByAbbrev(aAbbrev: string): TSMO_25;
   function  Get_SMO_10_ByName(aName: string; aCol, aRow: integer): TSMO_10;
   function  Get_SMO_10_ByAbbrev(aAbbrev: string; aCol, aRow: integer): TSMO_10;
   function  Get_SMO_5_ByName(aName: string; aCol, aRow: integer): TSMO_5;
   function  Get_SMO_5_ByAbbrev(aAbbrev: string; aCol, aRow: integer): TSMO_5;
   function  Get_SMO_2_ByName(aName: string; aCol, aRow: integer): TSMO_2;
   function  Get_SMO_2_ByAbbrev(aAbbrev: string; aCol, aRow: integer): TSMO_2;
   procedure GetSMONamesByCoord(aX, aY: double; var Name_50, Name_25, Name_10, Name_5, Name_2: string);
   procedure Add_SMO_50(aSMO_50: TSMO_50); virtual;
   procedure Add_SMO_50_ByName(aCol, aRow: integer; const aAbbrev, aName: string); virtual;
   procedure Add_SMO_25(aSMO_25: TSMO_25); virtual;
//   procedure Add_SMO_25_ByName(aCol, aRow: integer; const aAbbrev, aName: string); virtual;
   {
   procedure Add_SMO_10(aSMO_10: TSMO_10);
   procedure Add_SMO_5(aSMO_5: TSMO_5);
   }
   procedure Add_SMO_10_ByAbbrev(aCol, aRow: integer; const aAbbrev: string);
   procedure Add_SMO_5_ByAbbrev(aCol, aRow: integer; const aAbbrev: string);
   procedure Add_SMO_2_ByAbbrev(aCol, aRow: integer; const aAbbrev: string);
   property  WCS: TWCS read FWCS;
   property  SMO_50[X, Y: double]: TSMO_50 read GetSMO_50;
   property  SMO_25[X, Y: double]: TSMO_25 read GetSMO_25;
   property  SMO_10[X, Y: double]: TSMO_10 read GetSMO_10;
   property  SMO_5 [X, Y: double]: TSMO_5  read GetSMO_5;
   property  SMO_2 [X, Y: double]: TSMO_2  read GetSMO_2;
   property  SMO_50Name[X, Y: double]: string read GetSMO_50Name;
   property  SMO_25Name[X, Y: double]: string read GetSMO_25Name;
   property  SMO_10Name[X, Y: double]: string read GetSMO_10Name;
   property  SMO_5Name [X, Y: double]: string read GetSMO_5Name;
   property  SMO_2Name [X, Y: double]: string read GetSMO_2Name;
   property  SMO_50FullName[X, Y: double]: string read GetSMO_50FullName;
   property  SMO_25FullName[X, Y: double]: string read GetSMO_25FullName;
   property  SMO_10FullName[X, Y: double]: string read GetSMO_10FullName;
   property  SMO_5FullName [X, Y: double]: string read GetSMO_5FullName;
   property  SMO_2FullName [X, Y: double]: string read GetSMO_2FullName;
   property  ExtraMap[X, Y: double]: TUserMapSpace read GeTUserMapSpace;
published
   property  SMO50ByScript: string write SetSMO50ByScript stored false;
   property  AutoCreate50: boolean write SetAutoCreate50 stored false;
   property  Create10DetailByEntity: TEntity write SetCreate10DetailByEntity stored false;
   property  Create5DetailByEntity: TEntity write SetCreate5DetailByEntity stored false;
end;



implementation

uses
   SysUtils,
// KERNEL
   e_Class,
   u_String,
   Worlds;


{ TMapSpace }

procedure  TMapSpace.Load(aReader: TReader; aIndex: TEntity);
begin
   inherited Load(aReader, aIndex);
   FNumber   := aReader.ReadInteger;
   FAbbrev   := aReader.ReadString;
   FSelected := aReader.Readboolean;
   Scale     := aReader.ReadFloat;
end;


procedure  TMapSpace.Save(aWriter: TWriter);
begin
   inherited Save(aWriter);
   aWriter.WriteInteger(FNumber);
   aWriter.WriteString(FAbbrev);
   aWriter.WriteBoolean(FSelected);
   aWriter.WriteFloat(Scale);
end;

function  TMapSpace.GetPageWidth: double;
begin
   result := Width / Scale;
end;

function  TMapSpace.GetPageHeight: double;
begin
   result := Height / Scale;
end;

function  TMapSpace.GetTopMapSpace: TMapSpace;
begin
  result:= nil
end;
function  TMapSpace.GetBottomMapSpace: TMapSpace;
begin
  result:= nil
end;
function  TMapSpace.GetLeftMapSpace: TMapSpace;
begin
  result:= nil
end;
function  TMapSpace.GetRightMapSpace: TMapSpace;
begin
  result:= nil
end;
function  TMapSpace.GetTopMSName: string;
begin
  result:= ''
end;
function  TMapSpace.GetBottomMSName: string;
begin
  result:= ''
end;
function  TMapSpace.GetLeftMSName: string;
begin
  result:= ''
end;
function  TMapSpace.GetRightMSName: string;
begin
  result:= ''
end;

function  TMapSpace.GetTopMSFullName: string;
begin
  result:= ''
end;
function  TMapSpace.GetBottomMSFullName: string;
begin
  result:= ''
end;
function  TMapSpace.GetLeftMSFullName: string;
begin
  result:= ''
end;
function  TMapSpace.GetRightMSFullName: string;
begin
  result:= ''
end;




function  TMapSpace.GetScale: double;
begin
  result:= 0;
end;


procedure  TMapSpace.SetScale(aValue: double);
begin
end;



function  TMapSpace.GetSMO: TSMO;
begin
   result := TSMO(FindClassInOwners(TSMO));
end;

procedure TMapSpace.MakeDetail;
begin

end;


function  TMapSpace.GetSelfCopyWithDetail: TPersistent;
var
   MS: TMapSpace;
begin
   MS := TMapSpace(Self.ClassType.Create);
   MS.Assign(Self);
   MS.Name := Name;
   MS.Abbrev := Abbrev;
   MS.Number := Number;
   MS.MakeDetail;
   result := TPersistent(MS);
end;


function  TMapSpace.GetFullName: string;
var
   E: TEntity;
begin
   result := '';
   E := FindClassInOwners(TSMO_50);
   if E = nil then exit;
   result := TSMO_50(E).Name + '  '+ ColRowText;
end;


function  TMapSpace.GetColRowText: string;
begin
   result := '';
end;



function  TMapSpace.GetAbbrev: string;
begin
   result := FAbbrev;
end;

procedure TMapSpace.SetAbbrev(aValue: string);
begin
    FAbbrev := aValue;
end;


   { TUserMapSpace }
function  TUserMapSpace.GetScale: double;
begin
  result:= FScale
end;


procedure  TUserMapSpace.SetScale(aValue: double);
begin
  FScale:= aValue
end;


{ TSMO_2 }

function  TSMO_2.GetTopMapSpace: TMapSpace;
begin
   result := SMO.SMO_2[CenterX, CenterY + SMO_2_V];
end;

function  TSMO_2.GetBottomMapSpace: TMapSpace;
begin
   result := SMO.SMO_2[CenterX, CenterY - SMO_2_V];
end;

function  TSMO_2.GetLeftMapSpace: TMapSpace;
begin
   result := SMO.SMO_2[CenterX - SMO_2_H, CenterY];
end;

function  TSMO_2.GetRightMapSpace: TMapSpace;
begin
   result := SMO.SMO_2[CenterX + SMO_2_H, CenterY];
end;

function  TSMO_2.GetTopMSName: string;
begin
   result := SMO.SMO_2Name[CenterX, CenterY + SMO_2_V];
end;

function  TSMO_2.GetBottomMSName: string;
begin
   result := SMO.SMO_2Name[CenterX, CenterY - SMO_2_V];
end;

function  TSMO_2.GetLeftMSName: string;
begin
   result := SMO.SMO_2Name[CenterX - SMO_2_H, CenterY];
end;

function  TSMO_2.GetRightMSName: string;
begin
   result := SMO.SMO_2Name[CenterX + SMO_2_H, CenterY];
end;

function  TSMO_2.GetTopMSFullName: string;
begin
   result := SMO.SMO_2FullName[CenterX, CenterY + SMO_2_V];
end;

function  TSMO_2.GetBottomMSFullName: string;
begin
   result := SMO.SMO_2FullName[CenterX, CenterY - SMO_2_V];
end;

function  TSMO_2.GetLeftMSFullName: string;
begin
   result := SMO.SMO_2FullName[CenterX - SMO_2_H, CenterY];
end;

function  TSMO_2.GetRightMSFullName: string;
begin
   result := SMO.SMO_2FullName[CenterX + SMO_2_H, CenterY];
end;


function  TSMO_2.GetColRowText: string;
var
   s5: TSMO_5;
begin
   result := '';
   s5 := TSMO_5(findClassInOwners(TSMO_5));
   if s5 = nil then exit;
   result := s5.ColRowText + ' / ' + IntToStr(Number);
end;




procedure  TSMO_2.Load(aReader: TReader; aIndex: TEntity);
begin
   inherited Load(aReader, aIndex);
   aReader.Read(FCol, SizeOf(FCol) * 2);
end;


procedure  TSMO_2.Save(aWriter: TWriter);
begin
   inherited Save(aWriter);
   aWriter.Write(FCol, SizeOf(FCol) * 2);
end;


function  TSMO_2.GetScale: double;
begin
  result:= 2000;
end;


function  TSMO_2.Create_SMO_10: TSMO_10;
begin
   result := nil;
end;



function TSMO_2.GetFullName: string;
begin
   result := inherited GetFullName;
end;

{ TSMO_5 }

function  TSMO_5.GetTopMapSpace: TMapSpace;
begin
   result := SMO.SMO_5[CenterX, CenterY + SMO_5_V];
end;

function  TSMO_5.GetBottomMapSpace: TMapSpace;
begin
   result := SMO.SMO_5[CenterX, CenterY - SMO_5_V];
end;

function  TSMO_5.GetLeftMapSpace: TMapSpace;
begin
   result := SMO.SMO_5[CenterX - SMO_5_H, CenterY];
end;

function  TSMO_5.GetRightMapSpace: TMapSpace;
begin
   result := SMO.SMO_5[CenterX + SMO_5_H, CenterY];
end;

function  TSMO_5.GetTopMSName: string;
begin
   result := SMO.SMO_5Name[CenterX, CenterY + SMO_5_V];
end;

function  TSMO_5.GetBottomMSName: string;
begin
   result := SMO.SMO_5Name[CenterX, CenterY - SMO_5_V];
end;

function  TSMO_5.GetLeftMSName: string;
begin
   result := SMO.SMO_5Name[CenterX - SMO_5_H, CenterY];
end;

function  TSMO_5.GetRightMSName: string;
begin
   result := SMO.SMO_5Name[CenterX + SMO_5_H, CenterY];
end;

function  TSMO_5.GetTopMSFullName: string;
begin
   result := SMO.SMO_5FullName[CenterX, CenterY + SMO_5_V];
end;

function  TSMO_5.GetBottomMSFullName: string;
begin
   result := SMO.SMO_5FullName[CenterX, CenterY - SMO_5_V];
end;

function  TSMO_5.GetLeftMSFullName: string;
begin
   result := SMO.SMO_5FullName[CenterX - SMO_5_H, CenterY];
end;

function  TSMO_5.GetRightMSFullName: string;
begin
   result := SMO.SMO_5FullName[CenterX + SMO_5_H, CenterY];
end;


function  TSMO_5.GetColRowText: string;
begin
   result := IntToStr(Col)+'-'+IntToStr(Row);
end;




procedure  TSMO_5.Load(aReader: TReader; aIndex: TEntity);
begin
   inherited Load(aReader, aIndex);
   aReader.Read(FCol, SizeOf(FCol) * 2);
end;


procedure  TSMO_5.Save(aWriter: TWriter);
begin
   inherited Save(aWriter);
   aWriter.Write(FCol, SizeOf(FCol) * 2);
end;


function  TSMO_5.GetScale: double;
begin
  result:= 5000;
end;



function TSMO_5.Add_SMO_2_ByCR(aCol, aRow: integer): TSMO_2;
var
   d: double;
begin
   result := Get_SMO_2_ByCR(aCol, aRow);
   if assigned(result) then exit;
   result := Create_SMO_2;
   if aCol = 0 then d := MaxX
               else d := MaxX - SMO_2_H;
   result.Rect.MaxX := d;
   d := d - SMO_2_H;
   result.Rect.MinX := d;

   if aRow = 0 then d := MaxY
               else d := MaxY - SMO_2_V;
   result.Rect.MaxY := d;
   d := d - SMO_2_V;
   result.Rect.MinY   := d;
   result.Col    := aCol;
   result.Row    := aRow;
   if aCol = 0 then
      if aRow = 0 then result.Number := 2
                  else result.Number := 4
   else
      if aRow = 0 then result.Number := 1
                  else result.Number := 3;

   result.Name   := Name+' / '+IntToStr(result.Number);
   result.Abbrev := abbrev;
   AddSpace(result);
end;

function TSMO_5.Create_SMO_2: TSMO_2;
begin
   result := TSMO_2.Create;
end;

function TSMO_5.Get_SMO_2_ByCR(aCol, aRow: integer): TSMO_2;
var
   i, j: longInt;
   Col: TSpaces;
   aSMO_2: TSMO_2;
begin
   result := nil;
   for i := 0 to ColumnCount -1 do
   begin
      Col := Column[i];
      if not Col.NoSpaces and (TSMO_2(Col.Space[0]).Col = aCol) then
      begin
         for j := 0 to Col.Count -1 do
         begin
            aSMO_2 := TSMO_2(Col.Space[j]);
            if aSMO_2.Row = aRow then
            begin
               result := aSMO_2;
               exit;
            end;
         end;
      end;
   end;
end;

function TSMO_5.GetSelfCopyWithDetail: TPersistent;
var
   MS: TSMO_5;
begin
   MS := TSMO_5(Self.ClassType.Create);
   MS.Assign(Self);
   MS.Name := Name;
   MS.Abbrev := Abbrev;
   MS.Number := Number;
   MS.Col := Col;
   MS.Row := Row;
   MS.MakeDetail;
   MS.Name := ' ';
   result := TPersistent(MS);
end;

procedure TSMO_5.MakeDetail;
var
   H, V: integer;
begin
   for H := 0 to 1 do
   begin
      for V := 0 to 1 do
      begin
         Add_SMO_2_ByCR(H, V);
      end;
   end;

end;

{ TSMO_10 }


function  TSMO_10.GetTopMapSpace: TMapSpace;
begin
   result := SMO.SMO_10[CenterX, CenterY + SMO_10_V];
end;

function  TSMO_10.GetBottomMapSpace: TMapSpace;
begin
   result := SMO.SMO_10[CenterX, CenterY - SMO_10_V];
end;

function  TSMO_10.GetLeftMapSpace: TMapSpace;
begin
   result := SMO.SMO_10[CenterX - SMO_10_H, CenterY];
end;

function  TSMO_10.GetRightMapSpace: TMapSpace;
begin
   result := SMO.SMO_10[CenterX + SMO_10_H, CenterY];
end;

function  TSMO_10.GetTopMSName: string;
begin
   result := SMO.SMO_10Name[CenterX, CenterY + SMO_10_V];
end;

function  TSMO_10.GetBottomMSName: string;
begin
   result := SMO.SMO_10Name[CenterX, CenterY - SMO_10_V];
end;

function  TSMO_10.GetLeftMSName: string;
begin
   result := SMO.SMO_10Name[CenterX - SMO_10_H, CenterY];
end;

function  TSMO_10.GetRightMSName: string;
begin
   result := SMO.SMO_10Name[CenterX + SMO_10_H, CenterY];
end;

function  TSMO_10.GetTopMSFullName: string;
begin
   result := SMO.SMO_10FullName[CenterX, CenterY + SMO_10_V];
end;

function  TSMO_10.GetBottomMSFullName: string;
begin
   result := SMO.SMO_10FullName[CenterX, CenterY - SMO_10_V];
end;

function  TSMO_10.GetLeftMSFullName: string;
begin
   result := SMO.SMO_10FullName[CenterX - SMO_10_H, CenterY];
end;

function  TSMO_10.GetRightMSFullName: string;
begin
   result := SMO.SMO_10FullName[CenterX + SMO_10_H, CenterY];
end;


function  TSMO_10.GetSelfCopyWithDetail: TPersistent;
var
   MS: TSMO_10;
begin
   MS := TSMO_10(Self.ClassType.Create);
   MS.Assign(Self);
   MS.Name := Name;
   MS.Abbrev := Abbrev;
   MS.Number := Number;
   MS.Col := Col;
   MS.Row := Row;
   MS.MakeDetail;
   MS.Name := ' ';
   result := TPersistent(MS);
end;





function  TSMO_10.GetScale: double;
begin
  result:= 10000;
end;



function  TSMO_10.Create_SMO_5: TSMO_5;
begin
   result := TSMO_5.Create;
end;

function  TSMO_10.Get_SMO_5_ByCR(aCol, aRow: integer): TSMO_5;
var
   i, j: longInt;
   Col: TSpaces;
   aSMO_5: TSMO_5;
begin
   result := nil;
   for i := 0 to ColumnCount -1 do
   begin
      Col := Column[i];
      if not Col.NoSpaces and (TSMO_5(Col.Space[0]).Col = aCol) then
      begin
         for j := 0 to Col.Count -1 do
         begin
            aSMO_5 := TSMO_5(Col.Space[j]);
            if aSMO_5.Row = aRow then
            begin
               result := aSMO_5;
               exit;
            end;
         end;
      end;
   end;
end;


procedure TSMO_10.MakeDetail;
var
   H, V: integer;
begin
   for H := Col to Col + 1 do
   begin
      for V := Row to Row + 1 do
      begin
         Add_SMO_5_ByCR(H, V);
      end;
   end;

end;


function TSMO_10.Add_SMO_5_ByCR(aCol, aRow: integer): TSMO_5;
var
   d: double;
begin
   result := Get_SMO_5_ByCR(aCol, aRow);
   if assigned(result) then exit;
   result := Create_SMO_5;
   if aCol mod 2 = 0 then d := MaxX
                     else d := MaxX - SMO_5_H;
   result.Rect.MaxX := d;
   d := d - SMO_5_H;
   result.Rect.MinX := d;

   if aRow mod 2 = 0 then d := MaxY
                     else d := MaxY - SMO_5_V;
   result.Rect.MaxY := d;
   d := d - SMO_5_V;
   result.Rect.MinY   := d;
   result.Col    := aCol;
   result.Row    := aRow;
   result.Name   := Abbrev+' '+IntToStr(aCol)+'-'+IntToStr(aRow);
   result.Abbrev := abbrev;
   AddSpace(result);
end;

{
function  TSMO_10.Create_SMO_50: TSMO_50;
begin
end;
}





{ TSMO_25 }



function  TSMO_25.GetTopMapSpace: TMapSpace;
begin
   result := SMO.SMO_25[CenterX, CenterY + SMO_25_V];
end;

function  TSMO_25.GetBottomMapSpace: TMapSpace;
begin
   result := SMO.SMO_25[CenterX, CenterY - SMO_25_V];
end;

function  TSMO_25.GetLeftMapSpace: TMapSpace;
begin
   result := SMO.SMO_25[CenterX - SMO_25_H, CenterY];
end;

function  TSMO_25.GetRightMapSpace: TMapSpace;
begin
   result := SMO.SMO_25[CenterX + SMO_25_H, CenterY];
end;

function  TSMO_25.GetTopMSName: string;
begin
   result := SMO.SMO_25Name[CenterX, CenterY + SMO_25_V];
end;

function  TSMO_25.GetBottomMSName: string;
begin
   result := SMO.SMO_25Name[CenterX, CenterY - SMO_25_V];
end;

function  TSMO_25.GetLeftMSName: string;
begin
   result := SMO.SMO_25Name[CenterX - SMO_25_H, CenterY];
end;

function  TSMO_25.GetRightMSName: string;
begin
   result := SMO.SMO_25Name[CenterX + SMO_25_H, CenterY];
end;


function  TSMO_25.GetTopMSFullName: string;
begin
   result := SMO.SMO_25FullName[CenterX, CenterY + SMO_25_V];
end;

function  TSMO_25.GetBottomMSFullName: string;
begin
   result := SMO.SMO_25FullName[CenterX, CenterY - SMO_25_V];
end;

function  TSMO_25.GetLeftMSFullName: string;
begin
   result := SMO.SMO_25FullName[CenterX - SMO_25_H, CenterY];
end;

function  TSMO_25.GetRightMSFullName: string;
begin
   result := SMO.SMO_25FullName[CenterX + SMO_25_H, CenterY];
end;


function  TSMO_25.GetColRowText: string;
begin
   result := '25-' + intToStr(Number);
end;






function  TSMO_25.GetScale: double;
begin
  result:= 25000;
end;



procedure TSMO_25.Make5Detail;
var
  aSMO: TSMO;
  aSMO50: TSMO_50;
  H, V: integer;
  MS5: TSMO_5;
begin
   aSMO:= SMO;
   if aSMO = nil then
      exit;
   aSMO50:= aSMO.GetSMO_50(MinX + 10, MinY+10);
   if aSMO50=nil then
      exit;

   for H := Col to Col + 4 do
   begin
      for V := Row to Row + 4 do
      begin
        MS5:= aSMO50.Get_SMO_5_ByCR(H, V);
        AddElement(MS5);
      end;
   end;
end;



function  TSMO_25.GetSelfCopyWithDetail: TPersistent;
var
   MS: TSMO_25;
begin
   MS := TSMO_25(Self.ClassType.Create);
   MS.Assign(Self);
   MS.Name := Name;
   MS.Abbrev := Abbrev;
   MS.Number := Number;
   MS.Col:= Col;
   MS.Row:= Row;
   SMO.AddElement(MS);
   MS.Make5Detail;
   MS.Name := ' ';
   result := TPersistent(MS);
end;





{ TSMO_50 }



function  TSMO_50.GetSelfCopyWithDetail10: TPersistent;
var
   MS: TSMO_50;
begin
   MS := TSMO_50(Self.ClassType.Create);
   MS.Assign(Self);
   MS.Name := Name;
   MS.Abbrev := Abbrev;
   MS.Number := Number;
   MS.Col:= Col;
   MS.Row:= Row;
   MS.Make10Detail;
   MS.Name := ' ';
   result := TPersistent(MS);
end;



function  TSMO_50.GetSelfCopyWithDetail25: TPersistent;
var
   MS: TSMO_50;
begin
   MS := TSMO_50(Self.ClassType.Create);
   MS.Assign(Self);
   MS.Name := Name;
   MS.Abbrev := Abbrev;
   MS.Number := Number;
   MS.Col:= Col;
   MS.Row:= Row;
   MS.Make25Detail;
   MS.Name := ' ';
   result := TPersistent(MS);
end;





function  TSMO_50.Create_SMO_10: TSMO_10;
begin
   result := TSMO_10.Create;
end;



function  TSMO_50.GetTopMapSpace: TMapSpace;
begin
   result := SMO.SMO_50[CenterX, CenterY + SMO_50_V];
end;

function  TSMO_50.GetBottomMapSpace: TMapSpace;
begin
   result := SMO.SMO_50[CenterX, CenterY - SMO_50_V];
end;

function  TSMO_50.GetLeftMapSpace: TMapSpace;
begin
   result := SMO.SMO_50[CenterX - SMO_50_H, CenterY];
end;

function  TSMO_50.GetRightMapSpace: TMapSpace;
begin
   result := SMO.SMO_50[CenterX + SMO_50_H, CenterY];
end;

function  TSMO_50.GetTopMSName: string;
begin
   result := SMO.SMO_50Name[CenterX, CenterY + SMO_50_V];
end;

function  TSMO_50.GetBottomMSName: string;
begin
   result := SMO.SMO_50Name[CenterX, CenterY - SMO_50_V];
end;

function  TSMO_50.GetLeftMSName: string;
begin
   result := SMO.SMO_50Name[CenterX - SMO_50_H, CenterY];
end;

function  TSMO_50.GetRightMSName: string;
begin
   result := SMO.SMO_50Name[CenterX + SMO_50_H, CenterY];
end;


function  TSMO_50.GetTopMSFullName: string;
begin
   result := SMO.SMO_50FullName[CenterX, CenterY + SMO_50_V];
end;

function  TSMO_50.GetBottomMSFullName: string;
begin
   result := SMO.SMO_50FullName[CenterX, CenterY - SMO_50_V];
end;

function  TSMO_50.GetLeftMSFullName: string;
begin
   result := SMO.SMO_50FullName[CenterX - SMO_50_H, CenterY];
end;

function  TSMO_50.GetRightMSFullName: string;
begin
   result := SMO.SMO_50FullName[CenterX + SMO_50_H, CenterY];
end;


function  TSMO_50.GetColRowText: string;
begin
   result := '';
end;






function  TSMO_50.GetScale: double;
begin
  result:= 50000;
end;

function  TSMO_50.Create_SMO_25: TSMO_25;
begin
   result := TSMO_25.Create;
end;


function  TSMO_50.Get_SMO_10_ByCR(aCol, aRow: integer): TSMO_10;
var
   i, j: longInt;
   Col: TSpaces;
   aSMO_10: TSMO_10;
begin
   result := nil;
   aCol := aCol - (aCol mod 2);
   aRow := aRow - (aRow mod 2);
   for i := 0 to ColumnCount -1 do
   begin
      Col := Column[i];
      if not Col.NoSpaces and (TSMO_10(Col.Space[0]).Col = aCol) then
      begin
         for j := 0 to Col.Count -1 do
         begin
            aSMO_10 := TSMO_10(Col.Space[j]);
            if aSMO_10.Row = aRow then
            begin
               result := aSMO_10;
               exit;
            end;
         end;
      end;
   end;
end;


function  TSMO_50.Get_SMO_5_ByCR(aCol, aRow: integer): TSMO_5;
begin
   try
      result := Get_SMO_10_ByCR(aCol, aRow).Get_SMO_5_ByCR(aCol, aRow);
   except
      result := nil;
   end;
end;


function TSMO_50.Add_SMO_10_ByCR(aCol, aRow: integer): TSMO_10;
var
   d: double;
begin
   if aCol mod 2 > 0  then dec(aCol);
   if aRow mod 2 > 0  then dec(aRow);

   result := Get_SMO_10_ByCR(aCol, aRow);
   if assigned(result) then exit;
   result := Create_SMO_10;

   d := MaxX - (aCol div 2 * SMO_10_H);
   result.Rect.MaxX := d;
   d := d - SMO_10_H;
   result.Rect.MinX := d;
   d := MaxY - (aRow div 2 * SMO_10_V);
   result.Rect.MaxY := d;
   d := d - SMO_10_V;
   result.Rect.MinY := d;

   result.Col := aCol;
   result.Row := aRow;
   result.Name   := Abbrev+' '+IntToStr(aCol)+'-'+IntToStr(aRow);;
   result.Abbrev := abbrev;
   AddSpace(result);
end;


function TSMO_50.Add_SMO_5_ByCR(aCol, aRow: integer): TSMO_5;
begin
   result := Add_SMO_10_ByCr(aCol, aRow).Add_SMO_5_ByCr(aCol, aRow);
end;


procedure TSMO_50.Add_SMO_10(aSMO_10: TSMO_10);
begin
   AddSpace(aSMO_10);
end;



function  TSMO_50.Get25From5(aCol, aRow: integer): integer;
var
  Vlevo, Nahore: boolean;
begin
  Vlevo:= aCol > 4;
  Nahore:= aRow < 5;
  if Vlevo and Nahore then result:= 1
  else
  if not Vlevo and Nahore then result:= 2
  else
  if Vlevo and not Nahore then result:= 3
  else
    result:= 4
end;



function  TSMO_50.Get_SMO_25_ByCR(aCol, aRow: integer): TSMO_25;
var
  a25No, i: integer;
  aEl: TElement;
begin
   result := nil;
   a25No:= Get25From5(aCol, aRow);

   for i := 0 to count-1 do
   begin
     aEl:= Elements[I];
     if not (aEl is TSMO_25) then continue;
     if TSMO_25(aEl).Number=a25No then
     begin
       result:= TSMO_25(aEl);
       break
     end
   end;
end;


function TSMO_50.Add_SMO_25_ByNo(aNo: integer): TSMO_25;
begin
  result := Create_SMO_25;
  case aNo of
  1:begin { vlevo nahore }
      result.Rect.MaxX := Rect.MinX + SMO_25_H;
      result.Rect.MinX := Rect.MinX;
      result.Rect.MaxY := Rect.MaxY;
      result.Rect.MinY := Rect.MinY + SMO_25_V;
      result.Col:= 5;
      result.Row:= 0;
    end;
  2:begin { vpravo nahore }
      result.Rect.MaxX := Rect.MaxX;
      result.Rect.MinX := Rect.MinX + SMO_25_H;
      result.Rect.MaxY := Rect.MaxY;
      result.Rect.MinY := Rect.MinY + SMO_25_V;
      result.Col:= 0;
      result.Row:= 0;
    end;
  3:begin { vlevo dole }
      result.Rect.MaxX := Rect.MinX + SMO_25_H;
      result.Rect.MinX := Rect.MinX;
      result.Rect.MaxY := Rect.MinY + SMO_25_V;
      result.Rect.MinY := Rect.MinY;
      result.Col:= 5;
      result.Row:= 5;
    end;
  4:begin { vpravo dole }
      result.Rect.MaxX := Rect.MaxX;
      result.Rect.MinX := Rect.MinX + SMO_25_H;
      result.Rect.MaxY := Rect.MinY + SMO_25_V;
      result.Rect.MinY := Rect.MinY;
      result.Col:= 0;
      result.Row:= 5;
    end;
   end;

   result.Number:= aNo;
   result.Name   := Abbrev+'25-'+IntToStr(aNo);;
   result.Abbrev := abbrev;
   if NoElements then AddElement(result)
                 else InsertElement(0,result);
end;



function TSMO_50.Add_SMO_25_ByCR(aCol, aRow: integer): TSMO_25;
var
  a25No: integer;
begin
  a25No:= Get25From5(aCol, aRow);
  result := Get_SMO_25_ByCR(aCol, aRow);
  if assigned(result) then exit;
  result:= Add_SMO_25_ByNo(a25No)
end;



procedure TSMO_50.Add_SMO_25(aSMO_25: TSMO_25);
begin
   if NoElements then AddElement(aSMO_25)
                 else InsertElement(0,aSMO_25);
end;






procedure TSMO_50.Make10Detail;
var
   H, V: integer;
begin
   for H := 0 to 9 do
   begin
      for V := 0 to 9 do
      begin
         Add_SMO_10_ByCR(H, V);
      end;
   end;
end;


procedure TSMO_50.Make25Detail;
var
   H: integer;
begin
   for H:= 1 to 4 do
   begin
     Add_SMO_25_ByNo(H)
   end;
end;

procedure TSMO_50.MakeDetail;
begin
  Make10Detail;
  Make25Detail;
end;



{ TSMO }

procedure  TSMO.Load(aReader: TReader; aIndex: TEntity);
begin
   inherited Load(aReader, aIndex);
   {
   FWcs := aReader.ReadInteger;
   }
end;


procedure  TSMO.Save(aWriter: TWriter);
begin
   inherited Save(aWriter);
   {
   aWriter.WriteInteger(FWcs);
   }
end;


function    TSMO.Create_SMO_50: TSMO_50;
begin
   result := TSMO_50.Create;
end;


function    TSMO.GeTUserMapSpace(X, Y: double): TUserMapSpace;
begin
   result := TUserMapSpace(FindSpaceByCoordAndClass(X, Y, TUserMapSpace));
end;



function    TSMO.GetSMO_50(X, Y: double): TSMO_50;
begin
   result := TSMO_50(SpaceByCoord[X, Y]);
end;


function    TSMO.GetSMO_25(X, Y: double): TSMO_25;
var
   aSMO_50: TSMO_50;
   C,R: integer;
   Coord: TDoubleCoord;
begin
   result := nil;
   aSMO_50 := TSMO_50(SpaceByCoord[X, Y]);
   if aSMO_50 = nil then exit;

   Coord.x := aSMO_50.MaxX - X;
   Coord.Y := aSMO_50.MaxY - Y;
   C := round(Coord.x) div SMO_5_H;
   R := round(Coord.y) div SMO_5_V;

   result:= aSMO_50.Get_SMO_25_ByCR(C, R);
end;


function    TSMO.GetSMO_10(X, Y: double): TSMO_10;
begin
   result := TSMO_10(FindLevelSpace(x, y, 2));
end;


function    TSMO.GetSMO_5(X, Y: double): TSMO_5;
begin
   result := TSMO_5(FindLevelSpace(x, y, 3));
end;

function TSMO.GetSMO_50Name(aX, aY: double): string;
var
  aMS: TSMO_50;
begin
   try
      aMS := TSMO_50(SpaceByCoord[aX, aY]);
      if aMS = nil then
         exit;
      result := aMS.Name;
   except
      result := 'Nen';
   end;
end;


function TSMO.GetSMO_25Name(aX, aY: double): string;
var
   aSMO_50: TSMO_50;
   C,R: integer;
   a25No: integer;
   Coord: TDoubleCoord;
begin
   result := '';
   aSMO_50 := TSMO_50(SpaceByCoord[aX, aY]);
   if aSMO_50 = nil then exit;

   Coord.x := aSMO_50.MaxX - aX;
   Coord.Y := aSMO_50.MaxY - aY;
   C := round(Coord.x) div SMO_5_H;
   R := round(Coord.y) div SMO_5_V;
   a25No:= Get25From5(C,R);

   result := aSMO_50.Abbrev+'25-'+IntToStr(a25No);
end;




function TSMO.GetSMO_10Name(aX, aY: double): string;
var
   aSMO_50: TSMO_50;
   C,R: integer;
   Coord: TDoubleCoord;
begin
   result := '';
   aSMO_50 := TSMO_50(SpaceByCoord[aX, aY]);
   if aSMO_50 = nil then exit;

   Coord.x := aSMO_50.MaxX - aX;
   Coord.Y := aSMO_50.MaxY - aY;
   C := round(Coord.x) div SMO_5_H;
   R := round(Coord.y) div SMO_5_V;

   if C mod 2 > 0  then dec(C);
   if R mod 2 > 0  then dec(R);

   result := aSMO_50.Abbrev+' '+IntToStr(C)+'-'+IntToStr(R);
end;



function TSMO.GetSMO_5Name (aX, aY: double): string;
var
   aSMO_50: TSMO_50;
   C,R: integer;
   Coord: TDoubleCoord;
begin
   result := '';
   aSMO_50 := TSMO_50(SpaceByCoord[aX, aY]);
   if aSMO_50 = nil then exit;

   Coord.x := aSMO_50.MaxX - aX;
   Coord.Y := aSMO_50.MaxY - aY;
   C := round(Coord.x) div SMO_5_H;
   R := round(Coord.y) div SMO_5_V;

   result := aSMO_50.Abbrev+' '+IntToStr(C)+'-'+IntToStr(R);
end;

function TSMO.GetSMO_50FullName(aX, aY: double): string;
begin
   try
      result := TSMO_50(SpaceByCoord[aX, aY]).FullName;
   except
      result := 'Nen';
   end;
end;

function TSMO.GetSMO_25FullName(aX, aY: double): string;
var
   aSMO_50: TSMO_50;
   C,R: integer;
   a25No: integer;
   Coord: TDoubleCoord;
begin
   result := '';
   aSMO_50 := TSMO_50(SpaceByCoord[aX, aY]);
   if aSMO_50 = nil then exit;

   Coord.x := aSMO_50.MaxX - aX;
   Coord.Y := aSMO_50.MaxY - aY;
   C := round(Coord.x) div SMO_5_H;
   R := round(Coord.y) div SMO_5_V;
   a25No:= Get25From5(C,R);

   result := aSMO_50.FullName + '25-'+IntToStr(a25No);
end;



function TSMO.GetSMO_10FullName(aX, aY: double): string;
var
   aSMO_50: TSMO_50;
   C,R: integer;
   Coord: TDoubleCoord;
begin
   result := '';
   aSMO_50 := TSMO_50(SpaceByCoord[aX, aY]);
   if aSMO_50 = nil then exit;

   Coord.x := aSMO_50.MaxX - aX;
   Coord.Y := aSMO_50.MaxY - aY;
   C := round(Coord.x) div SMO_5_H;
   R := round(Coord.y) div SMO_5_V;

   if C mod 2 > 0  then dec(C);
   if R mod 2 > 0  then dec(R);

   result := aSMO_50.FullName+'  '+IntToStr(C)+'-'+IntToStr(R);
end;



function TSMO.GetSMO_5FullName (aX, aY: double): string;
var
   aSMO_50: TSMO_50;
   C,R: integer;
   Coord: TDoubleCoord;
begin
   result := '';
   aSMO_50 := TSMO_50(SpaceByCoord[aX, aY]);
   if aSMO_50 = nil then exit;

   Coord.x := aSMO_50.MaxX - aX;
   Coord.Y := aSMO_50.MaxY - aY;
   C := round(Coord.x) div SMO_5_H;
   R := round(Coord.y) div SMO_5_V;

   result := aSMO_50.FullName+'  '+IntToStr(C)+'-'+IntToStr(R);
end;

procedure    TSMO.GetSMONamesByCoord(aX, aY: double; var Name_50, Name_25, Name_10, Name_5, Name_2: string);
var
   aSMO_50: TSMO_50;
   C,R,CC,RR: integer;
   Coord: TDoubleCoord;
   i: integer;
begin
   Name_50 := ' ';
   Name_10 := ' ';
   Name_5  := ' ';
   Name_2  := ' ';
   aSMO_50 := TSMO_50(SpaceByCoord[aX, aY]);
   if aSMO_50 = nil then exit;

   Name_50 := aSMO_50.Name;

   Coord.x := aSMO_50.MaxX - aX;
   Coord.Y := aSMO_50.MaxY - aY;
   C := round(Coord.x) div SMO_5_H;
   R := round(Coord.y) div SMO_5_V;
   CC := C;
   RR := R;
   Name_25 := IntToStr(Get25From5(C,R));

   Name_5 := aSMO_50.Abbrev+' '+IntToStr(C)+'-'+IntToStr(R);

   if C mod 2 > 0  then dec(C);
   if R mod 2 > 0  then dec(R);

   Name_10 := aSMO_50.Abbrev+' '+IntToStr(C)+'-'+IntToStr(R);

   Coord.x := (aSMO_50.MaxX-CC*SMO_5_H) - aX;
   Coord.Y := (aSMO_50.MaxY-RR*SMO_5_V) - aY;
   C := round(Coord.x) div SMO_2_H;
   R := round(Coord.y) div SMO_2_V;
   if C = 0 then
      if R = 0 then i := 2
               else i := 4
   else
      if R = 0 then i := 1
               else i := 3;
   Name_2 := ' / '+ IntToStr(i);
end;



function    TSMO.Get_SMO_50_ByName(aName: string): TSMO_50;
var
   i, j: longInt;
   Col: TSpaces;
begin
   result := nil;
   for i := 0 to ColumnCount - 1 do
   begin
      Col := Column[i];
      for j := 0 to Col.Count - 1 do
      begin
         if SameText(TSMO_50(Col.Space[j]).Name,aName) then
         begin
            result := TSMO_50(Col.Space[j]);
            exit;
         end;
      end;
   end;
end;

function    TSMO.Get_SMO_50_ByAbbrev(aAbbrev: string): TSMO_50;
var
   i, j: longInt;
   Col: TSpaces;
begin
   result := nil;
   for i := 0 to ColumnCount - 1 do
   begin
      Col := Column[i];
      for j := 0 to Col.Count - 1 do
      begin
         if TSMO_50(Col.Space[j]).Abbrev = aAbbrev then
         begin
            result := TSMO_50(Col.Space[j]);
            exit;
         end;
      end;
   end;
end;


function    TSMO.Get_SMO_25_ByName(aName: string): TSMO_25;
begin
   result := nil;
end;

function    TSMO.Get_SMO_25_ByAbbrev(aAbbrev: string): TSMO_25;
var
   i, j: longInt;
   Col: TSpaces;
begin
   result := nil;
   for i := 0 to ColumnCount - 1 do
   begin
      Col := Column[i];
      for j := 0 to Col.Count - 1 do
      begin
         if TSMO_25(Col.Space[j]).Abbrev = aAbbrev then
         begin
            result := TSMO_25(Col.Space[j]);
            exit;
         end;
      end;
   end;
end;




function    TSMO.Get_SMO_10_ByName(aName: string; aCol, aRow: integer): TSMO_10;
begin
   result := nil;
end;

function    TSMO.Get_SMO_10_ByAbbrev(aAbbrev: string; aCol, aRow: integer): TSMO_10;
begin
   result := nil;
end;

function    TSMO.Get_SMO_5_ByName(aName: string; aCol, aRow: integer): TSMO_5;
begin
   result := nil;
end;

function    TSMO.Get_SMO_5_ByAbbrev(aAbbrev: string; aCol, aRow: integer): TSMO_5;
begin
   result := nil;
end;



constructor TSMO.Create;
begin
   inherited Create;
   FWCS := TWCSWorld(World).GetIDWCS(cs_JTSK);
end;


function  TSMO.CreateEnvelope: TElement;
begin
   result := Self;
end;


procedure TSMO.Create_SMO_50_ByScript(aScript: string);
var
   TF: TextFile;
   S: String;
   HPos, VPos : integer;
   ShortNamePos, NamePos: integer;
   i: integer;
   {---}
   procedure SkipWhiteSpace;
   begin
      while (S[i] = ' ') or (S[i] = #9)do Inc(i);
   end;
   {---}
   procedure AddSmoByString;
   var
      aH, aV: integer;
   begin
     aH := StrToInt(copy(S, HPos, 2));
     aV := StrToInt(copy(S, VPos, 2));
     Add_SMO_50_ByName(aH, aV, Copy(S, ShortNamePos, 4), Copy(S, NamePos, 30));
   end;
   {---}
begin
   try
      AssignFile(TF, aScript);
      Reset(TF);
      Readln(TF, S); { peskoen prvnho dku }
      Readln(TF, S);
      i := 1;
      { Find the positions of arguments }
      SkipWhiteSpace;
      HPos := i;         { pozice sloupce }
      Inc(i, 2);
      SkipWhiteSpace;
      VPos := i;         { pozice dku }
      Inc(i, 2);
      SkipWhiteSpace;
      ShortNamePos := i; { pozice zkratky }
      Inc(i, 4);
      SkipWhiteSpace;
      NamePos := i;      { pozice nzvu }
      AddSmoByString;

      while not Eof(TF) do
      begin
         Readln(TF, S);
         AddSmoByString;
      end;

   finally
      Close(TF);
   end;
end;

procedure TSMO.Add_SMO_50(aSMO_50: TSMO_50);
begin
   AddSpace(aSMO_50);
end;

procedure TSMO.Add_SMO_50_ByName(aCol, aRow: integer; const aAbbrev, aName: string);
var
   x,y: double;
   SMO_50: TSMO_50;
begin
   y := SMO_50_H;
   x := SMO_50_V;
   y := y* aCol;
   x := x* aRow;
   SMO_50 := Create_SMO_50;
   SMO_50.Rect.Min.x := y;
   SMO_50.Rect.Min.y := x;
   SMO_50.Rect.Max.x := y + SMO_50_H;
   SMO_50.Rect.Max.y := x + SMO_50_V;
   WCS.RecalculateToICS(SMO_50.Rect.Min);
   WCS.RecalculateToICS(SMO_50.Rect.Max);
   SMO_50.Rect.Sort;
   SMO_50.Name   := aName;
   SMO_50.Abbrev := aAbbrev;
   Add_SMO_50(SMO_50);
end;



procedure TSMO.Add_SMO_25(aSMO_25: TSMO_25);
begin
   AddSpace(aSMO_25);
end;


(*
procedure TSMO.Add_SMO_25_ByName(aCol, aRow: integer; const aAbbrev, aName: string);
var
   x,y: double;
   SMO_25: TSMO_25;
begin
   y := SMO_25_H;
   x := SMO_25_V;
   y := y* aCol;
   x := x* aRow;
   SMO_25 := Create_SMO_25;
   SMO_25.Rect.Min.x := y;
   SMO_25.Rect.Min.y := x;
   SMO_25.Rect.Max.x := y + SMO_50_H;
   SMO_25.Rect.Max.y := x + SMO_50_V;
   WCS.RecalculateToICS(SMO_25.Rect.Min);
   WCS.RecalculateToICS(SMO_25.Rect.Max);
   SMO_25.Rect.Sort;
   SMO_25.Name   := aName;
   SMO_25.Abbrev := aAbbrev;
   Add_SMO_25(SMO_25);
end;
*)




{
procedure TSMO.Add_SMO_10(aSMO_10: TSMO_10);
begin
end;

procedure TSMO.Add_SMO_5(aSMO_5: TSMO_5);
begin
end;

}
procedure TSMO.Add_SMO_10_ByAbbrev(aCol, aRow: integer; const aAbbrev: string);
begin
end;

procedure TSMO.Add_SMO_5_ByAbbrev(aCol, aRow: integer; const aAbbrev: string);
var
   SMO_50: TSMO_50;
begin
   SMO_50 := Get_SMO_50_ByAbbrev(aAbbrev);
   if not assigned(SMO_50) then exit;
   SMO_50.Add_SMO_5_ByCR(aCol, aRow);
end;


procedure TSMO.Add_SMO_2_ByAbbrev(aCol, aRow: integer;
  const aAbbrev: string);
begin

end;

function TSMO.Get_SMO_2_ByAbbrev(aAbbrev: string; aCol,
  aRow: integer): TSMO_2;
begin

end;

function TSMO.Get_SMO_2_ByName(aName: string; aCol, aRow: integer): TSMO_2;
begin

end;

function TSMO.GetSMO_2(aX, aY: double): TSMO_2;
begin

end;

function TSMO.GetSMO_2FullName(aX, aY: double): string;
var
   aSMO_50: TSMO_50;
   C,R: integer;
   Coord: TDoubleCoord;
   i: integer;
begin
   result := '';
   aSMO_50 := TSMO_50(SpaceByCoord[aX, aY]);
   if aSMO_50 = nil then exit;

   Coord.x := aSMO_50.MaxX - aX;
   Coord.Y := aSMO_50.MaxY - aY;
   C := round(Coord.x) div SMO_5_H;
   R := round(Coord.y) div SMO_5_V;

   result := aSMO_50.FullName+'  '+IntToStr(C)+'-'+IntToStr(R);
   Coord.x := (aSMO_50.MaxX-C*SMO_5_H) - aX;
   Coord.Y := (aSMO_50.MaxY-R*SMO_5_V) - aY;
   C := round(Coord.x) div SMO_2_H;
   R := round(Coord.y) div SMO_2_V;
   if C = 0 then
      if R = 0 then i := 2
               else i := 4
   else
      if R = 0 then i := 1
               else i := 3;
   result := result + ' / '+ IntToStr(i);
end;


function TSMO.GetSMO_2Name(aX, aY: double): string;
var
   aSMO_50: TSMO_50;
   C,R: integer;
   Coord: TDoubleCoord;
   i: integer;
begin
   result := '';
   aSMO_50 := TSMO_50(SpaceByCoord[aX, aY]);
   if aSMO_50 = nil then exit;

   Coord.x := aSMO_50.MaxX - aX;
   Coord.Y := aSMO_50.MaxY - aY;
   C := round(Coord.x) div SMO_5_H;
   R := round(Coord.y) div SMO_5_V;

   result := aSMO_50.Abbrev+' '+IntToStr(C)+'-'+IntToStr(R);

   Coord.x := (aSMO_50.MaxX-C*SMO_5_H) - aX;
   Coord.Y := (aSMO_50.MaxY-R*SMO_5_V) - aY;
   C := round(Coord.x) div SMO_2_H;
   R := round(Coord.y) div SMO_2_V;
   if C = 0 then
      if R = 0 then i := 2
               else i := 4
   else
      if R = 0 then i := 1
               else i := 3;
   result := result + ' / '+ IntToStr(i);
end;

procedure TSMO.SetSMO50ByScript(const Value: string);
begin
   Create_SMO_50_ByScript(Value);
end;

procedure TSMO.SetAutoCreate50(const Value: boolean);
var
   S :string;
begin
   if not Value then exit;
   Create_SMO_50_ByScript(GetAbsolutePath('MAPLSMO.TXT'));
end;

procedure TSMO.SetCreate10DetailByEntity(const Value: TEntity);
var
   RE: TRectEntity;
   ci,si: integer;
   aColumn: TSpaces;
   ST: TSpaceTree;
   _SMO50: TSMO_50;
   _SMO10: TSMO_10;
   H,V: integer;
begin
   if not (Value is TRectEntity) then exit;
   RE := TRectEntity(Value);
   if Matrix.NoSpaces then exit;
   for ci := 0 to ColumnCount - 1 do
   begin
      aColumn := Column[ci];
      if aColumn.IsOverRegion(RE.MinX,RE.MinY,RE.MaxX,RE.MaxY) then
      for si := 0 to aColumn.Count - 1 do
      begin
         ST := aColumn.Space[si];
         if not (ST is TSMO_50) then continue;
         _SMO50 := TSMO_50(ST);
         if _SMO50.IsOverRegion(RE.MinX,RE.MinY,RE.MaxX,RE.MaxY) then
         begin
            _SMO50.Make10Detail;
            for H := 9 downto 0 do
            begin
               for V := 9 downto 0 do
               begin
                  _SMO10 := _SMO50.Get_SMO_10_ByCR(H,V);
                  if _SMO10 = nil then continue;
                  if not _SMO10.IsOverRegion(RE.MinX,RE.MinY,RE.MaxX,RE.MaxY) then
                     _SMO10.Free;
               end;
            end;
            //_SMO50.FreeEmptySpaces;
         end;
      end;
   end;
end;

procedure TSMO.SetCreate5DetailByEntity(const Value: TEntity);
var
   RE: TRectEntity;
   ci,si: integer;
   aColumn: TSpaces;
   ST: TSpaceTree;
   _SMO50: TSMO_50;
   _SMO10: TSMO_10;
   _SMO5: TSMO_5;
   H,V,H5,V5: integer;
begin
   if not (Value is TRectEntity) then exit;
   RE := TRectEntity(Value);
   if Matrix.NoSpaces then exit;
   for ci := 0 to ColumnCount - 1 do
   begin
      aColumn := Column[ci];
      if aColumn.IsOverRegion(RE.MinX,RE.MinY,RE.MaxX,RE.MaxY) then
      for si := 0 to aColumn.Count - 1 do
      begin
         ST := aColumn.Space[si];
         if not (ST is TSMO_50) then continue;
         _SMO50 := TSMO_50(ST);
         if _SMO50.IsOverRegion(RE.MinX,RE.MinY,RE.MaxX,RE.MaxY) then
         begin
            _SMO50.Make10Detail;
            for H := 9 downto 0 do
            begin
               for V := 9 downto 0 do
               begin
                  _SMO10 := _SMO50.Get_SMO_10_ByCR(H,V);
                  if _SMO10 = nil then continue;
                  if not _SMO10.IsOverRegion(RE.MinX,RE.MinY,RE.MaxX,RE.MaxY) then
                     _SMO10.Free
                  else
                  begin
                     _SMO10.MakeDetail;
                     for H5 := Col+1 downto Col do
                     begin
                        for V5 := Row + 1 downto Row do
                        begin
                           _SMO5 := _SMO10.Get_SMO_5_ByCR(H5, V5);
                           if _SMO5 = nil then continue;
                           if not _SMO5.IsOverRegion(RE.MinX,RE.MinY,RE.MaxX,RE.MaxY) then
                              _SMO5.Free
                        end;
                     end;
                  end;
               end;
            end;
            //_SMO50.FreeEmptySpaces;
         end;
      end;
   end;
end;

{ TZMSpace }

function TZMSpace.GetAngle: double;
begin
   result := ZM.Angle;
end;

function TZMSpace.GetRAngle: double;
begin
   result := -Angle;
end;

function TZMSpace.GetScale: double;
begin
   result := ZM.Scale;
end;

function TZMSpace.GetZM: TMapsZM;
begin
   if FZM = nil then
   begin
      FZM := TMapsZM(FIndOrCreateElementName(TMapsZM,'ZM'));
   end;
   result := FZM;
end;

procedure TZMSpace.SetAngle(const Value: double);
begin
   // blbost;
end;

procedure TZMSpace.SetScale(aValue: double);
begin
   ZM.Scale := round(aValue);
end;

procedure TZMSpace.SetZM(const Value: TMapsZM);
begin
   if Value = nil then exit;
   ZM.Scale := Value.Scale;
   ZM.Map200 := Value.Map200;
   ZM.Map100 := Value.Map100;
   ZM.Map50 := Value.Map50;
   ZM.Map25 := Value.Map25;
   ZM.Map10 := Value.Map10;
   ZM.Color := Value.Color;
   ZM.PenWidth := Value.PenWidth;
   ZM.PenStyle := Value.PenStyle;
end;

INITIALIZATION
   RegisterOrbisClass(TMapSpace);
   RegisterOrbisClass(TUserMapSpace);
   RegisterOrbisClass(TZMSpace);
   RegisterOrbisClass(TSMO_2);
   RegisterOrbisClass(TSMO_5);
   RegisterOrbisClass(TSMO_10);
   RegisterOrbisClass(TSMO_25);
   RegisterOrbisClass(TSMO_50);
   RegisterOrbisClass(TSMO);
END.
