unit E_prnpg;
{
*********************************************************************
*                                                                   *
*  ORBIS ... Inteligentn entita tiskovch strnek pro mapper       *
*                                                                   *
*********************************************************************
}
interface

uses
// KERNEL
   Elements,
// GLIB
   g_Elems,
// DEVICE
   e_Device,
// MAPPER
   e_Desktp;

type


TPrintPages = class(TRectEntity)
private
   FDesktop: TDesktop;
   FScale: double;
   FPrintDevice: TDevice;
   FPages: TRectEntity;
   FRowNo, FColNo: longInt;
   FOverlap: double;
   FCentering: boolean;
    FMultiPage: boolean;
   function  GetDesktop: TDesktop;
   function  GetScale: double;
   function  GetPages: TRectEntity;
   function  GetPagesCount: longInt;
   function  GetOverlap: double;
   procedure SetDesktop(aValue: TDesktop);
   procedure SetScale(aValue: double);
   procedure SetPages(aValue: TRectEntity);
   procedure SetOverlap(aValue: double);
   procedure AddPageOfSize(aXMin, aYMin, aWidth, aHeight: double);
   procedure PreparePages;
   function  GetPrintPage(aIndex: longInt): TDrawRectangle;
   procedure SetCentering(aValue: boolean);
    procedure SetMultiPage(const Value: boolean);
public
   procedure FreePages; virtual;
   procedure AddPage(aValue: TDrawRectangle);
   procedure CenterPages; virtual;

   property  Desktop: TDesktop read GetDesktop write SetDesktop;
   property  Scale: double read GetScale write SetScale;
   property  PrintDevice: TDevice read FPrintDevice write FPrintDevice;
   property  Pages: TRectEntity read GetPages write SetPages;
   property  PagesCount: longInt read GetPagesCount;
   property  PrintPage[aIndex: longInt]: TDrawRectangle read GetPrintPage;
   property  RowNo: longInt read FRowNo;
   property  ColNo: longInt read FColNo;
   property  Overlap: double read GetOverlap write SetOverlap;
   property  Centering: boolean read FCentering write SetCentering;
   property  MultiPage: boolean read FMultiPage write SetMultiPage;
end;


implementation

uses
   Graphics,
   Printers_f;


function  TPrintPages.GetDesktop: TDesktop;
begin
   result := FDesktop;
end;

function  TPrintPages.GetScale: double;
begin
   if FScale <= 0 then FScale := 1;
   result := FScale;
end;


function  TPrintPages.GetPages: TRectEntity;
begin
   if FPages = nil then
   begin
      FPages := TRectEntity.Create;
      AddElement(FPages);
   end;
   result := FPages;
end;

function  TPrintPages.GetPagesCount: longInt;
begin
   result := Pages.Count;
end;

function  TPrintPages.GetOverlap: double;
begin
   result := FOverlap;
end;


procedure TPrintPages.PreparePages;
var
  PgWidth, PgHeight,{ rozmry v relu ne papru }
  MapWidth, MapHeight, aMinX, aMinY, Temp: double;
  Row, Col: longInt;
begin
  if FPages <> nil then
  begin
     FreeElement(FPages);
     FPages:= nil;
  end;

  Rect.Reset;

  if Desktop = nil then exit;
  if Scale = 0 then exit;
  if PrintDevice = nil then exit;

 { Nastav se strana zmenen o pekryv. Pekryv se pak vykresl
   automaticky, nebo se dokresl vdy ve a do kraje }
  PgWidth := (PrintDevice.RealWidth -  Overlap)*Scale;
  PgHeight:= (PrintDevice.RealHeight - Overlap)*Scale;


  MapWidth := Desktop.Width;
  MapHeight:= Desktop.Height;
  Temp := MapWidth  / PgWidth ;
  FColNo := trunc(Temp);
  if (Temp - FColNo) > 0.001 then inc(FColNo);
  Temp := MapHeight / PgHeight;
  FRowNo := trunc(Temp);
  if (Temp - FRowNo) > 0.001 then inc(FRowNo);
  if MultiPage then
  begin
     FRowNo := 1;
     FColNo := 1;
  end;

  aMinY:= DeskTop.MinY;
  for Row:= 1 to FRowNo do
  begin
    aMinX:= DeskTop.MinX;
    for Col:= 1 to FColNo do
    begin
       AddPageOfSize(aMinX - Overlap, aMinY - Overlap, PgWidth + Overlap, PgHeight + Overlap);
       aMinX:= aMinX + PgWidth;
    end;
    aMinY:= aMinY + PgHeight;
  end;
  if Centering then CenterPages;
end;

function  TPrintPages.GetPrintPage(aIndex: longInt): TDrawRectangle;
begin
   result := TDrawRectangle(Pages.Elements[aIndex]);
end;

procedure TPrintPages.SetCentering(aValue: boolean);
begin
   if aValue <> FCentering then
   begin
      FCentering := aValue;
      PreparePages;
   end;
end;


procedure TPrintPages.SetDesktop(aValue: TDesktop);
begin
   FDesktop := aValue;
   PreparePages
end;

procedure TPrintPages.SetScale(aValue: double);
begin
   FScale := aValue;
   PreparePages
end;


procedure TPrintPages.SetPages(aValue: TRectEntity);
begin
   if FPages <> nil then
   begin
      FreeElement(FPages);
      FPages := nil;
   end;
   FPages := aValue;
   AddElement(FPages);
end;

procedure TPrintPages.FreePages;
begin
   Pages.FreeAll;
end;


procedure TPrintPages.AddPageOfSize(aXMin, aYMin, aWidth, aHeight: double);
var
  RE: TDrawRectangle;
begin
  RE:= TDrawRectangle.Create;
  RE.MinX:= aXMin;
  RE.MinY:= aYMin;
  RE.Width:= aWidth;
  RE.Height:= aHeight;
  RE.PenColor:= clBlack;
  RE.PenStyle:= psSolid;
  RE.BrushStyle := bsSolid;
  RE.BrushColor:= clWhite;
  AddPage(RE);
end;



procedure TPrintPages.AddPage(aValue: TDrawRectangle);
begin
   Pages.AddAndUpdateRect(aValue);
   UpdateRect(aValue);
end;


procedure TPrintPages.CenterPages;
var
   cdX, cdY: double;
   i: longInt;
begin
   cdX := Desktop.Centerx - CenterX;
   cdY := Desktop.Centery - CenterY;
   for i := 0 to PagesCount - 1 do PrintPage[i].Move(cdX, cdY);
   Pages.ResetAndUpdate;
   ResetAndUpdate
end;




procedure TPrintPages.SetOverlap(aValue: double);
begin
   FOverlap := aValue;
   PreparePages
end;



procedure TPrintPages.SetMultiPage(const Value: boolean);
begin
  FMultiPage := Value;
end;

end.
