unit e_ThematicMap;

interface
uses
   i_Basic,
   e_InterpretContainer,
   Elements,
   g_Elems;
type

TMapProfile = class(TOwnerEntity)
  private
    FTime: string;
    FMapName: string;
    FComment: string;
    procedure SetComment(const Value: string);
    procedure SetMapName(const Value: string);
    procedure SetTime(const Value: string);
published
   property MapName: string read FMapName write SetMapName;
   property Comment: string read FComment write SetComment;
   property Time: string read FTime write SetTime;
end;

TThematicMapEntity = class(TRectEntity)
private
   FLayersPath: string;
   FLayers: TEntity;
    FComment: string;
   procedure SetLayers(const Value: TEntity);
   procedure SetLayersPath(const Value: string);
    procedure SetComment(const Value: string);
    procedure SetClearIt(const Value: boolean);
public
   function  FindEntity(const Value: string): TEntity;
   procedure SetReset(Value: boolean); virtual;
   procedure Clear; virtual;
   property  Layers: TEntity read FLayers write SetLayers;
published
   property LayersPath: string read FLayersPath write SetLayersPath;
   property Reset: boolean write SetReset stored false;
   property ClearIt: boolean write SetClearIt stored false ;
   property Comment: string read FComment write SetComment;
end;

TThematicLayer = class(TInterpretLayer)
private
   FLayerName: string;
   FComment: string;
    FSimpleLines: boolean;
    FLinePoints: boolean;
    FTextSize: double;
    FPropertyName: string;
    FItemsString: string;
    FFillBackGround: boolean;
    FNoColor: boolean;
    FSetupScript: string;
   procedure SetComment(const Value: string);
   procedure SetLayerName(const Value: string);
    procedure SetLinePoints(const Value: boolean);
    procedure SetSimpleLines(const Value: boolean);
    procedure SetPropertyName(const Value: string);
    procedure SetTextSize(const Value: double);
    procedure SetItemsString(const Value: string);
    procedure SetFillBackGround(const Value: boolean);
    procedure SetNoColor(const Value: boolean);
    procedure SetSetupScript(const Value: string);
protected
   procedure SetInterpretSimpleLines(const aInterpret: TElement);
   procedure SetInterpretLinePoints(const aInterpret: TElement);
   procedure SetInterpretTextProps(const aInterpret: TElement);
   procedure SetInterpretFillBackGround(const aInterpret: TElement);
public
   procedure Clear; virtual;
   procedure SetInterpret(const Value: TInterpret); override;
   function  IsLinesLayer: boolean;
   procedure SetAsText(const Value: Boolean); override;
published
   property LayerName: string read FLayerName write SetLayerName;
   property Comment: string read FComment write SetComment;
   property ItemsString: string read FItemsString write SetItemsString;
{ TODO -oPPK -cDodlat : Zrevidovat pi finalizaci prce s mapami - tohle je nouzovka }
   property SimpleLines: boolean read FSimpleLines write SetSimpleLines;
   property LinePoints: boolean read FLinePoints write SetLinePoints;
   property PropertyName: string read FPropertyName write SetPropertyName;
   property TextSize: double read FTextSize write SetTextSize;
   property FillBackGround: boolean read FFillBackGround write SetFillBackGround;
   property NoColor: boolean read FNoColor write SetNoColor;
   property SetupScript: string read FSetupScript write SetSetupScript;
end;

TThematicConnection = class(TThematicMapEntity)
  private
    FConnectionType: integer;
    procedure SetConnectionType(const Value: integer);
public
   procedure SetReset(Value: boolean); override;
   procedure Clear; override;
published
   // connection Type 0 - connect whole TThematicMaps elements ... fe for reset
   //                 1 - connect Thematic layers of these maps
   property ConnectionType: integer read FConnectionType write SetConnectionType;
end;

TThematicMap = class(TThematicMapEntity)
private
    FUserMap: boolean;
   procedure SetNewLayer(const Value: string);
    procedure SetUserMap(const Value: boolean);
    function GetLastLayer: TThematicLayer;
public
   procedure SetReset(Value: boolean); override;
   procedure Clear; override;
published
   property NewLayer: string write SetNewLayer stored false;
   property UserMap: boolean read FUserMap write SetUserMap;
   property LastLayer: TThematicLayer read GetLastLayer stored false;
end;

TThematicMaps = class(TThematicMapEntity)
private
    FUserMapsPath: string;
    FMapName: string;
    FVersion: string;
   procedure SetNewMap(const Value: string);
    procedure SetUserMapsPath(const Value: string);

    procedure SetMapName(const Value: string);
    function GetLastMap: TThematicMap;
    function GetActualMap: TThematicMap;
    procedure SetConnectMaps(const Value: TElement);
    procedure SetDisConnectMaps(const Value: TElement);
    procedure SetVersion(const Value: string);
public
   procedure SetReset(Value: boolean); override;
   procedure Clear; override;
published
   property Version: string read FVersion write SetVersion;
   property NewMap: string write SetNewMap stored false;
   property UserMapsPath: string read FUserMapsPath write SetUserMapsPath;
   property MapName: string read FMapName write SetMapName;
   property LastMap: TThematicMap read GetLastMap stored false;
   property ActualMap: TThematicMap read GetActualMap stored false;
   property ConnectMaps: TElement write SetConnectMaps stored false;
   property DisConnectMaps: TElement write SetDisConnectMaps stored false;
end;

implementation
uses
   SysUtils,
   e_Class,
   m_Orbis,
   t_CmdLn;

{ TThematicMapEntity }

procedure TThematicMapEntity.Clear;
begin

end;

function TThematicMapEntity.FindEntity(const Value: string): TEntity;
var
   OIL: TCommandLine;
   E: TElement;
begin
   result := nil;
   OIL := TCommandLine.Create;
   OIL.Line := Value;
   OIL.DoIt;
   E := TElement(StrToIntDef(OIL.ResultLine, 0));
   OIL.Free;
   if not (E is TEntity) then exit;
   result := TEntity(E);
end;


procedure TThematicMapEntity.SetClearIt(const Value: boolean);
begin
   if value then
      Clear;
end;

procedure TThematicMapEntity.SetComment(const Value: string);
begin
  FComment := Value;
end;

procedure TThematicMapEntity.SetLayers(const Value: TEntity);
begin
   FLayers := Value;
end;

procedure TThematicMapEntity.SetLayersPath(const Value: string);
begin
   FLayersPath := Value;
end;

procedure TThematicMapEntity.SetReset(Value: boolean);
begin
   ResetAndUpdate;
end;

{ TThematicMaps }

procedure TThematicMaps.Clear;
var
   i: integer;
   e: TElement;
   M: TThematicMap;
begin
   for i := LastIndex downto 0 do
   begin
      e := Elements[i];
      if e is TThematicConnection then
      begin
         TThematicConnection(e).Clear;
         continue;
      end;
      if not (e is TThematicMap) then continue;
      M := TThematicMap(e);
      //if M.UserMap then
      //   FreeElement(M)
      //else
         TThematicMap(e).Clear;
   end;
end;

function TThematicMaps.GetActualMap: TThematicMap;
begin
   result := TThematicMap(FindElementName(MapName));
end;

function TThematicMaps.GetLastMap: TThematicMap;
begin
   result := nil;
   if NoElements then exit;
   result := TThematicMap(Elements[LastIndex]);
end;

procedure TThematicMaps.SetConnectMaps(const Value: TElement);
var
   En: TEntity;
   e: TELement;
   MS: TThematicMaps;
   M,MM: TThematicMap;
   L,LL: TThematicLayer;
   C: TThematicConnection;
   i,j,k: integer;
   LastCIndex: integer;
   sConnect: string;
   S: string;
begin
   if not (Value is TThematicMaps) then exit;
   MS := TThematicMaps(Value);
   if MS.NoElements then exit;
   sConnect := '_CONNECT';
   for i := 0 to MS.LastIndex do
   begin
      e := MS.Elements[i];
      if not (e is TThematicMap) then continue;
      M := TThematicMap(e);
      e := FindElementName(M.Name);
      if e = nil then
      begin
         NewMap := M.Name;
         MM := LastMap;
      end
      else
         MM := TThematicMap(e);
      //
      LastCIndex := 0;
      for j := 0 to M.LastIndex do
      begin
         e := M.Elements[j];
         if not (e is TThematicLayer) then continue;
         L := TThematicLayer(e);
         S :=L.Name+sConnect;
         if L.AsText then S := S+'_txt';
         e := MM.FindElementName(S);
         if (e <> nil) and (e is TThematicConnection) then
            C := TThematicConnection(e)
         else
         begin
            C := TThematicConnection.Create;
            C.Name := S;
            C.ConnectionType := 1;
            e := MM.FindElementName(L.Name);
            for k := 0 to MM.LastIndex do
            begin
               e := MM.Elements[k];
               if not (e is TThematicLayer) then
               begin
                  e := nil;
                  continue;
               end;
               LL := TThematicLayer(e);
               if SameText(L.Name,LL.Name) and (LL.AsText = L.AsText) then
                  break
               else
                  e := nil;
            end;
            if (e <> nil) and (e is TThematicLayer) then
            begin
               LastCIndex := MM.IndexOf(e);
               MM.InsertElement(LastCIndex,C);
               Inc(LastCIndex,2);
            end
            else
            begin
               if LastCIndex > MM.LastIndex then
                 MM.AddElement(C)
               else
                 MM.InsertElement(LastCIndex,C);
               Inc(LastCIndex);
            end;
         end;
         if not L.IsMyOwner(C) then
            C.AddElement(L);
      end;
   end;
   // connect maps
   C := TThematicConnection(FindOrCreateElementName(TThematicConnection,sConnect));
   if not MS.IsMyOwner(C) then
      C.AddElement(MS);
end;

procedure TThematicMaps.SetDisConnectMaps(const Value: TElement);
var
   En: TEntity;
   e: TELement;
   M,MM: TThematicMap;
   MS: TThematicMaps;
   C: TThematicConnection;
   L: TThematicLayer;
   i,j: integer;
begin
   if not (Value is TThematicMaps) then exit;
   MS := TThematicMaps(Value);
   if MS.NoElements then exit;
   for i := 0 to MS.LastIndex do
   begin
      e := MS.Elements[i];
      if not (e is TThematicMap) then continue;
      M := TThematicMap(e);
      e := FindElementName(M.Name);
      if e = nil then  continue;
      MM := TThematicMap(e);
      for j := 0 to M.LastIndex do
      begin
         e := M.Elements[j];
         if not (e is TThematicLayer) then continue;
         L := TThematicLayer(e);
         En := L.FindClassWithOwnerInOwners(TThematicConnection,MM);
         En.ElementToFree := L;
      end;
   end;
   C := TThematicConnection(FindElementClass(TThematicConnection));
   if C <> nil then
      C.AddElement(MS);
end;

procedure TThematicMaps.SetMapName(const Value: string);
begin
   FMapName := Value;
end;

procedure TThematicMaps.SetNewMap(const Value: string);
var
   M: TThematicMap;
begin
   M := TThematicMap.Create;
   M.Name := Value;
   M.LayersPath := LayersPath;
   AddElement(M);
end;

procedure TThematicMaps.SetReset(Value: boolean);
var
   i: integer;
   e: TElement;
   En: TEntity;
   M: TThematicMap;
begin
   if not Value then exit;
   for i := 0 to LastIndex do
   begin
      e := Elements[i];
      if not (e is TThematicMap) then continue;
      TThematicMap(e).Reset := true;
   end;
   inherited;
   En := FindEntity(UserMapsPath);
   if En = nil then exit;
   for i := 0 to En.LastIndex do
   begin
      e := En.Elements[i];
      if not (e is TThematicMap) then continue;
      M := TThematicMap(e);
      M.UserMap := true;
      M.Reset := true;
      AddAndUpdateRect(M);
   end;
end;

procedure TThematicMaps.SetUserMapsPath(const Value: string);
begin
   FUserMapsPath := Value;
end;

procedure TThematicMaps.SetVersion(const Value: string);
begin
  FVersion := Value;
end;

{ TThematicMap }

procedure TThematicMap.Clear;
var
   i: integer;
   e: TElement;
   L: TThematicLayer;
begin
   for i := LastIndex downto 0 do
   begin
      e := Elements[i];
      if e is TThematicConnection then
      begin
         if TThematicConnection(e).ConnectionType < 1 then
            FreeElement(e)
         else
            TThematicConnection(e).Clear;
      end
      else
      if e is TThematicLayer then
      begin
        L := TThematicLayer(e);
        L.Clear;
      end;
   end;
end;

function TThematicMap.GetLastLayer: TThematicLayer;
begin
   result := nil;
   if NoElements then exit;
   result := TThematicLayer(Elements[LastIndex]);
end;

procedure TThematicMap.SetNewLayer(const Value: string);
var
   L: TThematicLayer;
begin
   L := TThematicLayer.Create;
   L.Name := Value;
   L.LayerName := Value;
   AddElement(L);
end;

procedure TThematicMap.SetReset(Value: boolean);
var
   i: integer;
   e: TElement;
   L: TThematicLayer;
begin
   Layers := FindEntity(LayersPath);
   if Layers = nil then
   begin
      PostTextMsg(tm_AddToProtocol,'ThematicMap '+Name+': Layers entity '+ LayersPath+' not found',nil);
   end;
   for i := 0 to LastIndex do
   begin
      e := Elements[i];
      if not (e is TThematicLayer) then continue;
      L := TThematicLayer(e);
      L.FreeAll;
      if Layers <> nil then
         L.AddElement(Layers.FindElementName(L.LayerName));
      L.ResetAndUpdate;
   end;
   inherited;
end;

procedure TThematicMap.SetUserMap(const Value: boolean);
begin
  FUserMap := Value;
end;

{ TThematicLayer }

procedure TThematicLayer.Clear;
begin
   FreeAll;
end;

function TThematicLayer.IsLinesLayer: boolean;
begin
  result :=  LayerType = 'LIN';
end;

procedure TThematicLayer.SetAsText(const Value: Boolean);
begin
   inherited;
   if not Value then exit;
   InterpretName := 'TGeneralTextInterpret';
   SetInterpretTextProps(Interpret);
end;

procedure TThematicLayer.SetComment(const Value: string);
begin
  FComment := Value;
end;

procedure TThematicLayer.SetFillBackGround(const Value: boolean);
begin
  FFillBackGround := Value;
  SetInterpretFillBackGround(Interpret);
end;

procedure TThematicLayer.SetInterpret(const Value: TInterpret);
begin
  inherited;
  if AsText then
  begin
     SetInterpretTextProps(Value);
  end
  else
  begin
     SetInterpretLinePoints(Value);
     SetInterpretSimpleLines(Value);
     SetInterpretFillBackGround(Value);
  end;
end;

procedure TThematicLayer.SetInterpretFillBackGround(const aInterpret: TElement);
begin
   if aInterpret = nil then exit;
   if not (aInterpret is TInterpret) then exit;
   aInterpret.SetPropertyBooleanByName('FillBackGround',FillBackGround);
end;

procedure TThematicLayer.SetInterpretLinePoints(const aInterpret: TElement);
begin
   if aInterpret = nil then exit;
   if not (aInterpret is TInterpret) then exit;
   if not IsLinesLayer then exit;
   aInterpret.SetPropertyBooleanByName('LinePoints',LinePoints);
end;

procedure TThematicLayer.SetInterpretSimpleLines(const aInterpret: TElement);
begin
   if aInterpret = nil then exit;
   if not (aInterpret is TInterpret) then exit;
   if not IsLinesLayer then exit;
   aInterpret.SetPropertyBooleanByName('SimpleLines',SimpleLines);
end;

procedure TThematicLayer.SetInterpretTextProps(const aInterpret: TElement);
begin
   if aInterpret = nil then exit;
   if not (aInterpret is TInterpret) then exit;
   if not AsText then exit;
   aInterpret.SetPropertyStringByName('ProcessClassName',LayerClassName);
   aInterpret.SetPropertyStringByName('PropText',PropertyName);
   aInterpret.SetPropertyStringByName('PropX','X');
   aInterpret.SetPropertyStringByName('PropY','Y');
   aInterpret.SetPropertyStringByName('ValHeight',FloatToStr(TextSize));
   aInterpret.SetPropertyStringByName('ValFont','Arial');
   aInterpret.SetPropertyBooleanByName('AbsSize',true);
   aInterpret.SetPropertyBooleanByName('UseColor',true);
end;

procedure TThematicLayer.SetItemsString(const Value: string);
begin
  FItemsString := Value;
end;

procedure TThematicLayer.SetLayerName(const Value: string);
begin
  FLayerName := Value;
end;

procedure TThematicLayer.SetLinePoints(const Value: boolean);
begin
  FLinePoints := Value;
  SetInterpretLinePoints(Interpret);
end;

procedure TThematicLayer.SetNoColor(const Value: boolean);
begin
  FNoColor := Value;
end;

procedure TThematicLayer.SetPropertyName(const Value: string);
begin
  FPropertyName := Value;
  SetInterpretTextProps(Interpret);
end;

procedure TThematicLayer.SetSetupScript(const Value: string);
begin
  FSetupScript := Value;
end;

procedure TThematicLayer.SetSimpleLines(const Value: boolean);
begin
  FSimpleLines := Value;
  SetInterpretSimpleLines(Interpret);
end;

procedure TThematicLayer.SetTextSize(const Value: double);
begin
  FTextSize := Value;
  SetInterpretTextProps(Interpret);
end;

{ TThematicConnection }

procedure TThematicConnection.Clear;
begin
   FreeAll;
end;

procedure TThematicConnection.SetConnectionType(const Value: integer);
begin
  FConnectionType := Value;
end;

procedure TThematicConnection.SetReset(Value: boolean);
var
   i: integer;
   e: TElement;
begin
   if COnnectionType=1 then exit; // LayersConnection
   if NoElements then exit;
   for i := 0 to LastIndex do     // Othervise, connection includes Maps objects of other proj.
   begin
      e := Elements[i];
      if e is TThematicMaps then
         TThematicMaps(e).Reset := true;
   end;
end;

{ TMapProfile }

procedure TMapProfile.SetComment(const Value: string);
begin
  FComment := Value;
end;

procedure TMapProfile.SetMapName(const Value: string);
begin
  FMapName := Value;
end;

procedure TMapProfile.SetTime(const Value: string);
begin
  FTime := Value;
end;

INITIALIZATION
   RegisterOrbisClass(TMapProfile);
   RegisterOrbisClass(TThematicLayer);
   RegisterOrbisClass(TThematicMap);
   RegisterOrbisClass(TThematicConnection);
   RegisterOrbisClass(TThematicMaps);
END.
