unit F_edtext;

interface

uses WinTypes, WinProcs, Classes, Graphics, Forms, Controls, Buttons,
  StdCtrls, ExtCtrls, Spin, Dialogs,
// GLIB  
  g_Elems;

type
  TEditTextDialog = class(TForm)
    OKBtn: TBitBtn;
    CancelBtn: TBitBtn;
    ed_Text: TEdit;
    FontPanel: TEdit;
    SpeedButton1: TSpeedButton;
    FontDialog: TFontDialog;
    ed_Height: TEdit;
    SpinButton1: TSpinButton;
    Label1: TLabel;
    Label2: TLabel;
    Label3: TLabel;
    SpinButton2: TSpinButton;
    Label4: TLabel;
    Combo_Units: TComboBox;
    Bevel1: TBevel;
    Bevel2: TBevel;
    ed_Orientation: TComboBox;
    Label5: TLabel;
    ColorPanel: TPanel;
    Label6: TLabel;
    TestPanel: TPanel;
    ColorDialog: TColorDialog;
    ed_Name: TEdit;
    Label7: TLabel;
    procedure ed_TextChange(Sender: TObject);
    procedure FontPanelChange(Sender: TObject);
    procedure ColorPanelClick(Sender: TObject);
    procedure ed_OrientationChange(Sender: TObject);
    procedure Combo_UnitsChange(Sender: TObject);
    procedure SpinButton2DownClick(Sender: TObject);
    procedure ed_HeightChange(Sender: TObject);
    procedure SpinButton2UpClick(Sender: TObject);
    procedure SpinButton1DownClick(Sender: TObject);
    procedure SpinButton1UpClick(Sender: TObject);
    procedure ed_NameChange(Sender: TObject);
  private
    FTextElement: TText;
    procedure SetTextElement(aValue: TText);
    procedure SetFont;
  public
    property TextElement: TText read FTextElement write SetTextElement;
    procedure SetColor(aValue: TColor);
    procedure SetOrientation(aValue: double);
    procedure SetHeight(aValue: double);
    procedure SetStyle(aValue: TFontStyles);
    procedure SetCharSet(aValue: TFontCharSet);
  end;


implementation

uses
   SysUtils;

{$R *.DFM}

procedure TEditTextDialog.SetColor(aValue: TColor);
begin
   TextElement.Color := aValue;
   ColorPanel.Color := aValue;
   TestPanel.Font.Color := aValue;
end;

procedure TEditTextDialog.SetFont;
begin
   FontDialog.Font.Name:= TextElement.FontName;
   FontDialog.Font.Color:= TextElement.Color;
   FontDialog.Font.Style:= TextElement.FontStyle;
   FontDialog.Font.CharSet:= TextElement.CharSet;
   FontPanel.Text := TextElement.FontName;
   TestPanel.Font := FontDialog.Font;
end;

procedure TEditTextDialog.SetOrientation(aValue: double);
begin
   TextElement.Orientation := aValue;
   ed_Orientation.text := FloatToStrF(aValue, ffFixed, 7, 2);
end;

procedure TEditTextDialog.SetHeight(aValue: double);
var
   s: string;
begin
   TextElement.Height := aValue;
   if Combo_Units.Text = 'mm' then s := FloatToStrF(aValue * 1000, ffFixed, 7, 0) else
   if Combo_Units.Text = 'cm' then s := FloatToStrF(aValue * 100, ffFixed, 5, 2)  else
      s := FloatToStrF(aValue , ffFixed, 2, 3);
   ed_Height.text := s;
end;


procedure TEditTextDialog.SetStyle(aValue: TFontStyles);
begin
  TextElement.FontStyle:= aValue;
  FontDialog.Font.Style := TextElement.FontStyle;
  TestPanel.Font.Style := aValue;
end;

procedure TEditTextDialog.SetCharSet(aValue: TFontCharSet);
begin
  TextElement.CharSet:= aValue;
  FontDialog.Font.CharSet := TextElement.CharSet;
  TestPanel.Font.CharSet := aValue;
end;


procedure TEditTextDialog.SetTextElement(aValue: TText);
begin
   if FTextElement <> nil then
   begin
      FTextElement.Free;
      FTextElement := nil;
   end;
   FTextElement := aValue;
   if FTextElement = nil then exit;
   ed_Name.Text := FTextElement.Name;
   ed_Text.Text := FTextElement.Text;
   SetFont;
   SetColor(FTextElement.Color);
   SetHeight(FTextElement.Height);
   SetOrientation(FTextElement.Orientation);
   SetStyle(FTextElement.FontStyle);
   SetCharSet(FTextElement.CharSet);
   FontDialog.Font.Height := 10;
end;


procedure TEditTextDialog.ed_TextChange(Sender: TObject);
begin
   if TextElement <> nil then TextElement.Text := ed_Text.Text;
end;

procedure TEditTextDialog.FontPanelChange(Sender: TObject);
begin
   FontDialog.Font.Color := TextElement.Color;
   if FontDialog.execute then
   begin
      FontPanel.Text := FontDialog.Font.Name;
      TestPanel.Font := FontDialog.Font;
      SetColor(FontDialog.Font.Color);
      TextElement.Font:= FontDialog.Font;
   end;
end;

procedure TEditTextDialog.ColorPanelClick(Sender: TObject);
begin
   if ColorDialog.Execute then SetColor(ColorDialog.Color);
end;

procedure TEditTextDialog.ed_OrientationChange(Sender: TObject);
var
   d: double;
begin
   try
      d := StrToFloat(ed_Orientation.Text);
      TextElement.Orientation := d;
   except
      SetOrientation(TextElement.Orientation );
   end;
end;

procedure TEditTextDialog.Combo_UnitsChange(Sender: TObject);
begin
   SetHeight(TextElement.Height);
end;

procedure TEditTextDialog.SpinButton2DownClick(Sender: TObject);
begin
   TextElement.Orientation := TextElement.Orientation - 1;
   SetOrientation(TextElement.Orientation);
end;

procedure TEditTextDialog.ed_HeightChange(Sender: TObject);
var
   D: double;
begin
   try
      D := StrToFloat(ed_Height.Text);
      if Combo_Units.Text = 'mm' then D := D/1000 else
         if Combo_Units.Text = 'cm' then D := D/100;
      TextElement.Height := D;
   except
      SetHeight(TextElement.Height);
   end;
end;

procedure TEditTextDialog.SpinButton2UpClick(Sender: TObject);
begin
   TextElement.Orientation := TextElement.Orientation + 1;
   SetOrientation(TextElement.Orientation);
end;

procedure TEditTextDialog.SpinButton1DownClick(Sender: TObject);
begin
   TextElement.Height := TextElement.Height - 0.001;
   SetHeight(TextElement.Height);
end;

procedure TEditTextDialog.SpinButton1UpClick(Sender: TObject);
begin
   TextElement.Height := TextElement.Height + 0.001;
   SetHeight(TextElement.Height);
end;

procedure TEditTextDialog.ed_NameChange(Sender: TObject);
begin
   if TextElement <> nil then TextElement.Name := ed_Name.Text;
end;


end.
