unit I_guide;
{****************************************************************************}
{                                                                            }
{          ORBIS ... Interpret Prvodce                                      }
{                    (c) 1996                                                }
{                                                                            }
{****************************************************************************}

interface
uses
   Elements,
   //i_Top_L,
   i_PolyLineElement,
   i_Basic,
   t_Recurs;

type

  TGuideInterpret = class(TRectInterpret)
  private
    FCnt: integer;
    FShow_50: boolean;
    FShow_25: boolean;
    FShow_10: boolean;
    FShow_5 : boolean;
    FShow_2 : boolean;
    FShow_U : boolean;
    FShow_ZM : boolean;
    FShow_Text: boolean;
    FFill_Page: boolean;
    FLineInterpret: TPolyLineElementInterpret;
  protected
    function  QGoInside(Element: TElement): boolean; override;
    function  QProcessBefore(Element: TElement): boolean; override;
    procedure Process(Element: TElement); override;
    function  IsItsClass(AObject: TObject): boolean; override;
  public
    procedure   DoIt; override;
    procedure   SetSubInterprets;  override;
    procedure   FreeSubInterprets; override;
    function    CanUniGoInside(Element: TElement): boolean; override;
  published
    property  Show_50: boolean read FShow_50 write FShow_50 default false;
    property  Show_25: boolean read FShow_25 write FShow_25 default false;
    property  Show_10: boolean read FShow_10 write FShow_10 default false;
    property  Show_5 : boolean read FShow_5  write FShow_5  default true;
    property  Show_2 : boolean read FShow_2  write FShow_2  default false;
    property  Show_U : boolean read FShow_U  write FShow_U  default true;
    property  Show_ZM : boolean read FShow_ZM  write FShow_ZM  default false;
    property  Show_Text: boolean read FShow_Text write FShow_Text default true;
    property  Fill_Page: boolean  read FFill_Page   write FFill_Page default true;
    property  LineInterpret: TPolyLineElementInterpret read FLineInterpret write FLineInterpret;
  end;


  TSelectGuide = class(TRecursor)
  private
    FSelect50: boolean;
    FSelect25: boolean;
    FSelect10: boolean;
    FSelect5: boolean;
    FSelect2: boolean;
    FSelectU: boolean;
    FOutEntity: TEntity;
    FSelectZM: boolean;
  protected
    function  QGoInside(Element: TElement): boolean; override;
    function  QProcessBefore(Element: TElement): boolean; override;
    procedure Process(Element: TElement); override;
  public
    procedure DoIt; override;
    property  Select50: boolean read FSelect50 write FSelect50;
    property  Select25: boolean read FSelect25 write FSelect25;
    property  Select10: boolean read FSelect10 write FSelect10;
    property  Select5: boolean read FSelect5 write FSelect5;
    property  Select2: boolean read FSelect2 write FSelect2;
    property  SelectU: boolean read FSelectU write FSelectU;
    property  SelectZM: boolean read FSelectZM write FSelectZM;
  end;


  TInvertGuide = class(TSelectGuide)
  protected
    procedure Process(Element: TElement); override;
  end;


implementation
uses
   Graphics, Forms,
   e_Class,
   e_Device,
   e_Guide,
   e_InterpretContainer,
   e_MSpace,
   e_Geometry,
   e_Zm,
   i_SelR,
   g_Elems,
   Tools;



{ TGuideInterpret }



procedure   TGuideInterpret.SetSubInterprets;
begin
   //LineInterpret := TInterpretTopLinii.Create;
   LineInterpret := TPolyLineElementInterpret.Create;
   NextInterpret := LineInterpret;
   //NextInterpret := TPolyLineElementInterpret.Create;
{   LineInterpret.Reset(Device); }
   NextInterpret := TSelectedElementInterpret.Create;
   NextInterpret := TInterpretZM.Create;
end;


procedure   TGuideInterpret.FreeSubInterprets;
begin
   LineInterpret.Free;
   FreeTool(TTool(Tools.FindElementClass(TSelectedElementInterpret)));
   FreeTool(TTool(Tools.FindElementClass(TInterpretZM)));
end;




function  TGuideInterpret.QGoInside(Element: TElement): boolean;
begin
  Result:= not ((Element is TAreaString)or (Element is TZMSpace));
end;


function  TGuideInterpret.QProcessBefore(Element: TElement): boolean;
begin
   Result:= (Element is TMapSpace) or (Element is TPolyLineElement) or (Element is TInterpretLayer)  or
            (Element is TSelectedElement) or (Element is TMapsZM);
end;



procedure TGuideInterpret.Process(Element: TElement);
var
  Rect: TRectangle;
  aColor: TColor;
  aClass: TClass;
  Labels: boolean;
  dX,dY,dd: double;
  ZME: TRectEntity;
  CC: T2DCoordinate;
  i: integer;
begin
    inc(FCnt);
    if FCnt > 500 then
    begin
       Application.ProcessMessages;
       FCnt := 0;
    end;
    if (Element is TPolyLineElement) or (Element is TInterpretLayer) or (Element is TSelectedElement)or (Element is TMapsZM) then
    begin
       if (Element.ClassType = TMapsZM) and not Show_ZM then exit;
       if Element is TInterpretLayer then
       begin
          LineInterpret.Color := TInterpretLayer(Element).Color;
          exit
       end;
       inherited Process(Element);
       exit;
    end;

    if TRectEntity(Element).IsOverRegion(MinX,MinY,MaxX,MaxY)then
    begin
//       Labels := false;
       aClass := Element.ClassType;
       if aClass = TGuidePage_50 then
       begin
           if not Show_50 then exit;
           aColor := clBlack;
           labels := Device.Projection.Scale < 5000000;
       end
       else
       if aClass = TGuidePage_25 then
       begin
           if not Show_25 then exit;
           aColor := clGray;
           labels := Device.Projection.Scale < 2500000;
       end
       else
       if aClass = TGuidePage_10 then
       begin
          if not Show_10 then exit;
          aColor := clRed;
          labels := Device.Projection.Scale < 1000000;
       end
       else
       if aClass = TGuidePage_5  then
       begin
          if not Show_5 then exit;
          aColor := clGreen;
          labels := Device.Projection.Scale < 500000;
       end
       else
       if aClass = TGuidePage_2  then
       begin
          if not Show_2 then exit;
          aColor := clMaroon;
          labels := Device.Projection.Scale < 200000;
       end
       else
       if Element is TUserMapSpace then
       begin
          if not Show_U then exit;
          aColor := clBlue;
          labels := Device.Projection.Scale < 1000000;
       end
       else
       if Element is TZMSpace then
       begin
          if not Show_ZM then exit;
          //aColor := clBlue;
          //labels := Device.Projection.Scale < 1000000;
       end
       else exit;

       Device.Pen.Color := aColor;
       Device.Pen.Width := 1;
       Device.Pen.Style := psSolid;
       Device.Pen.Mode  := pmCopy;
       if Fill_Page then
       begin
          Device.Brush.Style := bsSolid;
          aColor := clWhite;
          if Show_ZM then
          begin
             if (aClass = TZMSpace) then
             begin
                if TZMSpace(Element).Selected then aColor := clYellow;
                Device.Pen.Color :=  TZMSpace(Element).ZM.Color;
                Device.Brush.Color := aColor;
                ZME := TRectEntity.Create;
                for i := 1 to 4 do
                begin
                   CC := T2DCoordinate.Create;
                   CC.X := TZMSpace(Element).ZM.XCorn[i];
                   CC.Y := TZMSpace(Element).ZM.YCorn[i];
                   ZME.AddAndUpdateRect(CC);
                end;
                ZME.ExchangeElements(ZME.Elements[2],ZME.Elements[3]);
                Device.Polygon(ZME);
                ZME.Free;
                exit;
             end;
          end
          else
          if Show_U then
          begin
             if (aClass = TUserMapSpace) and TMapSpace(Element).Selected then aColor := clYellow;
          end
          else
          if Show_2 then
          begin
             if (aClass = TGuidePage_2) and TMapSpace(Element).Selected then aColor := clYellow;
          end
          else
          if Show_5 then
          begin
             if (aClass = TGuidePage_5) and TMapSpace(Element).Selected then aColor := clYellow;
          end
          else
          if Show_10 then
          begin
             if (aClass = TGuidePage_10) and TMapSpace(Element).Selected then aColor := clYellow;
          end
          else
          if Show_25 then
          begin
             if (aClass = TGuidePage_25) and TMapSpace(Element).Selected then
             begin
               aColor := clYellow;
               //Device.Brush.Style := bsDiagCross;
             end;
          end
          else
          if Show_50 then
          begin
             if (aClass = TGuidePage_50) and TMapSpace(Element).Selected then
             begin
               aColor := clYellow;
               //Device.Brush.Style := bsBDiagonal;
             end;
          end;
          Device.Brush.Color := aColor;
       end
       else
       begin
          Device.Brush.Style := bsClear;
       end;
       Rect:= TRectEntity(Element).Rect;
       Device.Rectangle(Rect.Min.X, Rect.Min.Y, Rect.Max.X, Rect.Max.Y);
       if not Show_Text then exit;
       if not Labels then exit;
       Device.Font.Name := 'Arial';
       Device.Font.Color := clBlack;
       Device.Font.Height := Device.Projection.GetPageLengthVert1(Rect.MaxY - Rect.MinY) div 10;
       Device.TextOut(Rect.Min.X, Rect.Max.Y, Element.Name);
       if aClass = TGuidePage_5  then
       begin
          Device.Font.Name := 'Arial';
          Device.Font.Color := clBlue;
          Device.Font.Height := Device.Projection.GetPageLengthVert1(Rect.MaxY - Rect.MinY) div 4;
          dd := SMO_5_V/5;
          dX := Rect.Min.X + dd * 2;
          dY := Rect.Max.Y - dd;
          if TGuidePage_5(Element).CIT then Device.TextOut(dx, dy, 'CIT');
          dy := dy - dd;
          if TGuidePage_5(Element).RAK then Device.TextOut(dx, dy, 'RAK');
          dy := dy - dd;
          if TGuidePage_5(Element).TRAK then Device.TextOut(dx, dy, 'TRAK');
       end
    end;
end;



function TGuideInterpret.IsItsClass(AObject: TObject): boolean;
begin
   result := (AObject is TSMO_2) or (AObject is TPolyLineElement);
end;

function TGuideInterpret.CanUniGoInside(Element: TElement): boolean;
begin
   result := false;
end;


procedure   TGuideInterpret.DoIt;
begin
   Application.ProcessMessages;
   inherited DoIt;
end;



{ TSelectGuide }


function  TSelectGuide.QGoInside(Element: TElement): boolean;
begin
  Result:= true;
end;

function  TSelectGuide.QProcessBefore(Element: TElement): boolean;
begin
  Result:= Element is TMapSpace {TSMO_5};
end;



procedure TSelectGuide.Process(Element: TElement);
var
  aClass: TClass;
begin
   aClass := Element.ClassType;
   if (Element is TSMO_2) then
   begin
   { 10 pouze kdy nen 5, kdy je oboj, tak pouze 5}
     if not SelectU then
     if
      ((aClass = TGuidePage_2) and Select2) or
      ((aClass = TGuidePage_5) and Select5 and not Select2) or
      ((aClass = TGuidePage_10) and Select10 and not (Select5 or Select2)) or
      ((aClass = TGuidePage_25) and Select25 and not(Select2 or Select5 or Select10)) or
      ((aClass = TGuidePage_50) and Select50 and not(Select2 or Select5 or Select10 or Select25))
     then
     begin
        if TMapSpace(Element).Selected then
           FOutEntity.AddElement(Element);
     end
   end
   else
   if Element is TMapSpace then
   begin
      if Element is TZMSpace then
      begin
         if SelectZM and TZMSpace(Element).Selected then
         FOutEntity.AddElement(Element)
      end
      else
      if SelectU and TMapSpace(Element).Selected then
         FOutEntity.AddElement(Element)
   end
   else  exit;
end;



procedure TSelectGuide.DoIt;
begin
   FOutEntity := TEntity.Create;
   inherited DoIt;
   if FOutEntity <> nil then
      Output:= FOutEntity
end;


           { TInvertGuide }


procedure TInvertGuide.Process(Element: TElement);
var
  aClass: TClass;
  SMO: TSMO_2;
  MS: TMapSpace;
begin
   aClass := Element.ClassType;
   if (Element is TSMO_2) then
   begin
     SMO:= TSMO_2(Element);
   { 10 pouze kdy nen 5, kdy je oboj, tak pouze 5}
     if not SelectU then
     if
      ((aClass = TGuidePage_2) and Select2) or
      ((aClass = TGuidePage_5) and Select5 and not Select2) or
      ((aClass = TGuidePage_10) and Select10 and not (Select5 or Select2)) or
      ((aClass = TGuidePage_25) and Select25 and not(Select2 or Select5 or Select10)) or
      ((aClass = TGuidePage_50) and Select50 and not(Select2 or Select5 or Select10 or Select25))
     then
     begin
       if not SMO.Selected then
         FOutEntity.AddElement(Element);
       SMO.Selected:= not SMO.Selected;
     end
   end
   else
   if Element is TMapSpace then
   begin
     if not SelectU then exit;
     MS:= TMapSpace(Element);
     if not MS.Selected then
        FOutEntity.AddElement(Element);
     MS.Selected:= not MS.Selected;
   end
   else  exit;
end;





BEGIN
   RegisterOrbisClass(TGuideInterpret);
END.