unit language;

interface
uses Classes,
     Elements,
     E_Type,
     T_OilOper,
     Tools,
     M_Orbis,
     T_Errors;

const MaxVariables=50;
      MaxVarLength=30;

      WorldIndex=0{poradi Worldu ve stacku};
      MToolIndex=1{poradi MainToolu ve stacku};
      ActiveElIndex=2{poradi Activeelemetu ve stacku};

type
 NameString=string[MaxVarLength];
 TToolClass=class of TTool;

TOilProgram=class;
TOilLine=class;

TOilThread=class(TThread)
  private
    FProgram:TOilProgram;
  protected
    constructor RunProgram(P:TOilProgram);
    procedure Execute; override;
  end;

TOilBreakPoint=class(TOwnerElement)
  private
   FActive:boolean;
   FLineNum:integer;
   FLine:TOilLine;
   FProgram:TOilProgram;
  public
   property Active:boolean read FActive write FActive;
   property LineNum:integer read FLineNum write FLineNum;
   property Line:TOilLine read FLine write FLine;
   property OilProgram:TOilProgram read FProgram write FProgram;
   constructor Create;override;
   procedure Pause;
  end;

TLineMessage = class(TOrbisMessage)
 private
   FLineNum:integer;
published
   property   LineNum: longInt   read FLineNum  write FLineNum;
end;

TErrorMessage = class(TOrbisTextMessage)
 private
   FLineNum:integer;
published
   property   LineNum: longInt   read FLineNum  write FLineNum;
end;

TOilLine=class(TOwnerEntity) {predelat na entitu a pridelat Program caller}
 private
  function GetBreakPoint:TOilBreakPoint;virtual;
  procedure SetBreakPoint(aBPoint:TOilBreakPoint);virtual;
  function GetSourceLine:integer;virtual;
  procedure SetSourceLine(aLine:integer);virtual;
 public
  property BreakPoint:TOilBreakPoint read GetBreakPoint write SetBreakPoint;
  property SourceLineNum:integer read GetSourceLine write SetSourceLine;
  procedure DoIt;virtual;
 end;

TOilBlock=class(TOilLine)
private
 FPrgLineList:TNameEntity;
 FVarInfoList:TNameEntity; {soupis promennych deklarovanych pro tento blok}
 FStackIndex:integer;{prvni misto v staku patrici tomuto bloku}
 FStack:TOwnerEntity;
 procedure SetVariable(index:integer;aVar:TTypeElement);virtual;{asi neni nutne}
 function GetVariable(index:integer):TTypeElement;virtual;
 procedure SetStack(aStack:TOwnerEntity);virtual;{asi neni nutne}
 function GetStack:TOwnerEntity;virtual;
 procedure SetPrgLineList(aList:TNameEntity);virtual;
 function GetPrgLineList:TNameEntity;virtual;
 function GetPrgLine(index:longint):TOilLIne;virtual;
 procedure SetPrgline(index:longint;aLine:TOilLIne);virtual;
 procedure CreateVarsInStack;virtual;
 procedure DestroyVarsInStack;virtual;
 procedure SetVarInfoList(aList:TNameEntity);virtual;
 function GetVarInfoList:TNameEntity;virtual;
 function GetVarType(index:byte):TType;virtual;
 procedure SetVarType(index:byte;aType:TType);virtual;
 function GetVarTypeName(index:byte):string;virtual;
 procedure SetVarTypeName(index:byte;aname:string);virtual;
 property Variable[index:integer]:TTypeElement read GetVariable write SetVariable;
 {pro debug}
 procedure SetNameStack(aStack:TStringListElement);virtual;abstract;
 function GetNameStack:TStringListElement;virtual;abstract;
 function GetVarName(index:integer):string;virtual;
 function GetVarDBInfoName(index:integer):string;virtual;
 procedure SetVarDBInfoName(index:integer;aName:string);virtual;
public
 property Stack:TOwnerEntity read GetStack write SetStack;
 property VarInfoList:TNameEntity read GetVarInfoList write SetVarInfoList;
 property VarType[index:byte]:TType read GetVarType write SetVarType;
 property VarTypeName[index:byte]:string read GetVarTypeName write SetVarTypeName;
 property PrgLinelist:TNameEntity read GetPrgLineList write SetPrgLineList;
 property PrgLine[index:longint]:TOilLIne read GetPrgLine write SetPrgline;
 property StackIndex:integer read FStackIndex write FStackIndex;
 procedure AddPrgLine(aLine:TOilLIne);virtual;
 procedure DoIt;override;
 {pro debug}
 procedure NewDBVarName(aName:string);virtual;{Do name Stacku}
 procedure ClearLastDBVarName;virtual;
 property NameStack:TStringListElement read GetNameStack write SetNameStack;
 property DBVarName[index:integer]:string read GetVarName;{ve namestacku pri behu}
 property VarDBInfoName[index:integer]:string read GetVarDBInfoName write SetVarDBInfoName;{ve varinfu}
end;

TDBOilBlock=class(TOilBlock)
 private
  FNameStack:TStringListElement;
  FBreakPoint:TOilBreakPoint;
  FSourceLine:integer;
  procedure SetNameStack(aStack:TStringListElement);override;
  function GetNameStack:TStringListElement;override;
  function GetBreakPoint:TOilBreakPoint;override;
  procedure SetBreakPoint(aBPoint:TOilBreakPoint);override;
  function GetSourceLine:integer;override;
  procedure SetSourceLine(aLine:integer);override;
  procedure CreateVarsInStack;override;
  procedure DestroyVarsInStack;override;
 public
  procedure DoIt;override;
end;


TParametr=class(TEntity)
private
 FExp:TOilOperator;
 function GetExp:TOilOperator;
 procedure SetExp(aExp:TOilOperator);
public
 VarIndex:longint;
 property Expression:TOilOperator read GetExp write SetExp;
end;

{na vstupu bude ocekavat entitu s TParametry}
TOilProc=class(TOilBlock)
private
 FSubProcs:TEntity;
 FCalls:integer;
 FParams:TEntity;
 procedure ReadParameters;virtual;
 function GetFuncResult:TTypeElement;virtual;
 function GetSubProcs:TEntity;
 procedure SetSubProcs(aEnt:TEntity);
public
 property SubProcs:TEntity read GetSubProcs write SetSubProcs;
 property FuncResult:TTypeElement read GetFuncResult;
 property Parameters:TEntity read FParams write FParams;
 procedure DoIt;override;
published
 property Calls:integer read FCalls write FCalls;
end;

TDBOilProc=class(TOilProc)
 private
  FNameStack:TStringListElement;
  FBreakPoint:TOilBreakPoint;
  FSourceLine:integer;
  procedure SetNameStack(aStack:TStringListElement);override;
  function GetNameStack:TStringListElement;override;
  function GetBreakPoint:TOilBreakPoint;override;
  procedure SetBreakPoint(aBPoint:TOilBreakPoint);override;
  function GetSourceLine:integer;override;
  procedure SetSourceLine(aLine:integer);override;
  procedure CreateVarsInStack;override;
  procedure DestroyVarsInStack;override;
public
  procedure DoIt;override;
end;

TOilCallBackProcedure = procedure of object;

TOilProgram=class(TOilProc)
private
 FThreadForRun:TOilThread;
    FOnEnd: TOilCallBackProcedure;
 procedure DestroyVarsInStack;override;
 procedure HandleError(E:TRunTimeError);virtual;
    function GetRunning: boolean;
    procedure SetOnEnd(const Value: TOilCallBackProcedure);
protected
 procedure Terminating(Sender:TObject);
 procedure Run;virtual;
 procedure CleanUp;virtual;
 {Run a CleanUp pro ucely Threadu}
public
 constructor Create;override;
 procedure Pause;virtual;
 procedure Continue;virtual;
 procedure Terminate;virtual;{nefunguje}
 procedure DoIt;override;
 property  OnEnd: TOilCallBackProcedure read FOnEnd write SetOnEnd;
published
   property Running: boolean read GetRunning;
end;

TDBOilProgram=class(TOilProgram)
private
 FToNotify:TTool;
 FNameStack:TStringListElement;
 FSourceLine:integer;
 FBreakPointList:TEntity;
 procedure SetNameStack(aStack:TStringListElement);override;
 function GetNameStack:TStringListElement;override;
 function GetSourceLine:integer;override;
 procedure SetSourceLine(aLine:integer);override;
 procedure CreateVarsInStack;override;
 procedure DestroyVarsInStack;override;
 function GetBPList:TEntity;virtual;
 procedure SetBPList(aList:TEntity);virtual;
 function GetBPNumber:integer;virtual;
 function GetBPoint(index:integer):TOilBreakPoint;virtual;
 procedure HandleError(E:TRunTimeError);override;
protected
 procedure Run;override;
 procedure CleanUp;override;
public
 procedure Continue;override;
 property BreakPointList:TEntity read GetBPList write SetBPList;
 property BreakPoints:integer read GetBPNumber;
 property BreakPoint[index:integer]:TOilBreakPoint read GetBPoint;
 property ToNotify:TTool read FToNotify write FToNotify;
   {^^^ misto toho by meli jit message}
end;


TVariable=class(TOilOperator)
  private
   function GetConst:TTypeElement;virtual;abstract;
   procedure SetConst(aConst:TTypeElement);virtual;abstract;
   function GetConstTyp:TType;virtual;abstract;
   function GetConstName:string;virtual;abstract;
  public
   procedure DoIt;override;
   procedure GetTypeInfo;override;
   property Constant:TTypeElement read GetConst write SetConst;
   property Typ:TType read GetConstTyp; {mozna zbytecne}
   property TypeName:string read GetConstName;
  end;

TVarInfo=class(TElement)
private
   FType:TType;
   FTypeName:NameString;
published
   property VarType:TType read FType write FType;
   property TypeName:NameString read FTypeName write FTypeName;
end;

TDBVarInfo=class(TVarInfo)
private
   FVarName:string;
public
   property VarName:string read FVarName write FVarName;
end;
TNormalVariable=class(TVariable)
  private
   FVarIndex:integer;
   FBlock:TOilBlock;
   function GetConst:TTypeElement;override;
   procedure SetConst(aConst:TTypeElement);override;
   function GetConstTyp:TType;override;
   function GetConstName:string;override;
  public
   property Block:TOilBlock read FBlock write FBlock;
  published
   property VarIndex:integer read FVarIndex write FVarIndex;
  end;

TCalledVariable=class(TNormalVariable)
private
   function GetConst:TTypeElement;override;
   procedure SetConst(aConst:TTypeElement);override;
   {function GetConstTyp:TType;override;}
 end;


TPropVariable=class(TVariable)
 private
  FInitialExp:TOilOperator; {kdyby bylo vicenasobne dedeni
  					pridal by se TOilPtrOperator}
  FPropName:string;
  function GetConst:TTypeElement;override;
  procedure SetConst(aConst:TTypeElement);override;
  function GetConstTyp:TType;override;
  function GetInitialExp:TOilOperator;
  procedure SetInitialExp(aOp:TOilOperator);
 public
  property InitialExp:TOilOperator read GetInitialExp write SetInitialExp;
 published
  property PropertyName:string read FPropName write FPropName;
 end;

TElementOperator=class(TOilPtrOperator)
private
 FElName:String;
public
  procedure DoIt;override;
  procedure GetTypeInfo;override;
published
  property ElementName:string read FElName write FElName;
  end;

TOwnerOperator=class(TElementOperator)
 public
  procedure DoIt;override;
 end;

TElementByNumberOperator=class(TOilPtrOperator)
private
 FPositionIndex:TOilOperator; {Cislovani jde od 0}
 function GetPositionIndex:TOilOperator;
 procedure SetPositionIndex(aVar:TOilOperator);
public
  procedure DoIt;override;
  procedure GetTypeInfo;override;
published
  property PositionIndex:TOilOperator read GetPositionIndex write SetPositionIndex;
  end;

TOwnerByNumberOperator=class(TElementByNumberOperator)
 public
  procedure DoIt;override;
 end;


TToolInputInfo=class(TEntity)
 private
  FPropIndex:longint;
  FInputExpr:TOilOperator;
  function GetInputExpr:TOilOperator;
  procedure SetInputExpr(aExp:TOilOperator);
 public
  property InputExpr:TOilOperator read GetInputExpr write SetInputExpr;
 published
  property PropIndex:longint read FPropIndex write FPropIndex;
end;

TToolOperator=class(TOilPtrOperator)
 private
  FToolClass:TToolClass;
  FPropInputList:TEntity;
  function GetPropInputList:TEntity;
  procedure SetPropInputList(aList:TEntity);
  function GetInputItem(index:longint):TToolInputinfo;virtual;
 public
  property PropInputList:TEntity read GetPropInputList write SetPropInputList;
  property PropInputItem[index:longint]:TToolInputInfo read GetInputItem;
  property ToolClass:TToolClass read FToolClass write FToolClass;
  procedure DoIt;override;
  procedure GetTypeInfo;override;
 end;

TProtectBlock=class(TOilBlock)
public
 procedure DoIt;override;
end;

TDBProtectBlock=class(TProtectBlock)
 private
  FNameStack:TStringListElement;
  FBreakPoint:TOilBreakPoint;
  FSourceLine:integer;
  procedure SetNameStack(aStack:TStringListElement);override;
  function GetNameStack:TStringListElement;override;
  function GetBreakPoint:TOilBreakPoint;override;
  procedure SetBreakPoint(aBPoint:TOilBreakPoint);override;
  function GetSourceLine:integer;override;
  procedure SetSourceLine(aLine:integer);override;
  procedure CreateVarsInStack;override;
  procedure DestroyVarsInStack;override;
 public
  procedure DoIt;override;
end;


TAssignLine=class(TOilLIne)
private
 FLeftVar:TVariable;
 FExpression:TOilOperator;
 function GetExpr:TOilOperator;
 procedure SetExpr(aExpr:TOilOperator);
 function GetLeftVar:TVariable;
 procedure SetLeftVar(aExpr:TVariable);
public
 procedure DoIt;override;
 property LeftVar:TVariable read GetLeftVar write SetLeftVar;
 property Expression:TOilOperator read GetExpr write Setexpr;
end;

TDBAssignLine=class(TAssignLine)
 private
  FBreakPoint:TOilBreakPoint;
  FSourceLine:integer;
  function GetBreakPoint:TOilBreakPoint;override;
  procedure SetBreakPoint(aBPoint:TOilBreakPoint);override;
  function GetSourceLine:integer;override;
  procedure SetSourceLine(aLine:integer);override;
 public
  procedure DoIt;override;
end;


TExprLine=class(TOilLIne)
private
 FOperator:TOilOperator;
 function GeTOilOperator:TOilOperator;
 procedure SeTOilOperator(aOp:TOilOperator);
public
 property Operator:TOilOperator read GeTOilOperator write SeTOilOperator;
 procedure DoIt;override;
end;

TDBExprLine=class(TExprLine)
 private
  FBreakPoint:TOilBreakPoint;
  FSourceLine:integer;
  function GetBreakPoint:TOilBreakPoint;override;
  procedure SetBreakPoint(aBPoint:TOilBreakPoint);override;
  function GetSourceLine:integer;override;
  procedure SetSourceLine(aLine:integer);override;
 public
  procedure DoIt;override;
end;


TIfLine=class(TOilLIne)
private
 FExpression:{TBooleanOperator}TOilOperator;
 FThenBlock,FElseBlock:TOilLIne;
 function GetExpr:TOilOperator;
 procedure SetExpr(aExpr:TOilOperator);
 function GetThenBlock:TOilLIne;
 procedure SetThenBlock(ablock:TOilLIne);
 function GetElseBlock:TOilLIne;
 procedure SetElseBlock(aBlock:TOilLIne);
 public
 procedure DoIt;override;
 property Expression:TOilOperator read GetExpr write SetExpr;
 property ThenBlock:TOilLIne read GetThenBlock write SetThenBlock;
 property ElseBlock:TOilLIne read GetElseBlock write SetElseBlock;
end;

TDBIfLine=class(TIfLine)
 private
  FBreakPoint:TOilBreakPoint;
  FSourceLine:integer;
  function GetBreakPoint:TOilBreakPoint;override;
  procedure SetBreakPoint(aBPoint:TOilBreakPoint);override;
  function GetSourceLine:integer;override;
  procedure SetSourceLine(aLine:integer);override;
 public
  procedure DoIt;override;
end;


TForLine=class(TOilLIne)
private
 FVariable:TNormalVariable;
 FBlock:TOilLIne;
 FStart,FEnd:TOilOperator;
 FIncrease:Boolean;
 function GetVar:TNormalVariable;
 procedure SetVar(aVar:TNormalVariable);
 function GetStart:TOilOperator;
 procedure SetStart(aVar:TOilOperator);
 function GetEnd:TOilOperator;
 procedure SetEnd(aVar:TOilOperator);
 function GetBlock:TOilLIne;
 procedure SetBlock(ablock:TOilLIne);
 public
 procedure DoIt;override;
 property Variable:TNormalVariable read GetVar write SetVar;
 property Block:TOilLIne read GetBlock write SetBlock;
 property Increase:boolean read FIncrease write FIncrease;
 {predelat na operatory}
 property StartValue:TOilOperator read GetStart write SetStart;
 property EndValue:TOilOperator read GetEnd write SetEnd;
end;

TDBForLine=class(TForLine)
 private
  FBreakPoint:TOilBreakPoint;
  FSourceLine:integer;
  function GetBreakPoint:TOilBreakPoint;override;
  procedure SetBreakPoint(aBPoint:TOilBreakPoint);override;
  function GetSourceLine:integer;override;
  procedure SetSourceLine(aLine:integer);override;
 public
  procedure DoIt;override;
end;

TWhileLine=class(TOilLIne)
  FBlock:TOilLIne;
  FExpression:{TBooleanOperator}TOilOperator;
 private
  function GetBlock:TOilLIne;
  procedure SetBlock(ablock:TOilLIne);
  function GetExpr:TOilOperator;
  procedure SetExpr(aExpr:TOilOperator);
 public
  procedure DoIt;override;
  property Block:TOilLIne read GetBlock write SetBlock;
  property Expression:TOilOperator read GetExpr write SetExpr;
 end;

TDBWhileLine=class(TWhileLine)
 private
  FBreakPoint:TOilBreakPoint;
  FSourceLine:integer;
  function GetBreakPoint:TOilBreakPoint;override;
  procedure SetBreakPoint(aBPoint:TOilBreakPoint);override;
  function GetSourceLine:integer;override;
  procedure SetSourceLine(aLine:integer);override;
 public
  procedure DoIt;override;
end;

TRepeatline=class(TOilBlock)
private
  FExpression:{TBooleanOperator}TOilOperator;
 private
  function GetExpr:TOilOperator;
  procedure SetExpr(aExpr:TOilOperator);
 public
  procedure DoIt;override;
  property Expression:TOilOperator read GetExpr write SetExpr;
end;

TDBRepeatLine=class(TRepeatLine)
 private
  FBreakPoint:TOilBreakPoint;
  FSourceLine:integer;
  function GetBreakPoint:TOilBreakPoint;override;
  procedure SetBreakPoint(aBPoint:TOilBreakPoint);override;
  function GetSourceLine:integer;override;
  procedure SetSourceLine(aLine:integer);override;
 public
  procedure DoIt;override;
end;

TCaseLine=class(TOilLIne)
  FExpression:TOilOperator;
  FCaseList,FBlockList:TEntity;
  {FBlock:TOilLIne;}
  FElseBlock:TOilLIne;
private
  function GetExp:TOilOperator;
  procedure SetExp(aExp:TOilOperator);
  {function GetBlock:TOilLIne;
  procedure SetBlock(ablock:TOilLIne);}
  function GetCaseList:TEntity;
  procedure SetCaseList(aList:TEntity);
  function GetCaseItem(index:integer):TTypeElement;
  procedure SetCaseItem(index:integer;Item:TTypeElement);
  function GetBlockList:TEntity;
  procedure SetBlockList(aList:TEntity);
  function GetBlockItem(index:integer):TOilLIne;
  procedure SetBlockItem(index:integer;Item:TOilLIne);
  function GetElseBlock:TOilLIne;
  procedure SetElseBlock(aBlock:TOilLIne);
public
  procedure DoIt;override;
  property Expression:TOilOperator read GetExp write SetExp;
  {property Block:TOilLIne read GetBlock write SetBlock;}
  property CaseList:TEntity read GetCaseList write SetCaseList;
  property CaseItem[index:integer]:TTypeElement read GetCaseItem write SetCaseItem;
  property BlockList:TEntity read GetBlockList write SetBlockList;
  property BlockItem[index:integer]:TOilLIne read GetBlockItem write SetBlockItem;
  property ElseBlock:TOilLIne read GetElseBlock write SetElseBlock;
end;

TDBCaseLine=class(TCaseLine)
 private
  FBreakPoint:TOilBreakPoint;
  FSourceLine:integer;
  function GetBreakPoint:TOilBreakPoint;override;
  procedure SetBreakPoint(aBPoint:TOilBreakPoint);override;
  function GetSourceLine:integer;override;
  procedure SetSourceLine(aLine:integer);override;
 public
  procedure DoIt;override;
end;

TOilFunc=class(TOilProc)
private
 FFuncResult:TTypeElement;
 function GetFuncResult:TTypeElement;override;
 procedure DestroyVarsInStack;override;
public
 procedure DoIt;override;
end;

TDBOilFunc=class(TOilFunc)
 private
  FNameStack:TStringListElement;
  FBreakPoint:TOilBreakPoint;
  FSourceLine:integer;
  procedure SetNameStack(aStack:TStringListElement);override;
  function GetNameStack:TStringListElement;override;
  function GetBreakPoint:TOilBreakPoint;override;
  procedure SetBreakPoint(aBPoint:TOilBreakPoint);override;
  function GetSourceLine:integer;override;
  procedure SetSourceLine(aLine:integer);override;
  procedure CreateVarsInStack;override;
  procedure DestroyVarsInStack;override;
public
  procedure DoIt;override;
end;



TCaller=class(TOilOperator)
private
  FParams:TNameEntity;
  function GetParams:TNameEntity;
  procedure SetParams(aPar:TNameEntity);
public
 property Parameters:TNameEntity read GetParams write SetParams;
end;


TProcCaller=class(TCaller)
private
 FProc:TOilProc;
 function GetProc:TOilProc;virtual;
 procedure SetProc(aProc:TOilProc);virtual;
public
 property OilProc:TOilProc read GetProc write SetProc;
 procedure GetTypeInfo;override;
 procedure DoIt;override;
end;


implementation
uses
     Dialogs,
     U_Props,
     SysUtils,
     E_Class,
     TypInfo,
     worlds;

function AssignableTypes(t1,t2:TType):boolean;
 begin
 result:=t1=t2;
 if not result then
   case T1 of
    _Other:result:=true;
    _Float:result:=t2 in [_Integer,_Byte];
    _Integer:result:=t2=_Byte;
    _String:result:=t2=_Char;
    {_Byte:result:=t2=_Integer;}
    end;
 end;

constructor TOilThread.RunProgram(P:TOilProgram);
  begin
  inherited create(true); {true=suspended}
  OnTerminate:=P.Terminating;
  FProgram:=P;
  FreeOnTerminate:=true;
  Resume;
  end;
  
procedure TOilThread.Execute;
  begin
  try
   FProgram.Run;
  except
   showmessage('Hruba Chyba OILu- nejspis spatne odalokovani');
  end;
  end;


{TOilBreakPoint}
constructor TOilBreakPoint.Create;
  begin
  inherited Create;
  FActive:=true;
  end;

procedure TOilBreakPoint.Pause;
var OM:TLineMessage;
    dumm:boolean;
begin
  if TDBOilProgram(OilProgram).ToNotify<>nil then
  begin
  OM := TLineMessage.CreateAll(tm_OilBreak,Self,TDBOilProgram(OilProgram).ToNotify,nil,true);
  Om.Linenum:=LineNum;
  TDBOilProgram(OilProgram).ToNotify.HandleMessage(TOrbisMessage(Om),dumm);
  end;
  OilProgram.pause;
end;

{TOilLIne}
function TOilLIne.GetBreakPoint:TOilBreakPoint;
  begin
  result:=nil;
  end;

procedure TOilLIne.SetBreakPoint(aBPoint:TOilBreakPoint);
  begin
  end;

function TOilLIne.GetSourceLine:integer;
  begin
  result:=-1;
  end;

procedure TOilLIne.SetSourceLine(aLine:integer);
  begin
  end;

procedure TOilLIne.DoIt;
  begin
  end;

{TOilBlock}
procedure TOilBlock.SetVariable(index:integer;aVar:TTypeElement);
{tady uz musi byt zkontrolovana kompatidebilita}
var El:TTypeElement;
 begin
  if Stack.Count>index then
  begin
  El:=TTypeElement(Stack.Elements[index]);
   case El.Typ of
    _Boolean:El.AsBoolean:=aVar.AsBoolean;
    _Char:El.AsChar:=aVar.AsChar;
    _Float:El.AsFloat:=aVar.AsFloat;
    _String:El.AsString:=aVar.AsString;
    _Integer:El.AsInteger:=aVar.AsInteger;
    _Pointer:El.AsPointer:=aVar.AsPointer
    else El.AsString:=aVar.AsString;
    end;{case}
  if aVar.IsPossibleToFree then
     aVar.Free;
  end
  else Stack.AddElement(aVar);
 end;

function TOilBlock.GetVariable(index:integer):TTypeElement;
 begin
  result:=TTypeElement(Stack.Elements[index]);
 end;

procedure TOilBlock.SetStack(aStack:TOwnerEntity);
 begin
  if assigned(FStack) then FreeElement(FStack);
  AddElement(aStack);
  FStack:=aStack;
 end;

function TOilBlock.GetStack:TOwnerEntity;
 begin
 if not assigned(FStack) then
   begin
   FStack:=TOwnerEntity.create;
   FStack.name:='STACK'+name;
   AddElement(FStack);
   end;
  result:=FStack;
 end;


procedure TOilBlock.SetPrgLineList(aList:TNameEntity);
   begin
   if assigned(FPrgLineList) then FreeElement(FPrgLineList);
   AddElement(aList);
   FPrgLineList:=aList;
   end;

function TOilBlock.GetPrgLineList:TNameEntity;
   begin
   if not assigned(FPrgLineList) then
      begin
      FPrgLineList:=TNameEntity.create;
      AddELement(FPrgLineList);
      FPrgLineList.Name:='Program Lines';
      end;
   result:=FPrgLineList;
   end;

function TOilBlock.GetPrgLine(index:longint):TOilLIne;
   begin
   result:=TOilLIne(PrgLineList.Elements[index]);
   end;

procedure TOilBlock.SetPrgline(index:longint;aLine:TOilLIne);
   begin
   if PrgLineList.Count>index then
   begin
   PrgLineList.InsertElement(index,aLine);
   PrgLineList.FreeElement(PrgLineList.Elements[index+1]);
   end
   else PrgLineList.AddElement(aLine);
   end;

procedure TOilBlock.AddPrgLine(aLine:TOilLIne);
   begin
   if aLine<>nil then
   	begin
   	PrgLineList.AddElement(aLine);
       end;
   end;


function TOilBlock.GetVarInfoList:TNameEntity;
    begin
    if not assigned(FVarInfoList) then
       begin
       FVarInfoList:=TNameEntity.Create;
       FVarInfoList.Name:='VarInfo';
       AddElement(FVarInfoList);
       end;
    result:=FVarInfoList;
    end;

procedure TOilBlock.SetVarInfoList(aList:TNameEntity);
    begin
    if assigned(FVarInfoList) then FreeElement(FVarInfoList);
    FVarInfoList:=aList;
    AddElement(FVarInfoList);
    end;


function TOilBlock.GetVarType(index:byte):TType;
   begin
   result:=TVarinfo(VarInfoList.Elements[index]).VarType;
   end;

procedure TOilBlock.SetVarType(index:byte;aType:TType);
   begin
   TVarinfo(VarInfoList.Elements[index]).VarType:=aType;
   end;

function TOilBlock.GetVarTypeName(index:byte):string;
   begin
   result:=TVarinfo(VarInfoList.Elements[index]).TypeName;
   end;

procedure TOilBlock.SetVarTypeName(index:byte;aName:string);
   begin
   TVarinfo(VarInfoList.Elements[index]).TypeName:=aName;
   end;


procedure TOilBlock.CreateVarsInStack;
var i:integer;
    aVar:TPointerElement;
   begin
   StackIndex:=Stack.Count;
   if assigned(FVarInfoList) then if VarInfolist.Count>0 then
      for i:=0 to VarInfolist.Count-1 do
      begin
      case VarType[i] of
            _Boolean:Stack.AddElement(TBooleanElement.CreateValue(false));
            _Char:Stack.AddElement(TCharElement.CreateValue(#0));
            _Byte:Stack.AddElement(TByteElement.CreateValue(0));
            _Float:Stack.AddElement(TFloatElement.CreateValue(0));
            _String:Stack.AddElement(TStringElement.CreateValue(''));
            _Integer:Stack.AddElement(TIntegerElement.CreateValue(0));
            _Pointer:begin
                     aVar:=TPointerElement.Create;
                     aVar.AClassName:=varTypeName[i];
                     aVar.Value:=nil;
                     Stack.AddElement(aVar);
                     end;
      end;{case}
      end;
   end;

procedure TOilBlock.DestroyVarsInStack;
var i,j:integer;
   begin
   if assigned(FVarInfoList) then if VarInfolist.Count>0 then
    begin
    j:=VarInfolist.Count;
    for i:=1 to j do
    Stack.FreeElement(Variable[Stack.Count-1]);
    end;
   end;                                       {skoda, ze nejde dat primo index}

{prodebug}
function TOilBlock.GetVarDBInfoName(index:integer):string;
   begin
   result:=(VarInfoList.Elements[index] as TDBVarinfo).VarName;
   end;

procedure TOilBlock.SetVarDBInfoName(index:integer;aName:string);
   begin
   (VarInfoList.Elements[index]as TDBVarinfo).VarName:=aName;
   end;

procedure TOilBlock.ClearLastDBVarName;
 begin
 {tohle je prilis slozite,chce to primy pristup pres index}
 NameStack.FreeElement(NameStack.Elements[namestack.count-1]);
 end;

function TOilBlock.GetVarName(index:integer):string;
  begin
  result:=NameStack.Strings[index]
  end;

procedure TOilBlock.NewDBVarName(aName:string);
  begin
  NameStack.AddString(aName);
  end;

procedure TOilBlock.DoIt;
var i:longint;
   begin
   CreateVarsInStack;
   if PrgLineList.Count>0 then
     for i:=0 to PrgLinelist.Count-1 do
      if PrgLine[i]<>nil then PrgLine[i].Doit;
   DestroyVarsInStack;
   end;

{TDBOilBlock}
procedure TDBOilBlock.CreateVarsInStack;
var i:integer;
  begin
  inherited CreateVarsInStack;
  if assigned(FVarInfoList) then if VarInfolist.Count>0 then
      for i:=0 to VarInfolist.Count-1 do
      NewDBVarName(VarDBInfoName[i]);
  end;

procedure TDBOilBlock.DestroyVarsInStack;
var i,j:integer;
  begin
  inherited DestroyVarsInStack;
  if assigned(FVarInfoList) then if VarInfolist.Count>0 then
    begin
    j:=VarInfolist.Count;
    for i:=1 to j do ClearLastDBVarName;
    end;
  end;


procedure TDBOilBlock.SetNameStack(aStack:TStringListElement);
 begin
  if assigned(FNameStack) then FreeElement(FNameStack);
  AddElement(aStack);
  FNameStack:=aStack;
 end;

function TDBOilBlock.GetNameStack:TStringListElement;
 begin
 if not assigned(FNameStack) then
   begin
   FNameStack:=TStringListElement.create;
   FNameStack.name:='NameSTACK'+name;
   AddElement(FNameStack);
   end;
  result:=FNameStack;
 end;

function TDBOilBlock.GetBreakPoint:TOilBreakPoint;
  begin
  result:=FBreakPoint;
  end;

procedure TDBOilBlock.SetBreakPoint(aBPoint:TOilBreakPoint);
  begin
  if assigned(FBreakPoint) then FreeElement(FBreakPoint);
  FBreakPoint:=aBPoint;
  if aBPoint<>nil then AddElement(FBreakPoint);
  end;

function TDBOilBlock.GetSourceLine:integer;
  begin
  result:=FSourceLine;
  end;

procedure TDBOilBlock.SetSourceLine(aLine:integer);
  begin
  FSourceLine:=aLine;
  end;

procedure TDBOilBlock.DoIt;
var i:integer;
  begin
  try
  if BreakPoint<>nil then if BreakPoint.Active then BreakPoint.Pause;
  CreateVarsInStack;
  if PrgLineList.Count>0 then
     for i:=0 to PrgLinelist.Count-1 do
      if PrgLine[i]<>nil then PrgLine[i].Doit;
   DestroyVarsInStack;
  except on E:TLangError do
            begin
            if E.Line<0 then E.Line:=SourceLineNum;
            raise;
            end;
    end;
  end;

{TOilProc}
function TOilProc.GetFuncResult:TTypeElement;
	begin
  result:=nil;
  end;

procedure TOilProc.ReadParameters;
var i:integer;
    P:TParametr;
    E:TTypeElement;
	begin
  if Parameters<>nil then
  begin
  if Parameters.Count>0 then for i:=0 to Parameters.Count-1 do
     begin
     P:=TParametr(Parameters.Elements[i]);
     P.Expression.Doit;
     E:=TTypeElement(P.Expression.Output);
     if P.VarIndex>=Calls then
     	begin
     	if not AssignableTypes(VarType[P.varIndex],E.typ) then
        begin
        if E.IsPossibleToFree then E.free;
        raise TRunTimeError.CreateError(3);
        end;
       end;
     Variable[StackIndex+P.VarIndex]:=E;
     end;
  end;
  end;

function TOilProc.GetSubProcs:TEntity;
  begin
  if not assigned(FSubProcs) then
     begin
     FSubProcs:=TEntity.create;
     Addelement(FSubProcs);
     FSubProcs.name:='Podprocedury';
     end;
  result:=FSubProcs;
  end;

procedure TOilProc.SetSubProcs(aEnt:TEntity);
  begin
  if not assigned(FSubProcs) then  FreeElement(FSubProcs);
  FSubProcs:=aEnt;
  Addelement(FSubProcs);
  end;

procedure TOilProc.DoIt;
var i:longint;
  begin
  CreateVarsInStack;
  ReadParameters;
  if PrgLineList.Count>0 then
  for i:=0 to PrgLinelist.Count-1 do if PrgLine[i]<>nil then
  PrgLine[i].DoIt;
  DestroyVarsInStack;
  end;

{TDBOilProc}
procedure TDBOilProc.CreateVarsInStack;
var i:integer;
  begin
  inherited CreateVarsInStack;
  if assigned(FVarInfoList) then if VarInfolist.Count>0 then
      for i:=0 to VarInfolist.Count-1 do
      NewDBVarName(VarDBInfoName[i]);
  end;

procedure TDBOilProc.DestroyVarsInStack;
var i,j:integer;
  begin
  inherited DestroyVarsInStack;
  if assigned(FVarInfoList) then if VarInfolist.Count>0 then
    begin
    j:=VarInfolist.Count;
    for i:=1 to j do ClearLastDBVarName;
    end;
  end;

procedure TDBOilProc.SetNameStack(aStack:TStringListElement);
 begin
  if assigned(FNameStack) then FreeElement(FNameStack);
  AddElement(aStack);
  FNameStack:=aStack;
 end;

function TDBOilProc.GetNameStack:TStringListElement;
 begin
 if not assigned(FNameStack) then
   begin
   FNameStack:=TStringListElement.create;
   FNameStack.name:='NameSTACK'+name;
   AddElement(FNameStack);
   end;
  result:=FNameStack;
 end;

function TDBOilProc.GetBreakPoint:TOilBreakPoint;
  begin
  result:=FBreakPoint;
  end;

procedure TDBOilProc.SetBreakPoint(aBPoint:TOilBreakPoint);
  begin
  if assigned(FBreakPoint) then FreeElement(FBreakPoint);
  FBreakPoint:=aBPoint;
  if aBPoint<>nil then AddElement(FBreakPoint);
  end;

function TDBOilProc.GetSourceLine:integer;
  begin
  result:=FSourceLine;
  end;

procedure TDBOilProc.SetSourceLine(aLine:integer);
  begin
  FSourceLine:=aLine;
  end;

procedure TDBOilProc.DoIt;
var i:integer;
  begin
  try
  if BreakPoint<>nil then if BreakPoint.Active then BreakPoint.Pause;
  CreateVarsInStack;
  ReadParameters;
  if PrgLineList.Count>0 then
  for i:=0 to PrgLinelist.Count-1 do if PrgLine[i]<>nil then
  PrgLine[i].DoIt;
  DestroyVarsInStack;
  except on E:TLangError do
            begin
            if E.Line<0 then E.Line:=SourceLineNum;
            raise;
            end;
  end;
  end;

{TOilProgram}
constructor TOilProgram.Create;
  begin
  inherited Create;
  FThreadForRun:=nil;{velmi dulezite - pro jistotu natvrdo}
  end;

procedure TOilProgram.HandleError(E:TRunTimeError);
begin
 showmessage(E.Message);
end;

procedure TOilProgram.Run;
var
   i:longint;
   PE:TPointerElement;
begin
  try
   StackIndex:=0;
   CreateVarsInStack;
   PE:=TPointerElement.CreateValue(World);
   PE.Name:='TELEMENT';
   Variable[WorldIndex]:=PE;
   PE:=TPointerElement.CreateValue(World.MainTool);
   PE.Name:='TTOOL';
   Variable[MToolIndex]:=PE;
   PE:=TPointerElement.CreateValue(World);
   PE.Name:='TELEMENT';
   Variable[ActiveElIndex]:=PE;
   {ulozit world,acctivelemment a maintool}
   if PrgLineList.Count>0 then
      for i:=0 to PrgLinelist.Count-1 do
      if PrgLine[i]<>nil then
         PrgLine[i].DoIt;
  except
     on E:TRunTimeError do
     begin
        HandleError(E);
     end;
  end;
  if Assigned(FOnEnd) then
     OnEnd;
end;

procedure TOilProgram.DestroyVarsInStack;
begin
   Stack.FreeAll;
end;

procedure TOilProgram.Terminating(Sender:TObject);
begin
 CleanUp;
end;

procedure TOilProgram.CleanUp;
 begin
 DestroyVarsInStack;
 FThreadForRun:=nil;
 end;

procedure TOilProgram.Pause;
 begin
 if not FThreadForRun.Suspended then FThreadForRun.Suspend;
 end;

procedure TOilProgram.Continue;
 begin
 if FThreadForRun.Suspended then FThreadForRun.Resume;
 end;

procedure TOilProgram.Terminate; {uonci program natvrdo}
 begin
 if not FThreadForRun.Suspended then FThreadForRun.Suspend;
 {nevim jestli funguje}
 FThreadForRun.free;
 FThreadForRun:=nil;
 {zavolat Windowsy aby to utly-nevim jak}
 end;

procedure TOilProgram.DoIt;
begin
    Run;
   //if Running then exit;
   //FThreadForRun:=TOilThread.RunProgram(self);{udelat vlakno}
end;

function TOilProgram.GetRunning: boolean;
begin
   result := FThreadForRun <> nil;
end;

procedure TOilProgram.SetOnEnd(const Value: TOilCallBackProcedure);
begin
  FOnEnd := Value;
end;

{TDBOilProgram}
procedure TDBOilProgram.HandleError(E:TRunTimeError);
var OM:TErrorMessage;
    dumm:boolean;
begin
  if ToNotify<>nil then
  begin
  OM := TErrorMessage.CreateAll(tm_OilError,Self,ToNotify,nil,true);
  OM.LineNum:=E.Line;
  OM.text:=E.Message;
  ToNotify.HandleMessage(TOrbisMessage(Om),dumm);
  end;
end;


procedure TDBOilProgram.Run;
var Om:TOrbisMessage;
    dumm:boolean;
 begin
 if ToNotify<>nil then
  begin
  OM := TOrbisMessage.CreateAll(tm_OilStart,Self,ToNotify,nil,true);
  {blbost}
  ToNotify.HandleMessage(Om,dumm);
  end;
 inherited
 end;

procedure TDBOilProgram.CreateVarsInStack;
var i:integer;
  begin
  inherited CreateVarsInStack;
  if assigned(FVarInfoList) then if VarInfolist.Count>0 then
      for i:=0 to VarInfolist.Count-1 do
      NewDBVarName(VarDBInfoName[i]);
  end;

procedure TDBOilProgram.DestroyVarsInStack;
begin
  inherited DestroyVarsInStack;
  NameStack.freeAll;
end;

procedure TDBOilProgram.Continue;
 var Om:TOrbisMessage;
    dumm:boolean;
 begin
 if ToNotify<>nil then
  begin
  OM := TOrbisMessage.CreateAll(tm_OilResume,Self,ToNotify,nil,true);
  {blbost}
  ToNotify.HandleMessage(Om,dumm);
  end;
 inherited;
 end;

procedure TDBOilProgram.CleanUp;
var Om:TOrbisMessage;
    dumm:boolean;
 begin
 if ToNotify<>nil then
  begin
  OM := TOrbisMessage.CreateAll(tm_OilEnded,Self,ToNotify,nil,true);
  {blbost}
  ToNotify.HandleMessage(Om,dumm);
  end;
 inherited
 end;


procedure TDBOilProgram.SetNameStack(aStack:TStringListElement);
 begin
  if assigned(FNameStack) then FreeElement(FNameStack);
  AddElement(aStack);
  FNameStack:=aStack;
 end;

function TDBOilProgram.GetNameStack:TStringListElement;
 begin
 if not assigned(FNameStack) then
   begin
   FNameStack:=TStringListElement.create;
   FNameStack.name:='NameSTACK'+name;
   AddElement(FNameStack);
   end;
  result:=FNameStack;
 end;

function TDBOilProgram.GetSourceLine:integer;
  begin
  result:=FSourceLine;
  end;

procedure TDBOilProgram.SetSourceLine(aLine:integer);
  begin
  FSourceLine:=aLine;
  end;

function TDBOilProgram.GetBPoint(index:integer):TOilBreakPoint;
 begin
  result:=TOilBreakPoint(BreakPointList.Elements[index]);
 end;

function TDBOilProgram.GetBPList:TEntity;
  begin
   if not assigned(FBreakPointList) then
   begin
   FBreakPointList:=TEntity.create;
   AddElement(FBreakPointList);
   end;
  result:=FBreakPointList;
  end;

procedure TDBOilProgram.SetBPList(aList:TEntity);
  begin
  if assigned(FBreakPointList) then FreeElement(FBreakPointList);
  AddElement(aList);
  FBreakPointList:=aList;
  end;

function TDBOilProgram.GetBPNumber:integer;
  begin
  result:=BreakPointList.Count;
  end;

{TVariable}
procedure TVariable.GetTypeInfo;
var i:TType;
    tmp:TType;
   begin
   new(FTypeList);
   For i:=_boolean to _other do TypeItem[i]:=false;
   tmp:=Typ;
   if tmp=_other then For i:=_boolean to pred(_other) do TypeItem[i]:=true else
      TypeItem[tmp]:=true;
   end;

procedure TVariable.DoIt;
 begin
  Output:=Constant;
 end;

{Constantni VAR}
function TNormalVariable.GetConst:TTypeElement;
 begin
  result:=Block.Variable[Block.StackIndex+VarIndex].SelfCopy;
 end;

function TNormalVariable.GetConstTyp:TType;
 begin
 result:=Block.VarType[Varindex]
 end;

function TNormalVariable.GetConstName:string;
 begin
 result:=Block.VarTypeName[Varindex];
 end;


procedure TNormalVariable.SetConst(aConst:TTypeElement);
{mozna by byla lepsi kontrola na vyssi urovni}
function Agrees:boolean;
         var p:TElement;
             CL:TClass;
         begin
         p:=TPointerElement(aConst).Value;
         if p=nil then result:=true
         else
         begin
         CL:=GetOrbisClass(TypeName);
         result:=p is CL;
         end
         end;

  begin
  if (aConst.Typ=_pointer) and (Block.Variable[Block.StackIndex+VarIndex].typ=_Pointer) then
    begin
    if Agrees then Block.Variable[Block.StackIndex+VarIndex]:=aConst
       else
        begin
        if aconst.OwnersCount=0 then aConst.free;
        raise TRunTimeError.CreateError(3);
        end;
    end
   else
    Block.Variable[Block.StackIndex+VarIndex]:=aConst;
  end;

{called VAR}
function TCalledVariable.GetConst:TTypeElement;
var El:TTypeElement;
 begin
  El:=Block.Variable[Block.StackIndex+VarIndex];
  result:=TVariable(TPointerElement(El).Value).Constant;
 end;

procedure TCalledVariable.SetConst(aConst:TTypeElement);
 begin
  TVariable(TPointerElement(Block.Variable[Block.StackIndex+VarIndex]).Value).Constant:=aConst;
 end;
{ tohle blbost
function TCalledVariable.GetConstTyp:TType;
var El:TTypeElement;
 begin
  El:=Block.Variable[Block.StackIndex+VarIndex];
  result:=TVariable(TPointerElement(El).Value).Typ;
 end;         }

{PropVAR}
function TPropVariable.GetInitialExp:TOilOperator;
 begin
  result:=FInitialExp;
 end;

procedure TPropVariable.SetInitialExp(aOp:TOilOperator);
 begin
  if assigned(FInitialExp) then FreeElement(FInitialExp);
  AddElement(aOp);
  FInitialExp:=aOp;
 end;

{odtud je potreba to predelat}
function TPropVariable.GetConstTyp:TType;
 begin
   result:=_Other;
 end;

function TPropVariable.GetConst:TTypeElement;
  var InitialValue:TTypeElement;
      OrbInst:TOrbisInstance;
      St:string;
      i:longint;
      f:Extended;
      P:pointer;
      PIndex:integer;
   begin
    result:=nil;
    InitialValue:=nil;
    OrbInst:=TOrbisInstance.Create;
    try
    InitialExp.Doit;
    InitialValue:=TTypeElement(InitialExp.Output);
    if not (InitialValue.Typ=_pointer) then raise TRunTimeError.CreateError(2);
    if InitialValue.AsPointer=nil then raise TRunTimeError.CreateError(2);
    if not (TObject(InitialValue.AsPointer) is TElement) then raise TRunTimeError.CreateError(4);
    OrbInst.Instance:=TElement(InitialValue.AsPointer);
    PIndex:=OrbInst.PropertyIndex(PropertyName);
    if PIndex<0 then raise TRunTimeError.CreateError(5);
    case OrbInst.PropertyTypeKind[PIndex] of
        {_Boolean,_Char,_Byte,_Float,_String,_Integer,_Pointer}
        tkString
       {$IFDEF Win32}
       ,tkLString,tkWString
       {$ENDIF}
                         :begin
                          OrbInst.GetPropertyValue(Pindex,St);
                          result:=TStringElement.CreateValue(St);
                          end;
       tkInteger:
                          begin
                          OrbInst.GetPropertyValue(Pindex,i);
                          result:=TIntegerElement.CreateValue(i);
                          end;
       tkChar
       {$IFDEF Win32}
       ,tkWChar
       {$ENDIF}:
                          begin
                          OrbInst.GetPropertyValue(Pindex,i);
                          result:=TCharElement.Create;
                          TCharElement(result).AsInteger:=i;
                          end;
       tkFloat:
                          begin
                          OrbInst.GetPropertyValue(Pindex,f);
                          result:=TFloatElement.CreateValue(f);
                          end;
       tkClass:
                          begin
                          OrbInst.GetPropertyValue(Pindex,p);
                          result:=TPointerElement.CreateValue(p);
                          end;
       tkEnumeration:
                         begin
                         St:=GetPropString(Initialvalue.AsPointer,PIndex);
                         AnsiUpperCase(St);
                         if St='TRUE' then result:=TBooleanElement.createvalue(true)
                         else
                         if St='FALSE' then result:=TBooleanElement.createvalue(false)
                         else raise TRunTimeError.CreateError(15);
                         end;
       else raise TRunTimeError.CreateError(15);
       end;{case}
    finally
    OrbInst.Free;
    if InitialValue<>nil then if InitialValue.OwnersCount=0 then  InitialValue.free;
    end;
  end;



procedure TPropVariable.SetConst(aConst:TTypeElement);
var Initialvalue:TTypeElement;
    OrbInst:TOrbisInstance;
    St:string;
     i:longint;
     f:Extended;
     P:pointer;
     c:char;
     b:boolean;
     PIndex:integer;
begin
  InitialValue:=nil;
  OrbInst:=TOrbisInstance.Create;
  try
    InitialExp.DoIt;
    InitialValue:=TTypeElement(InitialExp.Output);
    if not (InitialValue.Typ=_pointer) then raise TRunTimeError.CreateError(2);
    if InitialValue.AsPointer=nil then raise TRunTimeError.CreateError(2);
    if not (TObject(InitialValue.AsPointer) is TElement) then raise TRunTimeError.CreateError(4);
    OrbInst.Instance:=TElement(InitialValue.AsPointer);
    PIndex:=OrbInst.PropertyIndex(PropertyName);
    if PIndex<0 then raise TRunTimeError.CreateError(5);
    case OrbInst.PropertyTypeKind[PIndex] of
     {_Boolean,_Char,_Byte,_Float,_String,_Integer,_Pointer}
        tkString
       {$IFDEF Win32}
       ,tkLString,tkWString
       {$ENDIF}
                         :begin
                          if not (aConst.Typ in [_Char,_String]) then raise TRunTimeError.CreateError(3);
                          St:=aConst.AsString; {neni toprilis pomale}
                          OrbInst.SetPropertyValue(Pindex,St);
                          end;
       tkInteger:
                          begin
                          if not (aConst.Typ in [_Byte,_Integer]) then raise TRunTimeError.CreateError(3);
                          i:=aConst.AsInteger;
                          OrbInst.SetPropertyValue(Pindex,i);
                          end;
       tkChar
       {$IFDEF Win32}
       ,tkWChar
       {$ENDIF}:
                          begin
                          if not (aConst.Typ =_Char) then raise TRunTimeError.CreateError(3);
                          c:=aConst.AsChar;
                          OrbInst.SetPropertyValue(Pindex,c);
                          end;
       tkFloat:
                          begin
                          if not (aConst.Typ in [_Byte,_Integer,_Float]) then raise TRunTimeError.CreateError(3);
                          f:=aConst.AsFloat;
                          OrbInst.SetPropertyValue(Pindex,f);
                          end;
       tkClass:
                          begin
                          if not (aConst.Typ =_Pointer) then raise TRunTimeError.CreateError(3);
                          p:=aConst.AsPointer;
                          OrbInst.SetPropertyValue(Pindex,p);
                          end;
       tkEnumeration:
                         begin
                         if not (aConst.Typ =_Boolean) then raise TRunTimeError.CreateError(3);
                         b:=aConst.AsBoolean;
                         OrbInst.SetPropertyValue(Pindex,b);
                         end;
       else raise TRunTimeError.CreateError(15);
       end;{case}
 finally
 OrbInst.Free;
 if aConst.IsPossibleToFree then aConst.free;
 if InitialValue<>nil then if InitialValue.OwnersCount=0 then InitialValue.free;
 end;
end;

{TElementOperator}
procedure TElementOperator.GetTypeInfo;
var i:TType;
  begin
   new(FTypeList);
   For i:=_boolean to _other do TypeItem[i]:=false;
   TypeItem[_pointer]:=true;
  end;

procedure TElementOperator.DoIt;
var tmp:TElement;
	begin
  Output:=nil;
  try
  ReadFirstResult;
  if not (FirstResult.Typ=_pointer) then raise TRunTimeError.CreateError(2);;
  if FirstResult.AsPointer=nil then raise TRunTimeError.CreateError(2);;
  if not (TObject(FirstResult.AsPointer) is TEntity) then raise TRunTimeError.CreateError(14);
  tmp:=TEntity(FirstResult.AsPointer).FindElementName(Elementname);
  Output:=TPointerElement.CreateValue(tmp);
  finally
  ClearResults;
  end;
  end;

{TElementByNumberOperator}
procedure TElementByNumberOperator.GetTypeInfo;
var i:TType;
  begin
   new(FTypeList);
   For i:=_boolean to _other do TypeItem[i]:=false;
   TypeItem[_pointer]:=true;
  end;

procedure TElementByNumberOperator.DoIt;
var tmp:TElement;
    TmpVal:TTypeElement;
    ind:integer;
  begin
  Output:=nil;
  try
  ReadFirstResult;
  if not (FirstResult.Typ=_pointer) then raise TRunTimeError.CreateError(2);
  if FirstResult.AsPointer=nil then raise TRunTimeError.CreateError(2);
  if not (TObject(FirstResult.AsPointer) is TEntity) then raise TRunTimeError.CreateError(14);
  PositionIndex.Doit;
  TmpVal:=TTypeElement(PositionIndex.Output);
  if not(TmpVal.Typ in [_integer,_byte]) then
      begin
      PositionIndex.CleanData;
      raise TRunTimeError.CreateError(3);
      end;
  ind:=TmpVal.Asinteger;
  PositionIndex.CleanData;
  tmp:=TEntity(FirstResult.AsPointer).Elements[ind];
  Output:=TPointerElement.CreateValue(tmp);
  finally
  ClearResults;
  end;
  end;

function TElementByNumberOperator.GetPositionIndex:TOilOperator;
 begin
  result:=FPositionIndex;
 end;

procedure TElementByNumberOperator.SetPositionIndex(aVar:TOilOperator);
 begin
 if assigned(FPositionIndex) then FreeElement(FPositionIndex);
  AddElement(aVar);
  FPositionIndex:=aVar;
 end;

{TOwnerOperator}
procedure TOwnerOperator.DoIt;
var tmp:TElement;
	begin
  Output:=nil;
  try
  ReadFirstResult;
  if not (FirstResult.Typ=_pointer) then raise TRunTimeError.CreateError(2);;
  if FirstResult.AsPointer=nil then raise TRunTimeError.CreateError(2);;
  if TObject(FirstResult.AsPointer) is TOwnerElement then
     tmp:=TOwnerElement(FirstResult.AsPointer).FindOwnerName(Elementname)
     else raise TRunTimeError.CreateError(14);
  Output:=TPointerElement.CreateValue(tmp);
  finally
  ClearResults;
  end;
  end;

{TOwnerByNumberOperator}
procedure TOwnerByNumberOperator.DoIt;
var tmp:TElement;
    TmpVal:TTypeElement;
    ind:integer;
  begin
  Output:=nil;
  try
  ReadFirstResult;
  if not (FirstResult.Typ=_pointer) then raise TRunTimeError.CreateError(2);;
  if FirstResult.AsPointer=nil then raise TRunTimeError.CreateError(2);;
  if not (TObject(FirstResult.AsPointer) is TEntity) then raise TRunTimeError.CreateError(14);
  PositionIndex.Doit;
  TmpVal:=TTypeElement(PositionIndex.Output);
  if not(TmpVal.Typ in [_integer,_byte]) then
      begin
      PositionIndex.CleanData;
      raise TRunTimeError.CreateError(3);
      end;
  ind:=TmpVal.Asinteger;
  PositionIndex.CleanData;
  tmp:=TEntity(FirstResult.AsPointer).Owners[ind];
  Output:=TPointerElement.CreateValue(tmp);
  finally
  ClearResults;
  end;
  end;

(*
TToolInputInfo=class(TEntity)
 private
  FPropIndex:longint;
  FInputExpr:TOilOperator;
  function GetInputExpr:TOilOperator;
  procedure SetInputExpr(aExp:TOilOperator);
 public
  property InputExpr:TOilOperator read GetInputExpr write SetInputExpr;
 published
  property PropIndex:longint read FPropIndex write FPropIndex;
end;

TToolOperator=class(TOilPtrOperator)
 private
  FToolClass:TToolClass;
  FMainInput:TOilOperator;
  FPropInputList:TEntity;
  function GetPropInputList:TEntity;
  procedure SetPropInputList(aList:TEntity);
  function GetInputItem(index:longint):TToolInputinfo;virtual;
  function GetMainInput:TOilOperator;virtual;
  procedure SetMainInput(aInput:TOilOperator);virtual;
 public
  property PropInputList:TEntity read GetPropInputList write SetPropInputList;
  property PropInputItem[index:longint]:TToolInputInfo read GetInputItem;
  property MainInput:TOilOperator read getMainInput write SetMainInput;
  property ToolClass:TToolClass read FToolClass write FToolClass;
  procedure DoIt;override;
  procedure GetTypeInfo;override;
 end;


*)

{TToolInputInfo}
function TToolInputInfo.GetInputExpr:TOilOperator;
   begin
   result:=FInputExpr;
   end;

procedure TToolInputInfo.SetInputExpr(aExp:TOilOperator);
   begin
   if assigned(FInputExpr) then FreeElement(FInputExpr);
   AddElement(aExp);
   FInputExpr:=aExp;
   end;

{TToolOperator}
function TToolOperator.GetPropInputList:TEntity;
    begin
    if not assigned(FPropInputList) then
       begin
       FPropInputList:=TEntity.Create;
       AddElement(FPropInputList);
       end;
    result:=FPropInputList;
    end;

procedure TToolOperator.SetPropInputList(aList:TEntity);
    begin
    if assigned(FPropInputList) then FreeElement(FPropInputList);
    FPropInputList:=aList;
    AddElement(FPropInputList);
    end;

function TToolOperator.GetInputItem(index:longint):TToolInputInfo;
   begin
   result:=TToolInputInfo(PropInputList.Elements[index]);
   end;


procedure TToolOperator.GetTypeInfo;
  var i:TType;
  begin
   new(FTypeList);
   For i:=_boolean to _other do TypeItem[i]:=false;
   TypeItem[_pointer]:=true;
  end;

procedure TToolOperator.DoIt;
var
aRes:TTypeElement;
aInp:TElement;
theTool:TTool;
aInstance:TOrbisInstance;
PIndex,i,int:longint;
bool:boolean;
st:string;
ptr:pointer;
chr:char;
fl:extended;

  function CompatibleTypes(a:TTypeKind;b:TType):boolean;
      {cunarna, ale jinak to asi nejde}
      begin
      case a of
      tkInteger,tkInt64:result:=b in [_integer,_byte];
      tkChar,tkWChar:result:=b in [_char];
      tkEnumeration:result:=b in [_boolean];
      tkFloat:result:=b in [_integer,_byte,_float];
      tkString,tkLString,tkWString:result:=b in [_char,_string];
      tkClass, tkMethod:result:=b in [_pointer,_other];
      else result:=false;
      end;
      end;


  begin
  aInstance:=nil;
  theTool:=nil;
  try
  aInp:=nil;
  ReadFirstResult;
  aRes:=FirstResult;
  if assigned(ares) then
  	begin {pokud je vstup, tak musi byt spravny}
 	     if aRes.typ=_pointer then aInp:=aRes.AsPointer
    	   else raise TRunTimeError.CreateError(8);
   	end;
  if Toolclass=nil then raise TRunTimeError.CreateError(9);
  theTool:=ToolClass.Create;
  theTool.Input:=aInp;
  aInstance:=TOrbisInstance.Create;
  aInstance.Instance:=theTool;
  if PropInputList.Count>0 then
      begin
      for i:=0 to PropInputList.Count-1 do
      begin {naplneni propert}
      pindex:=PropInputItem[i].PropIndex-1;
      PropInputItem[i].InputExpr.DoIt;
      aRes:=TTypeElement(PropInputItem[i].InputExpr.Output);
      if CompatibleTypes(aInstance.PropertyTypeKind[PIndex],aRes.Typ)then
         begin
          case  aRes.Typ of
             _Integer:
                 begin
                 int:=aRes.AsInteger;
                 aInstance.SetPropertyValue(PIndex,int);
                 end;
             _Char:
                 begin
                 chr:=aRes.AsChar;
                 aInstance.SetPropertyValue(PIndex,chr);
                 end;
             _Boolean:
                 begin
                 bool:=aRes.AsBoolean;
                 aInstance.SetPropertyValue(PIndex,bool);
                 end;
             _Float:
                 begin
                 fl:=aRes.AsFloat;
                 aInstance.SetPropertyValue(PIndex,fl);
                 end;
             _String:
                 begin
                 st:=aRes.AsString;
                 aInstance.SetPropertyValue(PIndex,st);
                 end;
             _Pointer:
                 begin
                 ptr:=aRes.AsPointer;
                 aInstance.SetPropertyValue(PIndex,ptr);
                 end;
             end{case}
         end
        else
         raise TRunTimeError.CreateError(3);
         {typova nekompatibilita s propertou aInstance.PropertyTypename[PIndex];}
      end;
      end;
  theTool.DoIt;
  Output:=TPointerElement.CreateValue(theTool.Output);
  finally
  ClearResults;
  if aInstance<>nil then aInstance.Free;
  if theTool<>nil then theTool.Free;
  end;
  end;

{TExpressionline}
function TExprLine.GeTOilOperator:TOilOperator;
 begin
  result:=FOperator;
 end;

procedure TExprLine.SeTOilOperator(aOp:TOilOperator);
 begin
 if assigned(FOperator) then FreeElement(FOperator);
  AddElement(aOp);
  FOperator:=aOp;
 end;

procedure TExprLine.DoIt;
begin
  Operator.DoIt;
  {Ulozit vysledek do...StringListu}
  Operator.CleanData;
end;

{TDBExprLine}
function TDBExprLine.GetBreakPoint:TOilBreakPoint;
  begin
  result:=FBreakPoint;
  end;

procedure TDBExprLine.SetBreakPoint(aBPoint:TOilBreakPoint);
  begin
  if assigned(FBreakPoint) then FreeElement(FBreakPoint);
  FBreakPoint:=aBPoint;
  if aBPoint<>nil then AddElement(FBreakPoint);
  end;

function TDBExprLine.GetSourceLine:integer;
  begin
  result:=FSourceLine;
  end;

procedure TDBExprLine.SetSourceLine(aLine:integer);
  begin
  FSourceLine:=aLine;
  end;

procedure TDBExprLine.DoIt;
  begin
  try
  if BreakPoint<>nil then if BreakPoint.Active then BreakPoint.Pause;
  inherited DoIt;
    except on E:TLangError do
            begin
            if E.Line<0 then E.Line:=SourceLineNum;
            raise;
            end;
    end;
  end;

{IF}
function TIfLine.GetExpr:TOilOperator;
 begin
  result:=FExpression;
 end;

procedure TIfLine.SetExpr(aExpr:TOilOperator);
 begin
 if assigned(FExpression) then FreeElement(FExpression);
  AddElement(aExpr);
  FExpression:=aExpr;
 end;

function TIfLine.GetThenBlock:TOilLIne;
 begin
  result:=FThenBlock;
 end;

procedure TIfLine.SetThenBlock(aBlock:TOilLIne);
 begin
  if assigned(FThenBlock) then FreeElement(FThenBlock);
  AddElement(aBlock);
  FThenBlock:=aBlock;
 end;

function TIfLine.GetElseBlock:TOilLIne;
 begin
  result:=FElseBlock;
 end;

procedure TIfLine.SetElseBlock(aBlock:TOilLIne);
 begin
  if assigned(FElseBlock) then FreeElement(FElseBlock);
  AddElement(aBlock);
  FElseBlock:=aBlock;
 end;

procedure TIfLine.DoIt;
var TmpRes:TTypeElement;
 begin
  try
  Expression.DoIt;
  TmpRes:=TTypeElement(Expression.Output);
  if (TmpRes=nil) then raise TRunTimeError.CreateError(2);
  if(TmpRes.Typ<>_boolean) then raise TRunTimeError.CreateError(10);
  if TBooleanElement(TmpRes).Value then ThenBlock.DoIt
     else if ElseBlock<>nil then ElseBlock.DoIt;
  finally
  Expression.CleanData;
  end;
 end;

{TDBIfLine}
function TDBIfLine.GetBreakPoint:TOilBreakPoint;
  begin
  result:=FBreakPoint;
  end;

procedure TDBIfLine.SetBreakPoint(aBPoint:TOilBreakPoint);
  begin
  if assigned(FBreakPoint) then FreeElement(FBreakPoint);
  FBreakPoint:=aBPoint;
  if aBPoint<>nil then AddElement(FBreakPoint);
  end;

function TDBIfLine.GetSourceLine:integer;
  begin
  result:=FSourceLine;
  end;

procedure TDBIfLine.SetSourceLine(aLine:integer);
  begin
  FSourceLine:=aLine;
  end;

procedure TDBIfLine.DoIt;
  begin
  try
  if BreakPoint<>nil then if BreakPoint.Active then BreakPoint.Pause;
  inherited DoIt;
    except on E:TLangError do
            begin
            if E.Line<0 then E.Line:=SourceLineNum;
            raise;
            end;
    end;
  end;

{FOR}
function TForLine.GetBlock:TOilLIne;
 begin
  result:=FBlock;
 end;

procedure TForLine.SetBlock(aBlock:TOilLIne);
 begin
  if assigned(FBlock) then FreeElement(FBlock);
  AddElement(aBlock);
  FBlock:=aBlock;
 end;

function TForLine.GetVar:TNormalVariable;
 begin
  result:=FVariable;
 end;

procedure TForLine.SetVar(aVar:TNormalVariable);
 begin
 if assigned(FVariable) then FreeElement(FVariable);
  AddElement(aVar);
  FVariable:=aVar;
 end;

function TForLine.GetStart:TOilOperator;
 begin
  result:=FStart;
 end;

procedure TForLine.SetStart(aVar:TOilOperator);
 begin
 if assigned(FStart) then FreeElement(FStart);
  AddElement(aVar);
  FStart:=aVar;
 end;

function TForLine.GetEnd:TOilOperator;
 begin
  result:=FEnd;
 end;

procedure TForLine.SetEnd(aVar:TOilOperator);
 begin
 if assigned(FEnd) then FreeElement(FEnd);
  AddElement(aVar);
  FEnd:=aVar;
 end;


procedure TForLine.DoIt;
var TmpVal:TTypeElement;
    tp:TType;
    cVal,EVal,i,ind:longint;
 begin
  if not(Variable.Typ in [_integer,_char,_byte]) then raise TRunTimeError.CreateError(2);
  StartValue.Doit;
  TmpVal:=TTypeElement(StartValue.Output);
  tp:=TmpVal.Typ;
  try
  if (Variable.Typ=_char) then
      begin
      if not (Tp=_char) then
         begin
         raise TRunTimeError.CreateError(3);
         end;
      end
  else if not(Tp in[_byte,_integer]) then raise TRunTimeError.CreateError(3);
  CVal:=TmpVal.AsInteger;
  Variable.Constant:=TmpVal;
  except
   tmpval.free;
   raise;
  end;
  EndValue.DoIt;
  TmpVal:=TTypeElement(EndValue.Output);
  try
  tp:=TmpVal.Typ;
  if (Variable.Typ=_char) then
      begin
      if not (Tp=_char) then
         raise TRunTimeError.CreateError(3);
      end
  else if not(Tp in[_byte,_integer]) then raise TRunTimeError.CreateError(3);
  Eval:=TmpVal.AsInteger;
  ind:=Variable.Block.StackIndex+Variable.VarIndex;
  if Increase then
      begin
      for i:=CVal to EVal do
      begin
      Variable.Block.Variable[ind].AsInteger:=i;{tohle je trochu rychlejsi}
      Block.DoIt;
      end;
      end
      else
      for i:=CVal downto EVal do
      begin
      Variable.Block.Variable[ind].AsInteger:=i;
      Block.DoIt;
      end;
  finally
  EndValue.CleanData;
  end;
 end;

{TDBForLine}
function TDBForLine.GetBreakPoint:TOilBreakPoint;
  begin
  result:=FBreakPoint;
  end;

procedure TDBForLine.SetBreakPoint(aBPoint:TOilBreakPoint);
  begin
  if assigned(FBreakPoint) then FreeElement(FBreakPoint);
  FBreakPoint:=aBPoint;
  if aBPoint<>nil then AddElement(FBreakPoint);
  end;

function TDBForLine.GetSourceLine:integer;
  begin
  result:=FSourceLine;
  end;

procedure TDBForLine.SetSourceLine(aLine:integer);
  begin
  FSourceLine:=aLine;
  end;

procedure TDBForLine.DoIt;
  begin
  try
  if BreakPoint<>nil then if BreakPoint.Active then BreakPoint.Pause;
  inherited DoIt;
    except on E:TLangError do
            begin
            if E.Line<0 then E.Line:=SourceLineNum;
            raise;
            end;
    end;
  end;

{WHILE}
function TWhileLine.GetExpr:TOilOperator;
 begin
  result:=FExpression;
 end;

procedure TWhileLine.SetExpr(aExpr:TOilOperator);
 begin
 if assigned(FExpression) then FreeElement(FExpression);
  AddElement(aExpr);
  FExpression:=aExpr;
 end;

 function TWhileLine.GetBlock:TOilLIne;
 begin
  result:=FBlock;
 end;

procedure TWhileLine.SetBlock(aBlock:TOilLIne);
 begin
  if assigned(FBlock) then FreeElement(FBlock);
  AddElement(aBlock);
  FBlock:=aBlock;
 end;

procedure TWhileLine.DoIt;
var TmpRes:TTypeElement;
 begin
  try
  Expression.DoIt;
  TmpRes:=TTypeElement(Expression.Output);
  if (TmpRes=nil) then raise TRunTimeError.CreateError(2);
  if (TmpRes.Typ<>_boolean) then raise TRunTimeError.CreateError(10);
  while (TmpRes as TBooleanElement).value do
    begin
    Expression.CleanData;
    Block.Doit;
    Expression.DoIt;
    TmpRes:=TTypeElement(Expression.Output);
    if (TmpRes=nil) then raise TRunTimeError.CreateError(2);
    if (TmpRes.Typ<>_boolean) then raise TRunTimeError.CreateError(10);
    end;
  finally
  Expression.CleanData;
  end;
 end;

{TDBWhileLine}
function TDBWhileLine.GetBreakPoint:TOilBreakPoint;
  begin
  result:=FBreakPoint;
  end;

procedure TDBWhileLine.SetBreakPoint(aBPoint:TOilBreakPoint);
  begin
  if assigned(FBreakPoint) then FreeElement(FBreakPoint);
  FBreakPoint:=aBPoint;
  if aBPoint<>nil then AddElement(FBreakPoint);
  end;

function TDBWhileLine.GetSourceLine:integer;
  begin
  result:=FSourceLine;
  end;

procedure TDBWhileLine.SetSourceLine(aLine:integer);
  begin
  FSourceLine:=aLine;
  end;

procedure TDBWhileLine.DoIt;
  begin
  try
  if BreakPoint<>nil then if BreakPoint.Active then BreakPoint.Pause;
  inherited DoIt;
      except on E:TLangError do
            begin
            if E.Line<0 then E.Line:=SourceLineNum;
            raise;
            end;
    end;
  end;

{REPEAT}
function TRepeatLine.GetExpr:TOilOperator;
 begin
  result:=FExpression;
 end;

procedure TRepeatLine.SetExpr(aExpr:TOilOperator);
 begin
 if assigned(FExpression) then FreeElement(FExpression);
  AddElement(aExpr);
  FExpression:=aExpr;
 end;


procedure TRepeatLine.DoIt;
var TmpRes:TTypeElement;
    NotFirst:boolean;
    i:longint;
 begin
  try
   NotFirst:=false;
   repeat
    if NotFirst then Expression.CleanData;
    NotFirst:=true;
    if PrgLineList.Count>0 then
       for i:=0 to PrgLinelist.Count-1 do if PrgLine[i]<>nil then PrgLine[i].Doit;
    Expression.DoIt;
    TmpRes:=TTypeElement(Expression.Output);
    if (TmpRes=nil) then raise TRunTimeError.CreateError(2);
    if (TmpRes.Typ<>_boolean) then raise TRunTimeError.CreateError(10);
   until TBooleanElement(Tmpres).value;
  finally
  Expression.CleanData;
  end;
 end;

{TDBRepeatLine}
function TDBRepeatLine.GetBreakPoint:TOilBreakPoint;
  begin
  result:=FBreakPoint;
  end;

procedure TDBRepeatLine.SetBreakPoint(aBPoint:TOilBreakPoint);
  begin
  if assigned(FBreakPoint) then FreeElement(FBreakPoint);
  FBreakPoint:=aBPoint;
  if aBPoint<>nil then AddElement(FBreakPoint);
  end;

function TDBRepeatLine.GetSourceLine:integer;
  begin
  result:=FSourceLine;
  end;

procedure TDBRepeatLine.SetSourceLine(aLine:integer);
  begin
  FSourceLine:=aLine;
  end;

procedure TDBRepeatLine.DoIt;
  begin
  try
  if BreakPoint<>nil then if BreakPoint.Active then BreakPoint.Pause;
  inherited DoIt;
      except on E:TLangError do
            begin
            if E.Line<0 then E.Line:=SourceLineNum;
            raise;
            end;
    end;
  end;

{CASE}
 {function TCaseLine.GetBlock:TOilLIne;
 begin
  result:=FBlock;
 end;

procedure TCaseLine.SetBlock(aBlock:TOilLIne);
 begin
 if assigned(FBlock) then FreeElement(FBlock);
 AddElement(aBlock);
 FBlock:=aBlock;
 end;
  }
function TCaseLine.GetExp:TOilOperator;
 begin
 result:=FExpression;
 end;

procedure TCaseLine.SetExp(aExp:TOilOperator);
 begin
 if assigned(FExpression) then FreeElement(FExpression);
 AddElement(aExp);
 FExpression:=aExp;
 end;

procedure TCaseLine.SetCaseList(aList:TEntity);
 begin
 if assigned(FCaseList) then FreeElement(FCaseList);
 AddElement(aList);
 FCaseList:=aList;
 end;

function TCaseLine.GetCaseList:TEntity;
 begin
 if not assigned(FCaseList) then
   begin
   FCaseList:=TEntity.create;
   AddElement(FCaseList);
   end;
 result:=FCaseList;
 end;

procedure TCaseLine.SetCaseItem(index:integer;Item:TTypeElement);
 begin
 if CaseList.Count>index then
  begin
  CaseList.InsertElement(index,Item);
  CaseList.FreeElement(CaseList.Elements[index+1]);
  end
  else
  CaseList.AddElement(Item);
 end;

function TCaseLine.GetcaseItem(index:integer):TTypeElement;
 begin
 result:=TTypeElement(CaseList.Elements[index]);
 end;

procedure TCaseLine.SetBlockList(aList:TEntity);
 begin
 if assigned(FBlockList) then FreeElement(FBlockList);
 AddElement(aList);
 FBlockList:=aList;
 end;

function TCaseLine.GetBlockList:TEntity;
 begin
  if not assigned(FBlockList) then
   begin
   FBlockList:=TEntity.create;
   AddElement(FBlockList);
   end;
 result:=FBlockList;
 end;

procedure TCaseLine.SetBlockItem(index:integer;Item:TOilLIne);
 begin
 if BlockList.Count>index then
  begin
  BlockList.InsertElement(index,Item);
  BlockList.FreeElement(BlockList.Elements[index+1]);
  end
  else BlockList.AddElement(Item);
 end;

function TCaseLine.GetBlockItem(index:integer):TOilLIne;
 begin
 result:=TOilLIne(BlockList.Elements[index]);
 end;

function TCaseLine.GetElseBlock:TOilLIne;
 begin
  result:=FElseBlock;
 end;

procedure TCaseLine.SetElseBlock(aBlock:TOilLIne);
 begin
  if assigned(FElseBlock) then FreeElement(FElseBlock);
  AddElement(aBlock);
  FElseBlock:=aBlock;
 end;

procedure TCaseline.DoIt;
var i:longint;
    GoOn:boolean;
    TmpObj:TTypeElement;
    Res:TTypeElement;
    CanCompare,Equal:boolean;
 begin
 i:=0;
 GoOn:=true;
 try
 Expression.Doit;
 Res:=TTypeElement(Expression.Output);
 if Res.Typ in [_byte,_char,_integer,_float,_string,_boolean] then
 begin
 while (i < CaseList.Count)and GoOn do
  begin
  TmpObj:=CaseItem[i];
  case TmpObj.typ of
       _boolean:CanCompare:=Res.Typ=_boolean;
       _char,_string:CanCompare:=Res.Typ in [_char,_string];
       _integer,_float:CanCompare:=Res.Typ in [_integer,_float];
       else CanCompare:=false;
       end;
  if CanCompare then
     begin
     Equal:=false;
     case TmpObj.typ of
       _boolean:Equal:=Res.Asboolean=TmpObj.AsBoolean;
       _char,_string:Equal:=Res.AsString=TmpObj.AsString;
       _integer,_float:Equal:=Res.AsFloat=TmpObj.AsFloat;
       end;
     if Equal then if BlockItem[i]<>nil then
          begin
          BlockItem[i].DoIt;
          GoOn:=false;
         end
         else raise TRunTimeError.CreateError(12);
     end;
  inc(i);
  end;
  if GoOn then if ElseBlock<>nil then ElseBlock.DoIt;
  end else raise TRunTimeError.CreateError(13);
  finally
  Expression.Cleandata;
  end;
 end;

{TDBCaseLine}
function TDBCaseLine.GetBreakPoint:TOilBreakPoint;
  begin
  result:=FBreakPoint;
  end;

procedure TDBCaseLine.SetBreakPoint(aBPoint:TOilBreakPoint);
  begin
  if assigned(FBreakPoint) then FreeElement(FBreakPoint);
  FBreakPoint:=aBPoint;
  if aBPoint<>nil then AddElement(FBreakPoint);
  end;

function TDBCaseLine.GetSourceLine:integer;
  begin
  result:=FSourceLine;
  end;

procedure TDBCaseLine.SetSourceLine(aLine:integer);
  begin
  FSourceLine:=aLine;
  end;

procedure TDBCaseLine.DoIt;
  begin
  try
  if BreakPoint<>nil then if BreakPoint.Active then BreakPoint.Pause;
  inherited DoIt;
      except on E:TLangError do
            begin
            if E.Line<0 then E.Line:=SourceLineNum;
            raise;
            end;
    end;
  end;

{TCaller}
function TCaller.GetParams:TNameEntity;
   begin
   if not assigned(FParams) then
      begin
      FParams:=TNameEntity.create;
      AddElement(FParams);
      FParams.name:='Parametry';
      end;
   result:=FParams;
   end;

procedure TCaller.SetParams(aPar:TNameEntity);
   begin
   if assigned(FParams) then FreeElement(FParams);
   AddElement(aPar);
   FParams:=aPar;
   end;

{TProcCaller}
function TProcCaller.GetProc:TOilProc;
   begin
   result:=FProc;
   end;

procedure TProcCaller.SetProc(aProc:TOilProc);
   begin
   if assigned(FProc) then FreeElement(FProc);
   AddElement(aProc);
   FProc:=aProc;
   end;


procedure TProcCaller.DoIt;
   begin
   Output:=nil;
   OilProc.parameters:=Parameters;
   OilProc.Doit;
   Output:=OilProc.FuncResult;
   end;

procedure  TProcCaller.GetTypeInfo;
var i:TType;
	begin
  new(FTypeList);
  For i:=_boolean to _other do TypeItem[i]:=false;
  if OilProc is TOilFunc then
		TypeItem[OilProc.VarType[OilProc.Calls]]:=true;
  end;

{TParametr}
function TParametr.GetExp:TOilOperator;
 begin
 result:=FExp;
 end;

procedure TParametr.SetExp(aExp:TOilOperator);
 begin
 if FExp<>nil then FreeElement(FExp);
 AddElement(aExp);
 FExp:=aExp;
 end;


{TOilFunc}
function TOilFunc.GetFuncResult:TTypeElement;
 begin
 result:=FFuncResult;
 end;

procedure TOilFunc.DestroyVarsInStack;
var i:integer;
   begin
   if assigned(FVarInfoList) then if VarInfolist.Count>0 then
   	begin
        if VarInfoList.Count>(Calls+1) then
       	   for i:=1 to VarInfoList.Count-Calls-1 do Stack.FreeElement(Variable[Stack.Count-1]);
        Stack.RemoveElement(Variable[Stack.Count-1]);
   	if Calls>0 then for i:=1 to Calls do Stack.FreeElement(Variable[Stack.Count-1]);
   	end;
   end;


procedure TOilFunc.DoIt;
var i:longint;
	begin
  CreateVarsInStack;
  ReadParameters;
  if PrgLineList.Count>0 then
  for i:=0 to PrgLinelist.Count-1 do
    if PrgLine[i]<>nil then PrgLine[i].DoIt;
  FFuncResult:=Variable[StackIndex+Calls].SelfCopy;
  DestroyVarsInStack;
  end;

{TDBOilFunc}
procedure TDBOilFunc.CreateVarsInStack;
var i:integer;
  begin
  inherited CreateVarsInStack;
  if assigned(FVarInfoList) then if VarInfolist.Count>0 then
      for i:=0 to VarInfolist.Count-1 do
      NewDBVarName(VarDBInfoName[i]);
  end;

procedure TDBOilFunc.DestroyVarsInStack;
var i,j:integer;
  begin
  inherited DestroyVarsInStack;
  if assigned(FVarInfoList) then if VarInfolist.Count>0 then
    begin
    j:=VarInfolist.Count;
    for i:=1 to j do ClearLastDBVarName;
    end;
  end;

procedure TDBOilFunc.SetNameStack(aStack:TStringListElement);
 begin
  if assigned(FNameStack) then FreeElement(FNameStack);
  AddElement(aStack);
  FNameStack:=aStack;
 end;

function TDBOilFunc.GetNameStack:TStringListElement;
 begin
 if not assigned(FNameStack) then
   begin
   FNameStack:=TStringListElement.create;
   FNameStack.name:='NameSTACK'+name;
   AddElement(FNameStack);
   end;
  result:=FNameStack;
 end;

function TDBOilFunc.GetBreakPoint:TOilBreakPoint;
  begin
  result:=FBreakPoint;
  end;

procedure TDBOilFunc.SetBreakPoint(aBPoint:TOilBreakPoint);
  begin
  if assigned(FBreakPoint) then FreeElement(FBreakPoint);
  FBreakPoint:=aBPoint;
  if aBPoint<>nil then AddElement(FBreakPoint);
  end;

function TDBOilFunc.GetSourceLine:integer;
  begin
  result:=FSourceLine;
  end;

procedure TDBOilFunc.SetSourceLine(aLine:integer);
  begin
  FSourceLine:=aLine;
  end;

procedure TDBOilFunc.DoIt;
  begin
  try
  if BreakPoint<>nil then if BreakPoint.Active then BreakPoint.Pause;
  inherited DoIt;
      except on E:TLangError do
            begin
            if E.Line<0 then E.Line:=SourceLineNum;
            raise;
            end;
    end;
  end;

{TProtectBlock}
procedure TProtectBlock.DoIt;
var i:longint;
   begin
   try
   if PrgLineList.Count>0 then
     for i:=0 to PrgLinelist.Count-1 do if PrgLine[i]<>nil then PrgLine[i].Doit;
   except
     on E:TRunTimeError do;
   end;
   end;

{TDBProtectBlock}
procedure TDBProtectBlock.SetNameStack(aStack:TStringListElement);
 begin
  if assigned(FNameStack) then FreeElement(FNameStack);
  AddElement(aStack);
  FNameStack:=aStack;
 end;

function TDBProtectBlock.GetNameStack:TStringListElement;
 begin
 if not assigned(FNameStack) then
   begin
   FNameStack:=TStringListElement.create;
   FNameStack.name:='NameSTACK'+name;
   AddElement(FNameStack);
   end;
  result:=FNameStack;
 end;

procedure TDBProtectBlock.CreateVarsInStack;
var i:integer;
  begin
  inherited CreateVarsInStack;
  if assigned(FVarInfoList) then if VarInfolist.Count>0 then
      for i:=0 to VarInfolist.Count-1 do
      NewDBVarName(VarDBInfoName[i]);
  end;

procedure TDBProtectBlock.DestroyVarsInStack;
var i,j:integer;
  begin
  inherited DestroyVarsInStack;
  if assigned(FVarInfoList) then if VarInfolist.Count>0 then
    begin
    j:=VarInfolist.Count;
    for i:=1 to j do ClearLastDBVarName;
    end;
  end;

function TDBProtectBlock.GetBreakPoint:TOilBreakPoint;
  begin
  result:=FBreakPoint;
  end;

procedure TDBProtectBlock.SetBreakPoint(aBPoint:TOilBreakPoint);
  begin
  if assigned(FBreakPoint) then FreeElement(FBreakPoint);
  FBreakPoint:=aBPoint;
  if aBPoint<>nil then AddElement(FBreakPoint);
  end;

function TDBProtectBlock.GetSourceLine:integer;
  begin
  result:=FSourceLine;
  end;

procedure TDBProtectBlock.SetSourceLine(aLine:integer);
  begin
  FSourceLine:=aLine;
  end;

procedure TDBProtectBlock.DoIt;
  begin
  try
  if BreakPoint<>nil then if BreakPoint.Active then BreakPoint.Pause;
  inherited DoIt;
      except on E:TLangError do
            begin
            if E.Line<0 then E.Line:=SourceLineNum;
            raise;
            end;
    end;
  end;

{AssignLine}
function TAssignLine.GetExpr:TOilOperator;
 begin
  result:=FExpression;
 end;

procedure TAssignLine.SetExpr(aExpr:TOilOperator);
 begin
 if assigned(FExpression) then FreeElement(FExpression);
  AddElement(aExpr);
  FExpression:=aExpr;
 end;

function TAssignLine.GetLeftVar:TVariable;
 begin
  result:=FLeftVar;
 end;

procedure TAssignLine.SetLeftVar(aExpr:TVariable);
 begin
  if assigned(FLeftVar) then FreeElement(FLeftVar);
  FLeftVar:=aExpr;
  AddElement(FLeftVar);
 end;

procedure TAssignLine.DoIt;
var TmpRes:TTypeElement;
begin
 Tmpres:=nil;
 Expression.DoIt;
 TmpRes:=TTypeElement(Expression.Output);
 if (TmpRes=nil) then raise TRunTimeError.CreateError(1);
 if (Leftvar=nil) then raise TRunTimeError.CreateError(2);
 if AssignableTypes(LeftVar.typ,TmpRes.typ) then LeftVar.Constant:=TmpRes
    else
    begin
    Expression.CleanData;
    raise TRunTimeError.CreateError(3);
    end;
end;

{TDBAssignLIne}
function TDBAssignLIne.GetBreakPoint:TOilBreakPoint;
  begin
  result:=FBreakPoint;
  end;

procedure TDBAssignLIne.SetBreakPoint(aBPoint:TOilBreakPoint);
  begin
  if assigned(FBreakPoint) then FreeElement(FBreakPoint);
  FBreakPoint:=aBPoint;
  if aBPoint<>nil then AddElement(FBreakPoint);
  end;

function TDBAssignLIne.GetSourceLine:integer;
  begin
  result:=FSourceLine;
  end;

procedure TDBAssignLIne.SetSourceLine(aLine:integer);
  begin
  FSourceLine:=aLine;
  end;

procedure TDBAssignLIne.DoIt;
  begin
  try
  if BreakPoint<>nil then if BreakPoint.Active then BreakPoint.Pause;
  inherited DoIt;
      except on E:TLangError do
            begin
            if E.Line<0 then E.Line:=SourceLineNum;
            raise;
            end;
    end;
  end;


end.

