unit e_HierTempl;

interface
uses
// KERNEL
   Elements;


type
THierarchyTemplateStatus = (htsClear, htsFull, htsNew);

THierarchyTemplate = class(TOwnerEntity)
private
   FStatus: THierarchyTemplateStatus;
    FHierarchyElement: TElement;
    FTemplateElement: TElement;
   procedure SetHierarchyElement(const Value: TElement);
   procedure SetStatus(const Value: THierarchyTemplateStatus);
    procedure SetDoReset(const Value: boolean);
    procedure SetTemplateElement(const Value: TElement);
    function GetFullPath: string;
public
   procedure Reset;
   procedure GetClearList(var aResult: TEntity);
   procedure GetNewList(var aResult: TEntity);
published
   property FullPath: string read GetFullPath stored false;
   property Status: THierarchyTemplateStatus read FStatus write SetStatus stored false;
   property HierarchyElement: TElement read FHierarchyElement write SetHierarchyElement stored false;
   property TemplateElement: TElement read FTemplateElement write SetTemplateElement stored false;
   property DoReset: boolean write SetDoReset stored false;
end;

implementation
uses
   e_Class;

{ THierarchyTemplate }



procedure THierarchyTemplate.GetClearList(var aResult: TEntity);
var
   i: integer;
begin
   if aResult = nil then aResult := TEntity.Create;
   if Status = htsClear then
   begin
      aResult.AddElement(Self);
      exit;
   end;
   if NoElements then exit;
   for i := 0 to LastIndex do
       THierarchyTemplate(Elements[i]).GetClearList(aResult);
end;

function THierarchyTemplate.GetFullPath: string;
var
   HT: THierarchyTemplate;
begin
   result := '';
   HT := THierarchyTemplate(FindOwnerClass(THierarchyTemplate));
   if HT <> nil then result := HT.FullPath + '\';
   result := result + Name;
end;



procedure THierarchyTemplate.GetNewList(var aResult: TEntity);
var
   i: integer;
begin
   if aResult = nil then aResult := TEntity.Create;
   if Status = htsNew then
   begin
      aResult.AddElement(Self);
      exit;
   end;
   if NoElements then exit;
   for i := 0 to LastIndex do
       THierarchyTemplate(Elements[i]).GetNewList(aResult);
end;

procedure THierarchyTemplate.Reset;
var
   i: integer;
   E: TElement;
   HT: THierarchyTemplate;
begin
   Status := htsClear;
   if NoElements then exit;
   for i := LastIndex downto 0 do
   begin
      E := Elements[i];
      if not (E is THierarchyTemplate) then Continue;
      HT := THierarchyTemplate(E);
      if HT.Status = htsNew then FreeElement(HT)
      else HT.Reset;
   end;
end;

procedure THierarchyTemplate.SetDoReset(const Value: boolean);
begin
   if Value then Reset;
end;

procedure THierarchyTemplate.SetHierarchyElement(const Value: TElement);
var
   S: string;
   E: TEntity;
   El,EHT: TElement;
   HT: THierarchyTemplate;
   i: integer;
begin
   Status := htsFull;
   FHierarchyElement := Value;
   if not(Value is TEntity) then exit;
   E := TEntity(Value);
   if E.NoElements then exit;
   for i := 0 to E.LastIndex do
   begin
      El := E.Elements[i];
      S := El.Name;
      EHT := FindElementName(S);
      if EHT = nil then
      begin
         HT := THierarchyTemplate.Create;
         HT.Name := S;
         HT.Status := htsNew;
         AddElement(HT);
      end
      else
      begin
         HT := THierarchyTemplate(EHT);
         HT.HierarchyElement := El;
      end;
   end;
end;

procedure THierarchyTemplate.SetStatus(
  const Value: THierarchyTemplateStatus);
begin
  FStatus := Value;
end;

procedure THierarchyTemplate.SetTemplateElement(const Value: TElement);
var
   S: string;
   E: TEntity;
   El: TElement;
   HT: THierarchyTemplate;
   i: integer;
begin
   Status := htsClear;
   if not(Value is TEntity) then exit;
   E := TEntity(Value);
   if E.NoElements then exit;
   for i := 0 to E.LastIndex do
   begin
      El := E.Elements[i];
      S := El.Name;
      HT := THierarchyTemplate.Create;
      HT.Name := S;
      AddElement(HT);
      HT.TemplateElement := EL;
   end;
end;

initialization
   RegisterOrbisClass(THierarchyTemplate);
end.
