unit i_ErrorElement;

interface
uses
   Graphics,
   Elements,
   e_Device,
   i_Basic;
type

TErrorGeometryElementInterpret = class(TRectInterpret)
private
    FColor: TColor;
    FBigR: double;
    FSmallR: double;
    procedure SetColor(const Value: TColor);
    procedure SetBigR(const Value: double);
    procedure SetSmallR(const Value: double);
protected
   function  QGoInside(Element: TElement): boolean; override;
   function  QProcessBefore(Element: TElement): boolean; override;
   procedure Process(Element: TElement); override;
   function  IsItsClass(AObject: TObject): boolean; override;
   procedure DrawRect(X, Y, HalfSize: double);
   property  SmallR: double read FSmallR write SetSmallR;
   property  BigR: double read FBigR write SetBigR;
public
   procedure Reset(aDevice: TDevice); override;
   function  CanUniGoInside(Element: TElement): boolean; override;
published
   property Color: TColor read FColor write SetColor;
end;


implementation
uses
   e_Class,
   e_Chk,
   e_Geometry;

const
  PixelR: integer = 4;

{ TErrorGeometryElementInterpret }

function TErrorGeometryElementInterpret.CanUniGoInside(Element: TElement): boolean;
begin
   result := false;
end;

procedure TErrorGeometryElementInterpret.DrawRect(X, Y, HalfSize: double);
begin
   {
   Device.Rectangle(X-HalfSize,Y-HalfSize,X+HalfSize,Y+HalfSize);
   }
   Device.Pen.Style:= psSolid;
   Device.MoveTo(X+HalfSize, Y+HalfSize);
   Device.LineTo(X-HalfSize, Y+HalfSize);
   Device.LineTo(X-HalfSize, Y-HalfSize);
   Device.LineTo(X+HalfSize, Y-HalfSize);
   Device.LineTo(X+HalfSize, Y+HalfSize);
   Device.MoveTo(X, Y);

end;

function TErrorGeometryElementInterpret.IsItsClass(AObject: TObject): boolean;
begin
   result := aObject is TErrorGeometryElement;
end;

procedure TErrorGeometryElementInterpret.Process(Element: TElement);
var
   L: TPolyLineElement;
   A,Are: TAreaElement;
   EE: TErrorGeometryElement;
   P: TPointElement;
   aR: double;
   KillAre: boolean;
   i: integer;
   procedure ProcessArea(aElement: TElement);
   begin
      KillAre := false;
      A := TAreaElement(aElement);
      if A is TPolyLineAreaElement then
      begin
         Are := TPolyLineAreaElement(A).CreateAreaElement;
         KillAre :=  true;
      end
      else
      begin
         Are := A;
      end;
      Device.Brush.Color:= Color;
      Device.Brush.Style:= bsFDiagonal;
      //Device.Pen.Mode:= pmXOR;
      Device.Pen.Width:= 3;
      Device.Pen.Color:= Color;
      Device.Pen.Style:= psSolid;
      Device.PolyPolygon(Are);
      if KillAre then
         Are.Free;
   end;
begin
   if Color = 0 then Color := clGray;
   if not (Element is TErrorGeometryElement) then exit;
   EE := TErrorGeometryElement(Element);
   Element := EE.Geometry;
   if Element is TPolyLineElement then
   begin
      L := TPolyLineElement(Element);
      Device.Pen.Color:= Color;
      Device.Pen.Style:= psSolid;
      //Device.Pen.Mode:= pmXOR;
      Device.Pen.Width:= 1;
      Device.Brush.Style:= bsSolid;
      Device.Brush.Color := clWhite;
      if L.CoordsCount > 1 then
      begin
         DrawRect(L.Xi[0], L.Yi[0],BigR);
         Device.MoveTo(L.Xi[0], L.Yi[0]);
         for i := 1 to L.LastIndex do
         begin
             //Device.Pen.Style:= psDot;
             //Device.Pen.Width:= 1;
             Device.LineTo(L.Xi[i], L.Yi[i]);
             DrawRect(L.Xi[i], L.Yi[i],SmallR)
         end;
      end;
   end
   else
   if Element is TAreaElement then
   begin
      if Element is TMultiAreaElement then
      begin
      for i := 0 to TMultiAreaElement(Element).LastIndex do
      begin
         ProcessArea(TMultiAreaElement(Element).Elements[i])
      end;
      end
      else
         ProcessArea(Element);
   end
   else
   if Element is TPointElement then
   begin
      P := TPointElement(Element);
      Device.Pen.Color:= Color;
      Device.Pen.Style:= psSolid;
      //Device.Pen.Mode:= pmXOR;
      Device.Pen.Width:= 3;
      Device.Brush.Style:= bsSolid;
      Device.Brush.Color := clWhite;
      Device.MoveTo(P.X - BigR*2, P.Y);
      Device.LineTo(P.X + BigR*2, P.Y);
      Device.MoveTo(P.X, P.Y - BigR*2);
      Device.LineTo(P.X, P.Y + BigR*2);
      DrawRect(P.X, P.Y,BigR);
      DrawRect(P.X, P.Y,BigR*2);
   end;
   if (EE.ErrX <> 0) and (EE.ErrY<> 0)then
   begin
      Device.Pen.Color:= 255;
      Device.Pen.Style:= psSolid;
      //Device.Pen.Mode:= pmXOR;
      Device.Pen.Width:= 3;
      DrawRect(EE.ErrX, EE.ErrY,BigR*2);
      {
      Device.Font.Size := 10;
      Device.Font.Color := 255;
      Device.TextOut(EE.ErrX,EE.ErrY-BigR*2,'!');
      }
   end;
   begin
      Device.Pen.Color:= 255;
      Device.Pen.Style:= psDot;
      //Device.Pen.Mode:= pmXOR;
      Device.Pen.Width:= 1;
      Device.MoveTo(EE.MinX, EE.MinY);
      Device.LineTo(EE.MaxX, EE.MinY);
      Device.LineTo(EE.MaxX, EE.MaxY);
      Device.LineTo(EE.MinX, EE.MaxY);
      Device.LineTo(EE.MinX, EE.MinY);

   end;
end;

function TErrorGeometryElementInterpret.QGoInside(
  Element: TElement): boolean;
begin
   result := false;
end;

function TErrorGeometryElementInterpret.QProcessBefore(Element: TElement): boolean;
begin
   result := (Element is TErrorGeometryElement);
end;

procedure TErrorGeometryElementInterpret.Reset(aDevice: TDevice);
begin
   inherited Reset(aDevice);
   SmallR := Device.Projection.GetModelLength(PixelR);
   BigR := Device.Projection.GetModelLength(PixelR+2);
end;

procedure TErrorGeometryElementInterpret.SetBigR(const Value: double);
begin
  FBigR := Value;
end;

procedure TErrorGeometryElementInterpret.SetColor(const Value: TColor);
begin
   FColor := Value;
end;

procedure TErrorGeometryElementInterpret.SetSmallR(const Value: double);
begin
  FSmallR := Value;
end;

INITIALIZATION
   RegisterOrbisClass(TErrorGeometryElementInterpret);
END.
