unit t_Chk_TAX_LED;

interface
uses
  e_LesCel,
  e_LHCModel,
  Elements,
  t_CmdLn,
  Tools;


type

TCHeck_TAX_LED = class(TTool)
  private
    FScriptIsReady: boolean;
    FUserE: TEntity;
    FActualUserError: TElement;
    FLesniCelek: TLesniCelek;
    FDETLayer: TEntity;
    FForestModel: Tgis_ForestModel;
    FProtocol: boolean;
    FUseMaxValue: boolean;
    FMaxValue: integer;
    F_Vymera: boolean;
    F_Plocha: boolean;
    F_Tolerance: double;
    F_KVAL_P: boolean;
    FActualTAXisPorostniSkupina: boolean;
    FActualLEDElement: TElement;
    FActualTAXElement: TElement;
    FUserScript: string;
    FCmdLn: TCommandLine;
    FTAXDirectory: string;
    FLedNoEtaz: boolean;
    FLEDForcePorost: boolean;
    FConsistency: integer;
    function GetCmdLn: TCommandLine;
    procedure SetScriptIsReady(const Value: boolean);
    procedure SetNextUserError(const Value: string);
    procedure SetUserE(const Value: TEntity);
    procedure SetActualUserError(const Value: TElement);
    procedure SetLesniCelek(const Value: TLesniCelek);
    procedure SetDETLayer(const Value: TEntity);
    procedure SetForestModel(const Value: Tgis_ForestModel);
    procedure SetProtocol(const Value: boolean);
    procedure SetMaxValue(const Value: integer);
    procedure SetUseMaxValue(const Value: boolean);
    procedure Set_Plocha(const Value: boolean);
    procedure Set_Tolerance(const Value: double);
    procedure Set_Vymera(const Value: boolean);
    procedure Set_KVAL_P(const Value: boolean);
    procedure SetActualLEDElement(const Value: TElement);
    procedure SetActualTAXElement(const Value: TElement);
    procedure SetActualTAXisPorostniSkupina(const Value: boolean);
    procedure SetUserScript(const Value: string);
    procedure PrepareScript;
    property  CmdLn: TCommandLine read GetCmdLn;
    property  ScriptIsReady: boolean read FScriptIsReady write SetScriptIsReady;
    procedure SetTAXDirectory(const Value: string);
    procedure SetLedNoEtaz(const Value: boolean);
    procedure SetLEDForcePorost(const Value: boolean);
    procedure SetConsistency(const Value: integer);
public
   procedure DoIt; override;
   function  CreateLesniCelek: boolean;
   function  ErrElement(const S: string): TOwnerElement;
   procedure DestroyLesniCelek;
   property  LesniCelek: TLesniCelek read FLesniCelek write SetLesniCelek;
published
   property  ForestModel: Tgis_ForestModel read FForestModel write SetForestModel;
   property  DETLayer: TEntity read FDETLayer write SetDETLayer;
   property  Protocol: boolean read FProtocol write SetProtocol;
   property  MaxValue: integer read FMaxValue write SetMaxValue;
   property  UseMaxValue: boolean read FUseMaxValue write SetUseMaxValue;
   property  _Plocha: boolean read F_Plocha write Set_Plocha;
   property  _Vymera: boolean read F_Vymera write Set_Vymera;
   property  _Tolerance: double read F_Tolerance write Set_Tolerance;
   property  _KVAL_P: boolean read F_KVAL_P write Set_KVAL_P;
   property  ActualTAXElement: TElement read FActualTAXElement write SetActualTAXElement;
   property  ActualLEDElement: TElement read FActualLEDElement write SetActualLEDElement;
   property  ActualTAXisPorostniSkupina: boolean read FActualTAXisPorostniSkupina write SetActualTAXisPorostniSkupina;
   property  UserScript: string read FUserScript write SetUserScript;
   property  UserE: TEntity read FUserE write SetUserE;
   property  NextUserError: string write SetNextUserError;
   property  ActualUserError: TElement read FActualUserError write SetActualUserError;
   property  TAXDirectory: string read FTAXDirectory write SetTAXDirectory;
   property  LedNoEtaz: boolean read FLedNoEtaz write SetLedNoEtaz;
   property  LEDForcePorost: boolean read FLEDForcePorost write SetLEDForcePorost;
   property  Consistency: integer read FConsistency write SetConsistency;
end;



implementation
uses
   Forms,
   SysUtils,
   Dialogs,
   u_Math,
   eLesObec,
   e_Class,
   e_GeometryElement,
   e_LesOdd,
   G_elems,
   m_Orbis,
   u_String,
   t_Select,
   Worlds;

{ TCHeck_TAX_LED }

function TCHeck_TAX_LED.CreateLesniCelek: boolean;
begin
   result := true;
   LesniCelek := TLesniCelek.create;
   World.objectmodel.AddElement(LesniCelek);
   LesniCelek.NajdiTypDatLHP(false,TAXDirectory,false,true,true); // nalezen dat TAXu
   if LesniCelek.TypDatLHP=NeniLHP then
   begin
      showmessage('V adresi "" nejsou data LHP');
      LesniCelek.Free;
      LesniCelek := nil;
      result := false;
      exit;
   end;
   LesniCelek.nactiPrvniOddeleni;
end;

procedure TCHeck_TAX_LED.DestroyLesniCelek;
begin
   LesniCelek.Free;
end;

procedure TCHeck_TAX_LED.DoIt;
var
   j,k: integer;
   OE: TOwnerEntity;
   GE: TOwnerEntity;
   TE: TRectEntity;
   PE: TRectEntity;
   Q: TQuickEntity;
   E: TElement;
   S,SS,SV: string;
   LHC: integer;
   ODD: integer;
   DIL: string;
   POR: string;
   PSK: string;
   GEL: TGeometryElement;
   ErrG: TRectEntity;
   PSKDIL: TOwnerEntity;
   iChk,iErr: integer;
{ TODO -oPPK -cDodlat : kontrola BZL a tak }

   procedure CreateErrG;
   begin
      if ErrG <> nil then exit;
      ErrG := TRectEntity.Create;
      ErrG.UpdateRect(ForestModel.ActualSkupina);
      ErrG.Name := ' Props ' +IntToStr(ODD)+' '+DIL+' '+POR+' '+PSK;
      PE.AddAndUpdateRect(ErrG);
   end;

   procedure ProcessLesniPlosnyPrvek(_LP: TLesniPlosnyPrvek; const Str: string);
   var
      D1,D2: double;
      SS28: string;
      _PSK: Tgis_PorostniSkupina;
   begin
      if _LP = nil then exit;
      ActualTAXElement := _LP;
      Inc(iChk);
      if LedNoEtaz then
         StringAsTwoParams(PSK,'/',PSK,SS28);
      if LEDForcePorost then
      begin
         _PSK := Tgis_PorostniSkupina(PSKDIL.FindElementName(PSK));
         if _PSK <> nil then
         begin
           _PSK._Porost := ForestModel.ActualPorost;
         end;
      end;
      if ForestModel.GetSkupinaName(PSK) = nil then
      begin
         inc(iErr);
         GE.NextElement := ErrElement(Str+IntToStr(ODD)+' '+DIL+' '+POR+' '+PSK);
         exit;
      end;
      Q.AddElement(ForestModel.ActualSkupina);
      ActualLEDElement := ForestModel.ActualSkupina;
      ErrG := nil;
      if _Plocha then
      begin
         D1 := ForestModel.ActualSkupina.Plocha;
         D2 := _LP.Plocha0;
         if Abs(D1 - D2) > _Tolerance then
         begin
            CreateErrG;
            D1 := Simple_RoundTo(D1);
            D2 := Simple_RoundTo(D2);
            ErrG.NextElement := ErrElement('  PLOCHA L:'+FloatToStr(D1)+' T:'+FloatToStr(D2));
         end;
      end;
      if _Vymera then
      begin
         D1 := ForestModel.ActualSkupina.Vymera;
         D2 := _LP.Vymera;
         if Abs(D1 - D2) > _Tolerance then
         begin
            CreateErrG;
            D1 := Simple_RoundTo(D1);
            D2 := Simple_RoundTo(D2);
            ErrG.NextElement := ErrElement('  VYMERA L:'+FloatToStr(D1)+' T:'+FloatToStr(D2));
         end;
      end;
      if _KVAL_P then
      begin
         if ForestModel.ActualSkupina.KVAL_P <> _LP.KVAL_P then
         begin
            CreateErrG;
            ErrG.NextElement := ErrElement('  KVAL_P L:'+IntToStr(ForestModel.ActualSkupina.KVAL_P)+' T:'+IntToStr(_LP.KVAL_P));
         end;
      end;
      if ScriptIsReady then
      begin
         CmdLn.Input := Self;
         CmdLn.DoIt;
      end;
   end;

   procedure processSkupina(_PSK: TPorostniSkupina);
   begin
      if _PSK = nil then exit;
      PSK := RemoveAllSpaces(_PSK.PSK);
      ProcessLesniPlosnyPrvek(_PSK,'PSK ');
   end;

   procedure processBZL(_BZL: TBezlesi);
   begin
      if _BZL = nil then exit;
      if UseMaxValue and (_BZL.BZL > MaxValue) then
         exit;
      PSK := IntToStr(_BZL.BZL);
      ProcessLesniPlosnyPrvek(_BZL,'BZL ');
   end;

   procedure processOP(_OP: TOstatni);
   begin
      if _OP = nil then exit;
      if UseMaxValue and (_OP.OST > MaxValue) then
         exit;
      PSK := IntToStr(_OP.OST);
      ProcessLesniPlosnyPrvek(_OP,'OP  ');
   end;

   procedure processJP(_JP: TJinyPozemek);
   begin
      if _JP = nil then exit;
      if UseMaxValue and (_JP.JP > MaxValue) then
         exit;
      PSK := IntToStr(_JP.JP);
      ProcessLesniPlosnyPrvek(_JP,'JP  ');
   end;

   procedure processPorost(_POR: TPorost);
   var
      i: integer;
   begin
      if _POR = nil then exit;
      POR := _POR.POR;
      if LEDForcePorost then
         ForestModel.CreatePorostName(POR)
      else
      if ForestModel.GetPorostName(POR) = nil then
      begin
         Inc(IChk);
         Inc(IErr);
         GE.NextElement := ErrElement('Cel POR '+IntToStr(ODD)+' '+DIL+' '+POR);
         exit;
      end;
      ActualTAXisPorostniSkupina := true;
      for i := 0 to _POR.PorostniSkupinaMN.LastIndex do
      begin
         processSkupina(TPorostniSkupina(_POR.PorostniSkupinaMN.Elements[i]));
      end;
      ActualTAXisPorostniSkupina := false;
      for i := 0 to _POR.BezlesiMN.LastIndex do
      begin
         processBZL(TBezlesi(_POR.BezlesiMN.Elements[i]));
      end;
      for i := 0 to _POR.OstatniMN.LastIndex do
      begin
         processOP(TOstatni(_POR.OstatniMN.Elements[i]));
      end;
      for i := 0 to _POR.JinyPozemekMN.LastIndex do
      begin
         processJP(TJinyPozemek(_POR.JinyPozemekMN.Elements[i]));
      end;
   end;

   procedure processDilec(_DIL: TDilec);
   var
      i: integer;
      Sel: TAllSelector;
   begin
      if _DIL = nil then exit;
      DIL := _DIL.DIL;
      if ForestModel.GetDilecName(DIL) = nil then
      begin
         inc(Ichk);
         inc(IErr);
         GE.NextElement := ErrElement('Cel DIL '+IntToStr(ODD)+' '+DIL);
         exit;
      end;
      if LEDForcePorost then
      begin
         PSKDIL := TOwnerEntity.Create;
         AddElement(PSKDIL);
         Sel := TAllSelector.Create;
         Sel.Input := ForestModel.ActualDilec;
         Sel.Output := PSKDIL;
         Sel.Condition := 'CLASS(Tgis_PorostniSkupina)';
         Sel.DoIt;
         Sel.Free;
      end;
      for i := 0 to _DIL.PorostMN.LastIndex do
      begin
         ProcessPorost(TPorost(_DIL.PorostMN.Elements[i]))
      end;
      if LEDForcePorost then
         PSKDIL.Free;
   end;

   procedure processOddeleni(_ODD: TOddeleni);
   var
      i: integer;
   begin
      if _ODD = nil then exit;
      ODD := _ODD.ODD;
      ForestModel.ActualLS := nil;
      if ForestModel.GetOddeleniCislo(ODD) = nil then
      begin
         Inc(IChk);
         Inc(IErr);
         GE.NextElement := ErrElement('Cel ODD '+IntToStr(ODD));
         exit;
      end;
      for i := 0 to _ODD.DilecMN.LastIndex do
      begin
         processDilec(TDilec(_ODD.DilecMN.Elements[i]));
      end;
   end;

   function CHeckMaxValue(aE: TElement): boolean;
   var
      ii: integer;
      PSKNumber: integer;
   begin
      result := false;
      if UseMaxValue  then
      begin
         SV := '';
         SS := aE.Name;
         ii := 1;
         if SS <> '' then
         begin
            while not (ii > Length(SS)) and (SS[ii]=' ') do Inc(ii);
            while not (ii > Length(SS)) and (SS[ii] in ['0'..'9']) do
            begin
               SV := SV + SS[ii];
               Inc(ii);
            end;
            PSKNumber := StrToIntDef(SV,0);
            if (PSKNumber >= MaxValue) then
            begin
               result := true;
            end;
         end;
      end;
   end;
begin
   if ForestModel = nil then
   begin
      ShowMessage('Nen nastaven model LHC');
      exit;
   end;
   if (DetLayer = nil) or DetLayer.NoElements then
   begin
      ShowMessage('Nen nastavena vrstva Ploch DET nebo je przdn');
      exit;
   end;
   if not CreateLesniCelek then exit;
   iChk := 0;
   iErr := 0;
   PrepareScript;
   StartProgress('Kontrola Grafiky a TAXu',0,LesniCelek.SeznamOddeleni.Count -1);
   OE := TOwnerEntity.Create;
   AddElement(OE);
   Output := OE;
   OE.Name := 'Kontrola LED TAX';
   GE := TOwnerEntity.Create;
   OE.AddElement(GE);
   GE.Name := 'Chyb v Grafice';
   TE := TRectEntity.Create;
   OE.AddElement(TE);
   TE.Name := 'Chyb v TAXu';
   PE := TRectEntity.Create;
   OE.AddElement(PE);
   PE.Name := 'Nesoulad vlastnost';
   if ScriptIsReady then
   begin
      UserE := TRectEntity.Create;
      OE.AddElement(UserE);
      UserE.Name := 'USER';
   end;
   Q := TQuickEntity.Create;
   j := 0;
   LHC := -1;
   if MaxValue = 0 then
      UseMaxValue := false;
   ForestModel.LHC_Cislo:= LesniCelek.LHC.LHC_KOD;
   if ForestModel.ActualLHC = nil then
   begin
      if not ForestModel.NoElements then
      begin
         ForestModel.ActualLHC := Tgis_LHC(ForestModel.Elements[0]);
         LHC := ForestModel.ActualLHC.Cislo;
      end;
      GE.NextElement := ErrElement('! slo LHC '+'TAX '+IntToStr(LesniCelek.LHC.LHC_KOD)+' LED '+IntToStr(LHC));
   end
   else
      LHC := ForestModel.ActualLHC.Cislo;
   if LHC = -1 then
   begin
      DestroyLesniCelek;
      exit;
   end;
   for j := 0 to LesniCelek.SeznamOddeleni.Count - 1 do
   begin
      progress := j;
      ODD := LesniCelek.LHC.Oddeleni.ODD;
      ProgressText := 'ODD: '+IntToStr(LesniCelek.LHC.Oddeleni.ODD);
      Application.ProcessMessages;
      ProcessOddeleni(LesniCelek.LHC.Oddeleni);
      if j < (LesniCelek.SeznamOddeleni.Count -1) then
         LesniCelek.nactiOddeleni(true);
   end;
   Consistency := 100 - Round((iErr / IChk) * 100);
   {
   while true do
   begin
      progress := j;
      ODD := LesniCelek.LHC.Oddeleni.ODD;
      ProgressText := 'ODD: '+IntToStr(LesniCelek.LHC.Oddeleni.ODD);
      Application.ProcessMessages;
      ProcessOddeleni(LesniCelek.LHC.Oddeleni);
      if LesniCelek.PosledniOddeleni then break;
      LesniCelek.nactiOddeleni(true);
      inc(j);
   end;
   }
   //MaxProgress := DETLayer.LastIndex;
   for j := 0 to DETLayer.LastIndex do
   begin
      E := DETLayer.Elements[j];
      if not (E is TGeometryElement) then continue;
      GEL := TGeometryElement(E);
      if GEL.NoGeometry then continue;
      if Q.IndexOf(E) < 0 then
      begin
         if not CheckMaxValue(E) then
         begin
            ErrG := TRectEntity.Create;
            TE.NextElement := ErrG;
            ErrG.AddAndUpdateRect(e);
            S := E.GetPropertyStringByName('Oddeleni')+' ';
            S := S + E.GetPropertyStringByName('DILEC')+' ';
            S := S + E.GetPropertyStringByName('Porost')+' ';
            S := S + E.Name;
            ErrG.Name := S;
         end;
      end;
   end;
   TE.Update;
   if Protocol then
   begin
      PostTextMsg(tm_AddToProtocol,'****************',nil);
      PostTextMsg(tm_AddToProtocol,'Kontrola TAX LED',nil);
      PostTextMsg(tm_AddToProtocol,'****************',nil);
      PostTextMsg(tm_AddToProtocol,GE.Name,nil);
      PostTextMsg(tm_AddToProtocol,' ',nil);
      Application.ProcessMessages;
      for j := 0 to GE.LastIndex do
      begin
         if j mod 10 = 0 then
            Application.ProcessMessages;
         PostTextMsg(tm_AddToProtocol,GE.Elements[j].Name,nil);
      end;

      PostTextMsg(tm_AddToProtocol,PE.Name,nil);
      PostTextMsg(tm_AddToProtocol,' ',nil);
      for j := 0 to PE.LastIndex do
      begin
         if j mod 10 = 0 then
           Application.ProcessMessages;
         ErrG := TRectEntity(PE.Elements[j]);
         PostTextMsg(tm_AddToProtocol,ErrG.Name,nil);
         for k := 0 to ErrG.LastIndex do
         begin
            PostTextMsg(tm_AddToProtocol,ErrG.Elements[k].Name,nil);
         end;
      end;

      PostTextMsg(tm_AddToProtocol,' ',nil);
      PostTextMsg(tm_AddToProtocol,TE.Name,nil);
      PostTextMsg(tm_AddToProtocol,' ',nil);
      for j := 0 to TE.LastIndex do
      begin
         if j mod 10 = 0 then
            Application.ProcessMessages;
         E := TE.Elements[j];
         S := E.Name;
         PostTextMsg(tm_AddToProtocol,S,nil);
      end;
      PostTextMsg(tm_AddToProtocol,'****************',nil);
   end;
   Q.Clear;
   Q.Free;
   DestroyLesniCelek;
end;

function TCHeck_TAX_LED.ErrElement(const S: string): TOwnerElement;
begin
   result := TOwnerElement.Create;
   result.Name := S;
end;

function TCHeck_TAX_LED.GetCmdLn: TCommandLine;
begin
   if FCmdLn = nil then
   begin
      FCmdLn := TCommandLine.Create;
      AddElement(FCmdLn);
   end;
   result := FCmdLn;
end;

procedure TCHeck_TAX_LED.PrepareScript;
var
   S: string;
begin
   ScriptIsReady := false;
   if UserScript = '' then exit;
   S := GetAbsolutePath(UserScript);
   if not FileExists(S) then exit;
   CmdLn.FileName := S;
   CmdLn.Lines.LoadFromFile(S);
   CmdLn.Input := Self;
   ScriptIsReady := true;
end;

procedure TCHeck_TAX_LED.SetActualLEDElement(const Value: TElement);
begin
  FActualLEDElement := Value;
end;

procedure TCHeck_TAX_LED.SetActualTAXElement(const Value: TElement);
begin
  FActualTAXElement := Value;
end;

procedure TCHeck_TAX_LED.SetActualTAXisPorostniSkupina(
  const Value: boolean);
begin
  FActualTAXisPorostniSkupina := Value;
end;

procedure TCHeck_TAX_LED.SetActualUserError(const Value: TElement);
begin
  FActualUserError := Value;
end;

procedure TCHeck_TAX_LED.SetDETLayer(const Value: TEntity);
begin
  FDETLayer := Value;
end;

procedure TCHeck_TAX_LED.SetForestModel(const Value: Tgis_ForestModel);
begin
  FForestModel := Value;
end;

procedure TCHeck_TAX_LED.SetLedNoEtaz(const Value: boolean);
begin
  FLedNoEtaz := Value;
end;

procedure TCHeck_TAX_LED.SetLEDForcePorost(const Value: boolean);
begin
  FLEDForcePorost := Value;
end;

procedure TCHeck_TAX_LED.SetLesniCelek(const Value: TLesniCelek);
begin
  FLesniCelek := Value;
end;

procedure TCHeck_TAX_LED.SetMaxValue(const Value: integer);
begin
  FMaxValue := Value;
end;

procedure TCHeck_TAX_LED.SetNextUserError(const Value: string);
var
   ErrG: TRectEntity;
begin
   if UserE = nil then exit;
   ErrG := TRectEntity.Create;
   ErrG.UpdateRect(ForestModel.ActualSkupina);
   ErrG.Name := Value;
   if UserE is TRectEntity then
      TRectEntity(UserE).AddAndUpdateRect(ErrG)
   else
      UserE.AddElement(ErrG);
   ActualUserError := ErrG;
end;

procedure TCHeck_TAX_LED.SetProtocol(const Value: boolean);
begin
  FProtocol := Value;
end;

procedure TCHeck_TAX_LED.SetScriptIsReady(const Value: boolean);
begin
  FScriptIsReady := Value;
end;

procedure TCHeck_TAX_LED.SetTAXDirectory(const Value: string);
begin
  FTAXDirectory := Value;
end;

procedure TCHeck_TAX_LED.SetUseMaxValue(const Value: boolean);
begin
  FUseMaxValue := Value;
end;

procedure TCHeck_TAX_LED.SetUserE(const Value: TEntity);
begin
  FUserE := Value;
end;

procedure TCHeck_TAX_LED.SetUserScript(const Value: string);
begin
  FUserScript := Value;
end;

procedure TCHeck_TAX_LED.Set_KVAL_P(const Value: boolean);
begin
  F_KVAL_P := Value;
end;

procedure TCHeck_TAX_LED.Set_Plocha(const Value: boolean);
begin
  F_Plocha := Value;
end;

procedure TCHeck_TAX_LED.Set_Tolerance(const Value: double);
begin
  F_Tolerance := Value;
end;

procedure TCHeck_TAX_LED.Set_Vymera(const Value: boolean);
begin
  F_Vymera := Value;
end;

procedure TCHeck_TAX_LED.SetConsistency(const Value: integer);
begin
  FConsistency := Value;
end;

INITIALIZATION
   RegisterOrbisClass(TCHeck_TAX_LED);
END.

