unit u_Symbol;

interface
uses
// GLIB
   G_Elems;

type
   TCodePage = (cpIBM, cpLatin2, cpKamenickych, cpWindows);
   TTextAtrib  = set of (ZrusenT,OkT, txtZmena, T30);

const
    PathOfLibraries: string = 'C:\TTRIZENI';
    StrictRects: boolean = false;
    DefaultCodePage: TCodePage = cpLatin2;
    Absolutne = 0;
    Relativne = 1;
    Meritkove = 2;

Type

  TBGIBaseElement = class(T2DCoordinate)
   public
     FCislo   : longInt;
     FDruh    : smallInt;
     FBarva   : smallInt;
     FZnacka  : smallInt;
     FPresnost: byte;
  published
     property Cislo:    longInt read FCislo    write FCislo;
     property Druh:     smallInt read FDruh     write FDruh;
     property Barva:    smallInt read FBarva    write FBarva;
     property Znacka:   smallInt read FZnacka   write FZnacka;
     property Presnost: byte    read FPresnost write FPresnost;
  end;


  TRectBGIElement = class(TBGIBaseElement)
   protected
     FRect    : TRectangle;
     function  GetRect: TRectangle; virtual;
     procedure SetRect(aRect: TRectangle);
   public
     destructor  Destroy; override;
     function    GetRegion: TDoubleRect; override;
     function    IsInRegion(Minx, MinY, MaxX, MaxY: double): Boolean;
     function    IsOverRegion(Minx, MinY, MaxX, MaxY: double): Boolean;

     property Rect: TRectangle read GetRect write SetRect;
  end;




  TBGIText = class(TRectBGIElement)
  private
     FCodePage: TCodePage;

     procedure SetCodePage(aCode: TCodePage);
     function GetCodePage: TCodePage;
   protected
     FVychoziBod : TCoordinate;
     FVeta: string;
     function  GetVeta: string;
     procedure SetVeta(aVeta: string);
     function  GetVychoziBod: TCoordinate;
     procedure SetVychoziBod(aVychoziBod: TCoordinate);
   public
     Font       : byte;
     Vyska      : double;
     KoefSirky  : double;
     Smer       : double; { Uhel v Radianech }
     Sklon      : double; { Uhel v Radianech }
     Rozestupy  : double; { Zvetseni default hodnoty v Bodech }
     Atributy   : TTextAtrib;
     RezimTextu : byte;

     destructor  Destroy; override;
     property    VychoziBod: TCoordinate read GetVychoziBod write SetVychoziBod;
  published
     property CodePage: TCodePage read GetCodePage write SetCodePage;
     Property Veta : String read GetVeta write SetVeta;
  end;




implementation


{ TRectBGIElement }


function  TRectBGIElement.GetRect: TRectangle;
begin
   if not assigned(FRect) then FRect := TRectangle.Create;
   result := FRect;
end;

procedure TRectBGIElement.SetRect(aRect: TRectangle);
begin
   if Assigned(FRect) then FRect.Free;
   FRect := nil;
   FRect := aRect;
end;


destructor  TRectBGIElement.Destroy;
begin
   Rect := nil;
   inherited Destroy;
end;

function   TRectBGIElement.IsInRegion(Minx, MinY, MaxX, MaxY: double): Boolean;
begin
   result:= (MinX <= Rect.Min.X) and (MaxX >= Rect.Max.x) and
            (MinY <= Rect.Min.Y) and (MaxY >= Rect.Max.y)
end;


function   TRectBGIElement.IsOverRegion(Minx, MinY, MaxX, MaxY: double): Boolean;
begin
   result := false;
   if (MinX > Rect.Max.X) or (MaxX < Rect.Min.x) or
      (MinY > Rect.Max.Y) or (MaxY < Rect.Min.y) then exit;
   result := true;
end;



function   TRectBGIElement.GetRegion: TDoubleRect;
begin
   result := Rect.AsDoubleRect;
end;


{ TBGIText }

destructor  TBGIText.Destroy;
begin
   Veta := '';
   inherited Destroy;
end;


procedure TBGIText.SetCodePage(aCode: TCodePage);
begin
  FCodePage:= aCode
end;

function TBGIText.GetCodePage: TCodePage;
begin
  result:= FCodePage
end;

function    TBGIText.GetVeta: string;
begin
   result := FVeta;
end;

procedure   TBGIText.SetVeta(aVeta: string);
begin
   FVeta := aVeta;
end;

function TBGIText.GetVychoziBod: TCoordinate;
begin
   if not assigned(FVychoziBod) then FVychoziBod := T2DCoordinate.Create;
   result := FVychoziBod;
end;

procedure   TBGIText.SetVychoziBod(aVychoziBod: TCoordinate);
begin
   if assigned(FVychoziBod) then FVychoziBod.Free;
   FVychoziBod := nil;
   FVychoziBod := aVychoziBod;
end;



BEGIN
END.
