unit t_TabletManager;

interface
uses
   Dialogs,
   Windows,
   d_Tablet,
{$IFDEF WINTAB}
   WinTablet,
   WintabConsts,
{$ENDIF}
   Elements,
   f_TabletDialog,
   m_Intera,
   r_TabletManager,
   t_OrbisFrame,
   e_Transformation,
   t_Transformation;

type
TtmfStatus=(tmfsNIL,tmfsEditTransformation);
TTrEditStatus=(tresNIL,tresWaitingForMouse,tresWaitingForTablet);

TTabletManagerFrame = class(TOrbisFrame)
private
    FConfigFile: string;
    FTransformationTableFile: string;
    FDialog: TTabletDialog_Form;
    FTransformation: TTransformation_Afinni2D;
    FTransformationTable: TTransformationTable;
    FTablet: TTablet;
    FTest: boolean;
    FMoveTolerance: integer;
    FtmfStatus: TtmfStatus;
    FTrEditStatus: TTrEditStatus;
    FLastY: integer;
    FLastX: integer;
    FTY: double;
    FTX: double;
    FCounter: integer;
    FCursorVisible: boolean;
    FCharDelayTime: DWORD;
    FExtraDelayTime: Word;
    FWintab: boolean;
{$IFDEF WINTAB}
    FWinTablet: TWin_Tablet;
{$ENDIF}
    FLastButton: integer;
   function GetTMFrame: TTabletManager_Frame;
    procedure SetAction_Run(const Value: boolean);
    procedure SetAction_Settings(const Value: boolean);
    procedure SetAction_Transform(const Value: boolean);
    procedure SetConfigFile(const Value: string);
    procedure SetTransformationTableFile(const Value: string);
    procedure SetDialog(const Value: TTabletDialog_Form);
    procedure SetTablet(const Value: TTablet);
    procedure SetTransformation(const Value: TTransformation_Afinni2D);
    function GetTablet: TTablet;
    function GetTransformation: TTransformation_Afinni2D;
    procedure SetTest(const Value: boolean);
    procedure SetMoveTolerance(const Value: integer);
    procedure SettmfStatus(const Value: TtmfStatus);
    procedure SetTrEditStatus(const Value: TTrEditStatus);
    function GetTransformationTable: TTransformationTable;
    procedure SetTransformationTable(const Value: TTransformationTable);
    procedure SetLastX(const Value: integer);
    procedure SetLastY(const Value: integer);
    procedure SetAction_ShowTrafo(const Value: boolean);
    procedure SetCursorVisible(const Value: boolean);
    function GetBActionsString: string;
    procedure SetBActionsString(const Value: string);
    procedure SetExtraDelayTime(const Value: Word);
    procedure SetCharDelayTime(const Value: DWORD);
    {$IFDEF WINTAB}
    procedure SetWintab(const Value: boolean);
    procedure SetWinTablet(const Value: TWin_Tablet);
    function GetWinTablet: TWIN_Tablet;
    {$ENDIF}
    procedure SetLastButton(const Value: integer);
protected
   FBActions: Array[1..4] of integer;
   procedure DrawCross(aX,aY: integer);
   procedure DrawR(aX,aY: integer; aText: string);
   procedure ShowTrafo;
   procedure XORDraw;
   procedure LoadTable;
   procedure SaveTable;
   property LastX: integer read FLastX write SetLastX;
   property LastY: integer read FLastY write SetLastY;
   property TX: double read FTX write FTX;
   property TY: double read FTY write FTY;
   property Counter: integer read FCounter write FCounter;
public
   procedure CreateFrame; override;
   procedure RegisterCursors; override;
   procedure On_Activate; override;
   procedure EndFlashTool; override;
   procedure On_tm_MouseDown(var aMessage: TOrbisMouseEvent; var Handled: boolean); override;
   procedure On_tm_Review(var aMessage: TOrbisMessage; var Handled: boolean); override;
   procedure TabletProcedure(aX,aY:double; Button: integer; ButtonChanged: boolean);
   procedure TabletProcessError(Sender: TObject; Place,Code: DWord; Msg: String);
{$IFDEF WINTAB}
   procedure WTabletPacket(Sender: TObject; PacketNumber, ContextHandle: HCtx; pPacket: Pointer);
   procedure WTabletCursorChanged(Sender: TObject; PacketNumber, ContextHandle: HCtx; pPacket: Pointer);
   procedure WTabletProximity(Sender: TObject; ContextHandle: HCtx; Entering: Boolean);
   function  WTablet_Open: boolean;
{$ENDIF}
   property  tmfStatus: TtmfStatus read FtmfStatus write SettmfStatus;
   property  TrEditStatus: TTrEditStatus read FTrEditStatus write SetTrEditStatus;
   property  TMFrame: TTabletManager_Frame read GetTMFrame;
   property  Dialog: TTabletDialog_Form read FDialog write SetDialog;
   property  Tablet: TTablet read GetTablet write SetTablet;
  {$IFDEF WINTAB}
   property  WinTablet: TWIN_Tablet read GetWinTablet write SetWinTablet;
   {$ENDIF}
   property  Transformation: TTransformation_Afinni2D read GetTransformation write SetTransformation;
   property  TransformationTable: TTransformationTable read GetTransformationTable write SetTransformationTable;
   property  CursorVisible: boolean read FCursorVisible write SetCursorVisible;
   property  LastButton: integer read FLastButton write SetLastButton;
published
   property BActionsString: string read GetBActionsString write SetBActionsString;
   property ConfigFile: string read FConfigFile write SetConfigFile;
   property MoveTolerance: integer read FMoveTolerance write SetMoveTolerance;
   property TransformationTableFile: string read FTransformationTableFile write SetTransformationTableFile;
   property Test: boolean read FTest write SetTest stored false;
   property Action_Run: boolean write SetAction_Run stored false;
   property Action_Transform: boolean write SetAction_Transform stored false;
   property Action_Settings: boolean write SetAction_Settings stored false;
   property Action_ShowTrafo: boolean write SetAction_ShowTrafo stored false;
   property CharDelayTime: DWORD read FCharDelayTime write SetCharDelayTime;
   property ExtraDelayTime: Word read FExtraDelayTime write SetExtraDelayTime;
{$IFDEF WINTAB}
   property Wintab: boolean read FWintab write SetWintab;
{$ENDIF}
end;

{$IFDEF WINTAB}
  { Application-specific packet structure. Setting the context
    PktData variable will define what data to retrieve. }
  TMyPacket = record
    Cursor: Integer;
    Buttons: Integer;
    Coords: TPoint;
    Pressure: Integer;
    Orientation: TOrientation;
  end;
  PMyPacket = ^TMyPacket;

const
  lcPktData = PK_CURSOR or PK_BUTTONS or PK_X or PK_Y or PK_NORMAL_PRESSURE or PK_ORIENTATION;
  lcOptions = {CXO_SYSTEM or }CXO_MESSAGES or CXO_CSRMESSAGES;
  lcBtnMask = SBN_LCLICK or SBN_RCLICK or SBN_LDBLCLICK or SBN_RDBLCLICK;
{$ENDIF}


implementation
uses
   Buttons,
   Classes,
   Controls,
   Forms,
   Graphics,
   SysUtils,
   e_Class,
   f_TransformationTableDialog,
   m_Orbis,
   t_Loader,
   t_Saver,
   u_String;


{ TTabletManagerFrame }

procedure TTabletManagerFrame.CreateFrame;
begin
   aFrame := TTabletManager_Frame.Create(nil);
   TMFrame.Tool := Self;
   //WTablet_Open;
end;

procedure TTabletManagerFrame.DrawCross(aX,aY: integer);
begin
   Device.Pen.Style := psSolid;
   Device.Pen.Width := 1;
   Device.Pen.Color := clBlack;
   Device.Pen.Mode:= pmCopy;
   Device.MoveToDevice(aX-10,aY);
   Device.LineToDevice(aX+10,aY);
   Device.MoveToDevice(aX,aY-10);
   Device.LineToDevice(aX,aY+10);
   Application.processMessages;
end;

procedure TTabletManagerFrame.DrawR(aX,aY: integer; aText: string);
begin
   Device.Font.Color := clRed;
   Device.Font.Size := 20;
   Device.Font.Name := 'Arial';
   Device.TextOutDevice(aX+5,aY,aText);
   Application.processMessages;
end;

procedure TTabletManagerFrame.EndFlashTool;
begin
   On_Activate;
end;

function TTabletManagerFrame.GetBActionsString: string;
begin
   result := IntToStr(FBActions[1])+IntToStr(FBActions[2])+IntToStr(FBActions[3])+IntToStr(FBActions[4]);
end;

function TTabletManagerFrame.GetTablet: TTablet;
begin
   if FTablet = nil then
   begin
      FTablet := TTablet.Create;
      AddElement(FTablet);
      FTablet.onTabletAction := TabletProcedure;
   end;
   result := FTablet;
end;

function TTabletManagerFrame.GetTMFrame: TTabletManager_Frame;
begin
   result := TTabletManager_Frame(aFrame);
end;

function TTabletManagerFrame.GetTransformation: TTransformation_Afinni2D;
begin
   if FTransformation = nil then
   begin
      FTransformation := TTransformation_Afinni2D.Create;
      AddElement(FTransformation);
   end;
   result := FTransformation;
end;

function TTabletManagerFrame.GetTransformationTable: TTransformationTable;
begin
   if FTransformationTable = nil then
   begin
      FTransformationTable := TTransformationTable.Create;
      AddElement(FTransformationTable);
   end;
   result := FTransformationTable;
end;

{$IFDEF WINTAB}
function TTabletManagerFrame.GetWinTablet: TWIN_Tablet;
begin
   result := nil;
   if FWinTablet = nil then
   begin
      FWinTablet := TMFrame.WIN_Tablet;
      FWinTablet.OnPacket := WTabletPacket;
      FWinTablet.OnCursorChanged := WTabletCursorChanged;
      FWinTablet.OnProximity := WTabletProximity;
   end;
   result := FWinTablet;
end;
{$ENDIF}
procedure TTabletManagerFrame.LoadTable;
var
   S: string;
   T: TTransformationTable;
   L: TLoad;
begin
   S := ParamStr(0);
   S := ExtractFilePath(S);
   CheckBackSlash(S);
   S := S+'TrTable.ott';
   if not FileExists(S) then exit;
   try
      L := TLoad.Create;
      L.FileName := S;
      L.DoIt;
      T := TTransformationTable(L.Output);
      TransformationTable := T;
      L.Free;
   except
   end;
end;

procedure TTabletManagerFrame.On_Activate;
var
   S: string;
begin
   CursorIndex := -1;
   CursorIndex := 0;
   case TrEditStatus of
      tresWaitingForMouse: S := 'Sejmte clov bod my ...';
      tresWaitingForTablet: S := 'Sejmte zdrojov bod tabletem ...';
   else
      S := ' ';
   end;
   PostTextMsgToOwners(tm_ShowText,S);
   //Application.ProcessMessages;
end;

procedure TTabletManagerFrame.On_tm_MouseDown(var aMessage: TOrbisMouseEvent; var Handled: boolean);
var
   B: TIDBod;
begin
   if not (tmfStatus = tmfsEditTransformation) then exit;
   if FlashTool <> nil then exit;
   if aMessage.Button = mbRight then
   begin
      aMessage.Button := mbMiddle;
      if TrEditStatus = tresWaitingForTablet then
      begin
         if not TransformationTable.NoElements then
         begin
            TransformationTable.IDBod[TransformationTable.LastIndex].Free;
         end;
      end;
      Action_Transform := true;
      exit;
   end;
   if (TrEditStatus = tresWaitingForMouse) then
   if aMessage.Button = mbLeft then
   begin
      LastX := aMessage.X;
      LastY := aMessage.Y;
      B := TIDBod.Create;
      B.Cislo := TransformationTable.Count + 1;
      B.cX := Device.Projection.GetModelX(aMessage.X);
      B.cY := Device.Projection.GetModelY(aMessage.Y);
      TransformationTable.AddElement(B);
      TrEditStatus := tresWaitingForTablet;
      DrawCross(LastX,LastY);
   end;
end;

procedure TTabletManagerFrame.On_tm_Review(var aMessage: TOrbisMessage;var Handled: boolean);
begin
   CursorVisible := false;
   if not (tmfStatus = tmfsEditTransformation) then exit;
   ShowTrafo;
  //Application.ProcessMessages;
   //XORDraw;
   //if aMessage.Sender <> Self then
      //PostMsg(tm_Review,Self,nil,false);
end;

procedure TTabletManagerFrame.RegisterCursors;
begin
   NewCursorName('XHAIR');
end;

procedure TTabletManagerFrame.SaveTable;
var
   Save: TSave;
   S: string;
begin
   S := ParamStr(0);
   S := ExtractFilePath(S);
   CheckBackSlash(S);
   S := S+'TrTable.ott';
   Save := TSave.Create;
   Save.FileName := S;
   Save.Input := TransformationTable;
   Save.DoIt;
   Save.Free;
end;


procedure TTabletManagerFrame.SetAction_Run(const Value: boolean);
var
   Act: boolean;
begin
{$IFDEF WINTAB}
   Wintab := ConfigFile = '';
{$ENDIF}
   if CHarDelayTime < 20 then
      CharDelayTime := 50;
   if ExtraDelayTime < 20 then
      ExtraDelayTime := 50;
{$IFDEF WINTAB}
   if not Wintab then
{$ENDIF}
   begin
      Tablet.RTOCharDelayTime := CHarDelayTime;
      Tablet.RTOExtraDelayTime := ExtraDelayTime;
      Tablet.ConfigFile := ConfigFile;
      Tablet.MoveTolerance := MoveTolerance;
   end;
   if FTransformationTable = nil then
   begin
      LoadTable;
      Transformation.TransformationTable := TransformationTable;
   end;
{$IFDEF WINTAB}
   if Wintab then
   begin
      Act := WinTablet.ContextHandle = 0;
      if Act then
         WTablet_Open
      else
         WinTablet.Close;
      Act := WinTablet.ContextHandle <> 0;
   end
   else
{$ENDIF}
   begin
      Act := not(Tablet.Active);
      Tablet.Active := Act;
   end;
   tmFrame.Tablet_Transformation.Enabled := Act;
   tmFrame.Tablet_ShowTrafo.Enabled := Act;
   tmFrame.Tablet_Run.Checked := Act;
   if Act then
      tmFrame.TabletPanel.Color := clLime
   else
      tmFrame.TabletPanel.Color := clRed;
   XORDraw;
end;

procedure TTabletManagerFrame.SetAction_Settings(const Value: boolean);
var
   _ConfigFile: string;
   _MoveTolerance: integer;
   _Active: boolean;
   _CHarDelayTime: DWord;
   _ExtraDelayTime: Word;
begin
   if CHarDelayTime < 20 then
      CharDelayTime := 50;
   if ExtraDelayTime < 20 then
      ExtraDelayTime := 50;
   _ConfigFile := ConfigFile;
   _MoveTolerance := MoveTolerance;
{$IFDEF WINTAB}
   Wintab := ConfigFile = '';
   if WINTab then
      _Active := WinTablet.ContextHandle <> 0
   else
{$ENDIF}
      _Active := Tablet.Active;
   _CharDelayTime := CharDelayTime;
   _ExtraDelayTime := ExtraDelayTime;
   Dialog := TTabletDialog_Form.Create(Application);
   Dialog.Tool := Self;
   Dialog.ed_ConfigFile.Text := ConfigFile;
   Dialog.ed_MoveTolerance.Value := MoveTolerance;
   Dialog.ed_1.Value := FBActions[1];
   Dialog.ed_2.Value := FBActions[2];
   Dialog.ed_3.Value := FBActions[3];
   Dialog.ed_4.Value := FBActions[4];
   Dialog.ed_CharDelay.Text := IntToStr(CharDelayTime);
   Dialog.ed_ExtraDelay.Text :=IntToStr(ExtraDelayTime);
{$IFDEF WINTAB}
   if WinTab then
      if not WinTablet.Present then
         Dialog.l_Text.Caption := 'No WinTab digitizer present on system!';
   else
{$ENDIF}
      Tablet.OnProcessError := TabletProcessError;
   Dialog.ShowModal;
{$IFDEF WINTAB}
   if not WinTab then
{$ENDIF}
      Tablet.OnProcessError := nil;
   Test := false;
   Case Dialog.ModalResult of
   1:
      begin
         ConfigFile := Dialog.ed_ConfigFile.Text;
         if ConfigFile <> '' then
{$IFDEF WINTAB}
         if Wintab then
            WinTablet.Close;
{$ENDIF}
         MoveTolerance := Dialog.ed_MoveTolerance.Value;
         FBActions[1] := Dialog.ed_1.Value;
         FBActions[2] := Dialog.ed_2.Value;
         FBActions[3] := Dialog.ed_3.Value;
         FBActions[4] := Dialog.ed_4.Value;
         CharDelayTime := StrToIntDef(Dialog.ed_CharDelay.Text,50);
         ExtraDelayTime := StrToIntDef(Dialog.ed_ExtraDelay.Text,50);
      end;
   else
      begin
         ConfigFile := _ConfigFile;
         MoveTolerance := _MoveTolerance;
         CharDelayTime := _CharDelayTime;
         ExtraDelayTime := _ExtraDelayTime;
      end;
   end;
   Dialog.Free;
   Dialog := nil;
{$IFDEF WINTAB}
   if WinTab then
      if _Active then
         WTablet_Open;
   else
{$ENDIF}
      Tablet.Active := _Active;
end;

procedure TTabletManagerFrame.SetAction_ShowTrafo(const Value: boolean);
begin
   ShowTrafo;
end;

procedure TTabletManagerFrame.SetAction_Transform(const Value: boolean);
var
   D: TTransformationTableDialog_Form;
begin
   D := TTransformationTableDialog_Form.Create(Application);
   Transformation.TransformationTable := TransformationTable;
   Transformation.EvaluateDifferences;
   D.Caption := TransformationTableFile;
   D.Transformation := Transformation;
   D.TrTable := Transformation.TransformationTable;
   case D.ShowModal of
   1:begin //New
        TransformationTable := nil;
        tmfStatus := tmfsEditTransformation;
        TrEditStatus := tresWaitingForMouse;
        PostMsgToOwners(tm_ToolSelected,Self,false);
     end;
   4:begin //Continue
        tmfStatus := tmfsEditTransformation;
        TrEditStatus := tresWaitingForMouse;
        PostMsgToOwners(tm_ToolSelected,Self,false);
     end;
   6:begin
        PostMsgToOwners(tm_ToolSelected,nil,false);
        tmfStatus := tmfsNIL;
        TrEditStatus := tresNIL;
        SaveTable;
     end;
   else
      PostMsgToOwners(tm_ToolSelected,nil,false);
      tmfStatus := tmfsNIL;
      TrEditStatus := tresNIL;
      LoadTable;
   end;
   D.Free;
   //PostMsgToOwners(tm_ToolSelected,Self,false);
   {
   Dialog := TTabletDialog_Form.Create(Application);
   Dialog.ShowModal;
   Dialog.Free;
   Dialog := nil;
   }
end;

procedure TTabletManagerFrame.SetBActionsString(const Value: string);
var
   i: integer;
begin
   if (Value = '') or (Length(Value) < 4) then
   begin
      for i := 1 to 4 do FBActions[i] := i;
   end
   else
   begin
      for i := 1 to 4 do
      begin
         FBActions[i] := StrToIntDef(Value[i],0);
      end;
   end;
end;

procedure TTabletManagerFrame.SetCharDelayTime(const Value: DWORD);
begin
  FCharDelayTime := Value;
end;

procedure TTabletManagerFrame.SetConfigFile(const Value: string);
begin
  FConfigFile := Value;
end;

procedure TTabletManagerFrame.SetCursorVisible(const Value: boolean);
begin
  FCursorVisible := Value;
end;

procedure TTabletManagerFrame.SetDialog(const Value: TTabletDialog_Form);
begin
  FDialog := Value;
end;

procedure TTabletManagerFrame.SetExtraDelayTime(const Value: Word);
begin
  FExtraDelayTime := Value;
end;

procedure TTabletManagerFrame.SetLastButton(const Value: integer);
begin
  FLastButton := Value;
end;

procedure TTabletManagerFrame.SetLastX(const Value: integer);
begin
  FLastX := Value;
end;

procedure TTabletManagerFrame.SetLastY(const Value: integer);
begin
  FLastY := Value;
end;

procedure TTabletManagerFrame.SetMoveTolerance(const Value: integer);
begin
  FMoveTolerance := Value;
end;

procedure TTabletManagerFrame.SetTablet(const Value: TTablet);
begin
  ExchangeFieldElement(TElement(FTablet),Value);
end;

procedure TTabletManagerFrame.SetTest(const Value: boolean);
{$IFDEF WINTAB}
var
  ResX, ResY: TAxis;
{$ENDIF}  
begin
   FTest := Value;
   if Value then
   begin
      if Dialog = nil then
      begin
         FTest := false;
         exit;
      end;
      Dialog.Memo.Clear;
      Dialog.l_Text.Caption := 'Zatm nic...';
{$IFDEF WINTAB}
      if Wintab then
         WinTablet.Enabled := false
      else
{$ENDIF}
         Tablet.Active := false;
      Dialog.l_X.Caption := '-';
      Dialog.l_Y.Caption := '-';
      Dialog.l_Button.Caption := '-';
      Application.ProcessMessages;
{$IFDEF WINTAB}
      if not Wintab then
{$ENDIF}
      begin
         Tablet.ConfigFile := Dialog.ed_ConfigFile.Text;
         Tablet.MoveTolerance := Dialog.ed_MoveTolerance.Value;
         Tablet.RTOCharDelayTime := StrToIntDef(Dialog.ed_CharDelay.Text,50);
         Tablet.RTOExtraDelayTime := StrToIntDef(Dialog.ed_ExtraDelay.Text,50);
         Dialog.ed_CharDelay.Text := IntToStr(Tablet.RTOCharDelayTime);
         Dialog.ed_ExtraDelay.Text := IntToStr(Tablet.RTOExtraDelayTime);
      end;
      Dialog.Memo.Clear;
{$IFDEF WINTAB}
      if Wintab then
      begin
         if WTablet_Open then
         begin
            Dialog.l_Text.Caption := 'Tablet WINTAB zejm funguje';
            Dialog.Memo.Lines.Add('Tablet: '+ WinTablet.DeviceName);
            Dialog.Memo.Lines.Add('WintabID: '+ WinTablet.WinTabID);
            WTInfo(WTI_DEVICES, DVC_X, @ResX);
            WTInfo(WTI_DEVICES, DVC_Y, @ResY);
            Dialog.Memo.Lines.Add('Tablet resolution: ' + IntToStr(ResX.Max)  + ' x ' + IntToStr(ResY.Max));
         end
         else
            Dialog.l_Text.Caption := 'Tablet WINTAB evidentn stvkuje';
      end
      else
{$ENDIF}
      begin
         Tablet.Active := true;
         if Tablet.Active then
            Dialog.l_Text.Caption := Tablet.CommPort + ': Tablet zejm funguje'
         else
            Dialog.l_Text.Caption := Tablet.CommPort + ': Tablet evidentn stvkuje';
      end;
   end
   else
{$IFDEF WINTAB}
      if Wintab then
         WinTablet.Enabled := false
      else
{$ENDIF}
         Tablet.Active := false;
   Application.ProcessMessages;
end;

procedure TTabletManagerFrame.SettmfStatus(const Value: TtmfStatus);
begin
  FtmfStatus := Value;
end;

procedure TTabletManagerFrame.SetTransformation(const Value: TTransformation_Afinni2D);
begin
  ExchangeFieldElement(TElement(FTransformation), Value);
end;

procedure TTabletManagerFrame.SetTransformationTable(const Value: TTransformationTable);
begin
   ExchangeFieldElement(TElement(FTransformationTable),Value);
end;

procedure TTabletManagerFrame.SetTransformationTableFile(const Value: string);
begin
  FTransformationTableFile := Value;
end;

procedure TTabletManagerFrame.SetTrEditStatus(const Value: TTrEditStatus);
var
   S: string;
begin
   FTrEditStatus := Value;
   case Value  of
      tresWaitingForMouse: S := 'Sejmte clov bod my ...';
      tresWaitingForTablet: S := 'Sejmte zdrojov bod tabletem ...';
   else
      S := ' ';
   end;
   PostTextMsgToOwners(tm_ShowText,S);
   Application.ProcessMessages;
end;

{$IFDEF WINTAB}
procedure TTabletManagerFrame.SetWintab(const Value: boolean);
begin
  FWintab := Value;
end;

procedure TTabletManagerFrame.SetWinTablet(const Value: TWin_Tablet);
begin
   //FWinTablet := Value;
end;
{$ENDIF}

procedure TTabletManagerFrame.ShowTrafo;
var
   i: integer;
   B: TIDBod;
   aX,aY: integer;
begin
   if FTransformationTable = nil then exit;
   for i := 0 to TransformationTable.Lastindex do
   begin
      B := TransformationTable.IDBod[i];
      aX := Device.Projection.GetPageX(B.cX);
      aY := Device.Projection.GetPageY(B.cY);
      DrawCross(aX,aY);
      DrawR(aX,aY,B.Name);
   end;
end;

procedure TTabletManagerFrame.TabletProcedure(aX, aY: double; Button: integer; ButtonChanged: boolean);
var
   B: TIDBod;
   _X,_Y: extended;
   iX,iY: integer;
   C: integer;
   i: integer;
begin
   if Test then
   begin
      if Dialog = nil then exit;
      Dialog.l_X.Caption := FloatToStr(aX);
      Dialog.l_Y.Caption := FloatToStr(aY);
      Dialog.l_Button.Caption := IntToStr(Button);
   end
   else
   if (tmfStatus = tmfsEditTransformation) then
   begin
      if (TrEditStatus = tresWaitingForTablet) then
      begin
         if ButtonChanged and (FBActions[Button] = 1) then
         begin
            MessageBeep($FFFFFFFF);
            B := TransformationTable.IDBod[TransformationTable.LastIndex];
            B.zX := aX;
            B.zY := aY;
            TrEditStatus := tresWaitingForMouse;
            DrawR(LastX,LastY,B.Name);
         end;
      end;
   end
   else
   begin
      if Transformation.Error > 0 then exit;
      if Transformation.TransformationTable = nil then exit;
      TX := aX;
      TY := aY;
      {
      if ButtonChanged and (Button > 0) then
         Busy := not Busy;
      if Busy then
         Transformation.TransformXY(TX,TY,_X,_Y);
      }
      Transformation.TransformXY(TX,TY,_X,_Y);
      if ButtonChanged and (Button > 0) then
      begin
         if FBActions[Button] = 4 then XORDraw;
         MessageBeep($FFFFFFFF);
         for i := 0 to OwnersCount - 1 do
         begin
            PostExternalDeviceEvent(tm_ExternalDeviceDown,Owners[i],FBActions[Button],_X,_Y);
         end;
      end;
      iX := Device.Projection.GetPageX(_X);
      iY := Device.Projection.GetPageY(_Y);
      if (iX = LastX) and (iY = LastY) then exit;
      {
      iX := 100;
      if LastX = 100 then iX := 200;
      iY := 100;
      if LastY = 100 then iY := 200;
      }
      if CursorVisible then XORDraw
                       else CursorVisible := true;
      //Application.ProcessMessages;
      LastX := iX;
      LastY := iY;
      XORDraw;
      //Application.ProcessMessages;

   end;
end;

procedure TTabletManagerFrame.TabletProcessError(Sender: TObject; Place, Code: DWord; Msg: String);
begin
   if Dialog = nil then exit;
   Dialog.Memo.Lines.Add(Format('Code %d at %d Text: %s',[Code,Place,Msg]));
end;

{$IFDEF WINTAB}
procedure TTabletManagerFrame.WTabletCursorChanged(Sender: TObject;
  PacketNumber, ContextHandle: HCtx; pPacket: Pointer);
var
  Csr: Integer;
  CsrType: Integer;
  CsrPhysID: Integer;
begin
  Csr := TMyPacket(pPacket^).Cursor;
  WTInfo(WTI_CURSORS + Csr, CSR_TYPE, @CsrType);
  WTInfo(WTI_CURSORS + Csr, CSR_PHYSID, @CsrPhysID);
  {
  lblCursor.Caption := 'Cursor: ' + GetInfoAsString(WTI_CURSORS, CSR_NAME) + #13#10 + ' (type = $' +
  IntToHex(CsrType and $0F06, 4)
  + ', physid = $' + IntToHex(CsrPhysID, 4) + ')';
  case Csr of
    1: pnlCursor.Color := $996633;
    2: pnlCursor.Color := $663399;
    3: pnlCursor.Color := $339966;
  end;
  }
end;

procedure TTabletManagerFrame.WTabletPacket(Sender: TObject; PacketNumber,
  ContextHandle: HCtx; pPacket: Pointer);
var
  aBut: integer;
begin
  with TMyPacket(pPacket^) do
  begin
     aBut := Buttons;
     if aBut >= 16 then aBut := 5
     else if aBut >= 8 then aBut := 4
     else if aBut >= 4 then aBut := 3;

     //PressureBar.Position := Pressure;
     //Panels[0].Text := IntToStr(Cursor);
     //Panels[1].Text := IntToStr(Coords.X);
     //Panels[2].Text := IntToStr(Coords.Y);
     //Panels[3].Text := IntToStr(Pressure);
     //Panels[4].Text := IntToStr(Buttons);
     TabletProcedure(Coords.X, Coords.Y,aBut,aBut <> LastButton);
     LastButton := aBut;
  end;
end;

procedure TTabletManagerFrame.WTabletProximity(Sender: TObject;
  ContextHandle: HCtx; Entering: Boolean);
begin
   if Entering then
      tmFrame.TabletPanel.Color := clLime
   else
      tmFrame.TabletPanel.Color := clYellow;
end;

function TTabletManagerFrame.WTablet_Open: boolean;
var
   AContext: TLogContext;
   ResX, ResY: TAxis;
begin
   result := true;
   if WinTablet.ContextHandle <> 0 then exit; // je otevren
   with WinTablet, AContext do
   begin
      if not Present then
      begin
         result := false;
         //ShowMessage('No digitizer present on system.');
         Exit;
      end;
      { Fill context with defaults }
        DefaultContext(AContext);

      { Set option flags to initialize messaging }
  	Options := Options or lcOptions;
      { Bitfields in PktData indicates what data to receive in packets }
  	PktData := lcPktData;
      { Mask buttons (optional) }
        BtnDnMask := lcBtnMask;
        BtnUpMask := BtnDnMask;
        MsgBase := WT_DEFBASE;
      { This will allocate memory for the packets }
      try
        PacketSize := SizeOf(TMyPacket);
      except
      end;
      { Open the context and get handle }
      Enabled := True;
      Open(AContext);
   end;

  {
  cmbOutput.ItemIndex := 0;
  with Tablet.Pressure do
  begin
    PressureBar.Min := Min;
    PressureBar.Max := Max;
  end;
  WTInfo(WTI_DEVICES, DVC_X, @ResX);
  WTInfo(WTI_DEVICES, DVC_Y, @ResY);

  AddListRow('Device Name', Tablet.DeviceName);
  AddListRow('Wintab ID', Tablet.WintabID);
  AddListRow('Packet rate', IntToStr(Tablet.PacketRate) + ' Hz');
  AddListRow('Tablet resolution', IntToStr(ResX.Max)  + ' x ' + IntToStr(ResY.Max));
  AddListRow('Number of devices supported',
    IntToStr(GetInfoAsInteger(WTI_INTERFACE, IFC_NDEVICES)));
  AddListRow('Number of cursors supported',
    IntToStr(GetInfoAsInteger(WTI_INTERFACE, IFC_NCURSORS)));
  AddListRow('Number of contexts supported',
    IntToStr(GetInfoAsInteger(WTI_INTERFACE, IFC_NCONTEXTS)));
  }
end;
{$ENDIF}

procedure TTabletManagerFrame.XORDraw;
var
   pm: TPenMode;
   ps: TPenStyle;
   pc: integer;
   pw: integer;
begin
   // PPK 14.10.2002 ukl dn a obnoven vlastnost pera
   pm := Device.Pen.Mode;
   ps := Device.Pen.Style;
   pc := Device.Pen.Color;
   pw := Device.Pen.Width;
   Device.Pen.Color:= clSilver;
   Device.Pen.Style:= psSolid;
   Device.Pen.Mode:= pmXOR;
   Device.Pen.Width:= 1;
   Device.MoveToDevice(LastX-5,LastY);
   Device.LineToDevice(LastX+6,LastY);
   Device.MoveToDevice(LastX,LastY-5);
   Device.LineToDevice(LastX,LastY+6);
   Device.Pen.Color:= pc;
   Device.Pen.Style:= ps;
   Device.Pen.Mode:= pm;
   Device.Pen.Width:= pw;
end;

INITIALIZATION
   RegisterOrbisClass(TTabletManagerFrame);
END.
