unit E_props;

interface
uses
  Classes, TypInfo,
// KERNEL
  Elements,
  E_Type;

{
  TTypeKind = (tkUnknown, tkInteger, tkChar, tkEnumeration, tkFloat,
    tkString, tkSet, tkClass, tkMethod);
  TTypeKinds = set of TTypeKind;

  TOrdType = (otSByte, otUByte, otSWord, otUWord, otSLong);

  TFloatType = (ftSingle, ftDouble, ftExtended, ftComp);

  TMethodKind = (mkProcedure, mkFunction);
  TParamFlags = set of (pfVar, pfConst, pfArray);
}


type

{ TProperty ..... bude potomek e_Class.TOrbisClassProperty }

{ }

  TProperty = class(TNameElement)
  private
    FInstance: TPersistent;
    FPropInfo: PPropInfo;
    FIndex: integer;
    procedure SetPropInfo(aPropInfo: PPropInfo);
    function GetStored: boolean;
    function GetTypeKind: TTypeKind;
    function GetPropString: string;
    procedure SetPropString(aVal: string);
    function GetPropLongInt: longInt;
    property PropInfo: PPropInfo read FPropInfo write SetPropInfo;
  published
    property Instance: TPersistent write FInstance;
    property PropIndex: integer read FIndex write FIndex;
    property IsStored: boolean read GetStored;
    property TypeKind: TTypeKind read GetTypeKind;
    property PropString: string read GetPropString write SetPropString;
    property PropLongInt: longInt read GetPropLongInt;
  end;

  TInstanceProperties = class(TPropertyList)
  private
    FInstance: TPersistent;
    FPropList: PPropList;
    FPropCount: integer;
    FActualProperty: TProperty;
    procedure SetInstance(anInstance: TPersistent);
    function  GetIndexForName(aName: string): integer;
    function  GetPropName: string;
    procedure ISetPropIndex(aIndex: integer);
    function  IGetPropIndex: integer;
    procedure SetPropName(aName: string);
    function  IGetPropString: string;
    procedure ISetPropString(aVal: string);
    procedure SetList(aVal: boolean);
    function GetTypeKind: TTypeKind;
    function GetInstClassName: string;
  public
    destructor Destroy; override;
    function   GetPropertyStringByName(AName: string): string; override;
    procedure  SetPropertyStringByName(AName: string; aValue: string); override;
  published
    property Instance: TPersistent read FInstance write SetInstance;
    property InstClassName: string read GetInstClassName;
    property PropCount: integer read FPropCount;
    property PropName: string read GetPropName write SetPropName;
    property PropIndex: integer read IGetPropIndex write ISetPropIndex;
    property ActualProperty: TProperty read FActualProperty;
    property PropString: string read IGetPropString write ISetPropString;
    property PropKind: TTypeKind read GetTypeKind;
    property IsList: boolean write SetList;
  end;

{ Pedek Obecnch propert }

TProperties = class(TOwnerEntity);

{ Dynamick property ... entita typovch element, kter zpstupuje standardnmi prostedky }

TDynamicProperties = class(TProperties)
public
    destructor   Destroy; override;
    function     MakeSelfCopy: TDynamicProperties;
    function     HasPropertyByName(aName: string): boolean; override;
    function     GetPropertyStringByName(AName: string): string; override;
    procedure    SetPropertyStringByName(AName: string; aValue: string); override;
    function     GetPropertyLongIntByName(AName: string): longInt; override;
    procedure    SetPropertyLongIntByName(AName: string; aValue: longInt); override;
    function     GetPropertyBooleanByName(AName: string): boolean; override;
    procedure    SetPropertyBooleanByName(AName: string; aValue: boolean); override;
end;

implementation
uses
  SysUtils,
// KERNEL
  e_Class,
  U_Props;



{ TDynamicProperties }

destructor TDynamicProperties.Destroy;
begin
  inherited Destroy;
end;

function     TDynamicProperties.HasPropertyByName(aName: string): boolean;
begin
   result := FindElementName(aName) <> nil;
end;

function     TDynamicProperties.GetPropertyStringByName(AName: string): string;
var
   E: TTypeElement;
begin
   result := '';
   E := TTypeElement(FindElementName(aName));
   if E <> nil then result := E.AsString;
end;

procedure    TDynamicProperties.SetPropertyStringByName(AName: string; aValue: string);
var
   E: TTypeElement;
begin
   E := TTypeElement(FindElementName(aName));
   if E <> nil then
   begin
      E.AsString := aValue;
   end
   else
   begin
      E := TStringElement.Create;
      TStringElement(E).Name := aName;
      TStringElement(E).Value := aValue;
      AddElement(E);
   end;
end;

function     TDynamicProperties.GetPropertyLongIntByName(AName: string): longInt;
var
   E: TTypeElement;
begin
   result := 0;
   E := TTypeElement(FindElementName(aName));
   if E <> nil then result := E.AsInteger;
end;

procedure    TDynamicProperties.SetPropertyLongIntByName(AName: string; aValue: longInt);
var
   E: TTypeElement;
begin
   E := TTypeElement(FindElementName(aName));
   if E <> nil then
   begin
      E.AsInteger := aValue;
   end
   else
   begin
      E := TIntegerElement.Create;
      TIntegerElement(E).Value := aValue;
      AddElement(E);
   end;
end;

function     TDynamicProperties.GetPropertyBooleanByName(AName: string): boolean;
var
   E: TTypeElement;
begin
   result := false;
   E := TTypeElement(FindElementName(aName));
   if E <> nil then result := E.AsBoolean;
end;

procedure    TDynamicProperties.SetPropertyBooleanByName(AName: string; aValue: boolean);
var
   E: TTypeElement;
begin
   E := TTypeElement(FindElementName(aName));
   if E <> nil then
   begin
      E.AsBoolean := aValue;
   end
   else
   begin
      E := TBooleanElement.Create;
      TBooleanElement(E).Value := aValue;
      AddElement(E);
   end;
end;




function TDynamicProperties.MakeSelfCopy: TDynamicProperties;
var
   i: integer;
   e: TElement;
   NE: TElement;
begin
   result := TDynamicProperties.Create;
   for i := 0 to LastIndex do
   begin
      e := Elements[i];
      NE :=CreateElementCopy(e);
      result.AddElement(NE);
   end;
end;


{ TProperty }


procedure TProperty.SetPropInfo(aPropInfo: PPropInfo);
begin
  FPropInfo:= aPropInfo;
  Name:= FPropInfo^.Name
end;


function TProperty.GetStored: boolean;
begin
  result:= false;
  if FPropInfo = nil then exit;
  if FInstance = nil then exit;
  result:= IsStoredProp(FInstance, FPropInfo)
end;

function TProperty.GetTypeKind: TTypeKind;
begin
  result:= tkUnknown;
  if FPropInfo = nil then exit;
  result:= FPropInfo^.PropType^.Kind;
end;



function TProperty.GetPropString: string;
begin
  result:= '';
  if FPropInfo = nil then exit;
  if FInstance = nil then exit;
  FillWithData(FInstance, FPropInfo, result);
end;


function TProperty.GetPropLongInt: longInt;
begin
  result:= 0;
  if FPropInfo = nil then exit;
  if FInstance = nil then exit;
  if FPropInfo^.GetProc = nil then exit;

  result:= GetOrdProp(FInstance, FPropInfo);
end;


procedure TProperty.SetPropString(aVal: string);
begin
  SetPropByString(FInstance, FPropInfo, aVal)
end;



        { TInstanceProperties }


procedure TInstanceProperties.SetInstance(anInstance: TPersistent);
begin
  if FPropList <> nil then
     FreeMem(FPropList, FPropCount * SizeOf(Pointer));
  FPropList:=nil;

  if anInstance = nil then exit;
  FInstance:= anInstance;
  Name:= anInstance.ClassName + '_Instance';

  FPropCount:= GetTypeData(anInstance.ClassInfo)^.PropCount;
  if FPropCount > 0 then
  begin
    GetMem(FPropList, FPropCount*sizeOf(Pointer));
    try
      GetPropInfos(anInstance.ClassInfo, FPropList);
    except
      if FPropList <> nil then
         FreeMem(FPropList, FPropCount * SizeOf(Pointer));
      FPropList:= nil;
      FPropCount:= 0;
      FInstance:= nil;
    end
  end
end;



function   TInstanceProperties.GetPropertyStringByName(AName: string): string;
var
  Index: integer;
begin
  Index:= GetPropIndex(Self, aName);
  if Index >= 0 then
     result := GetPropString(Self, Index)
  else
  begin
    PropName:= aName;
    result:= PropString;
  end;
end;



procedure  TInstanceProperties.SetPropertyStringByName(AName: string; aValue: string);
var
  Index: integer;
begin
  Index:= GetPropIndex(Self, aName);
  if Index >= 0 then
     SetPropString(self, Index, aValue)
  else
  begin
    PropName:= aName;
    PropString:= aValue
  end
end;



function  TInstanceProperties.GetIndexForName(aName: string): integer;
var
  I: integer;
  Nasel: boolean;
  PropInfo: PPropInfo;

begin
  I:= 0;
  Nasel:= false;
  result:= -1;
  if FPropCount = 0 then exit;
  if FPropList = nil then exit;

  aName:= upperCase(aName);

  while (I < FPropCount) and not Nasel do
  begin
    PropInfo := FPropList^[I];
    if aName = UpperCase(PropInfo^.Name) then Nasel:= true;
    if not Nasel then inc(I)
  end;
  if Nasel then
  begin
    result:= I;
  end
end;




procedure TInstanceProperties.ISetPropIndex(aIndex: integer);
begin
  if aIndex < 0 then exit;
  if FPropList = nil then exit;
  if FInstance  = nil then exit;

  if ActualProperty <> nil then
  begin
    FActualProperty.Free;
    FActualProperty:= nil
  end;

  FActualProperty:= TProperty.Create;
  FActualProperty.Instance:= FInstance;
  FActualProperty.PropInfo:= FPropList^[aIndex];
  FActualProperty.PropIndex:= aIndex
end;

function  TInstanceProperties.IGetPropIndex: integer;
begin
  result:= -1;
  if FActualProperty = nil then exit;
  result:= FActualProperty.PropIndex
end;



procedure TInstanceProperties.SetPropName(aName: string);
begin
  if FPropList = nil then exit;
  if FInstance  = nil then exit;
  ISetPropIndex(GetIndexForName(aName));
end;


function  TInstanceProperties.GetPropName: string;
begin
  result:= '';
  if FActualProperty = nil then exit;
  result:= FActualProperty.Name
end;

function TInstanceProperties.IGetPropString: string;
begin
  result:= '';
  if FActualProperty = nil then exit;
  result:= FActualProperty.PropString
end;

procedure TInstanceProperties.ISetPropString(aVal: string);
begin
  if FActualProperty = nil then exit;
  FActualProperty.PropString:= aVal
end;


function TInstanceProperties.GetTypeKind: TTypeKind;
begin
  result:= tkUnknown;
  if FActualProperty = nil then exit;
  result:= FActualProperty.TypeKind
end;

function TInstanceProperties.GetInstClassName: string;
begin
  result:= 'Unknown class';
  if Instance = nil then exit;
  try
    result:= Instance.ClassName
  except
  end
end;


procedure TInstanceProperties.SetList(aVal: boolean);
var
  I: integer;
begin
  FreeAll;
  if not aVal then exit;

  if FPropList = nil then exit;
  if FInstance  = nil then exit;
  for I:= 0 to PropCount-1 do
  begin
    PropIndex:= I;
    AddNameString(PropName, PropString);
  end;
end;



destructor TInstanceProperties.Destroy;
begin
  if FPropList <> nil then
      FreeMem(FPropList, Count * SizeOf(Pointer));
  if ActualProperty <> nil then
     FActualProperty.Free;
  inherited destroy
end;



INITIALIZATION
   RegisterOrbisClass(TDynamicproperties)
END.
