unit e_Dataset;

interface
uses
   Classes,
   DB,
   Elements;

type

TDatasetElement = class(TOwnerEntity)
private
    FTable: TDataset;
    FNo_BDE: boolean;
    FError: integer;
    procedure SetNo_BDE(const Value: boolean);
    procedure SetTable(const Value: TDataset);
    function GetTableName: string;
    procedure SetTableName(const Value: string);
    function GetReadOnly: boolean;
    procedure SetReadOnly(const Value: boolean);
    function GetRecNo: integer;
    function GetRecordCount: integer;
    procedure SetRecNo(const Value: integer);
    procedure SetError(const Value: integer);
public
    constructor Create(AOwner: TComponent);
    Destructor Destroy; override;
    property  Table: TDataset read FTable write SetTable;
    property  No_BDE: boolean read FNo_BDE write SetNo_BDE;
    property  TableName: string read GetTableName write SetTableName;
    property  ReadOnly: boolean read GetReadOnly write SetReadOnly;
    property  RecNo: integer Read GetRecNo write SetRecNo;
    property  RecordCount: integer read GetRecordCount;
    property  Error: integer read FError write SetError;
end;

implementation
uses
   DBF,
   DBTables,
   DBITypes,
   DBIProcs,
   u_Global;
{ TDatasetElement }

constructor TDatasetElement.Create(AOwner: TComponent);
begin
  inherited Create;
  Error := 0;
  No_BDE := NoBDE;
  if NoBDE then
  begin
     Table := TDBF.Create(aOwner);
     //TDBF(Table).TableType:= ttDBase;
  end
  else
  begin
     Table := TTable.Create(aOwner);
     TTable(Table).TableType:= ttDBase;
  end;
end;

destructor TDatasetElement.Destroy;
begin
   if Table <> nil then
   begin
     Table.Active := false;
     Table.Free;
   end;
   inherited;
end;

function TDatasetElement.GetReadOnly: boolean;
begin
   if No_BDE then
     result := TDBF(Table).ReadOnly
   else
     result := TTable(Table).ReadOnly;
end;

function TDatasetElement.GetRecNo: integer;
var
  RP: RecProps;
  Vysledek: longInt;
begin
   Error := 0;
   if No_BDE then
      result := TDBF(Table).RecNo
   else
   begin
      Vysledek:= DbiGetRecord (TTable(Table).Handle, dbiNoLock, nil, @RP);

      if Vysledek <> 0 then
      begin
         result := -1;
         Error := Vysledek;
         exit
      end;
      result := RP.iPhyRecNum;
   end;
end;

function TDatasetElement.GetRecordCount: integer;
var
   Vysledek: integer;
   aRecCount: integer;
begin
   Error := 0;
   if No_BDE then
      result := TDBF(Table).RecordCount
   else
   begin
      Vysledek:= DBIGetRecordCount(TTable(Table).Handle, aRecCount);
      if  Vysledek <> 0 then
      begin
    {
        MessageDlg('Chyba nastaveni pro objekt c.' + IntToStr(aCislo)+': ' + IntToStr(Vysledek), mtWarning, [mbOK], 0);
    }
         result := -1;
         Error := Vysledek;
      end
      else
         result := aRecCount;
   end;
end;

function TDatasetElement.GetTableName: string;
begin
   if No_BDE then
     result := TDBF(Table).FilePath + TDBF(Table).TableName
   else
     result := TTable(Table).TableName;
end;

procedure TDatasetElement.SetError(const Value: integer);
begin
  FError := Value;
end;

procedure TDatasetElement.SetNo_BDE(const Value: boolean);
begin
  FNo_BDE := Value;
end;

procedure TDatasetElement.SetReadOnly(const Value: boolean);
begin
   if No_BDE then
      TDbf(Table).ReadOnly := Value
   else
      TTable(Table).ReadOnly := Value;
end;

procedure TDatasetElement.SetRecNo(const Value: integer);
var
   Vysledek: integer;
begin
   if No_BDE then
      Tdbf(Table).RecNo := Value
   else
   begin
      Vysledek:= DBISetToRecordNo(TTable(Table).Handle, Value);
      if  Vysledek <> 0 then
       begin
          Error := Vysledek;
       end;
   end;
end;

procedure TDatasetElement.SetTable(const Value: TDataset);
begin
  FTable := Value;
end;

procedure TDatasetElement.SetTableName(const Value: string);
begin
   if No_BDE  then
      TDbf(Table).TableName := Value
   else
      TTable(Table).TableName := Value;
end;

END.
