unit C_TLDBF;

interface
uses
  DB, DBTables, Classes,
// KERNEL
  Elements,
// SPACE
  e_DataSet,
  e_GeometryElement,
  //E_Space,
  E_Props,
// TOPOL
  T_TlDBF;

type
  TTopolDBFTable = class(TDataSetElement)
  private
    FNewIndexFile: boolean;
    FWrongDB: boolean;
    AktCislo: longInt;
    TLIndexFile: TFileStream;
    LastRecNo: longInt;
    FTransferScript: TMovedItemContainer;
    FCreateFileName: string;
    function GetCislo: longInt;
    function SetRecordForCislo(aCislo: longInt): boolean;
    procedure SetTransferScript(ScriptName: string);
    function  GetShortTableName: string;
    procedure ManageLongName(aLongName: string; var aShortName: string;
                             var aGoodAlias: boolean);
    procedure SetCislo(const Value: integer);
{$IFDEF ADO}
    procedure SetConnectionStringForTableName;
  private
    FNewIndexFile: boolean;
    FWrongDB: boolean;
{$ENDIF}
  public
    function FieldExists(var aName: string): boolean;
    function OKCreateTableFromCreateFile(aName: string): boolean;
    function OKCreateIndexFile: boolean;
    function OKIndexFile: boolean;
    function SetForName(aName: string): boolean;
    function SetForNameCislo(aName: string; aCislo: longInt): boolean;
    function SetForTopolObject(BlockDir: string; ObjTyp: char;
                               Druh: integer; aCislo: longInt): boolean;
    function SetForTopolEntity(aTLEntity: TGeometryElement): boolean;
    function AddForTopolObject(BlockDir: string; ObjTyp: char;
                               Druh: integer; aCislo: longInt): boolean;
    function AddForTopolEntity(aTLEntity: TGeometryElement): boolean;
    procedure SetFieldAsString(aName: string; aValue: string);
    function GetFieldAsString(aName: string): string;
    procedure FieldAsStringAndInteger
              (aName: string;
               var StrResult: string; var IntResult: longInt; var IntOK: boolean);
    function IntegerFieldAsFormatedString(aName: string; FormatString: string): string;
    procedure SetObjectNo(No: longInt);
    procedure TransferFieldsFromDBF(aTable: TDataSetElement);
    procedure TransferFieldsFromProperties(anElement: TElement);
    procedure TransferFieldsFromDynamicProperties
              (anElement: TElement; aDynProps: TDynamicProperties);
    procedure CreateAndAppend;
    destructor Destroy; override;
    property NewIndexFile: boolean read FNewIndexFile write FNewIndexFile;
    property WrongDB: boolean read FWrongDB write FWrongDB;
  published
    property CreateFileName: string read FCreateFileName write FCreateFileName;
    property TransferScript: string write SetTransferScript;
    property ShortTableName: string read GetShortTableName;
    property Cislo: integer write SetCislo;
  end;




procedure Register;


implementation
uses
  SysUtils, Dialogs, DBITypes, DBIProcs, Controls,
// KERNEL
  E_Type,
  U_String,
  Worlds,
// SPACE
  U_Props,
// TOPOL
  CreaDBF,
  E_TplReg,
  //E_TplPrp;
  e_TopoLElements;




destructor TTopolDBFTable.Destroy;
begin
  TLIndexFile.Free;
  if FTransferScript <> nil then
     FTransferScript.Free;
  inherited Destroy
end;



function TTopolDBFTable.GetCislo: longInt;
var
  Cislo: longInt;
  aCode: integer;
  CisloStr: string;

begin
  CisloStr:= Table.FieldByName('CISLO').AsString;

  val(CisloStr, Cislo, acode);

  if aCode <> 0 then
  begin
    Cislo:= 0
  end;

  result:= Cislo
end;



procedure TTopolDBFTable.ManageLongName(aLongName: string; var aShortName: string;
                                        var aGoodAlias: boolean);
var
  aTabName: string;
  aDotPos: integer;
begin
  aGoodAlias:= true;
  aShortName:= aLongName;

  aDotPos:= pos('.', aLongName);
  if aDotPos > 0 then
  begin
    aTabName  := copy(aLongName, 1, aDotPos-1);
    aShortName:= copy(aLongName, aDotPos+1, 255);
    aGoodAlias:= CompareText(aTabName, ShortTableName) = 0
  end;
end;


function TTopolDBFTable.FieldExists(var aName: string): boolean;
var
  aField: TField;
  I: integer;
  aGoodAlias: boolean;
begin
  result:= false;

  ManageLongName(aName, aName, aGoodAlias);
  if not aGoodAlias then exit;

  for I:= 0 to Table.FieldCount-1 do
  begin
    aField:= Table.Fields[I];
    if CompareText(aField.FieldName, aName) = 0 then
    begin
      result:= true;
      break
    end;
  end;
end;



function  TTopolDBFTable.GetShortTableName: string;
var
  aDotPos: integer;
begin
  result:= ExtractFileName(TableName);
  aDotPos:= pos('.', result);
  if aDotPos = 0 then exit;
  result:= copy(result, 1, aDotPos-1)
end;


function  TTopolDBFTable.OKCreateTableFromCreateFile(aName: string): boolean;
var
  OKej: boolean;
begin
  Table.Active:= false;
  result:= false;
  if (FCreateFileName <> '') and fileExists(FCreateFileName) then
  begin
    if extractFileExt(FCreateFileName) = '.DBF' then
       VytvorDBDleDBF(FCreateFileName, aName, OKej)
    else
       VytvorDBDleCRT(FCreateFileName, aName, OKej);
    result:= OKej
  end
end;




(*
procedure FillFielDefs();
begin
{
  TFieldType = (ftUnknown, ftString, ftSmallint, ftInteger, ftWord, ftBoolean,
                ftFloat, ftCurrency, ftBCD, ftDate, ftTime, ftDateTime, ftBytes,
                ftVarBytes, ftBlob, ftMemo, ftGraphic);
}

  FieldDefs.Add('P1', ftString, 20, false)
  FieldDefs.Add('P2', ftInteger, 20, false)
end;



procedure TTopolDBFTable.CreateTableForCRT(const CRTFile: string);
begin

  try
  CreateTable
  except
  end
end;
*)



procedure TTopolDBFTable.SetFieldAsString(aName: string; aValue: string);
var
  F: TField;
begin
  if not FieldExists(aName) then exit;
  try
    F:= Table.FieldByName(aName);
    if F<>nil then
       F.AsString:= aValue
  except
  end
end;



function TTopolDBFTable.GetFieldAsString(aName: string): string;
var
  F: TField;
begin
  result:= '';
  if not FieldExists(aName) then exit;
  try
    F:= Table.FieldByName(aName);
    if F<>nil then
    begin
       if F is TStringField then
          TStringField(F).Transliterate := DBTransLiterate;
       Result := F.AsString;
    end
    else
       Result:= '';
  except
  end
end;



procedure TTopolDBFTable.FieldAsStringAndInteger
          (aName: string;
           var StrResult: string; var IntResult: longInt; var IntOK: boolean);
var
  F: TField;
  code: integer;
  Str: string;
begin
  IntOK:= false;
  IntResult:= 0;

  F:= Table.FieldByName(aName);
  if F<>nil then
  begin
    StrResult:= F.AsString;
    Str:= BezVnejsichMezer(StrResult);
    val(Str, IntResult, code);
    if code = 0 then
       IntOK:= true
    else
      IntResult:= 0
  end
  else StrResult:= ''
end;



function TTopolDBFTable.IntegerFieldAsFormatedString
         (aName: string; FormatString: string): string;
var
  F: TField;
begin
  F:= Table.FieldByName(aName);
  if F<>nil then Result:= Format(FormatString, [F.AsInteger])
  else Result:= ''
end;




function TTopolDBFTable.SetRecordForCislo(aCislo: longInt): boolean;
var
  aRecNo, aRecCount: longInt;
  Vysledek: longint;
  Nasel: boolean;
  NalezeneCislo: longInt;
begin
{  MessageDlg('CISLO ' + IntToStr(aCislo), mtInformation, [mbOK], 0); }

  Result:= false;

  if not OKIndexFile then exit;

  Table.Active:= true;
  aRecCount:= RecordCount;
  if  Error <> 0 then
  begin
{
    MessageDlg('Chyba nastaveni pro objekt c.' + IntToStr(aCislo)+': ' + IntToStr(Vysledek), mtWarning, [mbOK], 0);
}
    Check(Error);
    exit;
  end;

  TLIndexFile.Seek(aCislo * sizeOf(longInt), 0);
  TLIndexFile.Read(aRecNo, sizeOf(longInt));


  if (aRecNo >= 0) and (aRecNo <= aRecCount) then
  begin
    Table.Active:= true;
    RecNo:= aRecNo;
    if  Error <> 0 then
    begin
{
      MessageDlg('Chyba nastaveni pro objekt c.' + IntToStr(aCislo)+': ' + IntToStr(Vysledek), mtWarning, [mbOK], 0);
}
      Check(Error);
    end
    else
    begin
      Table.Resync([]);
      NalezeneCislo:= GetCislo;
      Nasel:= NalezeneCislo = aCislo;
{
      Nasel:= Fields[FieldDefs.IndexOf('CISLO')].AsInteger = aCislo;
}
      if not Nasel then
      begin
        (* !!!!!!!!!!! p.k. 22.4.1998
        if MessageDlg('Pi prci s indexem objektu bylo misto ' + IntToStr(aCislo) + ' vraceno ' +
                       IntToStr(NalezeneCislo) + #13 +
{                      Fields[FieldDefs.IndexOf('CISLO')].AsString + #13 + }
                      '(' +TableName +')'+ #13 + #13 +
                      'Vytvoit nov indexov soubor ?',
                      mtWarning, [mbYES, mbNO], 0) = mrYES then
        *)
        if NewIndexFile then
        begin
           result := false;
           if not WrongDB then
           if MessageDlg('Pro objekt . ' + IntToStr(aCislo) +
                         ' nenalezen odpovdajc zznam v' + #13 +
                          '(' +TableName +')'+ #13 + #13 +
                             'Vypisovat dal nesrovnalosti?',
                             mtWarning, [mbYES, mbNO], 0) = mrYES then
           begin
              WrongDB := true;
           end;
           exit;
        end
        else
        begin
          if OKCreateIndexFile then
          begin
            Table.Active:= true;
            //ShowMessage('New ndex1 '+IntToStr(aRecNo)+'|'+IntToStr(aRecCount));
            NewIndexFile := true;
            WrongDB := false;
            result:= SetRecordForCislo(aCislo);
            {pk!!!}
          end
        end
      end
      else
        Result:= true
    end;
  end
  else
  begin
    (* !!!!!!!!!!! p.k. 22.4.1998
    if MessageDlg('Pi prci s indexem objektu nebylo pro ' + IntToStr(aCislo) +
                  ' nalezeno sprvn slo zznamu' + #13 +
                   '(' +TableName +')'+ #13 + #13 +
                      'Vytvoit nov indexov soubor ?',
                      mtWarning, [mbYES, mbNO], 0) = mrYES then
    *)
    if NewIndexFile then
    begin
       result := false;
       if not WrongDB then
       if MessageDlg('Pro objekt . ' + IntToStr(aCislo) +
                     ' nenalezen odpovdajc zznam v' + #13 +
                      '(' +TableName +')'+ #13 + #13 +
                         'Vypisovat dal nesrovnalosti?',
                         mtWarning, [mbYES, mbNO], 0) = mrYES then
       begin
          WrongDB := true;
       end;
       exit;
    end
    else
    begin
      if OKCreateIndexFile then
      begin
        Table.Active:= true;
        //ShowMessage('New ndex2 '+IntToStr(aCislo)+'|'+IntToStr(aRecNo)+'|'+IntToStr(aRecCount));
        NewIndexFile := true;
        WrongDB := false;
        result:= SetRecordForCislo(aCislo);
      end
    end
  end;

  LastRecNo:= aRecNo
end;


{$IFDEF ADO}
procedure TTopolDBFTable.SetConnectionStringForTableName;
begin
  ConnectionString:= World.MainTool.GetPropertyStringByName('ADOConnectionString');
  if ConnectionString = '' then
     ConnectionString:= 'Provider=MSDASQL.1;Persist Security Info=False;Extended Properties="DSN=Visual FoxPro Tables;UID=;SourceType=DBF;Exclusive=No;BackgroundFetch=Yes;Collate=Machine;Null=Yes;Deleted=Yes;';
  ConnectionString:= ConnectionString + 'SourceDB='+ extractFilePath(TableName) + '"';
end;
{$ENDIF}


function TTopolDBFTable.SetForName(aName: string): boolean;
begin
  Result:= false;

  if aName <> TableName then
  begin
    if not fileExists(aName) then
       exit;
    Table.Active:= false;
    readOnly:= true;
    TableName:= aName;
{$IFDEF ADO}
    SetConnectionStringForTableName;
{$ENDIF}
    if TLIndexFile <> nil then
    begin
      TLIndexFile.Free;
      TLIndexFile:= nil
    end;
    if FileExists(TableName) then
    begin
      result:= true;
      Table.Active:= true;
    end
    else
      Table.FieldDefs.Clear;
  end
  else
  begin
    Result:= true;
    Table.Active:= true;
  end
end;





function TTopolDBFTable.SetForNameCislo(aName: string; aCislo: longInt): boolean;
begin
  Result:= false;

  if aName <> TableName then
  begin
    if not fileExists(aName) then
       exit;
    Table.Active:= false;
    readOnly:= true;
    TableName:= aName;
{$IFDEF ADO}
    SetConnectionStringForTableName;
{$ENDIF}
    if TLIndexFile <> nil then
    begin
      TLIndexFile.Free;
      TLIndexFile:= nil
    end;
    if FileExists(TableName) then
    begin
      AktCislo:= aCislo;
      Table.Active:= true;
      Result:= SetRecordForCislo(AktCislo);
    end
    else
    begin
      Table.FieldDefs.Clear;
      TableName:= ''
    end
  end
  else
  begin
    AktCislo:= aCislo;
    Table.Active:= true;
    Result:= SetRecordForCislo(AktCislo);
  end
end;





function TTopolDBFTable.SetForTopolObject(BlockDir: string; ObjTyp: char;
                                           Druh: integer; aCislo: longInt): boolean;
var
  aName: string;
begin
  Result:= false;

  aName:= GetDBFNameForTopolObject(BlockDir, ObjTyp, Druh);
  if not fileExists(aName) then exit;
  if aName <> TableName then
  begin
    Table.Active:= false;
    TableName:= aName;
{$IFDEF ADO}
    SetConnectionStringForTableName;
{$ENDIF}
    readOnly:= true;
    if TLIndexFile <> nil then
    begin
      TLIndexFile.Free;
      TLIndexFile:= nil
    end;
    if FileExists(TableName) then
    begin
      AktCislo:= aCislo;
      Table.Active:= true;
      Result:= SetRecordForCislo(AktCislo);
    end
    else
      Table.FieldDefs.Clear
  end
  else
  begin
    AktCislo:= aCislo;
    Table.Active:= true;
    Result:= SetRecordForCislo(AktCislo);
  end
end;





function TTopolDBFTable.SetForTopolEntity(aTLEntity: TGeometryElement): boolean;
var
  aName: string;
  aTP: TTopolElement;
begin
  Result:= false;
  if not (aTlEntity is TTopolElement) then exit;
  aTP:= TTopolElement(aTlEntity);
  aName:= aTP.DBFName;
  if aName <> TableName then
  begin
    Table.Active:= false;
    TableName:= aName;
{$IFDEF ADO}
    SetConnectionStringForTableName;
{$ENDIF}
    ReadOnly:= true;
    if TLIndexFile <> nil then
    begin
      TLIndexFile.Free;
      TLIndexFile:= nil
    end;
    if FileExists(TableName) then
    begin
      AktCislo:= aTP.Cislo;
      Table.Active:= true;
      Result:= SetRecordForCislo(AktCislo);
    end
    else
      Table.FieldDefs.Clear
  end
  else
  begin
    AktCislo:= aTP.Cislo;
    Table.Active:= true;
    Result:= SetRecordForCislo(AktCislo);
  end
end;



function TTopolDBFTable.AddForTopolObject(BlockDir: string; ObjTyp: char;
                                          Druh: integer; aCislo: longInt): boolean;
var
  aName: string;

begin
  Result:= false;
  try
  aName:= GetDBFNameForTopolObject(BlockDir, ObjTyp, Druh);
  if aName <> TableName then
  begin
    Table.Active:= false;
    if FileExists(aName) then
    begin
      TableName:= aName;
{$IFDEF ADO}
    SetConnectionStringForTableName;
{$ENDIF}
      AktCislo:= aCislo;
      Table.Active:= true;
      { pridani zaznamu a nastaveni se na nej }
      Table.Append;
      Result:= true;
    end
    else
    if OKCreateTableFromCreateFile(aName) then
    begin
      TableName:= aName;
{$IFDEF ADO}
    SetConnectionStringForTableName;
{$ENDIF}
      Table.Active:= true;
      Table.Append;
      Result:= true
    end
  end
  else
  begin
    AktCislo:= aCislo;
    Table.Active:= true;
    Table.Append;
    Result:= true;
  end;
  except
    Result:= false;
    Table.Active:= false
  end
end;





function TTopolDBFTable.AddForTopolEntity(aTLEntity: TGeometryElement): boolean;
var
  aName: string;

  aTP: TTopolElement;
begin
  Result:= false;
  if not (aTlEntity is TTopolElement) then exit;
  aTP:= TTopolElement(aTlEntity);

  try
  aName:= aTP.DBFName;
  if aName <> TableName then
  begin
    Table.Active:= false;
    if FileExists(aName) then
    begin
      TableName:= aName;
{$IFDEF ADO}
    SetConnectionStringForTableName;
{$ENDIF}
      AktCislo:= aTP.Cislo;
      Table.Active:= true;
      { pridani zaznamu a nastaveni se na nej }
      Table.Append;
      Result:= true;
    end
    else
    if OKCreateTableFromCreateFile(aName) then
    begin
      TableName:= aName;
{$IFDEF ADO}
    SetConnectionStringForTableName;
{$ENDIF}
      Table.Active:= true;
      Table.Append;
      Result:= true
    end
  end
  else
  begin
    AktCislo:= aTP.Cislo;
    Table.Active:= true;
    Table.Append;
    Result:= true;
  end;
  except
    Result:= false;
    Table.Active:= false
  end
end;



procedure TTopolDBFTable.SetTransferScript(ScriptName: string);
begin
  if FTransferScript <> nil then
  begin
    if ScriptName = FTransferScript.FileName then
       exit
    else
    begin
      FTransferScript.Free;
      FTransferScript:= nil;
    end;
  end;

  if (ScriptName <> '') and FileExists(ScriptName) then
  begin
    FTransferScript:= TMovedItemContainer.Create;
    FTransferScript.ReadItems(ScriptName)
  end
end;


procedure TTopolDBFTable.SetObjectNo(No: longInt);
begin
  Table.FieldByName('CISLO').AsInteger:= No
end;




procedure TTopolDBFTable.TransferFieldsFromDBF(aTable: TDataSetElement);
var
  I: longInt;
  SName, TName: string;
  SField, TarField: TField;

begin
  if FTransferScript <> nil then
  begin
    for I:= 0 to FTransferScript.Count-1 do
    begin
      SName:= FTransferScript.GetSourceItem(I);
      if (SName <> '') and not (SName[1] in ['*', ';']) then
      begin
        TName:= FTransferScript.GetTargetItem(I);
        try
          SField:= aTable.Table.FieldByName(SName);
          TarField:= Table.FieldByName(TName);
          if (SField <> nil) and (TarField <> nil) then
          begin
            TarField.AsString:= SField.AsString;
          end
        except
          on E:EDatabaseError do
          begin
            SName:= E.Message;
          end;
        end
      end { if smysluplny nazev polozky }
    end;
  end
  else
  begin
    for I:= 0 to Table.FieldCount-1 do
    begin
      try
        TName:= Table.Fields[I].FieldName;
        if TName = 'CISLO' then continue;

        SField:= aTable.Table.FieldByName(TName);
        if SField <> nil then
        begin
          SName:= SField.AsString;
          Table.Fields[I].AsString:= SName
        end;
      except
        on E:EDatabaseError do
        begin
          SName:= E.Message;
        end;
      end
    end
  end;
end;




procedure TTopolDBFTable.TransferFieldsFromProperties(anElement: TElement);
var
  I: longInt;
  SName, TName, SString: string;
  TarField: TField;

begin
  if FTransferScript <> nil then
  begin
    for I:= 0 to FTransferScript.Count-1 do
    begin
      SName:= FTransferScript.GetSourceItem(I);
      if (SName <> '') and (SName[1] = '*') then
      begin
        SName:= copy(SName, 2, 255);
        TName:= FTransferScript.GetTargetItem(I);
        try
          SString:= anElement.GetPropertyStringByName(SName);
          TarField:= Table.FieldByName(TName);
          if (TarField <> nil) then
          begin
            TarField.AsString:= SString;
          end
        except
          on E:EDatabaseError do
          begin
            SName:= E.Message;
          end;
        end
      end { if smysluplny nazev polozky }
    end;
  end
{ nebezpen vc,
  else
  begin
    for I:= 0 to GetPropCount(anElement)-1 do
    begin
      try
        Fields[I].AsString:= GetPropString(anElement, I)
      except
        on E:EDatabaseError do
        begin
          SName:= E.Message;
        end;
      end
    end
  end;}
end;



procedure TTopolDBFTable.TransferFieldsFromDynamicProperties
           (anElement: TElement; aDynProps: TDynamicProperties);

var
  I: longInt;
  TypEl: TTypeElement;
  SName: string;
  TarField: TField;

begin
  { oekv v objektu aDynProps poloky o sprvnch nzvech pro tabulku DBF }
  if aDynProps = nil then exit;

  for I:= 0 to aDynProps.Count-1 do
  begin
    TypEl:= TTypeElement(aDynProps.Elements[I]);
    if (TypEl.UpperName <> '') then
    begin
    try
      TarField:= Table.FieldByName(TypEl.UpperName);
      if (TarField <> nil) then
      begin
        TarField.AsString:= TypEl.AsString;
      end
    except
      on E:EDatabaseError do
      begin
        SName:= E.Message;
      end;
      end
    end { if smysluplny nazev polozky }
  end;
end;




procedure TTopolDBFTable.CreateAndAppend;
begin
  if not fileExists(TableName) then
  begin
    Table.Active:= false;
    if not OKCreateTableFromCreateFile(TableName) then exit
  end;

  Table.Active:= true;
  Table.Append
end;


function TTopolDBFTable.OKCreateIndexFile: boolean;
var
  aIndexName: string;
  Cislo, I: longInt;
  aByte: byte;
  aRecNo, Vysledek: longInt;
  RP: RecProps;

begin
  result:= false;
  if TLIndexFile <> nil then TLIndexFile.Free;
  TLIndexFile:= nil;
  aIndexName:= ChangeFileExt(TableName, '.IND');
  if FileExists(aIndexName) then DeleteFile(aIndexName);

  TLIndexFile:= TFileStream.Create(aIndexName, fmCreate);
  if TLIndexFile = nil then
  begin
    MessageDlg('Index ' + aIndexName + ' nelze vytvorit', mtWarning, [mbOK], 0);
    exit
  end;

  Table.Active:= true;
  if RecordCount > 0 then
  begin
    Table.First;
    aRecNo:= 1;
    repeat
      Cislo:= GetCislo;
      aRecNo:= RecNo;
      if aRecNo < 0 then
      begin
        ShowMessage('Chyba prce s DBF ' + TableName + ' pi ten zznamu pro CISLO = ' + IntToStr(Cislo));
        TLIndexFile.Free;
        TLIndexFile:= nil;
        Table.Active:= false;
        exit
      end;
      if Cislo*sizeOf(longInt) > TLIndexFile.Size+1 then
      begin
        aByte:= $FF;
        for I:= TLIndexFile.Size to Cislo*sizeOf(longInt) do
        begin
          TLIndexFile.seek(I, 0);
          TLIndexFile.write(aByte, 1)
        end;
      end;
      TLIndexFile.seek(Cislo*sizeOf(longInt),0);
      TLIndexFile.write(aRecNo, sizeOf(longInt));

      if not Table.EOF then
      begin
        Table.next;
        inc(aRecNo)
      end
    until Table.EOF;
  end;

  TLIndexFile.Free;
  TLIndexFile:= nil;
  Table.Active:= false;
  result:= true;
end;




function TTopolDBFTable.OKIndexFile: boolean;
var
  aIndexName: string;
  aFE, aOK: boolean;
begin
  result:= false;
  if TLIndexFile <> nil then
  begin
    result:= true;
    exit;
  end;

  LastRecNo:= 0;
{ Nalezeni indexoveho souboru *.IND }
  aIndexName:= ChangeFileExt(TableName, '.IND');
  aFE:= FileExists(aIndexName);

  aOK:= false;
  if not aFE then
     aOK:= OKCreateIndexFile;

  if aFE or aOK then
  begin
    TLIndexFile:= TFileStream.Create(aIndexName, fmOpenRead);
    if TLIndexFile = nil then
    begin
      MessageDlg('Index ' + aIndexName + ' nelze vytvoit', mtWarning, [mbOK], 0);
      exit
    end;
    result:= true
  end
  else
  begin
    MessageDlg('Index ' + aIndexName + ' je nepstupn', mtWarning, [mbOK], 0);
    exit
  end
end;



procedure Register;
begin
  //RegisterComponents('Orbis', [TTopolDBFTable]);
end;




procedure TTopolDBFTable.SetCislo(const Value: integer);
begin
  SetRecordForCislo(Value);
end;

end.
