unit E_TopIS;

interface

uses
// KERNEL
   Elements,
// TOPOL
   e_ImpSrc;


type

TTopoLImportBlocks    = class(TImportSource);
TTopoLImportLibraries = class(TImportSource);
TTopoLImportRasters   = class(TImportSource);

TTopoLImportSource = class(TImportSource)
 protected
   FBlocks   : TTopoLImportBlocks;
   FLibraries: TTopoLImportLibraries;
   FRasters  : TTopoLImportRasters;
   function  GetNoBlocks: boolean;
   function  GetNoLibraries: boolean;
   function  GetNoRasters: boolean;
   function  GetBlocks   : TTopoLImportBlocks;
   function  GetLibraries: TTopoLImportLibraries;
   function  GetRasters: TTopoLImportRasters;
   procedure SetBlocks(aBlocks: TTopoLImportBlocks);
   procedure SetLibraries(aLibraries: TTopoLImportLibraries);
   procedure SetRasters(aRasters: TTopoLImportRasters);
   function  GetNewBlock: string;
   procedure SetNewBlock(aValue: string);
   function  GetNewRaster: string;
   procedure SetNewRaster(aValue: string);
   function  GetNewLibrary: string;
   procedure SetNewLibrary(aValue: string);
 public
   property NoBlocks: boolean read GetNoBlocks;
   property NoLibraries: boolean read GetNoLibraries;
   property NoRasters: boolean read GetNoRasters;
   property Blocks   : TTopoLImportBlocks read GetBlocks    write SetBlocks;
   property Libraries: TTopoLImportLibraries read GetLibraries write SetLibraries;
   property Rasters: TTopoLImportRasters read GetRasters write SetRasters;
 published
   property NewBlock: string read GetNewBlock write SetNewBlock;
   property NewRaster: string read GetNewRaster write SetNewRaster;
   property NewLibrary: string read GetNewLibrary write SetNewLibrary;
end;

implementation

uses
// KERNEL
   e_Class;


function  TTopoLImportSource.GetNewBlock: string;
begin
   result := '';
end;

procedure TTopoLImportSource.SetNewBlock(aValue: string);
begin
   Blocks.AddItem(aValue);
end;

function  TTopoLImportSource.GetNewRaster: string;
begin
   result := '';
end;

procedure TTopoLImportSource.SetNewRaster(aValue: string);
begin
   Rasters.AddItem(aValue);
end;

function  TTopoLImportSource.GetNewLibrary: string;
begin
   result := '';
end;

procedure TTopoLImportSource.SetNewLibrary(aValue: string);
begin
   Libraries.AddItem(aValue);
end;


function TTopoLImportSource.GetNoBlocks: boolean;
begin
   result := not assigned(FBlocks) or (FBlocks.Count < 1);
end;

function TTopoLImportSource.GetNoLibraries: boolean;
begin
   result := not assigned(FLibraries) or (FLibraries.Count < 1);
end;

function TTopoLImportSource.GetNoRasters: boolean;
begin
   result := not assigned(FRasters) or (FRasters.Count < 1);
end;

function  TTopoLImportSource.GetBlocks   : TTopoLImportBlocks;
begin
   if not assigned(FBlocks) then
   begin
      FBlocks := TTopoLImportBlocks.Create;
      AddElement(FBlocks);
   end;
   result := FBlocks;
end;

function  TTopoLImportSource.GetLibraries: TTopoLImportLibraries;
begin
   if not assigned(FLibraries) then
   begin
      FLibraries := TTopoLImportLibraries.Create;
      AddElement(FLibraries);
   end;
   result := FLibraries;
end;

function  TTopoLImportSource.GetRasters: TTopoLImportRasters;
begin
   if not assigned(FRasters) then
   begin
      FRasters := TTopoLImportRasters.Create;
      AddElement(FRasters);
   end;
   result := FRasters;
end;

procedure TTopoLImportSource.SetBlocks(aBlocks: TTopoLImportBlocks);
begin
   if assigned(FBlocks) then FreeElement(FBlocks);
   FBlocks := nil;
   FBlocks  := aBlocks;
end;

procedure TTopoLImportSource.SetLibraries(aLibraries: TTopoLImportLibraries);
begin
   if assigned(FLibraries) then FreeElement(FLibraries);
   FLibraries := nil;
   FLibraries := aLibraries;
end;

procedure TTopoLImportSource.SetRasters(aRasters: TTopoLImportRasters);
begin
   if assigned(FRasters) then FreeElement(FRasters);
   FRasters := nil;
   FRasters := aRasters;
end;



BEGIN
   RegisterOrbisClass(TTopoLImportBlocks);
   RegisterOrbisClass(TTopoLImportLibraries);
   RegisterOrbisClass(TTopoLImportRasters);
   RegisterOrbisClass(TTopoLImportSource);
END.
