unit T_TopExp;

interface
uses
// KERNEL
  Elements,
  Tools,
  T_Recurs,
// GLIB
  g_Elems,
  e_WCS,
// SPACE
  e_Geometry,
  //e_Space,
// SPACETOOL
  T_ToTool,
  T_Topol,
  //e_TopMDL,
// TOPOL
  e_TopoLElements,
  U_TopolT,
  C_TlDBF,
  T_TopImp;


type
   TTopolCoordWriter = class(TTopoLCoordReader)
   private
     FRewrite: boolean;
   protected
       procedure OpenStream; override;
    public
       constructor  CreateAll(aBlockDirectory: string; aWCS: TWCS; aRewrite: boolean);
       function     GetTopolSour(Coord: T2DCoordinate): TSour;
       procedure    WriteCoord(Coord: T2DCoordinate);
       procedure    AddCoord(Coord: T2DCoordinate; var Position: longInt);
   end;



TTopolLinWriter = class(TTopoLLinReader)
private
    FDruh: smallInt;
    FRewrite: boolean;
    FNoTopology: boolean;
    procedure   WriteLinLin(LinE: TTopoLLineElement; Lin: TLin);
    procedure SetNoTopology(const Value: boolean);

 protected
    FCoordWriter: TTopolCoordWriter;
    procedure   OpenStream; override;
    procedure   OpenCoords(aBlockDirectory: string; aWCS: TWCS); override;
 public
    constructor CreateAll(aBlockDirectory: string; aWCS: TWCS; aRewrite: boolean);
    destructor  Destroy; override;
    procedure   WriteLinCoords
                (LinE: TPolyLineElement; var FirstC, LastC: longInt);
    procedure   WriteLin(LinE: TTopoLLineElement);
    procedure   WriteLin2(LinE: TTopoLLineElement);
    procedure   WriteLinCisla(LinE: TTopoLLineElement;
                              aLeftArea, aRightArea: longInt);
    procedure   WriteLinSVazbami(LinE: TTopoLLineElement; aDrziSe: TDrziSe);
 published
   property Druh: smallInt read FDruh write FDruh;
   property NoTopology: boolean read FNoTopology write SetNoTopology;
end;



{
TWriteBlockLines = class(TTopolLinWriter)
  constructor Create(aBlockDirectory: string; aRewrite: boolean);
end;
}

TTopolObjectWriter = class(TTopolObjectReader)
private
  FDruh: smallInt;
protected
  FRewrite: boolean;
  procedure   OpenStream; override;
public
  constructor CreateAll(aBlockDirectory: string; aWCS: TWCS; aRewrite: boolean);
published
  property Druh: smallInt read FDruh write FDruh;
end;


TTopolAreaWriter = class(TTopoLObjectWriter)
public
  procedure   SetRecordSize; override;
  function    GetFileName: string; override;
  procedure   WriteArea(aPlocha: TTopoLAreaElement; FirstLineNumber: integer);
end;


TTopolPointWriter = class(TTopoLObjectWriter)
public
  procedure   SetRecordSize; override;
  function    GetFileName: string; override;
  procedure   WritePoint(aPoint: TTopoLPointElement);
end;


TTopolTextWriter = class(TTopoLTextReader)
protected
    procedure   OpenStream; override;
public
    constructor CreateBlockWCS(aBlockDirectory: string; aWCS: TWCS); override;
    procedure   OpenTTX(aBlockDirectory: string); override;
    procedure   WriteText(aText: TTopoLTextElement);
end;



(*

TExportTopolLiniiRecursor = class(TProchazecLiniiGauge)
private
  FExportLinii: TTopolLinWriter;
  FPocet: longInt;
  procedure ZapisDoBloku(aLin: TTopoLLineElement); virtual;
public
  FDruhLinii: integer;
  FBarvaLinii: integer;
  FZnackaLinii: integer;

  procedure Process(Element: TElement); override;
  constructor Create(aBlockDirectory: string; aWCS: TWCS; aRewrite: boolean);
  destructor Destroy; override;
end;

TExportTopolLiniiDBRecursorI = class(TExportTopolLiniiRecursor)
private
  procedure ZapisDoBloku(aLin: TPolyLineElement); override;
  procedure WriteLinWithConnection(aLin: TPolyLineElement);
public
  function  QGoInside(Element: TElement): boolean; override;
  function  QProcessBefore(Element: TElement): boolean; override;
  procedure Process(Element: TElement); override;
  procedure DoIt; override;
end;


TExportTopolLiniiDBFRecursor = class(TExportTopolLiniiRecursor)
private
  procedure ZapisDoBloku(aLin: TPolyLineElement); override;
protected
{  procedure WriteLinWithConnection(aLin: TPolyLineElement); }
public
  FExportDBFLin: TTopolDBFTable;
  FCreateDBFFileName: string;
  FDBFFileName      : string;
  FFieldName        : string;
  procedure DoIt; override;
end;


TExportTopolLiniiDBFRecursorPruseku = class(TExportTopolLiniiDBFRecursor)
private
  procedure ZapisDoBloku(aLin: TPolyLineElement); override;
end;


TExportTopolLiniiDBFRecursorVod = class(TExportTopolLiniiDBFRecursor)
private
  procedure ZapisDoBloku(aLin: TPolyLineElement); override;
end;



TExportTopolLiniiRecursorDleTridy = class(TProchazecLiniiDleTridy)
private
  FExportLinii: TTopolLinWriter;
  FPocet: longInt;
public
  FDruhLinii: integer;
  FBarvaLinii: integer;
  FZnackaLinii: integer;

    procedure Process(Element: TElement); override;

    constructor Create(aBlockDirectory: string; aWCS: TWCS; aRewrite: boolean);
    destructor Destroy; override;
end;




{ Export lini z uzl }
TExportAllLines = class(TTool)
private
  FRewrite: boolean;
public
  constructor Create(aRewrite: boolean);
  procedure DoIt; override;
end;



TExportNodedLines = class(TProchazecListuQTreeGauge)
private
  FRewrite: boolean;
  FPocetLinii, FLineNo: longInt;
  FExportLinii: TTopolLinWriter;
  FBlockDirectory: string;
  FWCS: TWCS;
  FExportSymbol: boolean;

{ smazat po 20.5.96
  procedure ExportLinie(Node: TTopolNode; Branch: TTopolNodeBranch;
                        Line: TPolyLineElement);
}
public
  constructor Create; override;
  procedure DoIt; override;
{  procedure Process(Element: TElement); override; }

published
  property Rewrite: boolean read FRewrite write FRewrite;
  property BlockDirectory: string read FBlockDirectory write FBlockDirectory;
  property WCS: TWCS read FWCS write FWCS;
  property ExportSymbol: boolean read FExportSymbol write FExportSymbol;
end;

*)

{
procedure ExportTopolLinie(const Adresar: string);
}



implementation

uses
  Forms, Controls,
  SysUtils, Classes, FileCtrl, Dialogs,
// KERNEL
  Worlds,
// SYMBOL
  u_Symbol,
// SPACETOOL
  E_Node,
  //F_Main,
// TOPOL
  CreaDBF;
  //E_TplPrp;




procedure SetAllBlockFiles(aBlockDirectory: string);
var
  aStream: TStream;
  aRecordSize: longInt;
  aName: string;
  aCoord: TSour;
  aLine: TLin;
  aAre: TPlocha;
  aPoint: TBod;
  aText: TTopText;
  aVeta: string;

begin
  aName:= aBlockDirectory + '\TN.CRD';
  aRecordSize:= sizeOf(TSour);
  aStream := TFileStream.Create(aName, fmCreate);
  aCoord.X:= 0; aCoord.Y:= 0;
  aStream.Write(aCoord, aRecordSize);
  aStream.Free;

  aName:= aBlockDirectory + '\TN.LIN';
  aRecordSize:= sizeOf(TLin);
  aStream := TFileStream.Create(aName, fmCreate);
  aLine.Atributy:= [ZrusenaL];
  aStream.Write(aLine, aRecordSize);
  aStream.Free;

  aName:= aBlockDirectory + '\TN.ARE';
  aRecordSize:= sizeOf(TPlocha);
  aStream := TFileStream.Create(aName, fmCreate);
  aAre.Atributy:= [ZrusenaP];
  aStream.Write(aAre, aRecordSize);
  aStream.Free;

  aName:= aBlockDirectory + '\TN.PNT';
  aRecordSize:= sizeOf(TBod);
  aStream := TFileStream.Create(aName, fmCreate);
  aPoint.Atributy:= [ZrusenB];
  aStream.Write(aPoint, aRecordSize);
  aStream.Free;

  aName:= aBlockDirectory + '\TN.TBX';
  aRecordSize:= sizeOf(TTopText);
  aStream := TFileStream.Create(aName, fmCreate);
  aText.Atributy:= [ZrusenT];
  aStream.Write(aText, aRecordSize);
  aStream.Free;

  aName:= aBlockDirectory + '\TN.TTX';
  aStream := TFileStream.Create(aName, fmCreate);
  aVeta:= 'Orbis-export pro Topol (2002) ... soubor textu pro blok ' +
          aBlockDirectory + chr(13) + chr(10);
  aRecordSize:= length(aVeta);

  aStream.Write(aVeta[1], aRecordSize);
  aStream.Free;
end;



               { TTopolCoordWriter }

constructor  TTopolCoordWriter.CreateAll(aBlockDirectory: string; aWCS: TWCS; aRewrite: boolean);
begin
  FRewrite:= aRewrite;
  inherited CreateBlockWCS(aBlockDirectory, aWCS)
end;


procedure TTopolCoordWriter.OpenStream;
begin
  if FRewrite then
     FStream := TFileStream.Create(SSS, fmCreate)
  else
     FStream := TFileStream.Create(SSS, fmOpenReadWrite)
end;


function TTopolCoordWriter.GetTopolSour(Coord: T2DCoordinate): TSour;
var
   PomCoord: T2DCoordinate;
begin
   PomCoord:= T2DCoordinate.Create;
   try
     PomCoord.X:= Coord.X;
     PomCoord.Y:= Coord.Y;
     WCS.RecalculateFromICS(PomCoord);
     result.Y:= PomCoord.X;
     result.X:= PomCoord.Y;
     PomCoord.Free
   except
     on E:Exception do
     begin
       ShowMessage(E.Message);
       Screen.Cursor:= crDefault;
       try
         ShowMessage(FloatToStr(Coord.X) + ' ' + FloatToStr(Coord.X))
       except
         on E:Exception do
         begin
           ShowMessage(E.Message + ' (After FloatToStr)');
         end
       end
     end
   end
end;


procedure TTopolCoordWriter.WriteCoord(Coord: T2DCoordinate);
var
  Sour: TSour;
begin
   Sour:= GetTopolSour(Coord);
   WriteIt(Sour);
end;


procedure TTopolCoordWriter.AddCoord(Coord: T2DCoordinate; var Position: longInt);
begin
  Position:= Size;
  SeekTo(Position);
  WriteCoord(Coord)
end;


                { TTopolLinWriter }

procedure TTopolLinWriter.OpenStream;
begin
  if FRewrite then
     FStream := TFileStream.Create(SSS, fmCreate)
  else
     FStream := TFileStream.Create(SSS, fmOpenReadWrite)
end;


constructor TTopolLinWriter.CreateAll(aBlockDirectory: string; aWCS: TWCS; aRewrite: boolean);
begin
  FRewrite:= aRewrite;

  if not DirectoryExists(aBlockDirectory) then
  begin
  { vytvoreni adresare bloku }
    ForceDirectories(aBlockDirectory);
    SetAllBlockFiles(aBlockDirectory);
  end;

  inherited CreateBlockWCS(aBlockDirectory, aWCS);
end;


procedure   TTopolLinWriter.OpenCoords(aBlockDirectory: string; aWCS: TWCS);
begin
   FCoordWriter := TTopoLCoordWriter.CreateAll(aBlockDirectory, aWCS, FRewrite);
end;



Destructor TTopolLinWriter.Destroy;
begin
   if assigned(FCoordWriter) then FCoordWriter.Free;
   inherited Destroy;
end;







procedure TTopolLinWriter.WriteLinCoords
          (LinE: TPolyLineElement; var FirstC, LastC: longInt);
var
  I: longInt;
begin
  FCoordWriter.AddCoord(T2DCoordinate(LinE.Elements[0]), FirstC);
  for I:= 1 to LinE.Count-1 do
  begin
    FCoordWriter.AddCoord(T2DCoordinate(LinE.Elements[I]), LastC);
  end
end;



procedure    TTopolLinWriter.WriteLin2(LinE: TTopoLLineElement);
var
   //Sour: TSour;
   aRect: TRectangle;
   //i: longInt;
   //OldPos: longInt;{!!!!!!!!!!!!!!!}
   L: TPolyLineElement;
   Lin: TLin;
begin
  L := TPolyLineElement(LinE.Geometry);
  if L = nil then exit;
  Lin.Cislo:=  LinE.ID;

  aRect:= TRectangle.Create;
  aRect.MinX:= L.Rect.MinX;
  aRect.MinY:= L.Rect.MinY;
  aRect.MaxX:= L.Rect.MaxX;
  aRect.MaxY:= L.Rect.MaxY;

  FWCS.RecalculateFromICS(aRect.Min);
  FWCS.RecalculateFromICS(aRect.Max);

  with Lin.OpsanyObdelnik do
  begin
    PH.Y:= aRect.MaxX;
    PH.X:= aRect.MaxY;
    LD.Y:= aRect.MinX;
    LD.X:= aRect.MinY;
  end;
  aRect.Free;

  if FDruh <= 0 then
     Lin.Druh       := LinE.Druh
  else
     Lin.Druh:= FDruh;

  Lin.Barva      := LinE.Barva;
  Lin.Znacka     := LinE.Znacka;
  Lin.Presnost   := LinE.Presnost;

  Lin.LevaPlocha := LinE.LeftArea;
  Lin.PravaPlocha:= LinE.RightArea;

  Lin.Delka      := L.Length;
  Lin.Vybrana    := LinE.Vyber;
  Lin.Atributy   := [];
  if NoTopology then
     Lin.Atributy   := [Netopologicka];
  Lin.Symbol     := LinE.Symbol;

  Lin.DrziSe     := LinE.DrziSe;

  GiveSourFromCoord(L.X, L.Y, Lin.PolohaSymbolu.Sour);

  Lin.PolohaSymbolu.UhelKolmice:= L.Angle;

  WriteLinCoords(L, Lin.PrvniSour, Lin.PosledniSour);

  SeekTo(Lin.Cislo);
  WriteIt(Lin);
end;



procedure    TTopolLinWriter.WriteLinCisla(LinE: TTopoLLineElement;
                                           aLeftArea, aRightArea: longInt);
var
   //Sour: TSour;
   aRect: TRectangle;
   //i: longInt;
   //OldPos: longInt;{!!!!!!!!!!!!!!!}
   L: TPolyLineElement;
   Lin: TLin;
begin
  L := TPolyLineElement(LinE.Geometry);
  if L = nil then exit;
  Lin.Cislo:=  LinE.ID;

  aRect:= TRectangle.Create;
  aRect.MinX:= L.Rect.MinX;
  aRect.MinY:= L.Rect.MinY;
  aRect.MaxX:= L.Rect.MaxX;
  aRect.MaxY:= L.Rect.MaxY;

  FWCS.RecalculateFromICS(aRect.Min);
  FWCS.RecalculateFromICS(aRect.Max);

  with Lin.OpsanyObdelnik do
  begin
    PH.Y:= aRect.MaxX;
    PH.X:= aRect.MaxY;
    LD.Y:= aRect.MinX;
    LD.X:= aRect.MinY;
  end;
  aRect.Free;

  if FDruh <= 0 then
     Lin.Druh       := LinE.Druh
  else
     Lin.Druh:= FDruh;

  Lin.Barva      := LinE.Barva;
  Lin.Znacka     := LinE.Znacka;
  Lin.Presnost   := LinE.Presnost;

  Lin.LevaPlocha := aLeftArea;
  Lin.PravaPlocha:= aRightArea;

  Lin.Delka      := L.Length;
  Lin.Vybrana    := LinE.Vyber;
  Lin.Atributy   := [];
  Lin.Symbol     := LinE.Symbol;

  Lin.DrziSe     := LinE.DrziSe;

  GiveSourFromCoord(L.X, L.Y, Lin.PolohaSymbolu.Sour);

  Lin.PolohaSymbolu.UhelKolmice:= L.Angle;

  WriteLinCoords(L, Lin.PrvniSour, Lin.PosledniSour);

  SeekTo(Lin.Cislo);
  WriteIt(Lin);
end;




procedure    TTopolLinWriter.WriteLinLin(LinE: TTopoLLineElement; Lin: TLin);
var
   //Sour: TSour;
   //aRect: TRectangle;
   //i: longInt;
   //OldPos: longInt;{!!!!!!!!!!!!!!!}
   L: TPolyLineElement;
begin
  L := TPolyLineElement(LinE.Geometry);
  if not (TElement(L) is TPolyLineElement) then exit;
  Lin.Cislo:=  LinE.Cislo;
(*
  with Lin.OpsanyObdelnik do
  begin
{    aRect:= LinE.Rect.CreateDuplicate; }
    aRect:= TRectangle.Create;
    aRect.MinX:= LinE.Rect.MinX;
    aRect.MinY:= LinE.Rect.MinY;
    aRect.MaxX:= LinE.Rect.MaxX;
    aRect.MaxY:= LinE.Rect.MaxY;

    FWCS.RecalculateFromICS(aRect.Min);
    FWCS.RecalculateFromICS(aRect.Max);

    PH.Y:= aRect.MaxX;
    PH.X:= aRect.MaxY;
    LD.Y:= aRect.MinX;
    LD.X:= aRect.MinY
    aRect.Free
  end;
*)
  Lin.Druh       := LinE.Druh;
  Lin.Barva      := LinE.Barva;
  Lin.Znacka     := LinE.Znacka;
  Lin.Presnost   := LinE.Presnost;

  Lin.LevaPlocha := LinE.LeftArea;
  Lin.PravaPlocha:= LinE.RightArea;

  Lin.Delka      := L.Length;
  Lin.Vybrana    := LinE.Vyber;
  Lin.Atributy   := [];
  Lin.Symbol     := LinE.Symbol;

  GiveSourFromCoord(L.X, L.Y, Lin.PolohaSymbolu.Sour);

  Lin.PolohaSymbolu.UhelKolmice:= L.Angle;

  WriteLinCoords(L, Lin.PrvniSour, Lin.PosledniSour);

  SeekTo(Lin.Cislo);
  WriteIt(Lin);
end;




procedure    TTopolLinWriter.WriteLin(LinE: TTopoLLineElement);
var
   Lin: TLin;
begin
  Lin.DrziSe:= NedrziSe;
  WriteLinLin(LinE, Lin)
end;



procedure    TTopolLinWriter.WriteLinSVazbami(LinE: TTopoLLineElement; aDrziSe: TDrziSe);
var
  Lin: TLin;
begin
  Lin.DrziSe:= aDrziSe;
  WriteLinLin(LinE, Lin)
end;



             {  TWriteBlockLines }

{
constructor  TWriteBlockLines.Create(aBlockDirectory: string; aRewrite: boolean);
var
  WCS: TWCS;

begin
  WCS:=  TWCSWorld(World).GetIDWCS(cs_JTSK);
  inherited Create(aBlockDirectory, WCS, aRewrite);
end;


}



procedure TTopolLinWriter.SetNoTopology(const Value: boolean);
begin
  FNoTopology := Value;
end;

{ TTopolObjectWriter }

procedure TTopolObjectWriter.OpenStream;
begin
  if FRewrite then
     FStream := TFileStream.Create(SSS, fmCreate)
  else
     FStream := TFileStream.Create(SSS, fmOpenReadWrite)
end;





constructor TTopolObjectWriter.CreateAll(aBlockDirectory: string; aWCS: TWCS; aRewrite: boolean);
begin
  FRewrite:= aRewrite;

  if not DirectoryExists(aBlockDirectory) then
  begin
  { vytvoreni adresare bloku }
    ForceDirectories(aBlockDirectory)
  end;

  inherited CreateBlockWCS(aBlockDirectory, aWCS);
end;


                         { TTopolAreaWriter }



procedure   TTopolAreaWriter.SetRecordSize;
begin
   FRecordSize := SizeOf(TPlocha);
end;



function    TTopolAreaWriter.GetFileName: string;
begin
   Result := 'TN.ARE';
end;




procedure TTopolAreaWriter.WriteArea(aPlocha: TTopoLAreaElement; FirstLineNumber: integer);
var
  Plo: TPlocha;
  aRect: TRectangle;
  X,Y,Z: extended;
  Are: TPolyLineAreaElement;
  TL: TTopoLLineElement;
  //PropsExists: boolean;
begin
  Are := TPolyLineAreaElement(aPlocha.Geometry);
  //PropsExists:= (aProps is TTopolAreaProps);

  aRect:= TRectangle.Create;

  aRect.MinX:= Are.Rect.MinX;
  aRect.MinY:= Are.Rect.MinY;
  aRect.MaxX:= Are.Rect.MaxX;
  aRect.MaxY:= Are.Rect.MaxY;

  FWCS.RecalculateFromICS(aRect.Min);
  FWCS.RecalculateFromICS(aRect.Max);

  with Plo.OpsanyObdelnik do
  begin
    PH.Y:= aRect.MaxX;
    PH.X:= aRect.MaxY;
    LD.Y:= aRect.MinX;
    LD.X:= aRect.MinY
  end;
  aRect.Free;

  //if PropsExists then
  begin
    Plo.Cislo      := aPlocha.Cislo;
    Plo.Druh       := aPlocha.Druh;
    Plo.Barva      := aPlocha.Barva;
    Plo.Znacka     := aPlocha.Znacka;
    Plo.CisloOtce  := aPlocha.PaterNo;
    Plo.MaEnklavu  := aPlocha.HasChildren;
    Plo.Vybrana    := aPlocha.Vyber;
  end;
  {
  if Are.Elements[0] is TPolyLineElement then
  begin
    TL := TTopoLLineElement(Are.Elements[0].FindOwnerClass(TTopoLLineElement));
    // !!! a co kdy topolskou linii nenajde ?
    Plo.PrvniLinie := TL.Id;
  end;
  }
  Plo.PrvniLinie := FirstLineNumber;

  Plo.Vymera     := Are.WholeArea;

  Plo.Atributy   := [];

  WCS.GiveWCSFromICS(Are.X, Are.Y, 0, X, Y, Z);
  Plo.Sour.Y:= X;
  Plo.Sour.X:= Y;
  SeekTo(Plo.Cislo);
  WriteIt(Plo);
end;



                         { TTopolPointWriter }



procedure   TTopolPointWriter.SetRecordSize;
begin
   FRecordSize := SizeOf(TBod);
end;


function    TTopolPointWriter.GetFileName: string;
begin
   Result := 'TN.PNT';
end;



procedure TTopolPointWriter.WritePoint(aPoint: TTopoLPointElement);
var
  Bod: TBod;
  //aRect: TRectangle;
  X,Y,Z: extended;
  P: TPointElement;
  //PropsExists: boolean;
begin
  P := TPointElement(aPoint.Geometry);
  //PropsExists:= (aProps is TTopolProps);
  Bod.Cislo:= aPoint.Id;
  Bod.Uhel := P.Angle;
  //if PropsExists then
  begin
    Bod.Druh       := aPoint.Druh;
    Bod.Barva      := aPoint.Barva;
    Bod.Znacka     := aPoint.Znacka;
    Bod.Presnost   := aPoint.Presnost;
    Bod.Vybran     := aPoint.Vyber;
  end;

  Bod.Atributy   := [];

  WCS.GiveWCSFromICS(P.X, P.Y, 0, X, Y, Z);
  Bod.Sour.Y:= X;
  Bod.Sour.X:= Y;
  SeekTo(Bod.Cislo);
  WriteIt(Bod);
end;


(*

             { TExportTopolLiniiRecursor }



constructor TExportTopolLiniiRecursor.Create
            (aBlockDirectory: string; aWCS: TWCS; aRewrite: boolean);
begin
  inherited Create;
  if not DirectoryExists(aBlockDirectory) then
  begin
  { vytvoreni adresare bloku }
    ForceDirectories(aBlockDirectory)
  end;

  FExportLinii:= TTopolLinWriter.Create(aBlockDirectory, aWCS, aRewrite);
  if aRewrite then
     FPocet:= 0
  else
     FPocet:= FExportLinii.GetSize-1;
end;




destructor TExportTopolLiniiRecursor.Destroy;
begin
  if FExportLinii <> nil then
     FExportLinii.Free;
  inherited Destroy;
end;



procedure TExportTopolLiniiRecursor.ZapisDoBloku(aLin: TTopoLLineElement);
begin
  if (FDruhLinii > 0) then
  begin
    aLin.Druh:= FDruhLinii;
    aLin.Barva:= FBarvaLinii;
    aLin.Znacka:= FZnackaLinii;
  end;
  FExportLinii.WriteLin(aLin);
end;



procedure TExportTopolLiniiRecursor.Process(Element: TElement);
var
  //Linie: TTopolLinie;
  Lin: TTopoLLineElement;
  I: longInt;
begin
  if Element is TTopoLLineElement then
  begin
    Lin:= TTopoLLineElement(Element);
    if not Lin.IsDone then
    begin
      inc(FPocet);
      TTopolLineProps(Lin.Properties).Cislo:= FPocet;
      ZapisDoBloku(Lin);
      Lin.IsDone:= true
    end;

    GaugeSet(FPocetLinii);
    inc(FPocetLinii)
  end
end;



             { TExportTopolLiniiDBFRecursor }


function GetKodPrusekuDleDruhu(FDruhLinii: integer): integer;
begin
  case FDruhLinii of
  22: result:= 1;
  10325: result:= 2;
  else result:= 9
  end
end;



function GetNazevProKodL(FDruhLinii: integer): string;
begin
  case FDruhLinii of
  21: result:= 'PE';
  22: result:= 'P4';
  24: result:= '1L';
  25: result:= 'TR';
  28: result:= '1S';
  34: result:= '2L';
  35: result:= 'OC';
  38: result:= '2S';
  350: result:= 'KP';
  352: result:= '1T';
  353: result:= '2T';
  354: result:= '3T';
  10325: result:= 'P6';
  10245: result:= '1L';
  10340: result:= '2L';
  10341: result:= '2L';
  10280: result:= '1S';
  10281: result:= '1S';
  10380: result:= '2S';
  10381: result:= '2S';
  else result:= 'XX'
  end
end;



procedure GiveNodeAndBranch(Lin: TPolyLineElement; var Node: TNode; var Branch: TNodeBranch);
var
  I: longInt;
  Nasel: boolean;
begin
  Node:= nil;
  Branch:= nil;
  I:= 0;
  Nasel:= false;
  while (I < Lin.OwnersCount) and not Nasel do
  begin
    if (Lin.Owners[I] is TNode) {and (TTopolPlocha(Owners[I]).Druh = aGroup)} then
    begin
      Node:= TNode(Lin.Owners[I]);
      Nasel:= true
    end;
    if not Nasel then inc(I)
  end;
  if Nasel then
     Branch:= Node.GetBranchOfLineSimp(Lin)
end;




procedure SaveAreaForLine(Plocha: TPolyLineAreaElement);
var
  FExportPloch: TTopolAreaWriter;
  aProps: TTopolAreaProps;
begin
  aProps:= TTopolAreaProps(Plocha.Properties);
  if not (aProps is TTopolAreaProps) then exit;

  FExportPloch:= TTopolAreaWriter.Create
        (aProps.BlockName, TWCSWorld(World).GetIDWCS(cs_JTSK), false);
  if FExportPloch <> nil then
  begin
    FExportPloch.WriteArea(Plocha);
    FExportPloch.Free;
  end
end;



          { TExportTopolLiniiDBRecursorI }


procedure TExportTopolLiniiDBRecursorI.WriteLinWithConnection(aLin: TPolyLineElement);
var
  Node, SecNode: TNode;
  Branch, SecBranch: TNodeBranch;
  Cislo: longInt;
  CPravePlochy: longInt;
  PravaPlocha: TPolyLineAreaElement;
begin
{
  if FPocet = 0 then inc(FPocet);
  GiveNodeAndBranch(aLin, Node, Branch);
  if (Node <> nil) and (Branch <> nil) then
  begin

    CPravePlochy:= Branch.GetRightAreaNo;
    Branch.SetRightAreaNo(CPravePlochy);
    PravaPlocha:= Branch.RightArea;

    Branch.GiveLineSecondNodeBranch(SecNode, SecBranch);
    SecBranch.SetRightAreaNo(SecBranch.GetRightAreaNo);
}

{
    Node.SetLineConnectionsForExport(Branch, Branch.FLine, FPocet);
}
{
    FExportLinii.WriteLinSVazbami(Branch.FLine, Branch.FLine.DrziSe);
}
    if (PravaPlocha <> nil) and (PravaPlocha.PrvniLinie > 0) then
    begin
{      PravaPlocha.PrvniLinie:= Branch.LineNo; }
      SaveAreaForLine(PravaPlocha);
      PravaPlocha.PrvniLinie:= -PravaPlocha.PrvniLinie
    end;
  end
  else
  begin
   if not aLin.IsDone then
   begin
{     aLin.Cislo:= FPocet; }
     inc(FPocet)
   end;

   FExportLinii.WriteLin(aLin);
  end
}
end;



procedure TExportTopolLiniiDBRecursorI.ZapisDoBloku(aLin: TPolyLineElement);
begin
{
  try
    FExportDBFLin.Append;
    FExportDBFLin.SetObjectNo(aLin.Cislo);
    if FFieldName <> '' then
    begin
      try
         FExportDBFLin.FieldByName(FFieldName).AsString:= GetNazevProKodL(aLin.Druh);
         FExportDBFLin.FieldByName('DELKA').AsFloat:= aLin.Delka;
         FExportDBFLin.FieldByName('CES_NAZ').AsInteger:= aLin.Druh;
      finally
      FExportDBFLin.Post
      end
    end;
  finally
    aLin.Druh:= FDruhLinii;
    aLin.Barva:= FBarvaLinii;
    aLin.Znacka:= FZnackaLinii;
    aLin.Vyber:= false;
 }
  WriteLinWithConnection(aLin);
end;



function  TExportTopolLiniiDBRecursorI.QGoInside(Element: TElement): boolean;
begin
  Result:= (Element is TOwnerEntity) and not (Element is TPolyLineString)
end;




function TExportTopolLiniiDBRecursorI.QProcessBefore(Element: TElement): boolean;
begin
  Result:= (Element is TPolyLineString)
end;



procedure TExportTopolLiniiDBRecursorI.Process(Element: TElement);
var
  Lin: TPolyLineElement;
  I: longInt;
begin
  if Element is TPolyLineString then
  begin
    for I:= 0 to TPolyLineString(Element).Count-1 do
    begin
      Lin:= TPolyLineElement(TPolyLineString(Element).Elements[I]);
      ZapisDoBloku(Lin);
      GaugeSet(FPocetLinii);
      inc(FPocetLinii)
    end;

  end
end;




procedure TExportTopolLiniiDBRecursorI.DoIt;
var
  OKej: boolean;
begin
  inherited DoIt
end;






      {  TExportTopolLiniiDBFRecursor }

procedure TExportTopolLiniiDBFRecursor.ZapisDoBloku(aLin: TPolyLineElement);
begin
  try
    if FDruhLinii > 0 then
    begin
      try
        FExportDBFLin.Append;
{        FExportDBFLin.SetObjectNo(aLin.Cislo); }
        if FFieldName <> '' then
        begin
          try
{
             FExportDBFLin.FieldByName(FFieldName).AsString:= GetNazevProKodL(aLin.Druh);
             FExportDBFLin.FieldByName('DELKA').AsFloat:= aLin.Delka;
             FExportDBFLin.FieldByName('CES_NAZ').AsInteger:= aLin.Druh;
}
          finally
          FExportDBFLin.Post
          end
        end;
      finally
{
        aLin.Druh:= FDruhLinii;
        aLin.Barva:= FBarvaLinii;
        aLin.Znacka:= FZnackaLinii;
        aLin.Symbol:= 0;
        aLin.Vyber:= false;
}
        FExportLinii.WriteLin(aLin)
{        WriteLinWithConnection(aLin);}
      end
    end;
  except
  end
end;



procedure TExportTopolLiniiDBFRecursorPruseku.ZapisDoBloku(aLin: TPolyLineElement);
begin
  try
    if FDruhLinii > 0 then
    begin
{
      try
        FExportDBFLin.Append;

        FExportDBFLin.SetObjectNo(aLin.Cislo);
        if FFieldName <> '' then
        begin
          try
             FExportDBFLin.FieldByName(FFieldName).AsInteger:=
                           GetKodPrusekuDleDruhu(aLin.Druh);
             FExportDBFLin.FieldByName('DELKA').AsFloat:= aLin.Delka;
          finally
          FExportDBFLin.Post
          end
        end;
      finally
        aLin.Druh:= FDruhLinii;
        aLin.Barva:= FBarvaLinii;
        aLin.Znacka:= FZnackaLinii;
        aLin.Symbol:= 0;

        FExportLinii.WriteLin(aLin)
      end
}
    end;
  except
  end
end;




procedure TExportTopolLiniiDBFRecursorVod.ZapisDoBloku(aLin: TPolyLineElement);
begin
  try
    if FDruhLinii > 0 then
    begin
{
      try
        FExportDBFLin.Append;
        FExportDBFLin.SetObjectNo(aLin.Cislo);
        try
           FExportDBFLin.FieldByName('DELKA').AsFloat:= aLin.Delka;
        finally
          FExportDBFLin.Post
        end
      finally
        aLin.Druh:= FDruhLinii;
        aLin.Barva:= FBarvaLinii;
        aLin.Znacka:= FZnackaLinii;
        aLin.Symbol:= 0;

        FExportLinii.WriteLin(aLin)
      end
}
    end;
  except
  end
end;





procedure TExportTopolLiniiDBFRecursor.DoIt;
var
  OKej: boolean;
begin
  FExportDBFLin:= TTopolDBFTable.Create(Application{MainForm});
  if (FExportDBFLin <> nil) then
  try
    FExportDBFLin.CreateFileName:= FCreateDBFFileName;
    VytvorDBDleCRT(FCreateDBFFileName, FDBFFileName, OKej);
    if OKej then
    begin
      FExportDBFLin.TableName:= FDBFFileName;
      FExportDBFLin.Active:= true;
      inherited DoIt
    end
  finally
    FExportDBFLin.Free
  end
end;



             { TExportTopolLiniiRecursorDleTridy }


constructor TExportTopolLiniiRecursorDleTridy.Create
            (aBlockDirectory: string; aWCS: TWCS; aRewrite: boolean);
begin
  inherited Create;
  if not DirectoryExists(aBlockDirectory) then
  begin
  { vytvoreni adresare bloku }
    ForceDirectories(aBlockDirectory)
  end;

  FExportLinii:= TTopolLinWriter.Create(aBlockDirectory, aWCS, aRewrite);
  FPocet:= 0;
end;


destructor TExportTopolLiniiRecursorDleTridy.Destroy;
begin
  if FExportLinii <> nil then
     FExportLinii.Free;
  inherited Destroy;
end;



procedure TExportTopolLiniiRecursorDleTridy.Process(Element: TElement);
var
  Linie: TPolyLineString;
  Lin: TPolyLineElement;
  I: longInt;
begin
  if Element is FClass then
  begin
    Linie:= TPolyLineString(TEntity(Element).FindElementClass(TPolyLineString));
    if Linie <> nil then
    begin
      for I:= 0 to Linie.Count-1 do
      begin
        Lin:= TPolyLineElement(Linie.Elements[I]);
        if not Lin.IsDone then
        begin
          inc(FPocet);
{
          Lin.Cislo:= FPocet;
          if FDruhLinii > 0 then
          begin
            Lin.Druh:= FDruhLinii;
            Lin.Barva:= FBarvaLinii;
            Lin.Znacka:= FZnackaLinii;
          end;
          FExportLinii.WriteLin(Lin);
          Lin.IsDone:= true
}
        end;

        GaugeSet(FPocetLinii);
        inc(FPocetLinii)
      end;
    end
  end
end;


constructor TExportAllLines.Create(aRewrite: boolean);
begin
  FRewrite:= aRewrite;
  inherited Create
end;


procedure TExportAllLines.DoIt;
var
  ExportLin: TExportTopolLiniiRecursor;
  WCS: TWCS;
begin
  if Input is TOwnerEntity then
  begin
    WCS:=  TWCSWorld(World).GetIDWCS(cs_JTSK);

    ExportLin:= TExportTopolLiniiRecursor.Create('\ORBIS\TEST.BLK', WCS, FRewrite);
    ExportLin.Input:= Input;
    ExportLin.DoIt;
    ExportLin.Free
  end
end;


constructor TExportNodedLines.Create;
begin
  FBlockDirectory:= '\ORBIS\TEST.BLK';

  if not DirectoryExists(FBlockDirectory) then
  begin
  { vytvoreni adresare bloku }
    ForceDirectories(FBlockDirectory)
  end;

  FWCS:= TWCSWorld(World).GetIDWCS(cs_JTSK);
  inherited Create;
end;


procedure TExportNodedLines.DoIt;
var
  Prochazec: TOznacLinie;
begin
  Prochazec:= TOznacLinie.Create;
  Prochazec.Input:= Input;
  Prochazec.DoIt;
  GaugeMaxValue(Prochazec.FPocetLinii);
  GaugeSetModulo(10);
  GaugeShow;
  Prochazec.Free;

  FExportLinii:= TTopolLinWriter.Create(FBlockDirectory, FWCS, FRewrite);

  FLineNo:= 1;
  inherited DoIt;

  FExportLinii.Free;

  GaugeHide
end;




{
procedure TExportNodedLines.Process(Element: TElement);
var
  Nodes: TOwnerEntity;
  DoNode: TNodeConector;
  I, J: longInt;
  Node: TTopolNode;
  Line: TPolyLineElement;
begin
  inc(FPocetListu);
  GaugeSet(FPocetListu);
  Nodes:= TOwnerEntity(TEntity(Element).FindElementName('NODES'));
  if Nodes <> nil then
  begin
    for I:= 0 to Nodes.Count-1 do
    begin
      if Nodes.Elements[I] is TTopolNode then
      begin
        Node:= TTopolNode(Nodes.Elements[I]);
        for J:= 0 to Node.Count-1 do
        begin
          if Node.Elements[J] is TTopolNodeBranch then
          begin
            Line:= TTopolNodeBranch(Node.Elements[J]).FLine;
            if (Line <> nil) then
            begin
            { nastaven vazeb linim }
              Node.SetLineConnectionsForExport
                   (TTopolNodeBranch(Node.Elements[J]), Line, FLineNo);

              if not ExportSymbol then
   {!!!!!       Line.Symbol:= 0; }

{              FExportLinii.WriteLinSVazbami(Line, Line.DrziSe) }

            {  ExportLinie(Node, TTopolNodeBranch(Node.Elements[J]), Line); }
            end
          end
        end
      end
    end;
  end
end;
}


{
procedure ExportTopolLinie(const Adresar: string);
var
  ExportLin: TExportTopolLiniiRecursorDleTridy;
  WCS: TWCS;
  Les: TForestSpaceUnit;
  Oznacovac: TOznacLinie;
  Pocet: longInt;

begin
  Les:= TForestSpaceUnit(World.ObjectModel.FindElementName('LES'));

  if Les <> nil then
  begin
    WCS:=  TWCSWorld(World).GetIDWCS(cs_JTSK);

    ExportLin:= TExportTopolLiniiRecursorDleTridy.Create(Adresar + '\oddeleni.blk', WCS);
    ExportLin.Input:= Les;
    ExportLin.SetContainerClass(TOddeleni);
    ExportLin.DoIt;
    ExportLin.Free;

    ExportLin:= TExportTopolLiniiRecursorDleTridy.Create(Adresar + '\porost.blk', WCS);
    ExportLin.Input:= Les;
    ExportLin.SetContainerClass(TPorost);
    ExportLin.DoIt;
    ExportLin.Free;

    ExportLin:= TExportTopolLiniiRecursorDleTridy.Create(Adresar + '\porskup.blk', WCS);
    ExportLin.Input:= Les;
    ExportLin.SetContainerClass(TPorostniSkupina);
    ExportLin.DoIt;
    ExportLin.Free;
  end
end;
}
*)


{ TTopolTextWriter }

constructor TTopolTextWriter.CreateBlockWCS(aBlockDirectory: string; aWCS: TWCS);
begin
  if not DirectoryExists(aBlockDirectory) then
  begin
  { vytvoreni adresare bloku }
    ForceDirectories(aBlockDirectory);
    SetAllBlockFiles(aBlockDirectory);
  end;
  inherited;
end;

procedure TTopolTextWriter.OpenStream;
begin
    FStream := TFileStream.Create(SSS, fmCreate);
end;

procedure TTopolTextWriter.OpenTTX(aBlockDirectory: string);
begin
   System.Assign(FText, aBlockDirectory+'\TN.TTX');
   System.Rewrite(FText);
end;

procedure TTopolTextWriter.WriteText(aText: TTopoLTextElement);
var
   Tx: U_TopoLT.TTopText;
   Veta: string;
   TP: TTextElement;
begin
   if aText = nil then
   begin
      Tx.Atributy := [ZrusenT];
      Veta := 'ORBIS 2002';
      FStream.Write(Tx,FRecordSize);
      System.Writeln(FText,Veta);
      exit;
   end;
   TP := TTextElement(aText.Geometry);
   if TP = nil then
      Veta := '?'
   else
      Veta := TP.Text;


   TX.Cislo   :=  aText.Cislo;
   TX.Druh    :=  aText.Druh;
   TX.Atributy:=  aText.TextAtrib;
   TX.Vybran  :=  aText.Vyber;
   TX.Barva   :=  aText.Barva;
   TX.DruhCary:=  aText.Znacka;


   Tx.RezimTextu  := aText.TextMode;
   Tx.Druh        := aText.Druh;
   Tx.Smer        := aText.Direction;
   Tx.Font        := aText.TopolFont;
   Tx.VychoziBod.Y:= -aText.StartPoint.X;
   Tx.VychoziBod.X:= -aText.StartPoint.Y;
   Tx.KoefVysky   := aText.Height;
   Tx.KoefSirky   := aText.WidthCoef;
   Tx.Rozestupy   := aText.Distance;
   Tx.SmerNapevno := true;
   FStream.Write(Tx,FRecordSize);
   System.Writeln(FText,Veta);

end;

end.

