{**********************************************************}
{                                                          }
{       DPMI-Unit pro Turbo Pascal pro Windows             }
{       Tento unit umoznuje pristup na vnitrni Windows     }
{       funkce pro zpracovani pameti.                      }
{       DPMI-funkce jsou pod systemem Windows k dispozici  }
{       jenom ve standardnim a rozsirenem modu.            }
{       Copyright (c) 1991 by FE                           }
{                                                          }
{**********************************************************}

UNIT DPMI;

{$G+,F+}

INTERFACE

USES
  WinProcs, WinTypes, OWindows, WinDos, Objects;

  { "Magicke" symboly }

  PROCEDURE __AHINCr; FAR;
  PROCEDURE __ROMBios; FAR;
  PROCEDURE __0000H; FAR;
  PROCEDURE __0040H; FAR;
  PROCEDURE __A000H; FAR;
  PROCEDURE __B000H; FAR;
  PROCEDURE __B800H; FAR;
  PROCEDURE __C000H; FAR;
  PROCEDURE __D000H; FAR;
  PROCEDURE __F000H; FAR;
  PROCEDURE __AHShift; FAR;

TYPE

  DWord = LongInt;

  PCallStructure = ^TCallStructure;
  TCallStructure = RECORD
    _EDI: DWord;
    _ESI: DWord;
    _EBP: DWord;
    _Reserved: DWord;
    _EBX: DWord;
    _EDX: DWord;
    _ECX: DWord;
    _EAX: DWord;
    _Flags: Word;
    _ES: Word;
    _DS: Word;
    _FS: Word;
    _GS: Word;
    _IP: Word;
    _CS: Word;
    _SP: Word;
    _SS: Word;
  END;

TYPE

  PDescriptor = ^TDescriptor;
  TDescriptor = RECORD
    LimitLo: Word;
    BaseLo: Word;
    BaseHi1: Byte;
    Flags: Byte;
    LimitAndFlags: Byte;
    BaseHi2: Byte;
  END;

  { DPMI Funkce }

  FUNCTION Global32BitAlloc(VAR Selector: Word; Size: LongInt): Boolean;
  FUNCTION Global32BitFree(Selector: Word): Boolean;
  FUNCTION Create16BitAlias(S32: Word; Offset: LongInt): Word;
  FUNCTION Free16BitAlias(Selector: Word): Boolean;
  FUNCTION CreateMapSelector(P: Pointer; Size: LongInt): Word;
  FUNCTION MapRealModeSegment(Segment: Word): Word;
  PROCEDURE GetDescriptor(VAR D: TDescriptor; Selector: Word);
  PROCEDURE SetDescriptor(D: TDescriptor; Selector: Word);
  FUNCTION DPMI_Intr(IntNo: Byte; VAR R: TRegisters): Boolean;
  PROCEDURE GetRMIntVec(IntNo: Byte; VAR P: Pointer);
  PROCEDURE SetRMIntVec(IntNo: Byte; P: Pointer);

  { Low-Level manipulace s pameti }

  FUNCTION GetValueB(Selector: Word; Offset: LongInt): Byte; FAR;
  PROCEDURE SetValueB(Selector: Word; Offset: LongInt; NewValue: Byte); FAR;
  FUNCTION GetValueW(Selector: Word; Offset: LongInt): Word; FAR;
  PROCEDURE SetValueW(Selector: Word; Offset: LongInt; NewValue: Word); FAR;
  FUNCTION GetValueL(Selector: Word; Offset: LongInt): LongInt; FAR;
  PROCEDURE SetValueL(Selector: Word; Offset: LongInt; NewValue: LongInt); FAR;
  PROCEDURE MoveMemB(Selector: Word; Source, Dest, Count: LongInt); FAR;
  FUNCTION ScanMemB(Selector: Word; Item: Byte; Start, Count: LongInt): LongInt; FAR;
  PROCEDURE FillMemB(Selector: Word; Item: Byte; Start, Count: LongInt); FAR;
  PROCEDURE FillMemW(Selector: Word; Item: Word; Start, Count: LongInt); FAR;
  PROCEDURE FillMemL(Selector: Word; Item: LongInt; Start, Count: LongInt); FAR;

  { Real-Mode manipulace s pameti}

  FUNCTION SetupRMArea(P: Pointer; Size: Word): LongInt;
  PROCEDURE FreeRMArea(S: Word);
  PROCEDURE MemS(S, O: Word; V: Byte);
  FUNCTION MemG(S, O: Word): Byte;

  { Protected-Mode-funkce }

  FUNCTION ValidSelector(S: Word): Boolean;
  FUNCTION SelectorLimit(S: Word): LongInt; { pouze pro Enhanced-mod }

VAR

  MajorVersion,
  MinorVersion: Byte;
  DPMI80386: Boolean;
  BackToRMOnInts: Boolean;
  VirtualMemory: Boolean;
  Processor: (P286, P386, P486);

  DPMIAvailable: Boolean;

TYPE

  PHSRecord = ^THSRecord;
  THSRecord = RECORD
    SelectorFromLDT: Word;
    HandleFromAlloc: LongInt;
  END;

  PHSCollection = ^THSCollection;
  THSCollection = OBJECT(TSortedCollection)
    FUNCTION Compare(Key1, Key2: Pointer): Integer; VIRTUAL;
    PROCEDURE FreeItem(Item: Pointer); VIRTUAL;
  END;

CONST

  { Collection pro ulozeni paru Handle/Selektor }

  HandleSelectorTable: PHSCollection = NIL;

  { Deskriptor-Management }

  LDTAllocDescriptor    = $0000;
  LDTFreeDescriptor     = $0001;
  LDTSegment2Descriptor = $0002;
  LDTGetNextSelINCVal   = $0003;
  LDTSetDescriptorBase  = $0007;
  LDTSetDescriptorSize  = $0008;
  LDTSetDescriptorAR    = $0009;
  LDTGetDescriptor      = $000B;
  LDTSetDescriptor      = $000C;

  { DOS-zprava pameti }

  DOSAllocMemBlock      = $0100;
  DOSFreeMemBlock       = $0101;
  DOSResizeMemBlock     = $0102;

  { Interrupt-funkce }

  GetRModeInt           = $0200;
  SetRModeInt           = $0201;
  GetExceptHandler      = $0202;
  SetExceptHandler      = $0203;
  GetPModeInt           = $0204;
  SetPModeInt           = $0205;

  { Interrupt-simulacni funkce }

  SimulateRMInt         = $0300;

  { nahrazovaci funkce }

  CallRealModeProcedure = $0301;
  AllocRealModeCallBack = $0303;
  FreeRealModeCallBack  = $0304;

  { Informacni funkce }

  GetDPMIVersion        = $0400;

  { Simulacni funkce }

  GetFreeMemInfo        = $0500;
  AllocMemBlock         = $0501;
  FreeMemBlock          = $0502;
  ResizeMemBlock        = $0503;
  LockLinearRegion      = $0600;
  UnlockLinearRegion    = $0601;
  GetPageSize           = $0604;

  { Mapovaci funkce }

  MapPhysicalAddress    = $0800;

CONST

  _DPMI = $31;

FUNCTION GetValBitField(Item: Byte; BitOffset: Byte;
  BitLength: Byte): Byte;

IMPLEMENTATION

  PROCEDURE __AHINCr; EXTERNAL 'KERNEL' INDEX 114;
  PROCEDURE __ROMBios; EXTERNAL 'KERNEL' INDEX 173;
  PROCEDURE __0000H; EXTERNAL 'KERNEL' INDEX 183;
  PROCEDURE __0040H; EXTERNAL 'KERNEL' INDEX 193;
  PROCEDURE __A000H; EXTERNAL 'KERNEL' INDEX 174;
  PROCEDURE __B000H; EXTERNAL 'KERNEL' INDEX 181;
  PROCEDURE __B800H; EXTERNAL 'KERNEL' INDEX 182;
  PROCEDURE __C000H; EXTERNAL 'KERNEL' INDEX 195;
  PROCEDURE __D000H; EXTERNAL 'KERNEL' INDEX 179;
  PROCEDURE __F000H; EXTERNAL 'KERNEL' INDEX 194;
  PROCEDURE __AHShift; EXTERNAL 'KERNEL' INDEX 113;

  FUNCTION GetValueB(Selector: Word; Offset: LongInt): Byte; EXTERNAL;
  PROCEDURE SetValueB(Selector: Word; Offset: LongInt; NewValue: Byte); EXTERNAL;
  FUNCTION GetValueW(Selector: Word; Offset: LongInt): Word; EXTERNAL;
  PROCEDURE SetValueW(Selector: Word; Offset: LongInt; NewValue: Word); EXTERNAL;
  FUNCTION GetValueL(Selector: Word; Offset: LongInt): LongInt; EXTERNAL;
  PROCEDURE SetValueL(Selector: Word; Offset: LongInt; NewValue: LongInt); EXTERNAL;
  PROCEDURE MoveMemB(Selector: Word; Source, Dest, Count: LongInt); EXTERNAL;
  FUNCTION ScanMemB(Selector: Word; Item: Byte; Start, Count: LongInt): LongInt; EXTERNAL;
  PROCEDURE FillMemB(Selector: Word; Item: Byte; Start, Count: LongInt); EXTERNAL;
  PROCEDURE FillMemW(Selector: Word; Item: Word; Start, Count: LongInt); EXTERNAL;
  PROCEDURE FillMemL(Selector: Word; Item: LongInt; Start, Count: LongInt); EXTERNAL;
  FUNCTION SelectorLimit(S: Word): LongInt; EXTERNAL;
  {$L MEM32}

PROCEDURE MemS(S, O: Word; V: Byte);
BEGIN
  Mem[MapRealModeSegment(S):O] := V;
END;

FUNCTION MemG(S, O: Word): Byte;
BEGIN
  MemG := Mem[MapRealModeSegment(S):O];
END;

FUNCTION SetupRMArea(P: Pointer; Size: Word): LongInt;
VAR
  L: LongInt;
BEGIN
  L := GlobalDOSAlloc(Size);
  SetupRMArea := L;
  IF L = 0 THEN
    Exit;
  Move(P^, Ptr(LongRec(L).Lo, 0)^, Size);
END;

PROCEDURE FreeRMArea(S: Word);
BEGIN
  GlobalDOSFree(S);
END;

FUNCTION ValidSelector(S: Word): Boolean;
BEGIN
  ValidSelector := False;
  ASM
    MOV   AX, WORD PTR S
    LSL   BX,AX
    JNZ   @@1
    INC   WORD PTR @Result
  @@1:
  END;
END;

PROCEDURE GetRMIntVec(IntNo: Byte; VAR P: Pointer);
BEGIN
  ASM
    MOV   AX, GetRModeInt
    MOV   BL, IntNo
    INT   _DPMI
    JC    @@1
    LES   DI, P
    MOV   WORD PTR ES:[DI+2], CX
    MOV   WORD PTR ES:[DI], DX
  @@1:
  END;
END;

PROCEDURE SetRMIntVec(IntNo: Byte; P: Pointer);
BEGIN
  ASM
    MOV   AX, SetRModeInt
    MOV   BL, IntNo
    MOV   CX, WORD PTR P.2
    MOV   DX, WORD PTR P.0
    INT   _DPMI
    JC    @@1
  @@1:
  END;
END;

FUNCTION DPMI_Intr(IntNo: Byte; VAR R: TRegisters): Boolean;
VAR
  C: TCallStructure;
BEGIN
  DPMI_Intr := False;
  WITH C DO BEGIN
    _EDI := LongInt(R.DI);
    _ESI := LongInt(R.SI);
    _EBP := LongInt(R.BP);
    _Reserved := 0;
    _EBX := LongInt(R.BX);
    _EDX := LongInt(R.DX);
    _ECX := LongInt(R.CX);
    _EAX := LongInt(R.AX);
    _Flags := LongInt(R.Flags);
    _ES := LongInt(R.ES);
    _DS := LongInt(R.DS);
    _FS := 0;
    _GS := 0;

    { jsou funkci ignorovany }

    _IP := 0;
    _CS := 0;

    { pouzity DPMI-Host-Stack }

    _SP := 0;
    _SS := 0;

  END;
  ASM
    MOV     AX, SimulateRMInt
    MOV     BL, IntNo
    MOV     BH, 0
    MOV     CX, 0
    PUSH    SS
    POP     ES
    LEA     DI, [BP+OFFSET C]
    INT     _DPMI
    JC      @@1
    INC     WORD PTR @Result
  @@1:
  END;
  WITH C DO BEGIN
    R.DI := _EDI;
    R.SI := _ESI;
    R.BP := _EBP;
    R.BX := _EBX;
    R.DX := _EDX;
    R.CX := _ECX;
    R.AX := _EAX;
    R.Flags := _Flags;
    R.ES := _ES;
    R.DS := _DS;
  END;
END;

FUNCTION GetValBitField(Item: Byte; BitOffset: Byte;
  BitLength: Byte): Byte;
BEGIN
  GetValBitField := (Item SHR BitOffset) AND ($FF SHR (8-BitLength));
END;

FUNCTION THSCollection.Compare(Key1, Key2: Pointer): Integer;
BEGIN
  IF PWord(Key1)^ > PWord(Key2)^ THEN
    Compare := +1
  ELSE IF PWord(Key1)^ = PWord(Key2)^ THEN
    Compare := 0
  ELSE
    Compare := -1;
END;

PROCEDURE THSCollection.FreeItem(Item: Pointer);
BEGIN
  FreeMem(Item, SizeOf(THSRecord));
END;

PROCEDURE GetDescriptor(VAR D: TDescriptor; Selector: Word);
BEGIN
  ASM
    MOV   BX, Selector
    LES   DI, D
    MOV   AX, LDTGetDescriptor
    INT   _DPMI
  END;
END;

PROCEDURE SetDescriptor(D: TDescriptor; Selector: Word);
BEGIN
  ASM
    MOV   BX, Selector
    PUSH  DS
    POP   ES
    LEA   DI, [BP+OFFSET D]
    MOV   AX, LDTSetDescriptor
    INT   _DPMI
  END;
END;

FUNCTION Global32BitAlloc(VAR Selector: Word; Size: LongInt): Boolean;
VAR
  LinearAddr: LongInt;
  ASelector: Word;
  Handle: LongInt;
  P: PHSRecord;
  Limit: LongInt;
BEGIN
  Global32BitAlloc := False;
  IF Size >= $100000 THEN
    IF Size MOD 4096 <> 0 THEN
      Size := Size OR $FFF + 1;
      { Alokace nad 1 MByte museji mit stranovu granulaci (4KB)
         }
  ASelector := 0;
  Limit := Size-1;
  ASM
    MOV   CX, Word Ptr Size.0
    MOV   BX, Word Ptr Size.2
    MOV   AX, AllocMemBlock
    INT   _DPMI
    JC    @@1
    MOV   Word Ptr Handle.0, DI
    MOV   Word Ptr Handle.2, SI
    MOV   Word Ptr LinearAddr.0, CX
    MOV   Word Ptr LinearAddr.2, BX

    MOV   CX, 1
    MOV   AX, LDTAllocDescriptor
    INT   _DPMI
    JC    @@2
    MOV   ASelector, AX
    MOV   BX, AX
    MOV   CX, Word Ptr LinearAddr.2
    MOV   DX, Word Ptr LinearAddr.0
    MOV   AX, LDTSetDescriptorBase
    INT   _DPMI
    JC    @@3

    MOV   CX, Word Ptr Limit.2
    MOV   DX, Word Ptr Limit.0
    MOV   BX, ASelector
    MOV   AX, LDTSetDescriptorSize
    INT   _DPMI
    JC    @@4

    INC   Word Ptr @Result
    MOV   AX, ASelector
    LES   DI, Selector
    STOSW
    JMP   @@1
  @@4:
  @@3:
  @@2:
    MOV   SI, Word Ptr Handle.2
    MOV   DI, Word Ptr Handle.0
    MOV   AX, FreeMemBlock
    INT   _DPMI
  @@1:
  END;
  IF ASelector = 0 THEN
    Exit;
  New(P);
  P^.HandleFromAlloc := Handle;
  P^.SelectorFromLDT := ASelector;
  HandleSelectorTable^.Insert(P);
END;

FUNCTION Global32BitFree(Selector: Word): Boolean;
VAR
  Handle: LongInt;
  I: Integer;
BEGIN
  Global32BitFree := False;
  IF HandleSelectorTable^.Search(@Selector, I) THEN BEGIN
    Handle := PHSRecord(HandleSelectorTable^.At(I))^.HandleFromAlloc;
    HandleSelectorTable^.AtFree(I);
  END
  ELSE
    Exit;
  ASM
    MOV   BX, Selector
    MOV   AX, LDTFreeDescriptor
    INT   _DPMI
    JC    @@1
    MOV   SI, Word Ptr Handle.2
    MOV   DI, Word Ptr Handle.0
    MOV   AX, FreeMemBlock
    INT   _DPMI
    JC    @@2
    INC   Word Ptr @Result
  @@2:
  @@1:
  END;
END;

FUNCTION Create16BitAlias(S32: Word; Offset: LongInt): Word;
LABEL
  _Exit;
VAR
  LinearAddr: LongInt;
  ASelector: Word;
  D: TDescriptor;
BEGIN
  Create16BitAlias := 0;
  ASelector := 0;
  ASM
    MOV   AX, DS
    MOV   ES, AX
    LEA   DI, [BP+OFFSET D]
    MOV   BX, S32
    MOV   AX, LDTGetDescriptor       { cisty 32-Bit-Deskriptor }
    INT   _DPMI
    JC    _Exit
    MOV   AX, D.TDescriptor.BaseLo
    MOV   WORD PTR LinearAddr, AX
    MOV   AL, D.TDescriptor.BaseHi1
    MOV   BYTE PTR LinearAddr.2, AL
    MOV   AL, D.TDescriptor.BaseHi2
    MOV   BYTE PTR LinearAddr.3, AL
  END;
    LinearAddr := LinearAddr + Offset;
  ASM
    MOV   CX, 1
    MOV   AX, LDTAllocDescriptor
    INT   _DPMI
    JC    @@2
    MOV   ASelector, AX
    MOV   BX, AX
    MOV   CX, Word Ptr LinearAddr.2
    MOV   DX, Word Ptr LinearAddr.0
    MOV   AX, LDTSetDescriptorBase
    INT   _DPMI
    JC    @@3

    XOR   CX, CX
    MOV   DX, $FFFF   { hranica segmentu je 64K }
    MOV   BX, ASelector
    MOV   AX, LDTSetDescriptorSize
    INT   _DPMI
    JC    @@4


    MOV   AX, ASelector
    MOV   Word Ptr @Result, AX
    JMP   @@1
  _Exit:
  @@4:
  @@3:
  @@2:
  @@1:
  END;
END;

FUNCTION Free16BitAlias(Selector: Word): Boolean;
BEGIN
  Free16BitAlias := False;
  ASM
    MOV   BX, Selector
    MOV   AX, LDTFreeDescriptor
    INT   _DPMI
    JC    @@1
    INC   Word Ptr @Result
  @@1:
  END;
END;

FUNCTION CreateMapSelector(P: Pointer; Size: LongInt): Word;
VAR
  LinearAddr: LongInt;
  ASelector: Word;
BEGIN
  CreateMapSelector := 0;
  ASM
    MOV   BX, Word Ptr P.2
    MOV   CX, Word Ptr P.0
    MOV   SI, Word Ptr Size.2
    MOV   DI, Word Ptr Size.0
    MOV   AX, MapPhysicalAddress
    INT   _DPMI
    JC    @@1
    MOV   Word Ptr LinearAddr.0, CX
    MOV   Word Ptr LinearAddr.2, BX
    MOV   CX, 1
    MOV   AX, LDTAllocDescriptor
    INT   _DPMI
    JC    @@2

    MOV   ASelector, AX
    MOV   BX, AX
    MOV   CX, Word Ptr LinearAddr.2
    MOV   DX, Word Ptr LinearAddr.0
    MOV   AX, LDTSetDescriptorBase
    INT   _DPMI
    JC    @@3

    MOV   CX, Word Ptr Size.2
    MOV   DX, Word Ptr Size.0
    MOV   BX, ASelector
    MOV   AX, LDTSetDescriptorSize
    INT   _DPMI
    JC    @@4
    MOV   AX, ASelector
    MOV   Word Ptr @Result, AX
  @@4:
  @@3:
  @@2:
  @@1:
  END;
END;

FUNCTION MapRealModeSegment(Segment: Word): Word;
BEGIN
  MapRealModeSegment := 0;
  ASM
    MOV   BX, Segment
    MOV   AX, LDTSegment2Descriptor
    INT   _DPMI
    JC    @@1
    MOV   Word Ptr @Result, AX
  @@1:
  END;
END;

FUNCTION IsProtectedMode: Boolean;
VAR
  W: Word;
BEGIN
  W := GetWinFlags;
  IsProtectedMode := (W AND wf_PMode) = wf_PMode;
END;

VAR
  SaveExit: Pointer;

PROCEDURE DPMIExit; FAR;
BEGIN
  ExitProc := SaveExit;
  Dispose(HandleSelectorTable, Done);
END;

BEGIN
  DPMIAvailable := IsProtectedMode;
  IF NOT DPMIAvailable THEN
    Exit;

  SaveExit := ExitProc;
  ExitProc := @DPMIExit;

  HandleSelectorTable := New(PHSCollection, Init(16, 16));

  DPMI80386 := False;
  BackToRMOnInts := False;
  VirtualMemory := False;
  ASM
    MOV   AX, GetDPMIVersion
    INT   _DPMI
    MOV   MinorVersion, AL
    MOV   MajorVersion, AH
    TEST  BX, 1
    JZ    @@1
    INC   DPMI80386
  @@1:
    TEST  BX, 2
    JZ    @@2
    INC   BackToRMOnInts
  @@2:
    TEST  BX, 4
    JZ    @@3
    INC   VirtualMemory
  @@3:
    SUB   CL, 2
    MOV   Processor, CL
  END;
END.