unit F_ProgEdit;

interface

uses
  tools,
  language,
  T_Debug,
  E_type,
  Elements,Dialogs, ComCtrls, StdCtrls, Controls, ToolWin,
  Classes,
  Windows, Messages,M_Orbis, SysUtils,  Graphics,  Forms,
  Menus, ExtCtrls, Grids;

type
  TProgEdit=class;

  TProgEditTool=class(TTool)
  private
   FCompiled,FRunning,FPaused,FDebugging:boolean;
   FOkno:TProgEdit;
   FProgram:TOilProgram;
   FBPManager:TBreakpointManager;
   function GetProg:TOilProgram;
   procedure SetProg(ap:TOilProgram);
   function Getman:TBreakpointManager;
   procedure SetMan(aMan:TBreakpointManager);
  protected
   procedure UkazChybu(Zprava:string;Line,Pos:integer);
   {tyhle procedury by mely byt napojeny na message}
   procedure NotifyRun(var aMessage: TOrbisMessage; var Handled: boolean);virtual;
   procedure NotifyResume(var aMessage: TOrbisMessage; var Handled: boolean);virtual;
   procedure NotifyBreak(var aMessage: TOrbisMessage; var Handled: boolean);virtual;
   procedure NotifyPause(var aMessage: TOrbisMessage; var Handled: boolean);virtual;
   procedure NotifyEnd(var aMessage: TOrbisMessage; var Handled: boolean);virtual;
   procedure NotifyError(var aMessage: TOrbisMessage; var Handled: boolean);virtual;
   procedure ReadWatches;
  public
   procedure HandleMessage(var aMessage: TOrbisMessage; var Handled: boolean); override;
   property  OilProg:TOilProgram read GetProg write SetProg;
   property  BPManager:TBreakpointManager read GetMan write SetMan;
   property  Okno:TProgEdit read FOkno write FOkno;
   property  Compiled:boolean read FCompiled write FCompiled;
   property  Running:boolean read FRunning write FRunning;
   property  Debugging:boolean read FDebugging write FDebugging;
   property  Paused:boolean read FPaused write FPaused;
   procedure DrawBreakpoints;
   procedure DrawBP(line:integer);
   procedure DrawInvalidBP(line:integer);
   procedure DrawNormal(line:integer);
   procedure DrawCurLine(line:integer);
   procedure SetBreakPoint;
   {procedure ClickAtLine(Line:integer);}
   procedure ShiftBPoints(Up:boolean);
   procedure Continue;
   procedure Kompiluj;
   procedure ResetProg;
   procedure RunProgram;
  end;

  TProgEdit = class(TForm)
    REdit: TRichEdit;
    CoolBar1: TCoolBar;
    StatusBar1: TStatusBar;
    ToolBar1: TToolBar;
    CompileButton: TToolButton;
    RunButton: TToolButton;
    ToolButton3: TToolButton;
    DebugBox: TCheckBox;
    LoadButton: TToolButton;
    SaveButton: TToolButton;
    ToolButton5: TToolButton;
    OpenDialog: TOpenDialog;
    SaveDialog: TSaveDialog;
    WatchButton: TToolButton;
    ContinueButton: TToolButton;
    ToolButton1: TToolButton;
    BPointButton: TToolButton;
    Splitter1: TSplitter;
    Panel1: TPanel;
    Header: THeader;
    SG: TStringGrid;
    ToolButton2: TToolButton;
    ResetButton: TToolButton;
    procedure LoadButtonClick(Sender: TObject);
    procedure SaveButtonClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure CompileButtonClick(Sender: TObject);
    procedure RunButtonClick(Sender: TObject);
    procedure REditChange(Sender: TObject);
    procedure BPointButtonClick(Sender: TObject);
    procedure REditKeyUp(Sender: TObject; var Key: Word;
      Shift: TShiftState);
    procedure HeaderSizing(Sender: TObject; ASection, AWidth: Integer);
    procedure Panel1Resize(Sender: TObject);
    procedure ContinueButtonClick(Sender: TObject);
    procedure WatchButtonClick(Sender: TObject);
    procedure ResetButtonClick(Sender: TObject);
  private
    Edlines:integer;
    FMainTool:TProgEditTool;
    { Private declarations }
  public
    { Public declarations }
  end;


implementation
uses  worlds,
      T_Errors,
      T_Compil,
      U_String;


function TProgEditTool.Getman:TBreakpointManager;
begin
  if not assigned(FBPManager) then
   begin
     FBPManager:=TBreakpointManager.Create;
     AddElement(FBPManager);
   end;
  result:=FBPManager;
end;

procedure TProgEditTool.SetMan(aMan:TBreakpointManager);
begin
  if assigned(FBPManager) then FreeElement(FBPManager);
  FBPManager:=aMan;
  AddElement(FBPManager);
end;


function TProgEditTool.GetProg:TOilProgram;
begin
  result:=FProgram;
end;

procedure TProgEditTool.SetProg(aP:TOilProgram);
begin
 if assigned(FProgram) then
  begin
  world.Objectmodel.FreeElement(FProgram);
  end;
 FProgram:=aP;
 world.Objectmodel.AddElement(FProgram);{tenhle tool vysi mimo world, jinak bych pridaval do nej}
end;

procedure TProgEditTool.Kompiluj;
var
   MC: TProgramCompiler;
begin
   Compiled:=false;
   MC := TProgramCompiler.Create;
   try
     MC.Source:=Okno.REdit.Lines;
     MC.Debuged:=Okno.DebugBox.Checked;
     Debugging:=Okno.DebugBox.Checked;
     MC.DoIt;
     OilProg:=TOilProgram(MC.Output);
     Compiled:=true;
     Okno.StatusBar1.SimpleText:='Kompilace OK';
     Application.ProcessMessages;
   except on E:TCompilerError do
       begin
       OilProg:=nil;
       UkazChybu(e.message,e.Line,e.Position);
       showMessage('Chyba pi kopilaci');
       end;
       end;
   MC.Free;
end;

procedure TProgEditTool.ResetProg;
begin
 OilProg.Terminate;
 okno.ResetButton.Enabled:=false;
 okno.CompileButton.Enabled:=true;
 okno.WatchButton.Enabled:=false;
 okno.BPointButton.Enabled:=true;
 okno.ContinueButton.Enabled:=false;
 okno.RunButton.Enabled:=true;
 okno.LoadButton.Enabled:=true;
 okno.REdit.readOnly:=false;
end;

procedure TProgEditTool.RunProgram;
begin
  if not Compiled then
  Kompiluj;
  if Compiled and Debugging then TDBOilprogram(OilProg).ToNotify:=self;
  BPManager.input:=OilProg;
  BPManager.DoIt;
  if OilProg<>nil then
  begin
    try
    OilProg.Doit;
    except on E:TCompilerError do
       ShowMessage(OilProg.Name+e.message);
       end;
  end;
end;

procedure TProgEditTool.SetBreakPoint;
var ln,pos:integer;
begin
 pos:=okno.REdit.SelStart;
 Ln:=okno.REdit.Perform(EM_LINEFROMCHAR,pos,0);
 if not BPManager.IsInList(Ln) then
 begin
   if Running then
       begin
       BPManager.NextLineNumber:=Ln;
       BPManager.AddBreakPoint(Ln)
       end
    else BPManager.NextLineNumber:=Ln;
   DrawBp(Ln);
 end
  else
 begin
   if Running then
      begin
      BPManager.NextLineNumber:=Ln;
      BPManager.DeleteBreakPoint(Ln)
      end
    else BPManager.DeleteLineNumber(Ln);
   DrawNormal(Ln);
 end;
end;


procedure TProgEditTool.UkazChybu(Zprava:string;Line,Pos:integer);
{line jde od nuly, Pos od jedne}
var ln:integer;
begin
 Ln:=Okno.REdit.Perform(EM_LINEINDEX,Line, 0);
 SendMessage(Okno.REdit.Handle, EM_SETSEL,Ln+Pos-1,Ln+Pos-1);
 SendMessage(Okno.REdit.Handle, EM_SCROLLCARET, 0,0);
 {barevne zvyaznit}
 Okno.StatusBar1.SimpleText:='Chyba pi kompilaci('+IntToStr(Line)+'\'+IntToStr(Pos)+'): '+Zprava;
 Application.ProcessMessages;
end;

procedure TProgEditTool.DrawBP(line:integer);
var Pos,Ln,old:integer;
begin
old:=okno.REdit.SelStart;
pos:=Okno.REdit.Perform(EM_LINEINDEX,Line, 0);
Ln:=Okno.REdit.Perform(EM_LineLength,pos, -1);
SendMessage(Okno.REdit.Handle, EM_SETSEL,pos,Ln+Pos);
Okno.REdit.SelAttributes.Color:=clRed;
okno.REdit.SelStart:=old;
end;

procedure TProgEditTool.DrawInvalidBP(line:integer);
var Pos,Ln,old:integer;
begin
old:=okno.REdit.SelStart;
pos:=Okno.REdit.Perform(EM_LINEINDEX,Line, 0);
Ln:=Okno.REdit.Perform(EM_LineLength,pos, -1);
SendMessage(Okno.REdit.Handle, EM_SETSEL,pos,Ln+Pos);
Okno.REdit.SelAttributes.Color:=clYellow;
okno.REdit.SelStart:=old;
end;

procedure TProgEditTool.DrawNormal(line:integer);
var Pos,Ln,old:integer;
begin
old:=okno.REdit.SelStart;
pos:=Okno.REdit.Perform(EM_LINEINDEX,Line, 0);
Ln:=Okno.REdit.Perform(EM_LineLength,pos, -1);
Okno.REdit.Perform(EM_SETSEL,pos,Ln+Pos);
Okno.REdit.SelAttributes.Color:=Okno.REdit.DefAttributes.Color;
okno.REdit.SelStart:=old;
end;

procedure TProgEditTool.DrawBreakpoints;
var n,i:integer;
begin
okno.REdit.SelectAll;
okno.REdit.SelAttributes:=Okno.REdit.DefAttributes;
n:=BPManager.Numbers;
if Debugging and Running then
begin
 for i:=0 to n-1 do if BPManager.IsValidBreakpoint(i) then DrawBp(BPManager.lineNumber[i])
       else DrawInvalidBp(BPManager.lineNumber[i])
end
else
if n>0 then for i:=0 to n-1 do DrawBp(BPManager.lineNumber[i]);
end;

procedure TProgEditTool.DrawCurLine(line:integer);
var Pos,Ln:integer;
begin
pos:=Okno.REdit.Perform(EM_LINEINDEX,Line, 0);
Ln:=Okno.REdit.Perform(EM_LineLength,pos, -1);
Okno.REdit.Perform(EM_SETSEL,pos,Ln+Pos);
Okno.REdit.SelAttributes.Color:=clBlue;
SendMessage(Okno.REdit.Handle, EM_SCROLLCARET, 0,0);
end;

procedure TProgEditTool.ShiftBPoints(Up:boolean);
var k,Li:integer;
    old,max,i,S:integer;
begin
  S:=-1;
  if Up then k:=-1 else k:=1;
  Li:=okno.REdit.Perform(EM_LINEFROMCHAR,okno.REdit.Selstart,0);
  okno.Edlines:=okno.REdit.Perform(EM_GETLINECOUNT,0,0);;
  max:=BPManager.numbers;
  if max>0 then
  begin
  old:=okno.REdit.SelStart;
  if not Up then
   begin
    for i:=0 to max-1 do
    if BPManager.lineNumber[i]>=Li then
      BPManager.lineNumber[i]:=BPManager.lineNumber[i]+k;
   end
   else
   begin
    for i:=0 to max-1 do
    if BPManager.lineNumber[i]>Li then
     begin
     if BPManager.lineNumber[i]=Li+1 then S:=i
     else
       BPManager.lineNumber[i]:=BPManager.lineNumber[i]+k;
     end;
   end;
  if S>-1 then  BPManager.deleteLinenumber(BPManager.lineNumber[S]);
  DrawBreakpoints;
  okno.REdit.SelStart:=old;
  end;
end;

procedure TProgEditTool.ReadWatches;
var  i:integer;
     E:TTypeElement;
     WM:TWatchManager;
begin
  if debugging and running and Paused then
     begin
     WM:=TWatchManager.Create;
     WM.Input:=OilProg;
     for i := 0 to okno.SG.RowCount - 1 do
        begin
        if BezVnejsichMezer(okno.SG.Cells[0,i])<>''then
           begin
           E:=WM.GetVarValue(BezVnejsichMezer(okno.SG.Cells[0,i]));
           if E=nil then okno.SG.Cells[1,i]:='promnn neexistuje'
              else
              begin
              if (E is TPointerElement) then
                begin
                if (TObject(E.AsPointer) is TElement) then okno.SG.Cells[1,i]:=TElement(E.AsPointer).Name
                   else okno.SG.Cells[1,i]:=E.AsString;
                end
                else okno.SG.Cells[1,i]:=E.AsString;
              E.Free;
              end;
           end
           else okno.SG.Cells[1,i]:='';
        end;
     WM.Free;
     okno.StatusBar1.simpleText:='Stack: '+IntToStr(OilProg.stack.count)+' NameStack: '+IntToStr(OilProg.namestack.count);
     end;
end;

procedure TProgEditTool.NotifyRun(var aMessage: TOrbisMessage; var Handled: boolean);
begin
 Running:=true;
 Paused:=false;
 handled:=true;
 aMessage.free;
 okno.ResetButton.Enabled:=true;
 okno.CompileButton.Enabled:=false;
 okno.WatchButton.Enabled:=false;
 okno.BPointButton.Enabled:=false;
 okno.ContinueButton.Enabled:=false;
 okno.RunButton.Enabled:=false;
 okno.LoadButton.Enabled:=false;
 DrawBreakPoints;
 okno.REdit.readOnly:=true;
end;

procedure TProgEditTool.NotifyResume(var aMessage: TOrbisMessage; var Handled: boolean);
begin
 aMessage.free;
 Running:=true;
 Paused:=false;
 handled:=true;
 okno.ResetButton.Enabled:=true;
 okno.CompileButton.Enabled:=false;
 okno.WatchButton.Enabled:=false;
 okno.BPointButton.Enabled:=true;
 okno.ContinueButton.Enabled:=false;
 okno.RunButton.Enabled:=false;
 okno.LoadButton.Enabled:=false;
end;

procedure TProgEditTool.NotifyBreak(var aMessage: TOrbisMessage; var Handled: boolean);
begin
 handled:=true;
 Paused:=true;
 okno.SetFocus;
 DrawBreakpoints;
 DrawCurLine(TLineMessage(amessage).LineNum);
 aMessage.free;
 okno.ResetButton.Enabled:=true;
 okno.CompileButton.Enabled:=false;
 okno.WatchButton.Enabled:=true;
 okno.BPointButton.Enabled:=true;
 okno.ContinueButton.Enabled:=true;
 okno.RunButton.Enabled:=false;
 okno.LoadButton.Enabled:=false;
 ReadWatches;
end;


procedure TProgEditTool.NotifyPause(var aMessage: TOrbisMessage; var Handled: boolean);
begin
 handled:=true;
 Paused:=true;
 aMessage.free;
 okno.ResetButton.Enabled:=true;
 okno.CompileButton.Enabled:=false;
 okno.WatchButton.Enabled:=true;
 okno.BPointButton.Enabled:=true;
 okno.ContinueButton.Enabled:=true;
 okno.RunButton.Enabled:=false;
 okno.LoadButton.Enabled:=false;
end;

procedure TProgEditTool.NotifyEnd(var aMessage: TOrbisMessage; var Handled: boolean);
begin
 Running:=false;
 Paused:=true;
 handled:=true;
 aMessage.free;
 okno.ResetButton.Enabled:=false;
 okno.CompileButton.Enabled:=true;
 okno.WatchButton.Enabled:=false;
 okno.BPointButton.Enabled:=true;
 okno.ContinueButton.Enabled:=false;
 okno.RunButton.Enabled:=true;
 okno.LoadButton.Enabled:=true;
 okno.StatusBar1.Simpletext:='';
 DrawBreakPoints;
 okno.REdit.ReadOnly:=false;
end;

procedure TProgEditTool.NotifyError(var aMessage: TOrbisMessage; var Handled: boolean);
  var Pos,Ln,Line:integer;
begin
  DrawBreakpoints;
  Line:=TErrorMessage(aMessage).lineNum;
  pos:=Okno.REdit.Perform(EM_LINEINDEX,Line, 0);
  Ln:=Okno.REdit.Perform(EM_LineLength,pos, -1);
  Okno.REdit.Perform(EM_SETSEL,pos,Ln+Pos);
  SendMessage(Okno.REdit.Handle, EM_SCROLLCARET, 0,0);
  Okno.Statusbar1.SimpleText:=TErrorMessage(aMessage).text;
  aMessage.free;
  showMessage('Chyba pi bhu');
end;

procedure TProgEditTool.HandleMessage(var aMessage: TOrbisMessage; var Handled: boolean);
begin
   Handled := false;
   inherited HandleMessage(aMessage, Handled);
   if Handled then exit;
   case aMessage.Message of
      tm_OilStart   :NotifyRun(aMessage,Handled);
      tm_OilResume  :NotifyResume(aMessage,Handled);
      tm_OilEnded   :NotifyEnd(aMessage,Handled);
      tm_OilError   :NotifyError(aMessage,Handled);
      tm_OilBreak   :NotifyBreak(aMessage,Handled);
      tm_OILPaused  :NotifyPause(aMessage,Handled);
   end;{case}
end;

procedure TProgEditTool.Continue;
begin
 OilProg.continue;
end; 

{$R *.DFM}

procedure TProgEdit.LoadButtonClick(Sender: TObject);
begin
   if OpenDialog.InitialDir = '' then OpenDialog.InitialDir := ExtractFilePath(ParamStr(0));
   if OpenDialog.Execute then
   begin
      REdit.Lines.Clear;
      REdit.Lines.LoadFromFile(OpenDialog.FileName);
      REdit.SelectAll;
      REdit.SelAttributes:=REdit.DefAttributes;
      REdit.SelStart:=0;
      FMaintool.BPManager.NumberList:=nil;
      OpenDialog.InitialDir := ExtractFilePath(OpenDialog.FileName);
      Caption := 'OIL: '+ OpenDialog.FileName;
      REdit.Modified := false;
      FMaintool.Compiled:=false;
      EdLines:=REdit.Perform(EM_GETLINECOUNT,0,0);;
   end;
end;

procedure TProgEdit.SaveButtonClick(Sender: TObject);
begin
   SaveDialog.InitialDir := OpenDialog.InitialDir;
   SaveDialog.FileName   := OpenDialog.FileName;
   if SaveDialog.Execute then
   begin
      REdit.Lines.SaveToFile(SaveDialog.FileName);
      REdit.Modified := false;
      Caption := 'OIL: '+ SaveDialog.FileName;
   end;
end;

procedure TProgEdit.FormCreate(Sender: TObject);
begin
  FmainTool:=TProgEditTool.Create;
  FmainTool.okno:=self;
  EdLines:=REdit.Perform(EM_GETLINECOUNT,0,0);
end;

procedure TProgEdit.FormDestroy(Sender: TObject);
begin
  FmainTool.Free;
end;

procedure TProgEdit.CompileButtonClick(Sender: TObject);
begin
 FMaintool.Kompiluj;
end;

procedure TProgEdit.RunButtonClick(Sender: TObject);
begin
 FmainTool.RunProgram;
end;

procedure TProgEdit.REditChange(Sender: TObject);
begin
FMainTool.compiled:=false;
end;

procedure TProgEdit.BPointButtonClick(Sender: TObject);
begin
  Fmaintool.setBreakpoint;
end;

procedure TProgEdit.REditKeyUp(Sender: TObject; var Key: Word;
  Shift: TShiftState);
var Li:integer;
begin
  Li:=REdit.Perform(EM_GETLINECOUNT,0,0);
  if EdLines<>Li then FMainTool.ShiftBPoints(EdLines>Li);
end;

procedure TProgEdit.HeaderSizing(Sender: TObject; ASection,
  AWidth: Integer);
begin
   if aWidth <> SG.ColWidths[0] then
   begin
      SG.ColWidths[0] := aWidth;
      SG.ColWidths[1] := Header.SectionWidth[1]-25;
   end;
end;

procedure TProgEdit.Panel1Resize(Sender: TObject);
begin
   SG.ColWidths[0] := Header.SectionWidth[0];
   SG.ColWidths[1] := Header.SectionWidth[1]-25;
end;

procedure TProgEdit.ContinueButtonClick(Sender: TObject);
begin
 FMainTool.Continue;
end;

procedure TProgEdit.WatchButtonClick(Sender: TObject);
begin
  FmainTool.ReadWatches;
end;

procedure TProgEdit.ResetButtonClick(Sender: TObject);
begin
 FMaintool.Resetprog;
end;

end.
