unit T_dgnexp;

interface
uses
   Classes,
   g_Elems,
   MSElems,
   Elements;

const
dgnerr_TemplateNotFound = 1;
dgnerr_WrongFileName    = 2;

type

TExpBuff = Array[0..63000] of byte;

TMSTextElement = class(TOwnerElement)
public
   Text: string;
   LengthT: double;
   Origin: TDoubleCoord;
end;

TDGNExport = class(TPersistent)
private
   FFileName: string;
   FTemplateFile: string;
   FStream: TStream;
   FError: longInt;
   FBuf: TExpBuff;
   FBufSize: word;
   FCT: TColor_Table_type5;
   FCTPosition: longInt;
   FLastColorIndex: byte;
   FComplex: boolean;
   NodeNumber: word;
protected
   function  GetColor(aIndex: byte): longInt;
   function  GetColorIndex(aValue: longInt): byte;
   procedure SaveColorTable;
   procedure CopyTemplate;
public
   function Open: boolean;
   procedure CLose;
   procedure SetBackgroundColor(aValue: longInt);
   procedure WriteText(font: byte; Color: longInt; x, y, Height, LengthT, Rotation: double; Text: string);
   procedure WriteTextNode(font: byte; Color: longInt; x, y, Height, LengthT, LineSp, Rotation: double; Texts: TEntity);
   function  GetMultiText(aText: string): TEntity;
   property  Complex: boolean read FComplex write FComplex;
   property  Stream: TStream read FStream write FStream;
   property  Error: longInt read FError write FError;
published
   property FileName: string read FFileName write FFileName;
   property TemplateFile: string read FTemplateFile write FTemplateFile;
end;


implementation

uses
   {!t_Main,}
   Dialogs,
   SysUtils,
   WinProcs,
   MathLib,
   u_String;

{ TDGNExport }

function  TDGNExport.GetColor(aIndex: byte): longInt;
begin
   with FCT.Color_Info[aIndex] do result := RGB(R,G,B);
end;

function  TDGNExport.GetColorIndex(aValue: longInt): byte;
var
   i: byte;
begin
   if FLastColorIndex > 0 then
   for i := 1 to FLastColorIndex do
   begin
      if GetColor(i) = aValue then
      begin
         result := i - 1;
         exit;
      end;
   end;
   if FLastColorIndex < 255 then Inc(FLastColorIndex);
   FCT.Color_Info[FLastColorIndex].R := GetRValue(aValue);
   FCT.Color_Info[FLastColorIndex].G := GetGValue(aValue);
   FCT.Color_Info[FLastColorIndex].B := GetBValue(aValue);
   result := FLastColorIndex - 1;
end;

procedure TDGNExport.SaveColorTable;
var
   EndP: longInt;
begin
   EndP := Stream.Position;
   Stream.Seek(FCTPosition, 0);
   Stream.Write(FCT, SizeOf(FCT));
   Stream.Seek(EndP, 0);
end;

procedure TDGNExport.SetBackgroundColor(aValue: longInt);
begin
   FCT.Color_Info[0].R := GetRValue(aValue);
   FCT.Color_Info[0].G := GetGValue(aValue);
   FCT.Color_Info[0].B := GetBValue(aValue);
end;




procedure TDGNExport.CopyTemplate;
var
   TemplateSize: longInt;
   W: word;
   EndP: longInt;
begin
   FCTPosition := 0;
   Stream := TFileStream.Create(TemplateFile, fmOpenRead{Write !!!!!!!});
   TemplateSize := Stream.Read(FBuf[0], SizeOf(FBuf));
   Stream.Free;
   Stream := TFileStream.Create(FileName, fmCreate{Write !!!!!!!});
   Stream.Write(FBuf[0], TemplateSize);
   Stream.Seek(0,0);
   Stream.read(W,2);
   while W <> $FFFF do
   begin
      Stream.read(W,2);
      Stream.Seek(Stream.Position+W shl 1,0);
      Stream.read(W,2);
      if ((W and mskType) = GROUP_DATA_ELM) and ((W and mskLevel) = 1) then
         FCTPosition := Stream.Position-2+SizeOf(Elm_hdr);
   end;
   EndP := Stream.Position - 2;
   if FCTPosition > 0 then
   begin
      Stream.Seek(FCTPosition, 0);
      Stream.Read(FCT, SizeOf(FCT));
      FLastColorIndex := 0;
   end;
   Stream.Seek(EndP, 0);
end;

function TDGNExport.Open: boolean;
begin
   result := true;
   Error := 0;
   NodeNumber := 0;
   if TemplateFile = '' then
   begin
      FTemplateFile := ExtractFilePath(ParamStr(0));
      FTemplateFile := CheckBackSlash(FTemplateFile)+'TEMPLATE.DGN';
   end;
   if not FileExists(TemplateFile) then
   begin
      ShowMessage('Pedlohov soubor neexistuje !');
      Error := dgnerr_TemplateNotFound;
      result := false;
      exit;
   end;
   if FileName = '' then
   begin
      ShowMessage('Neplatn nzev souboru pro export !');
      Error := dgnerr_WrongFileName;
      result := false;
      exit;
   end;
   CopyTemplate;
end;

procedure TDGNExport.CLose;
var
   EOD: word;
begin
   SaveColorTable;
   EOD := $FFFF;
   Stream.Write(EOD,2);
   Stream.Free;
   FStream := nil;
end;

function SwapLong( L: longInt ): LongInt;
type
   T2W = record
      L,H: word;
   end;
begin
   T2W(result).H := T2W(L).L;
   T2W(result).L := T2W(L).H;
end;

function GetMinTextRect(x,y,Height,LengthT,Angle: double): TDoubleRect;
var
   A,B,C,D: TDoubleCoord;
begin
   { kosodlnk }
   if LengthT = 0 then LengthT := Height;
   A.x := x;
   A.y := y;
   B.x := A.x + LengthT;
   B.y := y;
   C.x := B.x;
   C.y := B.y + Height;
   D.x := A.x;
   D.y := C.y;
   B := GetRotatedCrd(A, B, Angle);
   C := GetRotatedCrd(A, C, Angle);
   D := GetRotatedCrd(A, D, Angle);
   { obdlnk }
   with result do
   begin
      Min.x := A.x;
      if B.x < Min.x then Min.x := B.x;
      if C.x < Min.x then Min.x := C.x;
      if D.x < Min.x then Min.x := D.x;
      Min.y := A.y;
      if B.y < Min.y then Min.y := B.y;
      if C.y < Min.y then Min.y := C.y;
      if D.y < Min.y then Min.y := D.y;
      Max.x := A.x;
      if B.x > Max.x then Max.x := B.x;
      if C.x > Max.x then Max.x := C.x;
      if D.x > Max.x then Max.x := D.x;
      Max.y := A.y;
      if B.y > Max.y then Max.y := B.y;
      if C.y > Max.y then Max.y := C.y;
      if D.y > Max.y then Max.y := D.y;
   end;

end;

procedure TDGNExport.WriteTextNode(font: byte; Color: longInt; x, y, Height, LengthT,LineSp, Rotation: double; Texts: TEntity);
var
   EHDR: Elm_hdr;
   Text_node_2d: Ttext_node_2d;
   Text_2d: TText_2D;
   TextL: byte;
   TextW: word;
   Orig: double;
   MinRect: TDoubleRect;
   i: integer;
   E: TMSTextElement;
   MaxLengthT: double;
   MaxChars: byte;
   TotalWords: word;
   A,LastOrigin: TDoubleCoord;
begin
   if Texts = nil then exit;
   A.x := x;
   A.y := y;
   LastOrigin := A;
   MaxLengthT := 0;
   MaxChars := 0;
   TotalWords := 34 - 17;
   for i := Texts.Count-1 Downto 0 do
   begin
      E := TMSTextElement(Texts.Elements[i]);
      if E.LengthT > MaxLengthT then MaxLengthT := E.LengthT;
      TextL := Length(E.Text);
      TextW := (TextL + 1) div 2;
      TotalWords  := TotalWords+2+28+TextW + 1;      { words to follow }
      if TextL > MaxChars then MaxChars := TextL;
      E.Origin := A;
      //LastOrigin := A;
      A.Y := A.Y + Height + LineSp;
      A := GetRotatedCrd(E.Origin, A, Rotation);
   end;

   MinRect := GetMinTextRect(LastOrigin.x,LastOrigin.y,(Texts.Count*Height+LineSp)-LineSp,MaxLengthT,Rotation);
   FillChar(Ehdr, Sizeof(EHDR),0);
   FillChar(Text_node_2d, Sizeof(Text_node_2d),0);
   { Nastaven Headru }
   EHDR.Header := 1 or TEXT_NODE_ELM;     { Level , type }
   EHDR.Words  := 34;      { words to follow }
   orig := 2147483648.0;
   EHDR.xlow   := SwapLong(round(orig + MinRect.Minx* 1000));
   EHDR.ylow   := SwapLong(round(orig + MinRect.Miny* 1000));
   EHDR.xHigh  := SwapLong(round(orig + MinRect.Maxx* 1000));
   EHDR.yHigh  := SwapLong(round(orig + MinRect.Maxy* 1000));

   Stream.write(EHDR, SizeOf(EHDR));

   Text_node_2d.DHDR.AttIndx := 20; { words to Attribute linkage }
   Text_node_2d.DHDR.Props.Bits := [N];   { New element }
   Text_node_2d.DHDR.Symb := GetColorIndex(Color) shl 8;     { Color .... 2 for test !!!!!! }
   Text_node_2d.TotWords := TotalWords;
   Text_node_2d.NumStrngs := Texts.Count;
   Text_node_2d.NodeNumber := NodeNumber;
   Inc(NodeNumber);
   Text_node_2d.MaxLngth := MaxChars;
   Text_node_2d.MaxUsed := MaxChars;
   Text_node_2d.font := font;              { text font used }
   Text_node_2d.just :=  2;	     { justification type 2 ~ left-bottom }
   Text_node_2d.linespc:= round(LineSp * 1000); {!!!!!!!!!}{ line spacing }
   Text_node_2d.linespc:= SwapLong(Text_node_2d.linespc);  { line spacing }
   Text_node_2d.lngthmult:= round((Height * 1000000)/6 );  { length multiplier }
   Text_node_2d.lngthmult := SwapLong(Text_node_2d.lngthmult);
   Text_node_2d.hghtmult:= Text_node_2d.lngthmult;  { height multiplier }
   Text_node_2d.rotation:= round(DejStupne( Rotation)*360000);  { rotation angle }
   Text_node_2d.rotation := SwapLong(Text_node_2d.rotation);
   Text_node_2d.origin.x := round(LastOrigin.x* 1000);  { origin }
   Text_node_2d.origin.x := SwapLong(Text_node_2d.origin.x);
   Text_node_2d.origin.y := round(LastOrigin.y* 1000);  { origin }
   Text_node_2d.origin.y := SwapLong(Text_node_2d.origin.y);

   Stream.write(Text_node_2d, SizeOF(Text_node_2d));
   TextW := 0;
   Stream.write(TextW,2); {attindex}

   Complex := true;
   for i := 0 to Texts.Count - 1 do
   begin
      E := TMSTextElement(Texts.Elements[i]);
      WriteText(font,Color,E.Origin.x,E.Origin.y,Height,E.LengthT,Rotation,E.Text);
   end;
   Complex := false;
end;

function  TDGNExport.GetMultiText(aText: string): TEntity;
var
  Konec: boolean;
  PosCR: integer;
  WText: string;
  E: TMSTextElement;
begin
   result := nil;
   Konec:= false;
   repeat
      PosCR:= pos(#13+#10, aText);
      if PosCR = 0 then
      begin
         Konec:= true;
         WText:= aText;
         if result <> nil then
         begin
            E := TMSTextElement.Create;
            E.Text := WText;
            result.AddElement(E);
         end;
      end
      else
      begin
         if result = nil then result := TEntity.Create;
         WText:= copy(aText, 1, PosCR-1);
         aText:= copy(aText, PosCR+2, 255);
         E := TMSTextElement.Create;
         E.Text := WText;
         result.AddElement(E);
      end;
   until Konec
end;



procedure TDGNExport.WriteText(font: byte; Color: longInt; x, y, Height, LengthT, Rotation: double; Text: string);
var
   EHDR: Elm_hdr;
   Text_2d: TText_2D;
   TextL: byte;
   TextW: word;
   Orig: double;
   MinRect: TDoubleRect;
begin
   MinRect := GetMinTextRect(x,y,Height,LengthT,Rotation);
   FillChar(Ehdr, Sizeof(EHDR),0);
   FillChar(Text_2d, Sizeof(Text_2d),0);
   { Nastaven Headru }
   TextL := Length(Text);
   TextW := (TextL + 1) div 2;
   EHDR.Header := 1 or TEXT_ELM;     { Level , type }
   if Complex then EHDR.Header := EHDR.Header or mskComplex;
   EHDR.Words  := 28+TextW + 1;      { words to follow }
   orig := 2147483648.0;
   EHDR.xlow   := SwapLong(round(orig + MinRect.Minx* 1000));
   EHDR.ylow   := SwapLong(round(orig + MinRect.Miny* 1000));
   EHDR.xHigh  := SwapLong(round(orig + MinRect.Maxx* 1000));
   EHDR.yHigh  := SwapLong(round(orig + MinRect.Maxy* 1000));

   Stream.write(EHDR, SizeOf(EHDR));

   Text_2d.DHDR.AttIndx := 14+TextW; { words to Attribute linkage }
   Text_2d.DHDR.Props.Bits := [N];   { New element }
   Text_2d.DHDR.Symb := GetColorIndex(Color) shl 8;     { Color .... 2 for test !!!!!! }
   Text_2d.font := font;              (* text font used *)
   Text_2d.just :=  2;	     (* justification type 2 ~ left-bottom *)
   Text_2d.lngthmult:= round((Height * 1000000)/6 );  (* length multiplier *)
   Text_2d.lngthmult := SwapLong(Text_2d.lngthmult);
   Text_2d.hghtmult:= Text_2d.lngthmult;  (* height multiplier *)
   Text_2d.rotation:= round(DejStupne( Rotation)*360000);  (* rotation angle *)
   Text_2d.rotation := SwapLong(Text_2d.rotation);
   Text_2d.origin.x := round(x* 1000);  (* origin *)
   Text_2d.origin.x := SwapLong(Text_2d.origin.x);
   Text_2d.origin.y := round(y* 1000);  (* origin *)
   Text_2d.origin.y := SwapLong(Text_2d.origin.y);
   Text_2d.numchars := TextL;	  (* # of characters *)
   Text_2d.edflds   := 0;	  (* # of enter data fields *)

   Stream.write(Text_2d, SizeOF(Text_2d) -1);
   if TextL > 0 then Stream.write(Text[1], TextL);
   TextW := 0;
   if (TextL and 1) > 0 then Stream.write(TextW,1);
   Stream.write(TextW,2);
end;



end.
