unit T_GNed;


INTERFACE

uses
   Controls,
   Graphics,
   Wintypes,
   Elements,
   e_Desktp,
   g_Elems,
   m_InterA,
   t_Editor,
   T_InterA,
   t_Recurs;

type

TMapSheetEditor = class(TRectEntityEditor)
protected
   function  GetName: string; override;
   procedure SetSelectedElement(aValue: TRectEntity); override;
public
   procedure SetupDrawRectangle; virtual;
   procedure SetupSelect; override;
   procedure SetupText; virtual;
   procedure SetupView; virtual;
   procedure SetupDesktop; virtual;
   procedure on_tm_SelectToTop(var Handled: boolean); virtual;
   procedure on_tm_SelectToBottom(var Handled: boolean); virtual;
   procedure on_tm_SelectUp(var Handled: boolean); virtual;
   procedure on_tm_SelectDown(var Handled: boolean); virtual;
   procedure HandleMessage(var aMessage: TOrbisMessage; var Handled: boolean); override;
end;


TEditorIATool = class(TDrawRectTool)
private
   FEditor:  TRectEntityEditor;
   function  GetDesktop: TDesktop;
   function  GetActualColor: TColor;
   function  GetActualBrushColor: TColor;
   function  GetActualPenWidth: double;
   function  GetActualBrushStyle: TBrushStyle;
   function  GetActualFont: TFont;
   procedure SetActualFont(aValue: TFont);
   function  GetElementClass: TElementClass; virtual; abstract;
public
   function ItsMyClass(aClass: TElementClass): boolean;
   property ElementClass: TElementClass read GetElementClass;
   property Editor: TRectEntityEditor read FEditor write FEditor;
   property Desktop: TDesktop read GetDesktop;
   property ActualColor: TColor read GetActualColor;
   property ActualBrushColor: TColor read GetActualColor;
   property ActualFont: TFont read GetActualFont write SetActualFont;
   property ActualPenWidth: double read GetActualPenWidth;
   property ActualBrushStyle: TBrushStyle read GetActualBrushStyle;
end;




TNewDrawRect = class(TEditorIATool)
protected
   function  GetCursor: TCursor; override;
   function  GetName: string; override;
public
   procedure On_Rect(aRect: TDoubleRect); override;
end;



TNewText = class(TEditorIATool)
protected
   function  GetCursor: TCursor; override;
   function  GetName: string; override;
public
   procedure On_Rect(aRect: TDoubleRect); override;
end;


TNewTextBlock = class(TEditorIATool)
protected
   function  GetCursor: TCursor; override;
   function  GetName: string; override;
public
   procedure On_Rect(aRect: TDoubleRect); override;
end;


TNewView = class(TEditorIATool)
protected
   function  GetCursor: TCursor; override;
   function  GetName: string; override;
public
   procedure On_Rect(aRect: TDoubleRect); override;
end;


TDrawRectEditor = class(TEditorIATool)
protected
   function  GetCursor: TCursor; override;
   function  GetName: string; override;
public
   procedure On_Rect(aRect: TDoubleRect); override;
end;



TTextEditor = class(TEditorIATool)
protected
   function  GetCursor: TCursor; override;
   function  GetName: string; override;
public
   procedure On_Rect(aRect: TDoubleRect); override;
end;



TViewEditor = class(TEditorIATool)
protected
   function  GetCursor: TCursor; override;
   function  GetName: string; override;
public
   procedure On_Rect(aRect: TDoubleRect); override;
end;



IMPLEMENTATION

uses
   Classes,
   Dialogs,
   forms,
   e_Device,
   e_View,
   e_TxtBlk,
   f_EdText,
   f_EdView,
   m_Orbis,
   m_Proj,
   t_GNEdit,
   u_Cursor,
   F_RectEd;


{ TMapSheetEditor }

function  TMapSheetEditor.GetName: string;
begin
    result := 'Editor ablony mapovho listu';
end;


procedure TMapSheetEditor.SetSelectedElement(aValue: TRectEntity);
begin
   inherited SetSelectedElement(aValue);
   PostMsgToOwners(tm_SetPropertyEditors, nil, DontKill);
end;


procedure TMapSheetEditor.SetupSelect;
var
   R: TRectEntity;
begin
   R := SelectedElement;
   if R is TText then SetupText else
   if R is TView then SetupView else
   if R is TDeskTop then SetupDeskTop else
   if R is TDrawRectangle then SetupDrawRectangle
end;


procedure TMapSheetEditor.SetupDrawRectangle;
var
   ER: TEdRectangle;
begin
   ER:= TEdRectangle.Create;
   ER.Input:= SelectedElement;
   ER.DoIt;
   ER.Free
end;


procedure TMapSheetEditor.SetupText;
var
   R: TText;
   DD: TEditTextDialog;
   i: longInt;
begin
   R := TText(SelectedElement);
   if R is TText then
   begin
      Application.CreateForm(TEditTextDialog, DD);
      DD.TextElement := TText(R.CreateEnvelope);
      if DD.ShowModal = mrOK then
      begin
         R.Text   := DD.TextElement.Text;
         R.FontName   := DD.TextElement.FontName;
         R.Color  := DD.TextElement.Color;
         R.Height := DD.TextElement.Height;
         R.Orientation := DD.TextElement.Orientation;
         { ... }
      end;
      {
      ActualFont := DD.FontDialog.Font;
      }
      DD.TextElement.Free;
      DD.Free;
      for i := 0 to 30 do Application.ProcessMessages;
      PostMsgToOwners(tm_ReView, nil, DontKill);
   end;
end;

procedure TMapSheetEditor.SetupView;
var
   V: TView;
   VD: TSetupViewDialog;
   ER: TEdRectangle;
   VE: TViewDialog;
begin
   VE:= TViewDialog.Create;
   VE.INput:= SelectedElement;
   VE.DoIt;
   VE.Free;
{
   ER:= TEdRectangle.Create;
   ER.Input:= V;
   ER.DoIt;
   ER.Free
}
{
   Application.CreateForm(TSetupViewDialog, VD);
   if VD.ShowModal = mrOK then
   begin
   end;
   VD.Free;
}
end;


procedure TMapSheetEditor.SetupDesktop;
begin
end;


procedure TMapSheetEditor.on_tm_SelectToTop(var Handled: boolean);
var
   MS: TEntity;
   E: TRectEntity;
begin
   MS := Tentity(Input);
   if MS = nil then exit;
   E := SelectedElement;
   if E = nil then exit;
   MS.RemoveElement(E);
   MS.AddElement(E);
   SelectedElement := E;
   PostMsgToOwners(tm_ReView, nil, DontKill);
end;

procedure TMapSheetEditor.on_tm_SelectToBottom(var Handled: boolean);
var
   MS: TEntity;
   E: TRectEntity;
begin
   MS := Tentity(Input);
   if MS = nil then exit;
   E := SelectedElement;
   if E = nil then exit;
   MS.RemoveElement(E);
   MS.InsertElement(0, E);
   SelectedElement := E;
   PostMsgToOwners(tm_ReView, nil, DontKill);
end;

procedure TMapSheetEditor.on_tm_SelectUp(var Handled: boolean);
var
   MS: TEntity;
   E: TRectEntity;
   i: longInt;
begin
   MS := Tentity(Input);
   if MS = nil then exit;
   E := SelectedElement;
   if E = nil then exit;
   i := MS.IndexOf(E);
   if i = MS.Count -1 then exit;
   MS.RemoveElement(E);
   MS.InsertElement(i+1, E);
   SelectedElement := E;
   PostMsgToOwners(tm_ReView, nil, DontKill);
end;

procedure TMapSheetEditor.on_tm_SelectDown(var Handled: boolean);
var
   MS: TEntity;
   E: TRectEntity;
   i: longInt;
begin
   MS := Tentity(Input);
   if MS = nil then exit;
   E := SelectedElement;
   if E = nil then exit;
   i := MS.IndexOf(E);
   if i = 0 then exit;
   MS.RemoveElement(E);
   MS.InsertElement(i-1, E);
   SelectedElement := E;
   PostMsgToOwners(tm_ReView, nil, DontKill);
end;

procedure TMapSheetEditor.HandleMessage(var aMessage: TOrbisMessage; var Handled: boolean);
begin
   inherited HandleMessage(aMessage, Handled);
   if Handled then exit;
   case aMessage.Message of
      tm_SelectToTop     : on_tm_SelectToTop(Handled);
      tm_SelectToBottom  : on_tm_SelectToBottom(Handled);
      tm_SelectUp        : on_tm_SelectUp(Handled);
      tm_SelectDown      : on_tm_SelectDown(Handled);
   end;
end;



{ TEditorIATool }

function TEditorIATool.GetDesktop: TDesktop;
begin
   result := TDesktop(Input);
end;

function TEditorIATool.GetActualColor: TColor;
var
   aTool: TMSEditorFormTool;
   MSEForm: TMSEditorForm;
begin
   result := clBlack;
   aTool := TMSEditorFormTool(FindOwnerClass(TMSEditorFormTool));
   if aTool = nil then exit;
   MSEForm := aTool.GForm;
   if MSEForm = nil then exit;
   result := MSEForm.ColorPanel.Color;
end;

function TEditorIATool.GetActualBrushColor: TColor;
var
   aTool: TMSEditorFormTool;
   MSEForm: TMSEditorForm;
begin
   result := clBlack;
   aTool := TMSEditorFormTool(FindOwnerClass(TMSEditorFormTool));
   if aTool = nil then exit;
   MSEForm := aTool.GForm;
   if MSEForm = nil then exit;
   result := MSEForm.BrushColorPanel.Color;
end;

function  TEditorIATool.GetActualPenWidth: double;
var
   aTool: TMSEditorFormTool;
   MSEForm: TMSEditorForm;
begin
   result := clBlack;
   aTool := TMSEditorFormTool(FindOwnerClass(TMSEditorFormTool));
   if aTool = nil then exit;
   MSEForm := aTool.GForm;
   if MSEForm = nil then exit;
   result := MSEForm.PenWidth;
end;

function  TEditorIATool.GetActualBrushStyle: TBrushStyle;
var
   aTool: TMSEditorFormTool;
   MSEForm: TMSEditorForm;
begin
   result := bsClear;
   aTool := TMSEditorFormTool(FindOwnerClass(TMSEditorFormTool));
   if aTool = nil then exit;
   MSEForm := aTool.GForm;
   if MSEForm = nil then exit;
   if MSEForm.b_Fill.Down then result := bsSolid
                          else result := bsClear;
end;

function TEditorIATool.GetActualFont: TFont;
var
   aTool: TMSEditorFormTool;
   MSEForm: TMSEditorForm;
begin
   aTool := TMSEditorFormTool(FindOwnerClass(TMSEditorFormTool));
   if aTool = nil then exit;
   MSEForm := aTool.GForm;
   if MSEForm = nil then exit;
   result := MSEForm.FontDialog.Font;
end;


procedure TEditorIATool.SetActualFont(aValue: TFont);
var
   aTool: TMSEditorFormTool;
   MSEForm: TMSEditorForm;
begin
   aTool := TMSEditorFormTool(FindOwnerClass(TMSEditorFormTool));
   if aTool = nil then exit;
   MSEForm := aTool.GForm;
   if MSEForm = nil then exit;
   MSEForm.FontDialog.Font.Assign(aValue);
   MSEForm.FontPanel.Text := aValue.Name;
end;



function TEditorIATool.ItsMyClass(aClass: TElementClass): boolean;
begin
   result := aClass = ElementClass;
end;



{ TNewDrawRect }

function  TNewDrawRect.GetName: string;
begin
   result := 'Nov rmeek';
end;


function  TNewDrawRect.GetCursor: TCursor;
begin
   result := cr_NewDrawRect;
end;



procedure TNewDrawRect.On_Rect(aRect: TDoubleRect);
var
   DR: TDrawRectangle;
begin
   if Desktop = nil then exit;
   DR := TDrawRectangle.Create;
   DR.Minx := aRect.Minx;
   DR.Miny := aRect.Miny;
   DR.Maxx := aRect.Maxx;
   DR.Maxy := aRect.Maxy;

   DR.PenColor := ActualColor;
   DR.PenWidth := ActualPenWidth;
   DR.BrushColor := GetActualBrushColor;
   DR.BrushStyle := ActualBrushStyle;

   Desktop.AddElement(DR);
   Editor.SelectedElement := DR;
   PostMsgToOwners(tm_ReView, nil, DontKill);
end;



{ TNewText }

function  TNewText.GetName: string;
begin
   result := 'Nov Text';
end;


function  TNewText.GetCursor: TCursor;
begin
   result := cr_NewText;
end;



procedure TNewText.On_Rect(aRect: TDoubleRect);
var
   DR: TText;
   DD: TEditTextDialog;
   i: longInt;
begin
   if Desktop = nil then exit;
   DR := TText.Create;
   DR.Minx := aRect.Minx;
   DR.Miny := aRect.Miny;
   DR.Maxx := aRect.Maxx;
   DR.Maxy := aRect.Maxy;
   DR.Height := aRect.Maxy - aRect.Miny;
   DR.Font:= GetActualFont;
   DR.Color := GetActualColor;
   DR.Text := 'Text';
   Desktop.AddElement(DR);
   Editor.SelectedElement := DR;
   PostMsgToOwners(tm_ReView, nil, DontKill);
   for i := 0 to 30 do Application.ProcessMessages;
   Application.CreateForm(TEditTextDialog, DD);
   DD.TextElement := TText(DR.CreateEnvelope);
   if DD.ShowModal = mrOK then
   begin
      DR.Text := DD.TextElement.Text;
      DR.FontName:= DD.TextElement.FontName;
      DR.Color   :=(DD.TextElement.Color);
      DR.Height := DD.TextElement.Height;
      DR.Orientation := DD.TextElement.Orientation;
      { ... }
   end;
   ActualFont := DD.FontDialog.Font;
   DD.TextElement.Free;
   DD.Free;
   PostMsgToOwners(tm_ReView, nil, DontKill);
end;




{ TNewTextBlock }

function  TNewTextBlock.GetName: string;
begin
   result := 'Nov Text Blok';
end;


function  TNewTextBlock.GetCursor: TCursor;
begin
   result := cr_NewTxtBlk;
end;



procedure TNewTextBlock.On_Rect(aRect: TDoubleRect);
var
   DR: TTextBlock;
   i: longInt;
begin
   if Desktop = nil then exit;
   DR := TTextBlock.Create;
   DR.Minx := aRect.Minx;
   DR.Miny := aRect.Miny;
   DR.Maxx := aRect.Maxx;
   DR.Maxy := aRect.Maxy;
   DR.Height := aRect.Maxy - aRect.Miny;
   DR.Font:= GetActualFont;
   DR.Color := GetActualColor;
   Desktop.AddElement(DR);
   Editor.SelectedElement := DR;
   PostMsgToOwners(tm_ReView, nil, DontKill);
end;




{ TNewView }


function  TNewView.GetName: string;
begin
   result := 'Nov View';
end;


function  TNewView.GetCursor: TCursor;
begin
   result := cr_NewView;
end;



procedure TNewView.On_Rect(aRect: TDoubleRect);
var
   DR: TView;
begin
   if Desktop = nil then exit;
   DR := TView.Create;
   DR.Minx := aRect.Minx;
   DR.Miny := aRect.Miny;
   DR.Maxx := aRect.Maxx;
   DR.Maxy := aRect.Maxy;

   Desktop.AddElement(DR);
   Editor.SelectedElement := DR;
   PostMsgToOwners(tm_ReView, nil, DontKill);
end;


{ TDrawRectEditor }

function  TDrawRectEditor.GetName: string;
begin
   result := 'Editor Rmeku';
end;


function  TDrawRectEditor.GetCursor: TCursor;
begin
   result := cr_NewDrawRect;
end;



procedure TDrawRectEditor.On_Rect(aRect: TDoubleRect);
var
   DR: TDrawRectangle;
begin
   if Desktop = nil then exit;
   DR := TDrawRectangle.Create;
   DR.Minx := aRect.Minx;
   DR.Miny := aRect.Miny;
   DR.Maxx := aRect.Maxx;
   DR.Maxy := aRect.Maxy;

   DR.PenColor := GetActualColor;
   DR.BrushStyle := ActualBrushStyle;
   DR.PenWidth := ActualPenWidth;
   Desktop.AddElement(DR);
   PostMsgToOwners(tm_ReView, nil, DontKill);
end;



{ TTextEditor }

function  TTextEditor.GetName: string;
begin
   result := 'Editor Textu';
end;


function  TTextEditor.GetCursor: TCursor;
begin
   result := cr_NewText;
end;



procedure TTextEditor.On_Rect(aRect: TDoubleRect);
var
   DR: TText;
   DD: TEditTextDialog;
   i: longInt;
begin
   if Desktop = nil then exit;
   DR := TText.Create;
   DR.Minx := aRect.Minx;
   DR.Miny := aRect.Miny;
   DR.Maxx := aRect.Maxx;
   DR.Maxy := aRect.Maxy;
   DR.Height := aRect.Maxy - aRect.Miny;
   DR.Font:= GetActualFont;
   DR.Color := GetActualColor;
   DR.Text := 'Text';
   Desktop.AddElement(DR);
   PostMsgToOwners(tm_ReView, nil, DontKill);
   for i := 0 to 30 do Application.ProcessMessages;
   Application.CreateForm(TEditTextDialog, DD);
   DD.TextElement := TText(DR.CreateEnvelope);
   if DD.ShowModal = mrOK then
   begin
      DR.Text := DD.TextElement.Text;
      DR.FontName:= DD.TextElement.FontName;
      DR.Color:= (DD.TextElement.Color);
      DR.Height := DD.TextElement.Height;
      DR.Orientation := DD.TextElement.Orientation;
      { ... }
   end;
   ActualFont := DD.FontDialog.Font;
   DD.TextElement.Free;
   DD.Free;
   PostMsgToOwners(tm_ReView, nil, DontKill);
end;




{ TViewEditor }


function  TViewEditor.GetName: string;
begin
   result := 'Editor View';
end;


function  TViewEditor.GetCursor: TCursor;
begin
   result := cr_NewView;
end;



procedure TViewEditor.On_Rect(aRect: TDoubleRect);
var
   DR: TView;
begin
   if Desktop = nil then exit;
   DR := TView.Create;
   DR.Minx := aRect.Minx;
   DR.Miny := aRect.Miny;
   DR.Maxx := aRect.Maxx;
   DR.Maxy := aRect.Maxy;

   Desktop.AddElement(DR);
   PostMsgToOwners(tm_ReView, nil, DontKill);
end;

END.
