unit e_GNRAS;

interface
uses
  Graphics,
  Elements, G_Elems;

type

TGNRasters=class(TRectEntity)
private
  procedure CreateLayers(aLayers: string);
  function GetLayer(aLayer: char): TRectEntity;
  procedure AddRaster(aLayer: char; aFileName: string; aColor: longInt);
public
  procedure AddRasters(aLayers: string; aPath: string;
                       LColor, PColor, RColor, TColor, VColor: longInt);
end;

TGNLayer=class(TRectEntity)
private
   FSymbol: char;
   FDescription: string;
   FColor: TColor;
published
   property Symbol: char read FSymbol write FSymbol;
   property Description: string read FDescription write FDescription;
   property Color: TColor read FColor write FColor;
end;


implementation

uses
  Dialogs,
  SysUtils,
  e_Raster,
  U_String;
  //E_Raster;



function IsInLayers(aName, aLayers: string): boolean;
var
  I: integer;
begin
  result:= false;
  aName:= UpperCase(aName);
  aLayers:= UpperCase(aLayers);
  for I:= 1 to length(aLayers) do
  begin
    if aName[2] = aLayers[I] then
    begin
      result:= true;
      exit
    end
  end
end;



procedure TGNRasters.CreateLayers(aLayers: string);
var
  I: integer;
  LayChar: char;
  aLayRE: TRectEntity;
begin
  for I:= 1 to length(aLayers) do
  begin
    LayChar:= upCase(aLayers[I]);
    aLayRE:= TRectEntity.Create;
    aLayRE.Name:= 'VRSTVA ' + LayChar;
    AddElement(aLayRE)
  end
end;


function TGNRasters.GetLayer(aLayer: char): TRectEntity;
begin
  result:= TRectEntity(FindElementName('VRSTVA '+upCase(aLayer)));
end;



procedure TGNRasters.AddRaster(aLayer: char; aFileName: string; aColor: longInt);
var
  aRas: TRaster;
  aLayerRE: TRectEntity;
begin
  aLayerRE:= GetLayer(aLayer);
  if aLayerRE = nil then exit;

  aRas:= TRaster.Create;
  aRas.FileName:= aFileName;
  aRas.Color:= aColor;
  aLayerRE.AddAndUpdateRect(aRas);
  UpdateRect(aLayerRE)
end;


procedure TGNRasters.AddRasters(aLayers: string; aPath: string;
                                LColor, PColor, RColor, TColor, VColor: longInt);
var
  SearchRec: TSearchRec;
  RasName: string;
  aColor: longInt;
  Result: integer;
begin
  if aPath <> '' then
     aPath:= AddBackSlash(aPath);
  FreeAll;
  CreateLayers(aLayers);

  Result := FindFirst(aPath + '*.RAS', faAnyFile, SearchRec);
  while Result = 0 do
  begin
    if (ExtractFileExt(SearchRec.Name) = '.RAS') then
    begin
      RasName:= ExtractFileName(SearchRec.Name);
      try
        if IsInLayers(RasName, aLayers) then
        begin
          case upCase(RasName[2]) of
          'L': aColor:= LColor;
          'R': aColor:= RColor;
          'P': aColor:= PColor;
          'V': aColor:= VColor;
          'T': aColor:= TColor;
          end;

          AddRaster(upCase(RasName[2]), aPath + SearchRec.Name, aColor);
        end
      except
         ShowMessage('Nelze nast rastr "' + RasName + '"');
      end;
    end;
    Result := FindNext(SearchRec);
  end;
  FindClose(SearchRec);
end;



end.
