unit E_ProMem;

interface
uses
  classes, WinProcs,
  Elements, _IniFile;

type

TProjectionMemory = class(TElement)
private
  FStringList: TStringList;
  FActual: integer;
  FCenterX, FCenterY, FScale: double;
  FMaxCount: integer;

  procedure SetActValues;
  procedure RenameList;
public
  constructor Create;
  destructor  Destroy;
  procedure SetPrevProjection;
  procedure SetNextProjection;
  procedure ReadFromIni(aName: string);
  procedure SaveToIni(aName: string);
  function GetCenterX(aDef: double): double;
  function GetCenterY(aDef: double): double;
  function GetScale(aDef: double): double;
  procedure AddProjection(aCenterX, aCenterY, aScale: double);
  property MaxCount: integer read FMaxCount write FMaxCount;
end;


implementation
uses
  Sysutils,
  U_String;



function ParName(aInt: Integer): string;
begin
  result:= 'PROJ' + IntToStrZ(aInt, 2, true)
end;


constructor TProjectionMemory.Create;
begin
  inherited Create;
  FStringList:= TStringList.Create;
  FMaxCount:= 10;
end;

destructor  TProjectionMemory.Destroy;
begin
  FStringList.Free;
  inherited destroy
end;


procedure TProjectionMemory.SetActValues;
var
  PStr: string;
  aDel: char;
  aPos: longInt;
begin
  FScale:= 0;
  FCenterX:= 0;
  FCenterY:= 0;
  try
    if (FActual < 0) or (FActual >= FStringList.Count) then exit;
    PStr:= FStringList.Values['PROJ' + IntToStrZ(FActual, 2, true)];
    if PStr = '' then exit;
    FCenterX:= StrToInt(ReadToDelimiterWithoutWS(PStr, [','], 1, aDel, aPos));
    FCenterY:= StrToInt(ReadToDelimiterWithoutWS(PStr, [','], aPos, aDel, aPos));
    FScale  := StrToInt(ReadToDelimiterWithoutWS(PStr, [','], aPos, aDel, aPos));
  except
  end
end;


procedure TProjectionMemory.RenameList;
var
  I: integer;
  PStr: string;
  aName, aVal: string;
begin
  for I:= 0 to FStringList.Count-1 do
  begin
    StringAsParameter(FStringList.Strings[I], aName, aVal);
    FStringList.Strings[I]:= ParName(I) + '=' + aVal;
  end
end;


procedure TProjectionMemory.SetPrevProjection;
begin
  if FActual > 0 then
     dec(FActual)
  else
    MessageBeep(0);

  if FActual >= 0 then
     SetActValues
end;


procedure TProjectionMemory.SetNextProjection;
begin
  if FActual < FStringList.Count-1 then
     inc(FActual)
  else
    MessageBeep(0);

  if FActual <= FStringList.Count-1 then
     SetActValues
end;


procedure TProjectionMemory.ReadFromIni(aName: string);
var
  IniF: TRealIniFile;
  I: integer;
  PomStr: string;
begin
  IniF:= TRealIniFile.Create(aName);
  FStringList.Clear;
  IniF.ReadSection('PROJECTIONS', FStringList);

  I:= 0;
  while I < FStringList.Count do
  begin
    FActual:= I;
    PomStr:= IniF.ReadString('PROJECTIONS', FStringList[I], '');
    FStringList.Strings[I]:= ParName(I) + '=' + PomStr;

    SetActValues;
    if FScale = 0 then
       FStringList.Delete(I)
    else
      inc(I)
  end;
  IniF.Free;

  RenameList;

  FActual:= FStringList.Count-1;
end;


procedure TProjectionMemory.SaveToIni(aName: string);
var
  IniF: TRealIniFile;
  I: integer;
  aItem: string;
begin
  IniF:= TRealIniFile.Create(aName);
  IniF.EraseSection('PROJECTIONS');
  for I:= 0 to FStringList.Count-1 do
  begin
    aItem:= 'PROJ'+IntToStrZ(I, 2, true);
    IniF.WriteString('PROJECTIONS', aItem, FStringList.Values[aItem]);
  end;
  IniF.Free;
end;


function TProjectionMemory.GetCenterX(aDef: double): double;
begin
  result:= aDef;
  if FScale <=0 then exit;
  result:= FCenterX
end;


function TProjectionMemory.GetCenterY(aDef: double): double;
begin
  result:= aDef;
  if FScale <=0 then exit;
  result:= FCenterY
end;


function TProjectionMemory.GetScale(aDef: double): double;
begin
  result:= aDef;
  if FScale <=0 then exit;
  result:= FScale
end;



procedure TProjectionMemory.AddProjection(aCenterX, aCenterY, aScale: double);
var
  PStr: string;
begin
  try
    PStr:= IntToStr(round(aCenterX)) + ',' + IntToStr(round(aCenterY)) + ',' + IntToStr(round(aScale));
    inc(FActual);
    if FActual < FStringList.Count then
    begin
      FStringList.Strings[FActual]:= ParName(FActual) + '=' + PStr;
    end
    else
    begin
      if FActual >= MaxCount then
      begin
        FStringList.Delete(0);
        RenameList
      end;
      FActual:= FStringList.Count;
      FStringList.Add(ParName(FActual) + '=' + PStr);
    end;
  except
  end;
end;



end.

