unit i_GN;
{****************************************************************************}
{                                                                            }
{          ORBIS ... Interpret Pro GN                                        }
{                    (c) 1996                                                }
{                                                                            }
{****************************************************************************}

interface
uses
   Wintypes,
   e_View,
   e_Device,
   Elements,
   g_Elems,
   i_Basic;


type

  TGNTableTexts = class(TElement);

  TGNTableTextInterpret = class(TInterpret)
  private
     Rect: TDoubleRect;
     ZMAngle:  double;
{
     procedure   ViewText(QText: string;
                          QLX,QLY: double;
                          QPX,QPY: double;
                          QFName: string;
                          QFHeight: integer;
                          QFColor: longInt;
                          QFStyle: byte;
                          QFAlign: integer);
}                          
     procedure   ViewTextsFromTable;
   protected
     procedure Process(Element: TElement); override;
     function  IsItsClass(AObject: TObject): boolean; override;
     function  QProcessBefore(Element: TElement): boolean; override;
   public
     procedure   DoIt; override;
  end;


  TGNInterpret = class(TInterpret)
  protected
     function  QGoInside(Element: TElement): boolean; override;
     function  QProcessBefore(Element: TElement): boolean; override;
     function  QProcessAfter(Element: TElement): boolean; override;
{     procedure Process(Element: TElement); override; }
     function  IsItsClass(AObject: TObject): boolean; override;
     procedure SetSubInterprets; override;
     procedure FreeSubInterprets; override;
  public
     procedure DoIt; override;
  end;


  procedure ViewTextDev(aDevice: TCanvasDevice;
                        QText: string;
                        QLX,QLY: double;
                        QPX,QPY: double;
                        QFName: string;
                        QFHeight: integer;
                        QFColor: longInt;
                        QFStyle: byte;
                        QFAlign: integer;
                        ZMAngle: double);




implementation
uses
   Controls, Graphics, Dialogs, Forms, WinProcs, SysUtils,
   M_Orbis, Worlds, MathLib, AnalInt,
   e_Desktp,
   e_GFile,
   //e_TopRAS,
   e_Raster,
   e_TxtBlk,
   e_ZM,
   E_layscr,
   i_View,
   //i_Top_R,
   i_Raster,
   i_TTText,
   i_TxtBlk,
   i_Rect,
   i_Files,
   i_Krizku,
   i_SelR,
//   i_LayScr,
   Tools,
   T_Main;




function DejPocetCR(aText: string): integer;
var
  Konec: boolean;
  PosCR: integer;
begin
  Konec:= false;
  result:= 0;
  repeat
    PosCR:= pos(#13+#10, aText);
    if PosCR = 0 then
    begin
      Konec:= true;
    end
    else
    begin
      aText:= copy(aText, PosCR+2, 255);
      inc(result)
    end;
  until Konec
end;




procedure ViewTextDev(aDevice: TCanvasDevice;
                      QText: string;
                      QLX,QLY: double;
                      QPX,QPY: double;
                      QFName: string;
                      QFHeight: integer;
                      QFColor: longInt;
                      QFStyle: byte;
                      QFAlign: integer;
                      ZMAngle: double);
var
  FS: integer;
  Angle: double;
  aCrd: TDoubleCoord;
  Konec: boolean;
  aText, WText: string;
  PosCR: integer;
  aPocetCR: integer;
  aTM: TTextMetric;
  aPom: double;
  anOK: boolean;
begin
  FS:= round(10000.0 * QFHeight/ aDevice.Projection.Scale);
  if abs(FS) < 2 then
  with aDevice.Canvas do
  begin
    Pen.Color:= QFColor;
    Pen.Style:= psSolid;
    Pen.Width:= 1;
    aDevice.MoveTo(QLX, QLY);
    aDevice.LineTo(QLX+aDevice.Projection.GetModelLength(5), QLY)
  end
  else
  begin
    with aDevice.Canvas do
    begin
      Font.Name:= QFName;
      Font.Size:= FS;
      Font.Color:= QFColor;
      Font.Style:= TFontStyles(QFStyle);
    end;
    if (QPX=0) or (QPY=0) then
    begin
      if ZMAngle = 0 then
         Angle:= 350
      else
        Angle:= ZMAngle * 180/Pi
    end
    else
       Angle:= 180*ATan2M(QPX-QLX, QPY-QLY)/Pi;

    aCrd.X:= QLX;
    aCrd.Y:= QLY;
    Konec:= false;

    aText:= QText;
    aPocetCR:= DejPocetCR(QText);

    anOK:= GetTextMetrics(TCanvasDevice(aDevice).Canvas.Handle, aTM);
    aPom:= aDevice.TextModelHeight(QText);
    aPom:= (aPom / aTM.tmHeight) * aTM.tmDescent;

    VratPosunutyBod(aCrd, -(DejRadZeStupnu(Angle)+PiPul),
                    aPom, aCrd);

    VratPosunutyBod(aCrd, DejRadZeStupnu(Angle)+PiPul,
                    aDevice.TextModelHeight(QText)*(aPocetCR + 1), aCrd);

    repeat
      PosCR:= pos(#13+#10, aText);
      if PosCR = 0 then
      begin
        Konec:= true;
        WText:= aText;
      end
      else
      begin
        WText:= copy(aText, 1, PosCR-1);
        aText:= copy(aText, PosCR+2, 255);
      end;
      aDevice.SetTextOrientation(Angle);
      aDevice.TextOut(aCrd.X, aCrd.Y, WText);

      if PosCR > 0 then
      begin
        with aDevice.Canvas do
        begin
          Font.Name:= QFName;
          Font.Size:= FS;
          Font.Color:= QFColor;
          Font.Style:= TFontStyles(QFStyle);
        end;
        VratPosunutyBod(aCrd, DejRadZeStupnu(Angle)-PiPul,
                        aDevice.TextModelHeight(QText), aCrd);
      end;

    until Konec
  end
end;






            { TGNTableTextInterpret }

{
procedure TGNTableTextInterpret.ViewText(QText: string;
                                         QLX,QLY: double;
                                         QPX,QPY: double;
                                         QFName: string;
                                         QFHeight: integer;
                                         QFColor: longInt;
                                         QFStyle: byte;
                                         QFAlign: integer);
var
  FS: integer;
  Angle: double;
  aCrd: TDoubleCoord;
  Konec: boolean;
  aText, WText: string;
  PosCR: integer;

begin
  FS:= round(10000.0 * QFHeight/ Device.Projection.Scale);
  if abs(FS) < 2 then
  with TCanvasDevice(Device).Canvas do
  begin
    Pen.Color:= QFColor;
    Pen.Style:= psSolid;
    Pen.Width:= 1;
    Device.MoveTo(QLX, QLY);
    Device.LineTo(QLX+Device.Projection.GetModelLength(5), QLY)
  end
  else
  begin
    with TCanvasDevice(Device).Canvas do
    begin
      Font.Name:= QFName;
      Font.Size:= FS;
      Font.Color:= QFColor;
      Font.Style:= TFontStyles(QFStyle);
    end;
    if (QPX=0) or (QPY=0) then Angle:= 350
    else
       Angle:= 180*ATan2M(QPX-QLX, QPY-QLY)/Pi;

    aCrd.X:= QLX;
    aCrd.Y:= QLY;
    Konec:= false;

    aText:= QText;
    VratPosunutyBod(aCrd, DejRadZeStupnu(Angle)+PiPul,
                    Device.TextModelHeight(QText), aCrd);
    repeat
      PosCR:= pos(#13+#10, aText);
      if PosCR = 0 then
      begin
        Konec:= true;
        WText:= aText;
      end
      else
      begin
        WText:= copy(aText, 1, PosCR-1);
        aText:= copy(aText, PosCR+2, 255);
      end;
      TCanvasDevice(Device).SetTextOrientation(Angle);
      Device.TextOut(aCrd.X, aCrd.Y, WText);

      if PosCR > 0 then
      begin
        with TCanvasDevice(Device).Canvas do
        begin
          Font.Name:= QFName;
          Font.Size:= FS;
          Font.Color:= QFColor;
          Font.Style:= TFontStyles(QFStyle);
        end;
        VratPosunutyBod(aCrd, DejRadZeStupnu(Angle)-PiPul,
                        Device.TextModelHeight(QText), aCrd);
      end;

    until Konec
  end
end;

}


procedure TGNTableTextInterpret.ViewTextsFromTable;
begin
{$IFNDEF Debug}
{
Zmna 23.5.2000 Vaek zlobil vceuivatelsk provoz
misto TableText dam QueryText:
   pstup pes Query
}
  try
    with TMainTool(World.MainTool).TextDBForm do
    begin
      QueryText.DisableControls;
      QueryText.First;
      while not QueryText.EOF do
      begin
{        Memo1.Text:= NazevTisk.AsString; }

        ViewTextDev(TCanvasDevice(Device),
                 NazevTisk.AsString,
                 -LY.AsFloat,-LX.AsFloat, { pevod z Kovka }
                 -PY.AsFloat,-PX.AsFloat,
                 FontName.AsString,
                 FontHeight.AsInteger,
                 FontColor.AsInteger,
                 FontStyle.AsInteger,
                 FontAlign.AsInteger,
                 ZMAngle);

        QueryText.Next;
      end;
      QueryText.EnableControls;
      QueryText.Close;

{ to se pestalo pouvat u ped zmnou na QueryText     TableText.Close; }
    end;

  except
    ShowMessage('Problmy se zobrazenm text')
  end;
{$ENDIF}
end;




procedure TGNTableTextInterpret.Process(Element: TElement);
var
  ZM: TMapsZM;
  I, J, K, L: longInt;


  function IsOverCorns(aCorns: TCorns): boolean;
  var
    ZMRect, DevRect: TRectangle;
  begin
    result:= false;
    ZMRect:= CreateRectangleFromCorns(aCorns);
    DevRect:= TRectangle.Create;

    if ZMRect = nil then exit;
    DevRect.UpdateByDoubleRect(Device.Projection.ModelView);
    result:= ZMRect.IsOverLap(DevRect);

    DevRect.Free;
    ZMRect.Free
  end;

begin
  if not (Element is TGNTableTexts) then exit;
  if (Device.Projection.Scale < 1000) or (Device.Projection.Scale > 400000) then exit;

  ZM:= TMapsZM.Create;
  ZM.Scale:= 50000;
  ZMAngle:= 0;

  for I:= 1 to 35 do
  begin
    ZM.Map200:= I;
    if not IsOverCorns(ZM.FCorns200) then continue;

    for J:= 1 to 4 do
    begin
      ZM.Map100:= J;
      if not IsOverCorns(ZM.FCorns100) then continue;

      for K:= 1 to 4 do
      begin
        ZM.Map50:= K;
        if not IsOverCorns(ZM.FCorns50) then continue;
{$IFNDEF Debug}
        if Device.Projection.Scale > 100000 then
        begin
          TMainTool(World.MainTool).TextDBForm.GetTextsZM50(ZM.MapName);
          ZMAngle:= ZM.Angle;
          ViewTextsFromTable
        end
        else
        begin
          ZM.Scale:= 10000;
          for L:= 1 to 25 do
          begin
            ZM.Map10:= L;
            if not IsOverCorns(Zm.FCorns10) then continue;

            TMainTool(World.MainTool).TextDBForm.GetTextsZM(ZM.MapName);
            ZMAngle:= ZM.Angle;
            ViewTextsFromTable
          end { 10 }
        end
{$ENDIF}
      end { 50 }
    end  { 100 }
  end; { 200 }

  ZM.Free
end;








function  TGNTableTextInterpret.IsItsClass(AObject: TObject): boolean;
begin
  Result := (AObject is TGNTableTexts)
end;

function  TGNTableTextInterpret.QProcessBefore(Element: TElement): boolean;
begin
  Result:= (Element.ClassType = TGNTableTexts);
end;


procedure TGNTableTextInterpret.DoIt;
begin
   Rect := Device.Projection.ModelView;
   inherited DoIt;
end;





           { TGNInterpret }

function  TGNInterpret.QGoInside(Element: TElement): boolean;
begin
  Result:= (Element is TEntity) and not ((Element is TView)
            or (Element is TText))
            or (Element is TModelScript);
end;

function  TGNInterpret.QProcessBefore(Element: TElement): boolean;
begin
  Result:= (Element is TSelectedElement) or
           (Element is TText) or
           (Element is TGNTableTexts) or
           (Element is TTextBlock) or
           (Element is TMapsZM) or
           (Element is TBMPFileElement) or
           (Element is TLines)or
           (Element is TRaster) or
           (Element is TLayerScript)
           ;
end;

function  TGNInterpret.QProcessAfter(Element: TElement): boolean;
begin
  Result:= false
end;



{
procedure TGNInterpret.Process(Element: TElement);
var
  DT: TDeskTop;
begin
  if (Element is TDesktop) then
  begin
     DT:= TDesktop(Element);
     Device.Brush.Color := DT.BrushColor;
     Device.Brush.Style := DT.BrushStyle;
     Device.Pen.Style   := DT.PenStyle;
     Device.PenWidth    := DT.PenWidth;
     Device.Pen.Color   := DT.PenColor;
     with DT.Rect do Device.Rectangle(Min.x,Min.Y,Max.X,Max.Y);
  end
  else
  begin
    inherited Process(Element);
  end
end;
}


function TGNInterpret.IsItsClass(AObject: TObject): boolean;
begin
   result := AObject is TDesktop;
end;


procedure TGNInterpret.SetSubInterprets;
begin
   NextInterpret := TSelectedElementInterpret.Create;
   NextInterpret := TTextInterpret.Create;
   NextInterpret := TTextBlockInterpret.Create;
   NextInterpret := TInterpretZM.Create;
   NextInterpret := TBMPFileInterpret.Create;
   NextInterpret := TRasterInterpret.Create;
   NextInterpret := TGNTableTextInterpret.Create;
   NextInterpret := TLinesInterpret.Create;
//   NextInterpret := TInterpretLayerScript.Create;
end;


procedure TGNInterpret.FreeSubInterprets;
begin
  inherited FreeSubInterprets
end;


procedure TGNInterpret.DoIt;
begin
{$IFNDEF WIN32}
   PostTextMsg(tm_ShowText, IntToStr(MemAvail), World.MainTool);
{$ENDIF}
   Application.ProcessMessages;
   Reset(Device);
   inherited DoIt;
end;



END.
