unit T_GNedit;
{
*********************************************************************
*                                                                   *
*  ORBIS ... Nstroj pro zobrazen editoru text                    *                                                                   *
*                       Topol Pro 1996                             *
*                                                                   *
*********************************************************************
}


interface
uses
   Controls,
   Menus,
   Elements,
   E_ProMem,
   g_Elems,
   f_GNEdit,
   m_InterA,
   f_RectEd,
   t_InterA,
   M_Proj;
type


{ TGNEditorFormTool }

TGNEditorFormTool = class(TFormToolManager)
 private
   FForm: TGNEditorForm;
   FSizePanel: TF_rectedForm;
   FFileName: PString;
   FRole: TUserRole;
   FProjectionMem: TProjectionMemory;
   FActualTextColor: longInt;
   FActualTextCounter: integer;
   function  GetFileName: string;
   procedure SetFileName(aValue: string);
   function GetProjectionMem: TProjectionMemory;
   procedure ReadParams;
   procedure SaveParams;
   procedure SetRole(aRole: TUserRole);
   procedure SetRightProjection;
   procedure UpSetText(aUp: Boolean);
 protected
   procedure   SetInput(aInput: TElement); override;
   procedure   CreateForm; override;
   procedure   LinkDeviceToForm; override;
   procedure   CreateInterpret; override;
   procedure   CreateTools; override;
   function    GetVisible: boolean; override;
   procedure   SetVisible(aVisible: boolean); override;
   function    GetMainMenu: TMainMenu; override;
   function    GetCaption: string; override;
   procedure   SetCaption(const aValue: string); override;
   function    GetFormCursor: TCursor; override;
   procedure   SetFormCursor(aValue: TCursor); override;
   function    GetSizePanel: TF_rectedForm; virtual;
   procedure   ViewActualText(AColor: longInt);
   procedure   AddVector;
 public
   Coord: TDoubleCoord;

   constructor Create; override;
   destructor  Destroy; override;
   procedure   DoIt; override;

   procedure   On_tm_Review(var aMessage: TOrbisMessage; var Handled: boolean); override;
   procedure   on_tm_QuitTask; override;
   procedure   on_tm_ToolSelected(var aMessage: TOrbisMessage; var Handled: boolean); override;
   procedure   on_tm_ZoomIn(var Handled: boolean); virtual;
   procedure   on_tm_ZoomOut(var Handled: boolean); virtual;
   procedure   on_tm_Viewer(var Handled: boolean); virtual;
   procedure   on_tm_TableTextShow(var Handled: boolean);
   procedure   on_tm_ViewZM(var Handled: boolean);
   procedure   on_tm_NewView(var Handled: boolean);
   procedure   on_tm_NewTextTool(var Handled: boolean);
   procedure   on_tm_EditTextTool(var Handled: boolean);
   procedure   on_tm_ViewActualText(var Handled: boolean);
   procedure   on_tm_ViewActualTextTimer(var Handled: boolean);
   procedure   on_tm_PrevProjection;
   procedure   on_tm_NextProjection;
   procedure   on_tm_DBEditText(var aMessage: TOrbisMessage; var Handled: boolean);

   procedure   on_tm_AddProjection(var Handled: boolean);
   procedure   on_tm_ShowRAS(var Handled: boolean);
   procedure   on_tm_SetRAS(var Handled: boolean);
   procedure   on_tm_WSpaceSet(var Handled: boolean);
   procedure   on_tm_WSpaceMove(var aMessage: TOrbisTextMessage; var Handled: boolean);
   procedure   on_tm_WSpaceShow(var Handled: boolean);


   procedure   on_tm_MouseMove(var aMessage: TOrbisMouseEvent; var Handled: boolean); override;

   procedure   on_tm_ViewBySelect(var Handled: boolean); virtual;

   procedure   on_tm_New(var Handled: boolean); virtual;
   procedure   on_tm_Open(var Handled: boolean); virtual;
   procedure   on_tm_Save(var Handled: boolean); virtual;
   procedure   on_tm_SaveAs(var Handled: boolean); virtual;
   procedure   on_tm_SetupSelect(var Handled: boolean); virtual;
   procedure   on_tm_SelectToTop(var Handled: boolean); virtual;
   procedure   on_tm_SelectToBottom(var Handled: boolean); virtual;
   procedure   on_tm_DeleteSelect(var Handled: boolean); virtual;
   procedure   on_tm_SelectSizeChange(var Handled: boolean); virtual;

   procedure   on_tm_SetPropertyEditors(var Handled: boolean); virtual;
   procedure   on_tm_SizePanelOn (var Handled: boolean); virtual;
   procedure   on_tm_SizePanelOff (var Handled: boolean); virtual;

   procedure   On_tm_Keydown(var aMessage: TOrbisKeyEvent; var Handled: boolean); virtual;
   procedure   on_tm_Print(var Handled: boolean); override;

   procedure   HandleMessage(var aMessage: TOrbisMessage; var Handled: boolean); override;

   property    GForm: TGNEditorForm read FForm;
   property    SizePanel: TF_rectedForm read GetSizePanel;
   property    FileName: string read GetFileName write SetFileName;
   property    Role: TUserRole write SetRole;
   property    ProjectionMem: TProjectionMemory read GetProjectionMem;
end;




implementation


uses
   Dialogs, WinTypes,
   _IniFile,
   SysUtils,
   Graphics,
   Forms,
   AnalInt, e_device,
   MathLib,
   //e_TopRas,
   d_STDImg,
   m_Orbis,
   i_GN,
   e_GNRas,
   i_Krizku,
   t_Editor,
   t_Loader,
   t_Main,
   t_Print,
   t_Saver,
   t_GNIntr,
   u_String,
   e_ZM,
   f_PrPros,
   f_SRDlg,
   T_LayScr,
   u_DBlib,
   Worlds;

{ TGNEditorFormTool }



constructor TGNEditorFormTool.Create;
begin
   inherited Create;
   Device.Page.Color := clSilver;

   TMainTool(World.MainTool).TextDBForm.Tool:= self
end;





destructor  TGNEditorFormTool.Destroy;
begin
   FSizePanel.Free;
   if FForm <> nil then
      FForm.Free;
   FileName := '';
   if FProjectionMem <> nil then
      FProjectionMem.Free;
   inherited Destroy;
end;


procedure TGNEditorFormTool.SetRightProjection;
begin
   if (abs(Device.Projection.Center_X) > 1E7) or
      (abs(Device.Projection.Center_X) < 200000) then
      Device.Projection.Center_X:= -600000;

   if (abs(Device.Projection.Center_Y) > 1E7) or
      (abs(Device.Projection.Center_Y) < 200000) then
      Device.Projection.Center_Y:= -1000000;

   if (abs(Device.Projection.Scale) > 10000000) or
      (abs(Device.Projection.Scale) < 100) then
      Device.Projection.Scale:= 1000000;

   if (abs(Device.Projection.VirtualScale) > 1000000) or
      (abs(Device.Projection.VirtualScale) < 100) then
      Device.Projection.Scale:= 10000;
end;


procedure TGNEditorFormTool.ReadParams;
var
   Ini: TRealIniFile;
   X, Y: double;
begin
{
[TGNEditorFormTool]
Center_X=-476602,623025076
Center_Y=-1261992,6971815
Scale=6371,67194992066
PageColor=16777215
VirtualScale=7235,88414401501
}
   SetRightProjection;

   Ini:= TRealIniFile.Create(TMainTool(World.MainTool).IniFileName);

   X:= Ini.ReadDouble(Name, 'Center_X', Device.Projection.Center_X);
   Device.Projection.Center_X:= Ini.ReadDouble(Name, 'Center_X', Device.Projection.Center_X);

   Y:= Ini.ReadDouble(Name, 'Center_Y', Device.Projection.Center_Y);
   Device.Projection.Center_Y:= Ini.ReadDouble(Name, 'Center_Y', Device.Projection.Center_Y);

   Device.Projection.Scale:= Ini.ReadDouble(Name, 'Scale', Device.Projection.Scale);

   SetRightProjection;

   Device.Projection.VirtualScale:= Ini.ReadDouble(Name, 'Scale', 10000);

   Device.Page.Color := Ini.ReadInteger(Name, 'PageColor', clSilver);
   Ini.Free
end;



procedure  TGNEditorFormTool.SaveParams;
var
   Ini: TRealIniFile;
begin
   Ini:= TRealIniFile.Create(TMainTool(World.MainTool).IniFileName);
   Ini.WriteDouble(Name, 'Center_X', Device.Projection.Center_X);
   Ini.WriteDouble(Name, 'Center_Y', Device.Projection.Center_Y);
   Ini.WriteDouble(Name, 'Scale',    Device.Projection.Scale);
   Ini.WriteDouble(Name, 'VirtualScale', Device.Projection.VirtualScale);
   Ini.WriteInteger(Name, 'PageColor', Device.Page.Color);
   Ini.Free;

   if FProjectionMem <> nil then
      FProjectionMem.SaveToIni(TMainTool(World.MainTool).IniFileName)
end;



function TGNEditorFormTool.GetProjectionMem: TProjectionMemory;
begin
  if FProjectionMem = nil then
  begin
    FProjectionMem:= TProjectionMemory.Create;
    FProjectionMem.ReadFromIni(TMainTool(World.MainTool).IniFileName);
  end;
  result:= FProjectionMem
end;


procedure  TGNEditorFormTool.DoIt;
var
  aZM: TMapsZM;
  Lines: TLines;
  Rases: TGNRasters;
  E: TRectEntity;
  Texty: TGNTableTexts;
begin
   if Input = nil then
   begin
      E := TRectEntity.Create;
      Input := E;
   end;
   E := TRectEntity(Input);



   with TOptions(TMainTool(World.MainTool).Options) do
   begin
     aZM := TMapsZM(E.FindElementClass(TMapsZM));

     if ShowZM then
     begin
       if (aZM = nil) then
       begin
          GForm.b_ShowZM.Down:= true;
          aZM:= TMapsZM.Create;
          aZM.Scale:= 10000;
          E.AddAndUpdateRect(aZM);
       end
     end
     else
     begin
       GForm.b_ShowZM.Down:= false;
       if aZM <> nil then
          E.FreeElement(aZM)
     end;

     if ShowRAS then
     begin
       GForm.b_ShowRAS.Down:= true;
       Rases:= TGNRasters.Create;
       Rases.AddRasters(RasterLayers, RasterDirectory, LColor, PColor, RColor, TColor, VColor);
       E.AddElement(Rases)
     end
     else
       GForm.b_ShowRAS.Down:= false;

     if WorkSpaceShow then
     begin
       GForm.b_ShowWSpace.Down:= true;
       Lines:= TLines.Create;
       Lines.Width:= WorkSpaceWidth;
       Lines.Height:= WorkSpaceHeight;
       Lines.Color:= WorkSpaceColor;
       E.AddElement(Lines);
     end
     else
       GForm.b_ShowWSpace.Down:= false;

     if TableTextsView then
     begin
       GForm.b_ShowText.Down:= true;
       Texty:= TGNTableTexts.Create;
       E.AddElement(Texty)
     end
     else
       GForm.b_ShowText.Down:= false;

   end;


   ReadParams;
   PostMsg(tm_Review, Self, nil, DontKill);
end;



function    TGNEditorFormTool.GetFormCursor: TCursor;
begin
   result := GForm.Image.Cursor;
end;


procedure   TGNEditorFormTool.SetFormCursor(AValue: TCursor);
begin
   GForm.Image.Cursor := aValue;
end;



function    TGNEditorFormTool.GetSizePanel: TF_rectedForm;
begin
   if FSizePanel = nil then
   begin
      Application.CreateForm(TF_rectedForm, FSizePanel);
      FSizePanel.Tool := Self;
      FSizePanel.Editor := TRectEntityEditor(Tools.FindElementClass(TRectEntityEditor));
   end;
   result := FSizePanel;
end;



procedure TGNEditorFormTool.ViewActualText(AColor: longInt);
var
  FS: integer;
  Angle: double;
  aCrd: TDoubleCoord;

  QText: string;
  QLX,QLY: double;
  QPX,QPY: double;
  QFName: string;
  QFHeight: integer;
  QFColor: longInt;
  QFStyle: byte;
  QFAlign: integer;
  ZMName: string;
  ZM: TMapsZM;
  ZMAngle: double;
begin
  {$IFNDEF Debug}
  with TMainTool(World.MainTool).TextDBForm do
  begin
(*
      QText:= TableNazev_Tisk_001.AsString;
      QLX:= -TableSour_LDY_001.AsFloat;
      QLY:= -TableSour_LDX_001.AsFloat; { pevod z Kovka }

      QPX:= -TableSour_PDY_001.AsFloat;
      QPY:= -TableSour_PDX_001.AsFloat;

      QFName:= TableFont_Name_001.AsString;
      QFHeight:= TableFont_Height_001.AsInteger;
      ZMName:= TableMapa_001.AsString;

      if AColor < 0 then
         QFColor := TableFont_Color_001.AsInteger
      else
         QFColor:= AColor;

      QFStyle:=  TableFont_Style_001.AsInteger;
      QFAlign:=  TableFont_Align_001.AsInteger
  *)
      QText:= QueryNazev_Tisk_001.AsString;
      QLX:= -QuerySour_LDY_001.AsFloat;
      QLY:= -QuerySour_LDX_001.AsFloat; { pevod z Kovka }

      QPX:= -QuerySour_PDY_001.AsFloat;
      QPY:= -QuerySour_PDX_001.AsFloat;

      QFName:= QueryFont_Name_001.AsString;
      QFHeight:= QueryFont_Height_001.AsInteger;
      ZMName:= QueryMapa_001.AsString;

      if AColor < 0 then
         QFColor := QueryFont_Color_001.AsInteger
      else
         QFColor:= AColor;

      QFStyle:=  QueryFont_Style_001.AsInteger;
      QFAlign:=  QueryFont_Align_001.AsInteger
  end;

  ZMAngle:= 0;
  if ZMName <> '' then
  begin
    ZM:= TMapsZM.Create;
    ZM.MapName:= ZMName;
    ZMAngle:= ZM.Angle;
    ZM.Free;
  end;

  ViewTextDev(TCanvasDevice(Device),
              QText,
              QLX, QLY, { pevod z Kovka }
              QPX, QPY,
              QFName,
              QFHeight,
              QFColor,
              QFStyle,
              QFAlign,
              ZMAngle);

(*
  FS:= round(10000.0 * QFHeight/ Device.Projection.Scale);
  if abs(FS) < 2 then
  with TCanvasDevice(Device).Canvas do
  begin
    Pen.Color:= QFColor;
    Pen.Style:= psSolid;
    Pen.Width:= 1;
    Device.MoveTo(QLX, QLY);
    Device.LineTo(QLX+Device.Projection.GetModelLength(5), QLY)
  end
  else
  begin
    with TCanvasDevice(Device).Canvas do
    begin
      Font.Name:= QFName;
      Font.Size:= FS;
      Font.Color:= QFColor;
      Font.Style:= TFontStyles(QFStyle);
    end;
    if (QPX=0) or (QPY=0) then Angle:= 350
    else
       Angle:= 180*ATan2M(QPX-QLX, QPY-QLY)/Pi;

    aCrd.X:= QLX;
    aCrd.Y:= QLY;

    VratPosunutyBod(aCrd, DejRadZeStupnu(Angle)+PiPul,
                    Device.TextModelHeight(QText), aCrd);

    TCanvasDevice(Device).SetTextOrientation(Angle);
    Device.TextOut(aCrd.X, aCrd.Y, QText)
  end
*)
{$ENDIF Debug}

end;






procedure   TGNEditorFormTool.on_tm_QuitTask;
begin
   GForm.Free;
   FForm:= nil;
   inherited on_tm_QuitTask;
end;


procedure   TGNEditorFormTool.On_tm_ReView(var aMessage: TOrbisMessage; var Handled: boolean);
var
   ShowSizePanel: boolean;
begin
   ShowSizePanel := false;
   {
   if SizePanel.Visible = true then
   begin
      ShowSizePanel := true;
      SizePanel.Visible := false;
   end;
   }
   Application.ProcessMessages;
   Interpret.Reset(Device);
   Device.Clear;

   try
     GForm.StatusBar.Panels.Items[2].Text := 'M 1 : '+
          IntToStr(round(Device.Projection.Scale))
   except
     ShowMessage('Problem: Device Projection Scale')
   end;

   inherited On_tm_ReView(aMessage, Handled);


   Device.Close;
   Interpret.Close;
   if ShowSizePanel then SizePanel.Visible := true;

   SaveParams
end;





procedure   TGNEditorFormTool.on_tm_ToolSelected(var aMessage: TOrbisMessage; var Handled: boolean);
var
   aClass: TClass;
begin
   {
   GForm.Panel1.Visible := false;
   }
   inherited on_tm_ToolSelected(aMessage, Handled);
   Handled := true;
   aClass := aMessage.Data.ClassType;
   {
   if aClass = TDrawRectTool then GForm.b_Rect.Down := true else
   if aClass = TViewer then GForm.b_Viewer.Down := true;
   }
end;


procedure   TGNEditorFormTool.on_tm_ZoomIn(var Handled: boolean);
begin
   try
      FindMenuItemIn('Tools','TZoomIn').OnClick(nil);
   except
   end;
end;

procedure   TGNEditorFormTool.on_tm_ZoomOut(var Handled: boolean);
begin
   try
      FindMenuItemIn('Tools','TZoomOut').OnClick(nil);
   except
   end;
end;



procedure   TGNEditorFormTool.on_tm_Viewer(var Handled: boolean);
begin
   try
      FindMenuItemIn('Tools','TViewer').OnClick(nil);
   except
   end;
end;


procedure TGNEditorFormTool.on_tm_TableTextShow(var Handled: boolean);
var
  Texty: TGnTableTexts;
begin
  Texty:= TGNTableTexts(TEntity(Input).FindElementClass(TGNTableTexts));
  if not GForm.b_ShowText.Down then
  begin
    if Texty <> nil then
       TEntity(Input).FreeElement(Texty);
    TOptions(TMainTool(World.MainTool).Options).TableTextsView:= false
  end
  else
  begin
    Texty:= TGNTableTexts.Create;
    TEntity(Input).AddElement(Texty);
    TOptions(TMainTool(World.MainTool).Options).TableTextsView:= true
  end;
{  PostMsg(tm_Review, Self, nil, DontKill); }
end;





procedure   TGNEditorFormTool.on_tm_NewView(var Handled: boolean);
begin
   try
      FindMenuItemIn('Tools','TNewView').OnClick(nil);
   except
   end;
end;


procedure   TGNEditorFormTool.on_tm_NewTextTool(var Handled: boolean);
begin
   try
     if (Device.Projection.Scale > HorniMezEditace) or (Device.Projection.Scale < DolniMezEditace) then
     begin
        ShowMessage('Pro vkldn novch text' + #13+
                 'mus bt mtko v rozmez 1:' + IntToStr(DolniMezEditace) +
                 ' a 1:' + IntToStr(HorniMezEditace));

        if Device.Projection.Scale > HorniMezEditace then
        begin
           GForm.b_ZoomIn.Down:= true;
           PostMsg(tm_ZoomIn, self, nil, DontKill)
        end
        else
        begin
          GForm.b_ZoomOut.Down:= true;
          PostMsg(tm_ZoomOut, self, nil, DontKill)
        end
     end
     else
        FindMenuItemIn('Tools','TNewTextTool').OnClick(nil)
   except
   end;
end;


procedure TGNEditorFormTool.on_tm_ViewActualTextTimer(var Handled: boolean);
begin
  if FActualTextCounter = 0 then
  begin
    FActualTextColor:= -1;
    GForm.Timer.Enabled:= false
  end
  else
  begin
    if FActualTextCounter mod 2 = 0 then
       FActualTextColor:= clWhite
    else
       FActualTextColor:= clRed
  end;

  ViewActualText(FActualTextColor);
  dec(FActualTextCounter);
end;



procedure  TGNEditorFormTool.on_tm_DBEditText(var aMessage: TOrbisMessage; var Handled: boolean);
begin
  if (Device.Projection.Scale > HorniMezEditace) or (Device.Projection.Scale < DolniMezEditace) then
  begin
    Device.Projection.Scale:= 10000;
    On_tm_Review(aMessage, Handled);
  end;


  GForm.b_EditText.Down:= true;
  on_tm_ViewActualText(Handled);

  if ActiveToolClassName = 'TEditTextTool' then
  begin
    TEditTextTool(ActiveTool).Deselect;
    TEditTextTool(ActiveTool).On_Activate;
  end
  else
    on_tm_EditTextTool(Handled);

{
  with TMainTool(World.MainTool).TextDBForm do
  begin

  end;
}
  Handled:= true
end;



procedure TGNEditorFormTool.on_tm_ViewActualText(var Handled: boolean);
begin
  with TMainTool(World.MainTool).TextDBForm do
  begin
(*
    Device.Projection.Center_X:= -TableSour_LDY_001.AsFloat;
    Device.Projection.Center_Y:= -TableSour_LDX_001.AsFloat;
*)
  {$IFNDEF Debug}

    Device.Projection.Center_X:= -QuerySour_LDY_001.AsFloat;
    Device.Projection.Center_Y:= -QuerySour_LDX_001.AsFloat;
  {$ENDIF Debug}

  end;

  PostMsg(tm_Review, Self, nil, DontKill);

  FActualTextCounter:= 10;
  GForm.Timer.Enabled:= true;
end;


procedure   TGNEditorFormTool.on_tm_EditTextTool(var Handled: boolean);
begin
   try
      FindMenuItemIn('Tools','TEditTextTool').OnClick(nil);
   except
   end;
end;


procedure   TGNEditorFormTool.on_tm_PrevProjection;
begin
  ProjectionMem.SetPrevProjection;
  with Device.Projection do
  begin
    Center_X:= ProjectionMem.GetCenterX(Center_X);
    Center_Y:= ProjectionMem.GetCenterY(Center_Y);
    Scale:= ProjectionMem.GetScale(Scale);
  end;
  PostMsg(tm_Review, Self, nil, DontKill);
end;


procedure   TGNEditorFormTool.on_tm_NextProjection;
begin
  ProjectionMem.SetNextProjection;
  with Device.Projection do
  begin
    Center_X:= ProjectionMem.GetCenterX(Center_X);
    Center_Y:= ProjectionMem.GetCenterY(Center_Y);
    Scale:= ProjectionMem.GetScale(Scale);
  end;
  PostMsg(tm_Review, Self, nil, DontKill);
end;


procedure   TGNEditorFormTool.on_tm_AddProjection(var Handled: boolean);
begin
  with Device.Projection do
     ProjectionMem.AddProjection(Center_X, Center_Y, Scale)
end;


procedure   TGNEditorFormTool.on_tm_SetRAS(var Handled: boolean);
var
   R: TRasShowDialog;
   aLColor, aPColor, aRColor, aTColor, aVColor: TColor;
   aLayers: string;
begin
  with TOptions(TMainTool(World.MainTool).Options) do
  begin
    Application.CreateForm(TRasShowDialog, R);
    R.SetFormVar(LColor, PColor, RColor, TColor, VColor, RasterLayers);
    if R.ShowModal = mrOK then
    begin
      R.GetFormVar(aLColor, aPColor, aRColor, aTColor, aVColor, aLayers);
      LColor:= aLColor;
      PColor:= aPColor;
      RColor:= aRColor;
      TColor:= aTColor;
      VColor:= aVColor;
      RasterLayers:= aLayers;
    end;
    R.Free;
  end
end;


procedure TGNEditorFormTool.AddVector;
var
  MSC: TModelScriptCreator;

begin
  MSC:= TModelScriptCreator.Create;
  with TOptions(TMainTool(World.MainTool).Options) do
  begin
    MSC.DataDirectory:= RasterDirectory;
    MSC.NextLayer:= 'LIN=+*.BLK';
  end;

  MSC.DoIt;
  TEntity(Input).AddElement(MSC.Output);

  MSC.Free
end;



procedure   TGNEditorFormTool.on_tm_ViewZM(var Handled: boolean);
var
   aZM: TMapsZM;
begin
  aZM := TMapsZM(TEntity(Input).FindElementClass(TMapsZM));

  with TOptions(TMainTool(World.MainTool).Options) do
  begin
    if GForm.b_ShowZM.Down then
    begin
      if aZM = nil then
      begin
        aZM:= TMapsZM.Create;
        aZM.Scale:= 10000;
        TRectEntity(Input).AddAndUpdateRect(aZM);
      end;
      ShowZM:= true
    end
    else
    begin
      Application.ProcessMessages;
      ShowZM:= false;
      if aZM <> nil then
         TEntity(Input).FreeElement(aZM);
    end;
  end
end;




procedure   TGNEditorFormTool.on_tm_ShowRAS(var Handled: boolean);
var
   R: TGNRasters;
begin
   with TOptions(TMainTool(World.MainTool).Options) do
   begin
     if GForm.b_ShowRAS.Down then
     begin
{       GForm.b_ShowRAS.Down:= true; }
       R:= TGNRasters.Create;
       R.AddRasters(RasterLayers, RasterDirectory, LColor, PColor, RColor, TColor, VColor);
       TEntity(Input).AddElement(R);
       ShowRas:= true
     end
     else
     begin
{       GForm.b_ShowRAS.Down:= false; }
       Application.ProcessMessages;
       R:= TGNRasters(TEntity(Input).FindElementClass(TGNRasters));
       ShowRas:= false;
       if R <> nil then
          TEntity(Input).FreeElement(R);

{       AddVector; }
     end;
  end
{
   GForm.OpenDialog.FileName := '*.ras';
   if GForm.OpenDialog.Execute then
   begin
      try
         R := TTopRas.Create(GForm.OpenDialog.FileName);
         TRectEntity(Input).AddAndUpdateRect(R);
         PostMsg(tm_Review, Self, nil, DontKill);
      except
         ShowMessage('Chyba pipojen rastru '+ GForm.OpenDialog.FileName);
      end;
   end;
}
end;




procedure   TGNEditorFormTool.on_tm_WSpaceShow(var Handled: boolean);
var
   Lines: TLines;
begin
  Lines := TLines(TEntity(Input).FindElementClass(TLines));


  with TOptions(TMainTool(World.MainTool).Options) do
  begin
    if GForm.b_ShowWSpace.Down then
    begin
      if Lines = nil then
      begin
        Lines:= TLines.Create;
        Lines.Width:=  WorkSpaceWidth;
        Lines.Height:= WorkSpaceHeight;
        Lines.Color:=  WorkSpaceColor;

        TEntity(Input).AddElement(Lines);
      end;
      WorkSpaceShow:= true
    end
    else
    begin
      Application.ProcessMessages;
      WorkSpaceShow:= false;
      if Lines <> nil then
         TEntity(Input).FreeElement(Lines);
    end;
  end
end;






procedure   TGNEditorFormTool.on_tm_WSpaceSet(var Handled: boolean);
var
  PrPr: TFPrProstor;
  aChange: boolean;
  Lines: TLines;
begin
  Application.CreateForm(TFPrProstor, PrPr);
  PrPr.SpaceWidth:= TOptions(TMainTool(World.MainTool).Options) .WorkSpaceWidth;
  PrPr.SpaceHeight:= TOptions(TMainTool(World.MainTool).Options).WorkSpaceHeight;
  PrPr.SpaceColor:= TOptions(TMainTool(World.MainTool).Options) .WorkSpaceColor;

  if PrPr.ShowModal = mrOk then
  begin
    aChange:= false;
    if TOptions(TMainTool(World.MainTool).Options).WorkSpaceWidth <> PrPr.SpaceWidth then
    begin
      TOptions(TMainTool(World.MainTool).Options).WorkSpaceWidth:= PrPr.SpaceWidth;
      aChange:= true;
    end;
    if TOptions(TMainTool(World.MainTool).Options).WorkSpaceHeight <> PrPr.SpaceHeight then
    begin
      TOptions(TMainTool(World.MainTool).Options).WorkSpaceHeight:= PrPr.SpaceHeight;
      aChange:= true;
    end;
    if TOptions(TMainTool(World.MainTool).Options).WorkSpaceColor <> PrPr.SpaceColor then
    begin
      TOptions(TMainTool(World.MainTool).Options).WorkSpaceColor:= PrPr.SpaceColor;
      aChange:= true;
    end;

    if aChange then
    begin
      Lines := TLines(TEntity(Input).FindElementClass(TLines));
      if Lines <> nil then
         TEntity(Input).FreeElement(Lines);

{      PostMsg(tm_AddProjection, Self, nil, DontKill); }
{      PostMsg(tm_Review, Self, nil, DontKill) }
    end
  end;

  PrPr.Free
end;


procedure SetActualWS(aX, aY: double; var XBL, YBL: double);
begin
  with TOptions(TMainTool(World.MainTool).Options) do
  begin
    XBL:= (round(aX) div round(WorkSpaceWidth)-1)* WorkSpaceWidth;
    YBL:= (round(aY) div round(WorkSpaceHeight)-1)* WorkSpaceHeight;
  end;
end;


procedure TGNEditorFormTool.on_tm_WSpaceMove(var aMessage: TOrbisTextMessage; var Handled: boolean);
var
  aMes: char;
  XBL, YBL: double;
  WSRect: TDoubleRect;
begin
  with TOptions(TMainTool(World.MainTool).Options) do
  if (WorkSpaceWidth < 100) or (WorkSpaceHeight < 100) then
  begin
     on_tm_WSpaceSet(Handled);
     exit
  end;

  aMes:= aMessage.Text[1];
  SetActualWS(Device.Projection.Center_X, Device.Projection.Center_Y,
              XBL, YBL);

  with TOptions(TMainTool(World.MainTool).Options) do
  case aMes of
  'C':
    with WSRect do
    begin
      MinX:= XBL;
      MinY:= YBL;
      MaxX:= XBL + WorkSpaceWidth;
      MaxY:= YBL + WorkSpaceHeight;
    end;
  'L':
    with WSRect do
    begin
      MinX:= XBL - WorkSpaceWidth;
      MinY:= YBL;
      MaxX:= XBL;
      MaxY:= YBL + WorkSpaceHeight;
    end;
  'R':
    with WSRect do
    begin
      MinX:= XBL + WorkSpaceWidth;
      MinY:= YBL;
      MaxX:= XBL + 2*WorkSpaceWidth;
      MaxY:= YBL + WorkSpaceHeight;
    end;
  'U':
    with WSRect do
    begin
      MinX:= XBL;
      MinY:= YBL + WorkSpaceHeight;
      MaxX:= XBL + WorkSpaceWidth;
      MaxY:= YBL + 2*WorkSpaceHeight;
    end;
  'D':
    with WSRect do
    begin
      MinX:= XBL;
      MinY:= YBL - WorkSpaceHeight;
      MaxX:= XBL + WorkSpaceWidth;
      MaxY:= YBL;
    end;
  'X':
    with WSRect do
    begin
      MinX:= XBL - WorkSpaceWidth;
      MinY:= YBL + WorkSpaceHeight;
      MaxX:= XBL;
      MaxY:= YBL + 2*WorkSpaceHeight;
    end;
  'Y':
    with WSRect do
    begin
      MinX:= XBL + WorkSpaceWidth;
      MinY:= YBL + WorkSpaceHeight;
      MaxX:= XBL + 2*WorkSpaceWidth;
      MaxY:= YBL + 2*WorkSpaceHeight;
    end;
  'V':
    with WSRect do
    begin
      MinX:= XBL - WorkSpaceWidth;
      MinY:= YBL - WorkSpaceHeight;
      MaxX:= XBL;
      MaxY:= YBL;
    end;
  'W':
    with WSRect do
    begin
      MinX:= XBL + WorkSpaceWidth;
      MinY:= YBL - WorkSpaceHeight;
      MaxX:= XBL + 2*WorkSpaceWidth;
      MaxY:= YBL;
    end;
  end;

  Device.Projection.ChangeProjectionByRect(WSRect, Device.Page);
{  PostMsg(tm_AddProjection, Self, nil, DontKill); }
  PostMsg(tm_Review, Self, nil, DontKill);
  Handled:= true;
end;


procedure   TGNEditorFormTool.On_tm_MouseMove(var aMessage: TOrbisMouseEvent; var Handled: boolean);
begin
    Handled := true;
    begin
       Coord := Device.Projection.GetModelPoint(aMessage.x, aMessage.y);
       GForm.StatusBar.Panels.Items[0].Text := 'x: '+DoubleToStr(-Coord.x, 9, 2);
       GForm.StatusBar.Panels.Items[1].Text := 'y: '+DoubleToStr(-Coord.y, 10, 2);
       GForm.StatusBar.Panels.Items[3].Text := 'ZM10: '+GetZMName(10000, Coord.X, Coord.Y);
    end;
end;



procedure   TGNEditorFormTool.on_tm_ViewBySelect(var Handled: boolean);
begin
{
   T := TRecGuideSelect.Create;
   T.Select10 := not TGuideInterpret(Interpret).Show_5;
   T.Input := Guide;
   T.DoIt;
   R := TRectangle(T.Output);
   if R <> nil then
      Device.Projection.ChangeProjectionByRectangle(R, Device.Page);
   PostMsg(tm_Review, Self, nil, DontKill);
   R.Free;
   T.Free;
 }
end;



procedure   TGNEditorFormTool.on_tm_New(var Handled: boolean);
begin
end;

procedure   TGNEditorFormTool.on_tm_Open(var Handled: boolean);
begin
end;

procedure   TGNEditorFormTool.on_tm_Save(var Handled: boolean);
begin
end;

procedure   TGNEditorFormTool.on_tm_SaveAs(var Handled: boolean);
begin
end;

procedure   TGNEditorFormTool.on_tm_SetupSelect(var Handled: boolean);
var
   E: TRectEntityEditor;
begin
   E := TRectEntityEditor(ActiveTool);
   if not(E is TRectEntityEditor) then exit;
   E.SetupSelect;
end;

procedure   TGNEditorFormTool.on_tm_SelectToTop(var Handled: boolean);
begin
end;

procedure   TGNEditorFormTool.on_tm_SelectToBottom(var Handled: boolean);
begin
end;

procedure   TGNEditorFormTool.on_tm_DeleteSelect(var Handled: boolean);
begin
end;

procedure   TGNEditorFormTool.on_tm_SelectSizeChange(var Handled: boolean);
var
   E: TRectEntityEditor;
   El: TRectEntity;
   D: TDoublerect;
begin
   E := TRectEntityEditor(Tools.FindElementClass(TRectEntityEditor));
   if E = nil then exit;
   El := E.SelectedElement;
   if El is TRectEntity then
   begin
      D := E.Device.Projection.GetModelRect(E.ShiftRect.Left,E.ShiftRect.Top,E.ShiftRect.Right,E.ShiftRect.Bottom);
      SizePanel.SetXMin(D.Minx);
      SizePanel.SetYMin(D.Miny);
      SizePanel.SetXMax(D.Maxx);
      SizePanel.SetYMax(D.Maxy);
   end;
end;




procedure   TGNEditorFormTool.on_tm_SetPropertyEditors(var Handled: boolean);
{
var
   E: TRectEntityEditor;
   El: TRectEntity;
}
begin
{
   E := TRectEntityEditor(Tools.FindElementClass(TRectEntityEditor));
   if E = nil then exit;
   El := E.SelectedElement;
   if El = nil then exit;
   if El is TRectEntity then
   begin
      SizePanel.SetXMin(El.Minx);
      SizePanel.SetYMin(El.Miny);
      SizePanel.SetXMax(El.Maxx);
      SizePanel.SetYMax(El.Maxy);
      SizePanel.ed_Name.Text := El.Name;
   end;
   GForm.l_Comment.Caption := 'Vybran objekt: '+El.Name + ' (tda '+El.ClassName+')';
   if El is TDrawRectangle then
   begin
      GForm.ColorPanel.Color := TDrawRectangle(El).PenColor;
      GForm.PenWidth := TDrawRectangle(El).PenWidth;
      GForm.BrushColorPanel.Color := TDrawRectangle(El).BrushColor;
      if TDrawRectangle(El).BrushStyle = bsClear then GForm.b_Fill.Down := false
                                                 else GForm.b_Fill.Down := true;
   end
   else
   if El is TText then
   begin
      GForm.ColorPanel.Color := TText(El).Color;
      GForm.FontDialog.Font.Name := TText(El).FontName;
      GForm.FontDialog.Font.Color := TText(El).Color;
      GForm.FontPanel.Text := TText(El).FontName;
   end
}
end;

procedure   TGNEditorFormTool.on_tm_SizePanelOn (var Handled: boolean);
begin
end;


procedure   TGNEditorFormTool.on_tm_SizePanelOff (var Handled: boolean);
begin
end;

function  TGNEditorFormTool.GetFileName: string;
begin
end;


procedure TGNEditorFormTool.SetFileName(aValue: string);
begin
end;



procedure   TGNEditorFormTool.SetInput(aInput: TElement);
var
   i: longInt;
begin
   if FInput <> nil then
   begin
      FInput.Free;
      FInput := nil;
   end;
   FInput := aInput;
   try
      if aInput is TRectEntity then
      begin
         Device.Projection.ChangeProjectionByRectangle(TRectEntity(aInput).Rect, Device.Page);
      end;
      Interpret.Input := Input;
   except
   end;
   for i := 0 to Tools.Count -1 do Tool[i].Input := aInput;
end;


procedure   TGNEditorFormTool.CreateForm;
begin
   Application.CreateForm(TgnEditorForm, FForm);
   GForm.Tool := Self;
   GForm.IsPossibleToClose := false;
   {
   Form := TStandardForm(GForm);
   }
end;

procedure   TGNEditorFormTool.LinkDeviceToForm;
begin
   with TImageCanvasDevice(Device) do
   begin
      Bitmap := GForm.Image.Picture.Bitmap;
   end;
end;

procedure   TGNEditorFormTool.CreateInterpret;
begin
   Interpret := TGNInterpret.Create;
end;


procedure   TGNEditorFormTool.CreateTools;
{
var
   E: TMapSheetEditor;
   EI: TEditorIATool;
}
begin
{
   E := TMapSheetEditor.Create;
   NextTool := E;

   EI := TNewDrawRect.Create;
   EI.Editor := E;
   NextTool := EI;

   EI := TNewView.Create;
   EI.Editor := E;
   NextTool := EI;

   EI := TNewText.Create;
   EI.Editor := E;
   NextTool := EI;

   EI := TNewTextBlock.Create;
   EI.Editor := E;
   NextTool := EI;
}

   NextTool := TZoomOut.Create;
   NextTool := TViewer.Create;
   NextTool := TNewTextTool.Create;
   NextTool := TEditTextTool.Create;

   NextTool := TZoomIn.Create;
end;


function    TGNEditorFormTool.GetVisible: boolean;
begin
   try
      result := GForm.Visible;
   except
      result := false;
   end;
end;



procedure   TGNEditorFormTool.SetVisible(aVisible: boolean);
begin
   try
      GForm.Visible := aVisible;
      Device.SetupPage(nil);
   except
   end;
end;


function    TGNEditorFormTool.GetMainMenu: TMainMenu;
begin
   result := GForm.MainMenu;
end;


function    TGNEditorFormTool.GetCaption: string;
begin
   result := GForm.Caption;
end;

procedure   TGNEditorFormTool.SetCaption(const aValue: string);
begin
   GForm.Caption := aValue;
end;


procedure TGNEditorFormTool.UpSetText(aUp: Boolean);
var
  I, N: integer;
  aLX, aLY, aPX, aPY, anAngle: Double;
  anOK: Boolean;
  aTM: TTextMetric;
  aShift: Double;
  QFStyle: Byte;

  Resolution, Mult, TextH: Double;
  aVal: string;
begin
  {$IFNDEF Debug}

  try
    Resolution := 1000 * Screen.PixelsPerInch / 25.4;
    Mult := 10000 / Resolution;
     with TMainTool(World.MainTool).TextDBForm do
     begin
(*
        TableText.SetRange(['03-43-04'],['03-43-04']);
        TableText.DisableControls;
        TableText.First;
        while not TableText.EOF do
*)

        if u_DBlib.orbDataSetIsOpenMode(QueryText) then QueryText.Close;
        QueryText.ParamByName('Start').AsString := '03-43-04';
        QueryText.ParamByName('Konec').AsString := '03-43-04';
        QueryText.Prepared := True;
        orbOpenQuery(QueryText);

        QueryText.DisableControls;
        QueryText.First;
        while not QueryText.EOF do
        begin
           aLX:= LX.AsFloat;
           aLY:= LY.AsFloat;
           aPX:= PX.AsFloat;
           aPY:= PY.AsFloat;
           aVal:= NazevTisk.AsString;
           TextH := FontHeight.AsFloat;
//           TextSiz := round(TextH);
           {PK 22.4.1998}
           TextH := Abs(TextH);
           TextH := TextH * Mult;

           if (aPX=0) or (aPY=0) then
              anAngle:= GetZMAngleForXY(aLX, aLY)
           else
              anAngle:= ATan2M(aPX-aLX, aPY-aLY);

           with TCanvasDevice(Device).Canvas do
           begin
             Font.Name:= FontName.AsString;
             Font.Size:= 100;
             Font.Color:= FontColor.AsInteger;
             QFStyle := FontStyle.AsInteger;
             Font.Style:= TFontStyles(QFStyle);
           end;

           anOK:= GetTextMetrics(TCanvasDevice(Device).Canvas.Handle, aTM);
           aShift:= TextH;
           aShift:= (aShift / aTM.tmHeight) * aTM.tmDescent;

           if aUp then
           begin
             {Posun nahoru }
             PosunSour(aLX, aLY, aShift, (anAngle+Pi/2));

             if (aPX<>0) or (aPY<>0) then
                PosunSour(aPX, aPY, aShift, (anAngle+Pi/2));

           end
           else
           begin
             {Posun dolu }
             PosunSour(aLX, aLY, aShift, -(anAngle+Pi/2));

             if (aPX<>0) or (aPY<>0) then
                PosunSour(aPX, aPY, aShift, - (anAngle+Pi/2));
           end;

//           TableText.Edit;
           QueryText.Edit;
           LX.AsFloat := aLX;
           LY.AsFloat := aLY;

           if (aPX<>0) or (aPY<>0) then
           begin
              PX.AsFloat := aPX;
              PY.AsFloat := aPY;
           end;
{
           TableText.Post;
           TableText.Next;
}
           QueryText.Post;
           QueryText.Next;

        end;
//        TableText.EnableControls;
        QueryText.EnableControls;
      end;
   except
   end;
{$ENDIF Debug}

end;


procedure TGNEditorFormTool.On_tm_Keydown(var aMessage: TOrbisKeyEvent; var Handled: boolean);
begin
   case aMessage.Key of
   VK_ESCAPE: PostMsgToOwners(tm_BreakInterpret, nil, DontKill);
   VK_F8: on_Tm_PrevProjection;
   VK_F7: on_tm_NextProjection;
   VK_F11: UpSetText(True);
   VK_F12: UpSetText(false);
   end;
end;

procedure   TGNEditorFormTool.on_tm_Print(var Handled: boolean);
var
   P: TPrint;
begin
   //Tiled := Form.Tile.Checked;
   //if Interpret is TUniversalInterpret then
      //TUniversalInterpret(Interpret).Tile := false;
   P := TPrint.Create;
   P.Input := Interpret;
   P.DoIt;
   P.Free;
   //if Interpret is TUniversalInterpret then
   //   TUniversalInterpret(Interpret).Tile := false;
end;


procedure TGNEditorFormTool.HandleMessage(var aMessage: TOrbisMessage; var Handled: boolean);
begin
   inherited HandleMessage(aMessage, Handled);
   if Handled then exit;
   case aMessage.Message of
      tm_Keydown        : on_tm_Keydown(TOrbisKeyEvent(aMessage ), Handled);
      tm_ZoomIn         : on_tm_ZoomIn(Handled);
      tm_ZoomOut        : on_tm_ZoomOut(Handled);
      tm_TableTextShow  : on_tm_TableTextShow(Handled);
      tm_Viewer         : on_tm_Viewer(Handled);
      tm_ViewZM         : on_tm_ViewZM(Handled);
      tm_NewView        : on_tm_NewView(Handled);
      tm_NewTextTool    : on_tm_NewTextTool(Handled);
      tm_EditTextTool   : on_tm_EditTextTool(Handled);
      tm_ViewActualText : on_tm_ViewActualText(Handled);
      tm_ViewActualTextTimer : on_tm_ViewActualTextTimer(Handled);

      tm_ShowRAS        : on_tm_ShowRAS(Handled);
      tm_SetRAS          : on_tm_SetRAS(Handled);
      tm_WSpaceSet      : on_tm_WSpaceSet(Handled);
      tm_WSpaceShow     : on_tm_WSpaceShow(Handled);
      tm_WSpaceMove     : on_tm_WSpaceMove(TOrbisTextMessage(aMessage), Handled);
      tm_AddProjection  : on_tm_AddProjection(Handled);

      tm_New            : on_tm_New(Handled);
      tm_Open           : on_tm_Open(Handled);
      tm_Save           : on_tm_Save(Handled);
      tm_SaveAs         : on_tm_SaveAs(Handled);

      tm_SetupSelect    : on_tm_SetupSelect(Handled);
      tm_SelectToTop    : on_tm_SelectToTop(Handled);
      tm_SelectToBottom : on_tm_SelectToBottom(Handled);
      tm_DeleteSelect   : on_tm_DeleteSelect(Handled);
      tm_SelectSizeChange: on_tm_SelectSizeChange(Handled);

      tm_SetPropertyEditors:on_tm_SetPropertyEditors(Handled);
      tm_SizePanelOn      : on_tm_SizePanelOn(Handled);
      tm_SizePanelOff     : on_tm_SizePanelOff(Handled);

      tm_DBEditText       :    on_tm_DBEditText(aMessage, Handled);
   end;
end;


 procedure TGNEditorFormTool.SetRole(aRole: TUserRole);
 begin
   FRole:= aRole;
 end;





END.
