unit Main;

interface

uses
  SysUtils, WinTypes, WinProcs, Messages, Classes, Graphics, Controls,
  Forms, Dialogs, ExtCtrls, StdCtrls, DdeMan,
  u_CkTm;

const
   WM_EndOfInstalation = WM_User + 1;
   WM_ProcessNext      = WM_User + 2;
   WM_ProcessLast      = WM_User + 3;

   c_Caption      = 'CAPTION';
   c_DirsDlg      = 'DIRSDLG';
   c_OptionsDlg   = 'OPTIONSDLG';
   c_CopyFiles    = 'COPYFILES';
   c_DLG          = 'DLG';
   c_BMP          = 'BMP';
   c_INI          = 'INI';
   c_WIN          = 'WIN';
   c_Source       = 'SOURCE';
   c_True         = 'TRUE';
   c_GRP          = 'GRP';
   c_AddToGroup   = 'ADDTOGROUP';
   c_Quest        = 'QUEST';
   c_CombineConst = 'CC';
   c_EXE          = 'EXE';

type


  TDesktop = class(TForm)
    Image: TImage;
    Label1: TLabel;
    Label2: TLabel;
    aCaption: TLabel;
    DdeClientItem: TDdeClientItem;
    DdeClient: TDdeClientConv;
    Image1: TImage;
    procedure FormCreate(Sender: TObject);
    procedure FormShow(Sender: TObject);
  private
     FVarDirs: TStringList;
     FFixDirs: TStringList;
     FOptions: TStringList;
     FProcessScript: TStringList;
     FDisk: TStringList;
     FDiscCount: longInt;
     FProcessRow: longInt;
     FLastDialog: longInt;
     DiskCounts: array[0..10] of word;
     AllCount: word;
     SourceIndex: longInt;
     CKSM: double;
     VarDirsCount: integer;
     RBOName: string;
     RegError: boolean;
     App: TAppRec;
     RBO: TRegistrationInfo;
     FAppInstallInfo: TAppInstallInfo;
     function  CheckIniFile: boolean;
     function  GetVarDirs: TStringList;
     function  GetFixDirs: TStringList;
     function  GetOptions: TStringList;
     function  GetProcess: TStringList;
     function  GetDisk: TStringList;
     function  GetiniFileName: string;
     function  GetSection(const aSection: string): TStringList;
     function  GetDiscCount: longInt;
     function  ReadDisk: TStringList;
     function  IsOption(S: string): boolean;
     function  GetVarDir(AName: string): string;
     function  GetCombineFileName(AName: string): string;
     procedure ReadIniFile;
     procedure DrawDesktop;
     procedure Process;
     procedure ProcessActualRow;
     procedure ProcessDirsDlg;
     procedure ProcessOptionsDlg;
     procedure ProcessBMP(S: string);
     procedure ProcessINI(S: string);
     procedure ProcessCopyFiles;
     procedure WMEndOfInstalation;
     procedure WMProcessNext;
     procedure WMProcessLast;
     procedure Post(aMsg: Word);
     procedure PrepareDirectories;
     procedure JoinDirs;
     procedure ForceDirs;
     procedure FinishWork;
     procedure AddToGroup(S: string);
     procedure Exec(S: string);
     procedure PrepareRBO;
     function  CheckRegistry: boolean;
     procedure StoreUpdateToRegistry;
     procedure SetVarDir(aName: string; const Value: string);
     procedure SetAppInstallInfo(const Value: TAppInstallInfo);
    function GetAppInstallInfo: TAppInstallInfo;
  public
     procedure AppMessage(var Msg: TMsg; var Handled: Boolean);
     property  VarDirs: TStringList read GetVarDirs;
     property  FixDirs: TStringList read GetFixDirs;
     property  Options: TStringList read GetOptions;
     property  Disk: TStringList read GetDisk;
     property  ProcessScript: TStringList read GetProcess;
     property  IniFileName: string read GetIniFileName;
     property  DiscCount: longInt read GetDiscCount;
     property  ProcessRow: longInt read FProcessRow write FProcessRow;
     property  LastDialog: longInt read FLastDialog write FLastDialog;
     property  VarDir[aName: string]: string read GetVarDir write SetVarDir;
     property  AppInstallInfo: TAppInstallInfo read GetAppInstallInfo write SetAppInstallInfo;
  end;


var
  Desktop: TDesktop;

implementation

{$R *.DFM}
uses
   FileCtrl,
   IniFiles,
   LZExpand,
   f_Copy,
   f_DirDlg,
   f_OptDlg,
   u_String;


procedure TDesktop.PrepareRBO;
var
   SR: TSearchRec;
   S: string;
begin
   S := VarDir['ODKUD'];
   if FindFirst(S+'*.RBO', faAnyFile, SR) = 0 then
   begin
      RBOName := S + SR.Name;
   end
   else RBOName := '';
   if RBOName <> '' then RBO := LoadRBOFromFile(RBOName);
   RegError := not CheckRegistry;
   //RBOError := not InstallRBO(RBOName);
end;


function  TDesktop.GetVarDirs: TStringList;
begin
   if FVarDirs = nil then
   begin
      result := GetSection('VarDirs');
      FVarDirs := result;
      VarDirsCount := FVarDirs.Count;
   end
   else result := FVarDirs;
end;

function  TDesktop.GetFixDirs: TStringList;
begin
   if FFixDirs = nil then
   begin
      result := GetSection('FixDirs');
      FFixDirs := result;
   end
   else result := FFixDirs;
end;

function  TDesktop.GetOptions: TStringList;
begin
   if FOptions = nil then
   begin
      result := GetSection('Options');
      FOptions := result;
   end
   else result := FOptions;
end;

function  TDesktop.GetProcess: TStringList;
begin
   if FProcessScript = nil then
   begin
      result := GetSection('Process');
      FProcessScript := result;
   end
   else result := FProcessScript;
end;

function  TDesktop.GetDisk: TStringList;
begin
   if FDisk = nil then
   begin
      result := ReadDisk;
      FDisk := result;
   end
   else result := FDisk;
end;


function  TDesktop.GetiniFileName: string;
begin
   result := copy(ParamStr(0),1,Length(ParamStr(0))-4);
   result := result + '.INI';
end;

function  TDesktop.GetDiscCount: longInt;
var
   Ini: TIniFile;
begin
   Ini := TIniFile.Create(IniFileName);
   result := Ini.ReadInteger('Disks', 'Number', 0);
   Ini.Free;
end;


function  TDesktop.GetSection(const aSection: string): TStringList;
var
   Ini: TIniFile;
begin
   result := TStringList.Create;
   result.Duplicates := dupAccept;
   Ini := TIniFile.Create(IniFileName);
   Ini.ReadSectionValues(aSection, result);
   Ini.Free;
end;


function  TDesktop.ReadDisk: TStringList;
var
   i, j: longInt;
   S: string;
   Groups: TStringList;
   Group: TStringList;
   Param, Val: string;
begin
   result := TStringList.Create;
   AllCount := 0;
   for i := 1 to DiscCount  do
   begin
      S := 'DISK'+IntToStr(i);
      Groups := GetSection(S);
      result.AddObject(S, Groups);
      DiskCounts[i - 1] := 0;
      for j := 0 to Groups.Count -1 do
      begin
         StringAsParameter(Groups.Strings[j], Param, Val);
         Group := GetSection(Param);
         DiskCounts[i - 1] := DiskCounts[i - 1] + Group.Count;
         AllCount := AllCount + Group.Count;
         Groups.Objects[j] := Group;
      end;
   end;
end;


function  TDesktop.IsOption(S: string): boolean;
begin
   if S = c_True then result := true
                 else result := UpperCase(Options.Values[S]) = c_True;
end;


function  TDesktop.GetVarDir(AName: string): string;
var
   S : string;
   L,P: string;
begin
   result := '';
   S := VarDirs.Values[aName];
   if StringAsTwoParams(S, '+', L, P) then
   begin
      S := VarDirs.Values[L];
      CheckBackSlash(S);
      S := S + P;
   end;
   CheckBackSlash(S);
   result := S;
end;

function  TDesktop.GetCombineFileName(AName: string): string;
var
   S : string;
   L,P: string;
begin
   S := '';
   if StringAsTwoParams(aName, '+', L, P) then
   begin
      S := VarDir[L];
      CheckBackSlash(S);
      S := S + P;
   end
   else
   begin
      S := VarDir[AName];
      CheckBackSlash(S);
   end;
   result := S;
end;



procedure TDesktop.ReadIniFile;
var
   Ini: TIniFile;
   S: string;
begin
   if VarDirs       = nil then;
   if FixDirs       = nil then;
   if Options       = nil then;
   if ProcessScript = nil then;
   if Disk          = nil then;
   if DiscCount = 0 then
   begin
      ShowMessage('Chybika ... dn instalan disky nebo poruen soubory .. instalace kon - ahoj');
      FinishWork;
   end;
   Ini := TIniFile.Create(IniFileName);
   S := Ini.ReadString('Application','Name','');
   if S = '' then
   begin
      ShowMessage('Chyb jmno aplikace ... konme - est');
      FinishWork;
   end;
   App.AppName := S;
   S := Ini.ReadString('Application','version','0.0');
   SetAppVersionString(S, App);
   //S := Ini.ReadString('IQ','IQS','0');
   Ini.Free;
   //CKSM := StrToExtended(S);
end;



procedure TDesktop.Process;
begin
   if ProcessRow < 0 then
   begin
      Post(WM_EndOfInstalation);
      exit;
   end;
   if ProcessRow > ProcessScript.Count - 1 then
   begin
      StoreUpdateToRegistry;
      ShowMessage('Instalace byla spn dokonena');
      FinishWork;
   end
   else ProcessActualRow;
end;

procedure TDesktop.WMEndOfInstalation;
begin
   if MessageDlg('Opravdu ukonit instalaci ?', mtConfirmation,[mbYes, mbNo], 0) = mrYes then
      FinishWork
   else
   begin
      if ProcessRow < 0 then Post(WM_ProcessNext)
                        else Process;
   end;
end;

procedure TDesktop.ProcessActualRow;
var
   S, Param, Val: string;
   Pch: array[0..150] of char;
begin
   S := ProcessScript.Strings[ProcessRow];
   if StringAsParameter(ProcessScript.Strings[ProcessRow], Param, Val)then
   begin
      Param := UpperCase(Param);
      if Param = c_Caption then
      begin
         aCaption.Caption := Val;
         Post(WM_ProcessNext);
      end
      else
      if copy(Param,1,3) = c_DLG then
      begin
         Val := UpperCase(Val);
         if Val = c_DirsDlg then ProcessDirsDlg else
         if Val = c_OptionsDlg then ProcessOptionsDlg else
         if Val = c_CopyFiles then ProcessCopyFiles ;
      end
      else
      if copy(Param,1,3) = c_BMP then
      begin
         ProcessBMP(Val);
      end
      else
      if copy(Param,1,3) = c_INI then
      begin
         ProcessINI(Val);
      end
      else
      if copy(Param,1,3) = c_GRP then
      begin
          StrPcopy(Pch,'[CreateGroup("' + Val + '")]');
          DDEClient.ExecuteMacro(Pch, false);
          Post(WM_ProcessNext);
      end
      else
      if copy(Param,1,10) = c_AddToGroup then
      begin
          AddToGroup(val);
      end
      else
      if copy(Param,1,5) = c_Quest then
      begin
          if MessageDlg(Val,mtConfirmation,[mbYes,mbNo],0) = mrYes then Post(WM_ProcessNext)
          else
          begin
             StoreUpdateToRegistry;
             ShowMessage('Instalace byla spn dokonena');
             FinishWork;
          end;
      end
      else
      if copy(Param,1,3) = c_EXE then
      begin
         Exec(Val);
      end
      else Post(WM_ProcessNext);
   end
end;


procedure TDesktop.Exec(S: string);
var
   P: array[0..150] of char;
   aFile: string;
   aDir: string;
   Delimiters: TDelimiters;
   Delimiter: char;
   Pos: longInt;
   aMin: boolean;
begin
   Delimiters := [','];
   aFile := ReadToDelimiterWithoutWS( S, Delimiters, 1, Delimiter, Pos);
   aDir := ReadToDelimiterWithoutWS( S, Delimiters, Pos, Delimiter, Pos);
   aMin := Delimiter = ',';
   aDir := VarDir[aDir];
   RemoveBackSlash(aDir);
   {!ShowMessage('ChDir: '+aDir);}
   ChDir(aDir);
   CheckBackSlash(aDir);
   StrPcopy(P,aDir+aFile);
   try
      if aMin then WinExec(P, SW_MINIMIZE)
              else WinExec(P, SW_SHOW);
   except
   end;
   Post(WM_ProcessNext);
end;

procedure TDesktop.AddToGroup(S: string);
var
   P: array[0..150] of char;
   aFile: string;
   aDir: string;
   aName: string;
   WorkDir: string;
   aGroup: string;
   Delimiters: TDelimiters;
   Delimiter: char;
   Pos: longInt;
begin
   Delimiters := [','];
   aGroup := ReadToDelimiterWithoutWS( S, Delimiters, 1, Delimiter, Pos);
   aFile := ReadToDelimiterWithoutWS( S, Delimiters, Pos, Delimiter, Pos);
   aDir := ReadToDelimiterWithoutWS( S, Delimiters, Pos, Delimiter, Pos);
   aName := ReadToDelimiterWithoutWS( S, Delimiters, Pos, Delimiter, Pos);
   WorkDir := ReadToDelimiterWithoutWS( S, Delimiters, Pos, Delimiter, Pos);
   WorkDir := VarDir[WorkDir];
   aDir := VarDir[aDir];
   StrPcopy(P,'[AddItem('+aDir+aFile+', '+aName+', '+ aDir+aFile+',0,0,0,'+WorkDir+')]');
   DDEClient.ExecuteMacro( P, false);
   Post(WM_ProcessNext);
end;


procedure TDesktop.ProcessDirsDlg;
var
   D: TDirDlg;
   i: integer;
begin
   Application.CreateForm(TDirDlg, D);
   D.Dirs := VarDirs;
   case D.ShowModal of
      mrOK:
      begin
         Post(WM_ProcessNext);
         LastDialog := ProcessRow;
         AppInstallInfo.Dirs.Clear;
         for i := 0 to VarDirsCount - 1 do
         begin
            AppInstallInfo.Dirs.Add(VarDirs.Strings[i]);
         end;
      end;
      mrRetry: Post(WM_ProcessLast);
   else
      Post(WM_EndOfInstalation);
   end;
   D.Free;
end;

procedure TDesktop.ProcessOptionsDlg;
var
   D: TOptDlg;
begin
   Application.CreateForm(TOptDlg, D);
   D.Ops := Options;
   case D.ShowModal of
      mrOK:
      begin
         Post(WM_ProcessNext);
         LastDialog := ProcessRow;
      end;
      mrRetry: Post(WM_ProcessLast);
   else
      Post(WM_EndOfInstalation);
   end;
   D.Free;
end;

procedure TDesktop.ProcessBMP(S: string);
var
   Dir,BMPName,L,T: string;
begin
   Dir := '';
   BMPName := '';
   L := '';
   T := '';
   if S = '' then exit;
   if StringAsTwoParams(S, ',', Dir,BMPName) then
   begin
      if StringAsTwoParams(BMPName, ',', BMPName,L) then
      begin
         StringAsTwoParams(L, ',', L,T);
      end;
   end;
   Dir := VarDir[Dir];
   if BMPName = '' then exit;
   BMPName := Dir + BMPName;
   try
      Image1.Picture.LoadFromFile(BMPName);
   except
   end;
   if L <> '' then Image1.Left := StrToInt(L);
   if T <> '' then Image1.Top  := StrToInt(T);
   Post(WM_ProcessNext);
end;

procedure TDesktop.ProcessINI(S: string);
{ examp: INI1=Kam+Mapper.ini,Options,TemplateDirectory,Kam+Sablony,c}
var
   Ini: TIniFile;
   IniName, Section, aVar, aValue: string;
   constValue: boolean;
   CombineConst: boolean;
begin
   IniName := '';
   Section := '';
   aVar    := '';
   aValue  := '';
   ConstValue := false;
   CombineConst := false;
   if StringAsTwoParams(S, ',', IniName, S) then
      if StringAsTwoParams(S, ',', Section, S) then
         if StringAsTwoParams(S, ',', aVar, S) then
            ConstValue :=  StringAsTwoParams(S, ',', aValue, S);
   if ConstValue then CombineConst := UpperCase(S)= c_CombineConst;
   IniName := GetCombineFileName(IniName);
   RemoveBackSlash(IniName);
   if ConstValue then
   begin
      if CombineConst then
      begin
         StringAsTwoParams(aValue, '+', S, aValue);
         aValue := GetCombineFileName(aValue);
         aValue := S + aValue;
      end;
   end
   else
      aValue := GetCombineFileName(aValue);
   if IniName <> '' then
   if Section <> '' then
   if aVar    <> '' then
   if aValue  <> '' then
   begin
      try
         Ini := TIniFile.Create(IniName);
         Ini.WriteString(Section, aVar, aValue);
      except
      end;
      Ini.Free;
   end;
   Post(WM_ProcessNext);
end;


procedure TDesktop.ProcessCopyFiles;
var
   D: TCopyDlg;
   Res: longInt;
   i, j, k: longInt;
   Groups, Group: TStringList;
   Floppy: boolean;
   Source, Dest: String;
   SourceFile, DestFile: integer;
   S, Param, val: string;
   PCh: array[0..150] of char;
begin
   //LZStart;
   Application.CreateForm(TCopyDlg, D);
   Application.ProcessMessages;
   JoinDirs;
   ForceDirs;
   Source := VarDir['ODKUD'];
   Floppy := (Source[2] = ':') and ( Source[1] in ['A','a','B','b']);
   D.GInstall.MinValue := 0;
   D.GInstall.MaxValue := AllCount;
   for i := 0 to Disk.Count - 1 do
   begin
      if i > 0 then
      begin
         if Floppy then
         begin
            if MessageDlg('Vsute instalan disketu .'+IntToStr(i+1),
               mtConfirmation,[mbOK,mbCancel],0) <> mrOK then
            begin
               //LZDone;
               FinishWork;
               exit;
            end;
         end
         else
         begin
            Source := Copy(Source,1,Length(Source)-6)+'DISK'+IntToStr(i+1)+'\';
         end;
      end;
      D.GDisk.MinValue := 0;
      D.GDisk.MaxValue := DiskCounts[i];
      D.GDisk.Progress := 0;
      D.l_Disk.Caption := 'Disk .' + IntToStr(i+1);
      Groups := TStringList(Disk.Objects[i]);
      for j := 0 to Groups.Count - 1 do
      begin
         Group := TStringList(Groups.Objects[j]);
         StringAsParameter(Groups.Strings[j], Param, Val);
         D.l_Group.Caption := Param;
         if IsOption(Val)then
         begin
            for k := 0 to Group.Count - 1 do
            begin
               StringAsParameter(Group.Strings[k], Param, Val);
               Dest := VarDir[Val];
               S := Source + Param;
               while not FileExists(S) do
               begin
                  if MessageDlg('Vsute instalan disketu .'+IntToStr(i+1)+' nebo disk se souborem '+ Param,
                     mtConfirmation,[mbOK,mbCancel],0) <> mrOK then
                  begin
                     //LZDOne;
                     FinishWork;
                     exit;
                  end;
               end;
               D.Comment.Caption := 'Probh koprovn souboru '+ Param;
               Application.ProcessMessages;
               StrPCopy(Pch, Source + Param);
               SourceFile := _lOpen(Pch,OF_READ);
               DestFile := FileCreate(Dest + Param);
               Res := LZCopy(SourceFile, DestFile);
               if Res < 0 then ShowMessage('Chyba '+ IntToStr(Res));
               LZClose(SourceFile);
               LZClose(DestFile);
               D.GDisk.Progress := D.GDisk.Progress + 1;
               D.GInstall.Progress := D.GInstall.Progress + 1;
            end;
         end
         else
         begin
            D.GDisk.Progress := D.GDisk.Progress + Group.Count;
            D.GInstall.Progress := D.GInstall.Progress + Group.Count;
         end;
         Application.ProcessMessages;
      end;
   end;
   D.Free;
   Post(WM_ProcessNext);
   //LZDone;
end;

procedure TDesktop.WMProcessNext;
begin
   ProcessRow := ProcessRow + 1;
   Process;
end;

procedure TDesktop.WMProcessLast;
begin
   ProcessRow := LastDialog;
   Process;
end;

procedure TDesktop.Post(aMsg: Word);
begin
   PostMessage(Application.Handle, aMsg, 0, 0);
end;

procedure TDesktop.PrepareDirectories;
   procedure Prepare(Dirs: TStringList);
   var
      i: longInt;
      S, Param, Val: string;
      PCH: array[0..144] of char;
   begin
      for i := 0 to Dirs.Count - 1 do
      begin
         StringAsParameter(Dirs.Strings[i], Param, Val);
         Val := UpperCase(Val);
         if Val = c_Source then
         begin
            Dirs.Values[Param] := ExtractFilePath(ParamStr(0));
            SourceIndex := i;
         end
         else
         if Val = c_WIN then
         begin
            GetWindowsDirectory(PCh, 145); { adres windows }
            Dirs.Values[Param] := StrPas(PCh);
         end;
      end;
   end;
begin
   Prepare(VarDirs);
   Prepare(FixDirs);
end;


procedure TDesktop.JoinDirs;
var
   i: longInt;
begin
   for i := 0 to FixDirs.Count - 1 do
   begin
      VarDirs.Add(FixDirs.Strings[i]);
   end;
end;

function isRoot(const S: string): boolean;
begin
   result := (Length(S)= 3) and (S[2] = ':');
end;


procedure TDesktop.ForceDirs;
var
   i: longInt;
   Param, Val: string;
begin
   for i := 0 to VarDirs.Count - 1 do
   begin
      StringAsParameter(VarDirs.Strings[i], Param, Val);
      Val := VarDir[Param];
      if not (i = SourceIndex) then
      if not IsRoot(Val)then
      if not DirectoryExists(Val) then
      begin
         ForceDirectories(Val);
         if not DirectoryExists(Val) then
         begin
            ShowMessage('nelze vytvoit sloku '+Val+'... Instalace bude ukonena !');
            FinishWork;
         end;
      end;
   end;
end;


procedure TDesktop.FinishWork;
var
   i, j: longInt;
   Groups: TStringList;
begin
   VarDirs.Free;
   FixDirs.Free;
   Options.Free;
   for i := 0 to Disk.Count -1 do
   begin
      Groups := TStringList(Disk.Objects[i]);
      for j := 0 to Groups.Count - 1 do Groups.Objects[j].Free;
      Groups.Free;
   end;
   Disk.Free;
   ProcessScript.Free;
   AppInstallInfo.Free;
   Application.Terminate;
end;


procedure TDesktop.FormCreate(Sender: TObject);
var
   i, j, h, v: longInt;
   r, g, b: byte;
begin
   Application.OnMessage := AppMessage;
   Left := 0;
   Top  := 0;
   Width  := Screen.Width;
   Height := Screen.Height;
   LastDialog := -1;
   if not CheckIniFile then
   begin
      ShowMessage('Nemohu nalzt soubor '+ IniFileName);
      Application.Terminate;
      exit;
   end;
   ReadIniFile;
   PrepareDirectories;
   PrepareRBO;
end;



procedure TDesktop.AppMessage(var Msg: TMsg; var Handled: Boolean);
begin
   case Msg.Message of
      WM_User: DrawDesktop;
      WM_ProcessNext: WMProcessNext;
      WM_ProcessLast: WMProcessLast;
      WM_EndOfInstalation: WMEndOfInstalation;
   end;
end;



procedure  TDesktop.DrawDesktop;
var
   i, j, h, v: longInt;
   r, g, b: byte;
   ll,lt: integer;
begin
   Label1.Visible := false;
   Label2.Visible := false;
   aCaption.Visible := false;
   r := 0;
   g := 0;
   b := 0;
   h := Width div 10;
   inc(h);
   v := Height div 10;
   inc(v);
   i := 0;
   Image.Canvas.Brush.Color := clBlack;
   Image.Canvas.Brush.Style := bsSolid;
   Image.Canvas.Rectangle(0,0,Width,Height);
   while i < Width do
   begin
      j := 0;
      g := 0;
      while j < Height do
      begin
         Image.Canvas.Brush.Color := RGB(r,g,b);
         Image.Canvas.Pen.Color   := Image.Canvas.Brush.Color;
         Image.Canvas.Rectangle(i,j,i+h,j+v);
         inc(j,V);
         inc(g,25);
      end;
      inc(i,H);
      inc(b,25);
      Application.ProcessMessages;
   end;
   Label1.Visible := true;
   Label2.Visible := true;
   aCaption.Visible := true;
   ProcessRow := -1;
   Post(WM_ProcessNext);
end;



procedure TDesktop.FormShow(Sender: TObject);
begin
   if RegError then FinishWork
               else Post(WM_User);
end;

function TDesktop.CheckIniFile: boolean;
begin
   result := FileExists(IniFileName);
end;

procedure TDesktop.SetVarDir(aName: string; const Value: string);
begin
   VarDirs.Values[aName] := Value;
end;

function TDesktop.CheckRegistry: boolean;
var
   RBOModul: boolean;
   S,V: string;
   i: integer;
begin
   RBOModul := RBOName <> '';
   result := CheckRegistryForSetup(App,RBO,RBOModul,AppInstallInfo);
   AppInstallInfo.Modul := RBO.Reg.Key;
   for i := 0 to VarDirsCount - 1 do
   begin
      if i = SourceIndex then Continue;
      S := VarDirs.Names[i];
      V := AppInstallInfo.Dirs.Values[S];
      if V <> '' then VarDir[S] := V;
   end;
end;

procedure TDesktop.StoreUpdateToRegistry;
var
   RBOModul: boolean;
begin
   RBOModul := RBOName <> '';
   SetupRegistry(App,RBO,RBOModul,AppInstallInfo);
end;

procedure TDesktop.SetAppInstallInfo(const Value: TAppInstallInfo);
begin
   if FAppInstallInfo <> nil then
   begin
      FAppInstallInfo.Free;
      FAppInstallInfo := nil;
   end;
   FAppInstallInfo := Value;
end;

function TDesktop.GetAppInstallInfo: TAppInstallInfo;
begin
   if FAppInstallInfo = nil then FAppInstallInfo := TAppInstallInfo.Create;
   result := FAppInstallInfo;
end;

end.
