unit T_Main;

interface
uses
  Classes,
  WinTypes,
  // KERNEL
  Elements,
  e_Type,
  Tools,
  // OIL
  t_CmdLn,
  // APP
  t_MainTl,
  // MAIN
  HelpCtx;


const
  DefIniFileName = 'Revizor.INI';
  c_NewWindow = 'NEWWINDOW';
  c_CloseApplication = 'CLOSEAPPLICATION';
  c_Script = 'SCRIPT';
  c_BEGIN = 'BEGIN';
  c_END = 'END';


type

TOptions = class(TBaseOptions)
private
  FDelRAS: TStringListElement;
  FDeleteRAS: boolean;
  {Scripts}
  FStartScript: string;
  FDataSetScript: string;
  FControlSetScript: string;
  FRunScript: string;
  FEndScript: string;
  {Files}
  FINE: string;
  FPaletFile: string;
  {Directories}
  FWorkingDirectory: String;
  FDataDirectory: String;
  FServiceDirectory: String;
  FScriptDirectory: String;
  FFullUseMemory: boolean;
  function  GetDelRAS: TStringListElement;
public
  property   DelRAS: TStringListElement read GetDelRAS;
published
  property   DeleteRAS: boolean read FDeleteRas write FDeleteRAS;
  property   StartScript: string read FStartScript write FStartScript;
  property   DataSetScript: string read FDataSetScript write FDataSetScript;
  property   ControlSetScript: string read FControlSetScript write FControlSetScript;
  property   RunScript: string read FRunScript write FRunScript;
  property   EndScript: string read FEndScript write FEndScript;
  property   WorkingDirectory: string read FWorkingDirectory write FWorkingDirectory;
  property   ServiceDirectory: string read FServiceDirectory write FServiceDirectory;
  property   DataDirectory: string read FDataDirectory write FDataDirectory;
  property   ScriptDirectory: string read FScriptDirectory write FScriptDirectory;
  property   INE: string read FINE write FINE;
  property   PaletFile: string read FPaletFile write FPaletFile;
  property   FullUseMemory: boolean read FFullUseMemory write FFullUseMemory default false;
end;


TMainTool = class(TOldMainTool)
private
  FOptions: TOptions;
  ViewerIndex: integer;
  FSelectDone: boolean;
  FCancelProcess: boolean;
  FScriptReading: boolean;
  FCommandLine: TCommandLine;
  function  GetOptions: TBaseOptions; override;
  function  GetNOptions: TOptions; virtual;
  procedure SetOptions(aValue: TBaseOptions); override;
  procedure SetNOptions(aValue: TOptions); virtual;
  function  GetCommandLine: TCommandLine;
  procedure SetCommandLine(aValue: TCommandLine);
  procedure ExecScript(aFileName: string);

protected
  procedure ReadParamsFromIni; override;
  procedure ReadParams; override;
  procedure WriteParamsToIni; override;
  function  GetWorkingDirectory: string; override;
public
   StartDir: string;
   constructor Create; override;
   destructor  Destroy; override;
   procedure   DoIt; override;
   procedure  on_tm_SetOptions(var Handled: boolean); virtual;
   procedure  on_tm_QuitTask; override;
   procedure  on_tm_NewProject(var Handled: boolean); virtual;
   procedure  On_tm_OpenProject(var Handled: boolean); virtual;
   procedure  On_tm_ImportBlk(var Handled: boolean); virtual;
   procedure  on_tm_SelectDataDirectory(var Handled: boolean); virtual;
   procedure  on_tm_ControlSet(var Handled: boolean); virtual;
   procedure  on_tm_Run(var Handled: boolean); virtual;
   procedure  On_tm_AddView(var aMessage: TOrbisMessage; var Handled: boolean); virtual;
   procedure  On_tm_Explore(var aMessage: TOrbisMessage; var Handled: boolean); virtual;
   procedure  On_tm_ExploreOwners(var aMessage: TOrbisMessage; var Handled: boolean); virtual;
   procedure  on_tm_ShowText(var aMessage: TOrbisMessage; var Handled: boolean); virtual;
   procedure  On_tm_SetPalette(var Handled: boolean); virtual;
   procedure  on_tm_EndOfPrint(var Handled: boolean); virtual;
   procedure  on_tm_EndOfDDEScript; virtual;
   procedure  on_tm_SetDDEResult; virtual;
   procedure  HandleMessage(var aMessage: TOrbisMessage; var Handled: boolean); override;
   procedure  ExecuteMacro(M: TStrings); virtual;
   procedure  AddWorkTools; override;

   property   SelectDone: boolean read FSelectDone write FSelectDone;
   property   CancelProcess: boolean read FCancelProcess write FCancelProcess;
   property   ScriptReading: boolean read FScriptReading write FScriptReading;
published
   property   Options: TOptions read GetNOptions write SetNOptions;
   property   CommandLine: TCommandLine read GetCommandLine write SetCommandLine;
   property   ExecuteScript: string write ExecScript stored false;
end;



implementation
uses
  { Test}

  Controls, Forms, Dialogs, IniFiles, SysUtils, Graphics, WinProcs,
  _IniFile,
  // KERNEL
  M_Orbis,
  u_String,
  Worlds,
  // EXPLORER
  T_Explor,
  // APP
  F_Start,
  // MAIN
  F_Main,
  M_Proj;

{TOptions}

function  TOptions.GetDelRAS: TStringListElement;
begin
   if FDelRas = nil then
   begin
      FdelRas := TStringListElement.Create;
      AddElement(FDelRas);
   end;
   result := FDelRas;
end;



{TMainTool}

constructor TMainTool.Create;
var
   S: string;
begin
  inherited Create;
  getDir(0, S);
  Options.WorkingDirectory := S;
  Options.DataDirectory := S;
end;


destructor  TMainTool.Destroy;
begin
   inherited Destroy;
end;




function  TMainTool.GetOptions: TBaseOptions;
begin
   if FOptions = nil then
   begin
      FOptions := TOptions.Create;
      AddElement(FOptions);
   end;
   result := FOptions;
end;

function  TMainTool.GetNOptions: TOptions;
begin
   result := TOptions(GetOptions);
end;


procedure TMainTool.SetOptions(aValue: TBaseOptions);
begin
   if FOptions <> nil then
   begin
      FreeElement(FOptions);
      FOptions := nil;
   end;
   FOptions := TOptions(aValue);
end;

procedure TMainTool.SetNOptions(aValue: TOptions);
begin
   SetOptions(aValue);
end;

function  TMainTool.GetCommandLine: TCommandLine;
begin
   if FCommandLine = nil then
   begin
      FCommandLine := TCommandLine.Create;
      FCommandLine.Name := 'Hlavn interpret jazyka OIL';
      AddTool(FCommandLine);
   end;
   result := FCommandLine;
end;


procedure TMainTool.SetCommandLine(aValue: TCommandLine);
begin
   if FCommandLine <> nil then
   begin
      FreeTool(FCommandLine);
      FCommandLine := nil;
   end;
   FCommandLine := aValue;
   AddTool(FCommandLine);
end;


procedure TMainTool.ExecScript(aFileName: string);
begin
  CommandLine.FileName:= aFileName;
  CommandLine.DoIt
end;


function  TMainTool.GetWorkingDirectory: string;
var
   S: string;
begin
   S := Options.WorkingDirectory;
   result := CheckBackSlash(S);
end;



procedure TMainTool.DoIt;
var
  OIL: string;
begin
  inherited DoIt;
  Application.HelpFile := ChangeFileExt(ParamStr(0),'.HLP');
  {
  if not Application.HelpJump('Obsah') then
     ShowMessage('Soubor npovdy neexistuje');
  }
  OIL:= GetParamWithExt('.OIL');
  if (OIL <> '') and fileExists(OIL) then
  begin
     ExecuteScript := OIL;
  end
  else ExecuteScript := Options.StartScript;

end;





procedure TMainTool.ReadParamsFromIni;
begin
   inherited ReadParamsFromIni;
   {
   NactiBarvickuZeSouboru(Options.PaletFile);
   }
end;



procedure TMainTool.ReadParams;
begin
   ReadParamsFromIni;
end;




procedure TMainTool.WriteParamsToIni;
begin
   {
   Options.PaletFile := PaletFile;
   }
   inherited WriteParamsToIni;
end;


procedure  TMainTool.on_tm_SetOptions(var Handled: boolean);
//var
   //D: TF_Options;
begin
   Handled := true;
   (*
   Application.CreateForm(TF_Options, D);
   D.cb_DeleteRAS.Checked := Options.DeleteRAS;
   D.cb_FullUseMemory.Checked := Options.FullUseMemory;

   D.ed_Start.Text := Options.StartScript;
   D.ed_DataSet.Text := Options.DataSetScript;
   D.ed_ControlSet.Text := Options.ControlSetScript;
   D.ed_Run.Text := Options.RunScript;
   D.ed_End.Text := Options.EndScript;

   D.ed_Work.Text := Options.WorkingDirectory;
   D.ed_Data.Text := Options.DataDirectory;
   D.ed_EXE.Text := ExecPath;
   D.ed_Service.Text := Options.ServiceDirectory;
   D.ed_Script.Text := Options.ScriptDirectory;
   if D.ShowModal = mrOk then
   begin
      Options.DeleteRAS := D.cb_DeleteRAS.Checked;
      Options.FullUseMemory := D.cb_FullUseMemory.Checked;

      Options.StartScript := D.ed_Start.Text;
      Options.DataSetScript := D.ed_DataSet.Text;
      Options.ControlSetScript := D.ed_ControlSet.Text;
      Options.RunScript := D.ed_Run.Text;
      Options.EndScript := D.ed_End.Text;

      Options.WorkingDirectory := D.ed_Work.Text;
      Options.DataDirectory := D.ed_Data.Text;
      Options.ServiceDirectory := D.ed_Service.Text;
      PathOfLibraries:= Options.ServiceDirectory;
      Options.ScriptDirectory := D.ed_Script.Text;
   end;
   MainForm.b_Directory.Hint := 'Data: ' + Options.DataDirectory;
   D.Free;
   *)
end;


procedure  TMainTool.on_tm_QuitTask;
var
   i: longInt;
begin
   WriteParamsToIni;
   case MessageDlg('Opravdu ukonit aplikaci ?', mtConfirmation, mbYesNoCancel, 0) of
      mrYes:
      begin
        try
           if Options.DeleteRAS then
           for i := 0 to Options.DelRas.Count - 1 do
           begin
              SysUtils.DeleteFile(Options.DelRas.Strings[i]);
           end;
           {
           PostAppMessage(Project.MainHandle, TProjectWorld(World).WM_MODUL_END, 0, 0)
           }
        except
        end;
        Application.Terminate;
      end;
   end;
end;



procedure  TMainTool.On_tm_NewProject(var Handled: boolean);
begin
end;


procedure  TMainTool.On_tm_OpenProject(var Handled: boolean);
//var
   //Loader: TLoader;
begin
   (*
   if MainForm.OpenDialog.Execute then
   begin
      Loader := TLoader.CreateAll(MainForm.OpenDialog.FileName);
      Loader.DoIt;
      World.ObjectModel := TObjectModel(Loader.Output);
      Loader.Free;
   end;
   *)
end;


procedure  TMainTool.On_tm_ImportBlk(var Handled: boolean);
begin

end;

procedure  TMainTool.on_tm_SelectDataDirectory(var Handled: boolean);
begin

end;


procedure  TMainTool.on_tm_ControlSet(var Handled: boolean);
begin
(*
   Handled := true;
   ExecuteScript := Options.ControlSetScript;
*)
end;

procedure  TMainTool.on_tm_Run(var Handled: boolean);
begin
(*
   Handled := true;
   ExecuteScript := Options.RunScript;
*)
end;



procedure  TMainTool.On_tm_AddView(var aMessage: TOrbisMessage; var Handled: boolean);
begin

end;



procedure  TMainTool.On_tm_Explore(var aMessage: TOrbisMessage; var Handled: boolean);
var
   aTool: TExplorer;
begin
    aTool := TExplorer.Create;
    AddTool(aTool);
    if assigned(aTool) then
    begin
      aTool.Input:= aMessage.Data;
      aTool.DoIt
    end;
end;


procedure  TMainTool.On_tm_ExploreOwners(var aMessage: TOrbisMessage; var Handled: boolean);
var
   aTool: TExplorer;
begin
    aTool := TExplorer.Create;
    AddTool(aTool);
    if assigned(aTool) then
    begin
      aTool.Input:= aMessage.Data;
      aTool.ShowOwners := true;
      aTool.Caption := 'Model vlastnk';
      aTool.DoIt
    end;
end;


procedure  TMainTool.On_tm_ShowText(var aMessage: TOrbisMessage; var Handled: boolean);
begin
  if aMessage is TOrbisTextMessage then
     MainForm.StatusBar.Panels.Items[0].Text := TOrbisTextMessage(aMessage).Text;
  Handled:= true
end;




procedure  TMainTool.On_tm_SetPalette(var Handled: boolean);
begin
//  NastavBarvicky
end;






procedure TMainTool.On_tm_EndOfPrint(var Handled: boolean);
begin
end;


procedure TMainTool.On_tm_EndOfDDEScript;
begin
end;


procedure TMainTool.On_tm_SetDDEResult;
begin
   F_StartForm.DDEServerItem.Text := CommandLine.ResultLine;
end;



procedure   TMainTool.HandleMessage(var aMessage: TOrbisMessage; var Handled: boolean);
//var
   //PZ: TRectEntity;
   //Element: TElement;
begin
  {
  MainForm.Ed.Text:= intToStr(memAvail-MainForm.OrigMem);
  }
   Handled:= false; { vse se vyresi na miste }
   inherited HandleMessage(aMessage, Handled);
   if Handled then exit;
   case aMessage.Message of
     { SOUBOR }
      tm_NewProject:  on_tm_NewProject(Handled);
      tm_OpenProject: on_tm_OpenProject(Handled);
      tm_ImportBlk:   on_tm_ImportBlk(Handled);
      tm_SelectDataDirectory: on_tm_SelectDataDirectory(Handled);
      tm_ClearModel:;
      tm_SetOptions:  on_tm_SetOptions(Handled);
      tm_ControlSet:  on_tm_ControlSet(Handled);
      tm_Run:         on_tm_Run(Handled);
     { Mapov list }
      //tm_MapSheetList: PostMsg(tm_View, Self, TEntity(World.ObjectModel).FindElementClass(TMapSheets), DontKill);


     { ZOBRAZOVANI }
      tm_AddView: On_tm_AddView(aMessage, Handled);
      tm_Explore: On_tm_Explore(aMessage, Handled);
      tm_ExploreOwners: On_tm_ExploreOwners(aMessage, Handled);
{      tm_About: On_tm_About(Handled); }
      tm_SetPalette: On_tm_SetPalette(Handled);

      tm_EndOfPrint       : on_tm_EndOfPrint(Handled);
      tm_EndOfDDEScript   : on_tm_EndOfDDEScript;
      tm_SetDDEResult     : on_tm_SetDDEResult;
      tm_ShowText         : on_tm_ShowText(aMessage, Handled)
   end;
end;

procedure  TMainTool.ExecuteMacro(M: TStrings);
begin
   if M.Count < 1 then exit;
   if ScriptReading then
   begin
      if UpperCase(BezVnejsichMezer(M.Strings[0])) = c_END then on_tm_EndOfDDEScript
      else
      begin
         CommandLine.Line := '';
         CommandLine.Lines.Clear;
         CommandLine.Lines.Assign(M);
         CommandLine.DoIt;
         PostMsg(tm_SetDDEResult, Self, nil, DontKill);
      end;
   end
   else
   if UpperCase(M.Strings[0]) = c_Script then
   begin
      ScriptReading := true;
      MainForm.StatusBar.Panels.Items[0].Text := 'ORBIS zpracovv DDE Script';
   end;
end;


procedure  TMainTool.AddWorkTools;
begin
//  AddTool(TDeskTopSaveToText.Create);
//  AddTool(TViewScriptEditor.Create);
end;



END.
