unit F_CompareDBF;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, Grids, DBGrids, Db, DBTables, ExtCtrls;

type
  TCompareDBFForm = class(TForm)
    SourceQuery: TQuery;
    SourceDataSource: TDataSource;
    TargetQuery: TQuery;
    TargetDataSource: TDataSource;
    LeftPanel: TPanel;
    Panel4: TPanel;
    GroupBox2: TGroupBox;
    Panel2: TPanel;
    lbTarget: TLabel;
    TargetDBGrid: TDBGrid;
    Panel3: TPanel;
    btViewAll: TButton;
    btSum: TButton;
    GroupBox1: TGroupBox;
    Panel1: TPanel;
    lbSource: TLabel;
    SourceDBGrid: TDBGrid;
    Splitter1: TSplitter;
    RightPanel: TPanel;
    Splitter2: TSplitter;
    Panel7: TPanel;
    Panel5: TPanel;
    Ulozit: TButton;
    Memo: TMemo;
    SaveDialog: TSaveDialog;
    procedure btViewAllClick(Sender: TObject);
    procedure btSumClick(Sender: TObject);
    procedure UlozitClick(Sender: TObject);
  private
    FKU, FSKUPar: string;
    FTargetTableName: string;
    FSourceTableName: string;
    procedure SetSourceTableName(const Value: string);
    procedure SetTargetTableName(const Value: string);
    procedure AddErr(aMsg: string);
  public
    { Public declarations }
    function GetIni(Item, Default: string): string;

  published
    property SourceTableName: string read FSourceTableName write SetSourceTableName;
    property TargetTableName: string read FTargetTableName write SetTargetTableName;

  end;

var
  CompareDBFForm: TCompareDBFForm;

implementation

{$R *.DFM}
uses
  M_VyrZak;

{ TCompareDBFForm }

function TCompareDBFForm.GetIni(Item, Default: string): string;
var
  aGroup: string;
begin
  aGroup:= 'Compare_'+changeFileExt(extractFileName(SourceTableName), '') + '_' +
           changeFileExt(extractFileName(TargetTableName), '');
  result:= ReadIniStringG(aGroup, Item, Default);
  WriteIniStringG(aGroup, Item, result);
end;




procedure TCompareDBFForm.btViewAllClick(Sender: TObject);
begin
  with SourceQuery do
  begin
    SQL.Clear;
    SQL.Add(GetIni('ViewAllSource', 'SELECT * FROM'));
    SQL.Add('"' + SourceTableName + '";');
    Open
  end;


  with TargetQuery do
  begin
    SQL.Clear;
    SQL.Add(GetIni('ViewAllTarget', 'SELECT * FROM'));
    SQL.Add('"' + TargetTableName + '";');
    Open
  end;
end;


procedure TCompareDBFForm.btSumClick(Sender: TObject);
var
  aVym: double;
begin
  with SourceQuery do
  begin
    SQL.Clear;
    SQL.Add(GetIni('SelectAndSum_Source', 'SELECT KU_KOD,SKUPAR_CIS,SUM(PARVYM) AS SUM_OF_PAR FROM'));
    SQL.Add('"' + SourceTableName + '"');
    SQL.ADD(GetINI('GroupBy_Source', 'GROUP BY KU_KOD,SKUPAR_CIS;'));
    Open
  end;

  SourceQuery.First;

  while not SourceQuery.EOF do
  begin
    FKU:= SourceQuery.FieldByName('KU_KOD').AsString;
    if FKU = '' then
    begin
      AddErr('Ve zdroji je przdn poloka KU_KOD');
      SourceQuery.Next;
      continue;
    end;

    FSKUPar:= SourceQuery.FieldByName('SKUPAR_CIS').AsString;
    if FSKUPar = '' then
    begin
      AddErr('Ve zdroji je przdn poloka SKUPAR_CIS');
      SourceQuery.Next;
      continue;
    end;

    if FSKUPar = '0' then
    begin
      AddErr('Ve zdroji m poloka SKUPAR_CIS nulovou hodnotu');
      SourceQuery.Next;
      continue;
    end;

    aVym:= SourceQuery.FieldByName('SUM_OF_PAR').AsFloat;
//    aVym := SourceQuery.Fields.FieldByNumber(3).AsFloat;

    if aVym = 0 then
    begin
      AddErr('Ve zdroji m poloka SUM_OF_PAR nulovou hodnotu');
      SourceQuery.Next;
      continue;
    end;

    TargetQuery.RequestLive:= true;
    TargetQuery.SQL.Clear;
    TargetQuery.SQL.Add('SELECT KU_KOD, SKUPAR_CIS, SKUPAR_VYM FROM');
    TargetQuery.SQL.Add('"' + TargetTableName + '"');
    TargetQuery.SQL.Add('WHERE KU_KOD=' + FKU + ' AND SKUPAR_CIS=' + FSkuPar +';');
    TargetQuery.SQL.SaveToFile('SUMPARC.SQL');
    TargetQuery.Open;

    if TargetQuery.RecordCount=0 then
       AddErr('neexistuje zznam v cli')
    else
    if TargetQuery.RecordCount>1 then
    begin
      AddErr('existuje v cli vce zznam');
    end
    else
    begin
      TargetQuery.Edit;
      TargetQuery.FieldByName('SKUPAR_VYM').AsFloat:= aVym;
      TargetQuery.Post;
    end;
    TargetQuery.Close;

    SourceQuery.Next;
  end;


end;

procedure TCompareDBFForm.SetSourceTableName(const Value: string);
begin
  FSourceTableName := Value;
  lbSource.Caption:= value;
end;

procedure TCompareDBFForm.SetTargetTableName(const Value: string);
begin
  FTargetTableName := Value;
  lbTarget.Caption:= Value
end;

procedure TCompareDBFForm.UlozitClick(Sender: TObject);
begin
  if SaveDialog.Execute then
     Memo.Lines.SaveToFile(SaveDialog.FileName)
end;

procedure TCompareDBFForm.AddErr(aMsg: string);
begin
  aMsg:= 'Pro KU: "' + FKu + '", SkuPar: "' + FSkuPar + '" - ' + aMsg;
  Memo.Lines.Add(aMsg);
end;

end.
