unit f_laypan;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  ExtCtrls, StdCtrls,
// KERNEL
  Elements,
  Tools,
// INTERPRET
  E_LayCon;

type
  TDoForEach = procedure (aPanel: TPanel);

  TLayersForm = class(TForm)
    ScrollBox: TScrollBox;
    Panel1: TPanel;
    Panel2: TPanel;
    btAddNew: TButton;
    CheckBox1: TCheckBox;
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure ScrollBoxDragOver(Sender, Source: TObject; X, Y: Integer;
      State: TDragState; var Accept: Boolean);
    procedure btAddNewClick(Sender: TObject);
    procedure Panel2DragOver(Sender, Source: TObject; X, Y: Integer;
      State: TDragState; var Accept: Boolean);
  private
    StartedDrag: Boolean;
    DragDif: integer;
    LockMouseMove: Boolean;
    procedure PanelMouseMove(Sender: TObject; Shift: TShiftState; X,
      Y: Integer);
    procedure PanelDblClick(Sender: TObject);
    procedure PanelStartDrag(Sender: TObject;
      var DragObject: TDragObject);
    procedure PanelDragOver(Sender, Source: TObject; X, Y: Integer;
      State: TDragState; var Accept: Boolean);
    procedure PanelDragDrop(Sender, Source: TObject; X, Y: Integer);
    procedure ForEachDoIncr(Something: TDoForEach);
    procedure ForEachDoDecr(Something: TDoForEach);
  public
    { Public declarations }
    Tool: TTool;
  end;


  TLayerPanel = class(TPanel)
  private
    FLayer: TLayerContainer;
    FLayerSwitch: TCheckBox;
    procedure SetLayer(const Value: TLayerContainer);
    procedure CheckBoxClick(Sender: TObject);
    procedure AddLayerSwitch;
    function GetInterpretClass: string;
    procedure SetInterpretClass(const Value: string);
    function GetInterpretedClass: string;
    procedure SetInterpretedClass(const Value: string);
    function GetSwitchOff: Boolean;
    procedure SetSwitchOff(const Value: Boolean);
  public
    constructor Create(AOwner: TComponent); override;
  published
    property SwitchedOff: Boolean read GetSwitchOff write SetSwitchOff;
    property Layer: TLayerContainer read FLayer write SetLayer;
    property InterpretClass: string read GetInterpretClass write SetInterpretClass;
    property InterpretedClass: string read GetInterpretedClass write SetInterpretedClass;
  end;


  TLayersPanel = class(TTool)
  private
    aForm: TLayersForm;
    FLayers: TEntity;
    procedure AddLayers;
    function GetPanelCount: integer;
    function GetLayerPanel(aIndex: integer): TLayerPanel;
    function GetLayers: TEntity;
  public
    procedure AddPanel(aLayer: TLayerContainer);
    procedure AddLayerDialog;
    procedure UpdateLayers;
    procedure DoIt;
    property Panel[aIndex: integer]: TLayerPanel read GetLayerPanel;
  published
    property PanelCount: integer read GetPanelCount;
    property Layers: TEntity read GetLayers;
  end;

implementation

{$R *.DFM}
uses
// KERNEL
  E_Class,
// INTERPRET
  I_Basic,
// GLIB
  G_Elems,
// GISLAYER
  F_LayImp,
  T_LayScr,
  T_LMCre,
  F_EdLay;

procedure SetTop(aPanel: TPanel);
begin
  aPanel.Align:= alTop;
end;

procedure SetNoTop(aPanel: TPanel);
begin
  aPanel.Align:= alNone;
end;


procedure ShowRes(aPanel: TPanel);
begin
  ShowMessage(aPanel.Caption);
end;


                  { TLayerPanel }

procedure TLayerPanel.CheckBoxClick(Sender: TObject);
begin
  if Layer = nil then
     Exit;
  Layer.SwitchedOff:= not FLayerSwitch.Checked;
end;


procedure TLayerPanel.SetLayer(const Value: TLayerContainer);
begin
  FLayer := Value;
  Caption:= Value.LayerType + ': ' + Value.Name;
  FLayerSwitch.Checked:= not Layer.SwitchedOff;
end;


procedure TLayerPanel.AddLayerSwitch;
begin
  FLayerSwitch:= TCheckBox.Create(Self);
  FLayerSwitch.Top:= 5;
  FLayerSwitch.Left:= 5;
  FLayerSwitch.Width:= 15;
  FLayerSwitch.Height:= 15;
  FLayerSwitch.Checked:= true;
  FLayerSwitch.Parent:= Self;
  FLayerSwitch.Caption:= '';
  FLayerSwitch.OnClick:= CheckBoxClick;
end;



constructor TLayerPanel.Create;
begin
  inherited;
  AddLayerSwitch;
  Align:= alTop;
  DragMode:= dmAutomatic;
  Height:= 25;
end;


function TLayerPanel.GetInterpretClass: string;
begin
  Result:= '';
  if Layer = nil then
     Exit;
  if Layer.Interpret = nil then
     Exit;
  Result:= Layer.Interpret.ClassName;
end;


procedure TLayerPanel.SetInterpretClass(const Value: string);
begin
  if Layer = nil then
     Exit;
  if Layer.Interpret <> nil then
  begin
    if Layer.Interpret.ClassName = Value then
       Exit;
    Layer.Interpret.Free;
  end;
  Layer.Interpret:= TInterpret(CreateElementByName(Value));
end;

function TLayerPanel.GetInterpretedClass: string;
begin
  Result:= '';
  if Layer = nil then
     Exit;
  Result:= Layer.InterpretedClass.ClassName;
end;

procedure TLayerPanel.SetInterpretedClass(const Value: string);
begin
  if Layer = nil then
     Exit;
  Layer.InterpretedClass:= TElementClass(GetOrbisClass(Value))
end;

function TLayerPanel.GetSwitchOff: Boolean;
begin
  Result:= not FLayerSwitch.Checked
end;

procedure TLayerPanel.SetSwitchOff(const Value: Boolean);
begin
  FLayerSwitch.Checked:= not Value;
end;

{ TLayersPanel }

procedure TLayersPanel.AddLayers;
var
  I: Integer;
  aLayer: TLayerContainer;
begin
  for I:= 0 to FLayers.Count-1 do
  begin
    aLayer:= TLayerContainer(FLayers.Elements[I]);
    AddPanel(aLayer)
  end
end;


procedure TLayersPanel.UpdateLayers;
var
  aPanel: TLayerPanel;
  aLayer: TEntity;
  I, OldI, NewI: Integer;
  auxLayers: TEntity;
begin
  auxLayers:= TEntity.Create;
  for I:= 0 to Layers.Count-1 do
  begin
    aLayer:= TEntity(Layers.Elements[I]);
    auxLayers.AddElement(aLayer);
  end;

  Layers.FreeAll;

  for I:= 0 to PanelCount-1 do
  begin
    aPanel:= Panel[I];
    aLayer:= aPanel.Layer;
    OldI:= auxLayers.IndexOf(aLayer);
    if OldI > -1 then
       auxLayers.RemoveElement(aLayer);
    Layers.AddElement(aLayer);
  end;

end;

procedure TLayersPanel.DoIt;
begin
  FLayers:= TEntity(input);
  if not (FLayers is TEntity) then
     FLayers:= nil;

  aForm:= TLayersForm.Create(Application);
  aForm.Tool:= Self;
  if input = nil then
  begin
    AddPanel(nil);
    AddPanel(nil);
    AddPanel(nil);
  end
  else
  begin
    AddLayers;
  end;
  aForm.Show;
end;


procedure TLayersPanel.AddPanel(aLayer: TLayerContainer);
var
  aPanel: TLayerPanel;
  I: integer;
  aTop: integer;
begin
  aTop:= 5;
  for I:= 0 to aForm.ScrollBox.ControlCount-1 do
  begin
    aPanel:= TLayerPanel(aForm.ScrollBox.Controls[i]);
    if not (TObject(aPanel) is TPanel) then
       continue;
    aTop:= aPanel.Top+aPanel.Height;
  end;

  aPanel:= TLayerPanel.Create(aForm);
  aPanel.Layer:= aLayer;
  aPanel.DragMode:= dmManual;
  aPanel.OnMouseMove:= aForm.PanelMouseMove;
  aPanel.OnDblClick:= aForm.PanelDblClick;
  aPanel.OnStartDrag:= aForm.PanelStartDrag;
  aPanel.OnDragOver:= aForm.PanelDragOver;
  aPanel.OnDragDrop:= aForm.PanelDragDrop;

  aPanel.Parent:= aForm.ScrollBox;
end;



function TLayersPanel.GetPanelCount: integer;
begin
  Result:= 0;
  if aForm = nil then
     Exit;
  Result:= aForm.ScrollBox.ControlCount
end;


function TLayersPanel.GetLayerPanel(aIndex: integer): TLayerPanel;
begin
  Result:= nil;
  if aForm = nil then
     Exit;

  Result:= TLayerPanel(aForm.ScrollBox.Controls[aIndex]);
  if not (Result is TLayerPanel) then
     Result:= nil
end;

function TLayersPanel.GetLayers: TEntity;
begin
  Result:= FLayers;
end;




procedure TLayersPanel.AddLayerDialog;
var
  DD: TLayerImportDialog;
  MSC: TModelScriptCreator;
  MBSC: TModelByScriptCreator;
  aLC: TLayerContainer;
begin
   DD := TLayerImportDialog.Create;
   MSC := TModelScriptCreator.Create;
   MBSC := TModelByScriptCreator.Create;
   try
//      DD.InitialDir := Options.DataDirectory;
      DD.DoIt;
      MSC.Input := DD.Output;
//      MSC.DataDirectory := Options.DataDirectory;
      MSC.DoIt;
      MBSC.Input := MSC.Output;
      MBSC.DoIt;
      if not (MBSC.Output is TRectEntity) then
         Exit;
      MBSC.Output.Name := 'NONAME';
      Layers.AddElement(MBSC.Output);
      aLC:= TLayerContainer.Create;
      aLC.AddAndUpdateRect(MBSC.Output);
      AddPanel(aLC);
   except
   end;
   DD.Free;
   MSC.Free;
   MBSC.Free;
end;


{ TLayersForm }

procedure TLayersForm.ForEachDoIncr(Something: TDoForEach);
var
  I: integer;
  aPanel: TPanel;
begin
  for I:= 0 to ScrollBox.ControlCount-1 do
  begin
    aPanel:= TPanel(ScrollBox.Controls[i]);
    if not (TObject(aPanel) is TPanel) then
       continue;
    Something(aPanel);
  end;
end;


procedure TLayersForm.ForEachDoDecr(Something: TDoForEach);
var
  I: integer;
  aPanel: TPanel;
  aTop: integer;
begin
  for I:= ScrollBox.ControlCount-1 downto 0 do
  begin
    aPanel:= TPanel(ScrollBox.Controls[i]);
    aTop:= aPanel.Top;
    if not (TObject(aPanel) is TPanel) then
       continue;
    Something(aPanel);
    aPanel.Top:= aTop
  end;
end;


procedure TLayersForm.FormClose(Sender: TObject; var Action: TCloseAction);
begin
  Tool.Free
end;

procedure TLayersForm.PanelStartDrag(Sender: TObject;
  var DragObject: TDragObject);
begin
  if not (Sender is TPanel) then
     Exit;
  ForEachDoDecr(SetNoTop);
  ScrollBox.RemoveControl(TPanel(Sender));
  ScrollBox.InsertControl(TPanel(Sender));
  TPanel(Sender).BevelInner:= bvLowered;
  StartedDrag:= True;
end;

procedure TLayersForm.PanelDragOver(Sender, Source: TObject; X,
  Y: Integer; State: TDragState; var Accept: Boolean);
begin
  if not (Sender is TPanel) then
     Exit;

  if not (Source is TPanel) then
     Exit;

  if Sender <> Source then
  begin
    TPanel(Source).Top:= TPanel(Sender).Top;
    Exit;
  end;

  if StartedDrag then
  begin
    StartedDrag:= False;
    DragDif:= Y;
    Exit;
  end;

  TPanel(Source).Top:= TPanel(Source).Top + Y-DragDif;

  Accept:= true
end;

procedure TLayersForm.ScrollBoxDragOver(Sender, Source: TObject; X,
  Y: Integer; State: TDragState; var Accept: Boolean);
begin
  if not (Source is TPanel) then
     Exit;
  TPanel(Source).Top:= Y-DragDif;
  Accept:= False;
end;

{
procedure TLayersForm.ZaradSe(aPanel: TPanel);
begin
  for I:= 0 to aForm.ScrollBox.ControlCount-1 do
  begin
    aPanel:= TPanel(aForm.ScrollBox.Controls[i]);
    if not (TObject(aPanel) is TPanel) then
       continue;
    aTop:= aPanel.Top+aPanel.Height;
  end;
end;
}



procedure TLayersForm.PanelDragDrop(Sender, Source: TObject; X,
  Y: Integer);
begin
  if not (Source is TPanel) then
     Exit;
  if Sender <> Source then
     Exit;
  TPanel(Sender).BevelInner:= bvRaised;
  ForEachDoIncr(SetTop);
  TLayersPanel(Tool).UpdateLayers;
end;


procedure TLayersForm.PanelMouseMove(Sender: TObject; Shift: TShiftState;
  X, Y: Integer);
begin
  if not (Sender is TLayerPanel) then
     Exit;
  if (ssLeft in Shift) then
  begin
    if not LockMouseMove then
       TLayerPanel(Sender).BeginDrag(False, 2);
    LockMouseMove:= false;
  end
end;


procedure TLayersForm.PanelDblClick(Sender: TObject);
var
  aForm: Tf_edLayForm;
begin
  if not (Sender is TLayerPanel) then
     Exit;
  aForm:= Tf_edLayForm.Create(Application);
  aForm.Layer:= TLayerPanel(Sender).Layer;
  aForm.Show;
  aForm.FPanel:= TLayerPanel(Sender);
  aForm.Caption:= TLayerPanel(Sender).Caption;
  aForm.EdLayName.Text:= aForm.Layer.Name;
  LockMouseMove:= True;
end;


procedure TLayersForm.btAddNewClick(Sender: TObject);
begin
  TLayersPanel(Tool).AddLayerDialog
end;


procedure TLayersForm.Panel2DragOver(Sender, Source: TObject; X,
  Y: Integer; State: TDragState; var Accept: Boolean);
begin
  Accept:= False;

  if not (Source is TLayerPanel) then
     Exit;

  Accept:= true
end;




BEGIN
   RegisterOrbisClass(TLayersPanel);
end.
