unit ImagePanel;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  ExtCtrls,
  Menus;

type
  TMouseEventType = (meMouseMove, meMouseDown, meMouseUp,
                     meStartDrag, meDragOver, meDragDrop, meEndDrag);

  TImagePanel = class(TPanel)
  private
    FJPGCompressionQuality: integer;
    FGrayscale: boolean;
    FJPGPixelFormat: integer;
    FPixelX: integer;
    FPixelY: integer;
    procedure FOnClick(Sender: TObject);
    procedure FOnDblClick(Sender: TObject);

    function GetCanvas: TCanvas;
  protected
    procedure FOnMouseMove(Sender: TObject; Shift: TShiftState; X, Y: Integer); virtual;
    procedure FOnMouseDown(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer); virtual;
    procedure FOnMouseUp(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer); virtual;

    procedure DoOnMouseEvent(MouseEventType: TMouseEventType; Button: TMouseButton; Shift: TShiftState; X, Y: integer); virtual;

    procedure FOnStartDrag(Sender: TObject; var DragObject: TDragObject); virtual;
    procedure FOnDragOver(Sender, Source: TObject; X, Y: integer; State: TDragState; var Accept: boolean); virtual;
    procedure FOnDragDrop(Sender, Source: TObject; X, Y: integer); virtual;
    procedure FOnEndDrag(Sender, Source: TObject; X, Y: integer); virtual;
  public
    { Public declarations }
    FImage: TImage;
    procedure Resize; override;
    constructor Create(AOwner: TComponent); override;
    property Image: TImage read FImage write FImage;
    property Canvas: TCanvas read GetCanvas;
  published
    { Published declarations }

    procedure Clear;

    procedure ExportToJPGFile(aVal: string);
    procedure ExportToBMPFile(aVal: string);

    property PixelX: integer read FPixelX write FPixelX;
    property PixelY: integer read FPixelY write FPixelY;

    property JPGCompressionQuality: integer read  FJPGCompressionQuality write FJPGCompressionQuality;
    { kvalita 1..100 }
    property JPGGrayscale: boolean read FGrayscale write FGrayscale;
    property JPGPixelFormat: integer read FJPGPixelFormat write FJPGPixelFormat;
    { 0 .. 24-bitov, 1..8-bitov }
  end;

procedure Register;

implementation
uses
  JPEG;

procedure TImagePanel.FOnClick(Sender: TObject);
begin
  if assigned(self.onClick) then
  self.OnClick(Sender);
end;

procedure TImagePanel.FOnDblClick(Sender: TObject);
begin
  if assigned(self.onDblClick) then
  self.OnDblClick(Sender);
end;


procedure TImagePanel.DoOnMouseEvent(MouseEventType: TMouseEventType; Button: TMouseButton; Shift: TShiftState; X, Y: integer);
begin
end;


procedure TImagePanel.FOnMouseDown(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
begin
  DoOnMouseEvent(meMouseDown, Button, Shift, X, Y);

  if assigned(self.OnMouseDown) then
  self.OnMouseDown(Sender, Button, Shift, X, Y);
end;

procedure TImagePanel.FOnMouseMove(Sender: TObject; Shift: TShiftState; X, Y: Integer);
begin
  FPixelX:= X;
  FPixelY:= Y;
  DoOnMouseEvent(meMouseMove, mbLeft, Shift, X, Y);
  if assigned(self.OnMouseMove) then
  self.OnMouseMove(Sender, Shift, X, Y);
end;

procedure TImagePanel.FOnMouseUp(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
begin
  DoOnMouseEvent(meMouseUp, Button, Shift, X, Y);
  if assigned(self.OnMouseUp) then
  self.OnMouseUp(Sender, Button, Shift, X, Y);
end;

procedure TImagePanel.FOnStartDrag(Sender: TObject; var DragObject: TDragObject);
begin
  DoOnMouseEvent(meStartDrag, mbLeft, [], 0, 0);
  if assigned(self.OnStartDrag) then
  self.OnStartDrag(Sender, DragObject);
end;

procedure TImagePanel.FOnDragOver(Sender, Source: TObject; X, Y: integer; State: TDragState; var Accept: boolean);
begin
  DoOnMouseEvent(meDragOver, mbLeft, [], X, Y);
  if assigned(self.OnDragOver) then
  self.OnDragOver(Sender, Source, X, Y, State, Accept);
end;

procedure TImagePanel.FOnDragDrop(Sender, Source: TObject; X, Y: integer);
begin
  DoOnMouseEvent(meDragDrop, mbLeft, [], X, Y);
  if assigned(self.onDragDrop) then
  self.OnDragDrop(Sender, Source, X, Y);
end;

procedure TImagePanel.FOnEndDrag(Sender, Source: TObject; X, Y: integer);
begin
  DoOnMouseEvent(meEndDrag, mbLeft, [], X, Y);
  if assigned(self.OnEndDrag) then
  self.OnEndDrag(Sender, Source, X, Y);
end;



constructor TImagePanel.Create;
begin
  inherited;

  FImage:= TImage.Create(self);
  FImage.Name:= 'Image';
  FImage.Parent:= self;
  FImage.Align:= alClient;
  FImage.OnClick:= FOnClick;
  FImage.DragMode:= dmManual;

  FImage.OnDblClick  := FOnDblClick;
  FImage.OnDragDrop  := FOnDragDrop;
  FImage.OnDragOver  := FOnDragOver;

  FImage.OnEndDrag   := FOnEndDrag;
  FImage.OnMouseDown := FOnMouseDown;
  FImage.OnMouseMove := FOnMouseMove;
  FImage.OnMouseUp   := FOnMouseUp;
  FImage.OnStartDrag := FOnStartDrag;
  FImage.DragCursor  := CrCross;
end;




function TImagePanel.GetCanvas: TCanvas;
begin
  result:= FImage.Canvas;
end;


procedure TImagePanel.Clear;
begin
  Canvas.Pen.Style := psClear;
  Canvas.Brush.Color := clWhite;
  Canvas.Brush.Style := bsSolid;
  Canvas.Rectangle(0, 0, Image.Width, Image.Height);
end;



procedure TImagePanel.Resize;
begin
  inherited;

  Image.Picture.Bitmap.Height:= Height;
  Image.Picture.Bitmap.Width:= Width;

end;



procedure TImagePanel.ExportToJPGFile(aVal: string);
var
  aJPG: TJPEGImage;
begin
  aJPG:= TJPEGImage.Create;
  if JPGCompressionQuality > 0 then
  begin
    if JPGCompressionQuality > 100 then
       JPGCompressionQuality:= 100;
    aJPG.CompressionQuality:= JPGCompressionQuality;
  end;
  aJPG.GrayScale:= JPGGrayscale;
  if JPGPixelFormat > 1 then
     JPGPixelFormat:= 1;

  aJPG.PixelFormat:= TJPEGPixelFormat(JPGPixelFormat);

  aJPG.Assign(Image.Picture.Bitmap);
  aJPG.SaveToFile(aVal);
  aJPG.Free
end;


procedure TImagePanel.ExportToBMPFile(aVal: string);
begin
  Image.Picture.Bitmap.SaveToFile(aVal);
end;

{
procedure TImagePanel.ExportToGIFFile(aVal: string);
var
  aJPG: TJPEGImage;
begin
  aJPG:= TJPEGImage.Create;
  aJPG.Assign(Image.Picture.Bitmap);
  aJPG.SaveToFile(aVal);
  aJPG.Free
end;
}

procedure Register;
begin
  RegisterComponents('Orbis', [TImagePanel]);
end;

end.

