unit T_Main;

interface
uses
  Classes,
  WinTypes,
  ExtCtrls,
// KERNEL
  Elements,
  e_Type,
  HelpCtx,
  Tools,
// DEVICE
  d_txt,
// OIL
  t_CmdLn,
// GLIB
  g_Elems,
// INTERPRET
  e_InterpretContainer,
// GISLAYAER
  f_LayImp,
  t_LayScr,
  t_LMCre,
// DEVICE
  e_Proje,
// APP
  t_MainTl,
// .
  t_ImgTol;


const
  DefIniFileName = 'Revizor.INI';
  c_NewWindow = 'NEWWINDOW';
  c_CloseApplication = 'CLOSEAPPLICATION';
  c_Script = 'SCRIPT';
  c_BEGIN = 'BEGIN';
  c_END = 'END';


type




TOptions = class(TBaseOptions)
private
  FDelRAS: TStringListElement;
  FDeleteRAS: boolean;
  {Scripts}
  FStartScript: string;
  FDataSetScript: string;
  FControlSetScript: string;
  FRunScript: string;
  FEndScript: string;
  {Files}
  FINE: string;
  FPaletFile: string;
  {Directories}
  FWorkingDirectory: String;
  FDataDirectory: String;
  FServiceDirectory: String;
  FScriptDirectory: String;
  FFullUseMemory: boolean;
  function  GetDelRAS: TStringListElement;
  procedure SetServiceDirectory(aValue: string);
public
  property   DelRAS: TStringListElement read GetDelRAS;
published
//  property   DeleteRAS: boolean read FDeleteRas write FDeleteRAS;
  property   StartScript: string read FStartScript write FStartScript;
//  property   DataSetScript: string read FDataSetScript write FDataSetScript;
//  property   ControlSetScript: string read FControlSetScript write FControlSetScript;
//  property   RunScript: string read FRunScript write FRunScript;
  property   EndScript: string read FEndScript write FEndScript;
  property   WorkingDirectory: string read FWorkingDirectory write FWorkingDirectory;
  property   ServiceDirectory: string read FServiceDirectory write SetServiceDirectory;
  property   DataDirectory: string read FDataDirectory write FDataDirectory;
  property   ScriptDirectory: string read FScriptDirectory write FScriptDirectory;
//  property   INE: string read FINE write FINE;
//  property   PaletFile: string read FPaletFile write FPaletFile;
//  property   FullUseMemory: boolean read FFullUseMemory write FFullUseMemory default false;
end;


TOnSelectEvent = procedure of object;
TOnToolDoneEvent = procedure(aToolName: string) of object;
TOnToolChangeEvent = procedure(aToolName: string) of object;

TMainTool = class(TOldMainTool)
private
  FOptions: TOptions;
  ViewerIndex: integer;
  FSelectDone: boolean;
  FCancelProcess: boolean;
  FScriptReading: boolean;
  FCommandLine: TCommandLine;

  FOnSelect: TOnSelectEvent;
  FOnToolDone: TOnToolDoneEvent;
  FOnToolChange: TOnToolChangeEvent;
  FImage: TImage;
  FExecuteScript: string;

  FSelectedObject: TElement;

  FProtocol: TMemoFormDevice;
  FUseProtocol: boolean;
{$IFDEF ADO}
  FADOConnectionString: string;
{$ENDIF}
  function  GetOptions: TBaseOptions; override;
  function  GetNOptions: TOptions; virtual;
  procedure SetOptions(aValue: TBaseOptions); override;
  procedure SetNOptions(aValue: TOptions); virtual;
  function  GetCommandLine: TCommandLine;
  procedure SetCommandLine(aValue: TCommandLine);
  procedure ExecScript(aFileName: string);
  function  GetSourceModel: TOwnerEntity;
  procedure SetSourceModel(aValue: TOwnerEntity);
  function  GetData: TOwnerEntity;
  function  GetLayers: TRectEntity;
  function  GetThematic: TRectEntity;
  function  GetExePath: string;
  function GetImageTool: TImageToolManager;
  function GetProjection: TPageProjection;
    procedure SetImage(const Value: TImage);
protected
  function  GetWorkingDirectory: string; override;
  procedure ReadParamsFromIni; override;
  procedure ReadParams; override;
  procedure WriteParamsToIni; override;
public
   StartDir: string;
   Progress: integer;
   ProgressVisible: boolean;
   MinProgress: integer;
   MaxProgress: integer;
   Text: string;

   constructor Create; override;
   destructor  Destroy; override;
   procedure   LoadTools(aPackageName: string);
   procedure   DoIt; override;
   procedure  SetInterpret; virtual;
   procedure  on_tm_SetOptions(var Handled: boolean); virtual;
   procedure  on_tm_QuitTask; override;
   procedure  On_tm_ImportBlk(var Handled: boolean); virtual;
   procedure  on_tm_SelectDataDirectory(var Handled: boolean); virtual;
   procedure  on_tm_DataSet(var Handled: boolean); virtual;
   procedure  on_tm_ControlSet(var Handled: boolean); virtual;
   procedure  on_tm_Run(var Handled: boolean); virtual;
   procedure  On_tm_Explore(var aMessage: TOrbisMessage; var Handled: boolean); virtual;
   procedure  On_tm_ExploreOwners(var aMessage: TOrbisMessage; var Handled: boolean); virtual;
   procedure  On_tm_EditElement(var aMessage: TOrbisMessage; var Handled: boolean); virtual;
   procedure  On_tm_LoadElement(var aMessage: TOrbisMessage; var Handled: boolean); virtual;
   procedure  On_tm_SaveElement(var aMessage: TOrbisMessage; var Handled: boolean); virtual;
   procedure  On_tm_SetPalette(var Handled: boolean); virtual;
   procedure  on_tm_EndOfPrint(var Handled: boolean); virtual;
   procedure  on_tm_ShowLHC; virtual;
   procedure  On_tm_AddView(var aMessage: TOrbisMessage; var Handled: boolean); virtual;
   procedure  On_tm_Select(var aMessage: TOrbisMessage; var Handled: boolean); virtual;
   procedure  On_tm_ToolChange(var aMessage: TOrbisMessage; var Handled: boolean); virtual;
   procedure  on_tm_Done(aMessage: TOrbisMessage); override;
   procedure  on_tm_AddToProtocol(aMessage: TOrbisMessage); override;

   procedure  HandleMessage(var aMessage: TOrbisMessage; var Handled: boolean); override;

   procedure  ExecuteMacro(M: TStrings); virtual;
   procedure  AddWorkTools; override;
   procedure  Show;
   procedure  AreaSelected;
   procedure ExploreData(aData: TElement);
   procedure DoOnToolDone(aName: string);
   function  GetLayerByName(aName: string): TInterpretLayer;
   function  GetMapByName(aName: string): TRectEntity;

   property   SelectDone: boolean read FSelectDone write FSelectDone;
   property   CancelProcess: boolean read FCancelProcess write FCancelProcess;
   property   ScriptReading: boolean read FScriptReading write FScriptReading;
   property   SourceModel: TOwnerEntity read GetSourceModel write SetSourceModel;
   property   Data: TOwnerEntity read GetData;
   property   Layers: TRectEntity read GetLayers;
   property   Thematic: TRectEntity read GetThematic;
   property   ExePath: string read GetExePath;
   property   Image: TImage read FImage write SetImage;
   property   ImageTool: TImageToolManager read GetImageTool;
   property   SelectedObject: TElement read FSelectedObject write FSelectedObject;

   procedure ShowModel(aModel: TElement);
   property   Protocol: TMemoFormDevice read FProtocol write FProtocol;
published
{ prce s vrstvami }
   procedure AddToLayer(aLayer, aFunction, aParam: string);

{ prce s pohledy }
   procedure ClearMap(aMap: string);
   procedure AddToMap(aMap, aLayer: string);
   procedure ShowMap(aMap: string);
   procedure ReView;

   property   Options: TOptions read GetNOptions write SetNOptions;
   property   CommandLine: TCommandLine read GetCommandLine write SetCommandLine;
   property   ExecuteScript: string write ExecScript stored false;
   property   OnSelect: TOnSelectEvent read FOnSelect write FOnSelect;
   property   OnToolDone: TOnToolDoneEvent read FOnToolDone write FOnToolDone;
   property   OnToolChange: TOnToolChangeEvent read FOnToolChange write FOnToolChange;
   property   Projection: TPageProjection read GetProjection;
   property   UseProtocol: boolean read FUseProtocol write FUseProtocol;
{$IFDEF ADO}
   property   ADOConnectionString: string read FADOConnectionString write FADOConnectionString;
{$ENDIF}
end;



implementation
uses
  { Test}

  Controls, Forms, Dialogs, IniFiles, SysUtils, Graphics, WinProcs, DBTables,
  _IniFile,
// KERNEL
  M_Orbis,
  M_Proj,
  Worlds,
  u_String,
  E_Files,
  E_Index,
// OIL
  t_Select,
// EXPLORER
  T_Explor,
// GLIB
  t_Grp,
// SPACE
  e_Reg,
// APP
  f_SelDir,
  f_YesNo,
// 0TEMP
  e_DClass,
  T_EdiPro,
  t_Loader,
  t_Saver,
//  t_XML,
  F_ViewDT,
// INTERPRET
  e_IScrpt,
// TOPOL
  e_TplPrp,
//  T_TopImp,//!!!!!!!!!!!!!!!!!!!!!!

// GUI
  t_Intera,
// SYMBOL
  U_Symbol,
// MAPPER
   T_TisCre,
// TOPOL
  C_TlDBF,
// MAIN
  f_Option,
  F_Start;

{TOptions}

function  TOptions.GetDelRAS: TStringListElement;
begin
   if FDelRas = nil then
   begin
      FdelRas := TStringListElement.Create;
      AddElement(FDelRas);
   end;
   result := FDelRas;
end;

procedure TOptions.SetServiceDirectory(aValue: string);
begin
  if aValue = '' then
  begin
     aValue:= extractFilePath(Application.ExeName);
     aValue:= removeBackSlash(aValue);
  end;
   FServiceDirectory := aValue;
   PathOfLibraries := aValue;
end;


{TMainTool}




procedure  TMainTool.On_tm_AddView(var aMessage: TOrbisMessage; var Handled: boolean);
var
  Tool: TFormToolManager;
  ISC: TInterpretScript;
  TIS: TUniversalTopolScriptCreator;
  aPers: TPersistent;
  ViewData : TF_ViewData;

begin
  aPers := TPersistent(aMessage.Data);
  if aPers is TTable then
  begin
    ViewData := TF_ViewData.Create(Application);
    ViewData.DataSource1.DataSet:= TTable(aPers);

//    if (aPers is TTopolDBFTable) and (IntegerProp > 0) then
//       TTopolDBFTable(FPersistent).Cislo:= IntegerProp;

    ViewData.ShowModal;
  end
  else
  if aPers is TElement then
  begin
    Tool := TFormToolManager.Create;
    TIS := TUniversalTopolScriptCreator.Create;
    TIS.DoIt;
    ISC := TInterpretScript(TIS.Output);
    Tool.InterpretScript := ISC;
    TIS.Free;

    Tool.Input := TElement(aMessage.Data);
    Tool.Device.Page.Color := clWhite;
    World.MainTool.AddTool(Tool);
    Tool.DoIt;
  end;
  Handled := true;
end;




function  TMainTool.GetSourceModel: TOwnerEntity;
var
   S: string;
begin
   S := 'SOURCE';
   result := TOwnerEntity(World.ObjectModel.FindElementName(S));
   if result = nil then
   begin
      result := TOwnerEntity.Create;
      result.Name := S;
      World.ObjectModel.AddElement(result);
   end;
end;

procedure TMainTool.SetSourceModel(aValue: TOwnerEntity);
var
   S: string;
   E: TElement;
begin
   S := 'SOURCE';
   if aValue = nil then exit;
   if not(TElement(aValue) is TOwnerEntity) then exit;
   aValue.Name := S;
   E := World.ObjectModel.FindElementName(S);
   if E <> nil then World.FreeElement(E);
   World.ObjectModel.AddElement(aValue);
end;


function  TMainTool.GetData: TOwnerEntity;
var
   S: string;
begin
   S := 'DATA';
   result := TOwnerEntity(World.ObjectModel.FindElementName(S));
   if result = nil then
   begin
      result := TOwnerEntity.Create;
      result.Name := S;
      World.ObjectModel.AddElement(result);
   end;
end;


function  TMainTool.GetLayers: TRectEntity;
var
   S: string;

begin
   S := 'LAYERS';
   result := TRectEntity(World.ObjectModel.FindElementName(S));
   if result = nil then
   begin
      result := TRectEntity.Create;
      result.Name := S;
      World.ObjectModel.AddElement(result);
   end;
end;


function  TMainTool.GetThematic: TRectEntity;
var
   S: string;

begin
   S := 'THEMATIC';
   result := TRectEntity(Data.FindElementName(S));
   if result = nil then
   begin
      result := TRectEntity.Create;
      result.Name := S;
      Data.AddElement(result);
   end;
end;



function  TMainTool.GetExePath: string;
begin
  result := ExtractFilePath(ParamStr(0));
  CheckBackSlash(result);
end;





procedure TMainTool.Show;
var
   FM: TFileModel;
   E,EE: TEntity;
   RE: TRectEntity;
   R: TDoubleRect;
begin

   E := TEntity(World.ObjectModel.FindElementName('DATA'));
   if E = nil then exit; {!!!!!!!!!!!!}
   EE := TEntity(E.FindElementName('HIERARCHY'));
   if EE = nil then exit; {!!!!!!!!!!!!}

   if RE = nil then exit;

   R.Minx := RE.Minx;
   R.Maxx := RE.Maxx;
   R.Miny := RE.Miny;
   R.Maxy := RE.Maxy;

(*
   aWindow.Input := ActualMap;
   aWindow.Caption := MapName + ' '+ LHC+' '+ODDELENI+' '+DILEC+' '+POROST+' '+SKUPINA;
   aWindow.PageColor := clWhite;
//   aWindow.PageColor := clSilver;
   aWindow.Projection.ModelView := R;
   aWindow.DoIt;
*)
end;

constructor TMainTool.Create;
var
   S: string;
begin
  inherited Create;
  getDir(0, S);
  Options.WorkingDirectory := S;
  Options.DataDirectory := S;
end;


destructor  TMainTool.Destroy;
begin
  inherited Destroy;
end;


procedure TMainTool.LoadTools(aPackageName: string);
var
  aModule: HModule;
begin
  aPackageName:= changeFileExt(aPackageName, '.BPL');
  aModule:= LoadPackage(aPackageName);
end;


function  TMainTool.GetOptions: TBaseOptions;
begin
   if FOptions = nil then
   begin
      FOptions := TOptions.Create;
      AddElement(FOptions);
   end;
   result := FOptions;
end;

function  TMainTool.GetNOptions: TOptions;
begin
   result := TOptions(GetOptions);
end;


procedure TMainTool.SetOptions(aValue: TBaseOptions);
begin
   if FOptions <> nil then
   begin
      FreeElement(FOptions);
      FOptions := nil;
   end;
   FOptions := TOptions(aValue);
end;

procedure TMainTool.SetNOptions(aValue: TOptions);
begin
   SetOptions(aValue);
end;

function  TMainTool.GetCommandLine: TCommandLine;
begin
   if FCommandLine = nil then
   begin
      FCommandLine := TCommandLine.Create;
      FCommandLine.Name := 'Hlavn interpret jazyka OIL';
      AddTool(FCommandLine);
   end;
   result := FCommandLine;
end;


procedure TMainTool.SetCommandLine(aValue: TCommandLine);
begin
   if FCommandLine <> nil then
   begin
      FreeTool(FCommandLine);
      FCommandLine := nil;
   end;
   FCommandLine := aValue;
   AddTool(FCommandLine);
end;


procedure TMainTool.ExecScript(aFileName: string);
begin
  CommandLine.FileName:= aFileName;
  CommandLine.DoIt
end;


function  TMainTool.GetWorkingDirectory: string;
var
   S: string;
begin
   S := Options.WorkingDirectory;
   result := CheckBackSlash(S);
end;




procedure TMainTool.DoIt;
var
  OIL: string;
  S,SS: string;
begin
  //inherited DoIt;
  LoadCursors;
  ReadParams;
  AddWorkTools;
  Application.HelpFile := ChangeFileExt(ParamStr(0),'.HLP');
  {
  if not Application.HelpJump('Obsah') then
     ShowMessage('Soubor npovdy neexistuje');
  }
  AddElement(TElementIndex.Create);
  OIL:= GetParamWithExt('.OIL');
  if (OIL <> '') and fileExists(OIL) then
  begin
     ExecuteScript := OIL;
  end
  else ExecuteScript := Options.StartScript;
{
  S := ExtractFilePath(ParamStr(0));
  CheckBackSlash(S);
  SS := S + 'SysReg.Oil';
  if not FileExists(SS) then
  begin
     ShowMessage('Nelze nalzt soubor '+SS);
  end
  else ExecuteScript := SS;
  SS := S + 'Model.Oil';
  if not FileExists(SS) then
  begin
     ShowMessage('Nelze nalzt soubor '+SS);
  end
  else ExecuteScript := SS;
}
end;





procedure TMainTool.ReadParamsFromIni;
var
   Load: TLoad;
   S: string;
begin
   inherited ReadParamsFromIni;
   {
   NactiBarvickuZeSouboru(Options.PaletFile);
   }
   {
   S := ExePath + 'SOURCE.ORB';
   if FileExists(S) then
   begin
      Load := TLoad.Create;
      Load.FileName := S;
      Load.DoIt;
      SourceModel := TOwnerEntity(Load.Output);
      Load.Free;
   end;
   }
end;



procedure TMainTool.ReadParams;
begin
   ReadParamsFromIni;
end;




procedure TMainTool.WriteParamsToIni;
var
   Save: TSave;
begin
   {
   Options.PaletFile := PaletFile;
   }
   inherited WriteParamsToIni;
   Save := TSave.Create;
   Save.FileName := ExePath + 'SOURCE.ORB';
   Save.Input := SourceModel;
   Save.DoIt;
   Save.Free;
end;


procedure  TMainTool.on_tm_SetOptions(var Handled: boolean);
var
   D: TF_Options;
begin
   Handled := true;
   D:= TF_Options.Create(Application);

   D.ed_Work.Text := Options.WorkingDirectory;
   D.ed_EXE.Text := ExePath;
   D.ed_Service.Text := Options.ServiceDirectory;
   D.ed_Data.Text := Options.DataDirectory;
   if D.ShowModal = mrOk then
   begin
      Options.WorkingDirectory := D.ed_Work.Text;
      Options.ServiceDirectory := D.ed_Service.Text;
      PathOfLibraries:= Options.ServiceDirectory;
      Options.DataDirectory := D.ed_Data.Text;
   end;
   //MainForm.b_Directory.Hint := 'Data: ' + Options.DataDirectory;
   D.Free;
end;


procedure  TMainTool.on_tm_QuitTask;
var
   i: longInt;
begin
   WriteParamsToIni;
(*
   case MessageDlg('Opravdu ukonit aplikaci ?', mtConfirmation, mbYesNoCancel, 0) of
      mrYes:
      begin
        try

           if Options.DeleteRAS then
           for i := 0 to Options.DelRas.Count - 1 do
           begin
              SysUtils.DeleteFile(Options.DelRas.Strings[i]);
           end;
                 {
           PostAppMessage(Project.MainHandle, TProjectWorld(World).WM_MODUL_END, 0, 0)
           }
        except
        end;
        Application.Terminate;
      end;
   end;
*)
end;



procedure  TMainTool.On_tm_ImportBlk(var Handled: boolean);
var
  E: TRectEntity;
  DD: TLayerImportDialog;
  MSC: TModelScriptCreator;
  MBSC: TModelByScriptCreator;
begin

   DD := TLayerImportDialog.Create;
   MSC := TModelScriptCreator.Create;
   MBSC := TModelByScriptCreator.Create;
   try
      DD.InitialDir := Options.DataDirectory;
      DD.DoIt;
      MSC.Input := DD.Output;
      MSC.DataDirectory := Options.DataDirectory;
      MSC.DoIt;
      MBSC.Input := MSC.Output;
      MBSC.DoIt;
      MBSC.Output.Name := 'DATA';
      World.ObjectModel.AddElement(MBSC.Output);
   except
   end;
   DD.Free;
   MSC.Free;
   MBSC.Free;
end;

procedure  TMainTool.on_tm_SelectDataDirectory(var Handled: boolean);
//var
   //DD: TF_SelectDir;
begin
(*
   Handled := true;
   Application.CreateForm(TF_SelectDir, DD);
   try
     DD.Directory.Directory:= Options.DataDirectory;
     if DD.ShowModal = mrOK then
     begin
        Options.DataDirectory := DD.Directory.Directory;
     end;
   except
     on E:Exception do
     begin
       ShowMessage(E.Message + ': ' + Options.DataDirectory);
       Screen.Cursor:= crDefault;
       Options.DataDirectory:= ''
     end;
   end;
   DD.Free;
*)
end;

procedure  TMainTool.on_tm_DataSet(var Handled: boolean);
begin
(*
   Handled := true;
   ExecuteScript := Options.DataSetScript;
*)
end;

procedure  TMainTool.on_tm_ControlSet(var Handled: boolean);
begin
(*
   Handled := true;
   ExecuteScript := Options.ControlSetScript;
*)
end;

procedure  TMainTool.on_tm_Run(var Handled: boolean);
begin
(*
   Handled := true;
   ExecuteScript := Options.RunScript;
*)
end;



procedure  TMainTool.On_tm_Select(var aMessage: TOrbisMessage; var Handled: boolean);
begin
  SelectedObject:= aMessage.Data;
  if Assigned(OnSelect) then
     OnSelect;

  Handled:= true;
end;



procedure  TMainTool.On_tm_ToolChange(var aMessage: TOrbisMessage; var Handled: boolean);
begin
  if not (aMessage.Sender is TTool) then
     Exit;
  if not Assigned(OnToolChange) then
     Exit;

  OnToolChange(aMessage.Sender.Name);
  Handled:= True;
end;



function  TMainTool.GetImageTool: TImageToolManager;
var
   Tool: TImageToolManager;
   ISC: TInterpretScript;
   TIS: TUniversalTopolScriptCreator;
begin
   result:= TImageToolManager(GetToolByName('ImageTool'));
   if result <> nil then exit;
   try
   Tool := TImageToolManager.CreateImageTool(Image);
//   Tool.Image:= Image;
   Tool.Name:= 'ImageTool';
   TIS := TUniversalTopolScriptCreator.Create;
   TIS.DoIt;
   ISC := TInterpretScript(TIS.Output);
   Tool.InterpretScript := ISC;
   TIS.Free;
   Tool.Device.Page.Color := clWhite;
   AddTool(Tool);
   result:= Tool;
   except
    result:=nil;
   end;
end;

function TMainTool.GetProjection: TPageProjection;
begin
  if TElement(ImageTool.Projection) is TPageProjection then
    result:= TPageProjection(ImageTool.Projection)
  else
    result:= nil
end;


procedure TMainTool.ExploreData(aData: TElement);
var
   aTool: TExplorer;
begin
    aTool := TExplorer.Create;
    AddTool(aTool);
    if assigned(aTool) then
    begin
      aTool.Input:= aData;
{!}   aTool.Modal:= true;
      aTool.DoIt
    end;
end;



procedure  TMainTool.On_tm_Explore(var aMessage: TOrbisMessage; var Handled: boolean);
begin
  ExploreData(aMessage.Data);
  Handled:= true;
end;





procedure  TMainTool.On_tm_ExploreOwners(var aMessage: TOrbisMessage; var Handled: boolean);
var
   aTool: TExplorer;
begin
    aTool := TExplorer.Create;
    AddTool(aTool);
    if assigned(aTool) then
    begin
      aTool.Input:= aMessage.Data;
      aTool.ShowOwners := true;
      aTool.Caption := 'Model vlastnk';
      aTool.DoIt
    end;
end;

procedure  TMainTool.On_tm_EditElement(var aMessage: TOrbisMessage; var Handled: boolean);
var
   aTool: TObjectPropertyEditor;
begin
    aTool := TObjectPropertyEditor.Create;
    AddTool(aTool);
    if assigned(aTool) then
    begin
      aTool.Input:= aMessage.Data;
      aTool.DoIt
    end;
end;



procedure  TMainTool.On_tm_LoadElement(var aMessage: TOrbisMessage; var Handled: boolean);
//var
//   aTool: TXMLLoader;
begin
{
  if not (aMessage.Data is TEntity) then Exit;
  if not MainForm.odXML.Execute then
     Exit;
  aTool := TXMLLoader.Create;
  if assigned(aTool) then
  begin
    aTool.FileName:= MainForm.odXML.FileName;
    aTool.Input:= aMessage.Data;
    aTool.DoIt;
//    TEntity(aMessage.Data).AddElement(aTool.Output);
    aTool.Free
  end;
}
end;




procedure  TMainTool.On_tm_SaveElement(var aMessage: TOrbisMessage; var Handled: boolean);
//var
//   aTool: TXMLSaver;
begin
{
  if not MainForm.sdXML.Execute then
     Exit;
  aTool := TXMLSaver.Create;

  if assigned(aTool) then
  begin
    aTool.FileName:= changeFileExt(MainForm.sdXML.FileName, '.XML');
    aTool.Input:= aMessage.Data;
    aTool.DoIt;
    aTool.Free
  end;
}
end;





procedure  TMainTool.On_tm_SetPalette(var Handled: boolean);
begin
//  NastavBarvicky
end;






procedure TMainTool.On_tm_EndOfPrint(var Handled: boolean);
begin
   {
   if CancelProcess then
   begin
      MainForm.Dioda.Color := clBtnFace;
      MainForm.Dioda.Enabled := false;
   end;
   if Automat.Busy then Automat.ProcessMap := false;
   }
end;



procedure TMainTool.on_tm_Done(aMessage: TOrbisMessage);
begin
  if not assigned(aMessage.Sender) then
     exit;

  inherited on_tm_Done(aMessage);
  if not assigned(OnToolDone) then
     exit;

  OnToolDone(aMessage.Sender.Name);
  //Handled:= true;
end;




procedure   TMainTool.HandleMessage(var aMessage: TOrbisMessage; var Handled: boolean);
//var
   //PZ: TRectEntity;
   //Element: TElement;
begin
  {
  MainForm.Ed.Text:= intToStr(memAvail-MainForm.OrigMem);
  }
   Handled:= false; { vse se vyresi na miste }
   inherited HandleMessage(aMessage, Handled);
   if Handled then exit;
   case aMessage.Message of
     { SOUBOR }
      tm_ImportBlk:   on_tm_ImportBlk(Handled);
      tm_SelectDataDirectory: on_tm_SelectDataDirectory(Handled);
      tm_ClearModel:;
      tm_SetOptions:  on_tm_SetOptions(Handled);
      tm_DataSet:     on_tm_DataSet(Handled);
      tm_ControlSet:  on_tm_ControlSet(Handled);
      tm_Run:         on_tm_Run(Handled);
     { Mapov list }
      //tm_MapSheetList: PostMsg(tm_View, Self, TEntity(World.ObjectModel).FindElementClass(TMapSheets), DontKill);


     { ZOBRAZOVANI }
      tm_ReView :   On_tm_Addview(aMessage, Handled);
      tm_AddView:   On_tm_AddView(aMessage, Handled);
      tm_Explore: On_tm_Explore(aMessage, Handled);
      tm_EditElement: On_tm_EditElement(aMessage, Handled);
      tm_LoadElement     : On_tm_LoadElement(aMessage, Handled);
      tm_SaveElement     : On_tm_SaveElement(aMessage, Handled);
      tm_ExploreOwners: On_tm_ExploreOwners(aMessage, Handled);
{      tm_About: On_tm_About(Handled); }
      tm_SetPalette: On_tm_SetPalette(Handled);

      tm_EndOfPrint       : on_tm_EndOfPrint(Handled);
      tm_ShowLHC          : on_tm_ShowLHC;
     { Vybrn }
      tm_SelectOK: On_tm_Select(aMessage, Handled);
      tm_AddToProtocol    : on_tm_AddToProtocol(aMessage);

      tm_ToolChange: On_tm_ToolChange(aMessage, Handled);
   end;
end;



procedure  TMainTool.ExecuteMacro(M: TStrings);
begin
   if M.Count < 1 then exit;
   if ScriptReading then
   begin
      begin
         CommandLine.Line := '';
         CommandLine.Lines.Clear;
         CommandLine.Lines.Assign(M);
         CommandLine.DoIt;
         PostMsg(tm_SetDDEResult, Self, nil, DontKill);
      end;
   end
   else
   if UpperCase(M.Strings[0]) = c_Script then
   begin
      ScriptReading := true;
   end;
end;




procedure  TMainTool.AddWorkTools;
begin
//  AddTool(TDeskTopSaveToText.Create);
//  AddTool(TViewScriptEditor.Create);
end;


procedure  TMainTool.on_tm_ShowLHC;
begin
   Show;
end;



procedure  TMainTool.AreaSelected;
begin
  if Assigned(FOnSelect) then
     OnSelect;
end;


function  TMainTool.GetLayerByName(aName: string): TInterpretLayer;
begin
  result:= TInterpretLayer(Layers.FindElementName(aName));
  if result = nil then
  begin
    result:= TInterpretLayer.Create;
    result.Name:= aName;
    Layers.AddAndUpdateRect(result);
  end;
end;


function  TMainTool.GetMapByName(aName: string): TRectEntity;
begin
  if aName = '' then
     result:= TRectEntity(Thematic.Elements[0])
  else
     result:= TRectEntity(Thematic.FindElementName(aName));

  if result = nil then
  begin
    result:= TRectEntity.Create;
    result.Name:= aName;
    Thematic.AddAndUpdateRect(result);
  end;
end;



procedure TMainTool.DoOnToolDone(aName: string);
begin
  if not assigned(OnToolDone) then
     exit;
  OnToolDone(aName);
end;


{ prce s vrstvami }
procedure TMainTool.AddToLayer(aLayer, aFunction, aParam: string);
var
  MSC: TModelScriptCreator;
  MBSC: TModelByScriptCreator;
  aLay: TInterpretLayer;
begin
  MSC := TModelScriptCreator.Create;
  MSC.NextLayer:= aFunction + '=' + aParam;
  MSC.DoIt;

  MBSC := TModelByScriptCreator.Create;
  MBSC.Input:= MSC.Output;
  MBSC.DoIt;
  MBSC.Output.Name := aFunction;

  aLay:= GetLayerByName(aLayer);
  aLay.LayerType:= aFunction;
  aLay.Path:= aParam;
  aLay.AddAndUpdateRect(MBSC.Output);

  MBSC.Free;
  MSC.Free;
  DoOnToolDone('TModelByScriptCreator');
end;

{ prce s pohledy }
procedure TMainTool.ClearMap(aMap: string);
var
  aThem: TRectEntity;
begin
  aThem:= TRectEntity(Thematic.FindElementName(aMap));
  if aThem = nil then exit;
  aThem.Clear;
end;

procedure TMainTool.AddToMap(aMap, aLayer: string);
var
  aThem: TRectEntity;
  aLay: TInterpretLayer;
begin
  aThem:= GetMapByName(aMap);

  aLay:= GetLayerByName(aLayer);

  aThem.AddAndUpdateRect(aLay);
end;

procedure TMainTool.ShowMap(aMap: string);
var
  aThem: TRectEntity;
begin
  aThem:= GetMapByName(aMap);
  if aThem.Count = 0 then exit;
  ShowModel(aThem);
end;


procedure TMainTool.SetInterpret;
var
  aScrCrea: TUniversalTopolScriptCreator;
begin
  aScrCrea:= TUniversalTopolScriptCreator.Create;
  aScrCrea.DoIt;
  ImageTool.InterpretScript:= TInterpretScript(aScrCrea.Output);
  aScrCrea.Free;
end;


procedure TMainTool.ShowModel(aModel: TElement);
var
  aScrCrea: TUniversalTopolScriptCreator;
begin
  SetInterpret;
  ImageTool.input:= aModel;
  ImageTool.DoIt;
end;




procedure TMainTool.ReView;
begin
  if ImageTool = nil then
     exit;
  ImageTool.DoIt;
end;


procedure TMainTool.SetImage(const Value: TImage);
begin
  FImage := Value;
  ImageTool.Image:= FImage;
end;


procedure  TMainTool.on_tm_AddToProtocol(aMessage: TOrbisMessage);
var
   S: string;
begin
  if not UseProtocol then
     exit;
  inherited on_tm_AddToProtocol(aMessage);
end;



END.
