unit OrbisPanel;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  ExtCtrls,
  StdCtrls,
  ImagePanel,
// KERNEL
   Elements,
   Worlds,
   M_Orbis,
   Tools,
   E_index,
// MAIN
   T_Main
   ;


const
  OrbisPanelSaver: TOwnerEntity = nil;

type

  TWorldMouseMove = procedure(Sender: TObject; X: Double; Y: Double) of object;
  TOnSelectEvent = procedure of object;
  TOnToolDoneEvent = procedure(aToolName: string) of object;

  TOrbisPanel = class(TImagePanel)
  private
//    FWorld: TWorld;
    FScriptName: string;
    FOnMessage: TMessageEvent;
    FOnException: TExceptionEvent;
    FOnSelect: TOnSelectEvent;
    FOnToolDone: TOnToolDoneEvent;
    FOnToolChange: TOnToolChangeEvent;
    FOnWorldMouseMove: TWorldMouseMove;
    FActualMap: string;
    FLastTool: TTool;
    FNoReviewOnResize: boolean;
    FBusy: Boolean;
{
    procedure SetOnMouseDown(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
    procedure SetOnMouseUp(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
    procedure SetOnMouseMove(Sender: TObject; Shift: TShiftState; X, Y: Integer);

    procedure SetOnStartDrag(Sender: TObject; var DragObject: TDragObject);
    procedure SetOnDragOver(Sender, Source: TObject; X, Y: integer; State: TDragState; var Accept: boolean);
    procedure SetOnDragDrop(Sender, Source: TObject; X, Y: integer);
    procedure SetOnEndDrag(Sender, Source: TObject; X, Y: integer);
}
    procedure DestroyWorld;
    procedure Set_ScriptName(const Value: String);
    procedure RunMainTool;
    procedure SetScale(aVal: double);
    function  GetScale: double;
    procedure SetCenterX(aVal: double);
    function  GetCenterX: double;
    procedure SetCenterY(aVal: double);
    function  GetCenterY: double;

    function GetIsSelObj: boolean;
    function GetMainTool: TMainTool;
    function GetActiveTool: string;
    procedure SetActiveTool(const Value: string);
    function SelObjProperty(aPropertyName: string): string;
    function GetImageTool: TTechnology;
    function GetTools: string;
    function GetLayers: string;
    function GetMaps: string;
    function GetWorldX: double;
    function GetWorldY: double;
    procedure SetWorldX(const Value: double);
    procedure SetWorldY(const Value: double);
    function GetWorldHeight: double;
    function GetWorldWidth: double;
    function GetViewMaxX: double;
    function GetViewMaxY: double;
    function GetViewMinX: double;
    function GetViewMinY: double;
    function GetSelectedObject: TElement;
    function GetVirtualScale: double;
    procedure SetVirtualScale(const Value: double);
    function GetServiceDirectory: string;
    procedure SetServiceDirectory(const Value: string);
    function GetInitialized: Boolean;
    function GetBusy: Boolean;
    function GetUseProtocol: boolean;
    procedure SetUseProtocol(const Value: boolean);
  protected
    procedure FOnMouseMove(Sender: TObject; Shift: TShiftState; X, Y: Integer); override;
    procedure FOnMouseDown(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer); override;
    procedure FOnMouseUp(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer); override;

    property MainTool: TMainTool read GetMainTool;
    property SelectedObject: TElement read GetSelectedObject;

  public
//    property    OrbisWorld: TWorld read FWorld;
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    procedure CreateWorld; safecall;

    procedure Resize; override;
    procedure ShowModel(aModel: TElement);
    procedure ShowMap(aMap: string);
    procedure PrepareMap(aMap: string);

    procedure HandleMessage(var aMessage: TMessage); message WM_ORBISMESSAGE;

    procedure DoOnMouseEvent(MouseEventType: TMouseEventType; Button: TMouseButton; Shift: TShiftState; X, Y: integer); override;

//    procedure DoSelection(X, Y: integer); virtual;

    property ImageTool: TTechnology read GetImageTool;
    procedure SetUserKey(const Value: word; const Shift: TShiftState);

    function LayersObject: TEntity;
    function MapsObject: TEntity;
  published
{ obecn }

    procedure Init; virtual;
    property Initialized: Boolean read GetInitialized;
    procedure Done; virtual;
    property Busy: Boolean read GetBusy write FBusy;

    procedure LoadTools(aPackageName: string);
    procedure ExploreWorld;
    property ScriptName: String read FScriptName write Set_ScriptName stored false;

{ prce s vrstvami }
    procedure AddToLayer(aLayer, aFunction, aParam: string); {!}
    procedure NewLayer(aLayer, aFunction: string);
    property  Layers: string read GetLayers;

{ prce s pohledy }
    procedure ClearMap(aMap: string);
    procedure AddToMap(aMap, aLayer: string);
    property ActualMap: string read FActualMap write FActualMap;
    property Maps: string read GetMaps;

{ souadnice }
    property WorldX: double read GetWorldX write SetWorldX;
    property WorldY: double read GetWorldY write SetWorldY;

{ zobrazovn }
    procedure ReView; virtual;
    function Print: longint;

    property  ViewScale: double read GetScale write SetScale;
    property  VirtualScale: double read GetVirtualScale write SetVirtualScale;
    property  CenterX: double read GetCenterX write SetCenterX;
    property  CenterY: double read GetCenterY write SetCenterY;

    property  ViewWidth: double read GetWorldWidth;
    property  ViewHeight: double read GetWorldHeight;

    property ViewMinX: double read GetViewMinX;
    property ViewMaxX: double read GetViewMaxX;
    property ViewMinY: double read GetViewMinY;
    property ViewMaxY: double read GetViewMaxY;
    property NoReviewOnResize: boolean read FNoReviewOnResize write FNoReviewOnResize;
    property ServiceDirectory: string read GetServiceDirectory write SetServiceDirectory;

{ prce s vybranm objektem }
    procedure SetObjectProperty(aPropertyName, aValue: string);
    function  GetObjectProperty(aPropertyName: string): string;
    property  IsSelObj: boolean read GetIsSelObj;

{ prce s nstrojem }
    procedure CreateTool(aToolClass, aToolName: string);
    property  ActiveTool: string read GetActiveTool write SetActiveTool;
    property  Tools: string read GetTools;
{ pokud se jedn o interaktivn nstroj, pak ho uin aktivnm }

    procedure SetToolProperty(aToolName: string; aPropertyName, aValue: string);
    function  GetToolProperty(aToolName: string; aPropertyName: string): string;
    procedure ExecuteTool(aToolName: string);
    property  OnSelect: TOnSelectEvent read FOnSelect write FOnSelect;
    property  OnToolDone: TOnToolDoneEvent read FOnToolDone write FOnToolDone;
    property  OnToolChange: TOnToolChangeEvent read FOnToolChange write FOnToolChange;
    property  OnWorldMouseMove: TWorldMouseMove read FOnWorldMouseMove write FOnWorldMouseMove;
    procedure FreeTool(aToolName: string);
    property  UseProtocol: boolean read GetUseProtocol write SetUseProtocol;
  end;


procedure Register;

implementation
//{$R Orbispanel.DCR}
uses
   Props,
   M_Proj,
// KERNEL
   U_String,
   e_class,
// GUI
   T_InterA,
// GLIB
   G_Elems,
// SPACE
   E_Space,
// 0TEMP
   T_SpPick,

//   T_Meas,

   e_WCS;// WCSWorld ... pehodnotit  !!!!!!!!!!!!!!!!

//   U_HelStr;



constructor TOrbisPanel.Create;
begin
  Busy:= True;
  inherited;
end;


destructor TOrbisPanel.Destroy;
begin
  Busy:= True;
  inherited;
  MessageBeep(0)
end;

procedure TOrbisPanel.FOnMouseMove(Sender: TObject; Shift: TShiftState; X, Y: Integer);
begin
  inherited;
  if assigned(FOnWorldMouseMove) then
  begin
    FOnWorldMouseMove(Sender, WorldX, WorldY);
  end;
{
  if FWorld = nil then exit;

  if not (ImageTool is TInteractiveTool) then exit;
  TInteractiveTool(ImageTool).PostMouseEvent(tm_MouseMove, ImageTool, mbLeft, Shift, X, Y, true);
}
end;

procedure TOrbisPanel.FOnMouseDown(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
begin
  inherited;
{
  if FWorld = nil then exit;
  if not (ImageTool is TInteractiveTool) then exit;
  TInteractiveTool(ImageTool).PostMouseEvent(tm_MouseDown, ImageTool, mbLeft, Shift, X, Y, true);
}
end;

procedure TOrbisPanel.FOnMouseUp(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
begin
  inherited;
{
  if FWorld = nil then exit;
  if not (ImageTool is TInteractiveTool) then exit;
  TInteractiveTool(ImageTool).PostMouseEvent(tm_MouseUp, ImageTool, mbLeft, Shift, X, Y, true);
}
end;

procedure TOrbisPanel.DoOnMouseEvent
          (MouseEventType: TMouseEventType; Button: TMouseButton; Shift: TShiftState; X, Y: integer);
var
  aTM: integer;
begin
  if not (ImageTool is TInteractiveTool) then exit;

  aTM:= tm_MouseMove;
  case MouseEventType of
    meMouseMove: aTM:= tm_MouseMove;
    meMouseDown: aTM:= tm_MouseDown;
    meMouseUp: aTM:= tm_MouseUp;
    meStartDrag: aTM:= tm_StartDrag;
    meDragOver: aTM:= tm_DragOver;
    meDragDrop: aTM:= tm_DragDrop;
    meEndDrag: aTM:= tm_DragEnd;
  end;

  TInteractiveTool(ImageTool).PostMouseEvent(aTM, ImageTool, Button, Shift, X, Y, true);
end;

{
procedure TOrbisPanel.SetOnMouseDown(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
begin
  if not (ImageTool is TInteractiveTool) then exit;
  TInteractiveTool(ImageTool).PostMouseEvent(tm_MouseDown, ImageTool, Button, Shift, X, Y, true);
end;

procedure TOrbisPanel.SetOnMouseUp(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
begin
  if not (ImageTool is TInteractiveTool) then exit;
  TInteractiveTool(ImageTool).PostMouseEvent(tm_MouseUp, FLastTool, Button, Shift, X, Y, true);
end;

procedure TOrbisPanel.SetOnMouseMove(Sender: TObject; Shift: TShiftState; X, Y: Integer);
begin
  if not (ImageTool is TInteractiveTool) then exit;
  TInteractiveTool(ImageTool).PostMouseEvent(tm_MouseMove, FLastTool, mbLeft, Shift, X, Y, true);
end;



procedure TOrbisPanel.SetOnStartDrag(Sender: TObject; var DragObject: TDragObject);
begin
  if not (ImageTool is TInteractiveTool) then exit;
  TInteractiveTool(ImageTool).PostMouseEvent(tm_StartDrag, FLastTool, mbLeft, [], 0, 0, true);
end;

procedure TOrbisPanel.SetOnDragOver(Sender, Source: TObject; X, Y: integer; State: TDragState; var Accept: boolean);
begin
  if not (ImageTool is TInteractiveTool) then exit;
  TInteractiveTool(ImageTool).PostMouseEvent(tm_DragOver, FLastTool, mbLeft, [], X, Y, true);
end;

procedure TOrbisPanel.SetOnDragDrop(Sender, Source: TObject; X, Y: integer);
begin
  if not (ImageTool is TInteractiveTool) then exit;
  TInteractiveTool(ImageTool).PostMouseEvent(tm_DragDrop, FLastTool, mbLeft, [], X, Y, true);
end;

procedure TOrbisPanel.SetOnEndDrag(Sender, Source: TObject; X, Y: integer);
begin
  if not (ImageTool is TInteractiveTool) then exit;
  TInteractiveTool(ImageTool).PostMouseEvent(tm_DragEnd, FLastTool, mbLeft, [], X, Y, true);
end;

}


procedure TOrbisPanel.RunMainTool;
begin
  if World = nil then exit;
  if MainTool <> nil then
  begin
    MainTool.Image:= Image;
    MainTool.DoIt;
    MainTool.OnSelect:= OnSelect;
    MainTool.OnToolDone:= OnToolDone;
    MainTool.OnToolChange:= OnToolChange;
  end;
end;


procedure TOrbisPanel.CreateWorld;
begin
  if OrbisPanelSaver <> nil then
  begin
    OrbisClasses:= TOrbisClasses(OrbisPanelSaver.FindElementClass(TOrbisClasses));
  end;

  World := TWCSWorld.Create;
  World.MainTool:= TMainTool.Create;
//  FWorld:= World;
//  TMainTool(World.MainTool).MainPanel:= self;


{ O.S. 17.2.99 - stejn to njak nefunguje
  Application.OnMessage:= World.HandleWinMessage;
  Application.OnException := World.HandleException;
}
end;



procedure TOrbisPanel.DestroyWorld;
var
  AuxWorld: TEntity;
begin
  Busy:= True;
//  Application.OnMessage:= FOnMessage;
//  Application.OnException := FOnException;

{1.
  World.Free;
  World:= nil;
}

{2.}
  if OrbisPanelSaver = nil then
  begin
    OrbisPanelSaver:= TOwnerEntity.Create;
    OrbisPanelSaver.Name:= 'ORBISPANELSAVER';
    OrbisPanelSaver.AddElement(OrbisClasses);
  end;

  World.ObjectModel:= nil;


  World.MainTool:= nil;
  World.SystemRegister:= nil;

  AuxWorld:= World;
  World:= nil;
  AuxWorld.Free;
end;


procedure TOrbisPanel.Init;
begin
  Busy:= True;
  if World = nil then
     CreateWorld;

  FOnMessage   := Application.OnMessage;
  FOnException := Application.OnException;

{$IFDEF OCX}
  MessageHandle:= Handle;
{$ENDIF}

  RunMainTool;

  Image.Picture.Bitmap.Width := Width;
  Image.Picture.Bitmap.Height:= Height;

end;


procedure TOrbisPanel.Done;
begin
  Busy:= True;

  FActualMap:= '';
  FLastTool:= nil;

  if World = nil then exit;

{$IFDEF OCX}
  MessageHandle:= 0;
{$ENDIF}

  DestroyWorld;
end;



procedure TOrbisPanel.Resize;
begin
  inherited;
  if World = nil then exit;
  if not NoReviewOnResize then
     ReView;
end;

function TOrbisPanel.GetMainTool: TMainTool;
begin
  result:= nil;
  if World = nil then exit;
  result:= TMainTool(World.MainTool)
end;


procedure TOrbisPanel.SetScale(aVal: double);
begin
 if World = nil then exit;
 MainTool.ImageTool.Scale:= aVal
end;

function  TOrbisPanel.GetScale: double;
begin
  result:= 0;
  if World = nil then exit;
  result:= MainTool.ImageTool.Scale
end;

procedure TOrbisPanel.SetCenterX(aVal: double);
begin
  if World = nil then exit;
  MainTool.Projection.Center_X:= aVal
end;

function  TOrbisPanel.GetCenterX: double;
begin
  result:= 0;
  if World = nil then exit;
  result:= MainTool.Projection.Center_X;
end;

procedure TOrbisPanel.SetCenterY(aVal: double);
begin
  if World = nil then exit;
  MainTool.Projection.Center_Y:= aVal
end;

function  TOrbisPanel.GetCenterY: double;
begin
  result:= 0;
  if World = nil then exit;
  result:= MainTool.Projection.Center_Y;
end;



procedure TOrbisPanel.Set_ScriptName(const Value: String);
begin
  if World = nil then exit;
  MainTool.ExecuteScript:= Value;
end;


function TOrbisPanel.GetIsSelObj: boolean;
begin
  result:= false;
  if World = nil then exit;
  result:= SelectedObject is TSpaceEntity
end;


procedure TOrbisPanel.AddToLayer(aLayer, aFunction, aParam: string);
begin
  if World = nil then exit;
  MainTool.AddToLayer(aLayer, aFunction, aParam);
//  TMainTool(world.MainTool).ExploreData(World.MainTool)
end;

procedure TOrbisPanel.NewLayer(aLayer, aFunction: string);
var
  aLay: TRectEntity;
begin
  if World = nil then exit;
  aLay:= MainTool.GetLayerByName(aLayer);
  if aLay.Count > 0 then
     aLay.FreeAll
end;




procedure TOrbisPanel.ExploreWorld;
begin
  if World = nil then exit;
  MainTool.ExploreData(World)
end;


procedure TOrbisPanel.LoadTools(aPackageName: string);
begin
  if World = nil then exit;
  MainTool.LoadTools(aPackageName)
end;


procedure TOrbisPanel.ClearMap(aMap: string);
begin
  if World = nil then exit;
  MainTool.ClearMap(aMap);
end;

procedure TOrbisPanel.AddToMap(aMap, aLayer: string);
begin
  if World = nil then exit;
  MainTool.AddToMap(aMap, aLayer);
end;


procedure TOrbisPanel.ShowMap(aMap: string);
begin
  if World = nil then exit;

  if aMap <> '' then
     FActualMap:= aMap;

  TMainTool(world.MainTool).ShowMap(FActualMap);
end;

procedure TOrbisPanel.ShowModel(aModel: TElement);
begin
  if World = nil then exit;
  TMainTool(world.MainTool).ShowModel(aModel);
end;


procedure TOrbisPanel.PrepareMap(aMap: string);
var
  aThem: TRectEntity;
  aLay: TRectEntity;
begin
  if World = nil then exit;
  aThem:= MainTool.GetMapByName(aMap);
  if aThem.Count = 0 then
     exit;

  MainTool.SetInterpret;

  ImageTool.input:= aThem;
end;


procedure TOrbisPanel.ReView;
begin
  if World = nil then exit;
  TMainTool(world.MainTool).ReView;
end;


function TOrbisPanel.Print: longInt;
begin
  if World = nil then exit;
  TMainTool(world.MainTool).ImageTool.PostMsg(tm_Print,
            TMainTool(world.MainTool).ImageTool, nil, false);
end;


function TOrbisPanel.SelObjProperty(aPropertyName: string): string;
begin
  result:= '';
//@  if not IsSelObj then exit;
  result:= SelectedObject.GetPropertyStringByName(aPropertyName);
end;


procedure TOrbisPanel.HandleMessage(var aMessage: TMessage);
var
  Handled: boolean;
  aMsg: TMsg;
{  TMsg = packed record
    hwnd: HWND;
    message: UINT;
    wParam: WPARAM;
    lParam: LPARAM;
    time: DWORD;
    pt: TPoint;
  end;}
begin
  aMsg.WParam:= aMessage.WParam;
  aMsg.LParam:= aMessage.LParam;
  aMsg.message:= aMessage.Msg;

  World.HandleWinMessage(aMsg, Handled);
end;



function TOrbisPanel.GetActiveTool: string;
begin
  result:= '';
  if World = nil then exit;
  FLastTool:= MainTool.ImageTool.ActiveTool;
  result:= FLastTool.Name
end;



procedure TOrbisPanel.SetActiveTool(const Value: string);
var
  aEl: TElement;
  aTool: TTool;
begin
  if World = nil then exit;

  if (Value = '') then
  begin
    if FLastTool = nil then
       FLastTool:= MainTool.ImageTool.ActiveTool;

    if (FLastTool is TInteractiveTool) then
    // Vypnut aktivnho nstroje
    begin
      TInteractiveTool(FLastTool).On_DeActivate;
      FLastTool:= nil;
      MainTool.ImageTool.ActiveTool:= FLastTool;
      Image.Cursor:= 0;
    end;

    exit;
  end;


  aTool:= MainTool.ImageTool.GetToolByName(Value);

  if (FLastTool is TInteractiveTool) then
  begin
    if (aTool = FLastTool) then
    begin
      TInteractiveTool(FLastTool).On_DeActivate;
      TInteractiveTool(FLastTool).On_Activate;
      exit;
    end
    else
      TInteractiveTool(FLastTool).On_DeActivate;
  end;

  if aTool = nil then
  begin
    TElement(aTool):= CreateElementByName(Value);
    if not (aTool is TInteractiveTool) then exit;
    FLastTool:= aTool;
    TFormToolManager(MainTool.ImageTool).NextTool:= TInteractiveTool(FLastTool)
  end;

  if aTool = nil then exit;
  FLastTool:= aTool;

  MainTool.ImageTool.ActiveTool:= FLastTool;
  Image.Cursor:= TInteractiveTool(MainTool.ImageTool.ActiveTool).Cursor;

  if (FLastTool is TInteractiveTool) then
  begin
    TInteractiveTool(FLastTool).On_Activate;
  end;
end;


function TOrbisPanel.GetObjectProperty(aPropertyName: string): string;
begin
  result:= '';
  if World = nil then exit;
  if SelectedObject = nil then exit;
  result:= SelectedObject.GetPropertyStringByName(aPropertyName);
end;


procedure TOrbisPanel.SetObjectProperty(aPropertyName, aValue: string);
begin
  if World = nil then exit;
  if SelectedObject = nil then exit;
  SelectedObject.SetPropertyStringByName(aPropertyName, aValue)
end;


function TOrbisPanel.GetToolProperty(aToolName, aPropertyName: string): string;
begin
  result:= '';
  if World = nil then exit;
  if ImageTool.Tools.Count < 1 then exit;

  FLastTool:= TTool(ImageTool.Tools.FindElementName(aToolName));

  if not (FLastTool is TTool) then
     exit;
  result:= FLastTool.GetPropertyStringByName(aPropertyName)
end;


procedure TOrbisPanel.SetToolProperty(aToolName, aPropertyName, aValue: string);
begin
  if World = nil then exit;
  FLastTool:= MainTool.GetToolByName(aToolName);
  if FLastTool = nil then exit;
  FLastTool.SetPropertyStringByName(aPropertyName, aValue)
end;



procedure TOrbisPanel.CreateTool(aToolClass, aToolName: string);
begin
  if World = nil then exit;
  FLastTool:= MainTool.CreateTool(aToolClass, aToolName);
end;

procedure TOrbisPanel.FreeTool(aToolName: string);
var
  aTool: TTool;
begin
  if World = nil then exit;
  aTool:= MainTool.GetToolByName(aToolName);
  if aTool = nil then exit;
  MainTool.FreeTool(aTool);
end;


procedure TOrbisPanel.ExecuteTool(aToolName: string);
begin
  if World = nil then exit;
  FLastTool:= MainTool.GetToolByName(aToolName);
  if FLastTool = nil then Exit;
  FLastTool.DoIt;
  if not assigned(FOnToolDone) then exit;
  OnToolDone(aToolName);
end;



function TOrbisPanel.GetImageTool: TTechnology;
begin
  result:= nil;
  if World = nil then exit;
  result:= MainTool.ImageTool
end;


function TOrbisPanel.GetTools: string;
var
  I: integer;
begin
  result:= '';
  if World = nil then exit;
  if ImageTool.Tools.Count < 1 then exit;
  result:= '"' + ImageTool.Tool[0].Name + '"';

  for I:= 1 to ImageTool.Tools.Count-1 do
      result:= result + ',"' + ImageTool.Tool[I].Name + '"';
end;



function TOrbisPanel.GetLayers: string;
var
  aLayers: TEntity;
  I: integer;
begin
  result:= '';
  if World = nil then exit;
  aLayers:= MainTool.Layers;
  if aLayers.Count = 0 then exit;
  result:= '"' + aLayers.Elements[0].Name + '"';
  for I:= 1 to aLayers.Count-1 do
  begin
    result:= result + ',"' + aLayers.Elements[I].Name + '"';
  end
end;



function TOrbisPanel.GetMaps: string;
var
  aMaps: TEntity;
  I: integer;
begin
  result:= '';
  if World = nil then exit;
  aMaps:= MainTool.Thematic;
  if aMaps.Count = 0 then exit;
  result:= '"' + aMaps.Elements[0].Name + '"';
  for I:= 1 to aMaps.Count-1 do
  begin
    result:= result + ',"' + aMaps.Elements[I].Name + '"';
  end
end;



function TOrbisPanel.GetWorldX: double;
begin
  result:= 0;
  if MainTool = nil then exit;
  if MainTool.Projection = nil then exit;
  result:= MainTool.Projection.GetModelX(PixelX);
end;

function TOrbisPanel.GetWorldY: double;
begin
  result:= 0;
  if MainTool = nil then exit;
  if MainTool.Projection = nil then exit;
  result:= MainTool.Projection.GetModelY(PixelY);
end;

procedure TOrbisPanel.SetWorldX(const Value: double);
begin
  if MainTool = nil then exit;
  if MainTool.Projection = nil then exit;
  PixelX:= MainTool.Projection.GetPageX(Value);
end;

procedure TOrbisPanel.SetWorldY(const Value: double);
begin
  if MainTool = nil then exit;
  if MainTool.Projection = nil then exit;
  PixelY:= MainTool.Projection.GetPageY(Value);
end;



function TOrbisPanel.GetWorldHeight: double;
var
  aDR: TDoubleRect;
begin
  result:= 0;
  if MainTool = nil then exit;
  aDR:= MainTool.Projection.ModelView;
  result:= abs(aDR.MaxY - aDr.MinY);
end;

function TOrbisPanel.GetWorldWidth: double;
var
  aDR: TDoubleRect;
begin
  result:= 0;
  if MainTool = nil then exit;
  aDR:= MainTool.Projection.ModelView;
  result:= abs(aDR.MaxX - aDr.MinX);
end;

function TOrbisPanel.GetViewMaxX: double;
var
  aDR: TDoubleRect;
begin
  result:= 0;
  if MainTool = nil then exit;
  aDR:= MainTool.Projection.ModelView;
  result:= aDr.MaxX;
end;

function TOrbisPanel.GetViewMaxY: double;
var
  aDR: TDoubleRect;
begin
  result:= 0;
  if MainTool = nil then exit;
  aDR:= MainTool.Projection.ModelView;
  result:= aDr.MaxY;
end;

function TOrbisPanel.GetViewMinX: double;
var
  aDR: TDoubleRect;
begin
  result:= 0;
  if MainTool = nil then exit;
  aDR:= MainTool.Projection.ModelView;
  result:= aDr.MinX;
end;

function TOrbisPanel.GetViewMinY: double;
var
  aDR: TDoubleRect;
begin
  result:= 0;
  if MainTool = nil then exit;
  aDR:= MainTool.Projection.ModelView;
  result:= aDr.MinY;
end;

procedure TOrbisPanel.SetUserKey(const Value: word; const Shift: TShiftState);
begin
 if not (FLastTool is TInteractiveTool) then exit;
 TInteractiveTool(FLastTool).PostKeyEvent(tm_KeyDown, FLastTool, Value, Shift);
end;

function TOrbisPanel.GetSelectedObject: TElement;
begin
  result:= nil;
  if MainTool = nil then exit;
  result:= MainTool.SelectedObject;
end;

function TOrbisPanel.GetVirtualScale: double;
begin
  result:= 0;
  if World = nil then exit;
  result:= MainTool.ImageTool.Projection.VirtualScale;
end;

procedure TOrbisPanel.SetVirtualScale(const Value: double);
begin
 if World = nil then exit;
 MainTool.ImageTool.Projection.VirtualScale:= Value
end;

function TOrbisPanel.GetServiceDirectory: string;
begin
 Result:= '';
 if World = nil then exit;
 Result:= MainTool.Options.ServiceDirectory;
end;

procedure TOrbisPanel.SetServiceDirectory(const Value: string);
begin
 if World = nil then exit;
 MainTool.Options.ServiceDirectory:= Value;
end;

function TOrbisPanel.GetInitialized: Boolean;
begin
  result:= World <> nil;
end;

function TOrbisPanel.GetBusy: Boolean;
begin
  Result:= FBusy
end;


function TOrbisPanel.LayersObject: TEntity;
begin
  result:= nil;
  if World = nil then exit;
  result:= MainTool.Layers;
end;

function TOrbisPanel.MapsObject: TEntity;
begin
  result:= nil;
  if World = nil then exit;
  result:= MainTool.Thematic;
end;

function TOrbisPanel.GetUseProtocol: boolean;
begin
  result:= false;
  if MainTool = nil then exit;
  result:= MainTool.UseProtocol;
end;

procedure TOrbisPanel.SetUseProtocol(const Value: boolean);
begin
  if MainTool = nil then exit;
  MainTool.UseProtocol:= Value;
end;


procedure Register;
begin
  RegisterComponents('Orbis', [TOrbisPanel]);
end;

end.
