unit f_edLay;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs, extctrls,
// INTERPRET
  E_LayCon, StdCtrls;


type
  Tf_edLayForm = class(TForm)
    rgSymbol: TRadioGroup;
    cbShowLayer: TCheckBox;
    edSymbolItem: TEdit;
    GroupBox1: TGroupBox;
    rgColorSource: TRadioGroup;
    panColor: TPanel;
    edColorItem: TEdit;
    rgColorHow: TRadioGroup;
    btUse: TButton;
    edMinScaleNumber: TEdit;
    edMaxScaleNumber: TEdit;
    Label1: TLabel;
    Label2: TLabel;
    gbText: TGroupBox;
    cbAsText: TCheckBox;
    edTextItem: TEdit;
    cbTextAbsSize: TCheckBox;
    edTextHeight: TEdit;
    Label3: TLabel;
    edLayName: TEdit;
    rgLayType: TRadioGroup;
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure cbInterpretChange(Sender: TObject);
    procedure cbShowLayerClick(Sender: TObject);
    procedure panColorClick(Sender: TObject);
    procedure btUseClick(Sender: TObject);
    procedure rgColorSourceClick(Sender: TObject);
    procedure rgColorHowClick(Sender: TObject);
    procedure cbAsTextClick(Sender: TObject);
    procedure cbTextAbsSizeClick(Sender: TObject);
    procedure edTextItemChange(Sender: TObject);
    procedure edLayNameChange(Sender: TObject);
    procedure rgLayTypeClick(Sender: TObject);
  private
    FLayer: TLayerContainer;
    procedure SetEditor(const Value: TLayerContainer);
    procedure SetInterpret;
    procedure SetInterpretProps;
    function GetIsChanged: Boolean;
    procedure SetIsChanged(const Value: Boolean);
  public
    FPanel: TPanel;
  published
    property Layer: TLayerContainer read FLayer write SetEditor;
    property IsChanged: Boolean read GetIsChanged write SetIsChanged;
  end;


implementation

{$R *.DFM}

uses
// KERNEL
   Elements,
   e_Class,
// SPACE
   e_Space,
// INTERPRET
   I_Basic,
   I_Gener,
// MAPPER
   I_GArea,
   I_GLin,
   I_GPoint,
   I_GText,
// TOPOL
   I_Top_P,
   I_Top_B,
   I_Top_L,
   I_Top_T,
{$IFDEF RAS}
// RASTER
  I_Raster,
  e_Raster,
{$ENDIF}

  f_LayPan,
  F_ColTab;

procedure Tf_edLayForm.FormClose(Sender: TObject;
  var Action: TCloseAction);
begin
  Action:= caFree
end;

procedure Tf_edLayForm.cbInterpretChange(Sender: TObject);
var
  aEC: TElementClass;

begin
end;



procedure Tf_edLayForm.SetInterpret;
var
  aEC: TElementClass;

begin
  aEC:= nil;

  if FLayer = nil then
     Exit;

  if FLayer.LayerType = 'PNT' then
  begin
    aEC:= TGeneralPointInterpret;
    FLayer.InterpretedClass:= TPointElement;
    rgLayType.ItemIndex:= 0;
  end;
  if FLayer.LayerType = 'ARE' then
  begin
    aEC:= TGeneralAreaInterpret;
    FLayer.InterpretedClass:= TAreaElement;
    rgLayType.ItemIndex:= 2;
  end;

  if FLayer.LayerType = 'LIN' then
  begin
    aEC:= TGeneralLineInterpret;
    FLayer.InterpretedClass:= TPolyLineElement;
    rgLayType.ItemIndex:= 1;
  end;

  if FLayer.LayerType = 'TXT' then
  begin
    aEC:= TGeneralTextInterpret;
    FLayer.InterpretedClass:= TTextElement;
    rgLayType.ItemIndex:= 3;
  end;

{$IFDEF RAS}
  if FLayer.LayerType = 'RAS' then
  begin
    aEC:= TRasterInterpret;
    FLayer.InterpretedClass:= TRaster;
  end;
{$ENDIF}

  if FLayer.AsText then
  begin
    aEC:= TGeneralTextInterpret;
  end;


  if FLayer.Interpret <> nil then
  begin
    if FLayer.Interpret is  aEC then
       Exit;
    FLayer.Interpret.Free;
  end;


  if aEC = nil then
     Exit;

  FLayer.Interpret:= TInterpret(aEC.Create);

  if FLayer.Interpret is TGeneralColorInterpret then
     TGeneralColorInterpret(FLayer.Interpret).ColorByTopol:= True;
  if FLayer.Interpret is TGeneralColorInterpret then
     TGeneralColorInterpret(FLayer.Interpret).PropColor:= 'BARVA';
  if FLayer.Interpret is TGeneralBrushInterpret then
     TGeneralBrushInterpret(FLayer.Interpret).PropBrush:= 'ZNACKA';

  if FLayer.Interpret is TGeneralTextInterpret then
  begin
    TGeneralTextInterpret(FLayer.Interpret).PropText:= edTextItem.Text;
    TGeneralTextInterpret(FLayer.Interpret).ValFont:= 'ARIAL';
    TGeneralTextInterpret(FLayer.Interpret).ValHeight:= 10;
  end;
end;



procedure Tf_edLayForm.SetInterpretProps;
var
  aGCI: TGeneralColorInterpret;
  aGBI: TGeneralBrushInterpret;
  aGTI: TGeneralTextInterpret;
  aScale: Double;
  aCode: integer;
begin
  FLayer.AsText:= false;

  val(edMinScaleNumber.Text, aScale, aCode);
  if aCode > 0 then
  begin
    ShowMessage('patn zadan minimln mtkov slo');
    Exit;
  end;
  FLayer.MinScaleNumber:= aScale;

  val(edMaxScaleNumber.Text, aScale, aCode);
  if aCode > 0 then
  begin
    ShowMessage('patn zadan maximln mtkov slo');
    Exit;
  end;
  FLayer.MaxScaleNumber:= aScale;

  if FLayer.Interpret is TGeneralColorInterpret then
  begin
    aGCI:= TGeneralColorInterpret(FLayer.Interpret);
    case rgColorSource.ItemIndex of
    0: begin
         aGCI.ColorByTopol:= False;
         aGCI.UseColor:= True;
         aGCI.Color:= PanColor.Color;
       end;
    1: begin
         aGCI.UseColor:= false;
         aGCI.ColorByTopol:= True;
         aGCI.PropColor:= edColorItem.Text;
       end
    end;

    case rgColorHow.ItemIndex of
    0: begin
         aGCI.ColorByTopol:= False;
         aGCI.ColorByPalette:= False;
       end;
    1: begin
         aGCI.ColorByTopol:= true;
         aGCI.ColorByPalette:= false;
       end;
    2: begin
         aGCI.ColorByTopol:= false;
         aGCI.ColorByPalette:= true;
       end;
     end;
  end;

  if FLayer.Interpret is TGeneralBrushInterpret then
  begin
    aGBI:= TGeneralBrushInterpret(FLayer.Interpret);
    aGBI.PropBrush:= 'ZNACKA';
  end;


  if cbAsText.Checked then
  begin
    FLayer.AsText:= True;
    if not (FLayer.Interpret is TGeneralTextInterpret) then
    begin
      FLayer.Interpret.Free;
      FLayer.Interpret:= TGeneralTextInterpret.Create;
    end;

    aGTI:= TGeneralTextInterpret(FLayer.Interpret);
    aGTI.PropText:= edTextItem.Text;
    aGTI.ValFont:= 'ARIAL';
    aGTI.PropX:= 'X';
    aGTI.PropY:= 'Y';
    aGTI.ProcessClass:= FLayer.InterpretedClass;
    aGTI.ValHeight:= StrToInt(edTextHeight.Text)/1000;
    aGTI.AbsSize:= cbTextAbsSize.Checked;
  end;

  TLayerPanel(FPanel).Layer:= FLayer
end;



procedure Tf_edLayForm.SetEditor(const Value: TLayerContainer);
var
  GCI: TGeneralColorInterpret;
  aVal: Boolean;
begin
  FLayer := Value;
  if FLayer = nil then
     exit;

//  if FLayer.Interpret = nil then
  SetInterpret;

  aVal:= not FLayer.SwitchedOff;
  cbShowLayer.Checked:= aVal;

  panColor.Color:= clBtnFace;
  if (FLayer.Interpret is TGeneralColorInterpret) then
  begin
    GCI:= TGeneralColorInterpret(FLayer.Interpret);
    panColor.Color:= GCI.Color;
    if GCI.UseColor then
    begin
      rgColorSource.ItemIndex:= 0;
    end
{
    else
    if GCI.ToolColor <> nil then
    begin
      rgColorSource.ItemIndex:= 1;
    end
}
    else
    if GCI.PropColor <> '' then
    begin
      rgColorSource.ItemIndex:= 1;
      edColorItem.Text:= GCI.PropColor;
    end;

    if GCI.ColorByTopol then
       rgColorHow.ItemIndex:= 1
    else
    if GCI.ColorByPalette then
       rgColorHow.ItemIndex:= 2
    else
       rgColorHow.ItemIndex:= 0;
  end;

  if (FLayer.Interpret is TGeneralBrushInterpret) then
  begin
    if TGeneralBrushInterpret(FLayer.Interpret).ToolBrush <> nil then
    begin
      rgSymbol.ItemIndex:= 1;
    end
    else
    if TGeneralBrushInterpret(FLayer.Interpret).PropBrush <> '' then
    begin
      rgSymbol.ItemIndex:= 2;
//      edColorItem.Text:= TGeneralColorInterpret(FLayer.Interpret).PropColor;
    end;
  end;

  if FLayer.Interpret is TGeneralTextInterpret then
  begin
    cbAsText.Checked:= True;
    edTextItem.Text:= TGeneralTextInterpret(FLayer.Interpret).PropText;
    TGeneralTextInterpret(FLayer.Interpret).ValFont:= 'ARIAL';
    edTextHeight.Text:= IntToStr(round(TGeneralTextInterpret(FLayer.Interpret).ValHeight * 1000));
    cbTextAbsSize.Checked:= TGeneralTextInterpret(FLayer.Interpret).AbsSize;
  end;
end;



procedure Tf_edLayForm.cbShowLayerClick(Sender: TObject);
begin
  if Layer = nil then Exit;
  Layer.SwitchedOff:= not cbShowLayer.Checked;
end;

procedure Tf_edLayForm.panColorClick(Sender: TObject);
var
  aCD: TColorDialog;
begin
  aCD:= TColorDialog.Create(Self);
  if aCD.Execute then
  begin
     panColor.Color:= aCD.Color;
     IsChanged:= true
  end;
  aCD.Free;
end;


procedure Tf_edLayForm.btUseClick(Sender: TObject);
begin
  SetInterpretProps;
  IsChanged:= False;
end;

procedure Tf_edLayForm.rgColorSourceClick(Sender: TObject);
begin
  IsChanged:= true
end;

function Tf_edLayForm.GetIsChanged: Boolean;
begin
  Result:= btUse.Enabled
end;

procedure Tf_edLayForm.SetIsChanged(const Value: Boolean);
begin
  btUse.Enabled:= Value;
end;

procedure Tf_edLayForm.rgColorHowClick(Sender: TObject);
var
  aCTF: TColorTableForm;
begin
  if rgColorHow.ItemIndex = 2 then
  begin
    aCTF:= TColorTableForm.Create(Application);
    aCTF.Item:= edColorItem.Text;
    aCTF.ShowModal;
    aCTF.Free
  end;

  IsChanged:= True;
end;

procedure Tf_edLayForm.cbAsTextClick(Sender: TObject);
begin
  IsChanged:= true
end;

procedure Tf_edLayForm.cbTextAbsSizeClick(Sender: TObject);
begin
  IsChanged:= true
end;

procedure Tf_edLayForm.edTextItemChange(Sender: TObject);
begin
  IsChanged:= true
end;

procedure Tf_edLayForm.edLayNameChange(Sender: TObject);
begin
  FLayer.Name:= edLayName.Text;
  IsChanged:= True;

  TLayerPanel(FPanel).Layer:= FLayer
end;

procedure Tf_edLayForm.rgLayTypeClick(Sender: TObject);
var
  aLT: string;
begin
  if FLayer = nil then
     exit;
  case rgLayType.ItemIndex of
  0:
   begin
     aLT:= 'PNT';
   end;
  1:
   begin
     aLT:= 'LIN';
   end;
  2:
   begin
     aLT:= 'ARE';
   end;
  3:
   begin
     aLT:= 'TXT';
   end;
  4:
   begin
     aLT:= 'RAS';
   end;
  else
  end;
  FLayer.LayerType:= aLT;
  FLayer.SetLayerInterpretForType;

  IsChanged:= True;

//  TLayerPanel(FPanel).Layer:= FLayer
end;


end.

