unit T_editor;

INTERFACE

uses
   Controls,
   Graphics,
   Wintypes,
   Elements,
//   e_Desktp,
   g_Elems,
   m_InterA,
   T_InterA,
   t_Recurs;

type

TDragStatus = Set of (dsLeft, dsRight, dsTop, dsBottom);


TDrawLineTool = class(TInteractiveTool)
 private
   FDrawn  : boolean;
   function  GetAngle: double;
   procedure TakePoint;
 protected
   PrevCoord: TDoubleCoord;
   RefPoint: Wintypes.TPoint;
   DragStatus: TDragStatus;
   function  GetName: string; override;
 public
   Pnt1, Pnt2:  Wintypes.TPoint;
   procedure On_Erase(aCoord: TDoubleCoord); virtual;
   procedure On_Draw (aCoord: TDoubleCoord); virtual;
   procedure On_Point(aCoord: TDoubleCoord); virtual;
   procedure On_Line(aCrd1, aCrd2: TDoubleCoord); virtual;
   procedure On_tm_Keydown(var aMessage: TOrbisKeyEvent; var Handled: boolean); override;
   procedure On_tm_MouseDown(var aMessage: TOrbisMouseEvent; var Handled: boolean); override;
   procedure On_tm_MouseUp(var aMessage: TOrbisMouseEvent; var Handled: boolean); override;
   procedure On_tm_MouseMove(var aMessage: TOrbisMouseEvent; var Handled: boolean); override;
   procedure on_tm_SetDrawnToFalse(var Handled: boolean);

   procedure HandleMessage(var aMessage: TOrbisMessage; var Handled: boolean); override;
 published
   property Drawn: boolean read FDrawn write FDrawn;
   property Angle: double read GetAngle;
end;




TDrawRectTool = class(TInteractiveTool)
 private
   FDrawn  : boolean;
 protected
   PrevCoord: TDoubleCoord;
   RefPoint: Wintypes.TPoint;
   DragStatus: TDragStatus;
   function  GetName: string; override;
 public
   ShiftRect: TRect;
   procedure On_Erase(aCoord: TDoubleCoord); virtual;
   procedure On_Draw (aCoord: TDoubleCoord); virtual;
   procedure On_Point(aCoord: TDoubleCoord); virtual;
   procedure On_Rect(aRect: TDoubleRect); virtual;
   procedure On_tm_Keydown(var aMessage: TOrbisKeyEvent; var Handled: boolean); override;
   procedure On_tm_MouseDown(var aMessage: TOrbisMouseEvent; var Handled: boolean); override;
   procedure On_tm_MouseUp(var aMessage: TOrbisMouseEvent; var Handled: boolean); override;
   procedure On_tm_MouseMove(var aMessage: TOrbisMouseEvent; var Handled: boolean); override;
 published
   property Drawn: boolean read FDrawn write FDrawn;
end;



TEditor = class(TInteractiveTool)
protected
   function  GetCursor: TCursor; override;
public
   procedure On_tm_MouseMove(var aMessage: TOrbisMouseEvent; var Handled: boolean); override;
   procedure HandleMessage(var aMessage: TOrbisMessage; var Handled: boolean); override;
end;

TAreasEditor = class(TEditor)

end;


IMPLEMENTATION

uses
   WinProcs,
   Classes,
   Dialogs,
   forms,
   MathLib,
   e_Device,
   e_View,
   e_Class,
   {$IFNDEF NOMAP}
//   f_EdText,
//   f_MSEdit,
//   t_MSEdit,
   {$ENDIF}
   m_Orbis,
   m_Proj,
   u_Cursor;



{ tdrawLineTool }



function  tDrawLineTool.GetName: string;
begin
   result := 'Kresltko lini';
end;

procedure tDrawLineTool.On_Erase(aCoord: TDoubleCoord);
begin
end;

procedure tDrawLineTool.On_Draw(aCoord: TDoubleCoord);
begin
end;


procedure tDrawLineTool.TakePoint;
var
  C1: TDoubleCoord;
begin
  C1 := Device.Projection.GetModelPoint(Pnt1.X, Pnt1.Y);
  on_Point(C1)
end;


procedure tDrawLineTool.On_Point(aCoord: TDoubleCoord);
begin
end;





procedure tDrawLineTool.On_Line(aCrd1, aCrd2: TDoubleCoord);
begin

end;


procedure tDrawLineTool.On_tm_Keydown(var aMessage: TOrbisKeyEvent; var Handled: boolean);
var
  aPoint: WinTypes.TPoint;
begin
   GetCursorPos(aPoint);
   case aMessage.Key of
   VK_ESCAPE: PostMsgToOwners(tm_BreakInterpret, nil, DontKill);
   VK_Return: begin
                On_Erase(PrevCoord);
                Drawn:= false;
                TakePoint;
              end;
   VK_Down: begin
              SetCursorPos(aPoint.X, aPoint.Y+1)
            end;
   VK_Left: begin
             SetCursorPos(aPoint.X-1, aPoint.Y)
            end;
   VK_Right: begin
               SetCursorPos(aPoint.X+1, aPoint.Y)
             end;
   VK_Up: begin
           SetCursorPos(aPoint.X, aPoint.Y-1)
          end;
   end;
end;




procedure tDrawLineTool.On_tm_MouseDown(var aMessage: TOrbisMouseEvent; var Handled: boolean);
begin
   if Drawn then
   begin
      On_Erase(PrevCoord);
      Drawn:= false
   end;

   if aMessage.Button = mbLeft then
   begin
      Device.Pen.Mode  := pmXOR;
      Device.Pen.Color := clLime;
      Device.Pen.Style := psSolid;
      Device.Brush.Style := bsClear;
      Pnt1.X := aMessage.x;
      Pnt1.Y := aMessage.y;
      Pnt2.X := Pnt1.X;
      Pnt2.Y := Pnt1.Y;
      TCanvasDevice(Device).Canvas.Pen.Width := 1;

      TCanvasDevice(Device).Canvas.MoveTo(Pnt1.X, Pnt1.Y);
      TCanvasDevice(Device).Canvas.LineTo(Pnt2.X, Pnt2.Y);
   end;
   Handled := true;
end;


procedure tDrawLineTool.On_tm_MouseUp(var aMessage: TOrbisMouseEvent; var Handled: boolean);
var
   C1, C2: TDoubleCoord;
begin
    Handled := true;
{    if not aMessage.ActiveDevice then exit; }

    if Drawn then
    begin
       On_Erase(PrevCoord);
       Drawn:= false
    end;

    if aMessage.Button = mbLeft then
    begin
      Device.Pen.Mode  := pmXOR;
      Device.Pen.Color := clLime;
      Device.Pen.Style := psSolid;

      TCanvasDevice(Device).Canvas.MoveTo(Pnt1.X, Pnt1.Y);
      TCanvasDevice(Device).Canvas.LineTo(Pnt2.X, Pnt2.Y);

       if (abs(Pnt1.X-Pnt2.X) <= 4) and (abs(Pnt1.Y-Pnt2.Y) <= 4) then
       begin
          Pnt1.X := aMessage.x;
          Pnt1.Y := aMessage.y;
          Pnt2:= Pnt1;
          C1 := Device.Projection.GetModelPoint(Pnt1.X, Pnt1.Y);
          on_Point(C1)
       end
       else
       begin
         C1 := Device.Projection.GetModelPoint(Pnt1.X, Pnt1.Y);
         Pnt2.X := aMessage.x;
         Pnt2.Y := aMessage.y;
         C2 := Device.Projection.GetModelPoint(Pnt2.X, Pnt2.Y);
         on_Line(C1, C2);
       end;
       TCanvasDevice(Device).Canvas.Pen.Mode  := pmCopy;
       TCanvasDevice(Device).Canvas.Pen.Color := clBlack;
       TCanvasDevice(Device).Canvas.Pen.Width := 1;
    end;
end;


function TDrawLineTool.GetAngle: double;
begin
  result:= ATan2M(Pnt2.X-Pnt1.X, Pnt2.Y-Pnt1.Y)
end;


procedure tDrawLineTool.On_tm_MouseMove(var aMessage: TOrbisMouseEvent; var Handled: boolean);
var
   C: TDoubleCoord;
begin
   Handled := false;
{   if not aMessage.ActiveDevice then exit; }
   C := Device.Projection.GetModelPoint(aMessage.X, aMessage.Y);

   if Drawn then
   begin
      On_Erase(PrevCoord);
      Drawn:= false
   end;

   if not Drawn then
   begin
      On_Draw(C);
      Drawn:= true;
      PrevCoord:= C
   end;

   if  ssLeft in aMessage.Shift then
   begin
      Device.Pen.Mode  := pmXOR;
      Device.Pen.Color := clLime;
      Device.Pen.Style := psSolid;
      Device.Brush.Style := bsClear;
      TCanvasDevice(Device).Canvas.Pen.Width := 1;

      TCanvasDevice(Device).Canvas.MoveTo(Pnt1.X, Pnt1.Y);
      TCanvasDevice(Device).Canvas.LineTo(Pnt2.X, Pnt2.Y);

      Pnt2.X  := aMessage.x;
      Pnt2.Y  := aMessage.y;

      TCanvasDevice(Device).Canvas.MoveTo(Pnt1.X, Pnt1.Y);
      TCanvasDevice(Device).Canvas.LineTo(Pnt2.X, Pnt2.Y);
   end
   else
   begin
     Pnt1.X := aMessage.x;
     Pnt1.Y := aMessage.y;
     Pnt2:= Pnt1
   end
end;



procedure tDrawLineTool.on_tm_SetDrawnToFalse(var Handled: boolean);
begin
  Drawn:= false;
end;




procedure TDrawLineTool.HandleMessage(var aMessage: TOrbisMessage; var Handled: boolean);
begin
   Handled := false;

   inherited HandleMessage(aMessage, Handled);
   if Handled then exit;
{
   case aMessage.Message of
      tm_SetDrawnToFalse:      on_tm_SetDrawnToFalse(Handled);
   end;
}
end;





{ TDrawRectTool }

function  TDrawRectTool.GetName: string;
begin
   result := 'Kresltko obdlnku';
end;

procedure TDrawRectTool.On_Erase(aCoord: TDoubleCoord);
begin
end;

procedure TDrawRectTool.On_Draw(aCoord: TDoubleCoord);
begin
end;


procedure TDrawRectTool.On_Point(aCoord: TDoubleCoord);
begin

end;

procedure TDrawRectTool.On_Rect(aRect: TDoubleRect);
begin

end;


procedure TDrawRectTool.On_tm_Keydown(var aMessage: TOrbisKeyEvent; var Handled: boolean);
begin
   if aMessage.Key = VK_ESCAPE then PostMsgToOwners(tm_BreakInterpret, nil, DontKill);
end;






procedure TDrawRectTool.On_tm_MouseDown(var aMessage: TOrbisMouseEvent; var Handled: boolean);
begin
   if Drawn then
   begin
      On_Erase(PrevCoord);
      Drawn:= false
   end;

   if aMessage.Button = mbLeft then
   begin
      Device.Pen.Mode  := pmXOR;
      Device.Pen.Color := clLime;
      Device.Pen.Style := psSolid;
      Device.Brush.Style := bsClear;
      ShiftRect := Rect(aMessage.x,aMessage.y,aMessage.x,aMessage.y);
      TCanvasDevice(Device).Canvas.Pen.Width := 1;
      with ShiftRect do TCanvasDevice(Device).Canvas.Rectangle(Left, Top, Right, Bottom);
   end;
   Handled := true;
end;

procedure TDrawRectTool.On_tm_MouseUp(var aMessage: TOrbisMouseEvent; var Handled: boolean);
var
   R: TDoubleRect;
   C: TDoubleCoord;
begin
    Handled := true;
    if not aMessage.ActiveDevice then exit;

    if Drawn then
    begin
       On_Erase(PrevCoord);
       Drawn:= false
    end;

    if aMessage.Button = mbLeft then
    begin
       with ShiftRect do TCanvasDevice(Device).Canvas.Rectangle(Left, Top, Right, Bottom);
       TCanvasDevice(Device).Canvas.Pen.Mode  := pmCopy;
       TCanvasDevice(Device).Canvas.Pen.Color := clBlack;
       TCanvasDevice(Device).Canvas.Pen.Width := 1;
       with ShiftRect do
       begin
          if (Right-Left <= 2) and (Bottom - Top <= 2) then
          begin
             C := Device.Projection.GetModelPoint(ShiftRect.Left, ShiftRect.Bottom);
             on_Point(C);
          end
          else
          begin
             with ShiftRect do R := Device.Projection.GetModelRect(Left, Top, Right, Bottom);
             on_Rect(R);
          end;
       end;
    end;
end;


procedure TDrawRectTool.On_tm_MouseMove(var aMessage: TOrbisMouseEvent; var Handled: boolean);
var
   C: TDoubleCoord;
begin
   Handled := false;
   if not aMessage.ActiveDevice then exit;
   C := Device.Projection.GetModelPoint(aMessage.X, aMessage.Y);

   if Drawn then
   begin
      On_Erase(PrevCoord);
      Drawn:= false
   end;

   if not Drawn then
   begin
      On_Draw(C);
      Drawn:= true;
      PrevCoord:= C
   end;

   if  ssLeft in aMessage.Shift then
   begin
      Device.Pen.Mode  := pmXOR;
      Device.Pen.Color := clLime;
      Device.Pen.Style := psSolid;
      Device.Brush.Style := bsClear;
      TCanvasDevice(Device).Canvas.Pen.Width := 1;
      with ShiftRect do TCanvasDevice(Device).Canvas.Rectangle(Left, Top, Right, Bottom);
      ShiftRect.Right  := aMessage.x;
      ShiftRect.Bottom := aMessage.y;
      with ShiftRect do TCanvasDevice(Device).Canvas.Rectangle(Left, Top, Right, Bottom);
   end;
end;




procedure TEditor.HandleMessage(var aMessage: TOrbisMessage; var Handled: boolean);
begin
   Handled := false;

   inherited HandleMessage(aMessage, Handled);
   if Handled then exit;
{
   case aMessage.Message of
//      tm_ReView: on_tm_SetDrawnToFalse(Handled);
   end;
}
end;


procedure tEditor.On_tm_MouseMove(var aMessage: TOrbisMouseEvent; var Handled: boolean);
var
   C: TDoubleCoord;
begin
   Handled := false;
{   if not aMessage.ActiveDevice then exit; }
   C := Device.Projection.GetModelPoint(aMessage.X, aMessage.Y);

   if  ssLeft in aMessage.Shift then
   begin
      Device.Pen.Mode  := pmXOR;
      Device.Pen.Color := clLime;
      Device.Pen.Style := psSolid;
      Device.Brush.Style := bsClear;
      TCanvasDevice(Device).Canvas.Pen.Width := 1;


      TCanvasDevice(Device).Canvas.LineTo(aMessage.X, aMessage.Y);
      TCanvasDevice(Device).Canvas.Ellipse(100,100, aMessage.X, aMessage.Y);

   end
   else
   begin
   end
end;


function  TEditor.GetCursor: TCursor;
begin
   result := cr_NewBMP;
end;



BEGIN
   RegisterOrbisClass(TDrawLineTool);
   RegisterOrbisClass(TDrawRectTool);
   RegisterOrbisClass(TEditor);
END.
