unit T_ImgTol;

interface
uses
  ExtCtrls, Controls,
// KERNEL
   Elements,
   Tools,
   m_InterA,
   t_InterA;

type

TImageToolManager = class(TFormToolManager)
 private
   FImage: TImage;
    procedure SetImage(const Value: TImage);
 protected
   procedure   CreateForm; override;
   procedure   CreateTools; override;
   procedure   LinkDeviceToForm; override;
   function    GetVisible: boolean; override;
   procedure   SetVisible(aVisible: boolean);  override;
   procedure   SetActiveTool(aTool: TTool); override;
   function    GetFormCursor: TCursor; override;
   procedure   SetFormCursor(AValue: TCursor); override;
 public
   procedure   On_tm_RePaint(var Handled: boolean); override;
   procedure   SetNextTool(aTool: TInteractiveTool); override;
   function    AddTool(aTool: TTool): TTool; override;
   procedure   On_tm_MouseDown(var aMessage: TOrbisMouseEvent; var Handled: boolean); override;
   procedure   On_tm_MouseMove(var aMessage: TOrbisMouseEvent; var Handled: boolean); override;
   procedure   on_tm_Viewer(var Handled: boolean);
   procedure   on_tm_AreaSelector(var Handled: boolean);
   procedure   HandleMessage(var aMessage: TOrbisMessage; var Handled: boolean); override;
   constructor CreateImageTool(aImage: TImage);

   procedure   on_tm_Done(var aMessage: TOrbisMessage; var Handled: boolean); override;
 published
   property Image: TImage read FImage write SetImage;
end;



implementation

uses
   Forms, Classes, Dialogs,
// KERNEL
   Worlds,
// GLIB
   g_Elems,
// DEVICE
   D_STDImg,
// GUI
   T_SelAre,
   T_PanelShImg,

   m_Proj;



{ TImageToolManager }


procedure  TImageToolManager.SetActiveTool(aTool: TTool);
begin
   try
      aTool.Input:= input;
      ActiveToolIndex := Tools.IndexOf(aTool);
   except
      ActiveToolIndex := -1;
   end;
end;



procedure   TImageToolManager.SetNextTool(aTool: TInteractiveTool);
begin
   aTool.Device := Device;
   aTool.Manager:= Self;
   AddTool(aTool);
   ActiveTool := aTool;
end;


function    TImageToolManager.AddTool(aTool: TTool): TTool;
begin
    Tools.AddElement(aTool);
    Result:= aTool;
end;



constructor TImageToolManager.CreateImageTool(aImage: TImage);
begin
   inherited Create;
   SetImage(aImage);
end;


procedure   TImageToolManager.CreateForm;
var
  aData, aHier: TEntity;
begin
end;


procedure   TImageToolManager.LinkDeviceToForm;
begin
end;


procedure   TImageToolManager.CreateTools;
var
   T: TInterActiveTool;
begin
{$IFNDEF NOTIS}
   {
   NextTool := TSMO_10_Selector.Create;
   NextTool := TSMO_5_Selector.Create;
   }

   T := TPointSelector.Create;
   T.Name := 'Vybrn bod';
   NextTool := T;
   T := TLineSelector.Create;
   T.Name := 'Vybrn lini';
   NextTool := T;
   T := TAreaSelector.Create;
   T.Name := 'Vybrn ploch';
   NextTool := T;
   T := TViewer.Create;
   T.Name := 'VIEWER';
   NextTool := T;
   T := TShiftImage.Create;
   T.Name := 'Posun';
   NextTool := T;

{@
   T := TDrawLineTool.Create;
   T.Name := 'DRAWLINE';
   NextTool := T;
   T := TDrawRectTool.Create;
   T.Name := 'DRAWRECT';
   NextTool := T;
}

//   T := TPSkSelector.Create;
 //  T.Name := 'Vybrn porostn skupiny';
 //  TPSkSelector(T).StatusBar:= LesForm.StatusBar;
 //  NextTool := T;
//   TPSkSelector(T).OnSelect:= aHele;
//   AddElement(aHele);

(*
   T := TPointCollector.Create;
   T.Name := 'Snmn bod';
   NextTool := T;
   *)
{$ENDIF}
//   NextTool := TViewer.Create;
end;



procedure TImageToolManager.On_tm_MouseDown(var aMessage: TOrbisMouseEvent; var Handled: boolean);
begin
//  if aMessage.Button <> mbLeft then exit;

//  Image.BeginDrag(false, 2)
end;




procedure   TImageToolManager.On_tm_MouseMove(var aMessage: TOrbisMouseEvent; var Handled: boolean);
begin
  inherited;

  if ssLeft in aMessage.Shift then
     Image.BeginDrag(false, 2)
end;



procedure   TImageToolManager.on_tm_Viewer(var Handled: boolean);
begin
   try
      FindMenuItemIn('Tools','TViewer').OnClick(nil);
   except
   end;
end;

procedure   TImageToolManager.on_tm_AreaSelector(var Handled: boolean);
begin
   try
      FindMenuItemIn('Tools','TPSkSelector').OnClick(nil);
   except
   end;
end;


function    TImageToolManager.GetVisible: boolean;
begin
   try
      result := Image.Visible;
   except
      result := false;
   end;
end;

procedure   TImageToolManager.SetVisible(aVisible: boolean);
begin
   try
      Image.Visible := aVisible;
      Device.SetupPage(nil);
   except
   end;
end;


procedure TImageToolManager.On_tm_RePaint(var Handled: boolean);
var
   i: longInt;
begin
   Handled := true;
   {
   PostMsg(tm_BreakInterpret, Self, nil, DontKill);
   }
   Image.Refresh;
end;


procedure TImageToolManager.on_tm_Done(var aMessage: TOrbisMessage; var Handled: boolean);
begin
  inherited;

end;



procedure   TImageToolManager.HandleMessage(var aMessage: TOrbisMessage; var Handled: boolean);
begin
   inherited;
   case aMessage.Message of
      tm_Viewer:        on_tm_Viewer(Handled);
      tm_AreaSelector:  on_tm_AreaSelector(Handled);
   end;
end;



procedure TImageToolManager.SetImage(const Value: TImage);
 begin
  if FImage <> nil then
     Exit;
  FImage := Value;
  with TImageCanvasDevice(Device) do
   begin
    Canvas := FImage.Canvas;
    Bitmap := FImage.Picture.Bitmap;
   end;
 end;



function    TImageToolManager.GetFormCursor: TCursor;
begin
  result:= 0;
  if Image = nil then exit;
//o.s.   result := Form.PaintBox.Cursor;
   result := Image.Cursor;
end;


procedure   TImageToolManager.SetFormCursor(AValue: TCursor);
begin
  if Image = nil then exit;
//   Form.PaintBox.Cursor := aValue;
  Image.Cursor := aValue;
end;



end.
