unit VersionCheck;

// by Corbin Dunn
// cdunn@borland.com

interface

// Put a file on your webserver (with domain name "Host")
// which can be accessed from a web browser at "Host + FileName"
// and contains:
// Line1 = Version number string
// Line2 = Date released string
// Line3 = File Size string - must be an integer

// For example, here is "version.txt" from my web server, at:
// http://www.bluetreesoft.com/wallchanger/version.txt :
// 3.0.2.2
// 02-07-2002
// 1870000

// WallChanger is a shareware program written with delphi: www.bluetreesoft.com

procedure GetVersionFromWeb(const Host, FileName: string;
  var Version: string; var DateStr: string; var FileSize: Integer);

procedure GetWallChangerVersion(var Version, DateStr: string; var Size: Integer);

implementation

uses SysUtils, IdBaseComponent, IdComponent,
  IdTCPConnection, IdTCPClient, IdHTTP;

procedure GetWallChangerVersion(var Version, DateStr: string; var Size: Integer);
begin
  GetVersionFromWeb('www.bluetreesoft.com', '/wallchanger/version.txt',
    Version, DateStr, Size);
end;

procedure GetVersionFromWeb(const Host, FileName: string;
  var Version: string; var DateStr: string; var FileSize: Integer);
var
  Browser: TIdHTTP;
  Response: string;
  P: PChar;
  TempStr: string;

  procedure IncP(var Variable: string);
  begin
    while ( (P <> nil) and not (P^ in [#10, #13, #0]) ) do
    begin
      Variable := Variable + P^;
      Inc(P);
    end;
    while ( (P <> nil) and (P^ in [#10, #13, #0]) ) do
      Inc(P);
  end;

begin
  { TODO -ocorbin : put the version check in a thread, so it won't tie the system down }
  Browser := TIdHTTP.Create(nil);
  try
    Browser.Host := Host;
    Response := Browser.Get(FileName);
    Version := '';
    DateStr := '';
    FileSize := 0;
    if Length(Response) > 0 then
    begin
      // First line is the version

      P := @Response[1];

      IncP(Version);
      IncP(DateStr);
      
      IncP(TempStr);
      if TempStr <> '' then
        FileSize := StrToInt(TempStr);
    end;
  finally
    Browser.Free;
  end;
end;


end.
