unit pelDateTime;

interface

function UnixTimeToDateTime(const lw: LongWord): TDateTime;
function DateTimeToUnixTime(const lw: TDateTime):LongWord;

function UTCTimeToDateTime(const lw: LongWord): TDateTime;
function DateTimeToUTCTime(const lw: TDateTime):LongWord;

Function PalmDTToDateTime(const lw:LongWord):TDateTime;
function DateTimeToPalmDT(const lw: TDateTime):LongWord;

function PalmTimeToUTCTime(const lw: LongWord):LongWord;
function UTCTimeToPalmDT(const lw: LongWord):LongWord;

Function DateTimeAsStr(const lw:TDateTime):string;

implementation
Uses
  windows,Sysutils;
const
  unixdayone = 25569.0; // 1970-01-01
  nineteen0four=1462.0; // 1904-01-01

function DateTimeToPalmDT(const lw: TDateTime):LongWord;
begin
  result := trunc((lw-nineteen0four)*secsperday)
end;

Function PalmDTToDateTime(const lw:LongWord):TDateTime;
begin
  result:=nineteen0four+(lw/secsperday);
end;

Function GetTimeZoneBias:longint;
var
  TimeZoneInformation: TTimeZoneInformation;
begin
  if GetTimeZoneInformation(TimeZoneInformation)= TIME_ZONE_ID_DAYLIGHT then
  begin
    result:=TimeZoneInformation.bias+TimeZoneInformation.DaylightBias;
  end
  else
  begin
    result:=TimeZoneInformation.bias;
  end;
end;

function UnixTimeToDateTime(const lw: LongWord): TDateTime;
begin
  result := unixdayone + (lw / secsperday);
end;

function DateTimeToUnixTime(const lw: TDateTime):LongWord;
begin
  result := trunc((lw-unixdayone)*secsperday);
end;

function DateTimeToUTCTime(const lw: TDateTime):LongWord;
//var
//  bias:Longint;
begin
  result := trunc((lw-unixdayone)*secsperday)+(GetTimeZoneBias*60);
end;

function UTCTimeToDateTime(const lw: LongWord): TDateTime;
begin
  result := unixdayone + ((lw-(LongWord(GetTimeZoneBias)*60)) / secsperday);
end;

function PalmTimeToUTCTime(const lw: LongWord):LongWord;
begin
  result:=DateTimeToUTCTime(PalmDTToDateTime(lw));
end;

function UTCTimeToPalmDT(const lw: LongWord):LongWord;
begin
  result:=DateTimeToPalmDT(UTCTimeToDateTime(lw));
end;

Function DateTimeAsStr(const lw:TDateTime):string;
begin
  Result := FormatDateTime('yyyy-mm-dd hh":"mm":"ss ', lw);
end;
end.

