unit pfgCondMgre;
{**************************************************************************}
{* pfgCondMgre Unit                                                       *}
{*                                                                        *}
{* Header file conversion of CondMgre.h                                   *}
{*                                                                        *}
{* Version History:                                                       *}
{*   1/11/2000 - Initial manual header file convesrion                    *}
{*   9/7/2003  - Converted methods to use dynamically loaded DLL, using   *}
{*               a set of HeadConv converted method declarations          *}
{**************************************************************************}

interface

uses SysUtils, Windows, pfgWTypes;

{
/*****************************************************************************
 *
 * Copyright (c) 1998-1999 Palm Computing, Inc. or its subsidiaries.
 * All rights reserved.
 *
 ****************************************************************************/

/*****************************************************************
 *
 * Conduit Manager API
 *
 ****************************************************************/
} 

const
  ERR_CONDUIT_MGR              = -1000;
  ERR_INDEX_OUT_OF_RANGE       = (ERR_CONDUIT_MGR - 1);
  ERR_UNABLE_TO_DELETE         = (ERR_CONDUIT_MGR - 2);
  ERR_NO_CONDUIT               = (ERR_CONDUIT_MGR - 3);
  ERR_NO_MEMORY                = (ERR_CONDUIT_MGR - 4);
  ERR_CREATORID_ALREADY_IN_USE = (ERR_CONDUIT_MGR - 5);
  ERR_REGISTRY_ACCESS             = (ERR_CONDUIT_MGR - 6);
  ERR_UNABLE_TO_CREATE_CONDUIT    = (ERR_CONDUIT_MGR - 7);
  ERR_UNABLE_TO_SET_CONDUIT_VALUE = (ERR_CONDUIT_MGR - 8);
  ERR_INVALID_HANDLE              = (ERR_CONDUIT_MGR - 9);
  ERR_BUFFER_TOO_SMALL            = (ERR_CONDUIT_MGR - 10);
  ERR_VALUE_NOT_FOUND             = (ERR_CONDUIT_MGR - 11);
  ERR_INVALID_CREATOR_ID          = (ERR_CONDUIT_MGR - 12);
  ERR_INVALID_POINTER             = (ERR_CONDUIT_MGR - 13);
  ERR_UNABLE_TO_INSTALL_OLD       = (ERR_CONDUIT_MGR - 14);
  ERR_INVALID_CONDUIT_TYPE        = (ERR_CONDUIT_MGR - 15);
  ERR_INVALID_COM_PORT_TYPE       = (ERR_CONDUIT_MGR - 16);
  ERR_NO_LONGER_SUPPORTED         = (ERR_CONDUIT_MGR - 17);


  INVALID_CREATORID           = 0;
  INVALID_PRIORITY            = 2;
  INVALID_INTEGRATE           = 0;

  CONDUIT_COMPONENT           = 0;
  CONDUIT_APPLICATION         = 1;
  CONDUIT_CONDUITS            = 10;

  BACKUP_CONDUIT              = 0;
  MAX_SPECIAL                 = BACKUP_CONDUIT;

  DIRECT_COM_PORT             = 0;
  MODEM_COM_PORT              = 1;
  MAX_COM_PORT                = MODEM_COM_PORT;

  CM_CREATOR_ID_SIZE          = 8;  // Rounded up from 4+1

type
  CM_CREATORLIST_ITEM_TYPE = packed record
    // only need 4 + 1 string terminator, by lets make it even
    szCreatorID: Array [0..CM_CREATOR_ID_SIZE-1] of Char;
    iReserved: Integer;
  end;
  CM_CREATORLIST_TYPE = ^CM_CREATORLIST_ITEM_TYPE;

  CmConduitType =packed record
    iStructureVersion: Integer;
    iStructureSize: Integer;
    iType: Integer; // Types CONDUIT_X
    // only need 4 + 1 string terminator, by lets make it even.
    szCreatorID: Array [0..CM_CREATOR_ID_SIZE-1] of Char;
    dwPriority: DWORD;
    iConduitNameOffset: Integer;
    iDirectoryOffset: Integer;
    iFileOffset: Integer;
    iRemoteDBOffset: Integer;
    iUsernameOffset: Integer;
    iTitleOffset: Integer;
    iInfoOffset: Integer;
  end;

  FileInstallType = record
    szDir: Array[0..64-1] of TCHAR;
    szExt: Array[0..256-1] of TCHAR;
    dwMask: LongInt;
    szModule: Array[0..256-1] of TCHAR;
    dwCreatorID: LongInt;
    szName: Array[0..256-1] of TCHAR;
  end {FileInstallType};

const
  FILEINSTALLTYPE_SIZE = sizeof(FileInstallType);

const
  CM_MIN_CONDUITTYPE_SIZE  = sizeof(CmConduitType);
  CM_CONDUIT_BUFFER_OFFSET = sizeof(CmConduitType);

  CONDUIT_VERSION          = 100;

  //	API functions
  CM_INITIAL_LIB_VERSION = $0001;
  CM_UPDATE_1            = $0002;

var
  CmGetLibVersion: function: Word; stdcall; 

{/// Utilities for manipulating Creator ID's }

  CmConvertCreatorIDToString: function(dwID: LongInt;
                                       pString: PTCHAR;
                                       var piSize: Integer): Integer; stdcall;

  CmConvertStringToCreatorID: function(const pString: PTCHAR;
                                       var pdwID: LongInt): Integer; stdcall;

{/// }
{/// Functions for reading current conduit configuration }
{/// }

  CmGetConduitCount: function: Integer; stdcall;

  CmGetCreatorIDList: function(pCreatorList: CM_CREATORLIST_TYPE;
                               var piSize: Integer): Integer; stdcall;

  CmGetConduitCreatorID: function(iIndex: Integer;
                                  pCreatorID: PChar;
                                  var piSize: Integer): Integer; stdcall;

  CmGetConduitByCreator: function(const pCreatorID: PChar;
                                  var hStruct: Handle): Integer; stdcall;

{/// }
{/// Functions for installing a new conduit }
{/// }

  CmInstallConduit: function(hStruct: THandle): Integer; stdcall;

{/// Only defines Creator ID for a new conduit; other values must }
{/// be set separately. }

  CmInstallCreator: function(const pCreator: PChar;
                             iType: Integer): Integer; stdcall;

{/// }
{/// Functions for removing a conduit }
{/// }

  CmRemoveConduitByCreatorID: function(const pCreatorID: PChar): Integer; stdcall;

{/// }
{/// Creator ID based functions for accessing individual data items. }
{/// }

  CmSetCreatorPriority: function(const pCreatorID: PChar;
                                 dwPriority: LongInt): Integer; stdcall;

  CmGetCreatorPriority: function(const pCreatorID: PChar;
                                 var pdwPriority: LongInt): Integer; stdcall;

  CmSetCreatorName: function(const pCreatorID: PChar;
                             const pConduitName: PTCHAR): Integer; stdcall;

  CmGetCreatorName: function(const pCreatorID: PChar;
                             pConduitName: PTCHAR;
                             var piSize: Integer): Integer; stdcall;

  CmSetCreatorDirectory: function(const pCreatorID: PChar;
                                  const pDirectory: PTCHAR): Integer; stdcall;

  CmGetCreatorDirectory: function(const pCreatorID: PChar;
                                  pDirectory: PTCHAR;
                                  var piSize: Integer): Integer; stdcall;

  CmSetCreatorRemote: function(const pCreatorID: PChar;
                               const pRemoteDB: PTCHAR): Integer; stdcall;

  CmGetCreatorRemote: function(const pCreatorID: PChar;
                               pRemoteDB: PTCHAR;
                               var piSize: Integer): Integer; stdcall;

  CmSetCreatorUser: function(const pCreatorID: PChar;
                             const pUsername: PTCHAR): Integer; stdcall; 

  CmGetCreatorUser: function(const pCreatorID: PChar;
                             pUsername: PTCHAR;
                             var piSize: Integer): Integer; stdcall;


  CmSetCreatorTitle: function(const pCreatorID: PChar;
                              const pTitle: PTCHAR): Integer; stdcall;

  CmGetCreatorTitle: function(const pCreatorID: PChar;
                              pTitle: PTCHAR;
                              var piSize: Integer): Integer; stdcall;


  CmSetCreatorInfo: function(const pCreatorID: PChar;
                             const pInfo: PTCHAR): Integer; stdcall;

  CmGetCreatorInfo: function(const pCreatorID: PChar;
                             pInfo: PTCHAR;
                             var piSize: Integer): Integer; stdcall;


  CmSetCreatorFile: function(const pCreatorID: PChar; 
                             const pFile: PTCHAR): Integer; stdcall;

  CmGetCreatorFile: function(const pCreatorID: PChar;
                             pFile: PTCHAR;
                             var piSize: Integer): Integer; stdcall;

{/// Conduit Type - TO BE OBSOLETED }

  CmGetCreatorType: function(const pCreator: PChar): Integer; stdcall;

{/// }
{/// Functions for integrating applications with PalmPilot Desktop. }
{/// }
{/// Not guaranteed to be supported in future revisions. }

  CmSetCreatorIntegrate: function(const pCreatorID: PChar;
                                  dwIntegrate: LongInt): Integer; stdcall;

  CmGetCreatorIntegrate: function(const pCreatorID: PChar;
                                  var pdwIntegrate: LongInt): Integer; stdcall;


  CmSetCreatorModule: function(const pCreatorID: PChar;
                               const pModule: PTCHAR): Integer; stdcall;

  CmGetCreatorModule: function(const pCreatorID: PChar; 
                               pModule: PTCHAR; 
                               var piSize: Integer): Integer; stdcall; 


  CmSetCreatorArgument: function(const pCreatorID: PChar;
                                 const pArgument: PTCHAR): Integer; stdcall;

  CmGetCreatorArgument: function(const pCreatorID: PChar;
                                 pArgument: PTCHAR;
                                 var piSize: Integer): Integer; stdcall;

{/// }
{/// Functions for accessing other HotSync configuration info. }
{/// }
{/// Port access }

  CmSetComPort: function(iType: Integer;
                         const pPort: PTCHAR): Integer; stdcall;

  CmGetComPort: function(iType: Integer;
                         pPort: PTCHAR;
                         var piSize: Integer): Integer; stdcall;

{/// Backup conduit }

  CmSetBackupConduit: function(const pConduit: PTCHAR): Integer; stdcall;

  CmGetBackupConduit: function(pConduit: PTCHAR;
                               var piSize: Integer): Integer; stdcall;

{/// Notifiers }

  CmSetNotifierDll: function(iIndex: Integer;
                             const pNotifier: PTCHAR): Integer; stdcall;

  CmGetNotifierDll: function(iIndex: Integer; 
                             pNotifier: PTCHAR; 
                             var piSize: Integer): Integer; stdcall; 

{/// PC ident }

  CmSetPCIdentifier: function(dwPCID: LongInt): Integer; stdcall;

  CmGetPCIdentifier: function(var pdwPCID: LongInt): Integer; stdcall;

{/// Core path }

  CmGetCorePath: function(pPath: PTCHAR;
                          var piSize: Integer): Integer; stdcall;

  CmSetCorePath: function(const pPath: PTCHAR): Integer; stdcall;

{/// HotSync Path }

  CmGetHotSyncExecPath: function(pPath: PTCHAR;
                                 var piSize: Integer): Integer; stdcall;

  CmSetHotSyncExecPath: function(const pPath: PTCHAR): Integer; stdcall;


  CmSetCreatorValueDword: function(const pCreatorID: PChar;
                                   pValue: PTCHAR;
                                   dwValue: LongInt): Integer; stdcall;

  CmGetCreatorValueDword: function(const pCreatorID: PChar;
                                   pValue: PTCHAR;
                                   var dwValue: LongInt;
                                   dwDefault: LongInt): Integer; stdcall;

  CmSetCreatorValueString: function(const pCreatorID: PChar;
                                    pValue: PTCHAR;
                                    pString: PTCHAR): Integer; stdcall;

  CmGetCreatorValueString: function(const pCreatorID: PChar; 
                                    pValue: PTCHAR;
                                    pString: PTCHAR;
                                    var piSize: Integer; 
                                    pDefault: PTCHAR): Integer; stdcall; 


{/// Registry Restore Function }

  CmRestoreHotSyncSettings: function(bToDefaults: Bool): Integer; stdcall;


{/// general Palm information storage calls }

  PiSetValueDword: function(const pFolder: PChar;
                            const pKey: PChar;
                            dwValue: LongInt): LongInt; stdcall;

  PiGetValueDword: function(const pFolder: PChar;
                            const pKey: PChar;
                            var pdwValue: LongInt;
                            dwDefault: LongInt): LongInt; stdcall;

  PiSetValueString: function(const pFolder: PChar;
                             const pKey: PChar;
                             const pValue: PChar): LongInt; stdcall;

  PiGetValueString: function(const pFolder: PChar;
                             const pKey: PChar;
                             pValue: PChar;
                             var piLen: Integer;
                             const pDefault: PChar): LongInt; stdcall;

  PiSetHotsyncValueDword: function(const pFolder: PChar;
                                   const pKey: PChar;
                                   dwValue: LongInt): LongInt; stdcall;

  PiGetHotsyncValueDword: function(const pFolder: PChar;
                                   const pKey: PChar;
                                   var pdwValue: LongInt;
                                   dwDefault: LongInt): LongInt; stdcall;

  PiSetHotsyncValueString: function(const pFolder: PChar;
                                    const pKey: PChar;
                                    const pValue: PChar): LongInt; stdcall;

  PiGetHotsyncValueString: function(const pFolder: PChar;
                                    const pKey: PChar;
                                    pValue: PChar;
                                    var piLen: Integer;
                                    const pDefault: PChar): LongInt; stdcall;

{/// Install Conduit API }

  ImRegister: function(const sFIT: FILEINSTALLTYPE): Integer; stdcall;

  ImRegisterID: function(dwCreatorID: LongInt): Integer; stdcall;

  ImUnregisterID: function(dwCreatorID: LongInt): Integer; stdcall;

  ImSetDirectory: function(dwID: LongInt;
                           const pDirectory: PTCHAR): Integer; stdcall;

  ImSetExtension: function(dwID: LongInt;
                           const pExtension: PTCHAR): Integer; stdcall;

  ImSetMask: function(dwID: LongInt;
                      dwMask: LongInt): Integer; stdcall;

  ImSetModule: function(dwID: LongInt;
                        const pModule: PTCHAR): Integer; stdcall;

  ImSetName: function(dwID: LongInt;
                      const pConduitName: PTCHAR): Integer; stdcall;

  ImSetDWord: function(dwID: LongInt;
                       const pValue: PTCHAR;
                       dwValue: LongInt): Integer; stdcall;

  ImSetString: function(dwID: LongInt;
                        const pValue: PTCHAR;
                        pString: PTCHAR): Integer; stdcall;

  ImGetDirectory: function(dwID: LongInt;
                           pDirectory: PTCHAR;
                           var piSize: Integer): Integer; stdcall;

  ImGetExtension: function(dwID: LongInt; 
                           pExtension: PTCHAR; 
                           var piSize: Integer): Integer; stdcall; 

  ImGetMask: function(dwID: LongInt;
                      var pdwMask: LongInt): Integer; stdcall;

  ImGetModule: function(dwID: LongInt;
                        pModule: PTCHAR;
                        var piSize: Integer): Integer; stdcall;

  ImGetName: function(dwID: LongInt;
                      pConduitName: PTCHAR;
                      var piSize: Integer): Integer; stdcall;

  ImGetDWord: function(dwID: LongInt;
                       const pValue: PTCHAR;
                       var pdwMask: LongInt;
                       dwDefault: LongInt): Integer; stdcall;

  ImGetString: function(dwID: LongInt;
                        const pValue: PTCHAR;
                        pString: PTCHAR;
                        var piSize: Integer;
                        const pDefault: PTCHAR): Integer; stdcall;

{///Install Notifier Manager API }

  NmRegister: function(const pNotifierPath: PTCHAR): Integer; stdcall;

  NmUnregister: function(const pNotifierPath: PTCHAR): Integer; stdcall;

  NmGetCount: function(var pdwCount: LongInt): Integer; stdcall;

  NmFind: function(const pNotifier: PTCHAR): Integer; stdcall;

  NmGetByIndex: function(iIndex: Integer;
                         pNotifier: PTCHAR;
                         var piSize: Integer): Integer; stdcall;

  NmRenameByIndex: function(iIndex: Integer;
                            const pNotifier: PTCHAR): Integer; stdcall;



implementation

uses Dialogs, pfgUserData;

var
  DllHandle : THandle;
  DllLoaded : Boolean;


procedure LoadDLL;
begin
  if DLLLoaded then Exit;

  DLLHandle := LoadLibrary(PChar(PalmPath + 'CONDMGR.DLL'));
  DLLLoaded := DLLHandle >= 32;
  if DLLLoaded then
  begin
    @CmGetLibVersion := GetProcAddress(DLLHandle,'CmGetLibVersion');
    Assert(@CmGetLibVersion <> nil);
    @CmConvertCreatorIDToString := GetProcAddress(DLLHandle,'CmConvertCreatorIDToString');
    Assert(@CmConvertCreatorIDToString <> nil);
    @CmConvertStringToCreatorID := GetProcAddress(DLLHandle,'CmConvertStringToCreatorID');
    Assert(@CmConvertStringToCreatorID <> nil);
    @CmGetConduitCount := GetProcAddress(DLLHandle,'CmGetConduitCount');
    Assert(@CmGetConduitCount <> nil);
    @CmGetCreatorIDList := GetProcAddress(DLLHandle,'CmGetCreatorIDList');
    Assert(@CmGetCreatorIDList <> nil);
    @CmGetConduitCreatorID := GetProcAddress(DLLHandle,'CmGetConduitCreatorID');
    Assert(@CmGetConduitCreatorID <> nil);
    @CmGetConduitByCreator := GetProcAddress(DLLHandle,'CmGetConduitByCreator');
    Assert(@CmGetConduitByCreator <> nil);
    @CmInstallConduit := GetProcAddress(DLLHandle,'CmInstallConduit');
    Assert(@CmInstallConduit <> nil);
    @CmInstallCreator := GetProcAddress(DLLHandle,'CmInstallCreator');
    Assert(@CmInstallCreator <> nil);
    @CmRemoveConduitByCreatorID := GetProcAddress(DLLHandle,'CmRemoveConduitByCreatorID');
    Assert(@CmRemoveConduitByCreatorID <> nil);
    @CmSetCreatorPriority := GetProcAddress(DLLHandle,'CmSetCreatorPriority');
    Assert(@CmSetCreatorPriority <> nil);
    @CmGetCreatorPriority := GetProcAddress(DLLHandle,'CmGetCreatorPriority');
    Assert(@CmGetCreatorPriority <> nil);
    @CmSetCreatorName := GetProcAddress(DLLHandle,'CmSetCreatorName');
    Assert(@CmSetCreatorName <> nil);
    @CmGetCreatorName := GetProcAddress(DLLHandle,'CmGetCreatorName');
    Assert(@CmGetCreatorName <> nil);
    @CmSetCreatorDirectory := GetProcAddress(DLLHandle,'CmSetCreatorDirectory');
    Assert(@CmSetCreatorDirectory <> nil);
    @CmGetCreatorDirectory := GetProcAddress(DLLHandle,'CmGetCreatorDirectory');
    Assert(@CmGetCreatorDirectory <> nil);
    @CmSetCreatorRemote := GetProcAddress(DLLHandle,'CmSetCreatorRemote');
    Assert(@CmSetCreatorRemote <> nil);
    @CmGetCreatorRemote := GetProcAddress(DLLHandle,'CmGetCreatorRemote');
    Assert(@CmGetCreatorRemote <> nil);
    @CmSetCreatorUser := GetProcAddress(DLLHandle,'CmSetCreatorUser');
    Assert(@CmSetCreatorUser <> nil);
    @CmGetCreatorUser := GetProcAddress(DLLHandle,'CmGetCreatorUser');
    Assert(@CmGetCreatorUser <> nil);
    @CmSetCreatorTitle := GetProcAddress(DLLHandle,'CmSetCreatorTitle');
    Assert(@CmSetCreatorTitle <> nil);
    @CmGetCreatorTitle := GetProcAddress(DLLHandle,'CmGetCreatorTitle');
    Assert(@CmGetCreatorTitle <> nil);
    @CmSetCreatorInfo := GetProcAddress(DLLHandle,'CmSetCreatorInfo');
    Assert(@CmSetCreatorInfo <> nil);
    @CmGetCreatorInfo := GetProcAddress(DLLHandle,'CmGetCreatorInfo');
    Assert(@CmGetCreatorInfo <> nil);
    @CmSetCreatorFile := GetProcAddress(DLLHandle,'CmSetCreatorFile');
    Assert(@CmSetCreatorFile <> nil);
    @CmGetCreatorFile := GetProcAddress(DLLHandle,'CmGetCreatorFile');
    Assert(@CmGetCreatorFile <> nil);
    @CmGetCreatorType := GetProcAddress(DLLHandle,'CmGetCreatorType');
    Assert(@CmGetCreatorType <> nil);
    @CmSetCreatorIntegrate := GetProcAddress(DLLHandle,'CmSetCreatorIntegrate');
    Assert(@CmSetCreatorIntegrate <> nil);
    @CmGetCreatorIntegrate := GetProcAddress(DLLHandle,'CmGetCreatorIntegrate');
    Assert(@CmGetCreatorIntegrate <> nil);
    @CmSetCreatorModule := GetProcAddress(DLLHandle,'CmSetCreatorModule');
    Assert(@CmSetCreatorModule <> nil);
    @CmGetCreatorModule := GetProcAddress(DLLHandle,'CmGetCreatorModule');
    Assert(@CmGetCreatorModule <> nil);
    @CmSetCreatorArgument := GetProcAddress(DLLHandle,'CmSetCreatorArgument');
    Assert(@CmSetCreatorArgument <> nil);
    @CmGetCreatorArgument := GetProcAddress(DLLHandle,'CmGetCreatorArgument');
    Assert(@CmGetCreatorArgument <> nil);
    @CmSetComPort := GetProcAddress(DLLHandle,'CmSetComPort');
    Assert(@CmSetComPort <> nil);
    @CmGetComPort := GetProcAddress(DLLHandle,'CmGetComPort');
    Assert(@CmGetComPort <> nil);
    @CmSetBackupConduit := GetProcAddress(DLLHandle,'CmSetBackupConduit');
    Assert(@CmSetBackupConduit <> nil);
    @CmGetBackupConduit := GetProcAddress(DLLHandle,'CmGetBackupConduit');
    Assert(@CmGetBackupConduit <> nil);
    @CmSetNotifierDll := GetProcAddress(DLLHandle,'CmSetNotifierDll');
    Assert(@CmSetNotifierDll <> nil);
    @CmGetNotifierDll := GetProcAddress(DLLHandle,'CmGetNotifierDll');
    Assert(@CmGetNotifierDll <> nil);
    @CmSetPCIdentifier := GetProcAddress(DLLHandle,'CmSetPCIdentifier');
    Assert(@CmSetPCIdentifier <> nil);
    @CmGetPCIdentifier := GetProcAddress(DLLHandle,'CmGetPCIdentifier');
    Assert(@CmGetPCIdentifier <> nil);
    @CmGetCorePath := GetProcAddress(DLLHandle,'CmGetCorePath');
    Assert(@CmGetCorePath <> nil);
    @CmSetCorePath := GetProcAddress(DLLHandle,'CmSetCorePath');
    Assert(@CmSetCorePath <> nil);
    @CmGetHotSyncExecPath := GetProcAddress(DLLHandle,'CmGetHotSyncExecPath');
    Assert(@CmGetHotSyncExecPath <> nil);
    @CmSetHotSyncExecPath := GetProcAddress(DLLHandle,'CmSetHotSyncExecPath');
    Assert(@CmSetHotSyncExecPath <> nil);
    @CmSetCreatorValueDword := GetProcAddress(DLLHandle,'CmSetCreatorValueDword');
    Assert(@CmSetCreatorValueDword <> nil);
    @CmGetCreatorValueDword := GetProcAddress(DLLHandle,'CmGetCreatorValueDword');
    Assert(@CmGetCreatorValueDword <> nil);
    @CmSetCreatorValueString := GetProcAddress(DLLHandle,'CmSetCreatorValueString');
    Assert(@CmSetCreatorValueString <> nil);
    @CmGetCreatorValueString := GetProcAddress(DLLHandle,'CmGetCreatorValueString');
    Assert(@CmGetCreatorValueString <> nil);
    @CmRestoreHotSyncSettings := GetProcAddress(DLLHandle,'CmRestoreHotSyncSettings');
    Assert(@CmRestoreHotSyncSettings <> nil);
    @PiSetValueDword := GetProcAddress(DLLHandle,'PiSetValueDword');
    Assert(@PiSetValueDword <> nil);
    @PiGetValueDword := GetProcAddress(DLLHandle,'PiGetValueDword');
    Assert(@PiGetValueDword <> nil);
    @PiSetValueString := GetProcAddress(DLLHandle,'PiSetValueString');
    Assert(@PiSetValueString <> nil);
    @PiGetValueString := GetProcAddress(DLLHandle,'PiGetValueString');
    Assert(@PiGetValueString <> nil);
    @PiSetHotsyncValueDword := GetProcAddress(DLLHandle,'PiSetHotsyncValueDword');
    Assert(@PiSetHotsyncValueDword <> nil);
    @PiGetHotsyncValueDword := GetProcAddress(DLLHandle,'PiGetHotsyncValueDword');
    Assert(@PiGetHotsyncValueDword <> nil);
    @PiSetHotsyncValueString := GetProcAddress(DLLHandle,'PiSetHotsyncValueString');
    Assert(@PiSetHotsyncValueString <> nil);
    @PiGetHotsyncValueString := GetProcAddress(DLLHandle,'PiGetHotsyncValueString');
    Assert(@PiGetHotsyncValueString <> nil);
    @ImRegister := GetProcAddress(DLLHandle,'ImRegister');
    Assert(@ImRegister <> nil);
    @ImRegisterID := GetProcAddress(DLLHandle,'ImRegisterID');
    Assert(@ImRegisterID <> nil);
    @ImUnregisterID := GetProcAddress(DLLHandle,'ImUnregisterID');
    Assert(@ImUnregisterID <> nil);
    @ImSetDirectory := GetProcAddress(DLLHandle,'ImSetDirectory');
    Assert(@ImSetDirectory <> nil);
    @ImSetExtension := GetProcAddress(DLLHandle,'ImSetExtension');
    Assert(@ImSetExtension <> nil);
    @ImSetMask := GetProcAddress(DLLHandle,'ImSetMask');
    Assert(@ImSetMask <> nil);
    @ImSetModule := GetProcAddress(DLLHandle,'ImSetModule');
    Assert(@ImSetModule <> nil);
    @ImSetName := GetProcAddress(DLLHandle,'ImSetName');
    Assert(@ImSetName <> nil);
    @ImSetDWord := GetProcAddress(DLLHandle,'ImSetDWord');
    Assert(@ImSetDWord <> nil);
    @ImSetString := GetProcAddress(DLLHandle,'ImSetString');
    Assert(@ImSetString <> nil);
    @ImGetDirectory := GetProcAddress(DLLHandle,'ImGetDirectory');
    Assert(@ImGetDirectory <> nil);
    @ImGetExtension := GetProcAddress(DLLHandle,'ImGetExtension');
    Assert(@ImGetExtension <> nil);
    @ImGetMask := GetProcAddress(DLLHandle,'ImGetMask');
    Assert(@ImGetMask <> nil);
    @ImGetModule := GetProcAddress(DLLHandle,'ImGetModule');
    Assert(@ImGetModule <> nil);
    @ImGetName := GetProcAddress(DLLHandle,'ImGetName');
    Assert(@ImGetName <> nil);
    @ImGetDWord := GetProcAddress(DLLHandle,'ImGetDWord');
    Assert(@ImGetDWord <> nil);
    @ImGetString := GetProcAddress(DLLHandle,'ImGetString');
    Assert(@ImGetString <> nil);
    @NmRegister := GetProcAddress(DLLHandle,'NmRegister');
    Assert(@NmRegister <> nil);
    @NmUnregister := GetProcAddress(DLLHandle,'NmUnregister');
    Assert(@NmUnregister <> nil);
    @NmGetCount := GetProcAddress(DLLHandle,'NmGetCount');
    Assert(@NmGetCount <> nil);
    @NmFind := GetProcAddress(DLLHandle,'NmFind');
    Assert(@NmFind <> nil);
    @NmGetByIndex := GetProcAddress(DLLHandle,'NmGetByIndex');
    Assert(@NmGetByIndex <> nil);
    @NmRenameByIndex := GetProcAddress(DLLHandle,'NmRenameByIndex');
    Assert(@NmRenameByIndex <> nil);
  end;
end;

initialization
  DLLLoaded := False;
  LoadDLL;

  if (not DllLoaded) then
  begin
    MessageDlg('pfgCondMgre: Unable to load condmgr.dll',mtError,[mbOk],0);
    Abort;
  end;

finalization
  if DLLLoaded then FreeLibrary(DLLHandle);
end.
