{$WEAKPACKAGEUNIT ON}
unit pfgPalmSyncError;
{**************************************************************************}
{* pfgPalmSyncError Unit                                                  *}
{*                                                                        *}
{* This unit implements handling of errors that occur in the library.     *}
{*                                                                        *}
{* Copyright (C) 2000 - 2002 by Paul Gilbert, All Rights Reserved         *}
{**************************************************************************}

interface

uses SysUtils;

type
  EPalmSyncError = class(Exception)
  private
    FError: Integer;
  public
    constructor CreateErr(const AError: Integer);
    constructor CreateErrMsg(const AError: Integer; const Msg: string);
    property Error: Integer read FError write FError;
  end;

  // Added by Mert //
  EUserManagerError = class(Exception)
  private
    FError: Integer;
  public
    constructor CreateErr(const AError: Integer);
    constructor CreateErrMsg(const AError: Integer; const Msg: string);
    property Error: Integer read FError write FError;
  end;
  // End of Mert //

procedure PalmCheck(const doing: string; Error: Integer); overload;
procedure PalmCheck(Error: Integer); overload;
procedure PalmError(Error: Integer);

// Utility functions
function TranslatePalmError(const PalmError: integer): string;
function IsDirty(const Flags: integer): boolean;
function IsDeleted(const Flags: integer): boolean;

// Added by Mert //
function TranslateUserManagerErrors(const PalmError: integer): string;
procedure UserManagerCheck(Error: Integer);
// End Of Mert //

implementation

uses
  pfgCondMgre,
  pfgCondAPI,
  pfgSyncMgr,
  pfgWTypes,
  pfgHslog,
  pfgUserData,
  pfgPalmInstall;

function IsDirty(const Flags: integer): boolean;
begin
  result := (Flags and eRecAttrDirty) <> 0;
end;

function IsDeleted(const Flags: integer): boolean;
begin
  result := (Flags and eRecAttrDeleted) <> 0;
end;

// PalmError
// Raises the Palm sync exception with the given Sync error

procedure PalmError(Error: Integer);
begin
  raise EPalmSyncError.CreateErr(Error);
end;

procedure PalmCheck(const doing: string; Error: Integer); overload;
begin
  if Error <> SYNCERR_NONE then
    raise EPalmSyncError.CreateErrMsg(Error, doing);
end;

procedure PalmCheck(Error: Integer); overload;
begin
  if Error <> SYNCERR_NONE then
    raise EPalmSyncError.CreateErr(Error);
end;

{ EPalmSyncError }

constructor EPalmSyncError.CreateErr(const AError: Integer);
begin
  inherited CreateFmt('Error %d :%s', [AError, TranslatePalmError(AError)]);
  FError := AError;
end;

constructor EPalmSyncError.CreateErrMsg(const AError: Integer;
  const Msg: string);
begin
  inherited CreateFmt('Error %d,:%s at %s', [AError, TranslatePalmError(AError), Msg]);
  FError := AError;
end;


function TranslatePalmError(const PalmError: integer): string;
begin
  case PalmError of
    -1: Result := 'A nonspecific error.';
    CONDERR_ADD_LOCAL_RECORD: Result := 'Could not add a record to the desktop computer tables.';
    CONDERR_ADD_REMOTE_RECORD: Result := 'Could not add a record on the handheld.';
    CONDERR_BAD_LOCAL_BACKUP: Result := 'The backup table could not be read on the desktop computer.';
    CONDERR_BAD_LOCAL_TABLES: Result := 'The tables could not be read on the desktop computer.';
    CONDERR_BAD_REMOTE_TABLES: Result := 'The tables could not be read from the handheld.';
    CONDERR_BAD_SYNC_TYPE: Result := 'The specified synchronization type is not valid.';
    CONDERR_BUFFER_TOO_SMALL: Result := 'The buffer is too small to contain the data.';
    CONDERR_CHANGE_REMOTE_RECORD: Result := 'Could not change a record on the handheld.';
    CONDERR_CONDUIT_RESOURCE_FAILURE: Result := 'Could not read a resource from your conduit'' s DLL.';
    CONDERR_CONVERT_FROM_REMOTE_REC: Result := 'Could not convert a handheld record to a record on the desktop computer.';
    CONDERR_CONVERT_TO_LOCAL_CATS: Result := 'Could not convert a record'' s category to a category on the desktop computer.';
    CONDERR_CONVERT_TO_REMOTE_CATS: Result := 'Could not convert a record'' s category to a category on the handheld.';
    CONDERR_CONVERT_TO_REMOTE_REC: Result := 'Could not convert a desktop computer record to a record on the handheld.';
    CONDERR_DATE_MOVED: Result := 'Dates before 1970 were moved to a valid date.';
    CONDERR_INVALID_ARGSSIZE: Result := 'The argument size is not valid.';
    CONDERR_INVALID_ARGSSIZE_PTR: Result := 'The output arguments size is not valid.';
    CONDERR_INVALID_BUFFER_SIZE: Result := 'The buffer size is not valid for the buffer.';
    CONDERR_INVALID_INARGS_PTR: Result := 'The input arguments pointer is not valid.';
    CONDERR_INVALID_INARGS_STRUCT: Result := 'The input arguments structure is not valid.';
    CONDERR_INVALID_OUTSIZE_PTR: Result := 'The output size pointer is not valid.';
    CONDERR_INVALID_PTR: Result := 'A pointer is not valid.';
    CONDERR_LOCAL_MEMORY_ALLOC_FAILED: Result := 'Could not allocate memory on the desktop computer.';
    CONDERR_NO_LOCAL_CATEGORIES: Result := 'There are no categories defined on the desktop computer.';
    CONDERR_NO_REMOTE_CATEGORIES: Result := 'There are no categories defined on the handheld.';
    CONDERR_NONE: Result := 'The function call was successful.';
//      CONDERR_NONE: Result := 'Success';
    CONDERR_RAW_RECORD_ALLOCATE: Result := 'Could not allocate memory for a record.';
    CONDERR_REMOTE_CHANGES_NOT_SENT: Result := 'The changes were not successfully sent to the handheld.';
    CONDERR_REMOTE_RECS_NOT_PURGED: Result := 'The records on the handheld could not be successfully purged.';
    CONDERR_SAVE_REMOTE_CATEGORIES: Result := 'The categories could not be saved on the handheld.';
    CONDERR_SUBSCRIBE_FAILED: Result := 'The file link failed.';
    CONDERR_UNSUPPORTED_CFGCONDUIT_ENUM: Result := 'The specified CfgConduit enumeration value is not supported.';
    CONDERR_UNSUPPORTED_STRUCT_VERSION: Result := 'The version number of the structure is not supported on the handheld.';

//      DERR_ALL_KEYS_EQUAL: Result := 'The sort was successful, but all of the keys are equal.';
//      DERR_BAD_SCHEMA: Result := 'The schema object is not valid.';
//      DERR_BAD_TABLE: Result := 'The table is not valid.';
//      DERR_FILE_NOT_FOUND: Result := 'The specified file could not be opened.';
//      DERR_FILE_NOT_SAVE: Result := 'The file was not able to be written.';
//      DERR_INVALID_FILE_FORMAT: Result := 'The file could not be read.';
//      DERR_INVALID_KEY_SORT: Result := 'The specified sorting key is not valid.';
//      DERR_NO_CURRENT_FILENAME: Result := 'The m_TableName field does not contain a file name for the Save method to use.';
//      DERR_NONE: Result := 'The function call was successful.';
//      DERR_RECORD_NOT_POSITIONED: Result := 'Unable to locate record in table.';
//      ERR_NONE: Result := 'The function call was successful.';

    ERR_BUFFER_TOO_SMALL: Result := 'The buffer is too small to hold the requested information.';
    ERR_CONDUIT_MGR: Result := 'An otherwise nonspecific Conduit Manager error occurred.';
    ERR_CREATORID_ALREADY_IN_USE: Result := 'The creator ID that you specified to use as a new creator ID in the configuration entries is already in use.';
//      ERR_IA_INVALID_FILE_TYPE: Result := 'The PltGetFileCount function does not recognize the extension type.';
    ERR_INDEX_OUT_OF_RANGE: Result := 'The specified index value is out of range.';
    ERR_INVALID_COM_PORT_TYPE: Result := 'The specified communications port type is not valid. You must use one of the Conduit Communications Port Constants.';
    ERR_INVALID_CONDUIT_TYPE: Result := 'The specified conduit type is not valid. You must use one of the Conduit Information Type Constants.';
    ERR_INVALID_CREATOR_ID: Result := 'The specified conduit creator ID is not valid.';
    ERR_INVALID_HANDLE: Result := 'The specified structure handle is not valid.';
    ERR_INVALID_POINTER: Result := 'The specified pointer is not a valid pointer.';
    ERR_NO_CONDUIT: Result := 'The specified conduit does not exist.';
    ERR_NO_LONGER_SUPPORTED: Result := 'The value or feature that you specified is no longer supported.';
    ERR_NO_MEMORY: Result := 'Not enough memory to perform the requested operation.';
//      ERR_NONE: Result := 'The function call was successful.';


 (* Install Aide Error Codes  *)
 (* Removed slashes by Mert *)

      ERR_PILOT_BUFSIZE_TOO_SMALL: Result := 'The buffer supplied as a parameter to a function is too small to contain the information that the function attempted to store.';
      ERR_PILOT_COPY_FAILED: Result := 'The PltInstallFile function could not copy the file.';
      ERR_PILOT_FILE_ALREADY_EXISTS: Result := 'An attempt was made to install a file that is already installed in the user'' s install directory.';
      ERR_PILOT_INVALID_BUFFER: Result := 'A buffer parameter is NULL.';
      ERR_PILOT_INVALID_FILE_INDEX: Result := 'The value of the index parameter passed to the PltGetFileName function is greater than the number of files.';
      ERR_PILOT_INVALID_FILE_TYPE: Result := 'A file parameter is not a valid file type for use with the function.';
      ERR_PILOT_INVALID_FILENAME: Result := 'The file name parameter value in a call to the PltInstallFile function is NULL.';
      ERR_PILOT_INVALID_INDEX: Result := 'An index parameter exceeds the maximum value allowed for the index.';
      ERR_PILOT_INVALID_PATH: Result := 'The path parameter passed to the PltSetPath function is NULL.';
      ERR_PILOT_INVALID_PATH_TYPE: Result := 'The path type parameter value is not a valid path type value. For information about the path type constants, see Path Type Constants.';
      ERR_PILOT_INVALID_REGISTRY: Result := 'The requested path information is not stored in the Palm configuration entries. The Palm Desktop installation program should be run to correct this problem.';
      ERR_PILOT_INVALID_SOURCE_FILE: Result := 'A file passed to the install function is not a valid file.';
      ERR_PILOT_INVALID_USER: Result := 'The user name parameter value is either NULL or is not a valid user name.';
      ERR_PILOT_INVALID_USER_INDEX: Result := 'The value of the index parameter sent to the PltGetUser function is greater than the number of users.';
      ERR_PILOT_NO_USERS: Result := 'The users data file does not contain any user information.';
      ERR_PILOT_NO_USERSDAT_FILE: Result := 'There is no users data file specified in the Palm configuration entries.';
      ERR_PILOT_NO_DIRECTORY : Result := 'No directory was found for the specified user.';
      ERR_PILOT_NO_USER_FILE : Result := 'No users data file found.';
      ERR_PILOT_NOT_FOUND: Result := 'The specified user name could not be found in the database.';
      ERR_PILOT_BAD_FILENAME : Result := 'The specified file name is invalid.';
      ERR_PILOT_USER_ACCESS : Result := 'The Install Aide cannot access the user data.';
 //////////////// end of Mert ///////////////////////////////////


      ERR_REGISTRY_ACCESS: Result := 'Unable to access the configuration entries.';

    ERR_UNABLE_TO_CREATE_CONDUIT: Result := 'The conduit could not be installed.';
    ERR_UNABLE_TO_DELETE: Result := 'The configuration entries for the specified conduit could not be deleted.';
    ERR_UNABLE_TO_SET_CONDUIT_VALUE: Result := 'The specified conduit configuration entry value could not be set.';
    ERR_VALUE_NOT_FOUND: Result := 'The specified value could not be found in the configuration entries for the conduit.';

//      ERROR_HSAPI_FAILURE: Result := 'The API cannot communicate with the HotSync Manager.';
//      ERROR_HSAPI_HOTSYNC_NOT_FOUND: Result := 'HotSync Manager is not running.';
//      ERROR_HSAPI_INVALID_CONN_TYPE: Result := 'The specified connection type is not one defined in the HotSync API Connection Type (HSConnectionType) Constants.';
//      ERROR_HSAPI_INVALID_STATUS_FLAG: Result := 'The specified connection type status is not one defined in the HotSync API Connection Type Status Constants.';
//      ERROR_HSAPI_REG_FAILURE: Result := 'The API cannot access configuration entries.';
//      ERROR_HSAPI_UNABLE_TO_CLOSE: Result := 'The API cannot close the HotSync Manager application.';
//      ERROR_HSAPI_UNABLE_TO_START: Result := 'The API cannot start the HotSync Manager application.';
//      ERROR_HSAPI_UNKNOWN_STATUS_TYPE: Result := 'The specified status type is not one defined in the HotSync API Status Type (HsStatusType) Constants.';

    SYNCERR_ARG_MISSING: Result := 'An internal Desktop Link error that indicates a protocol implementation error.';
    SYNCERR_BAD_ARG: Result := 'An invalid parameter has been passed to a function, or the parameter is too large.';
    SYNCERR_BAD_ARG_WRAPPER: Result := 'An internal Desktop Link error that indicates a protocol implementation error';
    SYNCERR_BAD_OPERATION: Result := 'The requested operation is not supported on the given database type (record or resource).';
    SYNCERR_COMM_NOT_INIT: Result := 'An internal error code that indicates communications have not been initialized.';
    SYNCERR_FILE_ALREADY_EXIST: Result := 'The database could not be created because another database with the same name already exists on the handheld.';
    SYNCERR_FILE_ALREADY_OPEN: Result := 'The requested database is already opened.';
    SYNCERR_FILE_NOT_OPEN: Result := 'The attempt to open the database failed.';
    SYNCERR_FILE_OPEN: Result := 'Not used.';
    SYNCERR_LIMIT_EXCEEDED: Result := 'A data limit has been exceeded on the handheld. For example, this happens when the HotSync Manager error log size limit has been exceeded on the handheld.';
    SYNCERR_LOCAL_BUFF_TOO_SMALL: Result := 'The passed buffer is too small for the reply data.';
    SYNCERR_LOCAL_CANCEL_SYNC: Result := 'The HotSync Manager was cancelled by the desktop computer user.';
    SYNCERR_LOCAL_MEM: Result := 'A memory allocation error occurred on the desktop computer.';
    SYNCERR_LOST_CONNECTION: Result := 'The connection with the handheld was lost.';
    SYNCERR_MORE: Result := 'Not used.';
    SYNCERR_NO_FILES_OPEN: Result := 'An operation was requested on a database, and there are not any open databases.';
//      SYNCERR_NONE: Result := 'The function call succeeded, without error.';
    SYNCERR_NOT_FOUND: Result := 'The requested database, record, resource, etc. could not be found.';
    SYNCERR_READ_ONLY: Result := 'Caller does not have write access or database is in ROM';
    SYNCERR_RECORD_BUSY: Result := 'The requested record is in use by someone else and will remain so indefinitely.';
    SYNCERR_RECORD_DELETED: Result := 'The requested record has either been deleted or archived.';
    SYNCERR_REMOTE_BAD_ARG: Result := 'An invalid argument has been passed to the handheld.';
    SYNCERR_REMOTE_CANCEL_SYNC: Result := 'The HotSync Manager was cancelled by the handheld user.';
    SYNCERR_REMOTE_MEM: Result := 'There is insufficient memory on the handheld to receive or complete the request.';
    SYNCERR_REMOTE_NO_SPACE: Result := 'There is insufficient memory in the data store on the handheld to complete the request. This generally occurs when attempting to write a record or resource to a handheld database.';
    SYNCERR_REMOTE_SYS: Result := 'A generic system error on the handheld. This is returned when the exact cause is unknown.';
    SYNCERR_TOO_MANY_OPEN_FILES: Result := 'Request failed because there are too many open databases (for efficiency, the current Desktop Link implementation supports only one open database at a time).';
    SYNCERR_UNKNOWN: Result := 'An unknown error occurred (local/remote)';

//    expErrUnsupportedOperation: Result := 'Unsupported or undefined opcode and/or creator.';
//    expErrCardNotPresent: Result := 'No card is present.';
//    expErrInvalidSlotRefNum: Result := 'The slot reference number is not valid.';
//    expErrSlotDeallocated: Result := 'The slot reference number is within the valid range, but the slot driver has been deallocated.';
//    expErrCardNoSectorReadWrite: Result := 'The card does not support the slot driver block read/write API';
  end;
end;

(* User Data Error Codes  *)
(* Moved lines and Removed slashes by Mert *)
function TranslateUserManagerErrors(const PalmError: integer): string;
begin
  case PalmError of
    -1: Result := 'A nonspecific error.';
      ERR_UM_BAD_FILENAME: Result := 'The file passed to the API is not a valid file type.';
      ERR_UM_BUFSIZE_TOO_SMALL: Result := 'The supplied buffer is too small to contain the information requested.';
      ERR_UM_CANNOT_WRITE_TO_STORE: Result := 'The attempt to write to the sync.ini file failed.';
      ERR_UM_INVALID_POINTER: Result := 'The specified pointer is invalid.';
      ERR_UM_INVALID_USER: Result := 'The user parameter supplied is NULL or not an available user name.';
      ERR_UM_INVALID_USER_DIR: Result := 'The specified user directory is invalid.';
      ERR_UM_INVALID_USER_NAME: Result := 'The specified user name is invalid.';
      ERR_UM_NO_CORE_PATH: Result := 'The core (also called base) path does not exist.';
      ERR_UM_NO_DIRECTORY: Result := 'The user'' s directory does not exist.';
      ERR_UM_NO_USER_FOUND: Result := 'The user does not exist in the users data file.';
      ERR_UM_NO_USERS: Result := 'The users data file exists, but contains no information. A user ID is required for most functions.';
      ERR_UM_NO_USERSDAT_FILE: Result := 'No users data file was found. Most of the functions fail in this case. Call UmAddUser to create the file.';
      ERR_UM_NOT_FOUND: Result := 'The user name parameter is not already contained in the users data file.';
      ERR_UM_OTHER_USERSDAT_ACCESS_PROBLEM: Result := 'A miscellaneous error occurred while trying to access the user.dat file.';
      ERR_UM_SAVE_ERR: Result := 'A save was not successfully completed.';
      ERR_UM_SEMAPHORE_ACCESS: Result := 'The specified user index is invalid.';
      ERR_UM_STRING_TOO_BIG: Result := 'The string being retrieved is too large to be stored.';
      ERR_UM_UNABLE_TO_CREATE_NEW_FILE: Result := 'Creating a new file failed.';
      ERR_UM_USER_ACCESS: Result := 'The API cannot access the user data.';
      ERR_UM_USER_ALREADY_EXISTS: Result := 'The specified user already exists.';
      ERR_UM_USER_DIR_ALREADY_IN_USE: Result := 'The specified user directory is in use.';
  else
  end;
end;

procedure UserManagerCheck(Error: Integer);
Begin
  if Error <> SYNCERR_NONE then
    raise EUserManagerError.CreateErr(Error);
end;

////////////// End of Mert /////////////////////

{ EUserManagerError }

constructor EUserManagerError.CreateErr(const AError: Integer);
begin
  inherited CreateFmt('Error %d :%s', [AError, TranslateUserManagerErrors(AError)]);
  FError := AError;
end;

constructor EUserManagerError.CreateErrMsg(const AError: Integer;
  const Msg: string);
begin
  inherited CreateFmt('Error %d,:%s at %s', [AError, TranslateUserManagerErrors(AError), Msg]);
  FError := AError;
end;

end.

