unit pfgSyncMgr;
{**************************************************************************}
{* pfgSyncMgr Unit                                                        *}
{*                                                                        *}
{* Header file conversion of pfgSyncMgr.h                                 *}
{*                                                                        *}
{* Header file converted by Paul Gilbert Nov 2000.                        *}
{**************************************************************************}

interface

uses pfgWTypes, SysUtils, Windows;

{
/*****************************************************************************
 *
 * Copyright (c) 1998-1999 Palm Computing, Inc. or its subsidiaries.  
 * All rights reserved.
 *
 ****************************************************************************/

/////////////////////////////////////////////////////////////////////////////
//	 File:      syncmgr.h  
//
//	 Module:    SYNCMGR.DLL 
//
//
//	 Description:  Publicly consumable header file prototyping the 'C' API
//                 and the structures used for their parameters.
//                 When using the Microsoft compiler we asure packed 
//                 structures on single byte boundaries, with the pragma(1).
//
//
/////////////////////////////////////////////////////////////////////////////
//	 REVISION HISTORY:
// mja	7/23/95  initial 
//	vmk	8/12/96	renamed the unused "readOnlyDB" field in CDBList to m_miscFlags
//	jim 8/16/96 removed CSyncLog from CSyncProperties
// vmk	10/21/96	added SYNC API versioning scheme and SyncGetAPIVersion() function.
//	vmk	1/29/97	accurately documented SYNCERR_... error codes
//  jayita  10/10/97  moved conduit API to CondAPI.h to remove
//                             dependency on data subscription(aka FileLink) for Mac developers
//	dlw	5/24/99 	Added FSFileSpec to CSyncPreference for Mac version of generic conduit
//
/////////////////////////////////////////////////////////////////////////////
}

const
  // Maximum length of Viewer's database name,
  // *including* the null-terminator
  SYNC_DB_NAMELEN    = (32);
  DB_NAMELEN	     = SYNC_DB_NAMELEN;

  // Maximum size of Viewer's HotSync log
  SYNC_MAX_HH_LOG_SIZE = (2*1024);

  BIG_PATH             = 256;

  // Major API version numbers
  SYNCAPI_VER_MAJOR_2  = 2;

  // Minor API version numbers
  SYNCAPI_VER_MINOR_0  = 0;
  SYNCAPI_VER_MINOR_1  = 1;
  SYNCAPI_VER_MINOR_2  = 2;


type
  { Enumerations }
  //
  //  Common sync properties structure populated by HotSync.exe and passed
  //  into the conduit's OpenConduit() function as a parameter.
  //
  eSyncTypes = LongWord;
  eFirstSync = LongWord;
  eConnType = LongWord;
  eSyncPref = LongWord;
  // These attributes are used in CRawRecordInfo m_Attribs field:
  // (*All unused bits are reserved by Sync Manager and PalmOS*)
  eSyncRecAttrs = Integer;

  eDbOpenModes = Integer;
  //
  //  Flags which can be set in CreateDB structure or ReadDbList struct
  //
  eDbFlags = Integer;
  //
  // Miscellaneous flags which can be returned in CDbList's m_miscFlags member
  //
  eMiscDbListFlags = Integer;


const
  { eSyncTypes enumeration }
  eFast=0; eSlow=1; eHHtoPC=2; ePCtoHH=3; eInstall=4; eBackup=5;
  eDoNothing=6; eProfileInstall=7; eSyncTypeDoNotUse=$ffffffff;

  { eFirstSync enumeration }
  eNeither=0; ePC=1; eHH=2; eFirstSyncDoNotUse=$ffffffff;

  { eConnType enumeration }
  eCable=0; eModemConnType=1; eConnTypeDoNotUse=$ffffffff;
  eModem=eModemConnType;

  { eSyncPref enumeration }
  eNoPreference=0; ePermanentPreference=1; eTemporaryPreference=2;
  eSyncPrefDoNotUse=$ffffffff;

  { eSyncRecAttrs enumeration }
  eRecAttrDeleted = $80;	// indicates that this record has been deleted on the handheld
  eRecAttrDirty	  = $40;	// indicates that this record was modified
  eRecAttrBusy	  = $20;	// SYSTEM USE ONLY: indicates that this record is currently in use
			        // by some application on the remote, hand-held device.
				// CONDUITS: this attribute must be treated as read-only; do *not* pass
				// eRecAttrBusy when writing records.
  eRecAttrSecret       = $10;	// "secret" record - password protected (also known as "private")
  eRecAttrArchived     = $08;	// indicates that this record has been marked for archival
  eSyncRecAttrDoNotUse = $ffffffff;

  //
  // values for m_Attribs in CRawRecordInfo ( as used in our sample code )
  // the #defines below will be obsoleted, please use the enums (above) in your conduit
  DELETE_BIT   = eRecAttrDeleted;
  DIRTY_BIT    = eRecAttrDirty;
  PRIVATE_BIT  = eRecAttrSecret;
  ARCHIVE_BIT  = eRecAttrArchived;

  { eDbOpenModes enumeration }
  eDbShowSecret  = $0010;
  eDbExclusive   = $0020;
  eDbWrite       = $0040;
  eDbRead        = $0080;
  eDbOpenModesDoNotUse=$ffffffff;

  { eDbFlags enumeration }
  eRecord		= $0000;    // *this is not really a flag* -- see eResource
  eResource		= $0001;    // if set, indicates a resource database; if clear, indicates a record database
  eReadOnly		= $0002;    // if set, indicates a ROM-based database
  eAppInfoDirty		= $0004;    // set if Application Info Block is dirty
  eBackupDB		= $0008;    // set if database should be backed up to PC if no app-specific conduit has been supplied
  eOkToInstallNewer	= $0010;    // this tells the backup/restore conduit that it's OK for it to install a newer
                                    // version of this database with a different name if the current database is open.  This
                                    // mechanism is used to update the Graffiti Shortcuts databsae, for example.
  eResetAfterInstall    = $0020;    // if set, indicates that the device should be reset after this database is installed.  The
				    // actual reset will take place at end of sync.
  eCopyPrevention      	= $0040;    // supported from v3.0 on
  eOpenDB		= $8000;    // indicates that the databse is open
  eDbFlagsDoNotUse      = $ffffffff;

  //
  // Miscellaneous flags which can be returned in CDbList's m_miscFlags member
  //
  { eMiscDbListFlags enumeration }
  // eMiscDbFlagExcludeFromSync: if this miscellaneous flag is set, indicates that
  // the db should be excluded fro HotSync;  this is typically the result of the user
  // disabling synchronization for the owning application on the handheld;  this functionality
  // was defined in DLP v1.1 (beginning with PalmOS v2.0).
  eMiscDbFlagExcludeFromSync	= $0080;     // if set, indicates that this database should be excluded from HotSync. (DLP v1.1)

  // eMiscDbFlagRamBased: if this miscellaneous flag is set, indicates that the db is
  // in RAM; otherwise, it is in ROM; this flag is returned from devices with Desktop Link
  // Protocol (DLP) v1.2 or later (beginning with PalmOS v3.0).
  eMiscDbFlagRamBased		= $0040;     // if set, indicates that this database is RAM-based. (DLP v1.2)
  eMiscDbFlagsDoNotUse		= $ffffffff;
  // eExcludeFromSync is defined for backward compatibility; the new name is eMiscDbFlagExcludeFromSync
  eExcludeFromSync	        = eMiscDbFlagExcludeFromSync;


  // Buffer size for handheld username
  SYNC_REMOTE_USERNAME_BUF_SIZE	  = (64);
  REMOTE_USERNAME                 = SYNC_REMOTE_USERNAME_BUF_SIZE;   // for backward compatibility

  // Maximum handheld username length presently allowed (not including the null-terminator)
  SYNC_MAX_USERNAME_LENGTH	  = (20);

  // Buffer size for handheld memory card name
  SYNC_REMOTE_CARDNAME_BUF_SIZE	  = (32);
  REMOTE_CARDNAMELEN              = SYNC_REMOTE_CARDNAME_BUF_SIZE;   // for backward compatibility

  // Buffer size for handheld manufacturer name
  SYNC_REMOTE_MANUFNAME_BUF_SIZE  = (32);
  REMOTE_MANUFNAMELEN             = SYNC_REMOTE_MANUFNAME_BUF_SIZE;  // for backward compatibility

  // Buffer size for handheld password
  SYNC_REMOTE_PASSWORD_BUF_SIZE   = (64);
  PASSWORD_LENGTH		  = SYNC_REMOTE_PASSWORD_BUF_SIZE;   // for backward compatibility

  // Product ID buffer size in number of byts
  SYNC_MAX_PROD_ID_SIZE		  = (255);

  // Database info retrieval options

  SYNC_DB_INFO_OPT_GET_ATTRIBUTES    = ($80);
     // Set to get database attributes (name, creator, type,
     // flags, etc.) -- this is an option to allow find operations
     // to skip returning this data as a performance optimization
     // time, which we would rather avoid if it is not needed

  SYNC_DB_INFO_OPT_GET_SIZE	     = ($40);
     // Set to get record count and data size also -- this is
     // an option because the operation can take a long
     // time, which we would rather avoid if it is not needed

  SYNC_DB_INFO_OPT_GET_MAX_REC_SIZE  = ($20);
     // Set to get max rec/resource size also -- this is
     // an option because the operation can take a long
     // time, which we would rather avoid if it is not needed
     // (SYNC_DB_INFO_OPT_GET_MAX_REC_SIZE is only supported for
     // SyncReadOpenDbInfo)

  // Search option flags for the SyncFindDbByTypeCreatorParams structure
  SYNC_DB_SRCH_OPT_NEW_SEARCH	     = ($80);	// set to beging a new search; subsequent
																// iterations of the same search should
																// have the flag cleared
  SYNC_DB_SRCH_OPT_ONLY_LATEST       = ($40);   // set to search for the latest version

  ///////////////////////////////////////////////////////////////////////////////
  //
  // Option flags for SyncCloseDBEx
  //
  ///////////////////////////////////////////////////////////////////////////////
  SYNC_CLOSE_DB_OPT_UPDATE_BACKUP_DATE  = $80;	// Update the backup date after closing
  SYNC_CLOSE_DB_OPT_UPDATE_MOD_DATE	= $40;	// Update the modification date after closing


  ///////////////////////////////////////////////////////////////////////////////
  //
  // Error Code Classes
  //
  ///////////////////////////////////////////////////////////////////////////////
  COND_ERR_CLASS    = $00001000;    // Conduit error class
  TRANS_ERR_CLASS   = $00002000;    // Communications/Transport error class
  SYNC_ERR_CLASS    = $00004000;    // Sync Manager DLL error class
  HSAPP_ERR_CLASS   = $00008000;    // HotSync application error class


  ///////////////////////////////////////////////////////////////////////////////
  //
  //  Error Codes returned by the SyncManager calls as a result
  //  of communication over the link to the HH device.
  //
  ///////////////////////////////////////////////////////////////////////////////

  SYNC_FATAL_ERR_MASK	       = $10000000;
  SYNC_FATAL_ERR	       = (SYNC_FATAL_ERR_MASK + SYNC_ERR_CLASS);

  SYNCERR_NONE                 = $0000;			  // SUCCESS!

  SYNCERR_UNKNOWN              = (SYNC_ERR_CLASS + $01);  // An unknown error occurred (local/remote
  SYNCERR_MORE                 = (SYNC_ERR_CLASS + $02);  // NOT USED
  SYNCERR_NOT_FOUND	       = (SYNC_ERR_CLASS + $03);  // Requested database, record, resource, etc. not found
  SYNCERR_FILE_NOT_FOUND       = SYNCERR_NOT_FOUND;       // For backward compatibility
  SYNCERR_FILE_NOT_OPEN        = (SYNC_ERR_CLASS + $04);  // Attempt to open a database failed
  SYNCERR_FILE_OPEN            = (SYNC_ERR_CLASS + $05);  // NOT USED
  SYNCERR_RECORD_BUSY          = (SYNC_ERR_CLASS + $06);  // The requested record is in use by someone
		                                          // else and will remain so indefinitely
  SYNCERR_RECORD_DELETED       = (SYNC_ERR_CLASS + $07);  // The requested record has either been deleted
							  // or archived
  SYNCERR_READ_ONLY            = (SYNC_ERR_CLASS + $09);  // Caller does not have write access or database is in ROM
  SYNCERR_ROM_BASED            = SYNCERR_READ_ONLY;	  // Defined for backward compatility
  SYNCERR_COMM_NOT_INIT        = (SYNC_ERR_CLASS + $0A);  // Communications have not been intialized (this
							  // is an internal error code);
  SYNCERR_FILE_ALREADY_EXIST   = (SYNC_ERR_CLASS + $0B);  // Could not create database because another
							  // one with the same name already exists on remote
  SYNCERR_FILE_ALREADY_OPEN    = (SYNC_ERR_CLASS + $0C);  // The requested database is presently open by
							  // someone else
  SYNCERR_NO_FILES_OPEN        = (SYNC_ERR_CLASS + $0D);  // An operation was requested on a database when
							  // no databases were open
  SYNCERR_BAD_OPERATION        = (SYNC_ERR_CLASS + $0E);  // The requested operation is not supported on
							  // the given database type(record or resource);.
  SYNCERR_REMOTE_BAD_ARG       = (SYNC_ERR_CLASS + $0F);  // Invalid argument passed to remote
  SYNCERR_BAD_ARG_WRAPPER      = (SYNC_ERR_CLASS + $10);  // Internal Desktop Link error -- indicates
							  // protocol implementation error
  SYNCERR_ARG_MISSING          = (SYNC_ERR_CLASS + $11);  // Internal Desktop Link error -- indicates
							  // protocol implementation error
  SYNCERR_LOCAL_BUFF_TOO_SMALL = (SYNC_ERR_CLASS + $12);  // The passed buffer is too small for the
							  // reply data
  SYNCERR_REMOTE_MEM           = (SYNC_ERR_CLASS + $13);  // Insufficient memory on remote to receive
							  // or complete the request
  SYNCERR_REMOTE_NO_SPACE      = (SYNC_ERR_CLASS + $14);  // Insufficient memory in remote data store
							  // to complete the request (write record,
							  // resource, etc.);
  SYNCERR_REMOTE_SYS           = (SYNC_ERR_CLASS + $15);  // Generic remote system error (returned when
							  // exact cause is unknown);
							  // NEW v2.0 -- moved from fatal errors vmk 10/15/96
  SYNCERR_LOCAL_MEM            = (SYNC_ERR_CLASS + $16);  // Local (PC) memory allocation error
                                                          // NEW v2.0

  // NEW v2.1
  SYNCERR_BAD_ARG             = (SYNC_ERR_CLASS + $17);   // Invalid parameter to local function, or
                                                          // parameter is too big
  SYNCERR_LIMIT_EXCEEDED      = (SYNC_ERR_CLASS + $18);   // Data limit exceeded on remote (for example,
							  // when the hotsync error log size limit has
							  // has been exceeded on remote)
  SYNCERR_UNKNOWN_REQUEST     = (SYNC_ERR_CLASS + $19);   // This request (command) is not supported
                                                          // by remote

  // Fatal Errors

  SYNCERR_TOO_MANY_OPEN_FILES = (SYNC_FATAL_ERR + $403);  // Request failed because there are too many
							  // open databases (for efficiency, the current
							  // Desktop Link implementation supports only
							  // one open database at a time)
  SYNCERR_TOO_MANY_FILES      = SYNCERR_TOO_MANY_OPEN_FILES;// Defined for backward compatibility
  SYNCERR_REMOTE_CANCEL_SYNC  = (SYNC_FATAL_ERR + $405);   // The request to cancel HotSync was initiated
                                                          // from the remote
  SYNCERR_LOST_CONNECTION     = (SYNC_FATAL_ERR + TRANS_ERR_CLASS + $410);
                                                          // Connection is lost.  We add TRANS_ERR_CLASS
							  // because existing software checks that
							  // bit to detect connection loss.
							  // NEW v2.0
  SYNCERR_LOCAL_CANCEL_SYNC   =	(SYNC_FATAL_ERR + $411);  // The request to cancel HotSync was initiated
                                                          // from the PC


//
// Rom Version Macro functions
//
// $MMmfsbbb, where MM is major version, m is minor version
// f is bug fix, s is stage: 3-release,2-beta,1-alpha,0-development,
// bbb is build number for non-releases
// V1.12b3   would be: $01122003
// V2.00a2   would be: $02001002
// V1.01     would be: $01013000

function SYNCROMVMAJOR(l: LongWord): Byte;
function SYNCROMVMINOR(l: LongWord): Byte;

type
  //
  //  A single element for a ReadDBList function call.
  //

  CDbList = packed record
    m_CardNum: Integer;

    m_DbFlags: Word;       // contains Res/Record/Backup/ReadOnly (see enum eDbFlags)
    m_DbType: DWORD;
    m_Name: Array [0..SYNC_DB_NAMELEN-1] of Char;
    m_Creator: DWORD;
    m_Version: WORD;
    m_ModNumber: DWORD;
    m_Index: WORD;	   // not returned for SyncFindDbByName/TypeCreator
			   // and SyncReadOpenDbInfo
    m_CreateDate: Long;
    m_ModDate: Long;
    m_BackupDate: Long;
    m_miscFlags: LongWord;  // miscellaneous db list flags (see eMiscDbListFlags)
    m_RecCount: Long;	      // Unused - Not filled in by SyncManager calls
    m_dwReserved: Long;	    // Unused - set to null
  end;

  CDbListPtr = ^CDbList;
  PCDbList = CDbListPtr;
  PPCDbList = ^PCDbList;


  
  CSyncProperties = packed record
    m_SyncType: eSyncTypes;                     // Fast/Slow
    m_PathName: Array [0..BIG_PATH-1] of Char;  // Path to prepend for disk file names

    m_LocalName: Array [0..BIG_PATH-1] of Char; // Actual local disk file names
    m_UserName: Array [0..BIG_PATH-1] of Char;
    m_RemoteName: Array [0..SYNC_DB_NAMELEN-1] of PChar; // Names of remote database file names
    m_RemoteDbList: PPCDbList;			// CDbLists of remote databases
    m_nRemoteCount: Integer;	                // number of remote database files
    m_Creator: DWORD;                           // needed to create remote Db's
    m_CardNo: WORD;                             // needed to create remote Db's
    m_DbType: DWORD;                            // needed to create remote Db's
    m_AppInfoSize: DWORD;                       // convenience
    m_SortInfoSize: DWORD;                      // convenience
    m_FirstDevice: eFirstSync;                  // First time sync for 1 device
    m_Connection: eConnType;                    // Transfer medium
    m_Registry: Array [0..BIG_PATH-1] of Char;  // Full registry path for the conduit
    m_hKey: HKEY;                               // primary registry key
    m_dwReserved: DWORD;                        // Reserved - set to NULL
  end;

  CSyncPreference = packed record
    m_PathName: Array [0..BIG_PATH-1] of Char;  // Path to prepend for disk file names

    m_Registry: Array [0..BIG_PATH-1] of Char;  // Full registry path for the conduit
    m_hKey: HKEY;                               // primary registry key
    m_SyncPref: eSyncPref;                      // sync pref - perm/temp
    m_SyncType: eSyncTypes;                     // sync type - fast/install
    m_dwReserved: DWORD;                        // Reserved - set to NULL
  end;

  //
  //  Used by the calls to obtain the remote Categories
  //

  CDbGenInfo = packed record
    m_FileName: Array [0..SYNC_DB_NAMELEN-1] of Char;  // Name of remote database file
    m_TotalBytes: Word;			        // When reading, the caller must fill this in
						// with the size of the buffer pointed to by m_pBytes;
						// When writing, the caller must set both this field
						// and m_BytesRead to the size of the block being written.

    m_BytesRead: Word;                          // *This field is poorly named*
                                                // When reading, it will
						// be filled in with the actual size of
						// the app or sort block, which
						// may be bigger than the amount of data which is
						// copied to m_pBytes in the event the block is bigger
						// than the buffer (in this case, only the first m_TotalBytes
						// of record data will be copied to caller's buffer by
                                                // Sync API v2.1 or later, and *NOTHING* will
                                                // be copied by Sync API before v2.1).
                                                // When writing, the caller must set this field (in addition to
                                                // m_TotalBytes) to the size of the block being written.

    m_pBytes: PByte;				// pointer to caller's buffer

    m_dwReserved: DWORD;                        // Reserved - set to NULL
  end;


  //
  //  Used by all the Record Oriented API's. Houses the DT_Link version
  //  of a database's record layout, specifically that of the remote device.
  //  Raw bytes will be formatted into this structure by the DTLinkConverter
  //  object which resides inside of each Conduit.DLL.
  //

  CRawRecordInfo = packed record
    m_FileHandle: Byte;                         // Supplied by caller

    m_RecId: DWORD;                             // Supplied by caller when reading or deleting records by record id; supplied by
						// caller as the resource type when deleting a resource; filled in
						// by HH when reading (unique record id for records and resource type for resources).

    m_RecIndex: Word;                           // Supplied by caller when reading records or resources by index; supplied by caller
						// as the resource id when deleting a resource; filled in by handheld as the resource
						// id when reading a resource; filled in by HH when reading a record using Sync API v2.1
						// or later.

    m_Attribs: Byte;                            // Filled in by HH when reading, and by caller when writing

    m_CatId: Smallint;                          // Filled in by HH when reading, and by caller when writing

    m_ConduitId: Integer;                       // Ignore

    m_RecSize: DWORD;                           // When reading, filled in by HH with the actual record/resource size,
						// which might be bigger than buffer size m_TotalBytes (in this
						// case, only the first m_TotalBytes of record data will be copied
						// to caller's buffer by Sync API v2.1 or later, and NOTHING will
						// be copied by Sync API before v2.1).  When writing, filled in by
						// caller with record data size (same as m_TotalBytes).

                                                // ****NOTE that m_TotalBytes is defined as WORD, meaning that only
						// records and resources under 64K may be read or written using this
						// API (the actual maximum is ~63.8K bytes).

    m_TotalBytes: Word;                         // Supplied by caller: buffer size for reading; record data size for writing

    m_pBytes: PByte;                            // Buffer allocated by caller for reading or writing

    m_dwReserved: DWORD;                        // Reserved	- set to NULL
  end;


  //
  //  Used by all the Preferences Oriented API's. Houses the DT_Link version
  //  of a preference record layout, specifically that of the remote device.
  //  Raw bytes will be formatted into this structure by the DTLinkConverter
  //  object which resides inside of each Conduit.DLL.
  //

  CRawPreferenceInfo = packed record
    m_version: Word;		                // Pref version - Supplied by caller in Write calls and by HH in Read calls
    m_creator: DWORD;                           // Pref creator - Supplied by caller
    m_prefId: Word;		                // Pref id - Supplied by caller
    m_reqBytes: Word;		                // (read calls only)Number of preference bytes requested - Supplied by caller ($FFFF means get the entire preference)
    m_retBytes: Word;		                // (read calls only)Number of preference bytes copied to caller's buffer
    m_actSize: Word;		                // (read calls only)Actual pref size - Supplied by HH
    m_backedUp: BOOL;		                // If true, the target pref database is the backed-up database - Supplied by caller
    m_nBytes: Long;		                // Buffer size in read calls, data size in write calls - Supplied by caller
    m_pBytes: PByte;		                // Data buffer - Allocated and supplied by caller
    m_dwReserved: DWORD;	                // Reserved - set to NULL
  end;


  //
  //  Used in the CreateDB function call.
  //

  CDbCreateDB = packed record
    m_FileHandle: BYTE;        // Upon return gets filled in by SyncMgr.Dll
    m_Creator: DWORD;          // Supplied by caller, obtained from DbList
    m_Flags: eDbFlags;         // Supplied by caller, Res/Rec/RAM
    m_CardNo: BYTE;            // Supplied by caller, target card #
    m_Name: Array [0..SYNC_DB_NAMELEN-1] of Char; // Supplied by caller, target DBase Name
    m_Type: DWORD;             // 'DATA' or 'APPL'...
    m_Version: WORD;
    m_dwReserved: DWORD;       // Reserved - set to NULL
  end;

  //
  //
  //

  CUserIDInfo = packed record
    m_pName: Array [0..SYNC_REMOTE_USERNAME_BUF_SIZE-1] of Char;
    m_NameLength: Integer;
    m_Password: Array [0..SYNC_REMOTE_PASSWORD_BUF_SIZE-1] of Char;
    m_PasswdLength: Integer;
    m_LastSyncDate: Long;      // Date/Time of last synchronization
    m_LastSyncPC: DWORD;
    m_Id: DWORD;
    m_ViewerId: DWORD;
    m_dwReserved: DWORD;       // Reserved - set to NULL
  end;


  //
  //  A structure element for the SyncReadSingleCardInfo() function call.
  //

  CCardInfo = packed record
    m_CardNo: Byte;
    m_CardVersion: Word;
    m_CreateDate: Long;
    m_RomSize: DWORD;
    m_RamSize: DWORD;
    m_FreeRam: DWORD;
    m_CardNameLen: Byte;
    m_ManufNameLen: Byte;
    m_CardName: Array [0..SYNC_REMOTE_CARDNAME_BUF_SIZE-1] of Char;
    m_ManufName: Array [0..SYNC_REMOTE_MANUFNAME_BUF_SIZE-1] of Char;

    // added in v1.1 and v2.0
    m_romDbCount: Word;		// number of ROM-based databases
    m_ramDbCount: Word;		// number of RAM-based databases

    m_dwReserved: DWORD; 	// Reserved - set to NULL
  end;


  //
  // Used to obtain remote system information.
  //

  CSystemInfo = packed record
    m_RomSoftVersion: DWORD;	// Upon return is filled in
    m_LocalId: DWORD;	     	// Upon return is filled in
    m_ProdIdLength: Byte;    	// Upon return is filled in (actual len)
    m_AllocedLen: Byte;		// Supplied by caller: size of buffer for ProductIdText
    m_ProductIdText: PByte;	// Allocated by caller: bufer for ProductIdText
    m_dwReserved: DWORD;     	// Reserved - set to NULL
  end;


  //
  //  Used by the 'SyncCallApplication()' API
  //

  CCallAppParams = packed record
    m_CreatorID: DWORD;
    m_ActionCode: Word;
    m_ResultCode: Word;
    m_ParamSize: Word;
    m_pParams: PByte;
  end;


  //
  //	Used by the 'SyncCallRemoteModule()' API for Pilot v2.0 and greater
  //	vmk	12/5/96
  //

  CCallModuleParams = packed record
    // PASSED VALUES:
    m_dwCreatorID: DWORD;	 	// target application creator ID
    m_dwTypeID: DWORD;		 	// target application type ID
    m_wActionCode: Word;	 	// app-specific action code
    m_dwParamSize: DWORD;	 	// parameter block size
    m_pParam: Pointer;		 	// pointer to parameter block
    m_dwResultBufSize: DWORD;		// result buffer size
    m_pResultBuf: Pointer;	 	// pointer to result buffer

    // RETURNED VALUES
    m_dwResultCode: DWORD;	    	// result code returned by remote module

    m_dwActResultSize: DWORD;		// the actual result data size returned by
												// remote module; this value may be greater
					// than m_dwResultBufSize upon return -- in
				        // which case the caller's buffer wasn't
					// big enough to contain the entire result
					// data block and only m_dwResultBufSize bytes
					// were copied to the result buffer.
    // RESERVED:
    m_dwReserved: DWORD;		// RESERVED -- CALLER/CALLEE: SET TO NULL!!!
  end;


  //
  //  Used by ReadPositionXMap
  //

  CPositionInfo = packed record
    m_FileHandle: Byte;                 // Open database handle
    m_FirstPos: Word;                   // offset of first position to read
    m_MaxEntries: Word;                 // total number of record Id's to read
    m_NumReadIn: Word;                  // updated with number read in
    m_TotalBytes: Word;                 // total length of 'pBytes'
    m_pBytes: PByte;                    // buffer to contain all record Id's
  end;


///////////////////////////////////////////////////////////////////////////////
//
// Parameter to SyncFindDbByName, SyncFindDbByTypeCreator, and SyncReadOpenDbInfo
//
///////////////////////////////////////////////////////////////////////////////

type
  // Structure for returning database information
  SyncDatabaseInfoType = packed record

    // Basic database information:
    baseInfo: CDbList;

    // Extended database information:

    // Returned only if SYNC_DB_INFO_OPT_GET_SIZE is set:
    dwNumRecords: DWORD; 	// number of records/resources in the database
    dwTotalBytes: DWORD;	// total bytes of storage used by database (including overhead)
    dwDataBytes: DWORD;		// total bytes of storage used for data

    // Returned only for SyncReadOpenDbInfo and only if SYNC_DB_INFO_OPT_GET_SIZE
    // is set:
    dwAppBlkSize: DWORD; 	// app info block size in number of bytes
    dwSortBlkSize: DWORD;	// sort info block size in number of bytes

    // Returned only for SyncReadOpenDbInfo and only if SYNC_DB_INFO_OPT_GET_MAX_REC_SIZE
    // is set:
    dwMaxRecSize: DWORD; 	// size of the largest record or resource in the
	                        // database

    dwReserved: DWORD;		// RESERVED -- SET TO NULL!!!


    // FIELDS FOR INTERNAL USE:
    dwLocalID: DWORD;		// LocalID of the database (internal use)
    dwOpenRef: DWORD;           // db open ref of the database if it is currently opened
  end;

  // SyncFindDbByName parameter
  SyncFindDbByNameParams = packed record
    bOptFlags: Byte;		// bitfield of SYNC_DB_INFO_OPT_... flags
    dwCardNum: DWORD;		// memory card number
    pcDatabaseName: PChar;	// pointer to null-terminated database name
  end;


  // SyncFindDbByTypeCreator parameter
  SyncFindDbByTypeCreatorParams = packed record
    bOptFlags: Byte;		// bitfield of SYNC_DB_INFO_OPT_... flags
    bSrchFlags: Byte;		// bitfield of SYNC_DB_SRCH_OPT_... flags
    dwType: DWORD;		// database type (zero = wildcard)
    dwCreator: DWORD;		// database creator (zero = wildcard)
  end;


  // SyncReadOpenDbInfo parameter
  SyncReadOpenDbInfoParams = packed record
    bOptFlags: Byte;		// bitfield of SYNC_DB_INFO_OPT_... flags
    bDbHandle: Byte;		// open database handle (from SyncOpenDB or SyncCreateDB)
  end;

{
///////////////////////////////////////////////////////////////////////////////////////////////
//
// Sync Manager API version numbers (v2.0 and later)
//
///////////////////////////////////////////////////////////////////////////////////////////////

/**********************************************************************************************
 * (v2.0 and later)
 *
 * The SYNC MANAGER API VERSION scheme enables the conduits to identify the level
 * of service provided by the Sync Manager DLL.
 *
 * MAJOR AND MINOR VERSION NUMBERS:
 *
 * Beginning with the current version (v2.0), the Sync Manager API will strive to
 * maintain backward compatibility within a given MAJOR version number group.  As new
 * exported functions are added to the API or critical bugs are fixed, the MINOR version number of
 * the API will be incremented and the documentation of the new functions will identify the
 * API version number where they were first available.  The conduits can (and should) check
 * the Sync Manager API version number using the function SyncGetAPIVersion().
 * So, if a conduit requires a particular Sync Manager function
 * which was added in API version number 2.1, for example, the conduit should call SyncGetAPIVersion
 * to make sure that the MAJOR number is 2 and the MINOR number is 1 or greater.
 *
 * SYNC MANAGER DLL NAMING CONVENTION:
 *
 * The Sync Manager DLL will be named to reflect the MAJOR API version number.  For example, the release
 * version of the Sync Manager DLL which implements a 2.x API will be named Sync20.dll.  The debug
 * version of same will be named Sync20d.dll.  This convention permits a conduit which depends on given
 * MAJOR and MINOR Sync Manager API version numbers to link with a Sync Manager DLL which implements an
 * API with the same MAJOR version number but a higher MINOR version number.
 *
 * vmk	10/21/96
 *
 **********************************************************************************************/
}

// NEW

// 1.0 CALLS:
var
  SyncAddLogEntry: function (const pText: PChar): Integer; cdecl; 
  SyncRegisterConduit: function (var rHandle: CONDHANDLE): Integer; cdecl; 
  SyncUnRegisterConduit: function (handle: CONDHANDLE): Integer; cdecl; 
  SyncReadUserID: function (const rInfo: CUserIDInfo): Integer; cdecl; 
  SyncOpenDB: function (const pName: PChar; nCardNum: Integer; var rHandle: Byte;
    openMode: Byte = (eDbWrite or eDbRead or eDbExclusive)): Integer; cdecl; 

  SyncDeleteDB: function (const pName: PChar; nCardNum: Integer): Integer; cdecl;
    
  SyncCreateDB: function (const rDbStats: CDbCreateDB): Integer; cdecl; 
  SyncCloseDB: function (fHandle: Byte): Integer; cdecl; 
  SyncGetDBRecordCount: function (fHandle: Byte; var rNumRecs: Word): Integer; cdecl; 
  SyncPurgeDeletedRecs: function (fHandle: Byte): Integer; cdecl; 
  SyncPurgeAllRecs: function (fHandle: Byte): Integer; cdecl; 
  SyncResetSyncFlags: function (fHandle: Byte): Integer; cdecl; 
  SyncReadDBList: function (cardNo: Byte; startIx: Word; bRam: BOOL; pList:
    PCDbList; var rCnt: Integer): Integer; cdecl; 
  SyncWriteRec: function (const rInfo: CRawRecordInfo): Integer; cdecl; 
  SyncDeleteRec: function (const rInfo: CRawRecordInfo): Integer; cdecl; 
  SyncDeleteResourceRec: function (const rInfo: CRawRecordInfo): Integer; cdecl; 
  SyncDeleteAllResourceRec: function (fHandle: Byte): Integer; cdecl; 
  SyncReadRecordById: function (const rInfo: CRawRecordInfo): Integer; cdecl; 
  SyncReadRecordByIndex: function (const rInfo: CRawRecordInfo): Integer; cdecl; 
  SyncReadResRecordByIndex: function (const rInfo: CRawRecordInfo; bBody: BOOL = True):
    Integer; cdecl; 
  SyncReadNextModifiedRec: function (const rInfo: CRawRecordInfo): Integer; cdecl; 
  SyncReadDBAppInfoBlock: function (fHandle: Byte; const rInfo: CDbGenInfo): Integer; cdecl; 
  SyncWriteDBAppInfoBlock: function (fHandle: Byte; const rInfo: CDbGenInfo): Integer; cdecl; 
  SyncWriteResourceRec: function (const rInfo: CRawRecordInfo): Integer; cdecl; 
  SyncRebootSystem: function: Integer; cdecl; 
  SyncReadSystemInfo: function (const rInfo: CSystemInfo): Integer; cdecl; 
  SyncReadSingleCardInfo: function (const rInfo: CCardInfo): Integer; cdecl; 
  SyncReadSysDateTime: function (var rDate: Long): Integer; cdecl; 
  SyncWriteSysDateTime: function (lDate: Long): Integer; cdecl; 
  SyncReadDBSortInfoBlock: function (fHandle: Byte; const rInfo: CDbGenInfo): Integer; cdecl; 
  SyncWriteDBSortInfoBlock: function (fHandle: Byte; const rInfo: CDbGenInfo): Integer; cdecl; 
  SyncCallApplication: function (var rOutParams: CCallAppParams; var rInParams:
    CCallAppParams): Integer; cdecl; 
  SyncChangeCategory: function (fHandle: Byte; bFrom, bTo: Byte): Integer; cdecl; 
  SyncReadPositionXMap: function (const rInfo: CPositionInfo): Integer; cdecl; 
  SyncYieldCycles: function (wMaxMiliSecs: Word): Integer; cdecl; 

  // Sync API v2.0 CALLS ADDED HERE:
  SyncReadNextRecInCategory: function (const rInfo: CRawRecordInfo): Integer; cdecl; 
  SyncReadNextModifiedRecInCategory: function (const rInfo: CRawRecordInfo): Integer; cdecl; 
  SyncReadAppPreference: function (const rInfo: CRawPreferenceInfo): Integer; cdecl; 
  SyncWriteAppPreference: function (const rInfo: CRawPreferenceInfo): Integer; cdecl; 
  SyncResetRecordIndex: function (fHandle: Byte): Integer; cdecl; 
  SyncPurgeAllRecsInCategory: function (fHandle: Byte; category: Shortint): Integer; cdecl; 
  SyncGetAPIVersion: function (var pdwMajor: DWORD; var pdwMinor: DWORD): Integer; cdecl; 

  // Sync API v2.1 CALLS ADDED HERE:
  SyncCallRemoteModule: function (const pParams: CCallModuleParams): Integer; cdecl; 
  SyncReadFeature: function (dwFtrCreator: DWORD; wFtrNum: WORD; var pdwFtrValue:
    DWORD): Integer; cdecl;
  SyncGetHHOSVersion: function (var pwRomVMinor: Word): Word; cdecl; 
  SyncHostToHHWord: function (wValue: Word): Word; cdecl; 
  SyncHHToHostWord: function (wValue: Word): Word; cdecl;
  SyncHostToHHDWord: function (dwValue: DWORD): DWORD; cdecl; 
  SyncHHToHostDWord: function (dwValue: DWORD): DWORD; cdecl; 

  //		Sync API v2.2 PUBLIC API:

  // Get the max record size that may be allocated
  // on the remote device (if sufficient free space exists).
  // ($FFFFFFFF = any size up to available free space; 0 = unknown)
  SyncMaxRemoteRecSize: function (var rdwMaxRecSize: DWORD): Integer; cdecl;

  // Looks for a database given its name and memory card number;
  // returns comprehensive information about the database, if found
  SyncFindDbByName: function (var rParam: SyncFindDbByNameParams;
     const rInfo: SyncDatabaseInfoType): Integer; cdecl;

  // Looks for a database given its type and creator; can be iterated; wild cards are
  // supported; returns comprehensive information about the database, if found
  SyncFindDbByTypeCreator: function (var rParam: SyncFindDbByTypeCreatorParams;
     const rInfo: SyncDatabaseInfoType): Integer; cdecl;

  // Returns comprehensive information about an open database
  SyncReadOpenDbInfo: function (var rParam: SyncReadOpenDbInfoParams;
     const rInfo: SyncDatabaseInfoType): Integer; cdecl;

  // Closes a database and optionally updates its backup
  // and/or modification date; bOptFlags may be zero or one
  // or more SYNC_CLOSE_DB_OPT_... flags bitwise or'ed
  // together (***non-zero bOptFlags may only be passed to PalmOS v3.0
  // or later; passing non-zero flags to older version will cause the
  // close command to fail***)
  SyncCloseDBEx: function (dbHandle: Byte; bOptFlags: Byte): Integer; cdecl;

  // Sync API v2.3 calls added here }
  SyncLoopBackTest: function(dwSizeSend: LongInt; pDataSend: PBYTE;
                             var pdwSizeRecv: LongInt;
                             var pDataRecv: BYTE): Integer; cdecl;


implementation

uses Dialogs, pfgUserData;

var
  DllHandle : THandle;
  DllLoaded : Boolean;


function SYNCROMVMAJOR(l: LongWord): Byte;
begin
  Result := (l shr 24) and $FF;
end;

function SYNCROMVMINOR(l: LongWord): Byte;
begin
  Result := (l shr 20) and $F;
end;


procedure LoadDLL;
var
  OldCurrentDir: string;
begin
  if DLLLoaded then Exit;

  // We need to change the directory to the Palm one, since the stupid dum
  // sync20.dll can't find the cmds21.dll file otherwise
  OldCurrentDir := GetCurrentDir;
  try
    SetCurrentDir(PalmPath);
    DLLHandle := LoadLibrary('sync20.dll');
  finally
    SetCurrentDir(OldCurrentDir);
  end;
  DLLLoaded := DLLHandle >= 32;
  if DLLLoaded then
  begin
    @SyncAddLogEntry := GetProcAddress(DLLHandle,'SyncAddLogEntry');
    Assert(@SyncAddLogEntry <> nil);
    @SyncRegisterConduit := GetProcAddress(DLLHandle,'SyncRegisterConduit');
    Assert(@SyncRegisterConduit <> nil);
    @SyncUnRegisterConduit := GetProcAddress(DLLHandle,'SyncUnRegisterConduit');
    Assert(@SyncUnRegisterConduit <> nil);
    @SyncReadUserID := GetProcAddress(DLLHandle,'SyncReadUserID');
    Assert(@SyncReadUserID <> nil);
    @SyncOpenDB := GetProcAddress(DLLHandle,'SyncOpenDB');
    Assert(@SyncOpenDB <> nil);
    @SyncDeleteDB := GetProcAddress(DLLHandle,'SyncDeleteDB');
    Assert(@SyncDeleteDB <> nil);
    @SyncCreateDB := GetProcAddress(DLLHandle,'SyncCreateDB');
    Assert(@SyncCreateDB <> nil);
    @SyncCloseDB := GetProcAddress(DLLHandle,'SyncCloseDB');
    Assert(@SyncCloseDB <> nil);
    @SyncGetDBRecordCount := GetProcAddress(DLLHandle,'SyncGetDBRecordCount');
    Assert(@SyncGetDBRecordCount <> nil);
    @SyncPurgeDeletedRecs := GetProcAddress(DLLHandle,'SyncPurgeDeletedRecs');
    Assert(@SyncPurgeDeletedRecs <> nil);
    @SyncPurgeAllRecs := GetProcAddress(DLLHandle,'SyncPurgeAllRecs');
    Assert(@SyncPurgeAllRecs <> nil);
    @SyncResetSyncFlags := GetProcAddress(DLLHandle,'SyncResetSyncFlags');
    Assert(@SyncResetSyncFlags <> nil);
    @SyncReadDBList := GetProcAddress(DLLHandle,'SyncReadDBList');
    Assert(@SyncReadDBList <> nil);
    @SyncWriteRec := GetProcAddress(DLLHandle,'SyncWriteRec');
    Assert(@SyncWriteRec <> nil);
    @SyncDeleteRec := GetProcAddress(DLLHandle,'SyncDeleteRec');
    Assert(@SyncDeleteRec <> nil);
    @SyncDeleteResourceRec := GetProcAddress(DLLHandle,'SyncDeleteResourceRec');
    Assert(@SyncDeleteResourceRec <> nil);
    @SyncDeleteAllResourceRec := GetProcAddress(DLLHandle,'SyncDeleteAllResourceRec');
    Assert(@SyncDeleteAllResourceRec <> nil);
    @SyncReadRecordById := GetProcAddress(DLLHandle,'SyncReadRecordById');
    Assert(@SyncReadRecordById <> nil);
    @SyncReadRecordByIndex := GetProcAddress(DLLHandle,'SyncReadRecordByIndex');
    Assert(@SyncReadRecordByIndex <> nil);
    @SyncReadResRecordByIndex := GetProcAddress(DLLHandle,'SyncReadResRecordByIndex');
    Assert(@SyncReadResRecordByIndex <> nil);
    @SyncReadNextModifiedRec := GetProcAddress(DLLHandle,'SyncReadNextModifiedRec');
    Assert(@SyncReadNextModifiedRec <> nil);
    @SyncReadDBAppInfoBlock := GetProcAddress(DLLHandle,'SyncReadDBAppInfoBlock');
    Assert(@SyncReadDBAppInfoBlock <> nil);
    @SyncWriteDBAppInfoBlock := GetProcAddress(DLLHandle,'SyncWriteDBAppInfoBlock');
    Assert(@SyncWriteDBAppInfoBlock <> nil);
    @SyncWriteResourceRec := GetProcAddress(DLLHandle,'SyncWriteResourceRec');
    Assert(@SyncWriteResourceRec <> nil);
    @SyncRebootSystem := GetProcAddress(DLLHandle,'SyncRebootSystem');
    Assert(@SyncRebootSystem <> nil);
    @SyncReadSystemInfo := GetProcAddress(DLLHandle,'SyncReadSystemInfo');
    Assert(@SyncReadSystemInfo <> nil);
    @SyncReadSingleCardInfo := GetProcAddress(DLLHandle,'SyncReadSingleCardInfo');
    Assert(@SyncReadSingleCardInfo <> nil);
    @SyncReadSysDateTime := GetProcAddress(DLLHandle,'SyncReadSysDateTime');
    Assert(@SyncReadSysDateTime <> nil);
    @SyncWriteSysDateTime := GetProcAddress(DLLHandle,'SyncWriteSysDateTime');
    Assert(@SyncWriteSysDateTime <> nil);
    @SyncReadDBSortInfoBlock := GetProcAddress(DLLHandle,'SyncReadDBSortInfoBlock');
    Assert(@SyncReadDBSortInfoBlock <> nil);
    @SyncWriteDBSortInfoBlock := GetProcAddress(DLLHandle,'SyncWriteDBSortInfoBlock');
    Assert(@SyncWriteDBSortInfoBlock <> nil);
    @SyncCallApplication := GetProcAddress(DLLHandle,'SyncCallApplication');
    Assert(@SyncCallApplication <> nil);
    @SyncChangeCategory := GetProcAddress(DLLHandle,'SyncChangeCategory');
    Assert(@SyncChangeCategory <> nil);
    @SyncReadPositionXMap := GetProcAddress(DLLHandle,'SyncReadPositionXMap');
    Assert(@SyncReadPositionXMap <> nil);
    @SyncYieldCycles := GetProcAddress(DLLHandle,'SyncYieldCycles');
    Assert(@SyncYieldCycles <> nil);
    @SyncReadNextRecInCategory := GetProcAddress(DLLHandle,'SyncReadNextRecInCategory');
    Assert(@SyncReadNextRecInCategory <> nil);
    @SyncReadNextModifiedRecInCategory := GetProcAddress(DLLHandle,'SyncReadNextModifiedRecInCategory');
    Assert(@SyncReadNextModifiedRecInCategory <> nil);
    @SyncReadAppPreference := GetProcAddress(DLLHandle,'SyncReadAppPreference');
    Assert(@SyncReadAppPreference <> nil);
    @SyncWriteAppPreference := GetProcAddress(DLLHandle,'SyncWriteAppPreference');
    Assert(@SyncWriteAppPreference <> nil);
    @SyncResetRecordIndex := GetProcAddress(DLLHandle,'SyncResetRecordIndex');
    Assert(@SyncResetRecordIndex <> nil);
    @SyncPurgeAllRecsInCategory := GetProcAddress(DLLHandle,'SyncPurgeAllRecsInCategory');
    Assert(@SyncPurgeAllRecsInCategory <> nil);
    @SyncGetAPIVersion := GetProcAddress(DLLHandle,'SyncGetAPIVersion');
    Assert(@SyncGetAPIVersion <> nil);
    @SyncCallRemoteModule := GetProcAddress(DLLHandle,'SyncCallRemoteModule');
    Assert(@SyncCallRemoteModule <> nil);
    @SyncReadFeature := GetProcAddress(DLLHandle,'SyncReadFeature');
    Assert(@SyncReadFeature <> nil);
    @SyncGetHHOSVersion := GetProcAddress(DLLHandle,'SyncGetHHOSVersion');
    Assert(@SyncGetHHOSVersion <> nil);
    @SyncHostToHHWord := GetProcAddress(DLLHandle,'SyncHostToHHWord');
    Assert(@SyncHostToHHWord <> nil);
    @SyncHHToHostWord := GetProcAddress(DLLHandle,'SyncHHToHostWord');
    Assert(@SyncHHToHostWord <> nil);
    @SyncHostToHHDWord := GetProcAddress(DLLHandle,'SyncHostToHHDWord');
    Assert(@SyncHostToHHDWord <> nil);
    @SyncHHToHostDWord := GetProcAddress(DLLHandle,'SyncHHToHostDWord');
    Assert(@SyncHHToHostDWord <> nil);
    @SyncMaxRemoteRecSize := GetProcAddress(DLLHandle,'SyncMaxRemoteRecSize');
    Assert(@SyncMaxRemoteRecSize <> nil);
    @SyncFindDbByName := GetProcAddress(DLLHandle,'SyncFindDbByName');
    Assert(@SyncFindDbByName <> nil);
    @SyncFindDbByTypeCreator := GetProcAddress(DLLHandle,'SyncFindDbByTypeCreator');
    Assert(@SyncFindDbByTypeCreator <> nil);
    @SyncReadOpenDbInfo := GetProcAddress(DLLHandle,'SyncReadOpenDbInfo');
    Assert(@SyncReadOpenDbInfo <> nil);
    @SyncCloseDBEx := GetProcAddress(DLLHandle,'SyncCloseDBEx');
    Assert(@SyncCloseDBEx <> nil);
    @SyncLoopBackTest := GetProcAddress(DLLHandle,'SyncLoopBackTest');
    Assert(@SyncLoopBackTest <> nil);
  end;
end;


initialization
  DLLLoaded := False;
  LoadDLL;

  if (not DllLoaded) then
  begin
    MessageDlg('pfgSyncMgr: Unable to load sync20.dll',mtError,[mbOk],0);
    Abort;
  end;

finalization
  if DLLLoaded then FreeLibrary(DLLHandle);
end.
