unit T_cmdln;

interface
uses
   Classes,
   // KERNEL
   Elements,
   Tools;

const
  { Command Line }
   cl_FindElementByName  = 'FINDELEMENTBYNAME';
   cl_FindElementByClass = 'FINDELEMENTBYCLASS';
   cl_Element            = 'ELEMENT';
   cl_CreateElement      = 'CREATEELEMENT';
   cl_CreateTool         = 'CREATETOOL';
   cl_Create             = 'CREATE';
   cl_Free               = 'FREE';
   cl_FreeAll            = 'FREEALL';
   cl_DoIt               = 'DOIT';
   cl_AddElement         = 'ADDELEMENT';
   cl_FindOwnerByClass   = 'FINDOWNERBYCLASS';
   cl_FindOwnerByName    = 'FINDOWNERBYNAME';
   cl_ActiveElement      = 'ACTIVEELEMENT';
   cl_World              = 'WORLD';
   cl_Self               = 'SELF';
   cl_Input              = 'INPUT';
   cl_ObjectModel        = 'OBJECTMODEL';
   cl_MainTool           = 'MAINTOOL';
   cl_Close              = 'CLOSE';
   cl_Call               = 'CALL';
   cl_Var                = 'VAR';
   cl_IF                 = 'IF';
   cl_THEN               = 'THEN';
   cl_ELSE               = 'ELSE';
   cl_END                = 'END';
   cl_EXIT               = 'EXIT';


type


TPropertyInfo = record
   Element: TElement;
   List: boolean;
   Constant: boolean;
   Variable: boolean;
   PropertyName: string;
end;

TVar = class(TOwnerElement)
private
   FElementClass: TElementClass;
   FElement: TElement;
   procedure SetElement(aValue: TElement);
public
   property ElementClass: TElementClass read FElementClass write FElementClass;
published
   property Element: TElement read FElement write SetElement;
end;


TCommandLine = class(TTool)
private
   FLine: string;
   FLines: TStringList;
   FActualDirectory: string;
   FFileName: string;
   FResult: TStringList;
   FAES: boolean;
   FDeleteResult: boolean;
   FActiveElement: TElement;
   FLineNumber: longInt;
   FVars: TOwnerEntity;
   FActualVar: TVar;
   FByDDE: boolean;
   FParamList: TstringList;
   FComment: boolean;
   FSmartMode: boolean;
   function  GetLine: string;
   function  GetLines: TStringList;
   function  GetActualDirectory: string;
   function  GetFileName: string;
   function  GetActiveElement: TElement;
   function  GetResultLine: string;
   function  GetResultLines: TStringList;
   function  GetVars: TOwnerEntity;
   function  GetParamList: TStringList;
   procedure SetLine(aValue: string);
   procedure SetLines(aValue: TStringList);
   procedure SetActualDirectory(aValue: string);
   procedure SetFileName(aValue: string);
   procedure SetActiveElement(aValue: TElement);
   procedure SetNextLine(aValue: string);
   procedure SetResultLine(aValue: string);
   procedure SetResultLines(aValue: TStringList);
   procedure InterpretLine;
   procedure InterpretLines;
   procedure ShowError(aText: string; aNumber: longInt);
protected
   ActiveElementSetting: boolean;
   LeftSide:  TPropertyInfo;
   RightSide: TPropertyInfo;
   function  NumberToPropertyName(aNumber: longInt): string; override;
   function  Evaluate(var Info: TPropertyInfo): string;
   procedure SetInput(aInput: TElement); override;
   procedure SetNewVar(aName: string; aClass: TElementClass);
   procedure EvaluateResult(var Info: TPropertyInfo);
   procedure ClearInfo(var Info: TPropertyInfo);
   property  ParamList: TStringList read GetParamList;
public
   constructor Create; override;
   destructor Destroy; override;
   procedure DoIt; override;

   procedure HandleMessage(var aMessage: TOrbisMessage; var Handled: boolean); override;
   procedure on_tm_CMDLNextLine; virtual;

   procedure Reset;
   procedure CompileLine(S: string; var Info: TPropertyInfo);
   function  OnlyCreateElement(const aClass, aName: string): TElement;
   function  IsVar(aName: string): boolean;
   function  GetErrorPosition: string;

   property DeleteResult: boolean read FDeleteResult write FDeleteResult;
   property Lines: TStringList read GetLines write SetLines;
   property ActiveElement: TElement read GetActiveElement write SetActiveElement;
   property ResultLine: string read GetResultLine write SetResultLine;
   property ResultLines: TStringList read GetResultLines write SetResultLines;
   property LineNumber: longInt read FLineNumber write FLineNumber;
   property Comment: boolean read FComment write FComment;
published
   property Line: string read GetLine write SetLine;
   property NextLine: string write SetNextLine stored false;
   property ActualDirectory: string read GetActualDirectory write SetActualDirectory;
   property FileName: string read GetFileName write SetFileName;
   property Vars: TOwnerEntity read GetVars stored false;
   property ActualVar: TVar read FActualVar write FActualVar;
   property ByDDE: boolean read FByDDE write FByDDE;
   property SmartMode: boolean read FSmartMode write FSmartMode;
end;



implementation

uses
   Dialogs,
   Forms,
   SysUtils,
   // KERNEL
   e_Class,
   m_Orbis,
   u_Props,
   u_String,
   Worlds;


{ TVar }

procedure TVar.SetElement(aValue: TElement);
begin
   {!!!!!!!!!!!!!!!!!!!!!!!! typov kontrola !!!!!!!!!!!!!!!!!!!!!!}
   FElement := aValue;
end;


{ TCommandLine }

constructor TCommandLine.Create;
begin
   inherited Create;
   ActiveElement := World;
   DeleteResult := true;
end;

destructor TCommandLine.Destroy;
begin
   try
      Lines.Free;
   except
   end;
   try
      if FParamList <> nil then FParamList.Free;
   except
   end;
   try
      if DeleteResult then ResultLines.Free;
   except
   end;
   inherited Destroy;
end;

function TCommandLine.GetParamList: TStringList;
begin
   if FParamList = nil then FParamList := TStringList.Create;
   result := FParamList;
end;

function TCommandLine.OnlyCreateElement(const aClass, aName: string): TElement;
var
   C: TElementClass;
   E: TElement;
begin
   result := nil;
   C := TElementClass(GetOrbisClass(aClass));
   if C = nil then
   begin
     ShowMessage('Neregistrovan tda ' + aClass + GetErrorPosition);
     exit;
   end;
   E := C.Create;
   E.Name := aName;
   result := E;
end;


function  TCommandLine.GetVars: TOwnerEntity;
begin
   if FVars = nil then
   begin
      FVars := TOwnerEntity.Create;
      FVars.Name := 'VAR:';
      AddElement(FVars);
   end;
   result := FVars;
end;

procedure TCommandLine.SetInput(aInput: TElement);
begin
   inherited SetInput(aInput);
   if aInput = nil then exit;
   ActiveElement := aInput;
end;


procedure TCommandLine.SetNewVar(aName: string; aClass: TElementClass);
var
   V: TVar;
begin
   V := TVar.Create;
   V.Name := UpperCase(aName);
   V.ElementClass := aClass;
   Vars.AddElement(V);
   ActualVar := V;
end;

procedure TCommandLine.ShowError(aText: string; aNumber: longInt);
begin
   if not ByDDE then ShowMessage(aText)
                else {F_StartForm.DDEServerItem.Text}ResultLine := 'ERROR '+ IntToStr(aNumber)+': '+ aText;
end;


function  TCommandLine.IsVar(aName: string): boolean;
var
   V: TVar;
begin
   result := false;
   V := TVar(Vars.FindElementName(UpperCase(aName)));
   if V <> nil then
   begin
      ActualVar := V;
      result := true;
   end;
end;


function  TCommandLine.GetErrorPosition: string;
begin
  result:= #13 + #10 + FileName + ' ..' + IntToStr(LineNumber) +
           #13 + #10 + Line;
end;

procedure TCommandLine.DoIt;
var
   S: string;
   FNPath: string;
begin
   try
      LineNumber := 0;
      if Line <> '' then InterpretLine
      else if Lines.Count > 0 then InterpretLines
      else if FileName <> '' then
      begin
         S := FileName;
         FNPath:= ExtractFilePath(FileName);
         (*
         if (ActualDirectory = '') and (FNPath = '') then
            ActualDirectory:= TOldMainTool(World.MainTool).OptionPropString('WorkingDirectory');
         *)
         S := ActualDirectory + S;
         if not FileExists(S) then
         begin
            ShowError('Skriptov soubor '+ S + ' nenalezen !', 1);
            exit;
         end;
         Lines.LoadFromFile(S);
         InterpretLines;
      end;
   except
      on EX: Exception  do
         ShowError('Soubor ' + FileName + ', dek..'+IntToStr(LineNumber+1) + ':' + #13 + #10 +
                   '"' + Line + '"' + #13 + #13 + #10 +
                   'Chyba: '+EX.Message, 2);
   end;
end;


procedure TCommandLine.Reset;
begin
   ActiveElementSetting := false;
   ResultLines.Clear;
   with LeftSide do
   begin
      Element := nil;
      List := false;
      Constant := false;
      Variable := false;
      PropertyName := '';
   end;
   with RightSide do
   begin
      Element := nil;
      List := false;
      Constant := false;
      Variable := false;
      PropertyName := '';
   end;
end;


procedure TCommandLine.CompileLine(S: string; var Info: TPropertyInfo);
var
   i: longInt;
   Delimiters : TDelimiters;
   Delimiter: char;
   Position: longInt;
   SS: string;
   Finish: boolean;
   aClass: TElementClass;

   procedure EX(ms: string);
   begin
      raise Exception.Create(ms);
   end;

   procedure EntityError(Pos: longInt);
   begin
      Ex('Objekt na pozici '+ IntToStr(Pos)+' '+'nen Entita');
   end;

   procedure SyntaxError(Pos: longInt; Par: string);
   begin
      Ex('Syntaktick chyba - na pozici '+ IntToStr(Pos)+' '+'oekvm '+ Par);
   end;

   procedure EvaluateMethod;
   var
      aParam, Param1, Param2: string;
      aClass: TElementClass;
      Tool: TTool;
      Pom: longInt;
      L,P: string;
      ii: longInt;
      aInfo: TPropertyInfo;
   begin
      Pom := Position - 1;
      Delimiters := ['(',')',','];
      ParamList.Clear;
      repeat
         aParam := ReadToDelimiterWithoutWS(S,  Delimiters, Position, Delimiter, Position);
         if aParam <> '' then ParamList.Add(aParam);
      until (Delimiter = ')') or (Delimiter = #0);

      if ParamList.Count > 1 then
      begin
         Param1 := ParamList.Strings[0];
         Param2 := ParamList.Strings[1];
      end
      else
      if ParamList.Count > 0 then
      begin
         Param1 := ParamList.Strings[0];
         Param2 := Param1;
      end;

      SS := UpperCase(SS);
      if SS = cl_Close then
      begin
         PostMsg(tm_QuitTask, World.MainTool, nil, DontKill);
      end
      else
      if SS = cl_Free then
      begin
         Info.Element.Free;
         Info.Element := nil;
      end
      else
      if SS = cl_FreeAll then
      begin
         if not (Info.Element is TEntity) then EntityError(Pom);
         TEntity(Info.Element).FreeAll;
      end
      else
      if SS = cl_DoIt then
      begin
         if not (Info.Element is TTool) then Ex('Objekt na pozici '+ IntToStr(Pom)+' '+'nen Tool');
         TTool(Info.Element).DoIt;
      end
      else
      if SS = cl_FindElementByName then
      begin
         if not (Info.Element is TEntity) then EntityError(Pom);
         Info.Element := TEntity(Info.Element).FindElementName(Param1);
      end
      else
      if SS = cl_FindElementByClass then
      begin
         if not (Info.Element is TEntity) then EntityError(Pom);
         Info.Element := TEntity(Info.Element).FindElementClass(FindOrbisClass(CheckClassName(Param1)));
      end
      else
      if SS = cl_Create then
      begin
        aClass:= FindOrbisClass(CheckClassName(Param1));
        Info.Element := OnlyCreateElement(CheckClassName(Param1), Param2);
      end
      else
      if SS = cl_CreateElement then
      begin
        aClass:= FindOrbisClass(CheckClassName(Param1));
         if not (Info.Element is TEntity) then EntityError(Pom);
         TEntity(Info.Element).CreateElement(CheckClassName(Param1), Param2);
      end
      else
      if SS = cl_CreateTool then
      begin
        aClass:= FindOrbisClass(CheckClassName(Param1));
         if not (Info.Element is TTechnology) then Ex('Objekt na pozici '+ IntToStr(Pom)+' '+'nen Technology');
         TTechnology(Info.Element).CreateTool(CheckClassName(Param1),Param2);
      end
      else
      if SS = cl_Call then
      begin
         Tool := TCommandLine.Create;
         TCommandLine(Tool).FileName := GetAbsolutePath(Param1);
         TCommandLine(Tool).ActiveElement := ActiveElement;
         Tool.DoIt;
         Tool.Free;
      end
      else
      begin
         aClass := TElementClass(GetOrbisClass(CheckClassName(SS)));
         if aClass = nil then Ex('chybn funkce: '+ SS);
         Tool := TTool(aClass.Create);
         Tool.Input := Info.Element;
         if Tool.Input = nil then Tool.Input := ActiveElement;
         for ii := 0 to ParamList.Count-1 do
         begin
            if (ParamList.Strings[ii][1] in ['''','"']) or
               (not StringAsTwoParams(ParamList.Strings[ii], ':', L, P)) then
            begin
               ClearInfo(aInfo);
               CompileLine(ParamList.Strings[ii], aInfo);
               Tool.PropertyNumber[ii] := Evaluate(aInfo);
            end
            else
            begin
               if (P <> '') and (P[1] = ':') then Tool.PropertyNumber[ii] := ParamList.Strings[ii]
               else
               begin
                  ClearInfo(aInfo);
                  CompileLine(P, aInfo);
                  SetPropertyStringByName(L, Evaluate(aInfo));
               end;
            end;
         end;
         Tool.DoIt;
         Info.Element := Tool.Output;
         if Tool is TResultTool then
         begin
            Info.PropertyName := TResultTool(Tool).Result;
            Info.Constant := true;
         end;
         Tool.Free;
      end;
      if not (Delimiter = ')') then SyntaxError(Position -1, ' ")"');
      Pom := Position - 1;
      Delimiters := ['.','(',')',',','''','"'];
      SS := ReadToDelimiterWithoutWS(S,  Delimiters, Position, Delimiter, Position);
      if Delimiter = #0 then
      begin
         Finish := true;
         exit;
      end;
      if not (Delimiter = '.') then SyntaxError(Pom, 'oekvm "." nebo nic');
   end;


   procedure EvaluatePropertyAsObject;
   var
      E: TElement;
      I: longInt;
   begin
      if UpperCase(SS) = cl_Element then Info.List := true
      else
      if UpperCase(SS) = cl_World then Info.Element := World
      else
      if UpperCase(SS) = cl_ActiveElement then Info.Element := ActiveElement
      else
      if UpperCase(SS) = cl_Self then Info.Element := Self
      else
      if IsVar(SS) then Info.Element := ActualVar.Element
      else
      begin
         E := TElement(GetObjectPropByName(Info.Element,SS));
         if E = nil then
         begin
           I:= StrToLongInt(Info.Element.GetPropertyStringByName(SS));
           E:= TElement(I);
           if (E<>nil) then
           begin
             if not (TObject(E) is TObject) then
                Ex('Nemohu nalzt objekt '+ SS);
           end
           else
           begin
             if not (Info.Element is TEntity) then
                Ex('Nemohu nalzt element '+ SS);

             E := TEntity(Info.Element).FindElementName(SS);
             if E = nil then
                Ex('Nemohu nalzt element '+ SS)
           end
         end;
         Info.Element := E;
      end;
   end;


begin
   Info.Element := ActiveElement;
   Position := 1;
   Finish := false;
   Delimiters := ['.','(',')',',','''','"',':'];

   while true do
   begin
      if Finish then exit;
      if info.Element = nil then exit;
      SS := ReadToDelimiterWithoutWS(S,  Delimiters, Position, Delimiter, Position);
      case Delimiter of
         #0: { properta - konec }
         begin
            if UpperCase(SS)=cl_World then Info.Element := World
            else
            if UpperCase(SS)=cl_ActiveElement then Info.Element := ActiveElement
            else
            if UpperCase(SS)=cl_Var then exit
            else
            if IsVar(SS) then
            begin
               Info.Element := ActualVar.Element;
               Info.Variable := true;
               exit;
            end
            else
            Info.PropertyName := SS;
            exit;
         end;
         '.': { properta - objekt }
         begin
            EvaluatePropertyAsObject;
         end;
         '(':
         begin
            EvaluateMethod;
         end;
         '''': { konstanta }
         begin
            Delimiters := [''''];
            SS := ReadToDelimiterWithoutWS(S,  Delimiters, Position, Delimiter, Position);
            Info.PropertyName := SS;
            Info.Constant := true;
            exit;
         end;
         '"': { konstanta }
         begin
            Delimiters := ['"'];
            SS := ReadToDelimiter{ O.S.25.3.98 WithoutWS}(S,  Delimiters, Position, Delimiter, Position);
            Info.PropertyName := SS;
            Info.Constant := true;
            exit;
         end;
         ':': { promnn }
         begin
            Delimiters := [';'];
            S := ReadToDelimiterWithoutWS(S,  Delimiters, Position, Delimiter, Position);
            aClass := GetOrbisClass(S);
            if aClass = nil then Ex('Neexistujc tda: '+ S);
            if not IsVar(SS) then
               SetNewVar(SS, aClass);
            exit;
         end;
      end;
   end;
end;


function  TCommandLine.Evaluate(var Info: TPropertyInfo): string;
var
   El: TElement;
begin
   result := '';
   if Info.Constant then Result := Info.PropertyName
   else
   begin
      if Info.Element = nil then exit;
      Result := Info.Element.GetPropertyStringByName(Info.PropertyName);
      if Result = '' then
      begin
         if not(Info.Element is TEntity) then exit;
         El := TEntity(Info.Element).FindElementName(Info.PropertyName);
         Result := IntToStr(longInt(El));
      end;
   end;
end;


procedure TCommandLine.ClearInfo(var Info: TPropertyInfo);
begin
   Info.Element      := nil;
   Info.List         := false;
   Info.Constant     := false;
   Info.Variable     := false;
   Info.PropertyName := '';
end;


procedure TCommandLine.EvaluateResult(var Info: TPropertyInfo);
var
   i: longInt;
   El: TElement;
begin
   ResultLines.Clear;
   if Info.List then
   begin
      if not(Info.Element is TEntity) then exit;
      for i := 0 to TEntity(Info.Element).Count - 1 do
      begin
         if Info.Element = nil then exit;
         ResultLines.Add(TEntity(Info.Element).Elements[i].GetPropertyStringByName(Info.PropertyName));
      end;
   end
   else
   begin
      ResultLine := Evaluate(info);
   end;
end;


function  TCommandLine.NumberToPropertyName(aNumber: longInt): string;
begin
   case aNumber of
      0: result := 'FileName';
      1: result := 'ActualDirectory'
   end;
end;


function  TCommandLine.GetLine: string;
begin
   result := FLine;
end;


function  TCommandLine.GetLines: TStringList;
begin
   if FLines = nil then FLines := TStringList.Create;
   result := FLines;
end;


function  TCommandLine.GetActualDirectory: string;
begin
   result := FActualDirectory;
end;

function  TCommandLine.GetFileName: string;
begin
   result := FFileName;
end;

function  TCommandLine.GetActiveElement: TElement;
begin
   result := FActiveElement;
end;

function  TCommandLine.GetResultLine: string;
begin
   if (FResult = nil) or (FResult.Count < 1) then result := ''
                                             else result := FResult.Strings[0];
end;

function  TCommandLine.GetResultLines: TStringList;
begin
   if FResult = nil then FResult := TStringList.Create;
   result := FResult;
end;

procedure TCommandLine.SetLine(aValue: string);
begin
   FLine := aValue;
end;

procedure TCommandLine.SetLines(aValue: TStringList);
begin
   Line := '';
   if FLines <> nil then
   begin
      FLines.Free;
      FLines := nil;
   end;
   FLines := aValue;
end;

procedure TCommandLine.SetActualDirectory(aValue: string);
begin
   FActualDirectory := CheckBackSlash(aValue);
end;

procedure TCommandLine.SetFileName(aValue: string);
begin
   Line := '';
   Lines.Clear;
   FFileName := aValue;
end;

procedure TCommandLine.SetActiveElement(aValue: TElement);
begin
   FActiveElement := aValue;
end;

procedure TCommandLine.SetNextLine(aValue: string);
begin
   Lines.Add(aValue);
end;

procedure TCommandLine.SetResultLine(aValue: string);
begin
   ResultLines.Add(aValue);
end;

procedure TCommandLine.SetResultLines(aValue: TStringList);
begin
   if FResult <> nil then
   begin
      FResult.Free;
      FResult := nil;
   end;
   FResult :=aValue;
end;

procedure TCommandLine.InterpretLine;
var
   i: longInt;
   List: TStringList;
   L, R: string;
   TwoSides: boolean;
   E: TElement;
   RightIsElement: boolean;
   LeftVar: TVar;
begin
    Reset;
    RightIsElement := false;
    TwoSides := StringAsParameter(Line, L, R);
    { Zpracovn lev strany }
    { Nastaven Aktivnho elementu }
    if (L <> '') then
    begin
       if L[1] = ';' then exit;
       if L[1] = '*' then
       begin
          Comment := not Comment;
       end;
    end;
    if Comment then exit;
    if UpperCase(L) = cl_ActiveElement then
    begin
       ActiveElementSetting := true;
       if not TwoSides then
       begin
           ShowError('Chyba .. nastaven aktivnho elementu: '+ Line, 3);
           exit;
       end;
    end
    else
    begin
       CompileLine(L, LeftSide);
       if LeftSide.Variable then LeftVar := ActualVar;
    end;

    { Zpracovn prav strany }
    if TwoSides then
    begin
       CompileLine(R, RightSide);
    end;

    { Evaluate }

    if not twoSides then
    begin
       if LeftSide.Variable then LeftSide.Element := LeftVar.Element;
       EvaluateResult(LeftSide);
       exit;
    end;

    if ActiveElementSetting then
    begin
       if RightSide.List then
       begin
          ShowError('Chyba .. jako aktivn element nelze piadit seznam (klov slovo ELEMENT nahrate nzvem property): '
                      + Line, 4);
          exit;
       end;
       if RightSide.PropertyName = '' then E := RightSide.Element
       else
       begin
          E := TElement(GetObjectPropByName(TPersistent(RightSide.Element), RightSide.PropertyName));
          if E = nil then
          begin
             if (RightSide.Element is TEntity) then
                E := TEntity(RightSide.Element).FindElementName(RightSide.PropertyName);
             if E = nil then
             begin
                ShowError('Chyba .. Nemohu nalzt objekt '+ RightSide.PropertyName, 5);
                exit;
             end;
          end;
       end;
       ActiveElement := E;
       exit;
    end;

    if (RightSide.PropertyName = '') and not RightSide.Constant then
    begin
       E := RightSide.Element;
       if E = nil then
       begin
          ShowError('vsledn objekt na prav stran vrazu je chybn '+ Line, 6);
          exit;
       end;
       RightIsElement := true;
    end
    else
    begin
       EvaluateResult(RightSide);
       if RightSide.List then
       begin
          List := TStringList.Create;
          List.Assign(ResultLines);
       end;
    end;

    if LeftSide.List then
    begin
       if (LeftSide.Element = nil) or
          not (LeftSide.Element is TEntity) then
       begin
          ShowError('Chyba .. Zklad seznamu na lev stran nen entita (nebo je objekt pokozen) : '+ Line, 7);
          exit;
       end;
       for i := 0 to TEntity(LeftSide.Element).Count - 1 do
       begin
          if RightSide.List then SetObjectPropByName(TEntity(LeftSide.Element).Elements[i], LeftSide.PropertyName, List)
          else
          if RightIsElement then
             SetObjectPropByName(TEntity(LeftSide.Element).Elements[i], LeftSide.PropertyName, RightSide.Element)
          else
          begin
            if TObject(TEntity(LeftSide.Element).Elements[i]) is TElement then
               TEntity(LeftSide.Element).Elements[i].SetPropertyStringByName(LeftSide.PropertyName, ResultLine)
            else
              SetPropStringByName(TEntity(LeftSide.Element).Elements[i], LeftSide.PropertyName, ResultLine)
          end
       end;
       exit;
    end
    else
    begin
       if LeftSide.Variable then
       begin
          if not RightIsElement then
          begin
             try
                LeftVar.Element := TElement(StrToInt(ResultLine));
             except
                ShowError('Chyba .. objekt na prav stran vrazu nen element: '+ Line, 8)
             end;
          end
          else LeftVar.Element := RightSide.Element;
          exit;
       end;
       if (LeftSide.Element = nil) or (LeftSide.PropertyName = '') then
       begin
          ShowError('Chyba .. Chybn objekt nebo properta na lev stran vrazu : '+ Line, 9);
          exit;
       end;
       if RightSide.List then SetObjectPropByName(LeftSide.Element, LeftSide.PropertyName, List)
       else
       if RightIsElement then
          SetObjectPropByName(LeftSide.Element, LeftSide.PropertyName, RightSide.Element)
       else
       begin
         if TObject(LeftSide.Element) is TElement then
            LeftSide.Element.SetPropertyStringByName(LeftSide.PropertyName, ResultLine)
         else
           SetPropStringByName(LeftSide.Element, LeftSide.PropertyName, ResultLine)
       end
    end;
end;


procedure TCommandLine.InterpretLines;
var
   i: longInt;
begin
   if SmartMode then
   begin
      Busy := true;
      LineNumber := 0;
      PostMsg(tm_CMDLNextLine, Self, nil, DontKill);
   end
   else
   begin
      for i := 0 to Lines.Count -1 do
      begin
         LineNumber := i;
         Line := Lines.Strings[i];
         if Line = '' then continue;
         InterpretLine;
         Application.ProcessMessages;
      end;
      LineNumber := 0;
      Line := '';
   end;
end;


procedure TCommandLine.on_tm_CMDLNextLine;
var
   i: longInt;
begin
   while (LineNumber < Lines.Count) and (Lines.Strings[LineNumber] = '')   do Inc(FLineNumber);
   { End of Work ?!?}
   if LineNumber >= Lines.Count then
   begin
      LineNumber := 0;
      Line := '';
      Busy := false;
      PostMsg(tm_Done, World.MainTool, nil, DontKill);
      exit;
   end;
   Line := Lines.Strings[LineNumber];
   try
      InterpretLine;
   except
      on EX: Exception  do
      begin
         ShowError('Soubor ' + FileName + ', dek..'+IntToStr(LineNumber+1) + ':' + #13 + #10 +
                   '"' + Line + '"' + #13 + #13 + #10 +
                   'Chyba: '+EX.Message, 2);
         LineNumber :=  Lines.Count;
      end;
   end;
   Inc(FLineNumber);
   PostMsg(tm_CMDLNextLine, Self, nil, DontKill);
end;


procedure TCommandLine.HandleMessage(var aMessage: TOrbisMessage; var Handled: boolean);
begin
   Handled := false;
   inherited HandleMessage(aMessage, Handled);
   if Handled then exit;
   case aMessage.Message of
      tm_CMDLNextLine : on_tm_CMDLNextLine;
   end;
end;



BEGIN
   RegisterOrbisClass(TCommandLine);
END.
