unit T_cmdln;

interface
uses
   Classes,
   // KERNEL
   Elements,
   e_Type,
   Tools;

const
  { Command Line }
   cl_FindElementByName  = 'FINDELEMENTBYNAME';
   cl_FindElementByClass = 'FINDELEMENTBYCLASS';
   cl_Element            = 'ELEMENT';
   cl_CreateElement      = 'CREATEELEMENT';
   cl_CreateTool         = 'CREATETOOL';
   cl_Create             = 'CREATE';
   cl_Free               = 'FREE';
   cl_FreeAll            = 'FREEALL';
   cl_DoIt               = 'DOIT';
   cl_AddElement         = 'ADDELEMENT';
   cl_FindOwnerByClass   = 'FINDOWNERBYCLASS';
   cl_FindOwnerByName    = 'FINDOWNERBYNAME';
   cl_ActiveElement      = 'ACTIVEELEMENT';
   cl_World              = 'WORLD';
   cl_Self               = 'SELF';
   cl_Input              = 'INPUT';
   cl_ObjectModel        = 'OBJECTMODEL';
   cl_MainTool           = 'MAINTOOL';
   cl_Close              = 'CLOSE';
   cl_Call               = 'CALL';
   cl_Var                = 'VAR';
   cl_IF                 = 'IF';
   cl_FOR                = 'FOR';
   cl_WHILE              = 'WHILE';
   cl_REPEAT             = 'REPEAT';
   cl_UNTIL              = 'UNTIL';
   cl_IN                 = 'IN';
   cl_OUT                = 'OUT';
   cl_THEN               = 'THEN';
   cl_ELSE               = 'ELSE';
   cl_END                = 'END';
   cl_EXIT               = 'EXIT';
   cl_BREAK              = 'BREAK';
   cl_CONTINUE           = 'CONTINUE';
   cl_Integer            = 'INTEGER';
   cl_String             = 'STRING';
   cl_Float              = 'FLOAT';
   cl_Char               = 'CHAR';
   cl_Boolean            = 'BOOLEAN';


type


TPropertyInfo = record
   Element: TElement;
   List: boolean;
   IsConstant: boolean;
   IsVariable: boolean;
   Variable: TTypeElement;
   PropertyName: string;
end;

TElementVar = class(TPointerElement)
private
   FElementClass: TElementClass;
public
   property ElementClass: TElementClass read FElementClass write FElementClass;
end;

TFlowInfoType = (fi_NULL,fi_IF,fi_FOR,fi_While,fi_Repeat);
TFlowInfoStatus = (fi_IF_RUN,fi_IF_SKIP,fi_ELSE_RUN,fi_ELSE_SKIP,fi_FOR_Break,fi_FOR_RUN,fi_Shadow);

TFlowInfo = class(TOwnerElement)
  private
    FStatus: TFlowInfoStatus;
    FFlowType: TFlowInfoType;
    FActualIndex: integer;
    FFirstLine: integer;
    FEntity: TEntity;
    FVariable: TTypeElement;
    FCondition: string;
    F_Element: TElement;
    F_In: boolean;
    procedure SetFlowType(const Value: TFlowInfoType);
    procedure SetStatus(const Value: TFlowInfoStatus);
    procedure SetActualIndex(const Value: integer);
    procedure SetEntity(const Value: TEntity);
    procedure SetFirstLine(const Value: integer);
    procedure SetVariable(const Value: TTypeElement);
    procedure SetCondition(const Value: string);
    procedure Set_Element(const Value: TElement);
    procedure Set_In(const Value: boolean);
public
   function NewFlowIsShadow: boolean;
   property Variable: TTypeElement read FVariable write SetVariable;
   property Entity: TEntity read FEntity write SetEntity;
   property _Element: TElement read F_Element write Set_Element;
   property ActualIndex: integer read FActualIndex write SetActualIndex;
   property FirstLine: integer read FFirstLine write SetFirstLine;
   property Condition: string read FCondition write SetCondition;
   property _In: boolean read F_In write Set_In;
published
   property FlowType: TFlowInfoType read FFlowType write SetFlowType;
   property Status: TFlowInfoStatus read FStatus write SetStatus;
end;

TCommandLine = class(TTool)
private
   FLine: string;
   FLines: TStringList;
   FActualDirectory: string;
   FFileName: string;
   FResult: TStringList;
   FAES: boolean;
   FDeleteResult: boolean;
   FActiveElement: TElement;
   FLineNumber: longInt;
   FVars: TOwnerEntity;
   FFlowInfos: TOwnerEntity;
   FActualVar: TTypeElement;
   FByDDE: boolean;
   FParamList: TstringList;
   FComment: boolean;
   FSmartMode: boolean;
   function  GetLine: string;
   function  GetLines: TStringList;
   function  GetActualDirectory: string;
   function  GetFileName: string;
   function  GetActiveElement: TElement;
   function  GetResultLine: string;
   function  GetResultLines: TStringList;
   function  GetVars: TOwnerEntity;
   function  GetParamList: TStringList;
   procedure SetLine(aValue: string);
   procedure SetLines(aValue: TStringList);
   procedure SetActualDirectory(aValue: string);
   procedure SetFileName(aValue: string);
   procedure SetActiveElement(aValue: TElement);
   procedure SetNextLine(aValue: string);
   procedure SetResultLine(aValue: string);
   procedure SetResultLines(aValue: TStringList);
   procedure InterpretLine;
   procedure InterpretLines;
   procedure ShowError(aText: string; aNumber: longInt);
    function GetFlowInfos: TOwnerEntity;
    function GetLastFlowInfo: TFlowInfo;
protected
   ActiveElementSetting: boolean;
   LeftSide:  TPropertyInfo;
   RightSide: TPropertyInfo;
   function  Evaluate(var Info: TPropertyInfo): string;
   procedure SetInput(aInput: TElement); override;
   procedure SetNewVar(aName: string; aTyp: TType; aClass: TElementClass);
   procedure EvaluateResult(var Info: TPropertyInfo);
   procedure ClearInfo(var Info: TPropertyInfo);
   property  ParamList: TStringList read GetParamList;
public
   constructor Create; override;
   destructor Destroy; override;
   procedure DoIt; override;
   function  NumberToPropertyName(aNumber: longInt): string; override;

   procedure HandleMessage(var aMessage: TOrbisMessage; var Handled: boolean); override;
   procedure on_tm_CMDLNextLine; virtual;

   procedure Reset;
   procedure CompileLine(S: string; var Info: TPropertyInfo);
   function  OnlyCreateElement(const aClass, aName: string): TElement;
   function  IsVar(aName: string): boolean;
   function  GetErrorPosition: string;
   function  CheckFlow(var Row: integer): boolean;
   function  Process_IF(const S: string): boolean;
   function  GetFirstWord(var Lasti: integer): string;
   function  GetNextWord(Firsti: integer; var Lasti: integer): string;

   property DeleteResult: boolean read FDeleteResult write FDeleteResult;
   property Lines: TStringList read GetLines write SetLines;
   property ActiveElement: TElement read GetActiveElement write SetActiveElement;
   property ResultLine: string read GetResultLine write SetResultLine;
   property ResultLines: TStringList read GetResultLines write SetResultLines;
   property LineNumber: longInt read FLineNumber write FLineNumber;
   property Comment: boolean read FComment write FComment;
published
   property Line: string read GetLine write SetLine;
   property NextLine: string write SetNextLine stored false;
   property ActualDirectory: string read GetActualDirectory write SetActualDirectory;
   property FileName: string read GetFileName write SetFileName;
   property Vars: TOwnerEntity read GetVars stored false;
   property FlowInfos: TOwnerEntity read GetFlowInfos stored false;
   property ActualVar: TTypeElement read FActualVar write FActualVar;
   property LastFlowInfo: TFlowInfo read GetLastFlowInfo;
   property ByDDE: boolean read FByDDE write FByDDE;
   property SmartMode: boolean read FSmartMode write FSmartMode;
end;



implementation

uses
   Dialogs,
   Forms,
   SysUtils,
   Creacond,
   // KERNEL
   e_Class,
   m_Orbis,
   t_Oper,
   u_Props,
   u_String,
   Worlds;




{ TCommandLine }

constructor TCommandLine.Create;
begin
   inherited Create;
   ActiveElement := World;
   DeleteResult := true;
end;

destructor TCommandLine.Destroy;
begin
   try
      Lines.Free;
   except
   end;
   try
      if FParamList <> nil then FParamList.Free;
   except
   end;
   try
      if DeleteResult then ResultLines.Free;
   except
   end;
   inherited Destroy;
end;

function TCommandLine.GetParamList: TStringList;
begin
   if FParamList = nil then FParamList := TStringList.Create;
   result := FParamList;
end;

function TCommandLine.OnlyCreateElement(const aClass, aName: string): TElement;
var
   C: TElementClass;
   E: TElement;
begin
   result := nil;
   C := TElementClass(GetOrbisClass(aClass));
   if C = nil then
   begin
     ShowMessage('Neregistrovan tda ' + aClass + GetErrorPosition);
     exit;
   end;
   E := C.Create;
   E.Name := aName;
   result := E;
end;


function  TCommandLine.GetVars: TOwnerEntity;
begin
   if FVars = nil then
   begin
      FVars := TOwnerEntity.Create;
      FVars.Name := 'VAR:';
      AddElement(FVars);
   end;
   result := FVars;
end;

procedure TCommandLine.SetInput(aInput: TElement);
begin
   inherited SetInput(aInput);
   if aInput = nil then exit;
   ActiveElement := aInput;
end;


procedure TCommandLine.SetNewVar(aName: string; aTyp: TType; aClass: TElementClass);
var
   V: TTypeElement;
begin
   case aTyp of
      _Boolean : V := TBooleanElement.Create;
      _Integer : V := TIntegerElement.Create;
      _Float   : V := TFloatElement.Create;
      _String  : V := TStringElement.Create;
      _Char    : V := TCharElement.Create;
      _Pointer :
      begin
         V := TElementVar.Create;
         TElementVar(V).ElementClass := aClass;
      end;
   end;
   V.Name := UpperCase(aName);
   Vars.AddElement(V);
   ActualVar := V;
end;

procedure TCommandLine.ShowError(aText: string; aNumber: longInt);
begin
   if not ByDDE then ShowMessage(aText)
                else {F_StartForm.DDEServerItem.Text}ResultLine := 'ERROR '+ IntToStr(aNumber)+': '+ aText;
end;


function  TCommandLine.IsVar(aName: string): boolean;
var
   V: TTypeELement;
begin
   result := false;
   V := TTypeElement(Vars.FindElementName(UpperCase(aName)));
   if V <> nil then
   begin
      ActualVar := V;
      result := true;
   end;
end;


function  TCommandLine.GetErrorPosition: string;
begin
  result:= #13 + #10 + FileName + ' ..' + IntToStr(LineNumber) +
           #13 + #10 + Line;
end;

procedure TCommandLine.DoIt;
var
   S: string;
   FNPath: string;
begin
   try
      LineNumber := 0;
      if Line <> '' then InterpretLine
      else if Lines.Count > 0 then InterpretLines
      else if FileName <> '' then
      begin
         S := FileName;
         FNPath:= ExtractFilePath(FileName);
         (*
         if (ActualDirectory = '') and (FNPath = '') then
            ActualDirectory:= TOldMainTool(World.MainTool).OptionPropString('WorkingDirectory');
         *)
         S := ActualDirectory + S;
         if not FileExists(S) then
         begin
            ShowError('Skriptov soubor '+ S + ' nenalezen !', 1);
            exit;
         end;
         Lines.LoadFromFile(S);
         InterpretLines;
      end;
   except
      on EX: Exception  do
         ShowError('Soubor ' + FileName + ', dek..'+IntToStr(LineNumber+1) + ':' + #13 + #10 +
                   '"' + Line + '"' + #13 + #13 + #10 +
                   'Chyba: '+EX.Message, 2);
   end;
end;


procedure TCommandLine.Reset;
begin
   ActiveElementSetting := false;
   ResultLines.Clear;
   with LeftSide do
   begin
      Element := nil;
      List := false;
      IsConstant := false;
      IsVariable := false;
      Variable := nil;
      PropertyName := '';
   end;
   with RightSide do
   begin
      Element := nil;
      List := false;
      IsConstant := false;
      IsVariable := false;
      Variable := nil;
      PropertyName := '';
   end;
end;


procedure TCommandLine.CompileLine(S: string; var Info: TPropertyInfo);
var
   i: longInt;
   Delimiters : TDelimiters;
   Delimiter: char;
   Position: longInt;
   SS: string;
   Finish: boolean;
   aClass: TElementClass;
   FirstWord: boolean;

   procedure EX(ms: string);
   begin
      raise Exception.Create(ms);
   end;

   procedure EntityError(aPos: longInt);
   begin
      Ex('Objekt na pozici '+ IntToStr(aPos)+' '+'nen Entita');
   end;

   procedure SyntaxError(aPos: longInt; Par: string);
   begin
      Ex('Syntaktick chyba - na pozici '+ IntToStr(aPos)+' '+'oekvm '+ Par);
   end;

   procedure EvaluateExpression;
   var
      EC: TExpressionCreator;
      Op: TOperator;
      TE: TTypeElement;
   begin
      Info.IsConstant := true;
      Info.PropertyName := '0';
      Position := pos('}',S);
      if Position > 0 then
         SS := Copy(S,2,Position-2)
      else
         SS := Copy(S,2,Length(S)-1);
      EC := TExpressionCreator.Create;
      EC.Vars := Vars;
      EC.Command := SS;
      EC.DoIt;
      Op := TOperator(EC.Output);
      if Op <> nil then
      begin
         Op.ParamItem[0] := ActiveElement;
         Op.DoIt;
         if Op.Output <> nil then
         begin
            TE := TTypeElement(Op.Output);
            Info.PropertyName := TE.AsString;
         end;
         Op.CleanData;
         Op.Free;
      end;
      EC.Free;
   end;

   procedure EvaluateMethod;
   var
      aParam, Param1, Param2: string;
      aClass: TElementClass;
      Tool: TTool;
      Pom: longInt;
      L,P: string;
      ii: longInt;
      aInfo: TPropertyInfo;
   begin
      Pom := Position - 1;
      Delimiters := ['(',')',','];
      ParamList.Clear;
      repeat
         // opravit cteni parametru - nebere to carky v konstantach
         aParam := ReadToDelimiterWithoutWS(S,  Delimiters, Position, Delimiter, Position);
         if aParam <> '' then ParamList.Add(aParam);
      until (Delimiter = ')') or (Delimiter = #0);

      if ParamList.Count > 1 then
      begin
         Param1 := ParamList.Strings[0];
         Param2 := ParamList.Strings[1];
      end
      else
      if ParamList.Count > 0 then
      begin
         Param1 := ParamList.Strings[0];
         Param2 := Param1;
      end;

      SS := UpperCase(SS);
      if SS = cl_Close then
      begin
         PostMsg(tm_QuitTask, World.MainTool, nil, DontKill);
      end
      else
      if SS = cl_Free then
      begin
         Info.Element.Free;
         Info.Element := nil;
      end
      else
      if SS = cl_FreeAll then
      begin
         if not (Info.Element is TEntity) then EntityError(Pom);
         TEntity(Info.Element).FreeAll;
      end
      else
      if SS = cl_DoIt then
      begin
         if not (Info.Element is TTool) then Ex('Objekt na pozici '+ IntToStr(Pom)+' '+'nen Tool');
         TTool(Info.Element).DoIt;
      end
      else
      if SS = cl_FindElementByName then
      begin
         if not (Info.Element is TEntity) then EntityError(Pom);
         Info.Element := TEntity(Info.Element).FindElementName(Param1);
      end
      else
      if SS = cl_FindElementByClass then
      begin
         if not (Info.Element is TEntity) then EntityError(Pom);
         Info.Element := TEntity(Info.Element).FindElementClass(FindOrbisClass(CheckClassName(Param1)));
      end
      else
      if SS = cl_Create then
      begin
        aClass:= FindOrbisClass(CheckClassName(Param1));
        Info.Element := OnlyCreateElement(CheckClassName(Param1), Param2);
      end
      else
      if SS = cl_CreateElement then
      begin
        aClass:= FindOrbisClass(CheckClassName(Param1));
         if not (Info.Element is TEntity) then EntityError(Pom);
         TEntity(Info.Element).CreateElement(CheckClassName(Param1), Param2);
      end
      else
      if SS = cl_CreateTool then
      begin
        aClass:= FindOrbisClass(CheckClassName(Param1));
         if not (Info.Element is TTechnology) then Ex('Objekt na pozici '+ IntToStr(Pom)+' '+'nen Technology');
         TTechnology(Info.Element).CreateTool(CheckClassName(Param1),Param2);
      end
      else
      if SS = cl_Call then
      begin
         Tool := TCommandLine.Create;
         TCommandLine(Tool).FileName := GetAbsolutePath(Param1);
         TCommandLine(Tool).ActiveElement := ActiveElement;
         Tool.DoIt;
         Tool.Free;
      end
      else
      begin
         aClass := TElementClass(GetOrbisClass(CheckClassName(SS)));
         if aClass = nil then Ex('chybn funkce: '+ SS);
         Tool := TTool(aClass.Create);
         Tool.Input := Info.Element;
         if Tool.Input = nil then Tool.Input := ActiveElement;
         for ii := 0 to ParamList.Count-1 do
         begin
            if (ParamList.Strings[ii][1] in ['''','"']) or
               (not StringAsTwoParams(ParamList.Strings[ii], ':', L, P)) then
            begin
               ClearInfo(aInfo);
               CompileLine(ParamList.Strings[ii], aInfo);
               if not (aInfo.IsVariable or aInfo.IsConstant) and (aInfo.PropertyName = '') and (aInfo.Element <> nil) then
                  Tool.PropertyNumber[ii] := IntToStr(longInt(aInfo.Element))
               else
                  Tool.PropertyNumber[ii] := Evaluate(aInfo);
            end
            else
            begin
               if (P <> '') and (P[1] = ':') then Tool.PropertyNumber[ii] := ParamList.Strings[ii]
               else
               begin
                  ClearInfo(aInfo);
                  CompileLine(P, aInfo);
                  SetPropertyStringByName(L, Evaluate(aInfo));
               end;
            end;
         end;
         Tool.DoIt;
         Info.Element := Tool.Output;
         if Tool is TResultTool then
         begin
            Info.PropertyName := TResultTool(Tool).Result;
            Info.IsConstant := true;
         end;
         Tool.Free;
      end;
      if not (Delimiter = ')') then SyntaxError(Position -1, ' ")"');
      Pom := Position - 1;
      Delimiters := ['.','(',')',',','''','"'];
      SS := ReadToDelimiterWithoutWS(S,  Delimiters, Position, Delimiter, Position);
      if Delimiter = #0 then
      begin
         Finish := true;
         exit;
      end;
      if not (Delimiter = '.') then SyntaxError(Pom, 'oekvm "." nebo nic');
   end;


   procedure EvaluatePropertyAsObject;
   var
      E: TElement;
      I: longInt;
   begin
      if UpperCase(SS) = cl_Element then Info.List := true
      else
      if FirstWord and (UpperCase(SS) = cl_World) then Info.Element := World
      else
      if FirstWord and (UpperCase(SS) = cl_ActiveElement) then Info.Element := ActiveElement
      else
      if FirstWord and (UpperCase(SS) = cl_Self) then Info.Element := Self
      else
      if FirstWord and IsVar(SS) then
      begin
         if ActualVar.Typ = _Pointer then
            Info.Element := TElementVar(ActualVar)._Element
         else
            Info.Element := ActualVar;
            //Ex('Promnn nen element ale '+ActualVar.TypeAsString);
      end
      else
      begin
         E := TElement(GetObjectPropByName(Info.Element,SS));
         if E = nil then
         begin
           I:= StrToLongInt(Info.Element.GetPropertyStringByName(SS));
           E:= TElement(I);
           if (E<>nil) then
           begin
             if not (TObject(E) is TObject) then
                Ex('Nemohu nalzt objekt '+ SS);
           end
           else
           begin
             if not (Info.Element is TEntity) then
                Ex('Nemohu nalzt element '+ SS);

             E := TEntity(Info.Element).FindElementName(SS);
             if E = nil then
                Ex('Nemohu nalzt element '+ SS)
           end
         end;
         Info.Element := E;
      end;
   end;

   procedure ReadVariable;
   const
      errdef = 'Chybn definice promnn';
      errTyp= '- Neznm typ: ';
   var
      CH: char;
      STyp: string;
      aTyp: TType;
   begin
      Delimiters := [';'];
      S := ReadToDelimiterWithoutWS(S,  Delimiters, Position, Delimiter, Position);
      STyp := UpperCase(S);
      if Length(S) < 1 then Ex(errdef);
      CH := STyp[1];
      aClass := nil;
      aTyp := _Other;
      case CH of
         'T':
         begin
            aClass := GetOrbisClass(S);
            if aClass <> nil then  aTyp := _Pointer;
         end;
         'I':
         begin
            if STyp = cl_Integer then aTyp := _Integer;
         end;
         'S':
         begin
            if STyp = cl_String then aTyp := _String;
         end;
         'F':
         begin
            if STyp = cl_Float then aTyp := _Float;
         end;
         'B':
         begin
            if STyp = cl_Boolean then aTyp := _Boolean;
         end;
         'C':
         begin
            if STyp = cl_Char then aTyp := _CHar;
         end;
      end;
      if aTyp = _Other then
         Ex(errdef+errTyp+S)
      else
      begin
         if not IsVar(SS) then
            SetNewVar(SS, aTyp, aClass)
         else
         if ActualVar.Typ <> aTyp then
            Ex(errdef+' redeklarace jinho typu: '+S)
      end;
   end;

begin
   Info.Element := ActiveElement;
   Position := 1;
   FirstWord := true;
   Finish := false;
   Delimiters := ['.','(',')',',','''','"',':'];
   if (length(S)>1) and (S[1]='{') then
   begin
      EvaluateExpression;
      exit;
   end;
   while true do
   begin
      if Finish then exit;
      if info.Element = nil then exit;
      SS := ReadToDelimiterWithoutWS(S,  Delimiters, Position, Delimiter, Position);
      case Delimiter of
         #0: { properta - konec }
         begin
            if UpperCase(SS)=cl_Var then exit
            else
            if FirstWord and (UpperCase(SS)=cl_World) then Info.Element := World
            else
            if FirstWord and (UpperCase(SS)=cl_ActiveElement) then Info.Element := ActiveElement
            else
            if FirstWord and IsVar(SS) then
            begin
               Info.IsVariable := true;
               Info.Variable := ActualVar;
               if ActualVar.Typ = _Pointer then
                  Info.Element := TElementVar(ActualVar)._Element;
               exit;
            end
            else
            Info.PropertyName := SS;
            exit;
         end;
         '.': { properta - objekt }
         begin
            EvaluatePropertyAsObject;
         end;
         '(':
         begin
            EvaluateMethod;
         end;
         '''': { konstanta }
         begin
            Delimiters := [''''];
            SS := ReadToDelimiterWithoutWS(S,  Delimiters, Position, Delimiter, Position);
            Info.PropertyName := SS;
            Info.IsConstant := true;
            exit;
         end;
         '"': { konstanta }
         begin
            Delimiters := ['"'];
            SS := ReadToDelimiter{ O.S.25.3.98 WithoutWS}(S,  Delimiters, Position, Delimiter, Position);
            Info.PropertyName := SS;
            Info.IsConstant := true;
            exit;
         end;
         ':': { promnn }
         begin
            ReadVariable;
            exit;
         end;
      end;
      FirstWord := false;
   end;
end;


function  TCommandLine.Evaluate(var Info: TPropertyInfo): string;
var
   El: TElement;
begin
   //!!!!!!!!!!!
   result := '';
   if Info.IsVariable then
      result := Info.Variable.AsString
   else
   if Info.IsConstant then
      Result := Info.PropertyName
   else
   begin
      if Info.Element = nil then exit;
      Result := Info.Element.GetPropertyStringByName(Info.PropertyName);
      if Result = '' then
      begin
         if not(Info.Element is TEntity) then exit;
         El := TEntity(Info.Element).FindElementName(Info.PropertyName);
         Result := IntToStr(longInt(El));
      end;
   end;
end;


procedure TCommandLine.ClearInfo(var Info: TPropertyInfo);
begin
   Info.Element      := nil;
   Info.List         := false;
   Info.IsConstant   := false;
   Info.IsVariable   := false;
   Info.Variable     := nil;
   Info.PropertyName := '';
end;


procedure TCommandLine.EvaluateResult(var Info: TPropertyInfo);
var
   i: longInt;
   El: TElement;
begin
   ResultLines.Clear;
   if Info.List then
   begin
      if not(Info.Element is TEntity) then exit;
      for i := 0 to TEntity(Info.Element).Count - 1 do
      begin
         if Info.Element = nil then exit;
         ResultLines.Add(TEntity(Info.Element).Elements[i].GetPropertyStringByName(Info.PropertyName));
      end;
   end
   else
   begin
      ResultLine := Evaluate(info);
   end;
end;


function  TCommandLine.NumberToPropertyName(aNumber: longInt): string;
begin
   case aNumber of
      0: result := 'FileName';
      1: result := 'ActualDirectory'
   end;
end;


function  TCommandLine.GetLine: string;
begin
   result := FLine;
end;


function  TCommandLine.GetLines: TStringList;
begin
   if FLines = nil then FLines := TStringList.Create;
   result := FLines;
end;


function  TCommandLine.GetActualDirectory: string;
begin
   result := FActualDirectory;
end;

function  TCommandLine.GetFileName: string;
begin
   result := FFileName;
end;

function  TCommandLine.GetActiveElement: TElement;
begin
   result := FActiveElement;
end;

function  TCommandLine.GetResultLine: string;
begin
   if (FResult = nil) or (FResult.Count < 1) then result := ''
                                             else result := FResult.Strings[0];
end;

function  TCommandLine.GetResultLines: TStringList;
begin
   if FResult = nil then FResult := TStringList.Create;
   result := FResult;
end;

procedure TCommandLine.SetLine(aValue: string);
begin
   FLine := aValue;
end;

procedure TCommandLine.SetLines(aValue: TStringList);
begin
   Line := '';
   if FLines <> nil then
   begin
      FLines.Free;
      FLines := nil;
   end;
   FLines := aValue;
end;

procedure TCommandLine.SetActualDirectory(aValue: string);
begin
   FActualDirectory := CheckBackSlash(aValue);
end;

procedure TCommandLine.SetFileName(aValue: string);
begin
   Line := '';
   Lines.Clear;
   FFileName := aValue;
end;

procedure TCommandLine.SetActiveElement(aValue: TElement);
begin
   FActiveElement := aValue;
end;

procedure TCommandLine.SetNextLine(aValue: string);
begin
   Lines.Add(aValue);
end;

procedure TCommandLine.SetResultLine(aValue: string);
begin
   ResultLines.Add(aValue);
end;

procedure TCommandLine.SetResultLines(aValue: TStringList);
begin
   if FResult <> nil then
   begin
      FResult.Free;
      FResult := nil;
   end;
   FResult :=aValue;
end;

procedure TCommandLine.InterpretLine;
var
   i: longInt;
   List: TStringList;
   L, R: string;
   TwoSides: boolean;
   E: TElement;
   RightIsElement: boolean;
begin
    Reset;
    RightIsElement := false;
    TwoSides := StringAsParameter(Line, L, R);
    { Zpracovn lev strany }
    //if Comment then exit;
    if UpperCase(L) = cl_ActiveElement then
    begin
       ActiveElementSetting := true;
       if not TwoSides then
       begin
           ShowError('Chyba .. nastaven aktivnho elementu: '+ Line, 3);
           exit;
       end;
    end
    else
    begin
       CompileLine(L, LeftSide);
       //if LeftSide.Variable then LeftVar := ActualVar;
    end;

    { Zpracovn prav strany }
    if TwoSides then
    begin
       CompileLine(R, RightSide);
    end
    else
    begin  // pokud je jen leva strana
       //if LeftSide.Variable then LeftSide.Element := LeftVar.Element;
       EvaluateResult(LeftSide);
       exit;
    end;

    if ActiveElementSetting then
    begin
       if RightSide.List then
       begin
          ShowError('Chyba .. jako aktivn element nelze piadit seznam (klov slovo ELEMENT nahrate nzvem property): '
                      + Line, 4);
          exit;
       end;
       if RightSide.PropertyName = '' then E := RightSide.Element
       else
       begin
          E := TElement(GetObjectPropByName(TPersistent(RightSide.Element), RightSide.PropertyName));
          if E = nil then
          begin
             if (RightSide.Element is TEntity) then
                E := TEntity(RightSide.Element).FindElementName(RightSide.PropertyName);
             if E = nil then
             begin
                ShowError('Chyba .. Nemohu nalzt objekt '+ RightSide.PropertyName, 5);
                exit;
             end;
          end;
       end;
       ActiveElement := E;
       exit;
    end;

    if (RightSide.IsVariable and (RightSide.Variable.Typ = _Pointer))or
       ((RightSide.PropertyName = '') and not (RightSide.IsConstant or RightSide.IsVariable)) then
    begin
       RightIsElement := true;
    end
    else
    begin
       EvaluateResult(RightSide);
       if RightSide.List then
       begin
          List := TStringList.Create;
          List.Assign(ResultLines);
       end;
    end;

    if LeftSide.List then
    begin
       if (LeftSide.Element = nil) or
          not (LeftSide.Element is TEntity) then
       begin
          ShowError('Chyba .. Zklad seznamu na lev stran nen entita (nebo je objekt pokozen) : '+ Line, 7);
          exit;
       end;
       for i := 0 to TEntity(LeftSide.Element).Count - 1 do
       begin
          if RightSide.List then SetObjectPropByName(TEntity(LeftSide.Element).Elements[i], LeftSide.PropertyName, List)
          else
          if RightIsElement then
             SetObjectPropByName(TEntity(LeftSide.Element).Elements[i], LeftSide.PropertyName, RightSide.Element)
          else
          begin
            if TObject(TEntity(LeftSide.Element).Elements[i]) is TElement then
               TEntity(LeftSide.Element).Elements[i].SetPropertyStringByName(LeftSide.PropertyName, ResultLine)
            else
              SetPropStringByName(TEntity(LeftSide.Element).Elements[i], LeftSide.PropertyName, ResultLine)
          end
       end;
       exit;
    end
    else
    begin
       if LeftSide.IsVariable then
       begin
          if RightIsElement then
          begin
             if not (LeftSide.Variable.Typ = _Pointer) then
             begin
                ShowError('Promnn na lev stran nen element: '+ Line, 8)
             end
             else
                TElementVar(LeftSide.Variable)._Element := RightSide.Element;
          end
          else
          begin
             try
                LeftSide.Variable.AsString := ResultLine;
             except
                ShowError('Chyba .. vsledek na prav stran vrazu: '+ Line, 8)
             end;
          end;
          exit;
       end;
       if (LeftSide.Element = nil) or (LeftSide.PropertyName = '') then
       begin
          ShowError('Chyba .. Chybn objekt nebo properta na lev stran vrazu : '+ Line, 9);
          exit;
       end;
       if RightSide.List then SetObjectPropByName(LeftSide.Element, LeftSide.PropertyName, List)
       else
       if RightIsElement then
          SetObjectPropByName(LeftSide.Element, LeftSide.PropertyName, RightSide.Element)
       else
       begin
         if TObject(LeftSide.Element) is TElement then
            LeftSide.Element.SetPropertyStringByName(LeftSide.PropertyName, ResultLine)
         else
           SetPropStringByName(LeftSide.Element, LeftSide.PropertyName, ResultLine)
       end
    end;
end;


procedure TCommandLine.InterpretLines;
var
   i: longInt;
begin
   if SmartMode then
   begin
      Busy := true;
      LineNumber := 0;
      PostMsg(tm_CMDLNextLine, Self, nil, DontKill);
   end
   else
   begin
      FlowInfos.FreeAll;
      //for i := 0 to Lines.Count -1 do
      i := 0;
      while i < Lines.Count do
      begin
         LineNumber := i;
         Line := Lines.Strings[i];
         if Line = '' then
         begin
            Inc(i);
            continue;
         end;
         if CheckFlow(i) then
            InterpretLine;
         if BreakWork then
         begin
            BreakWork := false;
            Break;
         end;
         Application.ProcessMessages;
      end;
      LineNumber := 0;
      Line := '';
   end;
end;


procedure TCommandLine.on_tm_CMDLNextLine;
var
   i: longInt;
begin
   while (LineNumber < Lines.Count) and (Lines.Strings[LineNumber] = '')   do Inc(FLineNumber);
   { End of Work ?!?}
   if LineNumber >= Lines.Count then
   begin
      LineNumber := 0;
      Line := '';
      Busy := false;
      PostMsg(tm_Done, World.MainTool, nil, DontKill);
      exit;
   end;
   Line := Lines.Strings[LineNumber];
   try
      InterpretLine;
   except
      on EX: Exception  do
      begin
         ShowError('Soubor ' + FileName + ', dek..'+IntToStr(LineNumber+1) + ':' + #13 + #10 +
                   '"' + Line + '"' + #13 + #13 + #10 +
                   'Chyba: '+EX.Message, 2);
         LineNumber :=  Lines.Count;
      end;
   end;
   Inc(FLineNumber);
   PostMsg(tm_CMDLNextLine, Self, nil, DontKill);
end;


procedure TCommandLine.HandleMessage(var aMessage: TOrbisMessage; var Handled: boolean);
begin
   Handled := false;
   inherited HandleMessage(aMessage, Handled);
   if Handled then exit;
   case aMessage.Message of
      tm_CMDLNextLine : on_tm_CMDLNextLine;
   end;
end;



function TCommandLine.GetFlowInfos: TOwnerEntity;
begin
   if FFlowInfos = nil then
   begin
      FFlowInfos := TOwnerEntity.Create;
      FFlowInfos.Name := 'Flow_Infos';
      AddElement(FFlowInfos);
   end;
   result := FFlowInfos;
end;

function TCommandLine.GetLastFlowInfo: TFlowInfo;
begin
   result := nil;
   if FlowInfos.NoElements then exit;
   result := TFlowInfo(FLowInfos.Elements[FlowInfos.LastIndex]);
end;

function TCommandLine.CheckFlow(var Row: integer): boolean;
var
   S: string;
   kw: string;
   Varname: string;
   Lasti: integer;
   i: integer;
   FlowInfo,LFlowInfo: TFlowInfo;
   PropInfo: TPropertyInfo;
   E: TElement;
   En: TEntity;
   Len: integer;
   Err: boolean;
begin
   Inc(Row);
   result := false;
   i := pos('//',Line);
   case i of
      0:;
      1: exit; // pozn. na zatku dku
   else
      Line := Copy(Line,1,i-1);
   end;
   S := GetFirstWord(Lasti);
   if S = '' then exit;
   // Komentar
   if S[1] = ';' then exit;
   if S[1] = '*' then
      Comment := not Comment;
   if Comment then exit;
   kw := UpperCase(S);
   if kw = cl_IF then
   begin
      S := UpperCase(Line);
      i := Pos(cl_THEN,S);
      if i = 0 then
      begin
         ShowError('Chyba Syntaxe, chyb THEN'+GetErrorPosition, 0);
         BreakWork := true;
         exit;
      end;
      LFlowInfo := LastFlowInfo;
      S := Copy(Line,Lasti,i-Lasti);
      FlowInfo := TFlowInfo.Create;
      FlowInfos.AddElement(FlowInfo);
      FlowInfo.FlowType := fi_IF;
      if (LFLowInfo <> nil) and LFlowInfo.NewFlowIsShadow then
      begin
         FlowInfo.Status := fi_Shadow;
      end
      else
      begin
         if Process_IF(S) then
            FlowInfo.Status := fi_IF_RUN
         else
            FlowInfo.Status := fi_IF_SKIP;
      end;
      exit;
   end
   else
   if kw = cl_ELSE then
   begin
      FlowInfo := LastFlowInfo;
      if (FlowInfo = nil) or (FlowInfo.FlowType <> fi_IF) then
      begin
         ShowError('Chyba Syntaxe, chyb IF'+GetErrorPosition, 0);
         BreakWork := true;
         exit;
      end;
      case FlowInfo.Status of
         fi_IF_RUN : FlowInfo.Status := fi_ELSE_SKIP;
         fi_IF_SKIP: FlowInfo.Status := fi_ELSE_RUN;
      else
         FlowInfo.Status := fi_Shadow;
      end;
      exit;
   end
   else
   if kw = cl_FOR then
   begin
      LFlowInfo := LastFlowInfo;
      FlowInfo := TFlowInfo.Create;
      FlowInfos.AddElement(FlowInfo);
      FlowInfo.FlowType := fi_FOR;
      if (LFLowInfo <> nil) and LFlowInfo.NewFlowIsShadow then
      begin
         FlowInfo.Status := fi_Shadow;
         exit;
      end;
      i := Lasti;
      VarName := GetNextWord(i,Lasti);
      if VarName = '' then
      begin
         ShowError('Cyklus FOR, chyb promnn'+GetErrorPosition, 0);
         BreakWork := true;
         exit;
      end;
      i := Lasti;
      S := GetNextWord(i,Lasti);
      S := UpperCase(S);
      Len := Length(S);
      Err := false;
      case Len of
         2:
         begin
            if not (S = cl_IN) then
               Err := true;
            FlowInfo._In := true;
         end;
         3:
         begin
            if not (S = cl_OUT) then
               Err := true;
            FlowInfo._In := false;
         end;
         else
            err := true;
      end;
      if Err then
      begin
         ShowError('Cyklus FOR, chyb IN|OUT'+GetErrorPosition, 0);
         BreakWork := true;
         exit;
      end;
      S := Copy(Line,Lasti,Length(Line)-Lasti+1);
      CompileLine(S,PropInfo);
      if PropInfo.PropertyName = '' then
      begin
         E := PropInfo.Element;
      end
      else
      begin
         kw := Evaluate(PropInfo);
         E := TElement(StrToIntDef(kw,0));
      end;
      En := nil;
      if FlowInfo._In then
      begin
         if (E = nil) or not (E is TEntity) then
         begin
            ShowError('Cyklus FOR,"'+S+'" nen entita '+GetErrorPosition, 0);
         end
         else
            En := TEntity(E);
      end
      else
      begin
         if E = nil then
            ShowError('Cyklus FOR,"'+S+'" not Assigned '+GetErrorPosition, 0);
      end;
      if not IsVar(VarName) then
         SetNewVar(VarName,_Pointer, TElement)
      else
      begin
         if ActualVar.Typ <> _Pointer then
         begin
            ShowError('Cyklus FOR, redeklarace promnn "'+VarName+'" jinho typu ['+ActualVar.TypeAsString+'] '+GetErrorPosition, 0);
            FLowInfo.Status := fi_Shadow;
            exit;
         end;
      end;
      FlowInfo.Variable := ActualVar;
      FLowInfo.ActualIndex := 0;
      if FlowInfo._In then
      begin
         FlowInfo.Entity := En;
         if (En = nil) or En.NoElements then
         begin
            FLowInfo.Status := fi_Shadow;
            exit;
         end;
         TElementVar(FlowInfo.Variable)._Element := En.Elements[0];
      end
      else
      begin
         FlowInfo._Element := E;
         if (E = nil) or E.NoOwners then
         begin
            FLowInfo.Status := fi_Shadow;
            exit;
         end;
         TElementVar(FlowInfo.Variable)._Element := E.Owners[0];
      end;
      FlowInfo.Status := fi_FOR_RUN;
      FLowinfo.FirstLine := Row;
      exit;
   end
   else
   if kw = cl_WHILE then
   begin
      LFlowInfo := LastFlowInfo;
      FlowInfo := TFlowInfo.Create;
      FlowInfos.AddElement(FlowInfo);
      FlowInfo.FlowType := fi_While;
      if (LFLowInfo <> nil) and LFlowInfo.NewFlowIsShadow then
      begin
         FlowInfo.Status := fi_Shadow;
         exit;
      end;
      S := copy(Line,Lasti,length(Line)-Lasti+1);
      FlowInfo.Condition := S;
      if not Process_IF(S) then
      begin
         FlowInfo.Status := fi_Shadow;
         exit;
      end;
      FlowInfo.Status := fi_FOR_RUN;
      FLowinfo.FirstLine := Row;
      exit;
   end
   else
   if kw = cl_REPEAT then
   begin
      LFlowInfo := LastFlowInfo;
      FlowInfo := TFlowInfo.Create;
      FlowInfos.AddElement(FlowInfo);
      FlowInfo.FlowType := fi_Repeat;
      if (LFLowInfo <> nil) and LFlowInfo.NewFlowIsShadow then
      begin
         FlowInfo.Status := fi_Shadow;
         exit;
      end;
      FlowInfo.Status := fi_FOR_RUN;
      FLowinfo.FirstLine := Row;
      exit;
   end
   else
   if kw = cl_END then
   begin
      FlowInfo := LastFlowInfo;
      if FLowInfo = nil then
      begin
         ShowError('Chyba Syntaxe, chyb [IF,FOR,WHILE] '+GetErrorPosition, 0);
         BreakWork := true;
         exit;
      end;
      if FlowInfo.Status = fi_Shadow then
      begin
         FlowInfo.Free;
         exit;
      end;
      case FlowInfo.FlowType of
      fi_IF:
         begin
            FlowInfo.Free;
            exit;
         end;
      fi_FOR:
         begin
            i := FlowInfo.ActualIndex;
            Inc(i);
            FlowInfo.ActualIndex := i;
            if FlowInfo._In then // Elements
            begin
               if i > FlowInfo.Entity.LastIndex then
               begin
                  FlowInfo.Free;
                  exit;
               end;
               TElementVar(FlowInfo.Variable)._Element := FlowInfo.Entity.Elements[i];
            end
            else                 // Owners
            begin
               if i >= FlowInfo._Element.OwnersCount then
               begin
                  FlowInfo.Free;
                  exit;
               end;
               TElementVar(FlowInfo.Variable)._Element := FlowInfo._Element.Owners[i];
            end;
            Row := FlowInfo.FirstLine;
         end;
       fi_WHILE:
          begin
            if not Process_IF(FlowInfo.Condition) then
            begin
               FlowInfo.Free;
               exit;
            end;
            Row := FlowInfo.FirstLine;
          end;
      end;
   end
   else
   if kw = cl_UNTIL then
   begin
      FlowInfo := LastFlowInfo;
      if (FLowInfo = nil) or (FlowInfo.FlowType <> fi_Repeat) then
      begin
         ShowError('Chyba Syntaxe, chyb REPEAT '+GetErrorPosition, 0);
         BreakWork := true;
         exit;
      end;
      if FlowInfo.Status = fi_Shadow then
      begin
         FlowInfo.Free;
         exit;
      end;
      S := copy(Line,Lasti,length(Line)-Lasti+1);
      if Process_IF(S) then
      begin
         FlowInfo.Free;
         exit;
      end;
      Row := FlowInfo.FirstLine;
      exit;
   end
   else
   if kw = cl_EXIT then
   begin
      FlowInfo := LastFlowInfo;
      if (FlowInfo = nil) or not FlowInfo.NewFlowIsShadow then
      begin
         BreakWork := true;
         exit;
      end;
   end
   else
   if kw = cl_BREAK then
   begin
      FlowInfo := LastFlowInfo;
      Err := false;
      if FlowInfo = nil then
      begin
         Err := true;
      end
      else
      begin
         if not FlowInfo.NewFlowIsShadow then
         begin
            Err := true;
            for i := FlowInfos.LastIndex downTo 0 do
            begin
               FlowInfo := TFlowInfo(FlowInfos.Elements[i]);
               FlowInfo.Status := fi_Shadow;
               if FlowInfo.FlowType in [fi_FOR,fi_While,fi_Repeat] then
               begin
                  Err := false;
                  break;
               end;
            end;
         end;
      end;
      if Err then
      begin
         ShowError('Chyba Syntaxe, chyb [FOR,WHILE,REPEAT] '+GetErrorPosition, 0);
         BreakWork := true;
         exit;
      end;
   end
   else
   begin
      FlowInfo := LastFlowInfo;
      if (FlowInfo = nil) or not FlowInfo.NewFlowIsShadow then
         result := true;
   end;
end;

function TCommandLine.GetFirstWord(var Lasti: integer): string;
var
   i: integer;
   l: integer;
begin
   result := '';
   l := Length(Line);
   Lasti := l+1;
   i := 1;
   while (i <= l) and (Line[i] in [#0,#09,' ']) do inc(i);
   if i > l then exit;
   while (i <= l) and not (Line[i] in [#0,#09,' ','.']) do
   begin
      result := result + Line[i];
      inc(i);
   end;
   Lasti := i;
end;

function TCommandLine.GetNextWord(Firsti: integer; var Lasti: integer): string;
var
   i: integer;
   l: integer;
begin
   result := '';
   l := Length(Line);
   Lasti := l+1;
   i := Firsti;
   while (i <= l) and (Line[i] in [#0,#09,' ']) do inc(i);
   if i > l then exit;
   while (i <= l) and not (Line[i] in [#0,#09,' ','.']) do
   begin
      result := result + Line[i];
      inc(i);
   end;
   Lasti := i;
end;

function TCommandLine.Process_IF(const S: string): boolean;
var
   CondCre: TCondCreator;
   Cond: TCondition;
begin
   result := false;
   CondCre := TCondCreator.Create;
   CondCre.Command := S;
   COndCre.Vars := Vars;
   CondCre.DoIt;
   Cond := TCondition(COndCre.output);
   Cond.ParamItem[0] := ActiveElement;
   result := Cond.IsTrue;
   Cond.Free;
   CondCre.Free;
end;


{ TFlowInfo }

function TFlowInfo.NewFlowIsShadow: boolean;
begin
   result := false;
   if Status = fi_Shadow then
      result := true
   else
   if FlowType = fi_IF then
   begin
      if (Status = fi_IF_SKIP) or (Status = fi_ELSE_SKIP) then
         result := true;
   end;
end;

procedure TFlowInfo.SetActualIndex(const Value: integer);
begin
  FActualIndex := Value;
end;

procedure TFlowInfo.SetCondition(const Value: string);
begin
  FCondition := Value;
end;

procedure TFlowInfo.SetEntity(const Value: TEntity);
begin
  FEntity := Value;
end;

procedure TFlowInfo.SetFirstLine(const Value: integer);
begin
  FFirstLine := Value;
end;

procedure TFlowInfo.SetFlowType(const Value: TFlowInfoType);
begin
  FFlowType := Value;
end;

procedure TFlowInfo.SetStatus(const Value: TFlowInfoStatus);
begin
  FStatus := Value;
end;

procedure TFlowInfo.SetVariable(const Value: TTypeElement);
begin
  FVariable := Value;
end;

procedure TFlowInfo.Set_Element(const Value: TElement);
begin
  F_Element := Value;
end;

procedure TFlowInfo.Set_In(const Value: boolean);
begin
  F_In := Value;
end;

INITIALIZATION
   RegisterOrbisClass(TCommandLine);
END.
